// This code conforms with the UFC specification version 2.0.0
// and was automatically generated by FFC version 0.9.10.
//
// This code was generated with the option '-l dolfin' and
// contains DOLFIN-specific wrappers that depend on DOLFIN.
// 
// This code was generated with the following parameters:
// 
//   cache_dir:                      ''
//   convert_exceptions_to_warnings: False
//   cpp_optimize:                   False
//   cpp_optimize_flags:             '-O2'
//   epsilon:                        1e-14
//   error_control:                  False
//   form_postfix:                   True
//   format:                         'dolfin'
//   log_level:                      10
//   log_prefix:                     ''
//   no_ferari:                      True
//   optimize:                       True
//   output_dir:                     '.'
//   precision:                      15
//   quadrature_degree:              'auto'
//   quadrature_rule:                'auto'
//   representation:                 'auto'
//   split:                          False
//   swig_binary:                    'swig'
//   swig_path:                      ''

#ifndef __POISSON3D_5_H
#define __POISSON3D_5_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>

/// This class defines the interface for a finite element.

class poisson3d_5_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  poisson3d_5_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the topological dimension of the cell shape
  virtual unsigned int topological_dimension() const
  {
    return 3;
  }

  /// Return the geometric dimension of the cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = x[3][0] + x[2][0] + x[1][0] - x[0][0];
    const double C1 = x[3][1] + x[2][1] + x[1][1] - x[0][1];
    const double C2 = x[3][2] + x[2][2] + x[1][2] - x[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (d_00*(2.0*coordinates[0] - C0) + d_10*(2.0*coordinates[1] - C1) + d_20*(2.0*coordinates[2] - C2)) / detJ;
    double Y = (d_01*(2.0*coordinates[0] - C0) + d_11*(2.0*coordinates[1] - C1) + d_21*(2.0*coordinates[2] - C2)) / detJ;
    double Z = (d_02*(2.0*coordinates[0] - C0) + d_12*(2.0*coordinates[1] - C1) + d_22*(2.0*coordinates[2] - C2)) / detJ;
    
    
    // Reset values.
    *values = 0.0;
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923465, 0.00609788120679257, 0.00352061335622799, 0.00248944957812477, 0.00705946233325034, 0.00546823600997147, 0.00446479600584761, 0.00315708753234942, 0.00257775117585288, 0.00182274533665717, -0.00298267418725715, -0.00252081977982931, -0.00218309396769448, -0.00195261860521384, -0.00169101731601733, -0.00138070985681771, -0.0011273448773449, -0.000976309302606944, -0.000797153207506486, -0.000563672438672447, 0.00278687112460007, 0.00245778931055955, 0.00219831358911339, 0.00207721109304567, 0.0018579140830667, 0.00160900079398463, 0.00160900079398464, 0.00143913406048031, 0.0012463266558274, 0.00101762145320216, 0.000928957041533369, 0.000830884437218256, 0.00071956703024016, 0.000587524019939406, 0.000415442218609138, -0.00705593524717797, -0.00638233357089079, -0.00582624677716565, -0.00562868913761196, -0.00513826668309062, -0.00459580543596521, -0.00475711057319694, -0.00434262794914382, -0.00388416451811044, -0.00336378514516178, -0.00368484200521178, -0.00336378514516176, -0.00300866089851429, -0.00260557676948629, -0.00212744452363027, -0.00212744452363026, -0.00194208225905519, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828763};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923512, -0.00609788120679247, 0.00352061335622801, 0.00248944957812479, 0.00705946233325039, -0.00546823600997145, -0.00446479600584757, 0.00315708753234941, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982929, -0.00218309396769447, 0.00195261860521384, 0.00169101731601731, 0.00138070985681771, -0.00112734487734488, -0.000976309302606922, -0.00079715320750647, -0.000563672438672437, 0.00278687112460002, -0.00245778931055953, -0.00219831358911338, 0.00207721109304566, 0.00185791408306669, 0.00160900079398462, -0.00160900079398462, -0.00143913406048031, -0.0012463266558274, -0.00101762145320215, 0.000928957041533356, 0.00083088443721826, 0.000719567030240153, 0.0005875240199394, 0.000415442218609126, 0.00705593524717798, -0.00638233357089082, -0.00582624677716566, 0.00562868913761197, 0.00513826668309063, 0.0045958054359652, -0.00475711057319694, -0.00434262794914382, -0.00388416451811044, -0.00336378514516178, 0.00368484200521179, 0.00336378514516178, 0.0030086608985143, 0.00260557676948629, 0.00212744452363027, -0.00212744452363027, -0.00194208225905521, -0.00173705117965753, -0.00150433044925715, -0.00122828066840393, -0.000868525589828761};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923495, 0.0, -0.00704122671245611, 0.00248944957812476, 0.0, 0.0, 0.0, 0.00947126259704834, -0.0051555023517057, 0.00182274533665714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00450937950937952, -0.00292892790782074, 0.00159430641501296, -0.000563672438672428, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0046447852076667, -0.00332353774887305, 0.00215870109072045, -0.00117504803987881, 0.000415442218609132, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, -0.0097104112952761, 0.00694820471863011, -0.00451299134777143, 0.00245656133680786, -0.000868525589828765};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 3:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923497, 0.0, 0.0, -0.0074683487343743, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0109364720199429, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00563672438672438, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00623163327913698, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 4:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0, 0.0374718748311399, -0.0289614183593723, 0.0, 0.0, 0.0, 0.0284137877911448, 0.0051555023517057, -0.00619733414463434, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0315656565656566, -0.0113902751970807, 0.00265717735835493, -0.00018789081289078, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0232239260383337, -0.00166176887443654, -0.00647610327216137, 0.00646276421933344, -0.00290809553026389, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0213629048496074, -0.0277928188745204, 0.0243701532779658, -0.0157219925555703, 0.00607967912880132};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 5:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, 0.0, -0.028757485335526, 0.0536094339843701, 0.0, 0.0, 0.0, -0.0189425251940965, 0.0154665070551172, 0.0153110608279201, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0045093795093795, 0.0370997534990628, -0.0281660799985623, 0.0108976671476671, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.00431740218144092, -0.0123380044187275, 0.00872428659079175, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338355, -0.0487403065559316, 0.0412702304583721, -0.018239037386404};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 6:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0, 0.0409576306293855, -0.0449826285156206, 0.0, 0.0, 0.0, 0.0189425251940965, -0.0051555023517057, -0.021143845905223, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00455611007883226, 0.0356061766019561, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0302218152700864, 0.00411266813957585, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0379091273212801, -0.0550269739444961, 0.0303983956440067};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 7:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0, -0.0148144621425436, 0.0449826285156207, 0.0, 0.0, 0.0, 0.0, 0.0206220094068228, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0106287094334197, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0205633406978791, 0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.03439185871531, -0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 8:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997512, -0.0164047080299144, 0.00446479600584757, 0.0094712625970483, -0.00257775117585284, -0.00619733414463434, 0.0208787193108001, -0.0176457384588052, -0.00848980987436739, 0.0136683302364968, 0.00657617845117845, 0.00230118309469618, -0.00789141414141414, -0.0037967583990269, -0.00132858867917748, -0.000187890812890797, 0.0139343556230002, -0.0122889465527977, -0.00109915679455668, 0.0103860554652283, 0.000928957041533339, -0.00482700238195389, -0.00804500396992314, -0.000719567030240159, 0.0037389799674822, 0.00559691799261186, 0.00464478520766673, 0.000415442218609135, -0.00215870109072048, -0.00323138210966671, -0.00290809553026391, 0.0, 0.0, 0.0128177429097644, 0.0, -0.0113041867027994, -0.0183832217438608, 0.0, 0.00955378148811638, 0.0155366580724417, 0.0181644397838736, 0.0, -0.0074003273193559, -0.0120346435940572, -0.014070114555226, -0.0136156449512337, 0.0, 0.00427258096992146, 0.00694820471863012, 0.00812338442598859, 0.00786099627778515, 0.00607967912880133};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 9:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734313, -0.0249047128495238, 0.0143787426677629, 0.0536094339843699, -0.0141189246665007, 0.0109364720199429, 0.0133943880175427, -0.00631417506469884, -0.00773325352755864, 0.0153110608279201, -0.00298267418725714, 0.00252081977982934, 0.0276525235907967, -0.00195261860521383, -0.0214195526695527, -0.0243925408037797, 0.00112734487734489, 0.0123665844996876, 0.0140830399992812, 0.0108976671476671, 0.0, 0.0, 0.0197848223020205, 0.0, -0.0167212267476002, -0.00321800158796926, 0.0, 0.0129522065443228, 0.00249265331165478, -0.0106850252586227, 0.0, -0.00747795993496437, -0.00143913406048031, 0.00616900220936374, 0.00872428659079175, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, 0.0135389740433143, 0.0281402291104519, 0.0357410679969885, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.020635115229186, -0.018239037386404};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 10:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, -0.00446479600584751, 0.00631417506469886, 0.00257775117585291, -0.021143845905223, 0.0, 0.0, 0.00339592394974697, 0.0, -0.0026304713804714, 0.030835853468929, 0.0, 0.00151870335961077, -0.017803088300978, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, -0.0174485731815835, 0.00356167508620757, 0.0, 0.0, 0.0100739384233621, -0.00205633406978792, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, -0.0218868448636848, -0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440067};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 11:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, -0.0128297005588457, 0.00740723107127187, 0.0449826285156207, 0.0, 0.0, 0.0178591840233903, 0.0, -0.0103110047034115, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00920473237878483, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489395, 0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0297842233308237, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 12:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0324515955311978, -0.0335503995581135, -0.00801060507812423, 0.0211783869997512, 0.00109364720199426, -0.0169662248222207, -0.00568275755822893, 0.00567105258687628, 0.00838462854862287, 0.0208787193108001, -0.0109235523792603, -0.0162519217595033, 0.00499002532443534, 0.00958243145743148, 0.0108922666482287, -0.00187890812890813, -0.00466458889023306, -0.00558007245254537, -0.00469727032227031, 0.0139343556230002, -0.00245778931055953, -0.0120907247401236, -0.00415442218609133, 0.00464478520766674, 0.00965400476390777, 0.00643600317593853, -0.000719567030240148, -0.00498530662330958, -0.00661453944581401, -0.00464478520766674, -0.000415442218609135, 0.00215870109072044, 0.00323138210966672, 0.00290809553026393, 0.0, 0.0127646671417816, -0.00116524935543313, -0.0202632808954031, -0.00719357335632687, 0.00183832217438609, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, -0.020635115229186, -0.01143686949355, -0.00481385743762288, -0.000521115353897266, 0.00170195561890421, 0.0127646671417816, 0.00737991258440982, 0.00347410235931507, 0.000902598269554292, -0.000491312267361576, -0.000868525589828768};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 13:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, -0.0249047128495239, 0.0553363732971483, -0.00431340273437465, -0.0141189246665008, 0.0153110608279201, 0.00803663281052558, 0.00883984509057834, -0.0159820572902877, 0.000729098134662859, -0.00298267418725716, 0.0277290175781224, -0.00145539597846296, -0.0301571095694136, -0.0116492303992304, 0.00352848074520085, 0.0191648629148629, 0.00976309302606916, 0.00132858867917748, -0.00263047138047137, 0.0, 0.0196623144844763, -0.00219831358911337, -0.00727023882565978, -0.0148633126645335, 0.00402250198496157, -0.00724050357293084, 0.0129522065443227, 0.00685479660705068, -0.0045792965394097, 0.00928957041533348, -0.00664707549774609, -0.0057565362419212, -0.000293762009969712, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0202632808954031, -0.00411061334647248, 0.000459580543596519, -0.0399597288148542, -0.00303983956440068, 0.00427258096992148, -0.00100913554354854, 0.0464290092656685, 0.00908221989193679, -0.00270779480866284, -0.00234501909253765, 0.00127646671417816, -0.031911667854454, -0.00776832903622086, 0.000868525589828756, 0.00180519653910857, 0.000245656133680793, -0.000868525589828769};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 14:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437464, 0.0141189246665008, -0.00656188321196573, -0.00982255121286465, -0.016416855168217, 0.0149509568199466, 0.000729098134662881, 0.0, 0.0033610930397724, -0.000485131992821013, 0.0282044909641998, -0.0124007936507937, 0.0029148319199485, -0.0353234728234728, 0.00889526253486302, 0.00239145962251946, -0.00263047138047138, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920009, 0.000804500396992313, -0.00402250198496156, -0.0143913406048031, 0.00934744991870547, -0.00254405363300537, -0.00928957041533351, 0.0141250354327105, -0.00287826812096064, -0.00381890612960614, 0.00290809553026391, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320203, 0.00233049871086626, -0.000336378514516182, -0.0515877880729651, 0.00840946286290446, 0.00300866089851428, -0.00286613444643492, 0.000850977809452106, 0.0425488904726053, -0.00388416451811045, -0.00347410235931507, 0.00120346435940573, 0.00098262453472315, -0.000868525589828758};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 15:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0128297005588456, 0.0448791059024116, -0.00801060507812425, 0.0, 0.0174983552319087, -0.00357183680467803, 0.0132597676358675, -0.0175287079957995, 0.00838462854862292, 0.0, 0.0, 0.0, 0.00867830491206147, -0.00300625300625301, 0.000613648825252321, 0.0225468975468976, -0.019200749617936, 0.0122230158484327, -0.00469727032227033, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0160900079398463, -0.00719567030240153, 0.0024926533116548, -0.000508810726601081, 0.00464478520766676, -0.0108014976838374, 0.0107935054536023, -0.00734405024924251, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0257938940364826, -0.0134551405806471, 0.00601732179702859, -0.00208446141558904, 0.000425488904726057, -0.031911667854454, 0.0135945758133865, -0.00347410235931503, -0.000902598269554298, 0.0017195929357655, -0.000868525589828767};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 16:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734316, -0.0324515955311979, -0.0187359374155699, -0.0289614183593722, 0.0211783869997512, 0.0164047080299144, -0.00446479600584758, 0.00947126259704828, -0.00257775117585287, -0.00619733414463427, -0.0208787193108001, -0.0176457384588051, -0.0084898098743674, -0.0136683302364968, -0.00657617845117843, -0.00230118309469622, -0.00789141414141414, -0.00379675839902689, -0.00132858867917748, -0.0001878908128908, 0.0139343556230002, 0.0122889465527977, 0.00109915679455668, 0.0103860554652283, 0.000928957041533344, -0.0048270023819539, 0.00804500396992314, 0.000719567030240151, -0.00373897996748219, -0.00559691799261186, 0.00464478520766674, 0.000415442218609132, -0.00215870109072046, -0.00323138210966671, -0.00290809553026394, 0.0, 0.0, 0.0128177429097644, 0.0, 0.0113041867027994, 0.0183832217438608, 0.0, 0.0095537814881164, 0.0155366580724417, 0.0181644397838736, 0.0, 0.00740032731935589, 0.0120346435940572, 0.014070114555226, 0.0136156449512337, 0.0, 0.00427258096992147, 0.00694820471863012, 0.00812338442598858, 0.00786099627778515, 0.00607967912880134};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 17:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734315, 0.0249047128495239, 0.014378742667763, 0.0536094339843698, -0.0141189246665008, -0.0109364720199429, -0.0133943880175426, -0.00631417506469883, -0.00773325352755856, 0.0153110608279199, 0.00298267418725717, 0.00252081977982929, 0.0276525235907967, 0.00195261860521384, 0.0214195526695527, 0.0243925408037797, 0.00112734487734486, 0.0123665844996876, 0.0140830399992811, 0.0108976671476671, 0.0, 0.0, -0.0197848223020205, 0.0, -0.0167212267476003, -0.00321800158796924, 0.0, -0.0129522065443227, -0.00249265331165479, 0.0106850252586226, 0.0, -0.00747795993496438, -0.00143913406048031, 0.00616900220936373, 0.00872428659079181, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, -0.0135389740433143, -0.0281402291104519, -0.0357410679969885, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.020635115229186, -0.018239037386404};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 18:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, -0.0354703486038674, -0.0204788153146928, -0.0449826285156206, 0.0141189246665008, 0.0109364720199429, 0.0044647960058476, 0.00631417506469885, 0.00257775117585289, -0.0211438459052229, 0.0, 0.0, 0.00339592394974697, 0.0, 0.00263047138047138, -0.0308358534689291, 0.0, 0.00151870335961077, -0.017803088300978, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, 0.0174485731815835, -0.00356167508620759, 0.0, 0.0, 0.0100739384233621, -0.00205633406978796, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, 0.0218868448636849, 0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440067};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 19:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0128297005588457, 0.00740723107127186, 0.0449826285156208, 0.0, 0.0, -0.0178591840233903, 0.0, -0.0103110047034114, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00920473237878476, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489395, 0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0297842233308238, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 20:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734304, -0.0324515955311978, -0.0335503995581136, -0.00801060507812425, 0.0211783869997512, -0.00109364720199431, 0.0169662248222208, -0.00568275755822893, 0.00567105258687628, 0.00838462854862292, -0.0208787193108001, -0.0109235523792603, -0.0162519217595033, -0.00499002532443533, -0.00958243145743145, -0.0108922666482287, -0.00187890812890814, -0.00466458889023305, -0.00558007245254536, -0.00469727032227032, 0.0139343556230002, 0.00245778931055952, 0.0120907247401236, -0.00415442218609135, 0.00464478520766673, 0.00965400476390776, -0.00643600317593853, 0.000719567030240145, 0.00498530662330959, 0.00661453944581401, -0.00464478520766678, -0.000415442218609127, 0.00215870109072045, 0.00323138210966672, 0.00290809553026392, 0.0, 0.0127646671417816, -0.00116524935543313, 0.0202632808954031, 0.00719357335632687, -0.00183832217438607, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, 0.020635115229186, 0.01143686949355, 0.00481385743762287, 0.000521115353897249, -0.00170195561890421, 0.0127646671417816, 0.00737991258440981, 0.00347410235931507, 0.000902598269554275, -0.000491312267361575, -0.000868525589828767};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 21:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734308, 0.024904712849524, 0.0553363732971482, -0.00431340273437462, -0.0141189246665007, -0.0153110608279201, -0.0080366328105256, 0.00883984509057836, -0.0159820572902877, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, -0.00145539597846298, 0.0301571095694136, 0.0116492303992304, -0.00352848074520084, 0.0191648629148629, 0.00976309302606918, 0.00132858867917747, -0.00263047138047137, 0.0, -0.0196623144844763, 0.00219831358911336, -0.00727023882565976, -0.0148633126645335, 0.00402250198496157, 0.00724050357293085, -0.0129522065443228, -0.00685479660705068, 0.00457929653940971, 0.00928957041533352, -0.00664707549774611, -0.0057565362419212, -0.000293762009969707, 0.00290809553026392, 0.0, 0.0, 0.0, -0.0202632808954031, 0.00411061334647247, -0.000459580543596518, -0.0399597288148542, -0.00303983956440067, 0.00427258096992149, -0.00100913554354854, -0.0464290092656686, -0.00908221989193681, 0.00270779480866286, 0.00234501909253767, -0.00127646671417816, -0.031911667854454, -0.00776832903622084, 0.000868525589828741, 0.00180519653910859, 0.000245656133680784, -0.000868525589828764};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 22:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734313, -0.0354703486038675, -0.0492363006502186, -0.00431340273437463, 0.0141189246665008, 0.00656188321196573, 0.00982255121286465, -0.016416855168217, 0.0149509568199466, 0.000729098134662877, 0.0, 0.00336109303977242, -0.000485131992821003, -0.0282044909641998, 0.0124007936507937, -0.00291483191994852, -0.0353234728234728, 0.00889526253486301, 0.00239145962251945, -0.00263047138047139, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920008, 0.000804500396992315, 0.00402250198496156, 0.0143913406048031, -0.00934744991870546, 0.00254405363300537, -0.00928957041533352, 0.0141250354327105, -0.00287826812096063, -0.00381890612960613, 0.00290809553026392, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320202, 0.00233049871086625, -0.000336378514516179, 0.0515877880729651, -0.00840946286290444, -0.00300866089851428, 0.00286613444643491, -0.000850977809452102, 0.0425488904726053, -0.00388416451811045, -0.00347410235931505, 0.00120346435940571, 0.000982624534723158, -0.00086852558982876};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 23:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0.0, -0.0174983552319087, 0.00357183680467805, 0.0132597676358676, -0.0175287079957995, 0.00838462854862291, 0.0, 0.0, 0.0, -0.00867830491206148, 0.003006253006253, -0.000613648825252317, 0.0225468975468975, -0.019200749617936, 0.0122230158484327, -0.00469727032227032, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0160900079398463, 0.00719567030240154, -0.0024926533116548, 0.000508810726601093, 0.00464478520766676, -0.0108014976838374, 0.0107935054536023, -0.00734405024924251, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0257938940364825, 0.0134551405806471, -0.00601732179702859, 0.00208446141558904, -0.000425488904726058, -0.031911667854454, 0.0135945758133865, -0.00347410235931504, -0.000902598269554289, 0.00171959293576549, -0.00086852558982877};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 24:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734306, -0.0452812960900435, -0.0113287063442981, -0.00801060507812427, -0.00141189246665008, 0.0164047080299143, 0.0133943880175427, 0.0145226026488074, 0.0118576554089231, 0.00838462854862292, -0.00894802256177149, -0.0142846454190327, -0.0123708658169354, -0.014536160727703, -0.0125886844636845, -0.0102786178229764, -0.00939454064454065, -0.00813591085505764, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, 0.00245778931055954, 0.00219831358911338, 0.00830884437218262, 0.00743165633226679, 0.00643600317593851, 0.00965400476390779, 0.00863480436288184, 0.00747795993496439, 0.00610572871921294, 0.00650269929073342, 0.00581619106052785, 0.00503696921168106, 0.00411266813957582, 0.00290809553026391, 0.0352796762358899, 0.0191470007126724, 0.017478740331497, 0.00788016479265675, 0.00719357335632688, 0.00643412761035129, 0.00095142211463938, 0.000868525589828755, 0.000776832903622077, 0.000672757029032348, -0.00221090520312707, -0.00201827108709705, -0.00180519653910857, -0.00156334606169178, -0.00127646671417816, -0.00212744452363026, -0.00194208225905522, -0.00173705117965752, -0.00150433044925715, -0.00122828066840393, -0.000868525589828761};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 25:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0603750614533912, -0.00610007264692968, -0.00431340273437459, -0.00564756986660033, -0.0218729440398858, -0.0178591840233902, 0.00126283501293976, 0.00103110047034113, 0.000729098134662864, 0.0417574386216003, 0.0168054651988621, 0.0145539597846298, -0.000867830491206143, -0.000751563251563245, -0.000613648825252304, -0.00526094276094276, -0.00455611007883229, -0.00372004830169692, -0.00263047138047139, 0.00557374224920013, -0.0196623144844763, -0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992314, 0.00321800158796925, 0.00287826812096062, 0.00249265331165479, 0.00203524290640429, 0.00650269929073342, 0.00581619106052785, 0.00503696921168108, 0.00411266813957584, 0.00290809553026395, -0.0705593524717798, -0.0127646671417816, -0.0116524935543313, 0.00450295131008956, 0.00411061334647247, 0.00367664434877215, 0.00380568845855755, 0.00347410235931509, 0.00310733161448836, 0.00269102811612943, -0.00073696840104236, -0.000672757029032368, -0.000601732179702871, -0.000521115353897269, -0.000425488904726046, -0.00212744452363026, -0.00194208225905521, -0.00173705117965753, -0.00150433044925714, -0.00122828066840394, -0.000868525589828776};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 26:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734312, -0.0603750614533913, -0.00610007264692976, -0.00431340273437462, -0.00564756986660025, 0.0218729440398858, 0.0178591840233902, 0.00126283501293977, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.0168054651988621, 0.0145539597846299, 0.000867830491206133, 0.000751563251563239, 0.000613648825252299, -0.00526094276094275, -0.00455611007883226, -0.0037200483016969, -0.00263047138047138, 0.00557374224920004, 0.0196623144844763, 0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992315, -0.00321800158796928, -0.00287826812096062, -0.0024926533116548, -0.0020352429064043, 0.00650269929073342, 0.00581619106052786, 0.00503696921168108, 0.00411266813957581, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.0116524935543313, -0.00450295131008957, -0.00411061334647248, -0.00367664434877215, 0.00380568845855754, 0.00347410235931504, 0.00310733161448834, 0.00269102811612942, 0.000736968401042371, 0.000672757029032358, 0.000601732179702873, 0.000521115353897267, 0.000425488904726051, -0.00212744452363026, -0.00194208225905523, -0.00173705117965754, -0.00150433044925714, -0.00122828066840393, -0.000868525589828762};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 27:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812422, -0.0014118924666501, -0.0164047080299144, -0.0133943880175427, 0.0145226026488074, 0.0118576554089232, 0.00838462854862291, 0.00894802256177149, -0.0142846454190328, -0.0123708658169354, 0.014536160727703, 0.0125886844636845, 0.0102786178229764, -0.00939454064454066, -0.00813591085505765, -0.00664294339588734, -0.00469727032227034, -0.0083606133738001, -0.00245778931055953, -0.00219831358911339, 0.00830884437218266, 0.00743165633226679, 0.00643600317593851, -0.00965400476390776, -0.00863480436288184, -0.00747795993496436, -0.00610572871921293, 0.00650269929073342, 0.00581619106052785, 0.00503696921168106, 0.00411266813957581, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, 0.017478740331497, -0.00788016479265674, -0.00719357335632687, -0.00643412761035128, 0.000951422114639395, 0.000868525589828766, 0.000776832903622087, 0.000672757029032357, 0.00221090520312706, 0.00201827108709707, 0.00180519653910857, 0.00156334606169176, 0.00127646671417816, -0.00212744452363025, -0.00194208225905521, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828762};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 28:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.078755617076958, 0.0679219441350653, -0.00435719474780703, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, -0.0267887760350855, 0.0126283501293977, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, 0.0130985638061668, -0.0325436434202305, -0.0229226791726792, -0.0248527774227189, 0.0248015873015873, 0.0169226945785199, 0.0164744996218006, 0.0142797017797018, 0.0, 0.0245778931055954, 0.0219831358911339, -0.0363511941282991, -0.00743165633226679, -0.0345935170706696, 0.0362025178646541, 0.0, 0.0130864298861876, 0.0335815079556711, -0.0232239260383337, 0.00166176887443654, -0.00359783515120075, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0226083734055987, -0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.00908221989193681, 0.0, 0.0333014729371016, 0.0270779480866287, 0.00703505727761299, -0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598858, 0.00196524906944629, 0.00607967912880136};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 29:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.00377344134083692, 0.00217859737390364, -0.0215670136718729, 0.0, 0.0984282481794863, -0.0312535720409329, -0.0852413633734344, -0.0180442582309701, 0.0309866707231716, 0.0, 0.0453747560369276, -0.0116431678277039, 0.0, 0.0236742424242425, -0.00138070985681771, -0.0202922077922078, -0.0335199527228374, -0.000797153207506548, 0.00394570707070708, 0.0, 0.0, 0.0, 0.0363511941282991, 0.024152883079867, -0.00724050357293084, -0.0603375297744236, 0.0, -0.039259289658563, 0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064561, 0.0, -0.0555024548951693, 0.0, 0.0164151336477636, -0.00680782247561684, 0.0, 0.0427258096992147, 0.00694820471863017, -0.0108311792346515, -0.00393049813889261, 0.00607967912880133};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 30:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0301875307266957, 0.0610007264692974, -0.00308100195312476, 0.0, 0.0218729440398858, 0.0178591840233903, 0.0757701007763862, -0.0412440188136457, -0.00729098134662866, 0.0, 0.0, 0.0, 0.0520698294723688, -0.0127765752765753, 0.00184094647575698, -0.00901875901875901, 0.0286384062098029, -0.0297603864135752, 0.0142797017797018, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0402250198496157, 0.014391340604803, -0.0137095932141013, 0.00407048581280863, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0370016365967796, -0.030086608985143, 0.014070114555226, -0.00340391123780845, 0.0, -0.0427258096992148, 0.0243187165152054, 0.0, -0.00982624534723141, 0.00607967912880137};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 31:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.021478804657352, 0.00377344134083707, -0.0196073763651313, 0.00924300585937421, 0.0, -0.010936472019943, 0.102690308134494, 0.0094712625970483, -0.0695992817480271, -0.060150596109686, 0.0, -0.00504163955965864, 0.0465726713108155, 0.00650872868404603, 0.0214195526695526, -0.00782402252196704, -0.00450937950937952, -0.0289638426440052, -0.0263060558477138, 0.000563672438672408, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, -0.0431740218144092, -0.00249265331165474, -0.0605484764655282, 0.0, 0.0299118397398575, -0.00431740218144093, 0.0185070066280912, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, -0.0451299134777144, -0.0281402291104519, 0.023827378664659, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.018239037386404};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 32:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, -0.0150937653633478, 0.0130715842434209, 0.00924300585937411, 0.0, -0.0109364720199429, -0.00892959201169512, 0.00947126259704831, 0.123732056440937, -0.060150596109686, 0.0, 0.0, 0.0, -0.00650872868404609, 0.0574945887445887, -0.0266937238984759, 0.00676406926406926, 0.0126920209338899, 0.00637722566005184, 0.000563672438672477, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0431740218144092, 0.0199412264932383, -0.0142467003448302, 0.0, -0.0448677596097863, 0.0215870109072046, -0.0616900220936373, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0451299134777144, -0.0375203054806026, 0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 33:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, 0.0301875307266957, 0.0174287789912277, 0.0585390371093694, 0.0, 0.0218729440398859, 0.0178591840233903, -0.0189425251940965, 0.0103110047034114, 0.0838462854862291, 0.0, 0.0, 0.0, 0.0, 0.00526094276094274, 0.0533874477969518, 0.0, -0.00455611007883225, 0.0308232573569173, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.034897146363167, 0.0284934006896603, 0.0, 0.0, -0.0302218152700865, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0437736897273697, -0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 34:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769581, -0.0679219441350654, -0.00435719474780698, -0.00308100195312479, 0.0705946233325039, 0.0328094160598288, 0.0267887760350854, 0.0126283501293977, 0.0360885164619401, -0.00729098134662864, -0.0298267418725716, 0.0151249186789758, 0.0130985638061669, 0.0325436434202305, 0.0229226791726792, 0.024852777422719, 0.0248015873015874, 0.0169226945785198, 0.0164744996218006, 0.0142797017797018, 0.0, -0.0245778931055953, -0.0219831358911338, -0.036351194128299, -0.00743165633226677, -0.0345935170706695, -0.0362025178646541, 0.0, -0.0130864298861876, -0.0335815079556711, -0.0232239260383337, 0.00166176887443655, -0.00359783515120075, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, -0.0226083734055987, 0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.0090822198919368, 0.0, -0.0333014729371016, -0.0270779480866286, -0.00703505727761296, 0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598855, 0.00196524906944629, 0.00607967912880136};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 35:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, -0.00377344134083697, 0.00217859737390366, -0.0215670136718729, 0.0, -0.0984282481794862, 0.0312535720409329, -0.0852413633734344, -0.0180442582309701, 0.0309866707231716, 0.0, 0.0453747560369275, -0.0116431678277039, 0.0, -0.0236742424242424, 0.00138070985681773, -0.0202922077922079, -0.0335199527228374, -0.000797153207506549, 0.00394570707070709, 0.0, 0.0, 0.0, 0.036351194128299, 0.024152883079867, -0.00724050357293081, 0.0603375297744236, 0.0, 0.0392592896585629, -0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064561, 0.0, 0.0555024548951693, 0.0, -0.0164151336477636, 0.00680782247561685, 0.0, 0.0427258096992147, 0.00694820471863019, -0.0108311792346515, -0.00393049813889259, 0.00607967912880134};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 36:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, -0.0301875307266958, 0.0610007264692974, -0.00308100195312485, 0.0, -0.021872944039886, -0.0178591840233903, 0.0757701007763861, -0.0412440188136456, -0.00729098134662859, 0.0, 0.0, 0.0, -0.0520698294723688, 0.0127765752765752, -0.00184094647575697, -0.00901875901875897, 0.0286384062098028, -0.0297603864135752, 0.0142797017797018, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0402250198496157, -0.0143913406048031, 0.0137095932141014, -0.00407048581280863, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0370016365967795, 0.0300866089851429, -0.014070114555226, 0.00340391123780843, 0.0, -0.0427258096992147, 0.0243187165152053, 0.0, -0.00982624534723143, 0.00607967912880138};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 37:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573516, -0.00377344134083713, -0.0196073763651312, 0.00924300585937421, 0.0, 0.0109364720199428, -0.102690308134494, 0.00947126259704829, -0.0695992817480271, -0.060150596109686, 0.0, -0.00504163955965856, 0.0465726713108155, -0.00650872868404608, -0.0214195526695527, 0.00782402252196704, -0.00450937950937948, -0.0289638426440052, -0.0263060558477138, 0.000563672438672415, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, 0.0431740218144092, 0.00249265331165474, 0.0605484764655282, 0.0, 0.0299118397398575, -0.00431740218144093, 0.0185070066280912, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, 0.0451299134777144, 0.0281402291104519, -0.023827378664659, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.018239037386404};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 38:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937418, 0.0, 0.0109364720199429, 0.00892959201169517, 0.00947126259704834, 0.123732056440937, -0.0601505961096861, 0.0, 0.0, 0.0, 0.00650872868404608, -0.0574945887445887, 0.0266937238984759, 0.00676406926406923, 0.0126920209338899, 0.00637722566005182, 0.000563672438672456, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0431740218144092, -0.0199412264932384, 0.0142467003448302, 0.0, -0.0448677596097862, 0.0215870109072046, -0.0616900220936372, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0451299134777144, 0.0375203054806026, -0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 39:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, -0.0301875307266958, 0.0174287789912279, 0.0585390371093693, 0.0, -0.0218729440398857, -0.0178591840233903, -0.0189425251940965, 0.0103110047034115, 0.083846285486229, 0.0, 0.0, 0.0, 0.0, -0.00526094276094277, -0.0533874477969517, 0.0, -0.00455611007883225, 0.0308232573569172, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0348971463631671, -0.0284934006896603, 0.0, 0.0, -0.0302218152700864, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0437736897273697, 0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 40:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769573, -0.0377344134083695, -0.0566435317214904, -0.00308100195312477, 0.0423567739995023, 0.0546823600997147, 0.0446479600584757, 0.0378850503881931, 0.00515550235170579, -0.00729098134662861, 0.0417574386216002, -0.00504163955965862, 0.0295930515620807, -0.0247331689993752, 0.0101461038961039, 0.0266937238984759, -0.0202922077922078, 0.00216957622801539, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00219831358911336, -0.00519302773261414, -0.0130053985814669, -0.0168945083368385, 0.00402250198496157, -0.0158304746652834, -0.026796023100289, -0.0295110221428625, 0.00464478520766675, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, -0.0226083734055987, -0.0367664434877216, 0.0, -0.0047768907440582, -0.00776832903622085, -0.00908221989193678, 0.0, 0.00370016365967795, 0.00601732179702857, 0.007035057277613, 0.00680782247561685, 0.0, 0.00427258096992149, 0.0069482047186301, 0.00812338442598857, 0.00786099627778516, 0.00607967912880134};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 41:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.0, -0.00435719474780704, -0.0215670136718729, -0.127070321998507, 0.0, 0.0, 0.0284137877911449, 0.0360885164619399, 0.0309866707231716, 0.0, 0.0453747560369276, -0.0422064833754265, 0.0, 0.0, 0.0, -0.0202922077922078, -0.00618329224984381, 0.00159430641501296, 0.00394570707070708, 0.0836061337380012, 0.0, 0.0, -0.0103860554652283, 0.024152883079867, 0.043443021437585, 0.0, 0.0, 0.0, 0.0, 0.00464478520766673, -0.0108014976838374, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, 0.0769064574585865, 0.0, 0.0, 0.0, 0.0, -0.00955378148811641, -0.0155366580724417, -0.0181644397838736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00427258096992146, 0.00694820471863015, 0.00812338442598859, 0.00786099627778516, 0.00607967912880135};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 42:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, 0.0377344134083696, -0.0566435317214905, -0.00308100195312481, 0.0423567739995022, -0.0546823600997145, -0.0446479600584757, 0.0378850503881931, 0.00515550235170578, -0.0072909813466285, -0.0417574386216002, -0.00504163955965861, 0.0295930515620806, 0.0247331689993752, -0.0101461038961039, -0.0266937238984759, -0.0202922077922078, 0.00216957622801541, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055953, 0.00219831358911336, -0.00519302773261412, -0.0130053985814669, -0.0168945083368386, -0.00402250198496156, 0.0158304746652834, 0.026796023100289, 0.0295110221428625, 0.00464478520766677, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, 0.0226083734055987, 0.0367664434877216, 0.0, -0.00477689074405817, -0.00776832903622087, -0.0090822198919368, 0.0, -0.00370016365967795, -0.00601732179702859, -0.00703505727761297, -0.00680782247561685, 0.0, 0.0042725809699215, 0.00694820471863008, 0.00812338442598857, 0.00786099627778517, 0.00607967912880136};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 43:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, -0.0188672067041848, 0.00653579212171051, 0.00924300585937421, 0.0141189246665008, 0.0, -0.111619900146189, -0.00315708753234941, -0.05413277469291, -0.060150596109686, -0.00894802256177145, -0.00252081977982928, -0.0276525235907967, 0.000650872868404633, 0.0360750360750361, -0.0188697013765088, 0.00112734487734487, 0.0374251899332651, 0.0199288301876619, 0.000563672438672408, 0.0, 0.0, 0.0593544669060614, 0.0, 0.0167212267476003, 0.00321800158796919, 0.0, -0.0043174021814409, 0.0224338798048931, 0.0463017761206981, 0.0, -0.00747795993496438, 0.018708742786244, 0.0431830154655461, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0620433733855302, 0.0, 0.0, 0.017478740331497, 0.0363288795677472, 0.0, 0.0, -0.00451299134777145, -0.00938007637015065, -0.0119136893323295, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.020635115229186, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 44:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.0188672067041847, 0.00653579212171058, 0.00924300585937418, 0.0141189246665008, 0.0, 0.111619900146189, -0.00315708753234937, -0.05413277469291, -0.0601505961096861, 0.0089480225617715, -0.0025208197798293, -0.0276525235907967, -0.000650872868404624, -0.0360750360750361, 0.0188697013765088, 0.00112734487734488, 0.0374251899332651, 0.019928830187662, 0.000563672438672449, 0.0, 0.0, -0.0593544669060614, 0.0, 0.0167212267476003, 0.00321800158796924, 0.0, 0.00431740218144091, -0.0224338798048931, -0.0463017761206981, 0.0, -0.00747795993496438, 0.0187087427862439, 0.0431830154655461, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0620433733855302, 0.0, 0.0, 0.0174787403314969, 0.0363288795677472, 0.0, 0.0, 0.00451299134777145, 0.00938007637015064, 0.0119136893323295, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.0206351152291861, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 45:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0, -0.0348575579824557, 0.0585390371093694, -0.0282378493330015, 0.0, 0.0, 0.00631417506469885, -0.0206220094068228, 0.0838462854862291, 0.0, 0.0, -0.00679184789949388, 0.0, 0.0, 0.0, 0.0, 0.00151870335961077, -0.0616465147138343, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0450520222315696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0100739384233621, -0.0329013451166065, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0565115904387179, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440067};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 46:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0377344134083697, -0.0217859737390348, -0.05237703320312, 0.0423567739995024, 0.0546823600997146, 0.0446479600584757, 0.0, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, -0.00921750786359889, 0.0273366604729936, -0.00789141414141415, -0.0248527774227189, 0.0187890812890813, -0.0034713219648246, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111906, -0.0241814494802472, -0.0197335053839338, -0.00928957041533347, -0.00241350119097692, -0.0394205194526234, -0.0158304746652834, -0.000623163327913719, 0.00712335017241506, -0.0325134964536671, -0.0141250354327105, -0.00215870109072046, 0.00411266813957582, 0.00540074884191869, 0.0, -0.0510586685671264, 0.00466099742173251, -0.0405265617908061, -0.0143871467126537, 0.00367664434877213, -0.0114170653756726, -0.00564541633388696, -0.00155366580724414, 0.00100913554354854, 0.010317557614593, 0.00571843474677502, 0.00240692871881142, 0.000260557676948619, -0.000850977809452114, 0.0127646671417816, 0.00737991258440984, 0.00347410235931509, 0.000902598269554295, -0.000491312267361566, -0.000868525589828747};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 47:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, 0.0, -0.0217859737390348, 0.00308100195312479, -0.127070321998507, 0.0, 0.0, 0.0473563129852413, 0.0257775117585286, 0.0127592173566001, 0.0, -0.0352914769176103, 0.0509388592462045, 0.0, 0.0, 0.0, 0.00225468975468974, -0.00943765659186684, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0.0, 0.0, 0.0477758551400502, 0.00928957041533349, -0.0144810071458617, 0.0, 0.0, 0.0, 0.0, -0.0325134964536671, -0.0141250354327105, -0.00215870109072046, 0.00411266813957582, 0.00540074884191877, 0.0, 0.0765880028506897, -0.00699149613259876, 0.0, 0.0, 0.0, -0.0228341307513453, -0.011290832667774, -0.00310733161448838, 0.00201827108709705, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, 0.00737991258440982, 0.00347410235931506, 0.000902598269554268, -0.000491312267361585, -0.00086852558982881};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 48:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769581, 0.0377344134083697, -0.0217859737390347, -0.0523770332031201, 0.0423567739995024, -0.0546823600997146, -0.0446479600584757, 0.0, 0.0257775117585285, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.00921750786359894, -0.0273366604729937, 0.00789141414141414, 0.0248527774227189, 0.0187890812890813, -0.00347132196482457, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111903, 0.0241814494802472, -0.0197335053839338, -0.00928957041533349, -0.00241350119097694, 0.0394205194526234, 0.0158304746652834, 0.000623163327913702, -0.00712335017241507, -0.0325134964536671, -0.0141250354327105, -0.00215870109072045, 0.00411266813957581, 0.00540074884191867, 0.0, -0.0510586685671264, 0.0046609974217325, 0.0405265617908061, 0.0143871467126537, -0.00367664434877215, -0.0114170653756726, -0.00564541633388696, -0.00155366580724416, 0.00100913554354855, -0.010317557614593, -0.00571843474677502, -0.00240692871881143, -0.000260557676948624, 0.000850977809452106, 0.0127646671417816, 0.00737991258440984, 0.00347410235931506, 0.000902598269554285, -0.000491312267361562, -0.000868525589828736};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 49:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573518, -0.0188672067041849, 0.0108929868695176, 0.00308100195312463, 0.0141189246665008, -0.109364720199429, 0.0223239800292377, -0.0789271883087356, -0.0128887558792642, 0.0127592173566001, -0.00894802256177147, -0.0277290175781224, 0.00145539597846298, -0.0058578558156415, 0.0383297258297259, -0.0124263887113595, 0.0191648629148628, 0.0348216984596467, 0.00717437886755833, -0.0129644660894661, 0.0, 0.0589869434534289, -0.00659494076734012, 0.00727023882565972, 0.0148633126645335, -0.0040225019849616, 0.0506835250105158, -0.00431740218144091, -0.00810112326287806, 0.00356167508620759, 0.0650269929073342, -0.00166176887443653, -0.0115130724838425, -0.00205633406978792, 0.00540074884191875, 0.0, 0.0, 0.0, 0.0607898426862092, -0.0123318400394175, 0.0013787416307896, 0.0399597288148542, 0.00303983956440064, -0.00427258096992151, 0.00100913554354853, -0.0154763364218896, -0.00302740663064563, 0.00090259826955432, 0.000781673030845895, -0.000425488904726053, -0.031911667854454, -0.00776832903622088, 0.000868525589828737, 0.00180519653910857, 0.000245656133680776, -0.000868525589828788};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 50:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, 0.0188672067041847, 0.0108929868695174, 0.00308100195312465, 0.0141189246665008, 0.109364720199429, -0.0223239800292378, -0.0789271883087356, -0.0128887558792643, 0.0127592173566001, 0.00894802256177148, -0.0277290175781224, 0.00145539597846297, 0.00585785581564141, -0.0383297258297259, 0.0124263887113595, 0.019164862914863, 0.0348216984596467, 0.00717437886755831, -0.0129644660894661, 0.0, -0.0589869434534289, 0.00659494076734012, 0.00727023882565982, 0.0148633126645335, -0.00402250198496154, -0.0506835250105158, 0.00431740218144091, 0.00810112326287806, -0.00356167508620755, 0.0650269929073342, -0.00166176887443652, -0.0115130724838425, -0.0020563340697879, 0.00540074884191873, 0.0, 0.0, 0.0, -0.0607898426862092, 0.0123318400394175, -0.00137874163078958, 0.0399597288148542, 0.00303983956440069, -0.00427258096992147, 0.00100913554354853, 0.0154763364218896, 0.00302740663064561, -0.000902598269554305, -0.000781673030845883, 0.000425488904726056, -0.031911667854454, -0.00776832903622088, 0.000868525589828765, 0.00180519653910858, 0.00024565613368079, -0.000868525589828768};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 51:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.078755617076958, 0.0, 0.0435719474780696, -0.05237703320312, -0.0282378493330015, 0.0, 0.0, 0.0631417506469885, -0.0515550235170571, 0.0291639253865144, 0.0, -0.00672218607954483, 0.000970263985642005, 0.0, 0.0, 0.0, -0.0601250601250601, -0.0125835421224891, 0.0286975154702333, -0.015031265031265, 0.0, 0.0, 0.0, -0.0436214329539588, 0.0111474844984002, -0.00160900079398464, 0.0, 0.0, 0.0, 0.0, -0.0650269929073342, 0.031573608614294, -0.00287826812096058, -0.00822533627915168, 0.00540074884191872, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0532796384198057, 0.018239037386404, -0.00466099742173252, 0.000672757029032361, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0425488904726053, -0.00388416451811041, -0.00347410235931508, 0.00120346435940574, 0.000982624534723166, -0.000868525589828754};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 52:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281179, -0.0705946233325039, -0.0546823600997146, -0.0446479600584757, -0.0315708753234942, -0.0257775117585285, -0.0182274533665715, 0.0894802256177148, -0.0151249186789758, -0.0130985638061669, 0.0585785581564149, -0.028183621933622, 0.0690354928408859, 0.074404761904762, 0.00976309302606919, 0.039857660375324, 0.0597492784992785, 0.0, 0.0737336793167861, 0.0659494076734016, 0.0363511941282991, 0.00743165633226676, 0.0345935170706695, -0.0120675059548847, 0.0431740218144092, 0.0155790831978425, -0.0254405363300538, -0.0232239260383337, 0.0465295284842228, 0.0251848460584054, -0.0146881004984851, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, 0.0678251202167962, -0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.00908221989193678, 0.0, -0.0111004909790339, -0.00902598269554281, -0.00234501909253764, 0.00340391123780841, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598859, 0.00196524906944629, 0.0060796791288014};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 53:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, 0.0546823600997147, 0.0446479600584758, -0.0315708753234942, -0.0257775117585286, -0.0182274533665716, -0.0894802256177148, -0.0151249186789759, -0.0130985638061668, -0.058578558156415, 0.028183621933622, -0.0690354928408859, 0.0744047619047619, 0.00976309302606916, 0.0398576603753239, 0.0597492784992785, 0.0, -0.0737336793167862, -0.0659494076734017, 0.036351194128299, 0.00743165633226679, 0.0345935170706696, 0.0120675059548846, -0.0431740218144092, -0.0155790831978425, 0.0254405363300538, -0.0232239260383337, 0.0465295284842228, 0.0251848460584053, -0.014688100498485, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, -0.0678251202167963, 0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.00908221989193679, 0.0, 0.0111004909790338, 0.00902598269554289, 0.00234501909253761, -0.00340391123780841, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598856, 0.00196524906944628, 0.00607967912880138};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 54:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.217859737390348, -0.0770250488281177, 0.0, 0.0, 0.0, -0.0947126259704828, 0.0515550235170571, -0.0182274533665715, 0.0, -0.0907495120738551, 0.0232863356554078, 0.0, 0.0, 0.0, -0.0946969696969696, -0.0227805503941613, -0.0797153207506479, 0.0597492784992785, 0.0, 0.0, 0.0, -0.0727023882565981, -0.0483057661597341, 0.0144810071458617, 0.0, 0.0, 0.0, 0.0, 0.0464478520766673, -0.0781031370985168, 0.0431740218144092, 0.0293762009969701, -0.0328199352701214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0668764704168149, 0.0310733161448835, -0.0060548132612912, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0427258096992147, 0.00694820471863011, -0.0108311792346515, -0.0039304981388926, 0.00607967912880135};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    case 55:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.0, 0.231075146484353, 0.0, 0.0, 0.0, 0.0, 0.0, -0.109364720199429, 0.0, 0.0100832791193174, -0.0931453426216311, 0.0, 0.0, 0.0, -0.00450937950937958, -0.104139658944738, 0.0, -0.0924422799422799, 0.0, 0.0, 0.0, 0.0, -0.0668849069904009, -0.0579240285834467, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.0647610327216138, 0.0, 0.0835038859404355, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0699149613259879, 0.0242192530451648, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338355, 0.0270779480866287, 0.0, -0.0182390373864041};
      
      // Compute value(s).
      for (unsigned int r = 0; r < 56; r++)
      {
        *values += coefficients0[r]*basisvalues[r];
      }// end loop over 'r'
        break;
      }
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    // Helper variable to hold values of a single dof.
    double dof_values = 0.0;
    
    // Loop dofs and call evaluate_basis.
    for (unsigned int r = 0; r < 56; r++)
    {
      evaluate_basis(r, &dof_values, coordinates, c);
      values[r] = dof_values;
    }// end loop over 'r'
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    const double K_00 = d_00 / detJ;
    const double K_01 = d_10 / detJ;
    const double K_02 = d_20 / detJ;
    const double K_10 = d_01 / detJ;
    const double K_11 = d_11 / detJ;
    const double K_12 = d_21 / detJ;
    const double K_20 = d_02 / detJ;
    const double K_21 = d_12 / detJ;
    const double K_22 = d_22 / detJ;
    
    // Compute constants
    const double C0 = x[3][0] + x[2][0] + x[1][0] - x[0][0];
    const double C1 = x[3][1] + x[2][1] + x[1][1] - x[0][1];
    const double C2 = x[3][2] + x[2][2] + x[1][2] - x[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double X = (d_00*(2.0*coordinates[0] - C0) + d_10*(2.0*coordinates[1] - C1) + d_20*(2.0*coordinates[2] - C2)) / detJ;
    double Y = (d_01*(2.0*coordinates[0] - C0) + d_11*(2.0*coordinates[1] - C1) + d_21*(2.0*coordinates[2] - C2)) / detJ;
    double Z = (d_02*(2.0*coordinates[0] - C0) + d_12*(2.0*coordinates[1] - C1) + d_22*(2.0*coordinates[2] - C2)) / detJ;
    
    
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 3;
    }// end loop over 'r'
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      combinations[row] = new unsigned int [n];
      for (unsigned int col = 0; col < n; col++)
        combinations[row][col] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] = {{K_00, K_01, K_02}, {K_10, K_11, K_12}, {K_20, K_21, K_22}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values. Assuming that values is always an array.
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      values[r] = 0.0;
    }// end loop over 'r'
    
    switch (i)
    {
    case 0:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923465, 0.00609788120679257, 0.00352061335622799, 0.00248944957812477, 0.00705946233325034, 0.00546823600997147, 0.00446479600584761, 0.00315708753234942, 0.00257775117585288, 0.00182274533665717, -0.00298267418725715, -0.00252081977982931, -0.00218309396769448, -0.00195261860521384, -0.00169101731601733, -0.00138070985681771, -0.0011273448773449, -0.000976309302606944, -0.000797153207506486, -0.000563672438672447, 0.00278687112460007, 0.00245778931055955, 0.00219831358911339, 0.00207721109304567, 0.0018579140830667, 0.00160900079398463, 0.00160900079398464, 0.00143913406048031, 0.0012463266558274, 0.00101762145320216, 0.000928957041533369, 0.000830884437218256, 0.00071956703024016, 0.000587524019939406, 0.000415442218609138, -0.00705593524717797, -0.00638233357089079, -0.00582624677716565, -0.00562868913761196, -0.00513826668309062, -0.00459580543596521, -0.00475711057319694, -0.00434262794914382, -0.00388416451811044, -0.00336378514516178, -0.00368484200521178, -0.00336378514516176, -0.00300866089851429, -0.00260557676948629, -0.00212744452363027, -0.00212744452363026, -0.00194208225905519, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828763};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 1:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923512, -0.00609788120679247, 0.00352061335622801, 0.00248944957812479, 0.00705946233325039, -0.00546823600997145, -0.00446479600584757, 0.00315708753234941, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982929, -0.00218309396769447, 0.00195261860521384, 0.00169101731601731, 0.00138070985681771, -0.00112734487734488, -0.000976309302606922, -0.00079715320750647, -0.000563672438672437, 0.00278687112460002, -0.00245778931055953, -0.00219831358911338, 0.00207721109304566, 0.00185791408306669, 0.00160900079398462, -0.00160900079398462, -0.00143913406048031, -0.0012463266558274, -0.00101762145320215, 0.000928957041533356, 0.00083088443721826, 0.000719567030240153, 0.0005875240199394, 0.000415442218609126, 0.00705593524717798, -0.00638233357089082, -0.00582624677716566, 0.00562868913761197, 0.00513826668309063, 0.0045958054359652, -0.00475711057319694, -0.00434262794914382, -0.00388416451811044, -0.00336378514516178, 0.00368484200521179, 0.00336378514516178, 0.0030086608985143, 0.00260557676948629, 0.00212744452363027, -0.00212744452363027, -0.00194208225905521, -0.00173705117965753, -0.00150433044925715, -0.00122828066840393, -0.000868525589828761};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 2:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923495, 0.0, -0.00704122671245611, 0.00248944957812476, 0.0, 0.0, 0.0, 0.00947126259704834, -0.0051555023517057, 0.00182274533665714, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00450937950937952, -0.00292892790782074, 0.00159430641501296, -0.000563672438672428, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0046447852076667, -0.00332353774887305, 0.00215870109072045, -0.00117504803987881, 0.000415442218609132, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, -0.0097104112952761, 0.00694820471863011, -0.00451299134777143, 0.00245656133680786, -0.000868525589828765};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 3:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.00945067404923497, 0.0, 0.0, -0.0074683487343743, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0109364720199429, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00563672438672438, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00623163327913698, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 4:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0, 0.0374718748311399, -0.0289614183593723, 0.0, 0.0, 0.0, 0.0284137877911448, 0.0051555023517057, -0.00619733414463434, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0315656565656566, -0.0113902751970807, 0.00265717735835493, -0.00018789081289078, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0232239260383337, -0.00166176887443654, -0.00647610327216137, 0.00646276421933344, -0.00290809553026389, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0213629048496074, -0.0277928188745204, 0.0243701532779658, -0.0157219925555703, 0.00607967912880132};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 5:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.010023442173431, 0.0, -0.028757485335526, 0.0536094339843701, 0.0, 0.0, 0.0, -0.0189425251940965, 0.0154665070551172, 0.0153110608279201, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0045093795093795, 0.0370997534990628, -0.0281660799985623, 0.0108976671476671, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.00431740218144092, -0.0123380044187275, 0.00872428659079175, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338355, -0.0487403065559316, 0.0412702304583721, -0.018239037386404};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 6:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0, 0.0409576306293855, -0.0449826285156206, 0.0, 0.0, 0.0, 0.0189425251940965, -0.0051555023517057, -0.021143845905223, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00455611007883226, 0.0356061766019561, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0302218152700864, 0.00411266813957585, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0379091273212801, -0.0550269739444961, 0.0303983956440067};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 7:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0, -0.0148144621425436, 0.0449826285156207, 0.0, 0.0, 0.0, 0.0, 0.0206220094068228, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0106287094334197, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0205633406978791, 0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.03439185871531, -0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 8:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0324515955311978, -0.0187359374155699, -0.0289614183593722, 0.0211783869997512, -0.0164047080299144, 0.00446479600584757, 0.0094712625970483, -0.00257775117585284, -0.00619733414463434, 0.0208787193108001, -0.0176457384588052, -0.00848980987436739, 0.0136683302364968, 0.00657617845117845, 0.00230118309469618, -0.00789141414141414, -0.0037967583990269, -0.00132858867917748, -0.000187890812890797, 0.0139343556230002, -0.0122889465527977, -0.00109915679455668, 0.0103860554652283, 0.000928957041533339, -0.00482700238195389, -0.00804500396992314, -0.000719567030240159, 0.0037389799674822, 0.00559691799261186, 0.00464478520766673, 0.000415442218609135, -0.00215870109072048, -0.00323138210966671, -0.00290809553026391, 0.0, 0.0, 0.0128177429097644, 0.0, -0.0113041867027994, -0.0183832217438608, 0.0, 0.00955378148811638, 0.0155366580724417, 0.0181644397838736, 0.0, -0.0074003273193559, -0.0120346435940572, -0.014070114555226, -0.0136156449512337, 0.0, 0.00427258096992146, 0.00694820471863012, 0.00812338442598859, 0.00786099627778515, 0.00607967912880133};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 9:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734313, -0.0249047128495238, 0.0143787426677629, 0.0536094339843699, -0.0141189246665007, 0.0109364720199429, 0.0133943880175427, -0.00631417506469884, -0.00773325352755864, 0.0153110608279201, -0.00298267418725714, 0.00252081977982934, 0.0276525235907967, -0.00195261860521383, -0.0214195526695527, -0.0243925408037797, 0.00112734487734489, 0.0123665844996876, 0.0140830399992812, 0.0108976671476671, 0.0, 0.0, 0.0197848223020205, 0.0, -0.0167212267476002, -0.00321800158796926, 0.0, 0.0129522065443228, 0.00249265331165478, -0.0106850252586227, 0.0, -0.00747795993496437, -0.00143913406048031, 0.00616900220936374, 0.00872428659079175, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, 0.0135389740433143, 0.0281402291104519, 0.0357410679969885, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.020635115229186, -0.018239037386404};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 10:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0354703486038675, -0.0204788153146927, -0.0449826285156207, 0.0141189246665008, -0.0109364720199429, -0.00446479600584751, 0.00631417506469886, 0.00257775117585291, -0.021143845905223, 0.0, 0.0, 0.00339592394974697, 0.0, -0.0026304713804714, 0.030835853468929, 0.0, 0.00151870335961077, -0.017803088300978, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, -0.0174485731815835, 0.00356167508620757, 0.0, 0.0, 0.0100739384233621, -0.00205633406978792, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, -0.0218868448636848, -0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440067};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 11:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, -0.0128297005588457, 0.00740723107127187, 0.0449826285156207, 0.0, 0.0, 0.0178591840233903, 0.0, -0.0103110047034115, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00920473237878483, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489395, 0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0297842233308237, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 12:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0324515955311978, -0.0335503995581135, -0.00801060507812423, 0.0211783869997512, 0.00109364720199426, -0.0169662248222207, -0.00568275755822893, 0.00567105258687628, 0.00838462854862287, 0.0208787193108001, -0.0109235523792603, -0.0162519217595033, 0.00499002532443534, 0.00958243145743148, 0.0108922666482287, -0.00187890812890813, -0.00466458889023306, -0.00558007245254537, -0.00469727032227031, 0.0139343556230002, -0.00245778931055953, -0.0120907247401236, -0.00415442218609133, 0.00464478520766674, 0.00965400476390777, 0.00643600317593853, -0.000719567030240148, -0.00498530662330958, -0.00661453944581401, -0.00464478520766674, -0.000415442218609135, 0.00215870109072044, 0.00323138210966672, 0.00290809553026393, 0.0, 0.0127646671417816, -0.00116524935543313, -0.0202632808954031, -0.00719357335632687, 0.00183832217438609, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, -0.020635115229186, -0.01143686949355, -0.00481385743762288, -0.000521115353897266, 0.00170195561890421, 0.0127646671417816, 0.00737991258440982, 0.00347410235931507, 0.000902598269554292, -0.000491312267361576, -0.000868525589828768};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 13:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, -0.0249047128495239, 0.0553363732971483, -0.00431340273437465, -0.0141189246665008, 0.0153110608279201, 0.00803663281052558, 0.00883984509057834, -0.0159820572902877, 0.000729098134662859, -0.00298267418725716, 0.0277290175781224, -0.00145539597846296, -0.0301571095694136, -0.0116492303992304, 0.00352848074520085, 0.0191648629148629, 0.00976309302606916, 0.00132858867917748, -0.00263047138047137, 0.0, 0.0196623144844763, -0.00219831358911337, -0.00727023882565978, -0.0148633126645335, 0.00402250198496157, -0.00724050357293084, 0.0129522065443227, 0.00685479660705068, -0.0045792965394097, 0.00928957041533348, -0.00664707549774609, -0.0057565362419212, -0.000293762009969712, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0202632808954031, -0.00411061334647248, 0.000459580543596519, -0.0399597288148542, -0.00303983956440068, 0.00427258096992148, -0.00100913554354854, 0.0464290092656685, 0.00908221989193679, -0.00270779480866284, -0.00234501909253765, 0.00127646671417816, -0.031911667854454, -0.00776832903622086, 0.000868525589828756, 0.00180519653910857, 0.000245656133680793, -0.000868525589828769};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 14:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734309, 0.0354703486038674, -0.0492363006502187, -0.00431340273437464, 0.0141189246665008, -0.00656188321196573, -0.00982255121286465, -0.016416855168217, 0.0149509568199466, 0.000729098134662881, 0.0, 0.0033610930397724, -0.000485131992821013, 0.0282044909641998, -0.0124007936507937, 0.0029148319199485, -0.0353234728234728, 0.00889526253486302, 0.00239145962251946, -0.00263047138047138, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920009, 0.000804500396992313, -0.00402250198496156, -0.0143913406048031, 0.00934744991870547, -0.00254405363300537, -0.00928957041533351, 0.0141250354327105, -0.00287826812096064, -0.00381890612960614, 0.00290809553026391, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320203, 0.00233049871086626, -0.000336378514516182, -0.0515877880729651, 0.00840946286290446, 0.00300866089851428, -0.00286613444643492, 0.000850977809452106, 0.0425488904726053, -0.00388416451811045, -0.00347410235931507, 0.00120346435940573, 0.00098262453472315, -0.000868525589828758};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 15:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734311, -0.0128297005588456, 0.0448791059024116, -0.00801060507812425, 0.0, 0.0174983552319087, -0.00357183680467803, 0.0132597676358675, -0.0175287079957995, 0.00838462854862292, 0.0, 0.0, 0.0, 0.00867830491206147, -0.00300625300625301, 0.000613648825252321, 0.0225468975468976, -0.019200749617936, 0.0122230158484327, -0.00469727032227033, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0160900079398463, -0.00719567030240153, 0.0024926533116548, -0.000508810726601081, 0.00464478520766676, -0.0108014976838374, 0.0107935054536023, -0.00734405024924251, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0257938940364826, -0.0134551405806471, 0.00601732179702859, -0.00208446141558904, 0.000425488904726057, -0.031911667854454, 0.0135945758133865, -0.00347410235931503, -0.000902598269554298, 0.0017195929357655, -0.000868525589828767};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 16:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734316, -0.0324515955311979, -0.0187359374155699, -0.0289614183593722, 0.0211783869997512, 0.0164047080299144, -0.00446479600584758, 0.00947126259704828, -0.00257775117585287, -0.00619733414463427, -0.0208787193108001, -0.0176457384588051, -0.0084898098743674, -0.0136683302364968, -0.00657617845117843, -0.00230118309469622, -0.00789141414141414, -0.00379675839902689, -0.00132858867917748, -0.0001878908128908, 0.0139343556230002, 0.0122889465527977, 0.00109915679455668, 0.0103860554652283, 0.000928957041533344, -0.0048270023819539, 0.00804500396992314, 0.000719567030240151, -0.00373897996748219, -0.00559691799261186, 0.00464478520766674, 0.000415442218609132, -0.00215870109072046, -0.00323138210966671, -0.00290809553026394, 0.0, 0.0, 0.0128177429097644, 0.0, 0.0113041867027994, 0.0183832217438608, 0.0, 0.0095537814881164, 0.0155366580724417, 0.0181644397838736, 0.0, 0.00740032731935589, 0.0120346435940572, 0.014070114555226, 0.0136156449512337, 0.0, 0.00427258096992147, 0.00694820471863012, 0.00812338442598858, 0.00786099627778515, 0.00607967912880134};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 17:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734315, 0.0249047128495239, 0.014378742667763, 0.0536094339843698, -0.0141189246665008, -0.0109364720199429, -0.0133943880175426, -0.00631417506469883, -0.00773325352755856, 0.0153110608279199, 0.00298267418725717, 0.00252081977982929, 0.0276525235907967, 0.00195261860521384, 0.0214195526695527, 0.0243925408037797, 0.00112734487734486, 0.0123665844996876, 0.0140830399992811, 0.0108976671476671, 0.0, 0.0, -0.0197848223020205, 0.0, -0.0167212267476003, -0.00321800158796924, 0.0, -0.0129522065443227, -0.00249265331165479, 0.0106850252586226, 0.0, -0.00747795993496438, -0.00143913406048031, 0.00616900220936373, 0.00872428659079181, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0206811244618434, 0.0, 0.0, -0.017478740331497, -0.0363288795677472, 0.0, 0.0, -0.0135389740433143, -0.0281402291104519, -0.0357410679969885, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.020635115229186, -0.018239037386404};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 18:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, -0.0354703486038674, -0.0204788153146928, -0.0449826285156206, 0.0141189246665008, 0.0109364720199429, 0.0044647960058476, 0.00631417506469885, 0.00257775117585289, -0.0211438459052229, 0.0, 0.0, 0.00339592394974697, 0.0, 0.00263047138047138, -0.0308358534689291, 0.0, 0.00151870335961077, -0.017803088300978, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0225260111157848, 0.0, 0.0, 0.0174485731815835, -0.00356167508620759, 0.0, 0.0, 0.0100739384233621, -0.00205633406978796, -0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0282557952193589, 0.0, 0.0, 0.0, 0.0218868448636849, 0.047654757329318, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440067};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 19:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734309, 0.0128297005588457, 0.00740723107127186, 0.0449826285156208, 0.0, 0.0, -0.0178591840233903, 0.0, -0.0103110047034114, 0.0211438459052229, 0.0, 0.0, 0.0, 0.0, 0.0, -0.00920473237878476, 0.0, 0.0, -0.00531435471670986, 0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0178083754310377, 0.0, 0.0, 0.0, -0.0102816703489395, 0.0145404776513196, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0297842233308238, 0.0, 0.0, 0.0, 0.0, -0.017195929357655, -0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 20:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734304, -0.0324515955311978, -0.0335503995581136, -0.00801060507812425, 0.0211783869997512, -0.00109364720199431, 0.0169662248222208, -0.00568275755822893, 0.00567105258687628, 0.00838462854862292, -0.0208787193108001, -0.0109235523792603, -0.0162519217595033, -0.00499002532443533, -0.00958243145743145, -0.0108922666482287, -0.00187890812890814, -0.00466458889023305, -0.00558007245254536, -0.00469727032227032, 0.0139343556230002, 0.00245778931055952, 0.0120907247401236, -0.00415442218609135, 0.00464478520766673, 0.00965400476390776, -0.00643600317593853, 0.000719567030240145, 0.00498530662330959, 0.00661453944581401, -0.00464478520766678, -0.000415442218609127, 0.00215870109072045, 0.00323138210966672, 0.00290809553026392, 0.0, 0.0127646671417816, -0.00116524935543313, 0.0202632808954031, 0.00719357335632687, -0.00183832217438607, 0.0228341307513453, 0.0112908326677739, 0.00310733161448834, -0.00201827108709707, 0.020635115229186, 0.01143686949355, 0.00481385743762287, 0.000521115353897249, -0.00170195561890421, 0.0127646671417816, 0.00737991258440981, 0.00347410235931507, 0.000902598269554275, -0.000491312267361575, -0.000868525589828767};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 21:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734308, 0.024904712849524, 0.0553363732971482, -0.00431340273437462, -0.0141189246665007, -0.0153110608279201, -0.0080366328105256, 0.00883984509057836, -0.0159820572902877, 0.00072909813466286, 0.00298267418725715, 0.0277290175781224, -0.00145539597846298, 0.0301571095694136, 0.0116492303992304, -0.00352848074520084, 0.0191648629148629, 0.00976309302606918, 0.00132858867917747, -0.00263047138047137, 0.0, -0.0196623144844763, 0.00219831358911336, -0.00727023882565976, -0.0148633126645335, 0.00402250198496157, 0.00724050357293085, -0.0129522065443228, -0.00685479660705068, 0.00457929653940971, 0.00928957041533352, -0.00664707549774611, -0.0057565362419212, -0.000293762009969707, 0.00290809553026392, 0.0, 0.0, 0.0, -0.0202632808954031, 0.00411061334647247, -0.000459580543596518, -0.0399597288148542, -0.00303983956440067, 0.00427258096992149, -0.00100913554354854, -0.0464290092656686, -0.00908221989193681, 0.00270779480866286, 0.00234501909253767, -0.00127646671417816, -0.031911667854454, -0.00776832903622084, 0.000868525589828741, 0.00180519653910859, 0.000245656133680784, -0.000868525589828764};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 22:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734313, -0.0354703486038675, -0.0492363006502186, -0.00431340273437463, 0.0141189246665008, 0.00656188321196573, 0.00982255121286465, -0.016416855168217, 0.0149509568199466, 0.000729098134662877, 0.0, 0.00336109303977242, -0.000485131992821003, -0.0282044909641998, 0.0124007936507937, -0.00291483191994852, -0.0353234728234728, 0.00889526253486301, 0.00239145962251945, -0.00263047138047139, 0.0, 0.0, 0.0, 0.0218107164769794, -0.00557374224920008, 0.000804500396992315, 0.00402250198496156, 0.0143913406048031, -0.00934744991870546, 0.00254405363300537, -0.00928957041533352, 0.0141250354327105, -0.00287826812096063, -0.00381890612960613, 0.00290809553026392, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0266398192099028, -0.00911951869320202, 0.00233049871086625, -0.000336378514516179, 0.0515877880729651, -0.00840946286290444, -0.00300866089851428, 0.00286613444643491, -0.000850977809452102, 0.0425488904726053, -0.00388416451811045, -0.00347410235931505, 0.00120346435940571, 0.000982624534723158, -0.00086852558982876};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 23:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812425, 0.0, -0.0174983552319087, 0.00357183680467805, 0.0132597676358676, -0.0175287079957995, 0.00838462854862291, 0.0, 0.0, 0.0, -0.00867830491206148, 0.003006253006253, -0.000613648825252317, 0.0225468975468975, -0.019200749617936, 0.0122230158484327, -0.00469727032227032, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0160900079398463, 0.00719567030240154, -0.0024926533116548, 0.000508810726601093, 0.00464478520766676, -0.0108014976838374, 0.0107935054536023, -0.00734405024924251, 0.00290809553026393, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0257938940364825, 0.0134551405806471, -0.00601732179702859, 0.00208446141558904, -0.000425488904726058, -0.031911667854454, 0.0135945758133865, -0.00347410235931504, -0.000902598269554289, 0.00171959293576549, -0.00086852558982877};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 24:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0100234421734306, -0.0452812960900435, -0.0113287063442981, -0.00801060507812427, -0.00141189246665008, 0.0164047080299143, 0.0133943880175427, 0.0145226026488074, 0.0118576554089231, 0.00838462854862292, -0.00894802256177149, -0.0142846454190327, -0.0123708658169354, -0.014536160727703, -0.0125886844636845, -0.0102786178229764, -0.00939454064454065, -0.00813591085505764, -0.00664294339588733, -0.00469727032227032, -0.00836061337380014, 0.00245778931055954, 0.00219831358911338, 0.00830884437218262, 0.00743165633226679, 0.00643600317593851, 0.00965400476390779, 0.00863480436288184, 0.00747795993496439, 0.00610572871921294, 0.00650269929073342, 0.00581619106052785, 0.00503696921168106, 0.00411266813957582, 0.00290809553026391, 0.0352796762358899, 0.0191470007126724, 0.017478740331497, 0.00788016479265675, 0.00719357335632688, 0.00643412761035129, 0.00095142211463938, 0.000868525589828755, 0.000776832903622077, 0.000672757029032348, -0.00221090520312707, -0.00201827108709705, -0.00180519653910857, -0.00156334606169178, -0.00127646671417816, -0.00212744452363026, -0.00194208225905522, -0.00173705117965752, -0.00150433044925715, -0.00122828066840393, -0.000868525589828761};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 25:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734311, 0.0603750614533912, -0.00610007264692968, -0.00431340273437459, -0.00564756986660033, -0.0218729440398858, -0.0178591840233902, 0.00126283501293976, 0.00103110047034113, 0.000729098134662864, 0.0417574386216003, 0.0168054651988621, 0.0145539597846298, -0.000867830491206143, -0.000751563251563245, -0.000613648825252304, -0.00526094276094276, -0.00455611007883229, -0.00372004830169692, -0.00263047138047139, 0.00557374224920013, -0.0196623144844763, -0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992314, 0.00321800158796925, 0.00287826812096062, 0.00249265331165479, 0.00203524290640429, 0.00650269929073342, 0.00581619106052785, 0.00503696921168108, 0.00411266813957584, 0.00290809553026395, -0.0705593524717798, -0.0127646671417816, -0.0116524935543313, 0.00450295131008956, 0.00411061334647247, 0.00367664434877215, 0.00380568845855755, 0.00347410235931509, 0.00310733161448836, 0.00269102811612943, -0.00073696840104236, -0.000672757029032368, -0.000601732179702871, -0.000521115353897269, -0.000425488904726046, -0.00212744452363026, -0.00194208225905521, -0.00173705117965753, -0.00150433044925714, -0.00122828066840394, -0.000868525589828776};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 26:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0100234421734312, -0.0603750614533913, -0.00610007264692976, -0.00431340273437462, -0.00564756986660025, 0.0218729440398858, 0.0178591840233902, 0.00126283501293977, 0.00103110047034114, 0.000729098134662859, -0.0417574386216002, 0.0168054651988621, 0.0145539597846299, 0.000867830491206133, 0.000751563251563239, 0.000613648825252299, -0.00526094276094275, -0.00455611007883226, -0.0037200483016969, -0.00263047138047138, 0.00557374224920004, 0.0196623144844763, 0.0175865087129071, -0.0103860554652283, -0.00928957041533348, -0.00804500396992315, -0.00321800158796928, -0.00287826812096062, -0.0024926533116548, -0.0020352429064043, 0.00650269929073342, 0.00581619106052786, 0.00503696921168108, 0.00411266813957581, 0.00290809553026392, 0.0705593524717798, -0.0127646671417816, -0.0116524935543313, -0.00450295131008957, -0.00411061334647248, -0.00367664434877215, 0.00380568845855754, 0.00347410235931504, 0.00310733161448834, 0.00269102811612942, 0.000736968401042371, 0.000672757029032358, 0.000601732179702873, 0.000521115353897267, 0.000425488904726051, -0.00212744452363026, -0.00194208225905523, -0.00173705117965754, -0.00150433044925714, -0.00122828066840393, -0.000868525589828762};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 27:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.010023442173431, 0.0452812960900435, -0.0113287063442981, -0.00801060507812422, -0.0014118924666501, -0.0164047080299144, -0.0133943880175427, 0.0145226026488074, 0.0118576554089232, 0.00838462854862291, 0.00894802256177149, -0.0142846454190328, -0.0123708658169354, 0.014536160727703, 0.0125886844636845, 0.0102786178229764, -0.00939454064454066, -0.00813591085505765, -0.00664294339588734, -0.00469727032227034, -0.0083606133738001, -0.00245778931055953, -0.00219831358911339, 0.00830884437218266, 0.00743165633226679, 0.00643600317593851, -0.00965400476390776, -0.00863480436288184, -0.00747795993496436, -0.00610572871921293, 0.00650269929073342, 0.00581619106052785, 0.00503696921168106, 0.00411266813957581, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, 0.017478740331497, -0.00788016479265674, -0.00719357335632687, -0.00643412761035128, 0.000951422114639395, 0.000868525589828766, 0.000776832903622087, 0.000672757029032357, 0.00221090520312706, 0.00201827108709707, 0.00180519653910857, 0.00156334606169176, 0.00127646671417816, -0.00212744452363025, -0.00194208225905521, -0.00173705117965752, -0.00150433044925714, -0.00122828066840393, -0.000868525589828762};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 28:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.078755617076958, 0.0679219441350653, -0.00435719474780703, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, -0.0267887760350855, 0.0126283501293977, 0.03608851646194, -0.00729098134662863, 0.0298267418725716, 0.0151249186789758, 0.0130985638061668, -0.0325436434202305, -0.0229226791726792, -0.0248527774227189, 0.0248015873015873, 0.0169226945785199, 0.0164744996218006, 0.0142797017797018, 0.0, 0.0245778931055954, 0.0219831358911339, -0.0363511941282991, -0.00743165633226679, -0.0345935170706696, 0.0362025178646541, 0.0, 0.0130864298861876, 0.0335815079556711, -0.0232239260383337, 0.00166176887443654, -0.00359783515120075, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0226083734055987, -0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.00908221989193681, 0.0, 0.0333014729371016, 0.0270779480866287, 0.00703505727761299, -0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598858, 0.00196524906944629, 0.00607967912880136};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 29:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.00377344134083692, 0.00217859737390364, -0.0215670136718729, 0.0, 0.0984282481794863, -0.0312535720409329, -0.0852413633734344, -0.0180442582309701, 0.0309866707231716, 0.0, 0.0453747560369276, -0.0116431678277039, 0.0, 0.0236742424242425, -0.00138070985681771, -0.0202922077922078, -0.0335199527228374, -0.000797153207506548, 0.00394570707070708, 0.0, 0.0, 0.0, 0.0363511941282991, 0.024152883079867, -0.00724050357293084, -0.0603375297744236, 0.0, -0.039259289658563, 0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064561, 0.0, -0.0555024548951693, 0.0, 0.0164151336477636, -0.00680782247561684, 0.0, 0.0427258096992147, 0.00694820471863017, -0.0108311792346515, -0.00393049813889261, 0.00607967912880133};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 30:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0301875307266957, 0.0610007264692974, -0.00308100195312476, 0.0, 0.0218729440398858, 0.0178591840233903, 0.0757701007763862, -0.0412440188136457, -0.00729098134662866, 0.0, 0.0, 0.0, 0.0520698294723688, -0.0127765752765753, 0.00184094647575698, -0.00901875901875901, 0.0286384062098029, -0.0297603864135752, 0.0142797017797018, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0402250198496157, 0.014391340604803, -0.0137095932141013, 0.00407048581280863, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0370016365967796, -0.030086608985143, 0.014070114555226, -0.00340391123780845, 0.0, -0.0427258096992148, 0.0243187165152054, 0.0, -0.00982624534723141, 0.00607967912880137};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 31:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.021478804657352, 0.00377344134083707, -0.0196073763651313, 0.00924300585937421, 0.0, -0.010936472019943, 0.102690308134494, 0.0094712625970483, -0.0695992817480271, -0.060150596109686, 0.0, -0.00504163955965864, 0.0465726713108155, 0.00650872868404603, 0.0214195526695526, -0.00782402252196704, -0.00450937950937952, -0.0289638426440052, -0.0263060558477138, 0.000563672438672408, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, -0.0431740218144092, -0.00249265331165474, -0.0605484764655282, 0.0, 0.0299118397398575, -0.00431740218144093, 0.0185070066280912, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, -0.0451299134777144, -0.0281402291104519, 0.023827378664659, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.018239037386404};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 32:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573521, -0.0150937653633478, 0.0130715842434209, 0.00924300585937411, 0.0, -0.0109364720199429, -0.00892959201169512, 0.00947126259704831, 0.123732056440937, -0.060150596109686, 0.0, 0.0, 0.0, -0.00650872868404609, 0.0574945887445887, -0.0266937238984759, 0.00676406926406926, 0.0126920209338899, 0.00637722566005184, 0.000563672438672477, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0431740218144092, 0.0199412264932383, -0.0142467003448302, 0.0, -0.0448677596097863, 0.0215870109072046, -0.0616900220936373, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0451299134777144, -0.0375203054806026, 0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 33:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, 0.0301875307266957, 0.0174287789912277, 0.0585390371093694, 0.0, 0.0218729440398859, 0.0178591840233903, -0.0189425251940965, 0.0103110047034114, 0.0838462854862291, 0.0, 0.0, 0.0, 0.0, 0.00526094276094274, 0.0533874477969518, 0.0, -0.00455611007883225, 0.0308232573569173, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.034897146363167, 0.0284934006896603, 0.0, 0.0, -0.0302218152700865, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0437736897273697, -0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 34:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769581, -0.0679219441350654, -0.00435719474780698, -0.00308100195312479, 0.0705946233325039, 0.0328094160598288, 0.0267887760350854, 0.0126283501293977, 0.0360885164619401, -0.00729098134662864, -0.0298267418725716, 0.0151249186789758, 0.0130985638061669, 0.0325436434202305, 0.0229226791726792, 0.024852777422719, 0.0248015873015874, 0.0169226945785198, 0.0164744996218006, 0.0142797017797018, 0.0, -0.0245778931055953, -0.0219831358911338, -0.036351194128299, -0.00743165633226677, -0.0345935170706695, -0.0362025178646541, 0.0, -0.0130864298861876, -0.0335815079556711, -0.0232239260383337, 0.00166176887443655, -0.00359783515120075, -0.0146881004984851, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, -0.0226083734055987, 0.00459580543596521, 0.0, -0.0334382352084074, -0.0194208225905522, 0.0090822198919368, 0.0, -0.0333014729371016, -0.0270779480866286, -0.00703505727761296, 0.0102117337134253, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598855, 0.00196524906944629, 0.00607967912880136};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 35:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, -0.00377344134083697, 0.00217859737390366, -0.0215670136718729, 0.0, -0.0984282481794862, 0.0312535720409329, -0.0852413633734344, -0.0180442582309701, 0.0309866707231716, 0.0, 0.0453747560369275, -0.0116431678277039, 0.0, -0.0236742424242424, 0.00138070985681773, -0.0202922077922079, -0.0335199527228374, -0.000797153207506549, 0.00394570707070709, 0.0, 0.0, 0.0, 0.036351194128299, 0.024152883079867, -0.00724050357293081, 0.0603375297744236, 0.0, 0.0392592896585629, -0.0188259968842399, 0.0464478520766673, -0.0108014976838374, 0.0259044130886455, 0.010869194368879, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0334382352084074, -0.0155366580724417, 0.00302740663064561, 0.0, 0.0555024548951693, 0.0, -0.0164151336477636, 0.00680782247561685, 0.0, 0.0427258096992147, 0.00694820471863019, -0.0108311792346515, -0.00393049813889259, 0.00607967912880134};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 36:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, -0.0301875307266958, 0.0610007264692974, -0.00308100195312485, 0.0, -0.021872944039886, -0.0178591840233903, 0.0757701007763861, -0.0412440188136456, -0.00729098134662859, 0.0, 0.0, 0.0, -0.0520698294723688, 0.0127765752765752, -0.00184094647575697, -0.00901875901875897, 0.0286384062098028, -0.0297603864135752, 0.0142797017797018, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0402250198496157, -0.0143913406048031, 0.0137095932141014, -0.00407048581280863, -0.0464478520766673, 0.0182794576188018, -0.0345392174515273, 0.0364264892362429, -0.0178640154001927, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0370016365967795, 0.0300866089851429, -0.014070114555226, 0.00340391123780843, 0.0, -0.0427258096992147, 0.0243187165152053, 0.0, -0.00982624534723143, 0.00607967912880138};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 37:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573516, -0.00377344134083713, -0.0196073763651312, 0.00924300585937421, 0.0, 0.0109364720199428, -0.102690308134494, 0.00947126259704829, -0.0695992817480271, -0.060150596109686, 0.0, -0.00504163955965856, 0.0465726713108155, -0.00650872868404608, -0.0214195526695527, 0.00782402252196704, -0.00450937950937948, -0.0289638426440052, -0.0263060558477138, 0.000563672438672415, 0.0, 0.0, 0.0, 0.0, 0.0334424534952005, 0.0289620142917233, 0.0, 0.0431740218144092, 0.00249265331165474, 0.0605484764655282, 0.0, 0.0299118397398575, -0.00431740218144093, 0.0185070066280912, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0349574806629939, -0.0121096265225824, 0.0, 0.0, 0.0451299134777144, 0.0281402291104519, -0.023827378664659, 0.0, 0.0, 0.0312669212338355, 0.0270779480866286, 0.0, -0.018239037386404};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 38:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, 0.0150937653633479, 0.0130715842434208, 0.00924300585937418, 0.0, 0.0109364720199429, 0.00892959201169517, 0.00947126259704834, 0.123732056440937, -0.0601505961096861, 0.0, 0.0, 0.0, 0.00650872868404608, -0.0574945887445887, 0.0266937238984759, 0.00676406926406923, 0.0126920209338899, 0.00637722566005182, 0.000563672438672456, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0431740218144092, -0.0199412264932384, 0.0142467003448302, 0.0, -0.0448677596097862, 0.0215870109072046, -0.0616900220936372, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0451299134777144, 0.0375203054806026, -0.0119136893323295, 0.0, 0.0, -0.0469003818507533, 0.0162467688519772, 0.020635115229186, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 39:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769579, -0.0301875307266958, 0.0174287789912279, 0.0585390371093693, 0.0, -0.0218729440398857, -0.0178591840233903, -0.0189425251940965, 0.0103110047034115, 0.083846285486229, 0.0, 0.0, 0.0, 0.0, -0.00526094276094277, -0.0533874477969517, 0.0, -0.00455611007883225, 0.0308232573569172, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0348971463631671, -0.0284934006896603, 0.0, 0.0, -0.0302218152700864, 0.0164506725583033, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0437736897273697, 0.023827378664659, 0.0, 0.0, 0.0, -0.0379091273212801, -0.013756743486124, 0.0303983956440068};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 40:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769573, -0.0377344134083695, -0.0566435317214904, -0.00308100195312477, 0.0423567739995023, 0.0546823600997147, 0.0446479600584757, 0.0378850503881931, 0.00515550235170579, -0.00729098134662861, 0.0417574386216002, -0.00504163955965862, 0.0295930515620807, -0.0247331689993752, 0.0101461038961039, 0.0266937238984759, -0.0202922077922078, 0.00216957622801539, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00219831358911336, -0.00519302773261414, -0.0130053985814669, -0.0168945083368385, 0.00402250198496157, -0.0158304746652834, -0.026796023100289, -0.0295110221428625, 0.00464478520766675, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, -0.0226083734055987, -0.0367664434877216, 0.0, -0.0047768907440582, -0.00776832903622085, -0.00908221989193678, 0.0, 0.00370016365967795, 0.00601732179702857, 0.007035057277613, 0.00680782247561685, 0.0, 0.00427258096992149, 0.0069482047186301, 0.00812338442598857, 0.00786099627778516, 0.00607967912880134};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 41:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.0, -0.00435719474780704, -0.0215670136718729, -0.127070321998507, 0.0, 0.0, 0.0284137877911449, 0.0360885164619399, 0.0309866707231716, 0.0, 0.0453747560369276, -0.0422064833754265, 0.0, 0.0, 0.0, -0.0202922077922078, -0.00618329224984381, 0.00159430641501296, 0.00394570707070708, 0.0836061337380012, 0.0, 0.0, -0.0103860554652283, 0.024152883079867, 0.043443021437585, 0.0, 0.0, 0.0, 0.0, 0.00464478520766673, -0.0108014976838374, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, 0.0769064574585865, 0.0, 0.0, 0.0, 0.0, -0.00955378148811641, -0.0155366580724417, -0.0181644397838736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.00427258096992146, 0.00694820471863015, 0.00812338442598859, 0.00786099627778516, 0.00607967912880135};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 42:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, 0.0377344134083696, -0.0566435317214905, -0.00308100195312481, 0.0423567739995022, -0.0546823600997145, -0.0446479600584757, 0.0378850503881931, 0.00515550235170578, -0.0072909813466285, -0.0417574386216002, -0.00504163955965861, 0.0295930515620806, 0.0247331689993752, -0.0101461038961039, -0.0266937238984759, -0.0202922077922078, 0.00216957622801541, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055953, 0.00219831358911336, -0.00519302773261412, -0.0130053985814669, -0.0168945083368386, -0.00402250198496156, 0.0158304746652834, 0.026796023100289, 0.0295110221428625, 0.00464478520766677, -0.0108014976838375, -0.0194283098164841, -0.0217383887377579, -0.0178640154001927, 0.0, 0.0, -0.0512709716390577, 0.0, 0.0226083734055987, 0.0367664434877216, 0.0, -0.00477689074405817, -0.00776832903622087, -0.0090822198919368, 0.0, -0.00370016365967795, -0.00601732179702859, -0.00703505727761297, -0.00680782247561685, 0.0, 0.0042725809699215, 0.00694820471863008, 0.00812338442598857, 0.00786099627778517, 0.00607967912880136};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 43:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, -0.0188672067041848, 0.00653579212171051, 0.00924300585937421, 0.0141189246665008, 0.0, -0.111619900146189, -0.00315708753234941, -0.05413277469291, -0.060150596109686, -0.00894802256177145, -0.00252081977982928, -0.0276525235907967, 0.000650872868404633, 0.0360750360750361, -0.0188697013765088, 0.00112734487734487, 0.0374251899332651, 0.0199288301876619, 0.000563672438672408, 0.0, 0.0, 0.0593544669060614, 0.0, 0.0167212267476003, 0.00321800158796919, 0.0, -0.0043174021814409, 0.0224338798048931, 0.0463017761206981, 0.0, -0.00747795993496438, 0.018708742786244, 0.0431830154655461, 0.0461140862656136, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0620433733855302, 0.0, 0.0, 0.017478740331497, 0.0363288795677472, 0.0, 0.0, -0.00451299134777145, -0.00938007637015065, -0.0119136893323295, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.020635115229186, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 44:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573523, 0.0188672067041847, 0.00653579212171058, 0.00924300585937418, 0.0141189246665008, 0.0, 0.111619900146189, -0.00315708753234937, -0.05413277469291, -0.0601505961096861, 0.0089480225617715, -0.0025208197798293, -0.0276525235907967, -0.000650872868404624, -0.0360750360750361, 0.0188697013765088, 0.00112734487734488, 0.0374251899332651, 0.019928830187662, 0.000563672438672449, 0.0, 0.0, -0.0593544669060614, 0.0, 0.0167212267476003, 0.00321800158796924, 0.0, 0.00431740218144091, -0.0224338798048931, -0.0463017761206981, 0.0, -0.00747795993496438, 0.0187087427862439, 0.0431830154655461, 0.0461140862656137, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0620433733855302, 0.0, 0.0, 0.0174787403314969, 0.0363288795677472, 0.0, 0.0, 0.00451299134777145, 0.00938007637015064, 0.0119136893323295, 0.0, 0.0, -0.00781673030845888, -0.0162467688519772, -0.0206351152291861, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 45:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769578, 0.0, -0.0348575579824557, 0.0585390371093694, -0.0282378493330015, 0.0, 0.0, 0.00631417506469885, -0.0206220094068228, 0.0838462854862291, 0.0, 0.0, -0.00679184789949388, 0.0, 0.0, 0.0, 0.0, 0.00151870335961077, -0.0616465147138343, -0.0319414381914382, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0450520222315696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0100739384233621, -0.0329013451166065, -0.0643935438844155, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0565115904387179, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.01263637577376, 0.027513486972248, 0.0303983956440067};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 46:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769577, -0.0377344134083697, -0.0217859737390348, -0.05237703320312, 0.0423567739995024, 0.0546823600997146, 0.0446479600584757, 0.0, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, -0.00921750786359889, 0.0273366604729936, -0.00789141414141415, -0.0248527774227189, 0.0187890812890813, -0.0034713219648246, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111906, -0.0241814494802472, -0.0197335053839338, -0.00928957041533347, -0.00241350119097692, -0.0394205194526234, -0.0158304746652834, -0.000623163327913719, 0.00712335017241506, -0.0325134964536671, -0.0141250354327105, -0.00215870109072046, 0.00411266813957582, 0.00540074884191869, 0.0, -0.0510586685671264, 0.00466099742173251, -0.0405265617908061, -0.0143871467126537, 0.00367664434877213, -0.0114170653756726, -0.00564541633388696, -0.00155366580724414, 0.00100913554354854, 0.010317557614593, 0.00571843474677502, 0.00240692871881142, 0.000260557676948619, -0.000850977809452114, 0.0127646671417816, 0.00737991258440984, 0.00347410235931509, 0.000902598269554295, -0.000491312267361566, -0.000868525589828747};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 47:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, 0.0, -0.0217859737390348, 0.00308100195312479, -0.127070321998507, 0.0, 0.0, 0.0473563129852413, 0.0257775117585286, 0.0127592173566001, 0.0, -0.0352914769176103, 0.0509388592462045, 0.0, 0.0, 0.0, 0.00225468975468974, -0.00943765659186684, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0.0, 0.0, 0.0477758551400502, 0.00928957041533349, -0.0144810071458617, 0.0, 0.0, 0.0, 0.0, -0.0325134964536671, -0.0141250354327105, -0.00215870109072046, 0.00411266813957582, 0.00540074884191877, 0.0, 0.0765880028506897, -0.00699149613259876, 0.0, 0.0, 0.0, -0.0228341307513453, -0.011290832667774, -0.00310733161448838, 0.00201827108709705, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0127646671417816, 0.00737991258440982, 0.00347410235931506, 0.000902598269554268, -0.000491312267361585, -0.00086852558982881};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 48:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.0787556170769581, 0.0377344134083697, -0.0217859737390347, -0.0523770332031201, 0.0423567739995024, -0.0546823600997146, -0.0446479600584757, 0.0, 0.0257775117585285, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.00921750786359894, -0.0273366604729937, 0.00789141414141414, 0.0248527774227189, 0.0187890812890813, -0.00347132196482457, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111903, 0.0241814494802472, -0.0197335053839338, -0.00928957041533349, -0.00241350119097694, 0.0394205194526234, 0.0158304746652834, 0.000623163327913702, -0.00712335017241507, -0.0325134964536671, -0.0141250354327105, -0.00215870109072045, 0.00411266813957581, 0.00540074884191867, 0.0, -0.0510586685671264, 0.0046609974217325, 0.0405265617908061, 0.0143871467126537, -0.00367664434877215, -0.0114170653756726, -0.00564541633388696, -0.00155366580724416, 0.00100913554354855, -0.010317557614593, -0.00571843474677502, -0.00240692871881143, -0.000260557676948624, 0.000850977809452106, 0.0127646671417816, 0.00737991258440984, 0.00347410235931506, 0.000902598269554285, -0.000491312267361562, -0.000868525589828736};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 49:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573518, -0.0188672067041849, 0.0108929868695176, 0.00308100195312463, 0.0141189246665008, -0.109364720199429, 0.0223239800292377, -0.0789271883087356, -0.0128887558792642, 0.0127592173566001, -0.00894802256177147, -0.0277290175781224, 0.00145539597846298, -0.0058578558156415, 0.0383297258297259, -0.0124263887113595, 0.0191648629148628, 0.0348216984596467, 0.00717437886755833, -0.0129644660894661, 0.0, 0.0589869434534289, -0.00659494076734012, 0.00727023882565972, 0.0148633126645335, -0.0040225019849616, 0.0506835250105158, -0.00431740218144091, -0.00810112326287806, 0.00356167508620759, 0.0650269929073342, -0.00166176887443653, -0.0115130724838425, -0.00205633406978792, 0.00540074884191875, 0.0, 0.0, 0.0, 0.0607898426862092, -0.0123318400394175, 0.0013787416307896, 0.0399597288148542, 0.00303983956440064, -0.00427258096992151, 0.00100913554354853, -0.0154763364218896, -0.00302740663064563, 0.00090259826955432, 0.000781673030845895, -0.000425488904726053, -0.031911667854454, -0.00776832903622088, 0.000868525589828737, 0.00180519653910857, 0.000245656133680776, -0.000868525589828788};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 50:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {-0.0214788046573522, 0.0188672067041847, 0.0108929868695174, 0.00308100195312465, 0.0141189246665008, 0.109364720199429, -0.0223239800292378, -0.0789271883087356, -0.0128887558792643, 0.0127592173566001, 0.00894802256177148, -0.0277290175781224, 0.00145539597846297, 0.00585785581564141, -0.0383297258297259, 0.0124263887113595, 0.019164862914863, 0.0348216984596467, 0.00717437886755831, -0.0129644660894661, 0.0, -0.0589869434534289, 0.00659494076734012, 0.00727023882565982, 0.0148633126645335, -0.00402250198496154, -0.0506835250105158, 0.00431740218144091, 0.00810112326287806, -0.00356167508620755, 0.0650269929073342, -0.00166176887443652, -0.0115130724838425, -0.0020563340697879, 0.00540074884191873, 0.0, 0.0, 0.0, -0.0607898426862092, 0.0123318400394175, -0.00137874163078958, 0.0399597288148542, 0.00303983956440069, -0.00427258096992147, 0.00100913554354853, 0.0154763364218896, 0.00302740663064561, -0.000902598269554305, -0.000781673030845883, 0.000425488904726056, -0.031911667854454, -0.00776832903622088, 0.000868525589828765, 0.00180519653910858, 0.00024565613368079, -0.000868525589828768};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 51:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.078755617076958, 0.0, 0.0435719474780696, -0.05237703320312, -0.0282378493330015, 0.0, 0.0, 0.0631417506469885, -0.0515550235170571, 0.0291639253865144, 0.0, -0.00672218607954483, 0.000970263985642005, 0.0, 0.0, 0.0, -0.0601250601250601, -0.0125835421224891, 0.0286975154702333, -0.015031265031265, 0.0, 0.0, 0.0, -0.0436214329539588, 0.0111474844984002, -0.00160900079398464, 0.0, 0.0, 0.0, 0.0, -0.0650269929073342, 0.031573608614294, -0.00287826812096058, -0.00822533627915168, 0.00540074884191872, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0532796384198057, 0.018239037386404, -0.00466099742173252, 0.000672757029032361, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0425488904726053, -0.00388416451811041, -0.00347410235931508, 0.00120346435940574, 0.000982624534723166, -0.000868525589828754};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 52:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281179, -0.0705946233325039, -0.0546823600997146, -0.0446479600584757, -0.0315708753234942, -0.0257775117585285, -0.0182274533665715, 0.0894802256177148, -0.0151249186789758, -0.0130985638061669, 0.0585785581564149, -0.028183621933622, 0.0690354928408859, 0.074404761904762, 0.00976309302606919, 0.039857660375324, 0.0597492784992785, 0.0, 0.0737336793167861, 0.0659494076734016, 0.0363511941282991, 0.00743165633226676, 0.0345935170706695, -0.0120675059548847, 0.0431740218144092, 0.0155790831978425, -0.0254405363300538, -0.0232239260383337, 0.0465295284842228, 0.0251848460584054, -0.0146881004984851, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, 0.0678251202167962, -0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.00908221989193678, 0.0, -0.0111004909790339, -0.00902598269554281, -0.00234501909253764, 0.00340391123780841, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598859, 0.00196524906944629, 0.0060796791288014};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 53:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281177, -0.0705946233325038, 0.0546823600997147, 0.0446479600584758, -0.0315708753234942, -0.0257775117585286, -0.0182274533665716, -0.0894802256177148, -0.0151249186789759, -0.0130985638061668, -0.058578558156415, 0.028183621933622, -0.0690354928408859, 0.0744047619047619, 0.00976309302606916, 0.0398576603753239, 0.0597492784992785, 0.0, -0.0737336793167862, -0.0659494076734017, 0.036351194128299, 0.00743165633226679, 0.0345935170706696, 0.0120675059548846, -0.0431740218144092, -0.0155790831978425, 0.0254405363300538, -0.0232239260383337, 0.0465295284842228, 0.0251848460584053, -0.014688100498485, -0.0328199352701215, 0.0, 0.0, 0.0, 0.0, -0.0678251202167963, 0.0137874163078956, 0.0, 0.0334382352084074, 0.0194208225905522, -0.00908221989193679, 0.0, 0.0111004909790338, 0.00902598269554289, 0.00234501909253761, -0.00340391123780841, 0.0, -0.0213629048496074, -0.0191075629762328, -0.00812338442598856, 0.00196524906944628, 0.00607967912880138};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 54:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.217859737390348, -0.0770250488281177, 0.0, 0.0, 0.0, -0.0947126259704828, 0.0515550235170571, -0.0182274533665715, 0.0, -0.0907495120738551, 0.0232863356554078, 0.0, 0.0, 0.0, -0.0946969696969696, -0.0227805503941613, -0.0797153207506479, 0.0597492784992785, 0.0, 0.0, 0.0, -0.0727023882565981, -0.0483057661597341, 0.0144810071458617, 0.0, 0.0, 0.0, 0.0, 0.0464478520766673, -0.0781031370985168, 0.0431740218144092, 0.0293762009969701, -0.0328199352701214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0668764704168149, 0.0310733161448835, -0.0060548132612912, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0427258096992147, 0.00694820471863011, -0.0108311792346515, -0.0039304981388926, 0.00607967912880135};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    case 55:
      {
        
      // Array of basisvalues.
      double basisvalues[56] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
      
      // Declare helper variables.
      unsigned int rr = 0;
      unsigned int ss = 0;
      unsigned int tt = 0;
      double tmp5 = 0.0;
      double tmp6 = 0.0;
      double tmp7 = 0.0;
      double tmp0 = 0.5*(2.0 + Y + Z + 2.0*X);
      double tmp1 = 0.25*(Y + Z)*(Y + Z);
      double tmp2 = 0.5*(1.0 + Z + 2.0*Y);
      double tmp3 = 0.5*(1.0 - Z);
      double tmp4 = tmp3*tmp3;
      
      // Compute basisvalues.
      basisvalues[0] = 1.0;
      basisvalues[1] = tmp0;
      for (unsigned int r = 1; r < 5; r++)
      {
        rr = (r + 1)*((r + 1) + 1)*((r + 1) + 2)/6;
        ss = r*(r + 1)*(r + 2)/6;
        tt = (r - 1)*((r - 1) + 1)*((r - 1) + 2)/6;
        basisvalues[rr] = (basisvalues[ss]*tmp0*(1.0 + 2.0*r)/(1.0 + r) - basisvalues[tt]*tmp1*r/(1.0 + r));
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        rr = (r + 1)*(r + 1 + 1)*(r + 1 + 2)/6 + 1*(1 + 1)/2;
        ss = r*(r + 1)*(r + 2)/6;
        basisvalues[rr] = basisvalues[ss]*(r*(1.0 + Y) + (2.0 + Z + 3.0*Y)/2.0);
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 1; s < 5 - r; s++)
        {
          rr = (r + (s + 1))*(r + (s + 1) + 1)*(r + (s + 1) + 2)/6 + (s + 1)*((s + 1) + 1)/2;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          tt = (r + (s - 1))*(r + (s - 1) + 1)*(r + (s - 1) + 2)/6 + (s - 1)*((s - 1) + 1)/2;
          tmp5 = (2.0 + 2.0*r + 2.0*s)*(3.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp6 = (1.0 + 4.0*r*r + 4.0*r)*(2.0 + 2.0*r + 2.0*s)/(2.0*(1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          tmp7 = (1.0 + s + 2.0*r)*(3.0 + 2.0*r + 2.0*s)*s/((1.0 + 2.0*r + 2.0*s)*(1.0 + s)*(2.0 + s + 2.0*r));
          basisvalues[rr] = (basisvalues[ss]*(tmp2*tmp5 + tmp3*tmp6) - basisvalues[tt]*tmp4*tmp7);
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 5; r++)
      {
        for (unsigned int s = 0; s < 5 - r; s++)
        {
          rr = (r + s + 1)*(r + s + 1 + 1)*(r + s + 1 + 2)/6 + (s + 1)*(s + 1 + 1)/2 + 1;
          ss = (r + s)*(r + s + 1)*(r + s + 2)/6 + s*(s + 1)/2;
          basisvalues[rr] = basisvalues[ss]*(1.0 + r + s + Z*(2.0 + r + s));
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 4; r++)
      {
        for (unsigned int s = 0; s < 4 - r; s++)
        {
          for (unsigned int t = 1; t < 5 - r - s; t++)
          {
            rr = (r + s + t + 1)*(r + s + t + 1 + 1)*(r + s + t + 1 + 2)/6 + (s + t + 1)*(s + t + 1 + 1)/2 + t + 1;
            ss = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            tt = (r + s + t - 1)*(r + s + t - 1 + 1)*(r + s + t - 1 + 2)/6 + (s + t - 1)*(s + t - 1 + 1)/2 + t - 1;
            tmp5 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 2.0*r + 2.0*s + 2.0*t)/(2.0*(1.0 + t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp6 = (3.0 + 2.0*r + 2.0*s + 2.0*t)*(4.0 + 4.0*r*r + 4.0*s*s + 8.0*r*s + 8.0*r + 8.0*s)/(2.0*(1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            tmp7 = (2.0 + t + 2.0*r + 2.0*s)*(4.0 + 2.0*r + 2.0*s + 2.0*t)*t/((1.0 + t)*(2.0 + 2.0*r + 2.0*s + 2.0*t)*(3.0 + t + 2.0*r + 2.0*s));
            basisvalues[rr] = (basisvalues[ss]*(tmp6 + Z*tmp5) - basisvalues[tt]*tmp7);
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      for (unsigned int r = 0; r < 6; r++)
      {
        for (unsigned int s = 0; s < 6 - r; s++)
        {
          for (unsigned int t = 0; t < 6 - r - s; t++)
          {
            rr = (r + s + t)*(r + s + t + 1)*(r + s + t + 2)/6 + (s + t)*(s + t + 1)/2 + t;
            basisvalues[rr] *= std::sqrt((0.5 + r)*(1.0 + r + s)*(1.5 + r + s + t));
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Table(s) of coefficients.
      static const double coefficients0[56] = \
      {0.107394023286761, 0.0, 0.0, 0.231075146484353, 0.0, 0.0, 0.0, 0.0, 0.0, -0.109364720199429, 0.0, 0.0100832791193174, -0.0931453426216311, 0.0, 0.0, 0.0, -0.00450937950937958, -0.104139658944738, 0.0, -0.0924422799422799, 0.0, 0.0, 0.0, 0.0, -0.0668849069904009, -0.0579240285834467, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0299118397398575, -0.0647610327216138, 0.0, 0.0835038859404355, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0699149613259879, 0.0242192530451648, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0312669212338355, 0.0270779480866287, 0.0, -0.0182390373864041};
      
      // Tables of derivatives of the polynomial base (transpose).
      static const double dmats0[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {6.32455532033676, 0.0, 1.33226762955019e-14, 0.0, 1.14908083048704e-14, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 1.77635683940025e-14, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.52100554373646e-14, 11.2249721603218, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.06581410364015e-14, 0.0, -2.04281036531029e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, -1.59872115546023e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.58257569495583, 0.0, 8.36660026534078, -1.18321595661993, 0.0, 0.0, 0.0, 0.0, -1.36002320516582e-14, 0.0, 0.0, 1.59872115546023e-14, 0.0, -1.68753899743024e-14, 1.77635683940025e-14, -1.82076576038526e-14, 1.86517468137026e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, 1.50990331349021e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.02140518265514e-14, 1.31006316905768e-14, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0},
      {3.74165738677393, 0.0, 0.0, 8.69482604771367, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.28785870856518e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, -1.24344978758018e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.64174229766445e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, -1.38222766565832e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.49909083394699, 0.0, -3.3466401061363, -2.36643191323984, 15.4919333848297, 0.0, 0.0, 0.692820323027558, 0.565685424949244, 0.399999999999995, 0.0, -3.46389583683049e-14, -2.30926389122033e-14, 1.33226762955019e-14, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 1.33226762955019e-14, 2.4980018054066e-14, 0.0, 1.58896492367982e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.57651669496772e-14, 0.0, 0.0, 0.0},
      {3.73590047786365e-14, 4.89897948556636, 3.73034936274053e-14, 0.0, 0.0, 14.1985914794391, -0.82807867121082, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0},
      {-2.8421709430404e-14, 4.24264068711927, 0.0, 0.0, 0.0, 3.10862446895044e-14, 14.3427433120127, 1.24344978758018e-14, 2.44249065417534e-14, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, -3.46389583683049e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0769163338864e-14, 1.27675647831893e-14, 0.0, 0.0, 0.0},
      {3.6, 0.0, 8.76356092008267, -1.549193338483, 0.0, 0.0, 0.0, 9.52470471983255, -1.48131215963608, 0.261861468283196, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 1.4210854715202e-14, 0.0, 2.30926389122033e-14, 0.0, 1.24344978758018e-14, -2.57571741713036e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 1.24344978758018e-14, -1.4210854715202e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, -1.33226762955019e-14},
      {3.11769145362397, -1.75415237890775e-14, 3.16227766016835, 4.91934955049954, 0.0, -1.62647673107585e-14, 0.0, -1.33226762955019e-14, 10.690449676497, -2.41897262725904, 0.0, 0.0, 0.0, -2.22044604925031e-14, 1.15463194561016e-14, -2.98649993624167e-14, -1.40443212615082e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0325074129014e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.39808173319034e-14, -2.1316282072803e-14, 1.95399252334028e-14, 0.0, 0.0, 0.0, -2.46469511466785e-14, 2.03725925018716e-14, 0.0, 0.0},
      {2.54558441227157, 0.0, 0.0, 7.66811580507233, 0.0, 0.0, 0.0, 0.0, 0.0, 10.3691851174526, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 2.30926389122033e-14, 0.0, 1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.93178806284777e-14, 11.6057691494799, 0.0, 1.02140518265514e-14, 0.0, -4.20458932936039, -3.43303281162796, 0.0, 0.0, 0.0, 19.6214168703486, 0.0, 0.0, 0.611677741841193, 0.529728463363959, 0.432521479156766, 0.0, 1.50990331349021e-14, 1.50990331349021e-14, 1.0991207943789e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.18786224940063e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.53137212479105, 0.0, 3.54562104171166, -2.50713268211203, 4.78713553878169, 2.48689957516035e-14, 0.0, -5.56626745075488, -1.04880884817014, 0.741619848709556, -1.24344978758018e-14, 18.8034951158402, -0.60312238704646, 0.0, -2.30926389122033e-14, 0.0, 1.68183573174416, 0.701283781217658, 0.13213749452869, -0.0934353184302195, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.05298244095216, -1.86517468137026e-14, -1.58564993434418, 4.4848952846511, 4.28174419288837, 0.0, -1.4210854715202e-14, 0.19148542155127, -3.75233260785875, -4.31161222746203, -1.59872115546023e-14, -1.24344978758018e-14, 18.8807171866293, 0.0, -3.5527136788005e-14, -3.28626015289046e-14, 0.0, 0.844371341865044, 1.18187368057057, 1.04463861754667, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0},
      {3.99680288865056e-14, 2.59512887494072, 1.15463194561016e-14, 0.0, 2.1316282072803e-14, 10.0285307284481, -0.767649473578724, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.4130361329658, -1.26347611061159, 0.0967147429123489, 1.4210854715202e-14, 2.1316282072803e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.59872115546023e-14, 2.321153829896, 0.0, 0.0, 0.0, 3.92428337406973, 5.49285249860476, 1.77635683940025e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 17.3044631365566, -1.73008591662714, 1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 1.4210854715202e-14, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.0, 2.01017818278143, 0.0, 0.0, 1.37667655053519e-14, 2.30926389122033e-14, 8.91928088068603, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 16.8558564884488, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.96647939483829, 0.0, 8.12403840463596, -1.64130361329659, 1.24344978758018e-14, -1.06581410364015e-14, 0.0, 11.7728501222092, -2.40312296813958, 0.485504156227613, 0.0, -1.77635683940025e-14, 1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 10.2761860629321, -1.48323969741915, 0.302765035409786, -0.0611677741841339, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.22044604925031e-14, -1.06581410364015e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0},
      {2.65329983228433, -1.95399252334028e-14, 4.15220619990514, 2.93605316083771, 0.0, 0.0, -1.59872115546023e-14, 2.63248931621765, 9.82591326180786, -2.82261277137741, 0.0, 0.0, 1.28785870856518e-14, -1.99840144432528e-14, 2.1316282072803e-14, -2.8421709430404e-14, 1.04360964314765e-14, 11.6081867662439, -3.09487177319466, 0.683876505540214, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, -2.34812169708221e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, -1.24344978758018e-14},
      {2.29782505861521, 0.0, 1.49829835452874, 5.50917601898539, 0.0, 0.0, 2.04281036531029e-14, 0.0, 6.64803945320938, 4.13676892548488, 0.0, -1.59872115546023e-14, 0.0, 0.0, -2.08721928629529e-14, 0.0, -1.06581410364015e-14, 0.0, 12.5636136423841, -3.55352656109504, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 1.4210854715202e-14},
      {1.87616630392938, 0.0, 1.50990331349021e-14, 6.2283092998577, 0.0, 0.0, 0.0, 0.0, 0.0, 11.0541524195338, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.07611705604904e-14, 0.0, 11.6057691494799, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {4.83186700722507, 0.0, -4.41087092453691, -3.11895674167863, 16.6749979177077, 0.0, 1.55431223447522e-14, 2.13065307820924, 1.73967095350099, 1.23013312825381, 0.0, -4.36653943835006, -3.7815340802378, 0.0, 2.66453525910038e-14, 1.24344978758018e-14, -0.55793594346197, -0.483186700722504, -0.394520289089684, -0.278967971730996, 23.6981011897578, 0.0, 0.0, 0.490653381462661, 0.43885372573626, 0.380058475033048, 0.0, 0.0, 0.0, 0.0, 0.0626933893908861, 0.0560746721671623, 0.0485620906056381, 0.0396507809422189, 0.0280373360835759, 0.0, 0.0, -1.15463194561016e-14, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-1.33226762955019e-14, 6.91050664139876, -1.68753899743024e-14, 1.57651669496772e-14, 1.95399252334028e-14, 10.5706241650196, -2.72554057547699, 0.0, 0.0, 0.0, 4.67332857821917, -1.50990331349021e-14, 0.0, -7.86705867811455, -2.39719223514614, 0.618094504365238, 0.0, 2.30926389122033e-14, 0.0, 0.0, 0.0, 23.1055548876599, -0.469687194424188, 0.0, 0.0, 0.0, 1.62065746295905, 0.790669146636831, 0.240926884676767, -0.062120835029094, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 1.11022302462516e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24185123190546e-14, 6.30840061880561, 0.0, 0.0, 0.0, -1.52362350055012, 11.1963004093827, 0.0, -2.06501482580279e-14, -1.77635683940025e-14, 4.26614580154031, 0.0, 1.4210854715202e-14, 0.132992762321621, -4.72217953828914, -6.67684597013017, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05209757977173e-14, 23.1532444221696, 0.0, 0.0, 0.0, 0.0, 0.721778878560575, 1.1112512795529, 1.1908743922773, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.85449644663773, 1.15463194561016e-14, 5.98171817966649, -2.48806675764059, 1.90029237516522, 1.24344978758018e-14, 0.0, -0.339934634239512, -2.35922146606162, 0.981306762925322, 1.24344978758018e-14, 9.95226703056239, -0.430945803685663, 0.0, -2.04558592287185e-14, 1.77635683940025e-14, -6.45364422706454, 0.0770899289327289, 0.535021138879199, -0.222539456105666, 0.0, 0.0, 0.0, 21.5273297444417, -1.00024047132727, 0.0433116829031377, -1.15463194561016e-14, 0.0, 1.35447209004269e-14, 0.0, 2.75066129615001, 0.648615649442918, -0.00774782937530905, -0.0537716476536647, 0.02236605687735, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.51865775274498, -1.06581410364015e-14, 1.92724822331889, 2.72554057547698, 1.73472166622177, 2.57571741713036e-14, 0.0, -2.01705689521106, 3.42052627529743, -4.03112887414929, 0.0, 4.08831086321546, 5.37643791520783, 0.0, -4.61852778244065e-14, 0.0, 0.365669640132054, -6.25148194071025, -2.03981013744544, 1.80803322065279, 0.0, 0.0, 0.0, 0.0, 22.1880978975596, -1.2652152389098, 0.0, 0.0, 1.37667655053519e-14, 0.0, -1.06581410364015e-14, 1.98456380761455, 1.4711357857929, 0.363818136386332, -0.294009452979937, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, -1.59872115546023e-14, 1.77635683940025e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 2.1316282072803e-14, 3.73034936274053e-14, 0.0, 0.0, 0.0},
      {3.14718316987779, -1.86517468137026e-14, -0.861891607371326, 6.09449400220041, 1.55158222708543, 0.0, 0.0, 0.0693888666488709, -3.05941170815566, 1.802775637732, 0.0, 2.61457522299224e-14, 9.03122156537879, 0.0, -4.30766533554561e-14, -2.8421709430404e-14, 0.0, 0.403888506800978, -3.67461994579465, -5.49650682721196, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 21.7841260588564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.974215733960556, 1.78974858569578, 1.80791908794929, 0.0, 1.15463194561016e-14, -1.95399252334028e-14, 0.0, -1.73194791841524e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.95399252334028e-14, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14},
      {-3.81916720471054e-14, 1.54523626091312, 0.0, 0.0, -1.4210854715202e-14, 6.71778024562962, -0.609449400220049, 0.0, 1.77635683940025e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 14.6593983227032, -1.52344936219873, 0.138210132827965, 1.4210854715202e-14, 1.95399252334028e-14, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.119501277094, -1.47332496632642, 0.153112421864223, -0.0138906409944086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {1.95399252334028e-14, 1.41060126129514, 0.0, 0.0, 1.06581410364015e-14, 3.63405410063596, 2.50356888118887, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 3.56857028479909, 11.263055412878, -1.49298814645063, 0.0, -3.5527136788005e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 19.3673596462204, -2.65048896124155, 0.266287609379567, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.73034936274053e-14, 1.2616801237611, 0.0, 0.0, 0.0, 1.42204860051346, 4.47852016375305, 1.58137392070046e-14, 3.01980662698043e-14, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 8.27725128825318, 5.37909280973862, 3.37507799486048e-14, 2.57571741713036e-14, 0.0, -3.5527136788005e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 2.39808173319034e-14, 0.0, 0.0, 0.0, 19.965481322634, -2.61992366301005, 0.0, 0.0, 2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 1.06581410364015e-14, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14},
      {0.0, 1.09264703862702, 1.4210854715202e-14, -2.57571741713036e-14, -1.4210854715202e-14, -1.50990331349021e-14, 5.38682254607084, -1.66533453693773e-14, 0.0, 0.0, 1.68753899743024e-14, 0.0, -3.24185123190546e-14, -1.37667655053519e-14, -3.15303338993544e-14, 13.4377817958113, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.907670401189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.37587727269783e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.52336024752224, 0.0, 7.37120708415868, -1.62882203585592, 0.0, 0.0, 0.0, 12.0171306297065, -2.90724328050879, 0.642416074439626, 0.0, 0.0, 0.0, -1.06581410364015e-14, -2.1316282072803e-14, 0.0, 13.9858820944259, -2.72522906732405, 0.659300804656283, -0.145686271816939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.8043830863938, -1.40563403846548, 0.273895826790435, -0.0662622240310455, 0.0146420212340154, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {2.30350221379959, 0.0, 4.41588043316396, 1.78428514239951, 1.02140518265514e-14, 0.0, -2.48689957516035e-14, 4.26614580154033, 7.83741028656788, -2.63899331455873, 0.0, 0.0, 2.48689957516035e-14, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.23427840700305, 13.2221027072096, -4.67379217909106, 1.18363558466231, 0.0, 0.0, 0.0, 0.0, -1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 12.1258851598178, -3.11150358274813, 0.833612074594091, -0.192474367616153, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 2.42028619368284e-14, -2.43138842392909e-14, 1.11022302462516e-14, 0.0},
      {2.06031501455085, -1.4210854715202e-14, 2.25696201807214, 3.98978286964829, 0.0, 0.0, 0.0, 0.953939201416937, 8.01142041980429, 1.18019368870416, 0.0, 0.0, 1.77635683940025e-14, -1.4210854715202e-14, -3.5527136788005e-14, -3.19744231092045e-14, -1.4210854715202e-14, 5.55254896421457, 9.6224137438848, -3.59830837050572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.29896093881143e-14, 0.0, 0.0, 0.0, 13.3932520322736, -4.68666192508058, 1.18356086726903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.78428514239954, 1.99840144432528e-14, 0.814411017927992, 5.06770487035907, 0.0, 1.59872115546023e-14, 1.77635683940025e-14, 2.30926389122033e-14, 4.01510046524762, 6.58672305740272, 0.0, 1.4210854715202e-14, 0.0, 0.0, 1.95399252334028e-14, 2.1316282072803e-14, 4.08562073062058e-14, 0.0, 10.0159311874144, 2.83293314500741, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.0929454377398, -4.55552790213386, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 0.0, 0.0, 0.0, 1.17683640610267e-14, 0.0, 1.50990331349021e-14, -1.33226762955019e-14, -1.06581410364015e-14, 0.0},
      {1.45686271816938, -1.59872115546023e-14, 0.0, 5.0781645406624, -1.04360964314765e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, 10.0142755247553, 0.0, 0.0, 4.08562073062058e-14, -1.0547118733939e-14, -3.10862446895044e-14, -3.19744231092045e-14, 0.0, 2.46469511466785e-14, 0.0, 13.8784813613723, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 1.3988810110277e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 12.5535587278944, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.05193631583234e-14, 1.06581410364015e-14, 1.77635683940025e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats1[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.47722557505166, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 7.2456883730947, 4.18330013267039, -0.591607978309968, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.79856129989275e-14, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338697, 0.0, 0.0, 4.34741302385684, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-2.6457513110646, 0.0, 9.66091783079295, 0.683130051063959, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, 0.0, -2.1316282072803e-14, 0.0, 1.77635683940025e-14, -1.06581410364015e-14, 1.38222766565832e-14, 0.0, -1.06581410364015e-14, -2.35367281220533e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.24037034920392, 0.0, 0.0, 7.52994023880669, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.01030295240889e-14, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43773881688958e-14, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772445, 2.44948974278317, 2.82842712474622, -1, 9.16515138991167, 7.09929573971955, -0.414039335605414, -2.04939015319191, -0.478091443733753, 0.169030850945702, 0.0, -1.4210854715202e-14, -3.19744231092045e-14, 0.0, 0.0, 0.0, 2.26485497023532e-14, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974979, 2.12132034355964, -0.408248290463868, 3.17542648054294, 0.0, 0.0, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000004, -5.69209978830307, 4.38178046004134, -0.77459666924149, 1.4210854715202e-14, 10.998181667894, 0.96214047088474, 4.76235235991627, -0.740656079818032, 0.130930734141596, 0.0, 0.0, 0.0, 0.0, 2.68673971959288e-14, 1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.43218770176645e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681197, 2.73861278752581, 1.58113883008417, 2.45967477524978, 0.0, 0.0, 9.25820099772551, 0.0, 5.34522483824849, -1.20948631362952, 0.0, 0.0, 0.0, -1.46549439250521e-14, 0.0, -3.64153152077051e-14, 0.0, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.52378110129803e-14, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613579, 0.0, 0.0, 3.83405790253617, 0.0, 0.0, 0.0, 0.0, 0.0, 5.18459255872629, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.19615242270666, 2.8421709430404e-14, -3.16227766016838, -2.2360679774998, 0.0, 0.0, -1.06581410364015e-14, 13.7477270848675, 0.534522483824865, 0.377964473009229, -4.2632564145606e-14, -3.41948691584548e-14, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 3.111400026512e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.17683640610267e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, -2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.87627691161651e-14, 0.0, 0.0, 0.0},
      {-1.8, -2.48689957516035e-14, 3.65148371670108, -2.8401877872188, 0.0, 0.0, 0.0, 0.0, 12.3442679969674, 1.39659449751035, 0.0, 0.0, -1.59872115546023e-14, -1.15463194561016e-14, 0.0, -4.2632564145606e-14, 0.0, 0.0, -4.70734562441066e-14, -2.8421709430404e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 1.55431223447522e-14, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, -1.33226762955019e-14},
      {2.20454076850486, 1.02140518265514e-14, 0.0, 6.6407830863536, 0.0, 0.0, 0.0, 0.0, 0.0, 8.97997772825747, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239557, 3.07059789431496, 1.77281052085585, -1.25356634105601, 2.39356776939087, 6.30376871403132, -0.908295106229238, -2.78313372537744, -0.524404424085072, 0.370809924354778, 11.1242977306435, 9.4017475579201, -0.301561193523231, -2.42752078113804, -0.794200206656972, 0.114434427054267, 0.840917865872081, 0.350641890608835, 0.0660687472643539, -0.0467176592151086, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047607, 2.74642624930236, -0.792824967172086, 2.24244764232555, 2.14087209644418, -0.331662479035522, 6.49923072370876, 0.0957427107756397, -1.87616630392936, -2.15580611373101, 0.0, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932523, 0.590936840285288, 0.522319308773337, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0},
      {1.91485421551272, 1.29756443747036, 3.74574588632197, -1.05945692672796, -8.09173593712687, 5.01426536422406, -0.383824736789365, 1.80936716113938, -1.10800657553489, 0.313391585264003, 0.0, 12.7134831207354, 1.01946290306868, 8.20651806648289, -0.631738055305789, 0.04835737145617, -2.84282124887606, -0.227958835180083, 0.139595707137603, -0.0394836284566153, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715533, 1.16057691494798, 1.34011878852097, 1.89521416591737, 2.5331140255951, 1.96214168703486, 2.74642624930237, -0.566421515598876, 3.17129986868837, -1.82198870938952, 0.0, 0.0, 11.1699829238239, -1.06581410364015e-14, 8.65223156827827, -0.865042958313609, 1.08801856413265e-14, -2.49768411251822, -0.998865569685867, 0.441440386136639, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741913, 1.00508909139072, -0.193429485824669, 3.55615782880925, 0.0, 1.02140518265514e-14, 4.45964044034302, 0.0, -0.858258202906989, 2.67027285925187, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 8.42792824422442, 0.0, 0.0, -1.62195554683794, -2.29379153190449, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741912, 7.03562363973514, 4.06201920231799, -0.820651806648291, 1.06581410364015e-14, -6.79705818718657, -2.08116553882675, 5.8864250611046, -1.20156148406979, 0.24275207811381, 1.06581410364015e-14, -2.48689957516035e-14, 1.02140518265514e-14, 14.8323969741913, 0.856348838577658, 0.262202212042542, 5.13809303146606, -0.741619848709565, 0.151382517704904, -0.0305838870920609, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.70974345792274e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, -2.69693703815181, 2.07610309995257, 1.46802658041886, 0.0, 3.03973683071412, -6.38211787508101, 1.31624465810883, 4.91295663090392, -1.4113063856887, 0.0, 0.0, 0.0, 0.0, 13.4039795085888, 2.01017818278146, 0.0, 5.80409338312193, -1.54743588659734, 0.341938252770115, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1489125293076, 1.29756443747033, 0.749149177264371, 2.75458800949269, 1.22124532708767e-14, 1.4210854715202e-14, 5.75737105184055, 0.0, 3.3240197266047, 2.06838446274243, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, -1.33226762955019e-14, -2.1316282072803e-14, 10.8804085776374, 0.0, 0.0, 6.28180682119207, -1.77676328054754, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {0.93808315196469, 0.0, 0.0, 3.11415464992885, 0.0, 0.0, 0.0, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.03805852802452e-14, 0.0, 5.80288457473997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.42539535431071, 0.0, 9.38083151964686, 1.89521416591739, 1.24344978758018e-14, 0.0, -1.24344978758018e-14, -3.39852909359329, -2.77488738510234, -0.560611910581393, 1.77635683940025e-14, 2.8421709430404e-14, 0.0, 0.0, -2.1316282072803e-14, 1.50990331349021e-14, 17.7988763690296, 0.428174419288831, 0.349602949390069, 0.0706304617818787, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.82970843102536, 0.0, -1.49829835452875, 4.23782770691181, 0.0, 0.0, -1.95399252334028e-14, 3.79967103839268, -3.54562104171166, -4.07409060843206, 0.0, 0.0, 2.1316282072803e-14, 0.0, 2.14828155264968e-14, 1.77635683940025e-14, 1.46549439250521e-14, 16.7549743857359, 1.11676565710086, 0.98709071141532, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.24344978758018e-14, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 2.28705943072782e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 1.99840144432528e-14, -1.13242748511766e-14, 1.15463194561016e-14, 0.0},
      {-1.32664991614216, 0.0, 1.73008591662711, -3.18072425757421, 0.0, 0.0, 1.4210854715202e-14, 0.0, 7.67649473578738, -2.38836465270398, -1.06581410364015e-14, 0.0, 4.2632564145606e-14, 0.0, -3.19744231092045e-14, -3.19744231092045e-14, 0.0, 0.0, 14.5072114368499, 2.05162951662072, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {1.6248076809272, 0.0, 1.4210854715202e-14, 5.39387407630362, 0.0, 0.0, 0.0, 0.0, 0.0, 9.5731768126215, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.82076576038526e-14, 0.0, 10.0508909139073, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725406, 3.45525332069939, 0.997445717412047, -1.4106012612951, 3.23209352764251, 5.28531208250979, -1.36277028773848, -2.89087233497829, -0.393397896234719, 0.556348640264175, 2.33666428910958, 9.02784807228867, -0.732969916135149, -3.93352933905728, -1.19859611757306, 0.309047252182628, 1.64018416088946, 0.655588223176224, 0.0892142571199703, -0.126168012376113, 13.0996183150502, 11.55277744383, -0.234843597212095, -2.44096975264614, -0.907332870306509, 0.0736662483163242, 0.810328731479522, 0.395334573318414, 0.120463442338384, -0.0310604175145466, -0.311895674167857, -0.164844710568314, -0.0658890955530749, -0.00896637020657787, 0.0126803623514049, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482414, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275053, 5.59815020469135, 0.263899331455872, -1.93925611658551, -2.28543525082518, 2.13307290077015, -0.257539376818858, 9.14446135479764, 0.066496381160809, -2.36108976914459, -3.33842298506509, -0.023035022137996, 0.817905488277898, 1.1564636454577, 1.02505848514082, 0.0, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.36088943928029, 0.555625639776449, 0.595437196138647, 0.0, -0.125015768949707, -0.192474367616162, -0.206265495285709, -0.166687691932939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.04360964314765e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331885, 1.82834820066015, 2.99085908983324, -1.24403337882029, 0.950146187582622, 5.74064456311311, -0.721110255092795, -0.169967317119746, -1.17961073303081, 0.490653381462657, -10.3037210107158, 4.97613351528118, -0.215472901842828, 5.7817446699566, -1.30185581821822, 0.163532434528627, -3.22682211353226, 0.0385449644663671, 0.267510569439609, -0.111269728052835, 0.0, 14.555131461167, 1.03556292536503, 10.7636648722209, -0.500120235663638, 0.0216558414515706, -3.5732138395088, -0.581087203147983, 0.130841432732457, -0.016435628072264, 1.37533064807501, 0.324307824721451, -0.00387391468765164, -0.0268858238268332, 0.0111830284386736, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637252, 1.66904592079258, 0.963624111659448, 1.36277028773848, 0.867360833110894, 2.28430636201792, 2.96226264871973, -1.00852844760553, 1.71026313764871, -2.01556443707465, 2.41867732448956, 2.04415543160772, 2.68821895760393, -0.527798662911731, 7.07977053255618, -1.76652739792481, 0.182834820066031, -3.12574097035514, -1.01990506872272, 0.904016610326397, 0.0, 0.0, 13.1266557397187, 0.0, 11.0940489487798, -0.6326076194549, 0.0, -2.86447112137561, -1.66605356688628, 0.315075748468086, 0.0, 0.992281903807271, 0.735567892896452, 0.18190906819316, -0.14700472648997, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458432, -0.430945803685661, 3.04724700110021, 0.775791113542723, -0.120185042515454, 5.29905651979671, 0.0346944333244444, -1.52970585407782, 0.901387818865999, -1.04360964314765e-14, 1.06581410364015e-14, 4.5156107826894, 0.0, -0.699555414372449, 6.36462844462226, 0.0, 0.201944253400486, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 1.4210854715202e-14, 0.0, 1.11022302462516e-14, 10.8920630294282, 0.0, 0.0, -1.68739114875267, -3.09993548319961, 0.0, 0.0, 0.487107866980282, 0.894874292847889, 0.903959543974644, 0.0, 0.0, 0.0, 0.0, -1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14},
      {1.62882203585589, 0.772618130456562, 3.86595114186569, -1.05140010313426, 9.1544290607646, 3.3588901228148, -0.304724700110031, 3.87851223317101, -1.52475169285615, 0.414677792940102, 0.0, -9.46260092820841, -2.07602937339135, 7.32969916135159, -0.761724681099374, 0.0691050664139912, 0.564240504518065, -0.879563899362182, 0.345781182036202, -0.0940400840863373, 0.0, 0.0, 0.0, 16.3745228938128, 0.951027176355639, 0.208648802584719, 9.059750638547, -0.736662483163207, 0.076556210932115, -0.00694532049720431, -3.48709964663733, -0.0567083043941596, 0.088399497979587, -0.034752316375144, 0.00945138406569224, 0.0, 0.0, 0.0, 1.4432899320127e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 1.52100554373646e-14, 0.0, 0.0, 0.0},
      {1.48690428533298, 0.705300630647577, 2.03602754481991, 1.15175110689978, -2.93221479395415, 1.81702705031799, 1.25178444059443, 0.655663160391216, 3.61359041872289, -1.70346285967311, 0.0, 2.76420265655949, -9.08783875864325, 1.78428514239954, 5.63152770643901, -0.746494073225321, -0.618094504365241, 2.03210252328837, -2.1549427706255, 0.764033484562605, 0.0, 0.0, 0.0, 0.0, 15.0018922739643, 2.13860408462395, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689787, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392897, -0.124241670058189, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, -1.06581410364015e-14, 0.0, 0.0},
      {1.32992762321608, 0.630840061880541, 0.72843135908467, 2.57539376818857, 0.917928424547676, 0.711024300256726, 2.23926008187654, -0.205255035576785, 2.58567482211403, 0.761811750275049, 1.77635683940025e-14, 0.0, 5.34294273196308, 0.0, 4.13862564412655, 2.68954640486928, 1.24344978758018e-14, -1.19471831485578, 3.10562068169857, -2.32269806558127, 0.0, 0.0, 1.35447209004269e-14, 0.0, 1.46549439250521e-14, 12.8876627769294, 0.0, 0.0, 9.98274066131697, -1.30996183150503, 0.0, 0.0, -2.88176900403077, -1.51261363209512, 0.763985254692606, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.37667655053519e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.73194791841524e-14, -1.59872115546023e-14, -1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.1517511068998, 0.546323519313517, -0.105140010313417, 3.27118942773249, 0.0, 0.0, 2.69341127303542, 0.0, -0.518347241175128, 4.25171145123353, 1.4210854715202e-14, 0.0, -2.50910403565285e-14, 0.0, 0.0, 6.71889089790565, 0.0, 0.0, -1.29305115618717, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.45383520059452, 0.0, 0.0, 0.0, -1.81939143264587, -2.94058061635794, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376109, -6.78263087840207, 3.68560354207934, -0.814411017927949, -2.1316282072803e-14, 10.0216907612581, 2.6751056943961, 6.00856531485323, -1.45362164025438, 0.321208037219803, -2.1316282072803e-14, 1.06581410364015e-14, 0.0, -7.57008074256672, -2.27270584034422, -0.60665694841585, 6.99294104721298, -1.36261453366201, 0.329650402328141, -0.0728431359084754, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 18.7137404500718, 0.760821741084513, 0.22841553125065, 0.0609713174047695, 5.40219154319687, -0.702817019232745, 0.136947913395213, -0.0331311120155262, 0.00732101061700652, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689979, 3.82426463519459, 2.20794021658197, 0.892142571199754, 1.59872115546023e-14, -2.4630604269215, 6.78739640804926, 2.13307290077016, 3.91870514328394, -1.31949665727937, 0.0, -1.19904086659517e-14, 1.4210854715202e-14, 3.22490309931942, -7.63378455726036, -4.04762275910193, 1.11713920350151, 6.61105135360479, -2.33689608954554, 0.591817792331161, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.5022076529583, 1.79642743108411, 0.721929233499938, 0.0, 6.06294257990891, -1.55575179137407, 0.416806037297049, -0.096237183808074, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 1.12548859121375e-14, 0.0, 0.0, 0.0},
      {1.03015750727541, -1.46593983227031, 1.12848100903607, 1.99489143482414, 0.0, 1.10151410945722, -5.20357020296094, 0.476969600708472, 4.00571020990214, 0.590096844352075, 0.0, 0.0, 1.98729921407903e-14, 1.22124532708767e-14, 6.41153127835567, -6.2499410609466, 0.0, 2.7762744821073, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 15.4651953323153, 3.04407621455179, 0.0, 0.0, 6.69662601613679, -2.34333096254029, 0.591780433634507, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.64313007644523e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.892142571199775, 0.705300630647563, 0.407205508964006, 2.53385243517952, 0.0, 0.0, 3.47717900165118, 0.0, 2.00755023262381, 3.29336152870136, 0.0, 1.33226762955019e-14, -2.43000064514831e-14, 0.0, 0.0, 8.67405085085773, 1.86517468137026e-14, 0.0, 5.00796559370721, 1.4164665725037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 12.2048487632307, 0.0, 0.0, 0.0, 7.04647271886989, -2.27776395106693, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0},
      {0.72843135908469, 0.0, 0.0, 2.5390822703312, 0.0, 0.0, 0.0, 0.0, 0.0, 5.00713776237767, 0.0, 0.0, 2.04281036531029e-14, 0.0, -1.55431223447522e-14, -1.59872115546023e-14, 0.0, 1.23234755733392e-14, 0.0, 6.93924068068616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.2767793639472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.09901951359276, 0.0, -4.65474668125634, -3.29140294302191, -1.24344978758018e-14, 0.0, 1.77635683940025e-14, 13.4907375632321, 1.83585684909537, 1.29814682728312, -1.4210854715202e-14, 0.0, -2.8421709430404e-14, 0.0, -1.59872115546023e-14, 0.0, -3.53270434653116, -3.05941170815571, -0.416333199893259, -0.294392028877606, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 21.8326971917504, 0.355050145839439, 0.307482445914334, 0.0418430609640565, 0.0295875121532903, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0},
      {-2.6598552464322, 1.84297022087776e-14, 5.0990195135928, -2.06031501455088, 1.50990331349021e-14, 0.0, 0.0, -1.23153021346074, 9.049861877399, 3.04724700110021, 0.0, 0.0, -2.48689957516035e-14, 1.15463194561016e-14, -2.8421709430404e-14, 3.5527136788005e-14, 3.86988371918333, -3.81689227863023, -5.39683034546922, -1.36674464685443, 0.0, 0.0, 0.0, 0.0, 0.0, 1.46549439250521e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 21.00264918355, 0.898213715542066, 0.962572311333256, 0.22225025591058, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, -1.82076576038526e-14, 0.0, -1.06581410364015e-14, 0.0, 1.4210854715202e-14, 1.33226762955019e-14, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, -1.15463194561016e-14, 0.0, 0.0},
      {2.97380857066589, 1.4210854715202e-14, -0.814411017927924, 5.75875553449895, 0.0, 1.55431223447522e-14, -1.06581410364015e-14, 1.37689263682152, -2.89087233497829, 1.7034628596731, -1.4210854715202e-14, 3.01980662698043e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 8.01441409794463, -3.47218947830362, -5.19371076584693, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 19.3314941653941, 1.69115345252877, 1.70832296330019, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.73194791841524e-14, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 1.46549439250521e-14, 0.0, 0.0, 0.0},
      {-1.03015750727543, 0.0, 0.940400840863422, -2.9258407710754, -1.08801856413265e-14, 1.06581410364015e-14, 0.0, 0.0, 4.63623866886821, -3.80284633026899, -1.06581410364015e-14, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 2.17603712826531e-14, 0.0, 11.565401134477, -1.63559471386626, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.2731316843076, 2.63013526059783, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.26168012376113, 0.0, 0.0, 4.39781949681097, 0.0, 0.0, 0.0, 0.0, 0.0, 8.67261700493486, 0.0, 0.0, 3.19744231092045e-14, 0.0, -2.66453525910038e-14, -2.66453525910038e-14, 0.0, 2.1094237467878e-14, 1.06581410364015e-14, 12.0191174248973, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.13242748511766e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 10.8717007662564, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.32116539930394e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
      
      static const double dmats2[56][56] = \
      {{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {3.16227766016838, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.82574185835055, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.16397779494322, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.68753899743024e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.9580398915498, 5.6124860801609, -1.08012344973463, -0.76376261582597, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.29128784747791, 1.44913767461894, 4.18330013267039, -0.591607978309966, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.87082869338696, 7.09929573971953, 0.0, 4.34741302385683, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, -1.06581410364015e-14, -1.0769163338864e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32287565553229, 0.0, 3.86436713231719, -0.341565025531991, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.08012344973464, 1.06581410364015e-14, 7.09929573971954, 2.50998007960222, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.28785870856518e-14, 1.77635683940025e-14, 0.0, 0.0, 0.0, -1.24344978758018e-14, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {-3.81881307912987, 0.0, -2.1316282072803e-14, 8.87411967464942, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -2.1316282072803e-14, -1.24344978758018e-14, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.74954541697349, 5.79655069847576, -1.67332005306814, -1.18321595661993, 7.74596669241482, -1.19999999999999, -0.979795897113274, 0.346410161513779, 0.282842712474625, 0.200000000000001, 0.0, -2.30926389122033e-14, -1.06581410364015e-14, 0.0, -1.86517468137026e-14, -1.2712053631958e-14, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.11022302462516e-14, 0.0, 0.0, 0.0},
      {2.32379000772446, 2.44948974278318, 2.82842712474621, -1.00000000000001, 1.30930734141595, 7.09929573971953, -0.414039335605414, -2.04939015319192, -0.478091443733756, 0.169030850945708, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.01246117974978, 2.12132034355965, -0.408248290463868, 3.17542648054294, 9.07114735222144, 2.30926389122033e-14, 7.17137165600635, 0.0, -1.3801311186847, -1.56144011671765, 0.0, -2.57571741713036e-14, -1.77635683940025e-14, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.80000000000001, 0.632455532033672, 4.38178046004134, -0.774596669241498, 0.0, 3.14233761939829, -0.106904496764969, 4.76235235991627, -0.740656079818038, 0.130930734141598, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 1.15463194561016e-14, 0.0, 1.15463194561016e-14, -1.53210777398272e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.55884572681201, 0.547722557505165, 1.58113883008419, 2.45967477524978, 0.0, 9.07114735222144, 1.85164019954511, 0.0, 5.34522483824849, -1.20948631362952, 0.0, -1.15463194561016e-14, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, -1.77635683940025e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.27279220613578, -6.2609903369994, -1.24344978758018e-14, 3.83405790253616, 1.73194791841524e-14, 0.0, 10.5830052442584, 1.24344978758018e-14, 1.24344978758018e-14, 5.18459255872629, 0.0, 0.0, -1.99840144432528e-14, 1.59872115546023e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0, -1.68753899743024e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.03923048454133, 0.0, 3.16227766016838, -0.447213595499965, 0.0, 0.0, 0.0, 5.8918830363718, -0.534522483824835, 0.0755928946018458, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.900000000000011, 0.0, 1.46059348668042, 1.42009389360939, 0.0, 0.0, 0.0, 9.07114735222146, 4.93770719878696, -0.698297248755178, -1.06581410364015e-14, -3.01980662698043e-14, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.73484692283496, -2.48689957516035e-14, -6.26099033699944, 2.21359436211788, 0.0, -1.59872115546023e-14, 0.0, -1.06581410364015e-14, 10.5830052442584, 2.99332590941915, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, -1.06581410364015e-14, -1.82076576038526e-14, 1.55431223447522e-14, -2.39808173319034e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {5.7157676649773, 0.0, 2.30926389122033e-14, -4.69574275274956, 1.46549439250521e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 12.69960629311, 0.0, 1.95399252334028e-14, -1.59872115546023e-14, 0.0, 0.0, 0.0, 1.50990331349021e-14, 1.97619698383278e-14, 0.0, -1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.19904086659517e-14, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.34336985979644e-14, 0.0, 0.0, 0.0},
      {2.56904651573303, 5.80288457473996, -2.01017818278147, -1.42141062443802, 8.14215222512714, -2.10229466468019, -1.71651640581398, 0.728256234341421, 0.594618725379069, 0.420458932936038, 9.81070843517428, -1.18450885369837, -1.02581475831037, 0.305838870920601, 0.264864231681979, 0.216260739578378, -0.105945692672791, -0.0917516612761631, -0.0749149177264187, -0.0529728463363903, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0},
      {2.26568606239554, 3.07059789431496, 1.77281052085586, -1.25356634105602, 2.39356776939084, 6.30376871403132, -0.908295106229245, -2.78313372537744, -0.52440442408507, 0.37080992435478, 1.23603308118261, 9.4017475579201, -0.301561193523235, -2.42752078113805, -0.79420020665698, 0.114434427054264, 0.84091786587209, 0.350641890608836, 0.0660687472643509, -0.0467176592151097, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.02649122047609, 2.74642624930235, -0.792824967172094, 2.24244764232555, 2.1408720964442, -0.331662479035521, 6.49923072370876, 0.0957427107756468, -1.87616630392937, -2.15580611373102, 11.0554159678513, 0.0, 9.44035859331466, 0.0, -1.46249406456537, -2.04706526287667, 0.0, 0.422185670932526, 0.590936840285284, 0.522319308773341, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.22124532708767e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.9148542155127, 1.29756443747036, 3.74574588632199, -1.05945692672796, 0.404586796856352, 5.01426536422406, -0.383824736789361, 1.80936716113937, -1.10800657553489, 0.313391585264004, 0.0, 2.82521847127452, -0.050973145153439, 8.20651806648289, -0.631738055305783, 0.0483573714561669, -2.84282124887605, -0.227958835180075, 0.139595707137607, -0.0394836284566216, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.71269767715534, 1.160576914948, 1.34011878852096, 1.89521416591738, 0.361873432227861, 1.96214168703487, 2.74642624930238, -0.566421515598877, 3.17129986868838, -1.82198870938951, 0.0, 11.0554159678513, 1.59571184626056, 0.0, 8.65223156827827, -0.865042958313582, 0.0, -2.49768411251823, -0.998865569685865, 0.441440386136654, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, -1.59872115546023e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741914, 1.0050890913907, -0.193429485824662, 3.55615782880924, -8.46157280212811, 1.24344978758018e-14, 4.45964044034301, 0.0, -0.858258202906992, 2.67027285925186, -1.24344978758018e-14, 2.51465515077598e-14, 12.4373429638327, -1.24344978758018e-14, -1.84297022087776e-14, 8.42792824422441, 0.0, 0.0, -1.62195554683795, -2.29379153190451, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.59872115546023e-14, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48323969741915, 0.335029697130244, 4.06201920231797, -0.820651806648291, 1.32255317808472e-14, 1.69926454679663, -0.0991031208965123, 5.8864250611046, -1.20156148406979, 0.242752078113808, 0.0, -1.59872115546023e-14, 0.0, 4.94413232473044, -0.214087209644422, 0.0124858196210829, 5.13809303146607, -0.741619848709565, 0.151382517704899, -0.0305838870920683, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0},
      {1.32664991614217, 0.299659670905752, 2.07610309995255, 1.46802658041886, 1.28785870856518e-14, 0.868496237346889, 0.709124208342335, 1.31624465810883, 4.91295663090393, -1.41130638568871, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 3.82970843102537, -0.223353131420174, 0.0, 5.80409338312196, -1.54743588659733, 0.341938252770111, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11299858218672e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.14891252930757, 0.259512887494068, 0.749149177264346, 2.75458800949269, -1.06581410364015e-14, -8.46157280212809, 1.15147421036811, 0.0, 3.3240197266047, 2.06838446274244, 0.0, -1.4210854715202e-14, 0.0, 0.0, 12.4373429638327, 2.17608171552749, 0.0, 0.0, 6.28180682119207, -1.77676328054752, 0.0, 0.0, 0.0, 0.0, 1.45439216225896e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.4210854715202e-14, 0.0},
      {0.9380831519647, 7.62808987244124, 2.57571741713036e-14, 3.11415464992885, 0.0, 0.0, -7.5213980463361, 0.0, 0.0, 5.52707620976691, 0.0, 0.0, 0.0, -1.95399252334028e-14, -2.48689957516035e-14, 14.2141062443803, 1.62092561595273e-14, 2.08721928629529e-14, 1.95399252334028e-14, 5.80288457473996, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.33226762955019e-14, 1.35447209004269e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.30451205393456e-14, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.856348838577686, 0.0, 2.68023757704197, -0.473803541479342, 0.0, -1.24344978758018e-14, 0.0, 5.09779364038994, -0.792824967172089, 0.140152977645354, 0.0, -1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 0.0, 7.91061171956871, -0.642261628933262, 0.0998865569686025, -0.0176576154454686, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.765941686205085, 0.0, 1.49829835452879, 0.847565541382361, 1.4210854715202e-14, -1.4210854715202e-14, -1.33226762955019e-14, 1.62843044502544, 3.54562104171168, -0.814818121686415, 0.0, 0.0, 1.77635683940025e-14, 0.0, -1.06581410364015e-14, 0.0, 11.0554159678513, 7.18070330817254, -1.11676565710081, 0.197418142283062, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.663324958071078, 0.0, 0.69203436665088, 1.59036212878712, 0.0, 1.15463194561016e-14, 0.0, -8.46157280212813, 3.07059789431495, 1.19418232635198, 0.0, 2.48689957516035e-14, 1.24344978758018e-14, 1.06581410364015e-14, 0.0, 0.0, 1.59872115546023e-14, 12.4373429638327, 5.80288457473995, -1.02581475831035, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.541602560309059, 1.59872115546023e-14, 7.62808987244128, 1.79795802543455, 0.0, 2.30926389122033e-14, 1.59872115546023e-14, 1.33226762955019e-14, -7.52139804633612, 3.1910589375405, 0.0, 1.24344978758018e-14, 0.0, 0.0, -1.59872115546023e-14, 0.0, 2.81441536742477e-14, 0.0, 14.2141062443803, 3.35029697130245, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.36557432028894e-14, 0.0, 0.0, 0.0},
      {-5.36159180343549, -1.24344978758018e-14, 0.0, 8.89943818451479, 0.0, 0.0, 0.0, 0.0, 0.0, -5.26497863243528, 0.0, 0.0, 2.8421709430404e-14, 0.0, 0.0, -2.8421709430404e-14, 0.0, 0.0, 0.0, 16.583123951777, 0.0, 0.0, 0.0, 0.0, 0.0, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0},
      {2.41593350361254, 5.72988941019467, -2.20543546226844, -1.55947837083931, 8.33749895885383, -2.76779953857107, -2.25989885993661, 1.06532653910462, 0.869835476750488, 0.615066564126903, 10.333118277332, -2.18326971917503, -1.8907670401189, 0.724780051083751, 0.627677936394718, 0.512496888990052, -0.278967971730977, -0.241593350361241, -0.197260144544831, -0.13948398586549, 11.8490505948789, -1.16109782023228, -1.03851746182648, 0.245326690731329, 0.21942686286813, 0.190029237516522, -0.0814411017927927, -0.0728431359084656, -0.063084006188056, -0.0515078753637666, 0.0313466946954433, 0.02803733608358, 0.0242810453028187, 0.0198253904711077, 0.0140186680417866, -1.11022302462516e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {2.18529407725405, 3.45525332069938, 0.997445717412045, -1.41060126129509, 3.23209352764251, 5.28531208250979, -1.36277028773849, -2.89087233497829, -0.393397896234718, 0.556348640264176, 2.33666428910957, 9.02784807228865, -0.732969916135162, -3.93352933905727, -1.19859611757308, 0.30904725218262, 1.64018416088946, 0.655588223176228, 0.0892142571199872, -0.126168012376106, 1.1908743922773, 11.55277744383, -0.234843597212093, -2.44096975264614, -0.907332870306512, 0.0736662483163206, 0.810328731479523, 0.395334573318414, 0.120463442338386, -0.0310604175145457, -0.311895674167864, -0.164844710568319, -0.0658890955530742, -0.0089663702065863, 0.012680362351408, 0.0, -1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.99489143482415, 3.1542003094028, -1.09264703862702, 1.54523626091313, 2.9504842217604, -0.76181175027506, 5.59815020469135, 0.263899331455869, -1.93925611658552, -2.28543525082518, 2.13307290077016, -0.257539376818832, 9.14446135479766, 0.0664963811608024, -2.36108976914458, -3.33842298506509, -0.0230350221379972, 0.817905488277895, 1.15646364545769, 1.02505848514081, 13.0453753561106, 0.0, 11.5766222110848, 0.0, -1.39771878815471, -2.15192884958005, 0.0, 0.360889439280287, 0.555625639776446, 0.595437196138647, 0.0, -0.125015768949701, -0.192474367616164, -0.206265495285701, -0.166687691932942, 0.0, 1.11022302462516e-14, 0.0, 0.0, 1.48769885299771e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.92724822331889, 1.82834820066016, 2.99085908983326, -1.24403337882029, 0.950146187582627, 5.7406445631131, -0.721110255092791, -0.169967317119749, -1.17961073303081, 0.490653381462662, 0.2943920288776, 4.97613351528119, -0.215472901842842, 5.78174466995659, -1.30185581821822, 0.163532434528638, -3.22682211353226, 0.0385449644663596, 0.267510569439606, -0.111269728052828, 0.0, 2.646387538394, -0.0295875121532871, 10.7636648722209, -0.500120235663633, 0.0216558414515711, -3.57321383950879, -0.581087203147982, 0.130841432732456, -0.0164356280722646, 1.37533064807501, 0.324307824721457, -0.00387391468765269, -0.026885823826833, 0.0111830284386755, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.50990331349021e-14, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.75932887637246, 1.66904592079257, 0.963624111659423, 1.3627702877385, 0.867360833110901, 2.28430636201795, 2.96226264871972, -1.00852844760551, 1.71026313764872, -2.01556443707465, 0.268741924943287, 2.04415543160772, 2.68821895760392, -0.52779866291172, 7.07977053255618, -1.76652739792483, 0.182834820066033, -3.12574097035512, -1.01990506872273, 0.904016610326401, 0.0, 13.0453753561106, 1.4585173044132, 0.0, 11.0940489487798, -0.632607619454902, 0.0, -2.8644711213756, -1.66605356688627, 0.315075748468091, 0.0, 0.992281903807268, 0.735567892896455, 0.181909068193166, -0.147004726489969, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.86517468137026e-14, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 2.22044604925031e-14, 0.0, 0.0, 0.0},
      {1.57359158493889, 1.49284005458434, -0.430945803685661, 3.0472470011002, 0.775791113542718, -0.120185042515475, 5.29905651979671, 0.0346944333244363, -1.52970585407783, 0.901387818865992, -10.576283741361, 0.0, 4.5156107826894, -1.22124532708767e-14, -0.69955541437245, 6.36462844462227, 0.0, 0.201944253400485, -1.83730997289732, -2.74825341360598, 0.0, 0.0, 14.3499128917217, 0.0, 0.0, 10.8920630294282, 0.0, 0.0, -1.68739114875268, -3.09993548319961, 0.0, 0.0, 0.487107866980279, 0.894874292847887, 0.903959543974638, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.33226762955019e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.62882203585591, 0.772618130456571, 3.86595114186566, -1.05140010313427, 0.160604018609909, 3.35889012281481, -0.304724700110031, 3.87851223317101, -1.52475169285614, 0.414677792940092, 0.0, 1.135512111385, -0.0364215679542204, 7.32969916135161, -0.761724681099359, 0.0691050664139866, 0.564240504518043, -0.879563899362173, 0.345781182036198, -0.0940400840863459, 0.0, 0.0, 0.0, 4.46577897103986, -0.114123261162682, 0.00366050530850104, 9.05975063854701, -0.736662483163208, 0.0765562109321067, -0.00694532049720686, -3.48709964663733, -0.0567083043941579, 0.0883994979795846, -0.0347523163751422, 0.00945138406568935, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.48690428533297, 0.705300630647568, 2.03602754481989, 1.15175110689979, 0.146610739697711, 1.81702705031798, 1.25178444059442, 0.655663160391207, 3.61359041872287, -1.70346285967311, 1.77635683940025e-14, 0.614267257013228, 0.454391937932165, 1.78428514239953, 5.63152770643898, -0.746494073225319, -0.618094504365247, 2.03210252328835, -2.15494277062553, 0.764033484562615, 0.0, 0.0, 0.0, 13.0453753561106, 3.33375383865873, -0.106930204231201, 0.0, 9.68367982311022, -1.32524448062077, 0.133143804689785, 0.0, -3.3545250915713, -0.478206411017784, 0.384353057392903, -0.124241670058192, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.32992762321607, 0.630840061880552, 0.728431359084704, 2.57539376818857, 0.131132632078242, 0.711024300256739, 2.23926008187653, -0.205255035576781, 2.58567482211402, 0.761811750275054, 1.24344978758018e-14, -10.576283741361, 0.763277533137574, 0.0, 4.13862564412657, 2.68954640486932, 3.33066907387547e-14, -1.19471831485576, 3.10562068169861, -2.32269806558128, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 1.84109468241849, 0.0, 0.0, 9.98274066131698, -1.30996183150502, 0.0, 1.24344978758018e-14, -2.88176900403077, -1.51261363209513, 0.763985254692614, 0.0, 0.0, 0.0, 1.4210854715202e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0},
      {1.1517511068998, 0.546323519313524, -0.105140010313417, 3.27118942773248, 9.65295620481433, -1.26565424807268e-14, 2.69341127303543, 0.0, -0.518347241175116, 4.25171145123353, 2.22044604925031e-14, 0.0, -9.91526600752599, 0.0, -2.30926389122033e-14, 6.71889089790563, 0.0, 0.0, -1.29305115618718, 1.82865048192213, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 0.0, 0.0, 0.0, 9.45383520059451, 0.0, 0.0, 0.0, -1.81939143264586, -2.94058061635794, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {1.2616801237611, 0.199489143482413, 3.68560354207934, -0.814411017927955, 0.0, 1.02786571910339, -0.0786795792469475, 6.00856531485323, -1.45362164025439, 0.321208037219809, 0.0, 0.0, 0.0, 3.02803229702669, -0.233098034907112, 0.0178428514239979, 6.99294104721298, -1.36261453366202, 0.329650402328149, -0.0728431359084696, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.80499652729883, -0.304328696433804, 0.0234272339744266, -0.00179327404131646, 5.40219154319687, -0.702817019232744, 0.136947913395217, -0.0331311120155253, 0.00732101061700685, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.28785870856518e-14, 0.0, 0.0, 0.0},
      {1.15175110689976, 0.18210783977116, 2.20794021658199, 0.892142571199753, -2.1316282072803e-14, 0.615765106730376, 0.323209352764229, 2.13307290077017, 3.91870514328394, -1.31949665727937, 0.0, 0.0, 2.68673971959288e-14, 1.07496769977314, 1.90844613931509, -0.19274394090961, 1.11713920350153, 6.6110513536048, -2.33689608954553, 0.591817792331153, 0.0, 0.0, 0.0, 0.0, -1.08801856413265e-14, 0.0, 13.0453753561106, 5.83406921765278, -0.449106857771034, 0.0343775825476146, 0.0, 6.06294257990892, -1.55575179137406, 0.416806037297044, -0.0962371838080762, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 1.67643676718399e-14, -1.13242748511766e-14, 0.0, 0.0},
      {1.03015750727543, 0.162882203585586, 1.12848100903607, 1.99489143482413, 0.0, 0.314718316987769, 0.578174466995669, 0.476969600708461, 4.00571020990214, 0.590096844352074, 0.0, 0.0, 1.24344978758018e-14, -10.576283741361, 1.83186607953018, 0.694437895660716, 0.0, 2.77627448210728, 4.81120687194239, -1.79915418525287, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 14.3499128917217, 4.41862723780437, -0.33823069050576, -1.06581410364015e-14, 0.0, 6.69662601613678, -2.34333096254029, 0.591780433634509, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.89214257119979, 0.141060126129501, 0.407205508964017, 2.53385243517953, 0.0, 9.65295620481434, 0.695435800330237, 1.4210854715202e-14, 2.00755023262382, 3.29336152870135, 0.0, 1.59872115546023e-14, 0.0, 0.0, -9.91526600752598, 1.73481017017153, 3.9190872769268e-14, 2.69784194983913e-14, 5.00796559370721, 1.41646657250368, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574686, 2.44096975264613, 0.0, 0.0, 0.0, 7.0464727188699, -2.27776395106694, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.40998324127395e-14, 0.0, 1.36557432028894e-14, 0.0, 0.0, -1.33226762955019e-14},
      {0.728431359084686, -7.94708263760855, -1.25455201782643e-14, 2.53908227033119, -1.77635683940025e-14, -1.77635683940025e-14, 10.2207771580387, -1.50990331349021e-14, -1.06581410364015e-14, 5.00713776237769, 0.0, 1.46549439250521e-14, 0.0, 0.0, -1.24344978758018e-14, -8.49879943502226, -2.48689957516035e-14, 0.0, 0.0, 6.93924068068618, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 1.06581410364015e-14, 0.0, -1.59872115546023e-14, 17.9373911146521, 0.0, 0.0, 0.0, 0.0, 6.27677936394721, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, -1.24344978758018e-14, 0.0, 0.0, 0.0},
      {0.72843135908467, 0.0, 2.32737334062816, -0.470200420431697, 0.0, 0.0, 0.0, 4.49691252107735, -0.917928424547688, 0.185449546754729, 0.0, 1.4210854715202e-14, -1.77635683940025e-14, 0.0, -2.1316282072803e-14, 0.0, 7.06540869306229, -1.01980390271858, 0.208166599946608, -0.0420560041253615, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.92395326897747, -0.710100291678875, 0.102494148638112, -0.0209215304820314, 0.004226787450472, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.664963811608014, 4.9737991503207e-14, 1.45686271816939, 0.515078753637706, 0.0, 1.59872115546023e-14, 1.06581410364015e-14, 1.84729532019113, 2.58567482211398, -0.761811750275052, 1.4210854715202e-14, 3.01980662698043e-14, -1.77635683940025e-14, 0.0, 0.0, 2.08721928629529e-14, 1.71994831963704, 5.7253384179452, -1.54195152727693, 0.341686161713616, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 13.0453753561106, 9.33451074824443, -1.34732057331309, 0.275020660380929, -0.0555625639776334, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.77635683940025e-14, -1.95399252334028e-14, 0.0, -1.33226762955019e-14, 0.0, 1.15463194561016e-14},
      {0.594761714133166, 0.0, 0.814411017927977, 1.15175110689978, 0.0, 2.26485497023532e-14, 0.0, 0.59009684435208, 2.89087233497829, 0.340692571934619, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, -10.576283741361, 3.4347488991191, 3.47218947830368, -1.03874215316939, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.06581410364015e-14, 1.06581410364015e-14, 0.0, 0.0, 14.3499128917217, 8.28492607088319, -1.69115345252877, 0.341664592660037, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.1316282072803e-14, 0.0, 0.0, 0.0, 0.0, -1.02140518265514e-14, 0.0, 0.0, 0.0, 0.0},
      {0.515078753637714, 0.0, 0.376160336345356, 1.46292038553768, 0.0, 0.0, 1.15463194561016e-14, 9.65295620481434, 1.85449546754729, 1.90142316513447, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, -1.77635683940025e-14, 0.0, -9.91526600752598, 4.62616045379079, 0.817797356933092, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 15.9443476574685, 6.50925267372302, -1.31506763029894, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.95399252334028e-14, 0.0, 0.0, -1.59872115546023e-14},
      {0.420560041253717, -1.55431223447522e-14, -7.94708263760857, 1.46593983227029, 0.0, 0.0, 0.0, 0.0, 10.2207771580387, 2.89087233497828, 0.0, 0.0, 0.0, 0.0, 2.30926389122033e-14, 0.0, -1.15463194561016e-14, 0.0, -8.49879943502225, 4.00637247496572, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -1.15463194561016e-14, 0.0, 0.0, 0.0, 0.0, 17.9373911146521, 3.6239002554188, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.22124532708767e-14, 0.0, -1.22124532708767e-14, 0.0, -1.77635683940025e-14},
      {6.542378855465, 0.0, -1.19904086659517e-14, -6.91050664139872, -1.24344978758018e-14, 0.0, 0.0, 0.0, 0.0, 12.2649325896465, 0.0, 0.0, 3.19744231092045e-14, 0.0, -4.9737991503207e-14, -2.30926389122033e-14, 0.0, 1.75415237890775e-14, 0.0, -5.66586629001479, 0.0, 0.0, 0.0, 0.0, -1.06581410364015e-14, 0.0, 1.06581410364015e-14, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 2.17603712826531e-14, 20.4998755596024, 0.0, 0.0, 0.0, 0.0, -1.77635683940025e-14, 0.0, 0.0, 1.24344978758018e-14, 1.77635683940025e-14, 0.0, 0.0, -1.95399252334028e-14, 0.0, 0.0, 0.0, 0.0, 0.0, -1.86517468137026e-14, 2.57571741713036e-14, 1.15463194561016e-14, 1.95399252334028e-14}};
      
      // Compute reference derivatives.
      // Declare pointer to array of derivatives on FIAT element.
      double *derivatives = new double[num_derivatives];
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        derivatives[r] = 0.0;
      }// end loop over 'r'
      
      // Declare derivative matrix (of polynomial basis).
      double dmats[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Declare (auxiliary) derivative matrix (of polynomial basis).
      double dmats_old[56][56] = \
      {{1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0},
      {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0}};
      
      // Loop possible derivatives.
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        // Resetting dmats values to compute next derivative.
        for (unsigned int t = 0; t < 56; t++)
        {
          for (unsigned int u = 0; u < 56; u++)
          {
            dmats[t][u] = 0.0;
            if (t == u)
            {
            dmats[t][u] = 1.0;
            }
            
          }// end loop over 'u'
        }// end loop over 't'
        
        // Looping derivative order to generate dmats.
        for (unsigned int s = 0; s < n; s++)
        {
          // Updating dmats_old with new values and resetting dmats.
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              dmats_old[t][u] = dmats[t][u];
              dmats[t][u] = 0.0;
            }// end loop over 'u'
          }// end loop over 't'
          
          // Update dmats using an inner product.
          if (combinations[r][s] == 0)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats0[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 1)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats1[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
          if (combinations[r][s] == 2)
          {
          for (unsigned int t = 0; t < 56; t++)
          {
            for (unsigned int u = 0; u < 56; u++)
            {
              for (unsigned int tu = 0; tu < 56; tu++)
              {
                dmats[t][u] += dmats2[t][tu]*dmats_old[tu][u];
              }// end loop over 'tu'
            }// end loop over 'u'
          }// end loop over 't'
          }
          
        }// end loop over 's'
        for (unsigned int s = 0; s < 56; s++)
        {
          for (unsigned int t = 0; t < 56; t++)
          {
            derivatives[r] += coefficients0[s]*dmats[s][t]*basisvalues[t];
          }// end loop over 't'
        }// end loop over 's'
      }// end loop over 'r'
      
      // Transform derivatives back to physical element
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        for (unsigned int s = 0; s < num_derivatives; s++)
        {
          values[r] += transform[r][s]*derivatives[s];
        }// end loop over 's'
      }// end loop over 'r'
      
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
      
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] combinations[r];
      }// end loop over 'r'
      delete [] combinations;
      for (unsigned int r = 0; r < num_derivatives; r++)
      {
        delete [] transform[r];
      }// end loop over 'r'
      delete [] transform;
        break;
      }
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    // Compute number of derivatives.
    unsigned int num_derivatives = 1;
    for (unsigned int r = 0; r < n; r++)
    {
      num_derivatives *= 3;
    }// end loop over 'r'
    
    // Helper variable to hold values of a single dof.
    double *dof_values = new double[num_derivatives];
    for (unsigned int r = 0; r < num_derivatives; r++)
    {
      dof_values[r] = 0.0;
    }// end loop over 'r'
    
    // Loop dofs and call evaluate_basis_derivatives.
    for (unsigned int r = 0; r < 56; r++)
    {
      evaluate_basis_derivatives(r, n, dof_values, coordinates, c);
      for (unsigned int s = 0; s < num_derivatives; s++)
      {
        values[r*num_derivatives + s] = dof_values[s];
      }// end loop over 's'
    }// end loop over 'r'
    
    // Delete pointer.
    delete [] dof_values;
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[3];
    const double * const * x = c.coordinates;
    switch (i)
    {
    case 0:
      {
        y[0] = x[0][0];
      y[1] = x[0][1];
      y[2] = x[0][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 1:
      {
        y[0] = x[1][0];
      y[1] = x[1][1];
      y[2] = x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 2:
      {
        y[0] = x[2][0];
      y[1] = x[2][1];
      y[2] = x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 3:
      {
        y[0] = x[3][0];
      y[1] = x[3][1];
      y[2] = x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 4:
      {
        y[0] = 0.8*x[2][0] + 0.2*x[3][0];
      y[1] = 0.8*x[2][1] + 0.2*x[3][1];
      y[2] = 0.8*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 5:
      {
        y[0] = 0.6*x[2][0] + 0.4*x[3][0];
      y[1] = 0.6*x[2][1] + 0.4*x[3][1];
      y[2] = 0.6*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 6:
      {
        y[0] = 0.4*x[2][0] + 0.6*x[3][0];
      y[1] = 0.4*x[2][1] + 0.6*x[3][1];
      y[2] = 0.4*x[2][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 7:
      {
        y[0] = 0.2*x[2][0] + 0.8*x[3][0];
      y[1] = 0.2*x[2][1] + 0.8*x[3][1];
      y[2] = 0.2*x[2][2] + 0.8*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 8:
      {
        y[0] = 0.8*x[1][0] + 0.2*x[3][0];
      y[1] = 0.8*x[1][1] + 0.2*x[3][1];
      y[2] = 0.8*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 9:
      {
        y[0] = 0.6*x[1][0] + 0.4*x[3][0];
      y[1] = 0.6*x[1][1] + 0.4*x[3][1];
      y[2] = 0.6*x[1][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 10:
      {
        y[0] = 0.4*x[1][0] + 0.6*x[3][0];
      y[1] = 0.4*x[1][1] + 0.6*x[3][1];
      y[2] = 0.4*x[1][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 11:
      {
        y[0] = 0.2*x[1][0] + 0.8*x[3][0];
      y[1] = 0.2*x[1][1] + 0.8*x[3][1];
      y[2] = 0.2*x[1][2] + 0.8*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 12:
      {
        y[0] = 0.8*x[1][0] + 0.2*x[2][0];
      y[1] = 0.8*x[1][1] + 0.2*x[2][1];
      y[2] = 0.8*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 13:
      {
        y[0] = 0.6*x[1][0] + 0.4*x[2][0];
      y[1] = 0.6*x[1][1] + 0.4*x[2][1];
      y[2] = 0.6*x[1][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 14:
      {
        y[0] = 0.4*x[1][0] + 0.6*x[2][0];
      y[1] = 0.4*x[1][1] + 0.6*x[2][1];
      y[2] = 0.4*x[1][2] + 0.6*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 15:
      {
        y[0] = 0.2*x[1][0] + 0.8*x[2][0];
      y[1] = 0.2*x[1][1] + 0.8*x[2][1];
      y[2] = 0.2*x[1][2] + 0.8*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 16:
      {
        y[0] = 0.8*x[0][0] + 0.2*x[3][0];
      y[1] = 0.8*x[0][1] + 0.2*x[3][1];
      y[2] = 0.8*x[0][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 17:
      {
        y[0] = 0.6*x[0][0] + 0.4*x[3][0];
      y[1] = 0.6*x[0][1] + 0.4*x[3][1];
      y[2] = 0.6*x[0][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 18:
      {
        y[0] = 0.4*x[0][0] + 0.6*x[3][0];
      y[1] = 0.4*x[0][1] + 0.6*x[3][1];
      y[2] = 0.4*x[0][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 19:
      {
        y[0] = 0.2*x[0][0] + 0.8*x[3][0];
      y[1] = 0.2*x[0][1] + 0.8*x[3][1];
      y[2] = 0.2*x[0][2] + 0.8*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 20:
      {
        y[0] = 0.8*x[0][0] + 0.2*x[2][0];
      y[1] = 0.8*x[0][1] + 0.2*x[2][1];
      y[2] = 0.8*x[0][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 21:
      {
        y[0] = 0.6*x[0][0] + 0.4*x[2][0];
      y[1] = 0.6*x[0][1] + 0.4*x[2][1];
      y[2] = 0.6*x[0][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 22:
      {
        y[0] = 0.4*x[0][0] + 0.6*x[2][0];
      y[1] = 0.4*x[0][1] + 0.6*x[2][1];
      y[2] = 0.4*x[0][2] + 0.6*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 23:
      {
        y[0] = 0.2*x[0][0] + 0.8*x[2][0];
      y[1] = 0.2*x[0][1] + 0.8*x[2][1];
      y[2] = 0.2*x[0][2] + 0.8*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 24:
      {
        y[0] = 0.8*x[0][0] + 0.2*x[1][0];
      y[1] = 0.8*x[0][1] + 0.2*x[1][1];
      y[2] = 0.8*x[0][2] + 0.2*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 25:
      {
        y[0] = 0.6*x[0][0] + 0.4*x[1][0];
      y[1] = 0.6*x[0][1] + 0.4*x[1][1];
      y[2] = 0.6*x[0][2] + 0.4*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 26:
      {
        y[0] = 0.4*x[0][0] + 0.6*x[1][0];
      y[1] = 0.4*x[0][1] + 0.6*x[1][1];
      y[2] = 0.4*x[0][2] + 0.6*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 27:
      {
        y[0] = 0.2*x[0][0] + 0.8*x[1][0];
      y[1] = 0.2*x[0][1] + 0.8*x[1][1];
      y[2] = 0.2*x[0][2] + 0.8*x[1][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 28:
      {
        y[0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 29:
      {
        y[0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
      y[1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
      y[2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 30:
      {
        y[0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 31:
      {
        y[0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
      y[1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
      y[2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 32:
      {
        y[0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
      y[1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
      y[2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 33:
      {
        y[0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
      y[1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
      y[2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 34:
      {
        y[0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 35:
      {
        y[0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
      y[1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
      y[2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 36:
      {
        y[0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 37:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
      y[1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
      y[2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 38:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
      y[1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
      y[2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 39:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 40:
      {
        y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
      y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
      y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 41:
      {
        y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
      y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
      y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 42:
      {
        y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 43:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
      y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
      y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 44:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
      y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
      y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 45:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 46:
      {
        y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
      y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
      y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 47:
      {
        y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
      y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
      y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 48:
      {
        y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
      y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
      y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 49:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
      y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
      y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 50:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
      y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
      y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 51:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 52:
      {
        y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 53:
      {
        y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 54:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    case 55:
      {
        y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
      y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
      y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
      f.evaluate(vals, y, c);
      return vals[0];
        break;
      }
    }
    
    return 0.0;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    // Declare variables for result of evaluation.
    double vals[1];
    
    // Declare variable for physical coordinates.
    double y[3];
    const double * const * x = c.coordinates;
    y[0] = x[0][0];
    y[1] = x[0][1];
    y[2] = x[0][2];
    f.evaluate(vals, y, c);
    values[0] = vals[0];
    y[0] = x[1][0];
    y[1] = x[1][1];
    y[2] = x[1][2];
    f.evaluate(vals, y, c);
    values[1] = vals[0];
    y[0] = x[2][0];
    y[1] = x[2][1];
    y[2] = x[2][2];
    f.evaluate(vals, y, c);
    values[2] = vals[0];
    y[0] = x[3][0];
    y[1] = x[3][1];
    y[2] = x[3][2];
    f.evaluate(vals, y, c);
    values[3] = vals[0];
    y[0] = 0.8*x[2][0] + 0.2*x[3][0];
    y[1] = 0.8*x[2][1] + 0.2*x[3][1];
    y[2] = 0.8*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[4] = vals[0];
    y[0] = 0.6*x[2][0] + 0.4*x[3][0];
    y[1] = 0.6*x[2][1] + 0.4*x[3][1];
    y[2] = 0.6*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[5] = vals[0];
    y[0] = 0.4*x[2][0] + 0.6*x[3][0];
    y[1] = 0.4*x[2][1] + 0.6*x[3][1];
    y[2] = 0.4*x[2][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[6] = vals[0];
    y[0] = 0.2*x[2][0] + 0.8*x[3][0];
    y[1] = 0.2*x[2][1] + 0.8*x[3][1];
    y[2] = 0.2*x[2][2] + 0.8*x[3][2];
    f.evaluate(vals, y, c);
    values[7] = vals[0];
    y[0] = 0.8*x[1][0] + 0.2*x[3][0];
    y[1] = 0.8*x[1][1] + 0.2*x[3][1];
    y[2] = 0.8*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[8] = vals[0];
    y[0] = 0.6*x[1][0] + 0.4*x[3][0];
    y[1] = 0.6*x[1][1] + 0.4*x[3][1];
    y[2] = 0.6*x[1][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[9] = vals[0];
    y[0] = 0.4*x[1][0] + 0.6*x[3][0];
    y[1] = 0.4*x[1][1] + 0.6*x[3][1];
    y[2] = 0.4*x[1][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[10] = vals[0];
    y[0] = 0.2*x[1][0] + 0.8*x[3][0];
    y[1] = 0.2*x[1][1] + 0.8*x[3][1];
    y[2] = 0.2*x[1][2] + 0.8*x[3][2];
    f.evaluate(vals, y, c);
    values[11] = vals[0];
    y[0] = 0.8*x[1][0] + 0.2*x[2][0];
    y[1] = 0.8*x[1][1] + 0.2*x[2][1];
    y[2] = 0.8*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[12] = vals[0];
    y[0] = 0.6*x[1][0] + 0.4*x[2][0];
    y[1] = 0.6*x[1][1] + 0.4*x[2][1];
    y[2] = 0.6*x[1][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[13] = vals[0];
    y[0] = 0.4*x[1][0] + 0.6*x[2][0];
    y[1] = 0.4*x[1][1] + 0.6*x[2][1];
    y[2] = 0.4*x[1][2] + 0.6*x[2][2];
    f.evaluate(vals, y, c);
    values[14] = vals[0];
    y[0] = 0.2*x[1][0] + 0.8*x[2][0];
    y[1] = 0.2*x[1][1] + 0.8*x[2][1];
    y[2] = 0.2*x[1][2] + 0.8*x[2][2];
    f.evaluate(vals, y, c);
    values[15] = vals[0];
    y[0] = 0.8*x[0][0] + 0.2*x[3][0];
    y[1] = 0.8*x[0][1] + 0.2*x[3][1];
    y[2] = 0.8*x[0][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[16] = vals[0];
    y[0] = 0.6*x[0][0] + 0.4*x[3][0];
    y[1] = 0.6*x[0][1] + 0.4*x[3][1];
    y[2] = 0.6*x[0][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[17] = vals[0];
    y[0] = 0.4*x[0][0] + 0.6*x[3][0];
    y[1] = 0.4*x[0][1] + 0.6*x[3][1];
    y[2] = 0.4*x[0][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[18] = vals[0];
    y[0] = 0.2*x[0][0] + 0.8*x[3][0];
    y[1] = 0.2*x[0][1] + 0.8*x[3][1];
    y[2] = 0.2*x[0][2] + 0.8*x[3][2];
    f.evaluate(vals, y, c);
    values[19] = vals[0];
    y[0] = 0.8*x[0][0] + 0.2*x[2][0];
    y[1] = 0.8*x[0][1] + 0.2*x[2][1];
    y[2] = 0.8*x[0][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[20] = vals[0];
    y[0] = 0.6*x[0][0] + 0.4*x[2][0];
    y[1] = 0.6*x[0][1] + 0.4*x[2][1];
    y[2] = 0.6*x[0][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[21] = vals[0];
    y[0] = 0.4*x[0][0] + 0.6*x[2][0];
    y[1] = 0.4*x[0][1] + 0.6*x[2][1];
    y[2] = 0.4*x[0][2] + 0.6*x[2][2];
    f.evaluate(vals, y, c);
    values[22] = vals[0];
    y[0] = 0.2*x[0][0] + 0.8*x[2][0];
    y[1] = 0.2*x[0][1] + 0.8*x[2][1];
    y[2] = 0.2*x[0][2] + 0.8*x[2][2];
    f.evaluate(vals, y, c);
    values[23] = vals[0];
    y[0] = 0.8*x[0][0] + 0.2*x[1][0];
    y[1] = 0.8*x[0][1] + 0.2*x[1][1];
    y[2] = 0.8*x[0][2] + 0.2*x[1][2];
    f.evaluate(vals, y, c);
    values[24] = vals[0];
    y[0] = 0.6*x[0][0] + 0.4*x[1][0];
    y[1] = 0.6*x[0][1] + 0.4*x[1][1];
    y[2] = 0.6*x[0][2] + 0.4*x[1][2];
    f.evaluate(vals, y, c);
    values[25] = vals[0];
    y[0] = 0.4*x[0][0] + 0.6*x[1][0];
    y[1] = 0.4*x[0][1] + 0.6*x[1][1];
    y[2] = 0.4*x[0][2] + 0.6*x[1][2];
    f.evaluate(vals, y, c);
    values[26] = vals[0];
    y[0] = 0.2*x[0][0] + 0.8*x[1][0];
    y[1] = 0.2*x[0][1] + 0.8*x[1][1];
    y[2] = 0.2*x[0][2] + 0.8*x[1][2];
    f.evaluate(vals, y, c);
    values[27] = vals[0];
    y[0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[28] = vals[0];
    y[0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    y[1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    y[2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[29] = vals[0];
    y[0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[30] = vals[0];
    y[0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    y[1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    y[2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[31] = vals[0];
    y[0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    y[1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    y[2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[32] = vals[0];
    y[0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    y[1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    y[2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[33] = vals[0];
    y[0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[34] = vals[0];
    y[0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    y[1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    y[2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[35] = vals[0];
    y[0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[36] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    y[1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    y[2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[37] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    y[1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    y[2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[38] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[39] = vals[0];
    y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[40] = vals[0];
    y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[41] = vals[0];
    y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[42] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[43] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[44] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    f.evaluate(vals, y, c);
    values[45] = vals[0];
    y[0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    y[1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    y[2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[46] = vals[0];
    y[0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    y[1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    y[2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[47] = vals[0];
    y[0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    y[1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    y[2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    f.evaluate(vals, y, c);
    values[48] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[49] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    f.evaluate(vals, y, c);
    values[50] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    f.evaluate(vals, y, c);
    values[51] = vals[0];
    y[0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[52] = vals[0];
    y[0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[53] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    f.evaluate(vals, y, c);
    values[54] = vals[0];
    y[0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    y[1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    y[2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    f.evaluate(vals, y, c);
    values[55] = vals[0];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate function and change variables
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Map coordinate xhat from reference cell to coordinate x in cell
  virtual void map_from_reference_cell(double* x,
                                       const double* xhat,
                                       const ufc::cell& c) const
  {
    throw std::runtime_error("map_from_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Map from coordinate x in cell to coordinate xhat in reference cell
  virtual void map_to_reference_cell(double* xhat,
                                     const double* x,
                                     const ufc::cell& c) const
  {
    throw std::runtime_error("map_to_reference_cell not yet implemented (introduced in UFC 2.0).");
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 0;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::finite_element* create() const
  {
    return new poisson3d_5_finite_element_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class poisson3d_5_dofmap_0: public ufc::dofmap
{
private:

  unsigned int _global_dimension;
public:

  /// Constructor
  poisson3d_5_dofmap_0() : ufc::dofmap()
  {
    _global_dimension = 0;
  }

  /// Destructor
  virtual ~poisson3d_5_dofmap_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dofmap
  virtual const char* signature() const
  {
    return "FFC dofmap for FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return true;
        break;
      }
    case 1:
      {
        return true;
        break;
      }
    case 2:
      {
        return true;
        break;
      }
    case 3:
      {
        return true;
        break;
      }
    }
    
    return false;
  }

  /// Initialize dofmap for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    _global_dimension = m.num_entities[0] + 4.0*m.num_entities[1] + 6.0*m.num_entities[2] + 4.0*m.num_entities[3];
    return false;
  }

  /// Initialize dofmap for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dofmap for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the topological dimension of the associated cell shape
  virtual unsigned int topological_dimension() const
  {
    return 3;
  }

  /// Return the geometric dimension of the associated cell shape
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return _global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 56;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 56;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    switch (d)
    {
    case 0:
      {
        return 1;
        break;
      }
    case 1:
      {
        return 4;
        break;
      }
    case 2:
      {
        return 6;
        break;
      }
    case 3:
      {
        return 4;
        break;
      }
    }
    
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    unsigned int offset = 0;
    dofs[0] = offset + c.entity_indices[0][0];
    dofs[1] = offset + c.entity_indices[0][1];
    dofs[2] = offset + c.entity_indices[0][2];
    dofs[3] = offset + c.entity_indices[0][3];
    offset += m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset += 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset += 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
    offset += 4*m.num_entities[3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch (facet)
    {
    case 0:
      {
        dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
        break;
      }
    case 1:
      {
        dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
        break;
      }
    case 2:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
        break;
      }
    case 3:
      {
        dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
        break;
      }
    }
    
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    if (d > 3)
    {
    throw std::runtime_error("d is larger than dimension (3)");
    }
    
    switch (d)
    {
    case 0:
      {
        if (i > 3)
      {
      throw std::runtime_error("i is larger than number of entities (3)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 0;
          break;
        }
      case 1:
        {
          dofs[0] = 1;
          break;
        }
      case 2:
        {
          dofs[0] = 2;
          break;
        }
      case 3:
        {
          dofs[0] = 3;
          break;
        }
      }
      
        break;
      }
    case 1:
      {
        if (i > 5)
      {
      throw std::runtime_error("i is larger than number of entities (5)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 4;
        dofs[1] = 5;
        dofs[2] = 6;
        dofs[3] = 7;
          break;
        }
      case 1:
        {
          dofs[0] = 8;
        dofs[1] = 9;
        dofs[2] = 10;
        dofs[3] = 11;
          break;
        }
      case 2:
        {
          dofs[0] = 12;
        dofs[1] = 13;
        dofs[2] = 14;
        dofs[3] = 15;
          break;
        }
      case 3:
        {
          dofs[0] = 16;
        dofs[1] = 17;
        dofs[2] = 18;
        dofs[3] = 19;
          break;
        }
      case 4:
        {
          dofs[0] = 20;
        dofs[1] = 21;
        dofs[2] = 22;
        dofs[3] = 23;
          break;
        }
      case 5:
        {
          dofs[0] = 24;
        dofs[1] = 25;
        dofs[2] = 26;
        dofs[3] = 27;
          break;
        }
      }
      
        break;
      }
    case 2:
      {
        if (i > 3)
      {
      throw std::runtime_error("i is larger than number of entities (3)");
      }
      
      switch (i)
      {
      case 0:
        {
          dofs[0] = 28;
        dofs[1] = 29;
        dofs[2] = 30;
        dofs[3] = 31;
        dofs[4] = 32;
        dofs[5] = 33;
          break;
        }
      case 1:
        {
          dofs[0] = 34;
        dofs[1] = 35;
        dofs[2] = 36;
        dofs[3] = 37;
        dofs[4] = 38;
        dofs[5] = 39;
          break;
        }
      case 2:
        {
          dofs[0] = 40;
        dofs[1] = 41;
        dofs[2] = 42;
        dofs[3] = 43;
        dofs[4] = 44;
        dofs[5] = 45;
          break;
        }
      case 3:
        {
          dofs[0] = 46;
        dofs[1] = 47;
        dofs[2] = 48;
        dofs[3] = 49;
        dofs[4] = 50;
        dofs[5] = 51;
          break;
        }
      }
      
        break;
      }
    case 3:
      {
        if (i > 0)
      {
      throw std::runtime_error("i is larger than number of entities (0)");
      }
      
      dofs[0] = 52;
      dofs[1] = 53;
      dofs[2] = 54;
      dofs[3] = 55;
        break;
      }
    }
    
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dofmaps (for a mixed element)
  virtual unsigned int num_sub_dofmaps() const
  {
    return 0;
  }

  /// Create a new dofmap for sub dofmap i (for a mixed element)
  virtual ufc::dofmap* create_sub_dofmap(unsigned int i) const
  {
    return 0;
  }

  /// Create a new class instance
  virtual ufc::dofmap* create() const
  {
    return new poisson3d_5_dofmap_0();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_cell_integral_0_0: public ufc::cell_integral
{
public:

  /// Constructor
  poisson3d_5_cell_integral_0_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_cell_integral_0_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Number of operations (multiply-add pairs) for Jacobian data:      32
    // Number of operations (multiply-add pairs) for geometry tensor:    27
    // Number of operations (multiply-add pairs) for tensor contraction: 18292
    // Total number of operations (multiply-add pairs):                  18351
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    const double K_00 = d_00 / detJ;
    const double K_01 = d_10 / detJ;
    const double K_02 = d_20 / detJ;
    const double K_10 = d_01 / detJ;
    const double K_11 = d_11 / detJ;
    const double K_12 = d_21 / detJ;
    const double K_20 = d_02 / detJ;
    const double K_21 = d_12 / detJ;
    const double K_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensor
    const double G0_0_0 = det*(K_00*K_00 + K_01*K_01 + K_02*K_02);
    const double G0_0_1 = det*(K_00*K_10 + K_01*K_11 + K_02*K_12);
    const double G0_0_2 = det*(K_00*K_20 + K_01*K_21 + K_02*K_22);
    const double G0_1_0 = det*(K_10*K_00 + K_11*K_01 + K_12*K_02);
    const double G0_1_1 = det*(K_10*K_10 + K_11*K_11 + K_12*K_12);
    const double G0_1_2 = det*(K_10*K_20 + K_11*K_21 + K_12*K_22);
    const double G0_2_0 = det*(K_20*K_00 + K_21*K_01 + K_22*K_02);
    const double G0_2_1 = det*(K_20*K_10 + K_21*K_11 + K_22*K_12);
    const double G0_2_2 = det*(K_20*K_20 + K_21*K_21 + K_22*K_22);
    
    // Compute element tensor
    A[0] = 0.0262652567340064*G0_0_0 + 0.0262652567340064*G0_0_1 + 0.0262652567340064*G0_0_2 + 0.0262652567340064*G0_1_0 + 0.0262652567340064*G0_1_1 + 0.0262652567340064*G0_1_2 + 0.0262652567340064*G0_2_0 + 0.0262652567340064*G0_2_1 + 0.0262652567340064*G0_2_2;
    A[1] = -0.00277890512265508*G0_0_0 - 0.00277890512265508*G0_1_0 - 0.00277890512265507*G0_2_0;
    A[2] = -0.0027789051226551*G0_0_1 - 0.00277890512265513*G0_1_1 - 0.0027789051226551*G0_2_1;
    A[3] = -0.00277890512265508*G0_0_2 - 0.00277890512265509*G0_1_2 - 0.00277890512265509*G0_2_2;
    A[4] = -0.0048003493098979*G0_0_1 - 0.00480034930989792*G0_0_2 - 0.00480034930989787*G0_1_1 - 0.00480034930989793*G0_1_2 - 0.00480034930989792*G0_2_1 - 0.00480034930989794*G0_2_2;
    A[5] = -0.00336507226698182*G0_0_1 - 0.00336507226698183*G0_0_2 - 0.00336507226698188*G0_1_1 - 0.00336507226698186*G0_1_2 - 0.00336507226698185*G0_2_1 - 0.00336507226698186*G0_2_2;
    A[6] = -0.00336507226698204*G0_0_1 - 0.00336507226698203*G0_0_2 - 0.00336507226698202*G0_1_1 - 0.00336507226698203*G0_1_2 - 0.00336507226698202*G0_2_1 - 0.00336507226698201*G0_2_2;
    A[7] = -0.00480034930989781*G0_0_1 - 0.00480034930989782*G0_0_2 - 0.00480034930989783*G0_1_1 - 0.00480034930989783*G0_1_2 - 0.00480034930989783*G0_2_1 - 0.00480034930989784*G0_2_2;
    A[8] = -0.00480034930989781*G0_0_0 - 0.00480034930989782*G0_0_2 - 0.0048003493098978*G0_1_0 - 0.00480034930989781*G0_1_2 - 0.00480034930989781*G0_2_0 - 0.00480034930989783*G0_2_2;
    A[9] = -0.00336507226698195*G0_0_0 - 0.00336507226698194*G0_0_2 - 0.00336507226698198*G0_1_0 - 0.00336507226698198*G0_1_2 - 0.00336507226698195*G0_2_0 - 0.00336507226698196*G0_2_2;
    A[10] = -0.00336507226698204*G0_0_0 - 0.00336507226698199*G0_0_2 - 0.00336507226698205*G0_1_0 - 0.003365072266982*G0_1_2 - 0.00336507226698203*G0_2_0 - 0.00336507226698199*G0_2_2;
    A[11] = -0.00480034930989779*G0_0_0 - 0.00480034930989781*G0_0_2 - 0.00480034930989782*G0_1_0 - 0.00480034930989784*G0_1_2 - 0.00480034930989782*G0_2_0 - 0.00480034930989782*G0_2_2;
    A[12] = -0.00480034930989777*G0_0_0 - 0.00480034930989781*G0_0_1 - 0.00480034930989775*G0_1_0 - 0.0048003493098978*G0_1_1 - 0.00480034930989777*G0_2_0 - 0.00480034930989781*G0_2_1;
    A[13] = -0.00336507226698203*G0_0_0 - 0.00336507226698191*G0_0_1 - 0.00336507226698205*G0_1_0 - 0.0033650722669819*G0_1_1 - 0.00336507226698205*G0_2_0 - 0.00336507226698193*G0_2_1;
    A[14] = -0.00336507226698189*G0_0_0 - 0.00336507226698201*G0_0_1 - 0.00336507226698187*G0_1_0 - 0.00336507226698205*G0_1_1 - 0.00336507226698188*G0_2_0 - 0.00336507226698201*G0_2_1;
    A[15] = -0.00480034930989793*G0_0_0 - 0.00480034930989786*G0_0_1 - 0.00480034930989793*G0_1_0 - 0.00480034930989781*G0_1_1 - 0.00480034930989794*G0_2_0 - 0.00480034930989787*G0_2_1;
    A[16] = -0.017870503981615*G0_0_0 - 0.0178705039816149*G0_0_1 - 0.0551276864210886*G0_0_2 - 0.0178705039816149*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0551276864210886*G0_1_2 - 0.0178705039816149*G0_2_0 - 0.0178705039816149*G0_2_1 - 0.0551276864210887*G0_2_2;
    A[17] = 0.0214691349673987*G0_0_0 + 0.0214691349673986*G0_0_1 + 0.0551276864210888*G0_0_2 + 0.0214691349673987*G0_1_0 + 0.0214691349673987*G0_1_1 + 0.0551276864210887*G0_1_2 + 0.0214691349673986*G0_2_0 + 0.0214691349673986*G0_2_1 + 0.0551276864210887*G0_2_2;
    A[18] = -0.0155544887532734*G0_0_0 - 0.0155544887532734*G0_0_1 - 0.0388177200243169*G0_0_2 - 0.0155544887532734*G0_1_0 - 0.0155544887532734*G0_1_1 - 0.0388177200243169*G0_1_2 - 0.0155544887532734*G0_2_0 - 0.0155544887532734*G0_2_1 - 0.0388177200243169*G0_2_2;
    A[19] = 0.00480034930989781*G0_0_0 + 0.00480034930989781*G0_0_1 + 0.0153313684129655*G0_0_2 + 0.00480034930989784*G0_1_0 + 0.00480034930989784*G0_1_1 + 0.0153313684129655*G0_1_2 + 0.00480034930989784*G0_2_0 + 0.00480034930989784*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[20] = -0.0178705039816148*G0_0_0 - 0.0551276864210886*G0_0_1 - 0.0178705039816148*G0_0_2 - 0.0178705039816148*G0_1_0 - 0.0551276864210886*G0_1_1 - 0.0178705039816148*G0_1_2 - 0.0178705039816148*G0_2_0 - 0.0551276864210886*G0_2_1 - 0.0178705039816148*G0_2_2;
    A[21] = 0.0214691349673985*G0_0_0 + 0.0551276864210885*G0_0_1 + 0.0214691349673985*G0_0_2 + 0.0214691349673985*G0_1_0 + 0.0551276864210885*G0_1_1 + 0.0214691349673985*G0_1_2 + 0.0214691349673985*G0_2_0 + 0.0551276864210885*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[22] = -0.0155544887532733*G0_0_0 - 0.0388177200243167*G0_0_1 - 0.0155544887532733*G0_0_2 - 0.0155544887532733*G0_1_0 - 0.0388177200243168*G0_1_1 - 0.0155544887532733*G0_1_2 - 0.0155544887532733*G0_2_0 - 0.0388177200243167*G0_2_1 - 0.0155544887532733*G0_2_2;
    A[23] = 0.0048003493098979*G0_0_0 + 0.0153313684129655*G0_0_1 + 0.0048003493098979*G0_0_2 + 0.00480034930989791*G0_1_0 + 0.0153313684129655*G0_1_1 + 0.00480034930989791*G0_1_2 + 0.00480034930989792*G0_2_0 + 0.0153313684129655*G0_2_1 + 0.00480034930989792*G0_2_2;
    A[24] = -0.0551276864210885*G0_0_0 - 0.0178705039816148*G0_0_1 - 0.0178705039816148*G0_0_2 - 0.0551276864210885*G0_1_0 - 0.0178705039816148*G0_1_1 - 0.0178705039816148*G0_1_2 - 0.0551276864210885*G0_2_0 - 0.0178705039816148*G0_2_1 - 0.0178705039816148*G0_2_2;
    A[25] = 0.0551276864210883*G0_0_0 + 0.0214691349673985*G0_0_1 + 0.0214691349673985*G0_0_2 + 0.0551276864210883*G0_1_0 + 0.0214691349673984*G0_1_1 + 0.0214691349673984*G0_1_2 + 0.0551276864210884*G0_2_0 + 0.0214691349673985*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[26] = -0.0388177200243166*G0_0_0 - 0.0155544887532732*G0_0_1 - 0.0155544887532732*G0_0_2 - 0.0388177200243166*G0_1_0 - 0.0155544887532731*G0_1_1 - 0.0155544887532731*G0_1_2 - 0.0388177200243166*G0_2_0 - 0.0155544887532732*G0_2_1 - 0.0155544887532732*G0_2_2;
    A[27] = 0.0153313684129654*G0_0_0 + 0.0048003493098978*G0_0_1 + 0.00480034930989779*G0_0_2 + 0.0153313684129654*G0_1_0 + 0.00480034930989779*G0_1_1 + 0.00480034930989778*G0_1_2 + 0.0153313684129654*G0_2_0 + 0.0048003493098978*G0_2_1 + 0.0048003493098978*G0_2_2;
    A[28] = -0.0137421252872639*G0_0_0 - 0.0137421252872638*G0_0_1 - 0.0137421252872637*G0_0_2 - 0.013742125287264*G0_1_0 - 0.0137421252872639*G0_1_1 - 0.0137421252872637*G0_1_2 - 0.013742125287264*G0_2_0 - 0.0137421252872639*G0_2_1 - 0.0137421252872638*G0_2_2;
    A[29] = -0.00946499969937446*G0_0_0 - 0.0094649996993745*G0_0_1 - 0.00946499969937465*G0_0_2 - 0.00946499969937442*G0_1_0 - 0.0094649996993744*G0_1_1 - 0.00946499969937465*G0_1_2 - 0.0094649996993744*G0_2_0 - 0.00946499969937447*G0_2_1 - 0.00946499969937463*G0_2_2;
    A[30] = -0.0137421252872639*G0_0_0 - 0.0137421252872638*G0_0_1 - 0.0137421252872639*G0_0_2 - 0.013742125287264*G0_1_0 - 0.013742125287264*G0_1_1 - 0.013742125287264*G0_1_2 - 0.013742125287264*G0_2_0 - 0.0137421252872639*G0_2_1 - 0.013742125287264*G0_2_2;
    A[31] = -0.00946499969937472*G0_0_0 - 0.00946499969937471*G0_0_1 - 0.00946499969937471*G0_0_2 - 0.00946499969937471*G0_1_0 - 0.00946499969937476*G0_1_1 - 0.0094649996993747*G0_1_2 - 0.00946499969937467*G0_2_0 - 0.00946499969937468*G0_2_1 - 0.00946499969937467*G0_2_2;
    A[32] = -0.00946499969937456*G0_0_0 - 0.00946499969937464*G0_0_1 - 0.00946499969937439*G0_0_2 - 0.00946499969937462*G0_1_0 - 0.00946499969937461*G0_1_1 - 0.00946499969937443*G0_1_2 - 0.00946499969937457*G0_2_0 - 0.00946499969937462*G0_2_1 - 0.00946499969937441*G0_2_2;
    A[33] = -0.013742125287264*G0_0_0 - 0.013742125287264*G0_0_1 - 0.0137421252872642*G0_0_2 - 0.0137421252872641*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.0137421252872642*G0_1_2 - 0.0137421252872641*G0_2_0 - 0.0137421252872641*G0_2_1 - 0.0137421252872642*G0_2_2;
    A[34] = 0.0479930547204848*G0_0_0 + 0.03359853077735*G0_0_1 + 0.0335985307773497*G0_0_2 + 0.0479930547204848*G0_1_0 + 0.0335985307773498*G0_1_1 + 0.0335985307773497*G0_1_2 + 0.0479930547204848*G0_2_0 + 0.0335985307773499*G0_2_1 + 0.0335985307773497*G0_2_2;
    A[35] = -0.0345719095719092*G0_0_0 - 0.0298472385060924*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0345719095719092*G0_1_0 - 0.0298472385060922*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.0345719095719093*G0_2_0 - 0.0298472385060924*G0_2_1 - 0.0298472385060921*G0_2_2;
    A[36] = 0.0137421252872639*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0189195610202552*G0_0_2 + 0.013742125287264*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202552*G0_1_2 + 0.013742125287264*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[37] = -0.0345719095719095*G0_0_0 - 0.0298472385060924*G0_0_1 - 0.0298472385060925*G0_0_2 - 0.0345719095719096*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0345719095719095*G0_2_0 - 0.0298472385060924*G0_2_1 - 0.0298472385060925*G0_2_2;
    A[38] = 0.00946499969937457*G0_0_0 + 0.0117431758056757*G0_0_1 + 0.0117431758056758*G0_0_2 + 0.00946499969937464*G0_1_0 + 0.0117431758056757*G0_1_1 + 0.0117431758056757*G0_1_2 + 0.00946499969937459*G0_2_0 + 0.0117431758056757*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[39] = 0.013742125287264*G0_0_0 + 0.0189195610202555*G0_0_1 + 0.0189195610202554*G0_0_2 + 0.0137421252872641*G0_1_0 + 0.0189195610202555*G0_1_1 + 0.0189195610202554*G0_1_2 + 0.0137421252872641*G0_2_0 + 0.0189195610202555*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[40] = 0.0335985307773499*G0_0_0 + 0.0479930547204848*G0_0_1 + 0.0335985307773497*G0_0_2 + 0.0335985307773499*G0_1_0 + 0.0479930547204849*G0_1_1 + 0.0335985307773498*G0_1_2 + 0.0335985307773499*G0_2_0 + 0.0479930547204849*G0_2_1 + 0.0335985307773498*G0_2_2;
    A[41] = -0.0298472385060923*G0_0_0 - 0.034571909571909*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.034571909571909*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.0298472385060924*G0_2_0 - 0.034571909571909*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[42] = 0.0189195610202551*G0_0_0 + 0.0137421252872638*G0_0_1 + 0.0189195610202551*G0_0_2 + 0.018919561020255*G0_1_0 + 0.0137421252872638*G0_1_1 + 0.0189195610202551*G0_1_2 + 0.0189195610202551*G0_2_0 + 0.0137421252872638*G0_2_1 + 0.0189195610202551*G0_2_2;
    A[43] = -0.0298472385060925*G0_0_0 - 0.0345719095719096*G0_0_1 - 0.0298472385060924*G0_0_2 - 0.0298472385060925*G0_1_0 - 0.0345719095719097*G0_1_1 - 0.0298472385060926*G0_1_2 - 0.0298472385060925*G0_2_0 - 0.0345719095719096*G0_2_1 - 0.0298472385060924*G0_2_2;
    A[44] = 0.0117431758056758*G0_0_0 + 0.00946499969937476*G0_0_1 + 0.0117431758056757*G0_0_2 + 0.0117431758056758*G0_1_0 + 0.00946499969937481*G0_1_1 + 0.0117431758056757*G0_1_2 + 0.0117431758056758*G0_2_0 + 0.00946499969937473*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[45] = 0.0189195610202554*G0_0_0 + 0.013742125287264*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0189195610202555*G0_1_0 + 0.0137421252872641*G0_1_1 + 0.0189195610202554*G0_1_2 + 0.0189195610202554*G0_2_0 + 0.013742125287264*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[46] = 0.0335985307773498*G0_0_0 + 0.0335985307773498*G0_0_1 + 0.0479930547204848*G0_0_2 + 0.0335985307773497*G0_1_0 + 0.0335985307773497*G0_1_1 + 0.0479930547204848*G0_1_2 + 0.0335985307773498*G0_2_0 + 0.0335985307773498*G0_2_1 + 0.0479930547204849*G0_2_2;
    A[47] = -0.0298472385060923*G0_0_0 - 0.0298472385060922*G0_0_1 - 0.034571909571909*G0_0_2 - 0.0298472385060923*G0_1_0 - 0.0298472385060921*G0_1_1 - 0.034571909571909*G0_1_2 - 0.0298472385060924*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.034571909571909*G0_2_2;
    A[48] = 0.0189195610202551*G0_0_0 + 0.0189195610202551*G0_0_1 + 0.0137421252872638*G0_0_2 + 0.0189195610202551*G0_1_0 + 0.0189195610202551*G0_1_1 + 0.0137421252872638*G0_1_2 + 0.0189195610202552*G0_2_0 + 0.0189195610202551*G0_2_1 + 0.0137421252872639*G0_2_2;
    A[49] = -0.0298472385060922*G0_0_0 - 0.0298472385060924*G0_0_1 - 0.0345719095719092*G0_0_2 - 0.0298472385060922*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0345719095719093*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0345719095719093*G0_2_2;
    A[50] = 0.0117431758056758*G0_0_0 + 0.0117431758056758*G0_0_1 + 0.00946499969937469*G0_0_2 + 0.0117431758056758*G0_1_0 + 0.0117431758056758*G0_1_1 + 0.00946499969937469*G0_1_2 + 0.0117431758056758*G0_2_0 + 0.0117431758056758*G0_2_1 + 0.00946499969937467*G0_2_2;
    A[51] = 0.0189195610202553*G0_0_0 + 0.0189195610202553*G0_0_1 + 0.0137421252872639*G0_0_2 + 0.0189195610202552*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.013742125287264*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202553*G0_2_1 + 0.013742125287264*G0_2_2;
    A[52] = -0.0782878387045048*G0_0_0 - 0.0782878387045048*G0_0_1 - 0.078287838704505*G0_0_2 - 0.0782878387045048*G0_1_0 - 0.0782878387045051*G0_1_1 - 0.0782878387045051*G0_1_2 - 0.0782878387045048*G0_2_0 - 0.078287838704505*G0_2_1 - 0.0782878387045051*G0_2_2;
    A[53] = 0.044036909271284*G0_0_0 + 0.0440369092712836*G0_0_1 + 0.0440369092712837*G0_0_2 + 0.044036909271284*G0_1_0 + 0.0440369092712835*G0_1_1 + 0.0440369092712836*G0_1_2 + 0.044036909271284*G0_2_0 + 0.0440369092712836*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[54] = 0.0440369092712837*G0_0_0 + 0.0440369092712838*G0_0_1 + 0.0440369092712836*G0_0_2 + 0.0440369092712836*G0_1_0 + 0.044036909271284*G0_1_1 + 0.0440369092712837*G0_1_2 + 0.0440369092712837*G0_2_0 + 0.0440369092712839*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[55] = 0.0440369092712844*G0_0_0 + 0.0440369092712843*G0_0_1 + 0.0440369092712845*G0_0_2 + 0.0440369092712844*G0_1_0 + 0.0440369092712843*G0_1_1 + 0.0440369092712845*G0_1_2 + 0.0440369092712843*G0_2_0 + 0.0440369092712843*G0_2_1 + 0.0440369092712845*G0_2_2;
    A[56] = -0.00277890512265508*G0_0_0 - 0.00277890512265508*G0_0_1 - 0.00277890512265507*G0_0_2;
    A[57] = 0.0262652567340065*G0_0_0;
    A[58] = 0.00277890512265508*G0_0_1;
    A[59] = 0.00277890512265509*G0_0_2;
    A[60] = 0.00480034930989807*G0_0_1 + 0.00480034930989797*G0_0_2;
    A[61] = 0.00336507226698193*G0_0_1 + 0.00336507226698201*G0_0_2;
    A[62] = 0.00336507226698204*G0_0_1 + 0.00336507226698201*G0_0_2;
    A[63] = 0.00480034930989785*G0_0_1 + 0.00480034930989788*G0_0_2;
    A[64] = -0.0178705039816151*G0_0_0 + 0.0372571824394735*G0_0_2;
    A[65] = 0.0214691349673987*G0_0_0 - 0.03365855145369*G0_0_2;
    A[66] = -0.0155544887532733*G0_0_0 + 0.0232632312710436*G0_0_2;
    A[67] = 0.00480034930989784*G0_0_0 - 0.0105310191030676*G0_0_2;
    A[68] = -0.0178705039816151*G0_0_0 + 0.0372571824394736*G0_0_1;
    A[69] = 0.0214691349673989*G0_0_0 - 0.0336585514536899*G0_0_1;
    A[70] = -0.0155544887532734*G0_0_0 + 0.0232632312710435*G0_0_1;
    A[71] = 0.00480034930989798*G0_0_0 - 0.0105310191030675*G0_0_1;
    A[72] = -0.00480034930989795*G0_0_0 - 0.00480034930989794*G0_0_1;
    A[73] = -0.00336507226698182*G0_0_0 - 0.00336507226698181*G0_0_1;
    A[74] = -0.00336507226698206*G0_0_0 - 0.00336507226698207*G0_0_1;
    A[75] = -0.00480034930989787*G0_0_0 - 0.00480034930989786*G0_0_1;
    A[76] = -0.00480034930989792*G0_0_0 - 0.00480034930989793*G0_0_2;
    A[77] = -0.00336507226698202*G0_0_0 - 0.00336507226698199*G0_0_2;
    A[78] = -0.00336507226698187*G0_0_0 - 0.00336507226698187*G0_0_2;
    A[79] = -0.00480034930989797*G0_0_0 - 0.00480034930989798*G0_0_2;
    A[80] = 0.0153313684129654*G0_0_0 + 0.0105310191030674*G0_0_1 + 0.0105310191030674*G0_0_2;
    A[81] = -0.0388177200243166*G0_0_0 - 0.0232632312710432*G0_0_1 - 0.0232632312710432*G0_0_2;
    A[82] = 0.0551276864210883*G0_0_0 + 0.0336585514536896*G0_0_1 + 0.0336585514536896*G0_0_2;
    A[83] = -0.0551276864210885*G0_0_0 - 0.0372571824394735*G0_0_1 - 0.0372571824394735*G0_0_2;
    A[84] = 0.047993054720485*G0_0_0 + 0.0143945239431348*G0_0_1 + 0.0143945239431349*G0_0_2;
    A[85] = -0.0345719095719094*G0_0_0 - 0.00472467106581684*G0_0_1 - 0.00472467106581675*G0_0_2;
    A[86] = 0.0137421252872644*G0_0_0 - 0.00517743573299117*G0_0_1 - 0.00517743573299103*G0_0_2;
    A[87] = -0.0345719095719095*G0_0_0 - 0.00472467106581687*G0_0_1 - 0.00472467106581703*G0_0_2;
    A[88] = 0.00946499969937473*G0_0_0 - 0.00227817610630104*G0_0_1 - 0.0022781761063011*G0_0_2;
    A[89] = 0.0137421252872642*G0_0_0 - 0.00517743573299118*G0_0_1 - 0.00517743573299108*G0_0_2;
    A[90] = -0.0137421252872642*G0_0_0;
    A[91] = -0.00946499969937442*G0_0_0;
    A[92] = -0.0137421252872644*G0_0_0;
    A[93] = -0.0094649996993748*G0_0_0;
    A[94] = -0.00946499969937472*G0_0_0;
    A[95] = -0.0137421252872642*G0_0_0;
    A[96] = 0.0189195610202553*G0_0_0 + 0.00517743573299118*G0_0_1;
    A[97] = -0.0298472385060925*G0_0_0 + 0.00472467106581659*G0_0_1;
    A[98] = 0.0335985307773501*G0_0_0 - 0.0143945239431346*G0_0_1;
    A[99] = 0.0117431758056756*G0_0_0 + 0.0022781761063011*G0_0_1;
    A[100] = -0.0298472385060925*G0_0_0 + 0.00472467106581688*G0_0_1;
    A[101] = 0.0189195610202554*G0_0_0 + 0.0051774357329912*G0_0_1;
    A[102] = 0.0189195610202553*G0_0_0 + 0.00517743573299106*G0_0_2;
    A[103] = -0.0298472385060925*G0_0_0 + 0.00472467106581651*G0_0_2;
    A[104] = 0.0335985307773502*G0_0_0 - 0.0143945239431347*G0_0_2;
    A[105] = 0.0117431758056758*G0_0_0 + 0.00227817610630136*G0_0_2;
    A[106] = -0.0298472385060926*G0_0_0 + 0.0047246710658167*G0_0_2;
    A[107] = 0.0189195610202552*G0_0_0 + 0.00517743573299106*G0_0_2;
    A[108] = 0.0440369092712838*G0_0_0;
    A[109] = -0.0782878387045046*G0_0_0;
    A[110] = 0.0440369092712838*G0_0_0;
    A[111] = 0.0440369092712841*G0_0_0;
    A[112] = -0.0027789051226551*G0_1_0 - 0.00277890512265513*G0_1_1 - 0.0027789051226551*G0_1_2;
    A[113] = 0.00277890512265508*G0_1_0;
    A[114] = 0.0262652567340066*G0_1_1;
    A[115] = 0.00277890512265509*G0_1_2;
    A[116] = -0.0178705039816151*G0_1_1 + 0.0372571824394738*G0_1_2;
    A[117] = 0.0214691349673988*G0_1_1 - 0.0336585514536901*G0_1_2;
    A[118] = -0.0155544887532734*G0_1_1 + 0.0232632312710436*G0_1_2;
    A[119] = 0.00480034930989786*G0_1_1 - 0.0105310191030676*G0_1_2;
    A[120] = 0.00480034930989785*G0_1_0 + 0.0048003493098978*G0_1_2;
    A[121] = 0.003365072266982*G0_1_0 + 0.00336507226698202*G0_1_2;
    A[122] = 0.00336507226698194*G0_1_0 + 0.00336507226698194*G0_1_2;
    A[123] = 0.00480034930989786*G0_1_0 + 0.00480034930989786*G0_1_2;
    A[124] = -0.0105310191030677*G0_1_0 + 0.0048003493098978*G0_1_1;
    A[125] = 0.0232632312710435*G0_1_0 - 0.0155544887532734*G0_1_1;
    A[126] = -0.03365855145369*G0_1_0 + 0.0214691349673988*G0_1_1;
    A[127] = 0.0372571824394738*G0_1_0 - 0.0178705039816151*G0_1_1;
    A[128] = -0.00480034930989796*G0_1_0 - 0.00480034930989797*G0_1_1;
    A[129] = -0.00336507226698187*G0_1_0 - 0.00336507226698184*G0_1_1;
    A[130] = -0.00336507226698201*G0_1_0 - 0.00336507226698204*G0_1_1;
    A[131] = -0.00480034930989786*G0_1_0 - 0.00480034930989786*G0_1_1;
    A[132] = 0.0105310191030676*G0_1_0 + 0.0153313684129656*G0_1_1 + 0.0105310191030676*G0_1_2;
    A[133] = -0.0232632312710436*G0_1_0 - 0.0388177200243171*G0_1_1 - 0.0232632312710436*G0_1_2;
    A[134] = 0.03365855145369*G0_1_0 + 0.055127686421089*G0_1_1 + 0.03365855145369*G0_1_2;
    A[135] = -0.0372571824394738*G0_1_0 - 0.0551276864210889*G0_1_1 - 0.0372571824394738*G0_1_2;
    A[136] = -0.00480034930989796*G0_1_1 - 0.00480034930989796*G0_1_2;
    A[137] = -0.00336507226698186*G0_1_1 - 0.00336507226698187*G0_1_2;
    A[138] = -0.00336507226698212*G0_1_1 - 0.00336507226698211*G0_1_2;
    A[139] = -0.00480034930989781*G0_1_1 - 0.00480034930989781*G0_1_2;
    A[140] = -0.0051774357329913*G0_1_0 + 0.0137421252872641*G0_1_1 - 0.00517743573299135*G0_1_2;
    A[141] = -0.00472467106581686*G0_1_0 - 0.0345719095719095*G0_1_1 - 0.00472467106581683*G0_1_2;
    A[142] = 0.0143945239431351*G0_1_0 + 0.0479930547204852*G0_1_1 + 0.014394523943135*G0_1_2;
    A[143] = -0.00227817610630123*G0_1_0 + 0.00946499969937454*G0_1_1 - 0.00227817610630115*G0_1_2;
    A[144] = -0.0047246710658168*G0_1_0 - 0.0345719095719093*G0_1_1 - 0.00472467106581681*G0_1_2;
    A[145] = -0.00517743573299123*G0_1_0 + 0.0137421252872641*G0_1_1 - 0.00517743573299131*G0_1_2;
    A[146] = 0.00517743573299135*G0_1_0 + 0.0189195610202555*G0_1_1;
    A[147] = 0.00472467106581701*G0_1_0 - 0.0298472385060925*G0_1_1;
    A[148] = -0.0143945239431351*G0_1_0 + 0.03359853077735*G0_1_1;
    A[149] = 0.00227817610630118*G0_1_0 + 0.0117431758056756*G0_1_1;
    A[150] = 0.00472467106581679*G0_1_0 - 0.0298472385060926*G0_1_1;
    A[151] = 0.00517743573299124*G0_1_0 + 0.0189195610202554*G0_1_1;
    A[152] = -0.0137421252872641*G0_1_1;
    A[153] = -0.00946499969937485*G0_1_1;
    A[154] = -0.0137421252872641*G0_1_1;
    A[155] = -0.00946499969937471*G0_1_1;
    A[156] = -0.00946499969937462*G0_1_1;
    A[157] = -0.013742125287264*G0_1_1;
    A[158] = 0.0189195610202555*G0_1_1 + 0.00517743573299134*G0_1_2;
    A[159] = 0.0117431758056758*G0_1_1 + 0.00227817610630099*G0_1_2;
    A[160] = 0.0189195610202555*G0_1_1 + 0.00517743573299138*G0_1_2;
    A[161] = -0.0298472385060926*G0_1_1 + 0.00472467106581693*G0_1_2;
    A[162] = -0.0298472385060926*G0_1_1 + 0.00472467106581683*G0_1_2;
    A[163] = 0.0335985307773502*G0_1_1 - 0.0143945239431349*G0_1_2;
    A[164] = 0.0440369092712843*G0_1_1;
    A[165] = 0.0440369092712845*G0_1_1;
    A[166] = -0.0782878387045055*G0_1_1;
    A[167] = 0.044036909271284*G0_1_1;
    A[168] = -0.00277890512265508*G0_2_0 - 0.00277890512265509*G0_2_1 - 0.00277890512265509*G0_2_2;
    A[169] = 0.00277890512265509*G0_2_0;
    A[170] = 0.00277890512265509*G0_2_1;
    A[171] = 0.0262652567340067*G0_2_2;
    A[172] = -0.0105310191030677*G0_2_1 + 0.0048003493098978*G0_2_2;
    A[173] = 0.0232632312710437*G0_2_1 - 0.0155544887532732*G0_2_2;
    A[174] = -0.0336585514536902*G0_2_1 + 0.0214691349673986*G0_2_2;
    A[175] = 0.037257182439474*G0_2_1 - 0.017870503981615*G0_2_2;
    A[176] = -0.0105310191030677*G0_2_0 + 0.00480034930989777*G0_2_2;
    A[177] = 0.0232632312710438*G0_2_0 - 0.0155544887532731*G0_2_2;
    A[178] = -0.0336585514536903*G0_2_0 + 0.0214691349673986*G0_2_2;
    A[179] = 0.0372571824394741*G0_2_0 - 0.0178705039816149*G0_2_2;
    A[180] = 0.00480034930989779*G0_2_0 + 0.00480034930989777*G0_2_1;
    A[181] = 0.00336507226698199*G0_2_0 + 0.00336507226698196*G0_2_1;
    A[182] = 0.003365072266982*G0_2_0 + 0.00336507226698203*G0_2_1;
    A[183] = 0.00480034930989782*G0_2_0 + 0.0048003493098978*G0_2_1;
    A[184] = 0.0105310191030676*G0_2_0 + 0.0105310191030676*G0_2_1 + 0.0153313684129656*G0_2_2;
    A[185] = -0.0232632312710436*G0_2_0 - 0.0232632312710436*G0_2_1 - 0.0388177200243171*G0_2_2;
    A[186] = 0.0336585514536903*G0_2_0 + 0.0336585514536902*G0_2_1 + 0.0551276864210891*G0_2_2;
    A[187] = -0.0372571824394741*G0_2_0 - 0.0372571824394741*G0_2_1 - 0.0551276864210891*G0_2_2;
    A[188] = -0.00480034930989794*G0_2_0 - 0.00480034930989796*G0_2_2;
    A[189] = -0.00336507226698192*G0_2_0 - 0.00336507226698191*G0_2_2;
    A[190] = -0.00336507226698198*G0_2_0 - 0.00336507226698198*G0_2_2;
    A[191] = -0.00480034930989782*G0_2_0 - 0.00480034930989782*G0_2_2;
    A[192] = -0.00480034930989798*G0_2_1 - 0.00480034930989798*G0_2_2;
    A[193] = -0.00336507226698192*G0_2_1 - 0.00336507226698193*G0_2_2;
    A[194] = -0.00336507226698202*G0_2_1 - 0.00336507226698199*G0_2_2;
    A[195] = -0.00480034930989777*G0_2_1 - 0.00480034930989777*G0_2_2;
    A[196] = -0.0051774357329912*G0_2_0 - 0.00517743573299121*G0_2_1 + 0.0137421252872639*G0_2_2;
    A[197] = -0.00227817610630123*G0_2_0 - 0.0022781761063013*G0_2_1 + 0.00946499969937472*G0_2_2;
    A[198] = -0.00517743573299122*G0_2_0 - 0.00517743573299119*G0_2_1 + 0.013742125287264*G0_2_2;
    A[199] = -0.00472467106581705*G0_2_0 - 0.00472467106581688*G0_2_1 - 0.0345719095719093*G0_2_2;
    A[200] = -0.00472467106581691*G0_2_0 - 0.00472467106581699*G0_2_1 - 0.0345719095719095*G0_2_2;
    A[201] = 0.0143945239431353*G0_2_0 + 0.0143945239431353*G0_2_1 + 0.0479930547204851*G0_2_2;
    A[202] = 0.00517743573299149*G0_2_0 + 0.0189195610202556*G0_2_2;
    A[203] = 0.00227817610630118*G0_2_0 + 0.0117431758056758*G0_2_2;
    A[204] = 0.00517743573299122*G0_2_0 + 0.0189195610202552*G0_2_2;
    A[205] = 0.00472467106581666*G0_2_0 - 0.0298472385060929*G0_2_2;
    A[206] = 0.00472467106581691*G0_2_0 - 0.0298472385060925*G0_2_2;
    A[207] = -0.0143945239431353*G0_2_0 + 0.0335985307773502*G0_2_2;
    A[208] = 0.00517743573299142*G0_2_1 + 0.0189195610202555*G0_2_2;
    A[209] = 0.00227817610630107*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[210] = 0.00517743573299126*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[211] = 0.00472467106581676*G0_2_1 - 0.0298472385060928*G0_2_2;
    A[212] = 0.00472467106581678*G0_2_1 - 0.0298472385060924*G0_2_2;
    A[213] = -0.0143945239431352*G0_2_1 + 0.03359853077735*G0_2_2;
    A[214] = -0.0137421252872639*G0_2_2;
    A[215] = -0.00946499969937489*G0_2_2;
    A[216] = -0.0137421252872638*G0_2_2;
    A[217] = -0.0094649996993748*G0_2_2;
    A[218] = -0.00946499969937473*G0_2_2;
    A[219] = -0.013742125287264*G0_2_2;
    A[220] = 0.044036909271284*G0_2_2;
    A[221] = 0.0440369092712842*G0_2_2;
    A[222] = 0.0440369092712842*G0_2_2;
    A[223] = -0.0782878387045051*G0_2_2;
    A[224] = -0.0048003493098979*G0_1_0 - 0.00480034930989787*G0_1_1 - 0.00480034930989792*G0_1_2 - 0.00480034930989792*G0_2_0 - 0.00480034930989793*G0_2_1 - 0.00480034930989794*G0_2_2;
    A[225] = 0.00480034930989807*G0_1_0 + 0.00480034930989797*G0_2_0;
    A[226] = -0.0178705039816151*G0_1_1 + 0.0372571824394738*G0_2_1;
    A[227] = -0.0105310191030677*G0_1_2 + 0.0048003493098978*G0_2_2;
    A[228] = 0.179096479009673*G0_1_1 + 0.069911039963123*G0_1_2 + 0.069911039963123*G0_2_1 + 0.139822079926246*G0_2_2;
    A[229] = -0.147063705006413*G0_1_1 + 0.026383001643418*G0_1_2 - 0.0387263842124951*G0_2_1 - 0.108637424175618*G0_2_2;
    A[230] = 0.100717304493346*G0_1_1 - 0.0246345732456842*G0_1_2 + 0.0189782768992838*G0_2_1 + 0.0788945694544646*G0_2_2;
    A[231] = -0.0328352243499812*G0_1_1 + 0.0209159009072203*G0_1_2 - 0.00537576492437605*G0_2_1 - 0.0328352243499809*G0_2_2;
    A[232] = -0.010751529848752*G0_1_0 - 0.00537576492437568*G0_1_2 - 0.00537576492437605*G0_2_0 - 0.00537576492437594*G0_2_2;
    A[233] = -0.0139743792087542*G0_1_0 - 0.0116844599266478*G0_1_2 - 0.00228991928210659*G0_2_0 - 0.00228991928210665*G0_2_2;
    A[234] = -0.011169064988509*G0_1_0 - 0.00887914570640214*G0_1_2 - 0.00228991928210665*G0_2_0 - 0.00228991928210681*G0_2_2;
    A[235] = -0.0328352243499813*G0_1_0 - 0.0274594594256052*G0_1_2 - 0.00537576492437607*G0_2_0 - 0.00537576492437583*G0_2_2;
    A[236] = -0.0274594594256051*G0_1_0 - 0.00537576492437576*G0_1_1 - 0.0328352243499812*G0_2_0 - 0.005375764924376*G0_2_1;
    A[237] = 0.0599162925551811*G0_1_0 + 0.0283728175438242*G0_1_1 + 0.0788945694544648*G0_2_0 + 0.0189782768992837*G0_2_1;
    A[238] = -0.0699110399631228*G0_1_0 - 0.0574176057031955*G0_1_1 - 0.108637424175618*G0_2_0 - 0.0387263842124949*G0_2_1;
    A[239] = 0.069911039963123*G0_1_0 + 0.0895482395048363*G0_1_1 + 0.139822079926246*G0_2_0 + 0.0699110399631229*G0_2_1;
    A[240] = 0.0107515298487522*G0_1_0 + 0.0107515298487522*G0_1_1 + 0.00537576492437623*G0_1_2 + 0.00537576492437618*G0_2_0 + 0.0053757649243762*G0_2_1;
    A[241] = 0.0139743792087537*G0_1_0 + 0.0139743792087537*G0_1_1 + 0.00228991928210648*G0_1_2 + 0.00228991928210654*G0_2_0 + 0.00228991928210661*G0_2_1;
    A[242] = 0.0111690649885095*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.00228991928210679*G0_1_2 + 0.00228991928210686*G0_2_0 + 0.00228991928210684*G0_2_1;
    A[243] = 0.0328352243499812*G0_1_0 + 0.0328352243499812*G0_1_1 + 0.00537576492437618*G0_1_2 + 0.00537576492437603*G0_2_0 + 0.00537576492437603*G0_2_1;
    A[244] = 0.0274594594256051*G0_1_0 + 0.022083694501229*G0_1_1 + 0.0274594594256051*G0_1_2 + 0.0328352243499811*G0_2_0 + 0.0274594594256052*G0_2_1 + 0.0328352243499812*G0_2_2;
    A[245] = -0.0599162925551811*G0_1_0 - 0.0315434750113565*G0_1_1 - 0.059916292555181*G0_1_2 - 0.0788945694544647*G0_2_0 - 0.059916292555181*G0_2_1 - 0.0788945694544646*G0_2_2;
    A[246] = 0.069911039963123*G0_1_0 + 0.0124934342599269*G0_1_1 + 0.0699110399631229*G0_1_2 + 0.108637424175618*G0_2_0 + 0.0699110399631227*G0_2_1 + 0.108637424175618*G0_2_2;
    A[247] = -0.069911039963123*G0_1_0 + 0.0196371995417135*G0_1_1 - 0.069911039963123*G0_1_2 - 0.139822079926246*G0_2_0 - 0.0699110399631227*G0_2_1 - 0.139822079926246*G0_2_2;
    A[248] = 0.00537576492437613*G0_1_1 + 0.00537576492437615*G0_1_2 + 0.00537576492437603*G0_2_1 + 0.00537576492437605*G0_2_2;
    A[249] = 0.00228991928210681*G0_1_1 + 0.00228991928210676*G0_1_2 + 0.00228991928210675*G0_2_1 + 0.00228991928210671*G0_2_2;
    A[250] = 0.00228991928210701*G0_1_1 + 0.00228991928210704*G0_1_2 + 0.00228991928210697*G0_2_1 + 0.00228991928210703*G0_2_2;
    A[251] = 0.00537576492437556*G0_1_1 + 0.00537576492437568*G0_1_2 + 0.00537576492437579*G0_2_1 + 0.00537576492437591*G0_2_2;
    A[252] = 0.0425363923627813*G0_1_0 - 0.0187890812890811*G0_1_1 + 0.0212681961813906*G0_1_2 + 0.0118736555368505*G0_2_0 - 0.0093945406445403*G0_2_1 + 0.0118736555368497*G0_2_2;
    A[253] = -0.0845508658008646*G0_1_0 + 0.0439716694056975*G0_1_1 - 0.0422754329004319*G0_1_2 - 0.0429604514890968*G0_2_0 - 0.000685018588664028*G0_2_1 - 0.0429604514890962*G0_2_2;
    A[254] = 0.192588083213082*G0_1_0 - 0.0644569872000421*G0_1_1 + 0.0962940416065415*G0_1_2 + 0.124738623002512*G0_2_0 + 0.0284445813959708*G0_2_1 + 0.124738623002512*G0_2_2;
    A[255] = 0.0142875305635721*G0_1_0 - 0.0292600797158083*G0_1_1 + 0.00844856259352822*G0_1_2 + 0.00254435475789596*G0_2_0 - 0.00329461321214781*G0_2_1 + 0.00254435475789701*G0_2_2;
    A[256] = -0.0493213383838389*G0_1_0 + 0.1273155976932*G0_1_1 - 0.00704590548340657*G0_1_2 - 0.0429604514890976*G0_2_0 - 0.000685018588665316*G0_2_1 - 0.0429604514890979*G0_2_2;
    A[257] = 0.047755581609748*G0_1_0 - 0.0641307878721069*G0_1_1 + 0.0264873854283575*G0_1_2 + 0.0118736555368501*G0_2_0 - 0.00939454064454036*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[258] = -0.0425363923627812*G0_1_0 - 0.0613254736518625*G0_1_1 - 0.0212681961813911*G0_1_2 - 0.0118736555368502*G0_2_0 - 0.0212681961813909*G0_2_1;
    A[259] = 0.0845508658008653*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0429604514890974*G0_2_0 + 0.042275432900433*G0_2_1;
    A[260] = -0.192588083213082*G0_1_0 - 0.257045070413124*G0_1_1 - 0.096294041606541*G0_1_2 - 0.124738623002512*G0_2_0 - 0.0962940416065413*G0_2_1;
    A[261] = -0.0142875305635717*G0_1_0 - 0.04354761027938*G0_1_1 - 0.0058389679700434*G0_1_2 - 0.00254435475789591*G0_2_0 - 0.00583896797004392*G0_2_1;
    A[262] = 0.049321338383839*G0_1_0 + 0.176636936077039*G0_1_1 + 0.0422754329004327*G0_1_2 + 0.0429604514890976*G0_2_0 + 0.0422754329004325*G0_2_1;
    A[263] = -0.047755581609748*G0_1_0 - 0.111886369481855*G0_1_1 - 0.021268196181391*G0_1_2 - 0.0118736555368501*G0_2_0 - 0.0212681961813907*G0_2_1;
    A[264] = 0.0187890812890813*G0_1_1 + 0.00939454064454032*G0_1_2 + 0.00939454064454039*G0_2_1;
    A[265] = 0.00658922642429577*G0_1_1 + 0.00329461321214757*G0_1_2 + 0.00329461321214803*G0_2_1;
    A[266] = 0.0187890812890814*G0_1_1 + 0.00939454064454071*G0_1_2 + 0.00939454064454052*G0_2_1;
    A[267] = 0.0292600797158091*G0_1_1 + 0.00329461321214845*G0_1_2 + 0.00329461321214811*G0_2_1;
    A[268] = 0.0292600797158085*G0_1_1 + 0.00329461321214787*G0_1_2 + 0.00329461321214774*G0_2_1;
    A[269] = 0.0641307878721067*G0_1_1 + 0.0093945406445403*G0_1_2 + 0.0093945406445404*G0_2_1;
    A[270] = -0.0306627368259311*G0_1_1 - 0.0212681961813901*G0_1_2 - 0.0212681961813903*G0_2_1 - 0.0118736555368497*G0_2_2;
    A[271] = -0.00913358118219296*G0_1_1 - 0.0058389679700445*G0_1_2 - 0.00583896797004467*G0_2_1 - 0.00254435475789638*G0_2_2;
    A[272] = -0.0306627368259309*G0_1_1 - 0.0212681961813907*G0_1_2 - 0.0212681961813903*G0_2_1 - 0.0118736555368498*G0_2_2;
    A[273] = 0.0642612676032811*G0_1_1 + 0.0422754329004324*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[274] = 0.0642612676032814*G0_1_1 + 0.042275432900432*G0_1_2 + 0.0422754329004325*G0_2_1 + 0.0429604514890964*G0_2_2;
    A[275] = -0.128522535206562*G0_1_1 - 0.0962940416065411*G0_1_2 - 0.0962940416065407*G0_2_1 - 0.124738623002511*G0_2_2;
    A[276] = -0.0505608958299929*G0_1_1 - 0.00260959462348388*G0_1_2 - 0.00260959462348314*G0_2_1;
    A[277] = -0.0505608958299939*G0_1_1 - 0.00260959462348376*G0_1_2 - 0.00260959462348417*G0_2_1;
    A[278] = 0.0962288017409538*G0_1_1 - 0.0352295274170264*G0_1_2 - 0.035229527417028*G0_2_1;
    A[279] = -0.156575677409009*G0_1_1 - 0.00260959462348332*G0_1_2 - 0.00260959462348273*G0_2_1;
    A[280] = -0.00336507226698182*G0_1_0 - 0.00336507226698188*G0_1_1 - 0.00336507226698185*G0_1_2 - 0.00336507226698182*G0_2_0 - 0.00336507226698186*G0_2_1 - 0.00336507226698185*G0_2_2;
    A[281] = 0.00336507226698192*G0_1_0 + 0.00336507226698201*G0_2_0;
    A[282] = 0.0214691349673988*G0_1_1 - 0.0336585514536901*G0_2_1;
    A[283] = 0.0232632312710437*G0_1_2 - 0.0155544887532732*G0_2_2;
    A[284] = -0.147063705006413*G0_1_1 - 0.0387263842124951*G0_1_2 + 0.026383001643418*G0_2_1 - 0.108637424175618*G0_2_2;
    A[285] = 0.304226541205706*G0_1_1 + 0.0781312630270956*G0_1_2 + 0.0781312630270956*G0_2_1 + 0.281001149056703*G0_2_2;
    A[286] = -0.204187731314466*G0_1_1 + 0.0153052724667312*G0_1_2 - 0.0364951808094164*G0_2_1 - 0.204187731314466*G0_2_2;
    A[287] = 0.0788945694544648*G0_1_1 - 0.0246345732456845*G0_1_2 + 0.0189782768992835*G0_2_1 + 0.100717304493345*G0_2_2;
    A[288] = -0.0139743792087541*G0_1_0 - 0.00228991928210694*G0_1_2 - 0.0116844599266472*G0_2_0 - 0.00228991928210651*G0_2_2;
    A[289] = 0.026304713804714*G0_1_0 + 0.0131523569023573*G0_1_2 + 0.0131523569023566*G0_2_0 + 0.0132175967679432*G0_2_2;
    A[290] = 0.00523223722008385*G0_1_0 + 0.00578025209101535*G0_1_2 - 0.000548014870931543*G0_2_0 - 0.000482775005343854*G0_2_2;
    A[291] = 0.0788945694544649*G0_1_0 + 0.0599162925551811*G0_1_2 + 0.0189782768992835*G0_2_0 + 0.0283728175438236*G0_2_2;
    A[292] = -0.00887914570640269*G0_1_0 - 0.00228991928210686*G0_1_1 - 0.0111690649885091*G0_2_0 - 0.00228991928210647*G0_2_1;
    A[293] = 0.00578025209101596*G0_1_0 - 0.000482775005344491*G0_1_1 + 0.00523223722008439*G0_2_0 - 0.000548014870931489*G0_2_1;
    A[294] = 0.00480165410720979*G0_1_0 + 0.00506261356955813*G0_1_1 + 0.00960330821441928*G0_2_0 + 0.00480165410720959*G0_2_1;
    A[295] = -0.038726384212495*G0_1_0 - 0.0574176057031957*G0_1_1 - 0.108637424175618*G0_2_0 - 0.0699110399631228*G0_2_1;
    A[296] = 0.0139743792087538*G0_1_0 + 0.0139743792087538*G0_1_1 + 0.011684459926647*G0_1_2 + 0.0116844599266471*G0_2_0 + 0.0116844599266471*G0_2_1 + 0.00939454064454029*G0_2_2;
    A[297] = -0.0263047138047132*G0_1_0 - 0.0263047138047133*G0_1_1 - 0.0131523569023564*G0_1_2 - 0.0131523569023567*G0_2_0 - 0.0131523569023568*G0_2_1 + 6.52398655871915e-05*G0_2_2;
    A[298] = -0.00523223722008448*G0_1_0 - 0.00523223722008458*G0_1_1 + 0.000548014870931412*G0_1_2 + 0.000548014870931503*G0_2_0 + 0.000548014870931439*G0_2_1 + 6.52398655873046e-05*G0_2_2;
    A[299] = -0.0788945694544649*G0_1_0 - 0.0788945694544648*G0_1_1 - 0.0189782768992838*G0_1_2 - 0.0189782768992835*G0_2_0 - 0.0189782768992835*G0_2_1 + 0.00939454064454037*G0_2_2;
    A[300] = 0.00887914570640248*G0_1_0 + 0.00658922642429581*G0_1_1 + 0.00887914570640247*G0_1_2 + 0.0111690649885092*G0_2_0 + 0.00887914570640239*G0_2_1 + 0.0111690649885092*G0_2_2;
    A[301] = -0.00578025209101576*G0_1_0 - 0.00626302709636052*G0_1_1 - 0.00578025209101574*G0_1_2 - 0.00523223722008431*G0_2_0 - 0.00578025209101577*G0_2_1 - 0.00523223722008428*G0_2_2;
    A[302] = -0.00480165410720976*G0_1_0 + 0.0002609594623486*G0_1_1 - 0.00480165410720972*G0_1_2 - 0.00960330821441937*G0_2_0 - 0.00480165410720959*G0_2_1 - 0.00960330821441927*G0_2_2;
    A[303] = 0.038726384212495*G0_1_0 - 0.0186912214907008*G0_1_1 + 0.038726384212495*G0_1_2 + 0.108637424175618*G0_2_0 + 0.0387263842124949*G0_2_1 + 0.108637424175618*G0_2_2;
    A[304] = 0.00228991928210682*G0_1_1 + 0.00228991928210677*G0_1_2 + 0.00228991928210688*G0_2_1 + 0.00228991928210685*G0_2_2;
    A[305] = 0.00178757231708601*G0_1_1 + 0.00178757231708602*G0_1_2 + 0.00178757231708593*G0_2_1 + 0.00178757231708592*G0_2_2;
    A[306] = 0.00178757231708623*G0_1_1 + 0.0017875723170862*G0_1_2 + 0.00178757231708625*G0_2_1 + 0.00178757231708617*G0_2_2;
    A[307] = 0.00228991928210709*G0_1_1 + 0.00228991928210697*G0_1_2 + 0.00228991928210669*G0_2_1 + 0.00228991928210657*G0_2_2;
    A[308] = 0.0403182369328201*G0_1_0 + 0.0148746893538562*G0_1_1 + 0.00123955744615472*G0_1_2 + 0.0540186087061077*G0_2_0 + 0.0149399292194426*G0_2_1 + 0.00117431758056832*G0_2_2;
    A[309] = -0.0808321934624014*G0_1_0 - 0.0436454700777618*G0_1_1 - 0.00649136662591556*G0_1_2 - 0.108232937008977*G0_2_0 - 0.0338921101724917*G0_2_1 - 0.00352295274170329*G0_2_2;
    A[310] = 0.0109602974186295*G0_1_0 + 0.132306447410613*G0_1_1 - 0.0623693115012559*G0_1_2 + 0.146659217839771*G0_2_0 + 0.0733296089198853*G0_2_1 - 0.124738623002511*G0_2_2;
    A[311] = -0.0377738821749246*G0_1_0 + 0.0189848008858417*G0_1_1 - 0.0188869410874625*G0_1_2 - 0.00939454064454045*G0_2_0 + 0.00949240044292087*G0_2_1 - 0.0155923278753151*G0_2_2;
    A[312] = 0.167731694424402*G0_1_0 - 0.166948816037356*G0_1_1 + 0.0838658472122009*G0_1_2 + 0.0777006799142216*G0_2_0 - 0.00616516729797884*G0_2_1 + 0.182410664181497*G0_2_2;
    A[313] = -0.0823327103709038*G0_1_0 + 0.127087258163646*G0_1_1 - 0.0463855444324188*G0_1_2 + 0.00182671623643836*G0_2_0 + 0.0377738821749232*G0_2_1 - 0.0510175748891016*G0_2_2;
    A[314] = -0.0403182369328193*G0_1_0 - 0.0254435475789634*G0_1_1 - 0.0390786794866646*G0_1_2 - 0.0540186087061077*G0_2_0 - 0.0390786794866647*G0_2_1 - 0.0528442911255399*G0_2_2;
    A[315] = 0.0808321934624005*G0_1_0 + 0.0371867233846391*G0_1_1 + 0.0743408268364853*G0_1_2 + 0.108232937008977*G0_2_0 + 0.0743408268364848*G0_2_1 + 0.104709984267274*G0_2_2;
    A[316] = -0.0109602974186296*G0_1_0 + 0.121346149991983*G0_1_1 - 0.0733296089198853*G0_1_2 - 0.146659217839771*G0_2_0 - 0.0733296089198852*G0_2_1 - 0.271397840842282*G0_2_2;
    A[317] = 0.0377738821749236*G0_1_0 + 0.0567586830607655*G0_1_1 + 0.0188869410874611*G0_1_2 + 0.00939454064454081*G0_2_0 + 0.0188869410874621*G0_2_1 - 0.00619778723077384*G0_2_2;
    A[318] = -0.167731694424402*G0_1_0 - 0.334680510461758*G0_1_1 - 0.0838658472122012*G0_1_2 - 0.0777006799142215*G0_2_0 - 0.0838658472122009*G0_2_1 + 0.104709984267275*G0_2_2;
    A[319] = 0.0823327103709037*G0_1_0 + 0.209419968534551*G0_1_1 + 0.0359471659384855*G0_1_2 - 0.00182671623643845*G0_2_0 + 0.0359471659384849*G0_2_1 - 0.0528442911255403*G0_2_2;
    A[320] = -0.014874689353856*G0_1_1 - 0.014939929219443*G0_1_2 - 0.014939929219443*G0_2_1 - 0.0150051690850301*G0_2_2;
    A[321] = 0.00450155072550929*G0_1_1 + 0.00192457603481932*G0_1_2 + 0.00192457603481902*G0_2_1 - 0.000652398655870568*G0_2_2;
    A[322] = -0.0148746893538565*G0_1_1 - 0.0149399292194435*G0_1_2 - 0.0149399292194429*G0_2_1 - 0.0150051690850292*G0_2_2;
    A[323] = -0.0189848008858426*G0_1_1 - 0.00949240044292136*G0_1_2 - 0.00949240044292084*G0_2_1 - 0.000652398655870775*G0_2_2;
    A[324] = -0.0189848008858419*G0_1_1 - 0.00949240044292078*G0_1_2 - 0.00949240044292078*G0_2_1 - 0.000652398655871673*G0_2_2;
    A[325] = -0.127087258163646*G0_1_1 - 0.0377738821749235*G0_1_2 - 0.0377738821749231*G0_2_1 - 0.0150051690850295*G0_2_2;
    A[326] = -0.00130479731174177*G0_1_1 - 0.00123955744615499*G0_1_2 - 0.00123955744615488*G0_2_1 - 0.00117431758056797*G0_2_2;
    A[327] = -0.0156575677409005*G0_1_1 - 0.0130805930502108*G0_1_2 - 0.0130805930502107*G0_2_1 - 0.010503618359521*G0_2_2;
    A[328] = -0.00130479731174202*G0_1_1 - 0.00123955744615444*G0_1_2 - 0.0012395574461551*G0_2_1 - 0.00117431758056805*G0_2_2;
    A[329] = 0.00880738185425669*G0_1_1 + 0.0064913666259154*G0_1_2 + 0.0064913666259152*G0_2_1 + 0.00352295274170294*G0_2_2;
    A[330] = 0.00880738185425658*G0_1_1 + 0.00649136662591516*G0_1_2 + 0.00649136662591545*G0_2_1 + 0.00352295274170282*G0_2_2;
    A[331] = 0.0476251018785738*G0_1_1 + 0.0623693115012558*G0_1_2 + 0.0623693115012554*G0_2_1 + 0.124738623002511*G0_2_2;
    A[332] = 0.0391439193522529*G0_1_1 + 0.0319675341376737*G0_1_2 + 0.031967534137673*G0_2_1 + 0.026095946234836*G0_2_2;
    A[333] = 0.0391439193522537*G0_1_1 + 0.0319675341376735*G0_1_2 + 0.0319675341376733*G0_2_1 + 0.0260959462348361*G0_2_2;
    A[334] = -0.15657567740901*G0_1_1 - 0.0903572138381161*G0_1_2 - 0.0903572138381159*G0_2_1 - 0.1859336169232*G0_2_2;
    A[335] = 0.185933616923199*G0_1_1 + 0.0156575677408999*G0_1_2 + 0.0156575677408998*G0_2_1 + 0.0260959462348333*G0_2_2;
    A[336] = -0.00336507226698204*G0_1_0 - 0.00336507226698202*G0_1_1 - 0.00336507226698202*G0_1_2 - 0.00336507226698203*G0_2_0 - 0.00336507226698203*G0_2_1 - 0.00336507226698201*G0_2_2;
    A[337] = 0.00336507226698204*G0_1_0 + 0.00336507226698201*G0_2_0;
    A[338] = -0.0155544887532734*G0_1_1 + 0.0232632312710436*G0_2_1;
    A[339] = -0.0336585514536902*G0_1_2 + 0.0214691349673986*G0_2_2;
    A[340] = 0.100717304493346*G0_1_1 + 0.0189782768992838*G0_1_2 - 0.0246345732456842*G0_2_1 + 0.0788945694544646*G0_2_2;
    A[341] = -0.204187731314466*G0_1_1 - 0.0364951808094164*G0_1_2 + 0.0153052724667312*G0_2_1 - 0.204187731314466*G0_2_2;
    A[342] = 0.281001149056702*G0_1_1 + 0.078131263027095*G0_1_2 + 0.078131263027095*G0_2_1 + 0.304226541205705*G0_2_2;
    A[343] = -0.108637424175618*G0_1_1 + 0.0263830016434184*G0_1_2 - 0.0387263842124944*G0_2_1 - 0.147063705006412*G0_2_2;
    A[344] = -0.0111690649885095*G0_1_0 - 0.00228991928210672*G0_1_2 - 0.00887914570640296*G0_2_0 - 0.00228991928210713*G0_2_2;
    A[345] = 0.00523223722008423*G0_1_0 - 0.000548014870931928*G0_1_2 + 0.00578025209101669*G0_2_0 - 0.000482775005343424*G0_2_2;
    A[346] = 0.00960330821441964*G0_1_0 + 0.00480165410721007*G0_1_2 + 0.00480165410720938*G0_2_0 + 0.00506261356955721*G0_2_2;
    A[347] = -0.108637424175618*G0_1_0 - 0.0699110399631228*G0_1_2 - 0.0387263842124944*G0_2_0 - 0.0574176057031946*G0_2_2;
    A[348] = -0.0116844599266472*G0_1_0 - 0.00228991928210671*G0_1_1 - 0.0139743792087545*G0_2_0 - 0.00228991928210711*G0_2_1;
    A[349] = 0.0131523569023567*G0_1_0 + 0.0132175967679442*G0_1_1 + 0.0263047138047135*G0_2_0 + 0.0131523569023569*G0_2_1;
    A[350] = -0.000548014870931581*G0_1_0 - 0.000482775005344993*G0_1_1 + 0.00523223722008457*G0_2_0 + 0.00578025209101561*G0_2_1;
    A[351] = 0.0189782768992839*G0_1_0 + 0.0283728175438248*G0_1_1 + 0.0788945694544646*G0_2_0 + 0.0599162925551813*G0_2_1;
    A[352] = 0.0111690649885095*G0_1_0 + 0.0111690649885096*G0_1_1 + 0.00887914570640284*G0_1_2 + 0.00887914570640276*G0_2_0 + 0.00887914570640279*G0_2_1 + 0.00658922642429614*G0_2_2;
    A[353] = -0.00523223722008456*G0_1_0 - 0.00523223722008458*G0_1_1 - 0.00578025209101628*G0_1_2 - 0.00578025209101621*G0_2_0 - 0.00578025209101619*G0_2_1 - 0.00626302709636065*G0_2_2;
    A[354] = -0.00960330821441936*G0_1_0 - 0.00960330821441927*G0_1_1 - 0.00480165410720973*G0_1_2 - 0.00480165410720947*G0_2_0 - 0.00480165410720941*G0_2_1 + 0.000260959462348131*G0_2_2;
    A[355] = 0.108637424175618*G0_1_0 + 0.108637424175618*G0_1_1 + 0.0387263842124953*G0_1_2 + 0.0387263842124945*G0_2_0 + 0.0387263842124945*G0_2_1 - 0.0186912214907003*G0_2_2;
    A[356] = 0.0116844599266473*G0_1_0 + 0.00939454064454052*G0_1_1 + 0.0116844599266473*G0_1_2 + 0.0139743792087541*G0_2_0 + 0.0116844599266474*G0_2_1 + 0.013974379208754*G0_2_2;
    A[357] = -0.0131523569023568*G0_1_0 + 6.52398655874705e-05*G0_1_1 - 0.0131523569023568*G0_1_2 - 0.0263047138047133*G0_2_0 - 0.0131523569023567*G0_2_1 - 0.0263047138047133*G0_2_2;
    A[358] = 0.000548014870931887*G0_1_0 + 6.52398655866622e-05*G0_1_1 + 0.000548014870931821*G0_1_2 - 0.00523223722008412*G0_2_0 + 0.000548014870931332*G0_2_1 - 0.00523223722008427*G0_2_2;
    A[359] = -0.0189782768992839*G0_1_0 + 0.00939454064454077*G0_1_1 - 0.0189782768992839*G0_1_2 - 0.0788945694544647*G0_2_0 - 0.0189782768992837*G0_2_1 - 0.0788945694544647*G0_2_2;
    A[360] = 0.00228991928210664*G0_1_1 + 0.00228991928210667*G0_1_2 + 0.00228991928210656*G0_2_1 + 0.0022899192821066*G0_2_2;
    A[361] = 0.00178757231708636*G0_1_1 + 0.00178757231708635*G0_1_2 + 0.00178757231708648*G0_2_1 + 0.00178757231708649*G0_2_2;
    A[362] = 0.00178757231708615*G0_1_1 + 0.00178757231708614*G0_1_2 + 0.00178757231708616*G0_2_1 + 0.0017875723170862*G0_2_2;
    A[363] = 0.00228991928210653*G0_1_1 + 0.00228991928210663*G0_1_2 + 0.00228991928210691*G0_2_1 + 0.00228991928210703*G0_2_2;
    A[364] = 0.0540186087061084*G0_1_0 + 0.00117431758056718*G0_1_1 + 0.0149399292194432*G0_1_2 + 0.0403182369328212*G0_2_0 + 0.00123955744615521*G0_2_1 + 0.0148746893538554*G0_2_2;
    A[365] = -0.00939454064454048*G0_1_0 - 0.0155923278753133*G0_1_1 + 0.00949240044292103*G0_1_2 - 0.0377738821749242*G0_2_0 - 0.0188869410874619*G0_2_1 + 0.0189848008858425*G0_2_2;
    A[366] = 0.0018267162364395*G0_1_0 - 0.0510175748891017*G0_1_1 + 0.0377738821749245*G0_1_2 - 0.0823327103709031*G0_2_0 - 0.0463855444324181*G0_2_1 + 0.127087258163647*G0_2_2;
    A[367] = -0.108232937008977*G0_1_0 - 0.00352295274170204*G0_1_1 - 0.0338921101724914*G0_1_2 - 0.0808321934624018*G0_2_0 - 0.00649136662591469*G0_2_1 - 0.0436454700777603*G0_2_2;
    A[368] = 0.0777006799142201*G0_1_0 + 0.182410664181495*G0_1_1 - 0.00616516729798022*G0_1_2 + 0.1677316944244*G0_2_0 + 0.0838658472121986*G0_2_1 - 0.166948816037358*G0_2_2;
    A[369] = 0.146659217839771*G0_1_0 - 0.124738623002511*G0_1_1 + 0.0733296089198855*G0_1_2 + 0.0109602974186308*G0_2_0 - 0.0623693115012548*G0_2_1 + 0.132306447410612*G0_2_2;
    A[370] = -0.0540186087061088*G0_1_0 - 0.0528442911255414*G0_1_1 - 0.0390786794866656*G0_1_2 - 0.0403182369328203*G0_2_0 - 0.0390786794866656*G0_2_1 - 0.0254435475789647*G0_2_2;
    A[371] = 0.00939454064454097*G0_1_0 - 0.0061977872307729*G0_1_1 + 0.0188869410874614*G0_1_2 + 0.037773882174924*G0_2_0 + 0.0188869410874622*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[372] = -0.00182671623643959*G0_1_0 - 0.052844291125541*G0_1_1 + 0.0359471659384845*G0_1_2 + 0.0823327103709031*G0_2_0 + 0.0359471659384848*G0_2_1 + 0.209419968534549*G0_2_2;
    A[373] = 0.108232937008978*G0_1_0 + 0.104709984267276*G0_1_1 + 0.0743408268364871*G0_1_2 + 0.0808321934624012*G0_2_0 + 0.074340826836486*G0_2_1 + 0.0371867233846409*G0_2_2;
    A[374] = -0.07770067991422*G0_1_0 + 0.104709984267275*G0_1_1 - 0.0838658472121998*G0_1_2 - 0.1677316944244*G0_2_0 - 0.0838658472122009*G0_2_1 - 0.334680510461757*G0_2_2;
    A[375] = -0.146659217839771*G0_1_0 - 0.271397840842283*G0_1_1 - 0.073329608919887*G0_1_2 - 0.0109602974186309*G0_2_0 - 0.0733296089198857*G0_2_1 + 0.121346149991981*G0_2_2;
    A[376] = -0.00117431758056757*G0_1_1 - 0.00123955744615459*G0_1_2 - 0.00123955744615492*G0_2_1 - 0.00130479731174191*G0_2_2;
    A[377] = -0.0105036183595214*G0_1_1 - 0.0130805930502113*G0_1_2 - 0.0130805930502108*G0_2_1 - 0.0156575677409012*G0_2_2;
    A[378] = -0.0011743175805671*G0_1_1 - 0.0012395574461541*G0_1_2 - 0.00123955744615509*G0_2_1 - 0.00130479731174274*G0_2_2;
    A[379] = 0.00352295274170325*G0_1_1 + 0.00649136662591539*G0_1_2 + 0.00649136662591539*G0_2_1 + 0.00880738185425707*G0_2_2;
    A[380] = 0.00352295274170238*G0_1_1 + 0.00649136662591477*G0_1_2 + 0.00649136662591474*G0_2_1 + 0.00880738185425749*G0_2_2;
    A[381] = 0.124738623002511*G0_1_1 + 0.0623693115012554*G0_1_2 + 0.0623693115012547*G0_2_1 + 0.0476251018785728*G0_2_2;
    A[382] = -0.0150051690850302*G0_1_1 - 0.0149399292194431*G0_1_2 - 0.0149399292194426*G0_2_1 - 0.0148746893538555*G0_2_2;
    A[383] = -0.000652398655871321*G0_1_1 + 0.00192457603481867*G0_1_2 + 0.00192457603481833*G0_2_1 + 0.00450155072550852*G0_2_2;
    A[384] = -0.01500516908503*G0_1_1 - 0.0149399292194433*G0_1_2 - 0.0149399292194427*G0_2_1 - 0.0148746893538555*G0_2_2;
    A[385] = -0.000652398655870435*G0_1_1 - 0.00949240044292082*G0_1_2 - 0.00949240044292134*G0_2_1 - 0.0189848008858424*G0_2_2;
    A[386] = -0.0006523986558704*G0_1_1 - 0.00949240044292097*G0_1_2 - 0.00949240044292089*G0_2_1 - 0.0189848008858423*G0_2_2;
    A[387] = -0.0150051690850306*G0_1_1 - 0.0377738821749243*G0_1_2 - 0.0377738821749237*G0_2_1 - 0.127087258163647*G0_2_2;
    A[388] = 0.0260959462348355*G0_1_1 + 0.0319675341376736*G0_1_2 + 0.0319675341376736*G0_2_1 + 0.0391439193522526*G0_2_2;
    A[389] = 0.026095946234834*G0_1_1 + 0.0319675341376725*G0_1_2 + 0.0319675341376724*G0_2_1 + 0.0391439193522517*G0_2_2;
    A[390] = 0.0260959462348341*G0_1_1 + 0.0156575677409005*G0_1_2 + 0.0156575677408997*G0_2_1 + 0.1859336169232*G0_2_2;
    A[391] = -0.185933616923198*G0_1_1 - 0.0903572138381157*G0_1_2 - 0.0903572138381142*G0_2_1 - 0.156575677409009*G0_2_2;
    A[392] = -0.00480034930989781*G0_1_0 - 0.00480034930989783*G0_1_1 - 0.00480034930989783*G0_1_2 - 0.00480034930989782*G0_2_0 - 0.00480034930989783*G0_2_1 - 0.00480034930989784*G0_2_2;
    A[393] = 0.00480034930989785*G0_1_0 + 0.00480034930989788*G0_2_0;
    A[394] = 0.00480034930989786*G0_1_1 - 0.0105310191030676*G0_2_1;
    A[395] = 0.037257182439474*G0_1_2 - 0.017870503981615*G0_2_2;
    A[396] = -0.0328352243499812*G0_1_1 - 0.00537576492437605*G0_1_2 + 0.0209159009072203*G0_2_1 - 0.0328352243499809*G0_2_2;
    A[397] = 0.0788945694544648*G0_1_1 + 0.0189782768992835*G0_1_2 - 0.0246345732456845*G0_2_1 + 0.100717304493345*G0_2_2;
    A[398] = -0.108637424175618*G0_1_1 - 0.0387263842124944*G0_1_2 + 0.0263830016434184*G0_2_1 - 0.147063705006412*G0_2_2;
    A[399] = 0.139822079926246*G0_1_1 + 0.0699110399631223*G0_1_2 + 0.0699110399631223*G0_2_1 + 0.179096479009672*G0_2_2;
    A[400] = -0.0328352243499809*G0_1_0 - 0.00537576492437599*G0_1_2 - 0.0274594594256048*G0_2_0 - 0.00537576492437578*G0_2_2;
    A[401] = 0.0788945694544649*G0_1_0 + 0.0189782768992839*G0_1_2 + 0.0599162925551806*G0_2_0 + 0.0283728175438238*G0_2_2;
    A[402] = -0.108637424175618*G0_1_0 - 0.0387263842124951*G0_1_2 - 0.0699110399631224*G0_2_0 - 0.0574176057031952*G0_2_2;
    A[403] = 0.139822079926246*G0_1_0 + 0.0699110399631228*G0_1_2 + 0.0699110399631224*G0_2_0 + 0.0895482395048356*G0_2_2;
    A[404] = -0.00537576492437606*G0_1_0 - 0.00537576492437602*G0_1_1 - 0.0107515298487519*G0_2_0 - 0.00537576492437581*G0_2_1;
    A[405] = -0.00228991928210666*G0_1_0 - 0.00228991928210686*G0_1_1 - 0.0139743792087539*G0_2_0 - 0.0116844599266474*G0_2_1;
    A[406] = -0.00228991928210669*G0_1_0 - 0.00228991928210654*G0_1_1 - 0.0111690649885095*G0_2_0 - 0.00887914570640255*G0_2_1;
    A[407] = -0.00537576492437605*G0_1_0 - 0.00537576492437617*G0_1_1 - 0.0328352243499809*G0_2_0 - 0.027459459425605*G0_2_1;
    A[408] = 0.0328352243499809*G0_1_0 + 0.032835224349981*G0_1_1 + 0.0274594594256049*G0_1_2 + 0.027459459425605*G0_2_0 + 0.027459459425605*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[409] = -0.0788945694544645*G0_1_0 - 0.0788945694544647*G0_1_1 - 0.059916292555181*G0_1_2 - 0.0599162925551807*G0_2_0 - 0.0599162925551809*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[410] = 0.108637424175618*G0_1_0 + 0.108637424175618*G0_1_1 + 0.0699110399631232*G0_1_2 + 0.0699110399631226*G0_2_0 + 0.0699110399631225*G0_2_1 + 0.0124934342599273*G0_2_2;
    A[411] = -0.139822079926246*G0_1_0 - 0.139822079926246*G0_1_1 - 0.0699110399631234*G0_1_2 - 0.0699110399631225*G0_2_0 - 0.0699110399631225*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[412] = 0.00537576492437599*G0_1_0 + 0.00537576492437594*G0_1_2 + 0.0107515298487521*G0_2_0 + 0.00537576492437594*G0_2_1 + 0.010751529848752*G0_2_2;
    A[413] = 0.00228991928210678*G0_1_0 + 0.00228991928210679*G0_1_2 + 0.0139743792087539*G0_2_0 + 0.00228991928210681*G0_2_1 + 0.0139743792087539*G0_2_2;
    A[414] = 0.00228991928210654*G0_1_0 + 0.00228991928210656*G0_1_2 + 0.0111690649885092*G0_2_0 + 0.00228991928210671*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[415] = 0.00537576492437609*G0_1_0 + 0.00537576492437611*G0_1_2 + 0.0328352243499809*G0_2_0 + 0.00537576492437604*G0_2_1 + 0.0328352243499809*G0_2_2;
    A[416] = 0.00537576492437603*G0_1_1 + 0.00537576492437597*G0_1_2 + 0.0053757649243762*G0_2_1 + 0.00537576492437614*G0_2_2;
    A[417] = 0.00228991928210666*G0_1_1 + 0.0022899192821067*G0_1_2 + 0.00228991928210653*G0_2_1 + 0.00228991928210656*G0_2_2;
    A[418] = 0.00228991928210687*G0_1_1 + 0.00228991928210687*G0_1_2 + 0.00228991928210698*G0_2_1 + 0.00228991928210694*G0_2_2;
    A[419] = 0.00537576492437613*G0_1_1 + 0.00537576492437604*G0_1_2 + 0.00537576492437594*G0_2_1 + 0.00537576492437584*G0_2_2;
    A[420] = 0.0118736555368502*G0_1_0 + 0.0118736555368503*G0_1_1 - 0.00939454064454058*G0_1_2 + 0.0425363923627806*G0_2_0 + 0.0212681961813903*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[421] = 0.00254435475789621*G0_1_0 + 0.00254435475789622*G0_1_1 - 0.00329461321214765*G0_1_2 + 0.0142875305635723*G0_2_0 + 0.00844856259352822*G0_2_1 - 0.0292600797158087*G0_2_2;
    A[422] = 0.0118736555368493*G0_1_0 + 0.0118736555368495*G0_1_1 - 0.00939454064454097*G0_1_2 + 0.0477555816097475*G0_2_0 + 0.026487385428357*G0_2_1 - 0.0641307878721069*G0_2_2;
    A[423] = -0.0429604514890969*G0_1_0 - 0.0429604514890968*G0_1_1 - 0.000685018588664223*G0_1_2 - 0.0845508658008643*G0_2_0 - 0.0422754329004322*G0_2_1 + 0.0439716694056971*G0_2_2;
    A[424] = -0.0429604514890968*G0_1_0 - 0.0429604514890969*G0_1_1 - 0.000685018588664756*G0_1_2 - 0.0493213383838377*G0_2_0 - 0.00704590548340474*G0_2_1 + 0.127315597693201*G0_2_2;
    A[425] = 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0284445813959707*G0_1_2 + 0.192588083213081*G0_2_0 + 0.0962940416065405*G0_2_1 - 0.0644569872000421*G0_2_2;
    A[426] = -0.0118736555368496*G0_1_0 - 0.02126819618139*G0_1_2 - 0.0425363923627812*G0_2_0 - 0.0212681961813906*G0_2_1 - 0.0613254736518618*G0_2_2;
    A[427] = -0.00254435475789623*G0_1_0 - 0.00583896797004381*G0_1_2 - 0.014287530563572*G0_2_0 - 0.00583896797004407*G0_2_1 - 0.0435476102793804*G0_2_2;
    A[428] = -0.0118736555368494*G0_1_0 - 0.0212681961813902*G0_1_2 - 0.0477555816097476*G0_2_0 - 0.0212681961813906*G0_2_1 - 0.111886369481854*G0_2_2;
    A[429] = 0.0429604514890963*G0_1_0 + 0.0422754329004314*G0_1_2 + 0.0845508658008651*G0_2_0 + 0.0422754329004328*G0_2_1 + 0.128522535206561*G0_2_2;
    A[430] = 0.0429604514890968*G0_1_0 + 0.0422754329004318*G0_1_2 + 0.0493213383838377*G0_2_0 + 0.0422754329004323*G0_2_1 + 0.176636936077038*G0_2_2;
    A[431] = -0.124738623002511*G0_1_0 - 0.0962940416065399*G0_1_2 - 0.192588083213081*G0_2_0 - 0.0962940416065408*G0_2_1 - 0.257045070413124*G0_2_2;
    A[432] = -0.0118736555368499*G0_1_1 - 0.0212681961813903*G0_1_2 - 0.0212681961813907*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[433] = -0.00254435475789602*G0_1_1 - 0.00583896797004424*G0_1_2 - 0.0058389679700442*G0_2_1 - 0.00913358118219224*G0_2_2;
    A[434] = -0.0118736555368503*G0_1_1 - 0.0212681961813909*G0_1_2 - 0.0212681961813904*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[435] = 0.0429604514890964*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.0422754329004322*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[436] = 0.0429604514890967*G0_1_1 + 0.0422754329004328*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[437] = -0.124738623002511*G0_1_1 - 0.0962940416065407*G0_1_2 - 0.0962940416065405*G0_2_1 - 0.128522535206562*G0_2_2;
    A[438] = 0.00939454064454037*G0_1_2 + 0.00939454064453999*G0_2_1 + 0.0187890812890804*G0_2_2;
    A[439] = 0.00329461321214794*G0_1_2 + 0.00329461321214827*G0_2_1 + 0.00658922642429644*G0_2_2;
    A[440] = 0.00939454064454058*G0_1_2 + 0.0093945406445402*G0_2_1 + 0.0187890812890808*G0_2_2;
    A[441] = 0.00329461321214787*G0_1_2 + 0.00329461321214846*G0_2_1 + 0.0292600797158091*G0_2_2;
    A[442] = 0.0032946132121477*G0_1_2 + 0.00329461321214783*G0_2_1 + 0.0292600797158087*G0_2_2;
    A[443] = 0.00939454064454091*G0_1_2 + 0.00939454064454034*G0_2_1 + 0.0641307878721069*G0_2_2;
    A[444] = -0.00260959462348373*G0_1_2 - 0.00260959462348307*G0_2_1 - 0.0505608958299923*G0_2_2;
    A[445] = -0.00260959462348355*G0_1_2 - 0.00260959462348385*G0_2_1 - 0.0505608958299935*G0_2_2;
    A[446] = -0.00260959462348288*G0_1_2 - 0.00260959462348333*G0_2_1 - 0.15657567740901*G0_2_2;
    A[447] = -0.0352295274170276*G0_1_2 - 0.0352295274170273*G0_2_1 + 0.0962288017409537*G0_2_2;
    A[448] = -0.00480034930989781*G0_0_0 - 0.0048003493098978*G0_0_1 - 0.00480034930989781*G0_0_2 - 0.00480034930989782*G0_2_0 - 0.00480034930989781*G0_2_1 - 0.00480034930989783*G0_2_2;
    A[449] = -0.0178705039816151*G0_0_0 + 0.0372571824394735*G0_2_0;
    A[450] = 0.00480034930989785*G0_0_1 + 0.0048003493098978*G0_2_1;
    A[451] = -0.0105310191030677*G0_0_2 + 0.00480034930989777*G0_2_2;
    A[452] = -0.010751529848752*G0_0_1 - 0.00537576492437605*G0_0_2 - 0.00537576492437568*G0_2_1 - 0.00537576492437594*G0_2_2;
    A[453] = -0.0139743792087541*G0_0_1 - 0.0116844599266472*G0_0_2 - 0.00228991928210694*G0_2_1 - 0.00228991928210651*G0_2_2;
    A[454] = -0.0111690649885095*G0_0_1 - 0.00887914570640295*G0_0_2 - 0.00228991928210672*G0_2_1 - 0.00228991928210713*G0_2_2;
    A[455] = -0.0328352243499809*G0_0_1 - 0.0274594594256048*G0_0_2 - 0.00537576492437599*G0_2_1 - 0.00537576492437578*G0_2_2;
    A[456] = 0.179096479009672*G0_0_0 + 0.0699110399631227*G0_0_2 + 0.0699110399631227*G0_2_0 + 0.139822079926245*G0_2_2;
    A[457] = -0.147063705006413*G0_0_0 + 0.0263830016434181*G0_0_2 - 0.0387263842124953*G0_2_0 - 0.108637424175617*G0_2_2;
    A[458] = 0.100717304493345*G0_0_0 - 0.0246345732456844*G0_0_2 + 0.0189782768992839*G0_2_0 + 0.0788945694544644*G0_2_2;
    A[459] = -0.032835224349981*G0_0_0 + 0.0209159009072205*G0_0_2 - 0.00537576492437604*G0_2_0 - 0.0328352243499808*G0_2_2;
    A[460] = 0.0895482395048359*G0_0_0 + 0.0699110399631226*G0_0_1 + 0.0699110399631225*G0_2_0 + 0.139822079926245*G0_2_1;
    A[461] = -0.0574176057031955*G0_0_0 - 0.0699110399631228*G0_0_1 - 0.0387263842124948*G0_2_0 - 0.108637424175617*G0_2_1;
    A[462] = 0.0283728175438242*G0_0_0 + 0.0599162925551812*G0_0_1 + 0.0189782768992836*G0_2_0 + 0.0788945694544647*G0_2_1;
    A[463] = -0.00537576492437617*G0_0_0 - 0.0274594594256053*G0_0_1 - 0.00537576492437603*G0_2_0 - 0.032835224349981*G0_2_1;
    A[464] = 0.010751529848752*G0_0_0 + 0.0107515298487519*G0_0_1 + 0.00537576492437593*G0_0_2 + 0.00537576492437586*G0_2_0 + 0.00537576492437582*G0_2_1;
    A[465] = 0.0139743792087542*G0_0_0 + 0.0139743792087543*G0_0_1 + 0.00228991928210696*G0_0_2 + 0.00228991928210716*G0_2_0 + 0.00228991928210723*G0_2_1;
    A[466] = 0.0111690649885092*G0_0_0 + 0.0111690649885092*G0_0_1 + 0.00228991928210658*G0_0_2 + 0.00228991928210643*G0_2_0 + 0.00228991928210645*G0_2_1;
    A[467] = 0.0328352243499811*G0_0_0 + 0.032835224349981*G0_0_1 + 0.00537576492437613*G0_0_2 + 0.00537576492437606*G0_2_0 + 0.00537576492437602*G0_2_1;
    A[468] = 0.00537576492437595*G0_0_0 + 0.00537576492437602*G0_0_2 + 0.00537576492437587*G0_2_0 + 0.00537576492437588*G0_2_2;
    A[469] = 0.00228991928210687*G0_0_0 + 0.0022899192821067*G0_0_2 + 0.00228991928210685*G0_2_0 + 0.00228991928210676*G0_2_2;
    A[470] = 0.00228991928210642*G0_0_0 + 0.00228991928210656*G0_0_2 + 0.00228991928210663*G0_2_0 + 0.00228991928210676*G0_2_2;
    A[471] = 0.00537576492437618*G0_0_0 + 0.00537576492437615*G0_0_2 + 0.00537576492437605*G0_2_0 + 0.005375764924376*G0_2_2;
    A[472] = 0.0220836945012289*G0_0_0 + 0.0274594594256049*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.0274594594256048*G0_2_0 + 0.0328352243499806*G0_2_1 + 0.0328352243499806*G0_2_2;
    A[473] = -0.0315434750113563*G0_0_0 - 0.0599162925551806*G0_0_1 - 0.0599162925551806*G0_0_2 - 0.0599162925551802*G0_2_0 - 0.0788945694544637*G0_2_1 - 0.0788945694544637*G0_2_2;
    A[474] = 0.0124934342599269*G0_0_0 + 0.0699110399631225*G0_0_1 + 0.0699110399631225*G0_0_2 + 0.0699110399631218*G0_2_0 + 0.108637424175617*G0_2_1 + 0.108637424175617*G0_2_2;
    A[475] = 0.0196371995417134*G0_0_0 - 0.0699110399631225*G0_0_1 - 0.0699110399631225*G0_0_2 - 0.0699110399631221*G0_2_0 - 0.139822079926245*G0_2_1 - 0.139822079926245*G0_2_2;
    A[476] = -0.0644569872000425*G0_0_0 + 0.192588083213082*G0_0_1 + 0.0962940416065404*G0_0_2 + 0.0284445813959699*G0_2_0 + 0.124738623002511*G0_2_1 + 0.12473862300251*G0_2_2;
    A[477] = 0.0439716694056967*G0_0_0 - 0.0845508658008653*G0_0_1 - 0.0422754329004323*G0_0_2 - 0.000685018588664639*G0_2_0 - 0.0429604514890972*G0_2_1 - 0.0429604514890966*G0_2_2;
    A[478] = -0.0187890812890813*G0_0_0 + 0.0425363923627807*G0_0_1 + 0.0212681961813898*G0_0_2 - 0.00939454064454018*G0_2_0 + 0.0118736555368499*G0_2_1 + 0.0118736555368497*G0_2_2;
    A[479] = 0.127315597693201*G0_0_0 - 0.0493213383838382*G0_0_1 - 0.00704590548340538*G0_0_2 - 0.000685018588664407*G0_2_0 - 0.0429604514890971*G0_2_1 - 0.0429604514890971*G0_2_2;
    A[480] = -0.0292600797158093*G0_0_0 + 0.0142875305635717*G0_0_1 + 0.00844856259352777*G0_0_2 - 0.00329461321214824*G0_2_0 + 0.00254435475789633*G0_2_1 + 0.00254435475789655*G0_2_2;
    A[481] = -0.0641307878721067*G0_0_0 + 0.0477555816097482*G0_0_1 + 0.0264873854283572*G0_0_2 - 0.00939454064454012*G0_2_0 + 0.0118736555368504*G0_2_1 + 0.0118736555368502*G0_2_2;
    A[482] = 0.018789081289081*G0_0_0 + 0.00939454064454054*G0_0_2 + 0.00939454064454033*G0_2_0;
    A[483] = 0.00658922642429617*G0_0_0 + 0.00329461321214835*G0_0_2 + 0.00329461321214866*G0_2_0;
    A[484] = 0.0187890812890813*G0_0_0 + 0.00939454064454058*G0_0_2 + 0.00939454064454021*G0_2_0;
    A[485] = 0.0292600797158086*G0_0_0 + 0.00329461321214758*G0_0_2 + 0.00329461321214757*G0_2_0;
    A[486] = 0.0292600797158093*G0_0_0 + 0.00329461321214779*G0_0_2 + 0.00329461321214825*G0_2_0;
    A[487] = 0.0641307878721067*G0_0_0 + 0.00939454064454045*G0_0_2 + 0.00939454064454008*G0_2_0;
    A[488] = -0.061325473651862*G0_0_0 - 0.0425363923627808*G0_0_1 - 0.0212681961813905*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.01187365553685*G0_2_1;
    A[489] = 0.128522535206562*G0_0_0 + 0.0845508658008648*G0_0_1 + 0.0422754329004324*G0_0_2 + 0.0422754329004326*G0_2_0 + 0.0429604514890966*G0_2_1;
    A[490] = -0.257045070413124*G0_0_0 - 0.192588083213081*G0_0_1 - 0.0962940416065409*G0_0_2 - 0.0962940416065408*G0_2_0 - 0.12473862300251*G0_2_1;
    A[491] = -0.0435476102793811*G0_0_0 - 0.0142875305635725*G0_0_1 - 0.00583896797004462*G0_0_2 - 0.00583896797004486*G0_2_0 - 0.00254435475789678*G0_2_1;
    A[492] = 0.17663693607704*G0_0_0 + 0.0493213383838385*G0_0_1 + 0.042275432900433*G0_0_2 + 0.042275432900433*G0_2_0 + 0.0429604514890973*G0_2_1;
    A[493] = -0.111886369481855*G0_0_0 - 0.0477555816097481*G0_0_1 - 0.0212681961813907*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.0118736555368503*G0_2_1;
    A[494] = -0.0306627368259312*G0_0_0 - 0.0212681961813906*G0_0_2 - 0.0212681961813905*G0_2_0 - 0.0118736555368502*G0_2_2;
    A[495] = 0.0642612676032813*G0_0_0 + 0.0422754329004329*G0_0_2 + 0.0422754329004327*G0_2_0 + 0.0429604514890969*G0_2_2;
    A[496] = -0.128522535206562*G0_0_0 - 0.0962940416065402*G0_0_2 - 0.0962940416065405*G0_2_0 - 0.12473862300251*G0_2_2;
    A[497] = -0.00913358118219186*G0_0_0 - 0.00583896797004421*G0_0_2 - 0.00583896797004396*G0_2_0 - 0.00254435475789565*G0_2_2;
    A[498] = 0.0642612676032805*G0_0_0 + 0.0422754329004324*G0_0_2 + 0.0422754329004319*G0_2_0 + 0.0429604514890966*G0_2_2;
    A[499] = -0.0306627368259306*G0_0_0 - 0.0212681961813901*G0_0_2 - 0.0212681961813903*G0_2_0 - 0.01187365553685*G0_2_2;
    A[500] = -0.0505608958299926*G0_0_0 - 0.00260959462348318*G0_0_2 - 0.00260959462348358*G0_2_0;
    A[501] = 0.0962288017409542*G0_0_0 - 0.0352295274170279*G0_0_2 - 0.0352295274170267*G0_2_0;
    A[502] = -0.0505608958299927*G0_0_0 - 0.00260959462348314*G0_0_2 - 0.00260959462348396*G0_2_0;
    A[503] = -0.15657567740901*G0_0_0 - 0.00260959462348363*G0_0_2 - 0.00260959462348319*G0_2_0;
    A[504] = -0.00336507226698195*G0_0_0 - 0.00336507226698198*G0_0_1 - 0.00336507226698195*G0_0_2 - 0.00336507226698194*G0_2_0 - 0.00336507226698198*G0_2_1 - 0.00336507226698196*G0_2_2;
    A[505] = 0.0214691349673987*G0_0_0 - 0.03365855145369*G0_2_0;
    A[506] = 0.003365072266982*G0_0_1 + 0.00336507226698202*G0_2_1;
    A[507] = 0.0232632312710438*G0_0_2 - 0.0155544887532731*G0_2_2;
    A[508] = -0.0139743792087542*G0_0_1 - 0.00228991928210659*G0_0_2 - 0.0116844599266478*G0_2_1 - 0.00228991928210665*G0_2_2;
    A[509] = 0.026304713804714*G0_0_1 + 0.0131523569023566*G0_0_2 + 0.0131523569023573*G0_2_1 + 0.0132175967679432*G0_2_2;
    A[510] = 0.00523223722008424*G0_0_1 + 0.00578025209101669*G0_0_2 - 0.000548014870931928*G0_2_1 - 0.000482775005343425*G0_2_2;
    A[511] = 0.0788945694544649*G0_0_1 + 0.0599162925551806*G0_0_2 + 0.0189782768992839*G0_2_1 + 0.0283728175438238*G0_2_2;
    A[512] = -0.147063705006413*G0_0_0 - 0.0387263842124953*G0_0_2 + 0.0263830016434181*G0_2_0 - 0.108637424175617*G0_2_2;
    A[513] = 0.304226541205706*G0_0_0 + 0.0781312630270961*G0_0_2 + 0.0781312630270961*G0_2_0 + 0.281001149056702*G0_2_2;
    A[514] = -0.204187731314467*G0_0_0 + 0.0153052724667307*G0_0_2 - 0.0364951808094172*G0_2_0 - 0.204187731314466*G0_2_2;
    A[515] = 0.078894569454465*G0_0_0 - 0.0246345732456845*G0_0_2 + 0.018978276899284*G0_2_0 + 0.100717304493345*G0_2_2;
    A[516] = -0.0574176057031955*G0_0_0 - 0.0387263842124951*G0_0_1 - 0.0699110399631225*G0_2_0 - 0.108637424175617*G0_2_1;
    A[517] = 0.00506261356955798*G0_0_0 + 0.00480165410720994*G0_0_1 + 0.00480165410720936*G0_2_0 + 0.00960330821441932*G0_2_1;
    A[518] = -0.000482775005344252*G0_0_0 + 0.00578025209101571*G0_0_1 - 0.000548014870931204*G0_2_0 + 0.00523223722008425*G0_2_1;
    A[519] = -0.00228991928210658*G0_0_0 - 0.00887914570640225*G0_0_1 - 0.00228991928210672*G0_2_0 - 0.0111690649885092*G0_2_1;
    A[520] = 0.013974379208754*G0_0_0 + 0.013974379208754*G0_0_1 + 0.0116844599266473*G0_0_2 + 0.0116844599266474*G0_2_0 + 0.0116844599266475*G0_2_1 + 0.00939454064454058*G0_2_2;
    A[521] = -0.0263047138047137*G0_0_0 - 0.0263047138047138*G0_0_1 - 0.0131523569023569*G0_0_2 - 0.0131523569023571*G0_2_0 - 0.0131523569023572*G0_2_1 + 6.52398655868728e-05*G0_2_2;
    A[522] = -0.0052322372200843*G0_0_0 - 0.00523223722008446*G0_0_1 + 0.000548014870931753*G0_0_2 + 0.000548014870931946*G0_2_0 + 0.000548014870931877*G0_2_1 + 6.52398655875241e-05*G0_2_2;
    A[523] = -0.0788945694544651*G0_0_0 - 0.078894569454465*G0_0_1 - 0.018978276899284*G0_0_2 - 0.018978276899284*G0_2_0 - 0.0189782768992839*G0_2_1 + 0.00939454064454016*G0_2_2;
    A[524] = 0.00228991928210682*G0_0_0 + 0.0022899192821067*G0_0_2 + 0.00228991928210691*G0_2_0 + 0.00228991928210688*G0_2_2;
    A[525] = 0.00178757231708605*G0_0_0 + 0.00178757231708606*G0_0_2 + 0.00178757231708609*G0_2_0 + 0.00178757231708606*G0_2_2;
    A[526] = 0.00178757231708649*G0_0_0 + 0.0017875723170865*G0_0_2 + 0.00178757231708617*G0_2_0 + 0.00178757231708616*G0_2_2;
    A[527] = 0.0022899192821066*G0_0_0 + 0.00228991928210661*G0_0_2 + 0.00228991928210673*G0_2_0 + 0.00228991928210678*G0_2_2;
    A[528] = 0.00658922642429577*G0_0_0 + 0.00887914570640253*G0_0_1 + 0.00887914570640243*G0_0_2 + 0.00887914570640258*G0_2_0 + 0.0111690649885095*G0_2_1 + 0.0111690649885095*G0_2_2;
    A[529] = -0.0062630270963603*G0_0_0 - 0.00578025209101579*G0_0_1 - 0.00578025209101569*G0_0_2 - 0.00578025209101595*G0_2_0 - 0.00523223722008475*G0_2_1 - 0.00523223722008456*G0_2_2;
    A[530] = 0.000260959462348103*G0_0_0 - 0.00480165410720994*G0_0_1 - 0.00480165410720999*G0_0_2 - 0.00480165410720967*G0_2_0 - 0.00960330821441887*G0_2_1 - 0.00960330821441905*G0_2_2;
    A[531] = -0.0186912214907003*G0_0_0 + 0.0387263842124953*G0_0_1 + 0.0387263842124952*G0_0_2 + 0.0387263842124949*G0_2_0 + 0.108637424175617*G0_2_1 + 0.108637424175617*G0_2_2;
    A[532] = 0.132306447410614*G0_0_0 + 0.010960297418631*G0_0_1 - 0.0623693115012548*G0_0_2 + 0.0733296089198871*G0_2_0 + 0.146659217839773*G0_2_1 - 0.12473862300251*G0_2_2;
    A[533] = -0.0436454700777614*G0_0_0 - 0.0808321934624011*G0_0_1 - 0.00649136662591536*G0_0_2 - 0.0338921101724918*G0_2_0 - 0.108232937008977*G0_2_1 - 0.0035229527417028*G0_2_2;
    A[534] = 0.014874689353856*G0_0_0 + 0.0403182369328203*G0_0_1 + 0.00123955744615523*G0_0_2 + 0.0149399292194421*G0_2_0 + 0.0540186087061079*G0_2_1 + 0.00117431758056765*G0_2_2;
    A[535] = -0.166948816037357*G0_0_0 + 0.167731694424402*G0_0_1 + 0.0838658472122009*G0_0_2 - 0.00616516729797994*G0_2_0 + 0.0777006799142212*G0_2_1 + 0.182410664181496*G0_2_2;
    A[536] = 0.0189848008858437*G0_0_0 - 0.0377738821749229*G0_0_1 - 0.0188869410874612*G0_0_2 + 0.009492400442922*G0_2_0 - 0.00939454064454032*G0_2_1 - 0.0155923278753141*G0_2_2;
    A[537] = 0.127087258163646*G0_0_0 - 0.0823327103709049*G0_0_1 - 0.0463855444324194*G0_0_2 + 0.0377738821749228*G0_2_0 + 0.00182671623643766*G0_2_1 - 0.0510175748891029*G0_2_2;
    A[538] = -0.0148746893538565*G0_0_0 - 0.0149399292194429*G0_0_2 - 0.0149399292194436*G0_2_0 - 0.0150051690850306*G0_2_2;
    A[539] = 0.00450155072550963*G0_0_0 + 0.00192457603481905*G0_0_2 + 0.00192457603481895*G0_2_0 - 0.000652398655870764*G0_2_2;
    A[540] = -0.0148746893538561*G0_0_0 - 0.0149399292194431*G0_0_2 - 0.0149399292194423*G0_2_0 - 0.0150051690850296*G0_2_2;
    A[541] = -0.0189848008858428*G0_0_0 - 0.00949240044292131*G0_0_2 - 0.00949240044292101*G0_2_0 - 0.000652398655870082*G0_2_2;
    A[542] = -0.0189848008858437*G0_0_0 - 0.00949240044292172*G0_0_2 - 0.00949240044292201*G0_2_0 - 0.000652398655871731*G0_2_2;
    A[543] = -0.127087258163646*G0_0_0 - 0.037773882174923*G0_0_2 - 0.0377738821749228*G0_2_0 - 0.0150051690850298*G0_2_2;
    A[544] = -0.0254435475789638*G0_0_0 - 0.0403182369328201*G0_0_1 - 0.039078679486665*G0_0_2 - 0.0390786794866655*G0_2_0 - 0.0540186087061086*G0_2_1 - 0.0528442911255407*G0_2_2;
    A[545] = 0.0371867233846392*G0_0_0 + 0.0808321934624008*G0_0_1 + 0.0743408268364854*G0_0_2 + 0.0743408268364858*G0_2_0 + 0.108232937008977*G0_2_1 + 0.104709984267274*G0_2_2;
    A[546] = 0.121346149991983*G0_0_0 - 0.0109602974186314*G0_0_1 - 0.0733296089198859*G0_0_2 - 0.0733296089198859*G0_2_0 - 0.146659217839773*G0_2_1 - 0.271397840842283*G0_2_2;
    A[547] = 0.0567586830607662*G0_0_0 + 0.0377738821749235*G0_0_1 + 0.0188869410874619*G0_0_2 + 0.018886941087462*G0_2_0 + 0.00939454064454075*G0_2_1 - 0.00619778723077246*G0_2_2;
    A[548] = -0.334680510461759*G0_0_0 - 0.167731694424402*G0_0_1 - 0.0838658472122007*G0_0_2 - 0.083865847212201*G0_2_0 - 0.0777006799142211*G0_2_1 + 0.104709984267276*G0_2_2;
    A[549] = 0.209419968534551*G0_0_0 + 0.0823327103709049*G0_0_1 + 0.0359471659384854*G0_0_2 + 0.0359471659384852*G0_2_0 - 0.00182671623643764*G0_2_1 - 0.052844291125541*G0_2_2;
    A[550] = -0.0013047973117418*G0_0_0 - 0.00123955744615507*G0_0_2 - 0.00123955744615513*G0_2_0 - 0.00117431758056814*G0_2_2;
    A[551] = 0.00880738185425679*G0_0_0 + 0.00649136662591475*G0_0_2 + 0.00649136662591556*G0_2_0 + 0.00352295274170291*G0_2_2;
    A[552] = 0.0476251018785737*G0_0_0 + 0.0623693115012547*G0_0_2 + 0.0623693115012551*G0_2_0 + 0.12473862300251*G0_2_2;
    A[553] = -0.0156575677409007*G0_0_0 - 0.0130805930502102*G0_0_2 - 0.0130805930502106*G0_2_0 - 0.0105036183595209*G0_2_2;
    A[554] = 0.00880738185425682*G0_0_0 + 0.00649136662591526*G0_0_2 + 0.00649136662591513*G0_2_0 + 0.00352295274170298*G0_2_2;
    A[555] = -0.00130479731174224*G0_0_0 - 0.00123955744615512*G0_0_2 - 0.001239557446155*G0_2_0 - 0.00117431758056773*G0_2_2;
    A[556] = 0.0391439193522522*G0_0_0 + 0.0319675341376723*G0_0_2 + 0.0319675341376729*G0_2_0 + 0.0260959462348346*G0_2_2;
    A[557] = -0.15657567740901*G0_0_0 - 0.0903572138381155*G0_0_2 - 0.0903572138381164*G0_2_0 - 0.185933616923199*G0_2_2;
    A[558] = 0.0391439193522515*G0_0_0 + 0.0319675341376715*G0_0_2 + 0.0319675341376729*G0_2_0 + 0.0260959462348351*G0_2_2;
    A[559] = 0.185933616923199*G0_0_0 + 0.0156575677409019*G0_0_2 + 0.0156575677409007*G0_2_0 + 0.026095946234836*G0_2_2;
    A[560] = -0.00336507226698204*G0_0_0 - 0.00336507226698205*G0_0_1 - 0.00336507226698203*G0_0_2 - 0.00336507226698199*G0_2_0 - 0.003365072266982*G0_2_1 - 0.00336507226698199*G0_2_2;
    A[561] = -0.0155544887532733*G0_0_0 + 0.0232632312710436*G0_2_0;
    A[562] = 0.00336507226698194*G0_0_1 + 0.00336507226698194*G0_2_1;
    A[563] = -0.0336585514536903*G0_0_2 + 0.0214691349673986*G0_2_2;
    A[564] = -0.011169064988509*G0_0_1 - 0.00228991928210665*G0_0_2 - 0.00887914570640214*G0_2_1 - 0.00228991928210681*G0_2_2;
    A[565] = 0.00523223722008385*G0_0_1 - 0.000548014870931544*G0_0_2 + 0.00578025209101535*G0_2_1 - 0.000482775005343853*G0_2_2;
    A[566] = 0.00960330821441965*G0_0_1 + 0.00480165410720938*G0_0_2 + 0.00480165410721008*G0_2_1 + 0.00506261356955721*G0_2_2;
    A[567] = -0.108637424175618*G0_0_1 - 0.0699110399631224*G0_0_2 - 0.0387263842124951*G0_2_1 - 0.0574176057031952*G0_2_2;
    A[568] = 0.100717304493345*G0_0_0 + 0.0189782768992839*G0_0_2 - 0.0246345732456844*G0_2_0 + 0.0788945694544644*G0_2_2;
    A[569] = -0.204187731314467*G0_0_0 - 0.0364951808094171*G0_0_2 + 0.0153052724667307*G0_2_0 - 0.204187731314466*G0_2_2;
    A[570] = 0.281001149056703*G0_0_0 + 0.0781312630270957*G0_0_2 + 0.0781312630270957*G0_2_0 + 0.304226541205706*G0_2_2;
    A[571] = -0.108637424175618*G0_0_0 + 0.0263830016434183*G0_0_2 - 0.0387263842124952*G0_2_0 - 0.147063705006412*G0_2_2;
    A[572] = 0.0283728175438241*G0_0_0 + 0.0189782768992838*G0_0_1 + 0.0599162925551807*G0_2_0 + 0.0788945694544644*G0_2_1;
    A[573] = -0.000482775005343989*G0_0_0 - 0.000548014870931593*G0_0_1 + 0.00578025209101624*G0_2_0 + 0.00523223722008439*G0_2_1;
    A[574] = 0.0132175967679435*G0_0_0 + 0.0131523569023569*G0_0_1 + 0.0131523569023565*G0_2_0 + 0.0263047138047137*G0_2_1;
    A[575] = -0.00228991928210666*G0_0_0 - 0.0116844599266473*G0_0_1 - 0.00228991928210672*G0_2_0 - 0.0139743792087541*G0_2_1;
    A[576] = 0.0111690649885094*G0_0_0 + 0.0111690649885093*G0_0_1 + 0.00887914570640274*G0_0_2 + 0.00887914570640234*G0_2_0 + 0.0088791457064023*G0_2_1 + 0.00658922642429587*G0_2_2;
    A[577] = -0.00523223722008414*G0_0_0 - 0.00523223722008403*G0_0_1 - 0.00578025209101601*G0_0_2 - 0.00578025209101551*G0_2_0 - 0.00578025209101538*G0_2_1 - 0.00626302709636038*G0_2_2;
    A[578] = -0.00960330821441947*G0_0_0 - 0.00960330821441934*G0_0_1 - 0.00480165410720994*G0_0_2 - 0.00480165410721022*G0_2_0 - 0.00480165410721019*G0_2_1 + 0.000260959462348024*G0_2_2;
    A[579] = 0.108637424175618*G0_0_0 + 0.108637424175618*G0_0_1 + 0.0387263842124954*G0_0_2 + 0.0387263842124952*G0_2_0 + 0.0387263842124952*G0_2_1 - 0.0186912214907002*G0_2_2;
    A[580] = 0.0022899192821066*G0_0_0 + 0.00228991928210672*G0_0_2 + 0.00228991928210643*G0_2_0 + 0.00228991928210646*G0_2_2;
    A[581] = 0.0017875723170863*G0_0_0 + 0.00178757231708624*G0_0_2 + 0.00178757231708634*G0_2_0 + 0.00178757231708629*G0_2_2;
    A[582] = 0.00178757231708595*G0_0_0 + 0.001787572317086*G0_0_2 + 0.00178757231708621*G0_2_0 + 0.0017875723170863*G0_2_2;
    A[583] = 0.00228991928210665*G0_0_0 + 0.00228991928210666*G0_0_2 + 0.00228991928210672*G0_2_0 + 0.00228991928210668*G0_2_2;
    A[584] = 0.00939454064454075*G0_0_0 + 0.0116844599266473*G0_0_1 + 0.0116844599266473*G0_0_2 + 0.0116844599266473*G0_2_0 + 0.0139743792087537*G0_2_1 + 0.0139743792087537*G0_2_2;
    A[585] = 6.52398655868465e-05*G0_0_0 - 0.0131523569023569*G0_0_1 - 0.0131523569023569*G0_0_2 - 0.0131523569023568*G0_2_0 - 0.0263047138047132*G0_2_1 - 0.0263047138047133*G0_2_2;
    A[586] = 6.52398655873055e-05*G0_0_0 + 0.000548014870931639*G0_0_1 + 0.000548014870931669*G0_0_2 + 0.00054801487093154*G0_2_0 - 0.00523223722008495*G0_2_1 - 0.00523223722008478*G0_2_2;
    A[587] = 0.00939454064454042*G0_0_0 - 0.0189782768992837*G0_0_1 - 0.0189782768992837*G0_0_2 - 0.0189782768992837*G0_2_0 - 0.0788945694544643*G0_2_1 - 0.0788945694544643*G0_2_2;
    A[588] = -0.0510175748891031*G0_0_0 + 0.00182671623643766*G0_0_1 + 0.037773882174923*G0_0_2 - 0.0463855444324201*G0_2_0 - 0.0823327103709049*G0_2_1 + 0.127087258163645*G0_2_2;
    A[589] = -0.0155923278753141*G0_0_0 - 0.00939454064454051*G0_0_1 + 0.00949240044292153*G0_0_2 - 0.0188869410874618*G0_2_0 - 0.0377738821749239*G0_2_1 + 0.0189848008858424*G0_2_2;
    A[590] = 0.00117431758056772*G0_0_0 + 0.0540186087061078*G0_0_1 + 0.0149399292194429*G0_0_2 + 0.00123955744615541*G0_2_0 + 0.0403182369328201*G0_2_1 + 0.0148746893538561*G0_2_2;
    A[591] = 0.182410664181496*G0_0_0 + 0.0777006799142199*G0_0_1 - 0.00616516729798081*G0_0_2 + 0.0838658472122*G0_2_0 + 0.167731694424401*G0_2_1 - 0.166948816037357*G0_2_2;
    A[592] = -0.00352295274170347*G0_0_0 - 0.108232937008978*G0_0_1 - 0.0338921101724925*G0_0_2 - 0.00649136662591575*G0_2_0 - 0.0808321934624011*G0_2_1 - 0.0436454700777615*G0_2_2;
    A[593] = -0.12473862300251*G0_0_0 + 0.146659217839772*G0_0_1 + 0.0733296089198866*G0_0_2 - 0.0623693115012548*G0_2_0 + 0.0109602974186307*G0_2_1 + 0.132306447410614*G0_2_2;
    A[594] = -0.00117431758056734*G0_0_0 - 0.00123955744615454*G0_0_2 - 0.00123955744615433*G0_2_0 - 0.00130479731174098*G0_2_2;
    A[595] = -0.0105036183595212*G0_0_0 - 0.0130805930502112*G0_0_2 - 0.0130805930502105*G0_2_0 - 0.0156575677409011*G0_2_2;
    A[596] = -0.00117431758056764*G0_0_0 - 0.0012395574461543*G0_0_2 - 0.00123955744615533*G0_2_0 - 0.00130479731174204*G0_2_2;
    A[597] = 0.00352295274170245*G0_0_0 + 0.0064913666259151*G0_0_2 + 0.00649136662591444*G0_2_0 + 0.00880738185425568*G0_2_2;
    A[598] = 0.00352295274170326*G0_0_0 + 0.00649136662591511*G0_0_2 + 0.00649136662591562*G0_2_0 + 0.00880738185425702*G0_2_2;
    A[599] = 0.12473862300251*G0_0_0 + 0.0623693115012554*G0_0_2 + 0.0623693115012549*G0_2_0 + 0.0476251018785741*G0_2_2;
    A[600] = -0.052844291125541*G0_0_0 - 0.054018608706108*G0_0_1 - 0.0390786794866652*G0_0_2 - 0.039078679486665*G0_2_0 - 0.0403182369328196*G0_2_1 - 0.0254435475789635*G0_2_2;
    A[601] = -0.0061977872307726*G0_0_0 + 0.00939454064454019*G0_0_1 + 0.0188869410874618*G0_0_2 + 0.0188869410874618*G0_2_0 + 0.0377738821749233*G0_2_1 + 0.0567586830607663*G0_2_2;
    A[602] = -0.0528442911255411*G0_0_0 - 0.00182671623643766*G0_0_1 + 0.0359471659384853*G0_0_2 + 0.0359471659384854*G0_2_0 + 0.0823327103709051*G0_2_1 + 0.209419968534551*G0_2_2;
    A[603] = 0.104709984267274*G0_0_0 + 0.108232937008977*G0_0_1 + 0.0743408268364857*G0_0_2 + 0.0743408268364854*G0_2_0 + 0.0808321934624004*G0_2_1 + 0.0371867233846386*G0_2_2;
    A[604] = 0.104709984267276*G0_0_0 - 0.0777006799142196*G0_0_1 - 0.0838658472122004*G0_0_2 - 0.0838658472122007*G0_2_0 - 0.167731694424401*G0_2_1 - 0.334680510461758*G0_2_2;
    A[605] = -0.271397840842283*G0_0_0 - 0.146659217839772*G0_0_1 - 0.0733296089198861*G0_0_2 - 0.0733296089198853*G0_2_0 - 0.0109602974186306*G0_2_1 + 0.121346149991983*G0_2_2;
    A[606] = -0.01500516908503*G0_0_0 - 0.014939929219443*G0_0_2 - 0.0149399292194424*G0_2_0 - 0.0148746893538556*G0_2_2;
    A[607] = -0.00065239865587076*G0_0_0 - 0.00949240044292092*G0_0_2 - 0.00949240044292173*G0_2_0 - 0.0189848008858431*G0_2_2;
    A[608] = -0.0150051690850299*G0_0_0 - 0.0377738821749231*G0_0_2 - 0.037773882174923*G0_2_0 - 0.127087258163645*G0_2_2;
    A[609] = -0.000652398655870713*G0_0_0 + 0.0019245760348191*G0_0_2 + 0.00192457603481886*G0_2_0 + 0.00450155072550932*G0_2_2;
    A[610] = -0.000652398655871739*G0_0_0 - 0.00949240044292157*G0_0_2 - 0.00949240044292143*G0_2_0 - 0.0189848008858428*G0_2_2;
    A[611] = -0.0150051690850295*G0_0_0 - 0.014939929219443*G0_0_2 - 0.0149399292194427*G0_2_0 - 0.014874689353856*G0_2_2;
    A[612] = 0.0260959462348346*G0_0_0 + 0.0319675341376724*G0_0_2 + 0.0319675341376722*G0_2_0 + 0.0391439193522524*G0_2_2;
    A[613] = 0.026095946234836*G0_0_0 + 0.0156575677409018*G0_0_2 + 0.0156575677409022*G0_2_0 + 0.1859336169232*G0_2_2;
    A[614] = 0.0260959462348356*G0_0_0 + 0.0319675341376737*G0_0_2 + 0.0319675341376724*G0_2_0 + 0.0391439193522523*G0_2_2;
    A[615] = -0.185933616923198*G0_0_0 - 0.0903572138381162*G0_0_2 - 0.0903572138381145*G0_2_0 - 0.15657567740901*G0_2_2;
    A[616] = -0.00480034930989779*G0_0_0 - 0.00480034930989782*G0_0_1 - 0.00480034930989782*G0_0_2 - 0.00480034930989781*G0_2_0 - 0.00480034930989784*G0_2_1 - 0.00480034930989782*G0_2_2;
    A[617] = 0.00480034930989784*G0_0_0 - 0.0105310191030676*G0_2_0;
    A[618] = 0.00480034930989786*G0_0_1 + 0.00480034930989786*G0_2_1;
    A[619] = 0.0372571824394741*G0_0_2 - 0.0178705039816149*G0_2_2;
    A[620] = -0.0328352243499813*G0_0_1 - 0.00537576492437607*G0_0_2 - 0.0274594594256052*G0_2_1 - 0.00537576492437583*G0_2_2;
    A[621] = 0.0788945694544649*G0_0_1 + 0.0189782768992835*G0_0_2 + 0.059916292555181*G0_2_1 + 0.0283728175438236*G0_2_2;
    A[622] = -0.108637424175618*G0_0_1 - 0.0387263842124944*G0_0_2 - 0.0699110399631228*G0_2_1 - 0.0574176057031946*G0_2_2;
    A[623] = 0.139822079926246*G0_0_1 + 0.0699110399631224*G0_0_2 + 0.0699110399631228*G0_2_1 + 0.0895482395048356*G0_2_2;
    A[624] = -0.032835224349981*G0_0_0 - 0.00537576492437604*G0_0_2 + 0.0209159009072205*G0_2_0 - 0.0328352243499808*G0_2_2;
    A[625] = 0.078894569454465*G0_0_0 + 0.018978276899284*G0_0_2 - 0.0246345732456845*G0_2_0 + 0.100717304493345*G0_2_2;
    A[626] = -0.108637424175618*G0_0_0 - 0.0387263842124952*G0_0_2 + 0.0263830016434183*G0_2_0 - 0.147063705006412*G0_2_2;
    A[627] = 0.139822079926246*G0_0_0 + 0.0699110399631229*G0_0_2 + 0.0699110399631229*G0_2_0 + 0.179096479009672*G0_2_2;
    A[628] = -0.00537576492437608*G0_0_0 - 0.00537576492437607*G0_0_1 - 0.027459459425605*G0_2_0 - 0.0328352243499809*G0_2_1;
    A[629] = -0.00228991928210669*G0_0_0 - 0.00228991928210686*G0_0_1 - 0.00887914570640245*G0_2_0 - 0.0111690649885095*G0_2_1;
    A[630] = -0.00228991928210667*G0_0_0 - 0.00228991928210655*G0_0_1 - 0.0116844599266473*G0_2_0 - 0.0139743792087539*G0_2_1;
    A[631] = -0.00537576492437607*G0_0_0 - 0.00537576492437618*G0_0_1 - 0.00537576492437591*G0_2_0 - 0.010751529848752*G0_2_1;
    A[632] = 0.0328352243499809*G0_0_0 + 0.0328352243499809*G0_0_1 + 0.0274594594256049*G0_0_2 + 0.027459459425605*G0_2_0 + 0.027459459425605*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[633] = -0.0788945694544645*G0_0_0 - 0.0788945694544647*G0_0_1 - 0.059916292555181*G0_0_2 - 0.0599162925551808*G0_2_0 - 0.0599162925551809*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[634] = 0.108637424175618*G0_0_0 + 0.108637424175618*G0_0_1 + 0.0699110399631232*G0_0_2 + 0.0699110399631229*G0_2_0 + 0.0699110399631229*G0_2_1 + 0.0124934342599273*G0_2_2;
    A[635] = -0.139822079926246*G0_0_0 - 0.139822079926246*G0_0_1 - 0.0699110399631234*G0_0_2 - 0.0699110399631229*G0_2_0 - 0.0699110399631229*G0_2_1 + 0.019637199541713*G0_2_2;
    A[636] = 0.00537576492437599*G0_0_0 + 0.00537576492437594*G0_0_2 + 0.00537576492437602*G0_2_0 + 0.00537576492437602*G0_2_2;
    A[637] = 0.00228991928210676*G0_0_0 + 0.00228991928210677*G0_0_2 + 0.00228991928210666*G0_2_0 + 0.00228991928210666*G0_2_2;
    A[638] = 0.00228991928210661*G0_0_0 + 0.00228991928210663*G0_0_2 + 0.0022899192821065*G0_2_0 + 0.00228991928210652*G0_2_2;
    A[639] = 0.00537576492437611*G0_0_0 + 0.00537576492437613*G0_0_2 + 0.00537576492437594*G0_2_0 + 0.00537576492437598*G0_2_2;
    A[640] = 0.00537576492437597*G0_0_1 + 0.00537576492437591*G0_0_2 + 0.005375764924376*G0_2_0 + 0.0107515298487522*G0_2_1 + 0.0107515298487521*G0_2_2;
    A[641] = 0.00228991928210678*G0_0_1 + 0.00228991928210681*G0_0_2 + 0.00228991928210668*G0_2_0 + 0.0139743792087539*G0_2_1 + 0.013974379208754*G0_2_2;
    A[642] = 0.00228991928210681*G0_0_1 + 0.00228991928210681*G0_0_2 + 0.00228991928210677*G0_2_0 + 0.0111690649885097*G0_2_1 + 0.0111690649885096*G0_2_2;
    A[643] = 0.00537576492437618*G0_0_1 + 0.0053757649243761*G0_0_2 + 0.00537576492437602*G0_2_0 + 0.032835224349981*G0_2_1 + 0.0328352243499809*G0_2_2;
    A[644] = 0.01187365553685*G0_0_0 + 0.0118736555368502*G0_0_1 - 0.00939454064454068*G0_0_2 + 0.0264873854283575*G0_2_0 + 0.0477555816097481*G0_2_1 - 0.0641307878721067*G0_2_2;
    A[645] = 0.00254435475789617*G0_0_0 + 0.00254435475789612*G0_0_1 - 0.00329461321214768*G0_0_2 + 0.00844856259352797*G0_2_0 + 0.0142875305635725*G0_2_1 - 0.0292600797158082*G0_2_2;
    A[646] = 0.0118736555368494*G0_0_0 + 0.0118736555368496*G0_0_1 - 0.00939454064454101*G0_0_2 + 0.0212681961813897*G0_2_0 + 0.0425363923627803*G0_2_1 - 0.0187890812890815*G0_2_2;
    A[647] = -0.0429604514890971*G0_0_0 - 0.042960451489097*G0_0_1 - 0.000685018588664376*G0_0_2 - 0.00704590548340508*G0_2_0 - 0.0493213383838381*G0_2_1 + 0.127315597693201*G0_2_2;
    A[648] = -0.0429604514890966*G0_0_0 - 0.0429604514890967*G0_0_1 - 0.000685018588664582*G0_0_2 - 0.0422754329004323*G0_2_0 - 0.0845508658008649*G0_2_1 + 0.0439716694056966*G0_2_2;
    A[649] = 0.124738623002512*G0_0_0 + 0.124738623002512*G0_0_1 + 0.0284445813959708*G0_0_2 + 0.0962940416065412*G0_2_0 + 0.192588083213082*G0_2_1 - 0.0644569872000419*G0_2_2;
    A[650] = -0.0118736555368496*G0_0_0 - 0.02126819618139*G0_0_2 - 0.0212681961813906*G0_2_0 - 0.0306627368259309*G0_2_2;
    A[651] = -0.002544354757896*G0_0_0 - 0.00583896797004374*G0_0_2 - 0.0058389679700442*G0_2_0 - 0.00913358118219163*G0_2_2;
    A[652] = -0.0118736555368494*G0_0_0 - 0.0212681961813904*G0_0_2 - 0.0212681961813898*G0_2_0 - 0.0306627368259304*G0_2_2;
    A[653] = 0.0429604514890963*G0_0_0 + 0.0422754329004314*G0_0_2 + 0.0422754329004323*G0_2_0 + 0.0642612676032806*G0_2_2;
    A[654] = 0.0429604514890966*G0_0_0 + 0.0422754329004318*G0_0_2 + 0.0422754329004323*G0_2_0 + 0.06426126760328*G0_2_2;
    A[655] = -0.124738623002512*G0_0_0 - 0.09629404160654*G0_0_2 - 0.0962940416065411*G0_2_0 - 0.128522535206561*G0_2_2;
    A[656] = -0.0118736555368497*G0_0_1 - 0.0212681961813901*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.042536392362781*G0_2_1 - 0.061325473651862*G0_2_2;
    A[657] = -0.00254435475789622*G0_0_1 - 0.00583896797004445*G0_0_2 - 0.00583896797004378*G0_2_0 - 0.014287530563572*G0_2_1 - 0.043547610279381*G0_2_2;
    A[658] = -0.0118736555368502*G0_0_1 - 0.0212681961813909*G0_0_2 - 0.0212681961813912*G0_2_0 - 0.0477555816097483*G0_2_1 - 0.111886369481855*G0_2_2;
    A[659] = 0.0429604514890962*G0_0_1 + 0.0422754329004322*G0_0_2 + 0.0422754329004319*G0_2_0 + 0.0845508658008648*G0_2_1 + 0.128522535206562*G0_2_2;
    A[660] = 0.0429604514890968*G0_0_1 + 0.042275432900433*G0_0_2 + 0.0422754329004333*G0_2_0 + 0.0493213383838382*G0_2_1 + 0.17663693607704*G0_2_2;
    A[661] = -0.124738623002511*G0_0_1 - 0.0962940416065407*G0_0_2 - 0.0962940416065412*G0_2_0 - 0.192588083213082*G0_2_1 - 0.257045070413125*G0_2_2;
    A[662] = 0.00939454064454052*G0_0_2 + 0.00939454064454038*G0_2_0 + 0.0187890812890807*G0_2_2;
    A[663] = 0.00329461321214786*G0_0_2 + 0.0032946132121484*G0_2_0 + 0.0292600797158094*G0_2_2;
    A[664] = 0.00939454064454069*G0_0_2 + 0.00939454064454022*G0_2_0 + 0.0641307878721065*G0_2_2;
    A[665] = 0.00329461321214802*G0_0_2 + 0.00329461321214822*G0_2_0 + 0.0065892264242959*G0_2_2;
    A[666] = 0.00329461321214774*G0_0_2 + 0.00329461321214755*G0_2_0 + 0.0292600797158084*G0_2_2;
    A[667] = 0.00939454064454094*G0_0_2 + 0.00939454064454076*G0_2_0 + 0.0187890812890813*G0_2_2;
    A[668] = -0.00260959462348397*G0_0_2 - 0.00260959462348386*G0_2_0 - 0.0505608958299932*G0_2_2;
    A[669] = -0.00260959462348332*G0_0_2 - 0.00260959462348297*G0_2_0 - 0.15657567740901*G0_2_2;
    A[670] = -0.00260959462348322*G0_0_2 - 0.00260959462348375*G0_2_0 - 0.0505608958299921*G0_2_2;
    A[671] = -0.0352295274170276*G0_0_2 - 0.0352295274170275*G0_2_0 + 0.0962288017409537*G0_2_2;
    A[672] = -0.00480034930989777*G0_0_0 - 0.00480034930989775*G0_0_1 - 0.00480034930989777*G0_0_2 - 0.00480034930989781*G0_1_0 - 0.0048003493098978*G0_1_1 - 0.00480034930989781*G0_1_2;
    A[673] = -0.0178705039816151*G0_0_0 + 0.0372571824394736*G0_1_0;
    A[674] = -0.0105310191030677*G0_0_1 + 0.0048003493098978*G0_1_1;
    A[675] = 0.00480034930989779*G0_0_2 + 0.00480034930989777*G0_1_2;
    A[676] = -0.0274594594256051*G0_0_1 - 0.0328352243499812*G0_0_2 - 0.00537576492437577*G0_1_1 - 0.005375764924376*G0_1_2;
    A[677] = -0.00887914570640269*G0_0_1 - 0.0111690649885091*G0_0_2 - 0.00228991928210686*G0_1_1 - 0.00228991928210647*G0_1_2;
    A[678] = -0.0116844599266472*G0_0_1 - 0.0139743792087545*G0_0_2 - 0.00228991928210671*G0_1_1 - 0.00228991928210711*G0_1_2;
    A[679] = -0.00537576492437606*G0_0_1 - 0.0107515298487519*G0_0_2 - 0.00537576492437602*G0_1_1 - 0.00537576492437581*G0_1_2;
    A[680] = 0.0895482395048359*G0_0_0 + 0.0699110399631225*G0_0_2 + 0.0699110399631226*G0_1_0 + 0.139822079926245*G0_1_2;
    A[681] = -0.0574176057031955*G0_0_0 - 0.0699110399631225*G0_0_2 - 0.0387263842124951*G0_1_0 - 0.108637424175617*G0_1_2;
    A[682] = 0.0283728175438241*G0_0_0 + 0.0599162925551807*G0_0_2 + 0.0189782768992838*G0_1_0 + 0.0788945694544644*G0_1_2;
    A[683] = -0.00537576492437608*G0_0_0 - 0.027459459425605*G0_0_2 - 0.00537576492437607*G0_1_0 - 0.0328352243499809*G0_1_2;
    A[684] = 0.179096479009672*G0_0_0 + 0.0699110399631224*G0_0_1 + 0.0699110399631224*G0_1_0 + 0.139822079926245*G0_1_1;
    A[685] = -0.147063705006412*G0_0_0 + 0.0263830016434179*G0_0_1 - 0.0387263842124948*G0_1_0 - 0.108637424175618*G0_1_1;
    A[686] = 0.100717304493345*G0_0_0 - 0.024634573245684*G0_0_1 + 0.0189782768992835*G0_1_0 + 0.0788945694544648*G0_1_1;
    A[687] = -0.0328352243499813*G0_0_0 + 0.0209159009072198*G0_0_1 - 0.00537576492437609*G0_1_0 - 0.0328352243499811*G0_1_1;
    A[688] = 0.00537576492437578*G0_0_0 + 0.0053757649243757*G0_0_1 + 0.00537576492437578*G0_1_0 + 0.00537576492437574*G0_1_1;
    A[689] = 0.00228991928210711*G0_0_0 + 0.0022899192821072*G0_0_1 + 0.00228991928210726*G0_1_0 + 0.00228991928210734*G0_1_1;
    A[690] = 0.00228991928210621*G0_0_0 + 0.00228991928210623*G0_0_1 + 0.00228991928210626*G0_1_0 + 0.00228991928210629*G0_1_1;
    A[691] = 0.00537576492437612*G0_0_0 + 0.00537576492437608*G0_0_1 + 0.00537576492437609*G0_1_0 + 0.00537576492437605*G0_1_1;
    A[692] = 0.0107515298487517*G0_0_0 + 0.00537576492437575*G0_0_1 + 0.0107515298487518*G0_0_2 + 0.0053757649243758*G0_1_0 + 0.0053757649243758*G0_1_2;
    A[693] = 0.0139743792087543*G0_0_0 + 0.00228991928210698*G0_0_1 + 0.0139743792087541*G0_0_2 + 0.0022899192821069*G0_1_0 + 0.0022899192821068*G0_1_2;
    A[694] = 0.0111690649885089*G0_0_0 + 0.00228991928210652*G0_0_1 + 0.011169064988509*G0_0_2 + 0.00228991928210652*G0_1_0 + 0.00228991928210665*G0_1_2;
    A[695] = 0.0328352243499813*G0_0_0 + 0.00537576492437614*G0_0_1 + 0.0328352243499812*G0_0_2 + 0.00537576492437611*G0_1_0 + 0.00537576492437606*G0_1_2;
    A[696] = 0.0220836945012288*G0_0_0 + 0.0274594594256047*G0_0_1 + 0.0274594594256047*G0_0_2 + 0.0274594594256047*G0_1_0 + 0.0328352243499805*G0_1_1 + 0.0328352243499805*G0_1_2;
    A[697] = -0.0315434750113562*G0_0_0 - 0.0599162925551804*G0_0_1 - 0.0599162925551804*G0_0_2 - 0.0599162925551802*G0_1_0 - 0.0788945694544636*G0_1_1 - 0.0788945694544636*G0_1_2;
    A[698] = 0.0124934342599269*G0_0_0 + 0.0699110399631222*G0_0_1 + 0.0699110399631223*G0_0_2 + 0.0699110399631219*G0_1_0 + 0.108637424175617*G0_1_1 + 0.108637424175617*G0_1_2;
    A[699] = 0.0196371995417135*G0_0_0 - 0.0699110399631223*G0_0_1 - 0.0699110399631223*G0_0_2 - 0.0699110399631221*G0_1_0 - 0.139822079926245*G0_1_1 - 0.139822079926245*G0_1_2;
    A[700] = -0.0644569872000429*G0_0_0 + 0.0962940416065407*G0_0_1 + 0.192588083213081*G0_0_2 + 0.0284445813959697*G0_1_0 + 0.124738623002511*G0_1_1 + 0.12473862300251*G0_1_2;
    A[701] = 0.127315597693201*G0_0_0 - 0.00704590548340603*G0_0_1 - 0.0493213383838381*G0_0_2 - 0.000685018588664654*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.0429604514890967*G0_1_2;
    A[702] = -0.064130787872107*G0_0_0 + 0.0264873854283574*G0_0_1 + 0.047755581609747*G0_0_2 - 0.00939454064454032*G0_1_0 + 0.0118736555368499*G0_1_1 + 0.0118736555368495*G0_1_2;
    A[703] = 0.043971669405697*G0_0_0 - 0.0422754329004326*G0_0_1 - 0.0845508658008649*G0_0_2 - 0.000685018588664481*G0_1_0 - 0.042960451489097*G0_1_1 - 0.0429604514890971*G0_1_2;
    A[704] = -0.0292600797158088*G0_0_0 + 0.00844856259352803*G0_0_1 + 0.0142875305635727*G0_0_2 - 0.00329461321214811*G0_1_0 + 0.00254435475789637*G0_1_1 + 0.00254435475789672*G0_1_2;
    A[705] = -0.0187890812890809*G0_0_0 + 0.0212681961813904*G0_0_1 + 0.0425363923627807*G0_0_2 - 0.00939454064454017*G0_1_0 + 0.0118736555368502*G0_1_1 + 0.0118736555368501*G0_1_2;
    A[706] = 0.0187890812890812*G0_0_0 + 0.00939454064454095*G0_0_1 + 0.00939454064454038*G0_1_0;
    A[707] = 0.0292600797158093*G0_0_0 + 0.00329461321214781*G0_0_1 + 0.00329461321214863*G0_1_0;
    A[708] = 0.064130787872107*G0_0_0 + 0.00939454064454065*G0_0_1 + 0.00939454064454034*G0_1_0;
    A[709] = 0.00658922642429559*G0_0_0 + 0.00329461321214776*G0_0_1 + 0.00329461321214757*G0_1_0;
    A[710] = 0.0292600797158088*G0_0_0 + 0.00329461321214779*G0_0_1 + 0.00329461321214812*G0_1_0;
    A[711] = 0.0187890812890809*G0_0_0 + 0.00939454064454093*G0_0_1 + 0.00939454064454013*G0_1_0;
    A[712] = -0.0306627368259306*G0_0_0 - 0.0212681961813901*G0_0_1 - 0.0212681961813902*G0_1_0 - 0.0118736555368498*G0_1_1;
    A[713] = 0.0642612676032807*G0_0_0 + 0.0422754329004324*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.0429604514890966*G0_1_1;
    A[714] = -0.128522535206562*G0_0_0 - 0.0962940416065402*G0_0_1 - 0.0962940416065406*G0_1_0 - 0.12473862300251*G0_1_1;
    A[715] = -0.00913358118219274*G0_0_0 - 0.00583896797004514*G0_0_1 - 0.00583896797004493*G0_1_0 - 0.00254435475789688*G0_1_1;
    A[716] = 0.0642612676032811*G0_0_0 + 0.0422754329004328*G0_0_1 + 0.0422754329004329*G0_1_0 + 0.0429604514890972*G0_1_1;
    A[717] = -0.0306627368259304*G0_0_0 - 0.0212681961813904*G0_0_1 - 0.0212681961813901*G0_1_0 - 0.0118736555368502*G0_1_1;
    A[718] = -0.0613254736518618*G0_0_0 - 0.0212681961813902*G0_0_1 - 0.0425363923627808*G0_0_2 - 0.0212681961813904*G0_1_0 - 0.0118736555368501*G0_1_2;
    A[719] = 0.128522535206562*G0_0_0 + 0.0422754329004326*G0_0_1 + 0.0845508658008652*G0_0_2 + 0.0422754329004325*G0_1_0 + 0.0429604514890966*G0_1_2;
    A[720] = -0.257045070413124*G0_0_0 - 0.0962940416065404*G0_0_1 - 0.192588083213081*G0_0_2 - 0.0962940416065403*G0_1_0 - 0.12473862300251*G0_1_2;
    A[721] = -0.0435476102793806*G0_0_0 - 0.00583896797004441*G0_0_1 - 0.014287530563572*G0_0_2 - 0.00583896797004411*G0_1_0 - 0.00254435475789597*G0_1_2;
    A[722] = 0.176636936077038*G0_0_0 + 0.0422754329004321*G0_0_1 + 0.049321338383838*G0_0_2 + 0.042275432900432*G0_1_0 + 0.0429604514890967*G0_1_2;
    A[723] = -0.111886369481854*G0_0_0 - 0.0212681961813899*G0_0_1 - 0.0477555816097474*G0_0_2 - 0.0212681961813901*G0_1_0 - 0.0118736555368497*G0_1_2;
    A[724] = -0.0505608958299925*G0_0_0 - 0.00260959462348269*G0_0_1 - 0.00260959462348376*G0_1_0;
    A[725] = 0.0962288017409541*G0_0_0 - 0.0352295274170271*G0_0_1 - 0.0352295274170264*G0_1_0;
    A[726] = -0.15657567740901*G0_0_0 - 0.00260959462348406*G0_0_1 - 0.00260959462348391*G0_1_0;
    A[727] = -0.0505608958299923*G0_0_0 - 0.00260959462348312*G0_0_1 - 0.00260959462348312*G0_1_0;
    A[728] = -0.00336507226698203*G0_0_0 - 0.00336507226698205*G0_0_1 - 0.00336507226698205*G0_0_2 - 0.00336507226698192*G0_1_0 - 0.0033650722669819*G0_1_1 - 0.00336507226698192*G0_1_2;
    A[729] = 0.0214691349673989*G0_0_0 - 0.0336585514536899*G0_1_0;
    A[730] = 0.0232632312710435*G0_0_1 - 0.0155544887532734*G0_1_1;
    A[731] = 0.00336507226698199*G0_0_2 + 0.00336507226698196*G0_1_2;
    A[732] = 0.0599162925551811*G0_0_1 + 0.0788945694544648*G0_0_2 + 0.0283728175438242*G0_1_1 + 0.0189782768992837*G0_1_2;
    A[733] = 0.00578025209101596*G0_0_1 + 0.00523223722008438*G0_0_2 - 0.00048277500534449*G0_1_1 - 0.000548014870931489*G0_1_2;
    A[734] = 0.0131523569023567*G0_0_1 + 0.0263047138047135*G0_0_2 + 0.0132175967679442*G0_1_1 + 0.0131523569023569*G0_1_2;
    A[735] = -0.00228991928210666*G0_0_1 - 0.0139743792087539*G0_0_2 - 0.00228991928210686*G0_1_1 - 0.0116844599266474*G0_1_2;
    A[736] = -0.0574176057031955*G0_0_0 - 0.0387263842124948*G0_0_2 - 0.0699110399631228*G0_1_0 - 0.108637424175617*G0_1_2;
    A[737] = 0.00506261356955798*G0_0_0 + 0.00480165410720936*G0_0_2 + 0.00480165410720994*G0_1_0 + 0.00960330821441932*G0_1_2;
    A[738] = -0.000482775005343988*G0_0_0 + 0.00578025209101624*G0_0_2 - 0.000548014870931594*G0_1_0 + 0.00523223722008438*G0_1_2;
    A[739] = -0.00228991928210669*G0_0_0 - 0.00887914570640245*G0_0_2 - 0.00228991928210686*G0_1_0 - 0.0111690649885095*G0_1_2;
    A[740] = -0.147063705006412*G0_0_0 - 0.0387263842124948*G0_0_1 + 0.0263830016434179*G0_1_0 - 0.108637424175618*G0_1_1;
    A[741] = 0.304226541205705*G0_0_0 + 0.0781312630270955*G0_0_1 + 0.0781312630270955*G0_1_0 + 0.281001149056702*G0_1_1;
    A[742] = -0.204187731314465*G0_0_0 + 0.0153052724667307*G0_0_1 - 0.0364951808094164*G0_1_0 - 0.204187731314466*G0_1_1;
    A[743] = 0.0788945694544649*G0_0_0 - 0.0246345732456837*G0_0_1 + 0.0189782768992838*G0_1_0 + 0.100717304493345*G0_1_1;
    A[744] = 0.00228991928210703*G0_0_0 + 0.00228991928210709*G0_0_1 + 0.00228991928210701*G0_1_0 + 0.00228991928210702*G0_1_1;
    A[745] = 0.00178757231708576*G0_0_0 + 0.00178757231708575*G0_0_1 + 0.00178757231708564*G0_1_0 + 0.00178757231708562*G0_1_1;
    A[746] = 0.00178757231708656*G0_0_0 + 0.00178757231708654*G0_0_1 + 0.00178757231708637*G0_1_0 + 0.00178757231708635*G0_1_1;
    A[747] = 0.00228991928210664*G0_0_0 + 0.00228991928210664*G0_0_1 + 0.00228991928210679*G0_1_0 + 0.00228991928210681*G0_1_1;
    A[748] = 0.0139743792087542*G0_0_0 + 0.0116844599266475*G0_0_1 + 0.0139743792087543*G0_0_2 + 0.0116844599266474*G0_1_0 + 0.00939454064454038*G0_1_1 + 0.0116844599266475*G0_1_2;
    A[749] = -0.026304713804714*G0_0_0 - 0.0131523569023571*G0_0_1 - 0.0263047138047138*G0_0_2 - 0.013152356902357*G0_1_0 + 6.52398655871533e-05*G0_1_1 - 0.013152356902357*G0_1_2;
    A[750] = -0.00523223722008392*G0_0_0 + 0.000548014870931807*G0_0_1 - 0.00523223722008401*G0_0_2 + 0.000548014870931896*G0_1_0 + 6.52398655872308e-05*G0_1_1 + 0.000548014870931832*G0_1_2;
    A[751] = -0.0788945694544649*G0_0_0 - 0.0189782768992838*G0_0_1 - 0.0788945694544649*G0_0_2 - 0.0189782768992838*G0_1_0 + 0.00939454064454043*G0_1_1 - 0.0189782768992839*G0_1_2;
    A[752] = 0.00658922642429591*G0_0_0 + 0.00887914570640265*G0_0_1 + 0.00887914570640269*G0_0_2 + 0.00887914570640251*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.0111690649885096*G0_1_2;
    A[753] = -0.00626302709636041*G0_0_0 - 0.00578025209101598*G0_0_1 - 0.00578025209101612*G0_0_2 - 0.00578025209101605*G0_1_0 - 0.00523223722008453*G0_1_1 - 0.00523223722008468*G0_1_2;
    A[754] = 0.000260959462348421*G0_0_0 - 0.00480165410720941*G0_0_1 - 0.00480165410720942*G0_0_2 - 0.00480165410720934*G0_1_0 - 0.00960330821441885*G0_1_1 - 0.00960330821441883*G0_1_2;
    A[755] = -0.0186912214907007*G0_0_0 + 0.0387263842124947*G0_0_1 + 0.0387263842124948*G0_0_2 + 0.0387263842124947*G0_1_0 + 0.108637424175617*G0_1_1 + 0.108637424175617*G0_1_2;
    A[756] = 0.132306447410613*G0_0_0 - 0.0623693115012556*G0_0_1 + 0.0109602974186307*G0_0_2 + 0.0733296089198859*G0_1_0 - 0.124738623002511*G0_1_1 + 0.146659217839772*G0_1_2;
    A[757] = -0.166948816037356*G0_0_0 + 0.0838658472122014*G0_0_1 + 0.167731694424402*G0_0_2 - 0.0061651672979789*G0_1_0 + 0.182410664181496*G0_1_1 + 0.0777006799142215*G0_1_2;
    A[758] = 0.127087258163647*G0_0_0 - 0.0463855444324186*G0_0_1 - 0.0823327103709027*G0_0_2 + 0.0377738821749238*G0_1_0 - 0.0510175748891018*G0_1_1 + 0.00182671623643916*G0_1_2;
    A[759] = -0.0436454700777618*G0_0_0 - 0.00649136662591512*G0_0_1 - 0.0808321934624013*G0_0_2 - 0.0338921101724923*G0_1_0 - 0.00352295274170281*G0_1_1 - 0.108232937008978*G0_1_2;
    A[760] = 0.0189848008858418*G0_0_0 - 0.0188869410874629*G0_0_1 - 0.0377738821749247*G0_0_2 + 0.00949240044292122*G0_1_0 - 0.0155923278753141*G0_1_1 - 0.00939454064454114*G0_1_2;
    A[761] = 0.0148746893538563*G0_0_0 + 0.00123955744615551*G0_0_1 + 0.0403182369328204*G0_0_2 + 0.0149399292194432*G0_1_0 + 0.00117431758056744*G0_1_1 + 0.0540186087061083*G0_1_2;
    A[762] = -0.0148746893538561*G0_0_0 - 0.0149399292194434*G0_0_1 - 0.0149399292194428*G0_1_0 - 0.0150051690850302*G0_1_1;
    A[763] = -0.0189848008858433*G0_0_0 - 0.00949240044292083*G0_0_1 - 0.00949240044292166*G0_1_0 - 0.000652398655871232*G0_1_1;
    A[764] = -0.127087258163647*G0_0_0 - 0.0377738821749238*G0_0_1 - 0.0377738821749238*G0_1_0 - 0.0150051690850298*G0_1_1;
    A[765] = 0.00450155072550948*G0_0_0 + 0.00192457603481942*G0_0_1 + 0.00192457603481969*G0_1_0 - 0.000652398655869752*G0_1_1;
    A[766] = -0.0189848008858417*G0_0_0 - 0.00949240044292104*G0_0_1 - 0.00949240044292122*G0_1_0 - 0.000652398655871437*G0_1_1;
    A[767] = -0.0148746893538564*G0_0_0 - 0.0149399292194433*G0_0_1 - 0.0149399292194432*G0_1_0 - 0.0150051690850305*G0_1_1;
    A[768] = -0.00130479731174212*G0_0_0 - 0.00123955744615496*G0_0_1 - 0.00123955744615458*G0_1_0 - 0.00117431758056721*G0_1_1;
    A[769] = 0.0088073818542568*G0_0_0 + 0.00649136662591516*G0_0_1 + 0.00649136662591476*G0_1_0 + 0.00352295274170281*G0_1_1;
    A[770] = 0.0476251018785739*G0_0_0 + 0.0623693115012553*G0_0_1 + 0.0623693115012557*G0_1_0 + 0.124738623002511*G0_1_1;
    A[771] = -0.0156575677409001*G0_0_0 - 0.01308059305021*G0_0_1 - 0.0130805930502101*G0_1_0 - 0.0105036183595207*G0_1_1;
    A[772] = 0.00880738185425644*G0_0_0 + 0.00649136662591497*G0_0_1 + 0.00649136662591459*G0_1_0 + 0.00352295274170251*G0_1_1;
    A[773] = -0.00130479731174244*G0_0_0 - 0.00123955744615543*G0_0_1 - 0.00123955744615474*G0_1_0 - 0.00117431758056743*G0_1_1;
    A[774] = -0.025443547578964*G0_0_0 - 0.0390786794866653*G0_0_1 - 0.0403182369328197*G0_0_2 - 0.0390786794866647*G0_1_0 - 0.0528442911255404*G0_1_1 - 0.0540186087061075*G0_1_2;
    A[775] = 0.0371867233846391*G0_0_0 + 0.0743408268364848*G0_0_1 + 0.0808321934624001*G0_0_2 + 0.074340826836485*G0_1_0 + 0.104709984267274*G0_1_1 + 0.108232937008977*G0_1_2;
    A[776] = 0.121346149991983*G0_0_0 - 0.0733296089198859*G0_0_1 - 0.0109602974186308*G0_0_2 - 0.0733296089198856*G0_1_0 - 0.271397840842282*G0_1_1 - 0.146659217839772*G0_1_2;
    A[777] = 0.0567586830607655*G0_0_0 + 0.0188869410874621*G0_0_1 + 0.0377738821749232*G0_0_2 + 0.018886941087461*G0_1_0 - 0.00619778723077391*G0_1_1 + 0.00939454064453987*G0_1_2;
    A[778] = -0.334680510461757*G0_0_0 - 0.0838658472121997*G0_0_1 - 0.167731694424402*G0_0_2 - 0.0838658472121996*G0_1_0 + 0.104709984267276*G0_1_1 - 0.0777006799142214*G0_1_2;
    A[779] = 0.209419968534549*G0_0_0 + 0.0359471659384842*G0_0_1 + 0.0823327103709032*G0_0_2 + 0.0359471659384845*G0_1_0 - 0.0528442911255407*G0_1_1 - 0.00182671623643885*G0_1_2;
    A[780] = 0.0391439193522519*G0_0_0 + 0.0319675341376713*G0_0_1 + 0.0319675341376723*G0_1_0 + 0.0260959462348334*G0_1_1;
    A[781] = -0.156575677409009*G0_0_0 - 0.0903572138381158*G0_0_1 - 0.0903572138381154*G0_1_0 - 0.185933616923198*G0_1_1;
    A[782] = 0.185933616923199*G0_0_0 + 0.0156575677409016*G0_0_1 + 0.0156575677409006*G0_1_0 + 0.0260959462348353*G0_1_1;
    A[783] = 0.0391439193522519*G0_0_0 + 0.0319675341376728*G0_0_1 + 0.0319675341376724*G0_1_0 + 0.0260959462348351*G0_1_1;
    A[784] = -0.00336507226698189*G0_0_0 - 0.00336507226698187*G0_0_1 - 0.00336507226698188*G0_0_2 - 0.00336507226698201*G0_1_0 - 0.00336507226698205*G0_1_1 - 0.00336507226698201*G0_1_2;
    A[785] = -0.0155544887532734*G0_0_0 + 0.0232632312710435*G0_1_0;
    A[786] = -0.03365855145369*G0_0_1 + 0.0214691349673988*G0_1_1;
    A[787] = 0.003365072266982*G0_0_2 + 0.00336507226698203*G0_1_2;
    A[788] = -0.0699110399631228*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0574176057031955*G0_1_1 - 0.0387263842124949*G0_1_2;
    A[789] = 0.00480165410720979*G0_0_1 + 0.00960330821441927*G0_0_2 + 0.00506261356955813*G0_1_1 + 0.0048016541072096*G0_1_2;
    A[790] = -0.000548014870931582*G0_0_1 + 0.00523223722008456*G0_0_2 - 0.000482775005344994*G0_1_1 + 0.00578025209101561*G0_1_2;
    A[791] = -0.00228991928210669*G0_0_1 - 0.0111690649885095*G0_0_2 - 0.00228991928210654*G0_1_1 - 0.00887914570640255*G0_1_2;
    A[792] = 0.0283728175438242*G0_0_0 + 0.0189782768992836*G0_0_2 + 0.0599162925551812*G0_1_0 + 0.0788945694544647*G0_1_2;
    A[793] = -0.000482775005344252*G0_0_0 - 0.000548014870931207*G0_0_2 + 0.00578025209101571*G0_1_0 + 0.00523223722008425*G0_1_2;
    A[794] = 0.0132175967679435*G0_0_0 + 0.0131523569023565*G0_0_2 + 0.0131523569023569*G0_1_0 + 0.0263047138047137*G0_1_2;
    A[795] = -0.00228991928210667*G0_0_0 - 0.0116844599266473*G0_0_2 - 0.00228991928210655*G0_1_0 - 0.0139743792087539*G0_1_2;
    A[796] = 0.100717304493345*G0_0_0 + 0.0189782768992835*G0_0_1 - 0.024634573245684*G0_1_0 + 0.0788945694544648*G0_1_1;
    A[797] = -0.204187731314465*G0_0_0 - 0.0364951808094164*G0_0_1 + 0.0153052724667307*G0_1_0 - 0.204187731314466*G0_1_1;
    A[798] = 0.281001149056702*G0_0_0 + 0.0781312630270954*G0_0_1 + 0.0781312630270954*G0_1_0 + 0.304226541205705*G0_1_1;
    A[799] = -0.108637424175618*G0_0_0 + 0.0263830016434178*G0_0_1 - 0.038726384212495*G0_1_0 - 0.147063705006412*G0_1_1;
    A[800] = 0.00228991928210675*G0_0_0 + 0.00228991928210673*G0_0_1 + 0.00228991928210664*G0_1_0 + 0.00228991928210664*G0_1_1;
    A[801] = 0.00178757231708625*G0_0_0 + 0.00178757231708625*G0_0_1 + 0.00178757231708662*G0_1_0 + 0.00178757231708666*G0_1_1;
    A[802] = 0.00178757231708605*G0_0_0 + 0.00178757231708607*G0_0_1 + 0.0017875723170861*G0_1_0 + 0.00178757231708611*G0_1_1;
    A[803] = 0.00228991928210668*G0_0_0 + 0.00228991928210668*G0_0_1 + 0.00228991928210657*G0_1_0 + 0.00228991928210656*G0_1_1;
    A[804] = 0.0111690649885091*G0_0_0 + 0.00887914570640228*G0_0_1 + 0.0111690649885091*G0_0_2 + 0.0088791457064023*G0_1_0 + 0.00658922642429598*G0_1_1 + 0.00887914570640232*G0_1_2;
    A[805] = -0.00523223722008379*G0_0_0 - 0.0057802520910155*G0_0_1 - 0.00523223722008413*G0_0_2 - 0.0057802520910155*G0_1_0 - 0.00626302709636048*G0_1_1 - 0.00578025209101569*G0_1_2;
    A[806] = -0.00960330821441969*G0_0_0 - 0.00480165410721007*G0_0_1 - 0.00960330821441956*G0_0_2 - 0.00480165410720997*G0_1_0 + 0.000260959462348258*G0_1_1 - 0.00480165410720985*G0_1_2;
    A[807] = 0.108637424175618*G0_0_0 + 0.0387263842124952*G0_0_1 + 0.108637424175618*G0_0_2 + 0.038726384212495*G0_1_0 - 0.0186912214907005*G0_1_1 + 0.0387263842124951*G0_1_2;
    A[808] = 0.00939454064454036*G0_0_0 + 0.0116844599266472*G0_0_1 + 0.0116844599266472*G0_0_2 + 0.0116844599266473*G0_1_0 + 0.0139743792087538*G0_1_1 + 0.0139743792087537*G0_1_2;
    A[809] = 6.52398655872362e-05*G0_0_0 - 0.0131523569023567*G0_0_1 - 0.0131523569023566*G0_0_2 - 0.0131523569023565*G0_1_0 - 0.0263047138047136*G0_1_1 - 0.0263047138047134*G0_1_2;
    A[810] = 6.52398655869473e-05*G0_0_0 + 0.000548014870931487*G0_0_1 + 0.000548014870931473*G0_0_2 + 0.000548014870931205*G0_1_0 - 0.00523223722008452*G0_1_1 - 0.00523223722008452*G0_1_2;
    A[811] = 0.0093945406445408*G0_0_0 - 0.0189782768992834*G0_0_1 - 0.0189782768992834*G0_0_2 - 0.0189782768992834*G0_1_0 - 0.0788945694544645*G0_1_1 - 0.0788945694544645*G0_1_2;
    A[812] = -0.0510175748891018*G0_0_0 + 0.0377738821749238*G0_0_1 + 0.00182671623643827*G0_0_2 - 0.046385544432419*G0_1_0 + 0.127087258163646*G0_1_1 - 0.0823327103709039*G0_1_2;
    A[813] = 0.182410664181495*G0_0_0 - 0.0061651672979801*G0_0_1 + 0.0777006799142206*G0_0_2 + 0.0838658472121996*G0_1_0 - 0.166948816037357*G0_1_1 + 0.167731694424401*G0_1_2;
    A[814] = -0.124738623002512*G0_0_0 + 0.0733296089198851*G0_0_1 + 0.146659217839771*G0_0_2 - 0.0623693115012556*G0_1_0 + 0.132306447410613*G0_1_1 + 0.0109602974186301*G0_1_2;
    A[815] = -0.015592327875314*G0_0_0 + 0.00949240044292113*G0_0_1 - 0.00939454064454046*G0_0_2 - 0.0188869410874615*G0_1_0 + 0.0189848008858425*G0_1_1 - 0.0377738821749238*G0_1_2;
    A[816] = -0.00352295274170222*G0_0_0 - 0.0338921101724915*G0_0_1 - 0.108232937008977*G0_0_2 - 0.00649136662591563*G0_1_0 - 0.0436454700777619*G0_1_1 - 0.080832193462401*G0_1_2;
    A[817] = 0.00117431758056752*G0_0_0 + 0.0149399292194428*G0_0_1 + 0.0540186087061081*G0_0_2 + 0.00123955744615452*G0_1_0 + 0.0148746893538565*G0_1_1 + 0.0403182369328197*G0_1_2;
    A[818] = -0.00117431758056749*G0_0_0 - 0.00123955744615465*G0_0_1 - 0.00123955744615475*G0_1_0 - 0.00130479731174184*G0_1_1;
    A[819] = 0.00352295274170308*G0_0_0 + 0.00649136662591531*G0_0_1 + 0.00649136662591528*G0_1_0 + 0.00880738185425736*G0_1_1;
    A[820] = 0.124738623002512*G0_0_0 + 0.0623693115012554*G0_0_1 + 0.0623693115012556*G0_1_0 + 0.0476251018785733*G0_1_1;
    A[821] = -0.0105036183595215*G0_0_0 - 0.0130805930502111*G0_0_1 - 0.0130805930502117*G0_1_0 - 0.015657567740902*G0_1_1;
    A[822] = 0.00352295274170217*G0_0_0 + 0.00649136662591489*G0_0_1 + 0.00649136662591559*G0_1_0 + 0.00880738185425726*G0_1_1;
    A[823] = -0.00117431758056746*G0_0_0 - 0.00123955744615447*G0_0_1 - 0.00123955744615451*G0_1_0 - 0.0013047973117412*G0_1_1;
    A[824] = -0.0150051690850301*G0_0_0 - 0.0149399292194429*G0_0_1 - 0.0149399292194434*G0_1_0 - 0.0148746893538565*G0_1_1;
    A[825] = -0.000652398655870654*G0_0_0 - 0.00949240044292102*G0_0_1 - 0.00949240044292028*G0_1_0 - 0.0189848008858426*G0_1_1;
    A[826] = -0.0150051690850301*G0_0_0 - 0.0377738821749238*G0_0_1 - 0.037773882174924*G0_1_0 - 0.127087258163646*G0_1_1;
    A[827] = -0.000652398655870979*G0_0_0 + 0.0019245760348186*G0_0_1 + 0.00192457603481845*G0_1_0 + 0.00450155072550883*G0_1_1;
    A[828] = -0.000652398655871016*G0_0_0 - 0.00949240044292107*G0_0_1 - 0.00949240044292079*G0_1_0 - 0.0189848008858423*G0_1_1;
    A[829] = -0.0150051690850297*G0_0_0 - 0.0149399292194427*G0_0_1 - 0.0149399292194431*G0_1_0 - 0.0148746893538563*G0_1_1;
    A[830] = -0.0528442911255405*G0_0_0 - 0.0390786794866649*G0_0_1 - 0.054018608706108*G0_0_2 - 0.0390786794866656*G0_1_0 - 0.025443547578964*G0_1_1 - 0.0403182369328206*G0_1_2;
    A[831] = -0.00619778723077289*G0_0_0 + 0.0188869410874619*G0_0_1 + 0.00939454064454071*G0_0_2 + 0.0188869410874626*G0_1_0 + 0.0567586830607665*G0_1_1 + 0.0377738821749237*G0_1_2;
    A[832] = -0.0528442911255409*G0_0_0 + 0.0359471659384848*G0_0_1 - 0.00182671623643835*G0_0_2 + 0.0359471659384847*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0823327103709041*G0_1_2;
    A[833] = 0.104709984267274*G0_0_0 + 0.0743408268364854*G0_0_1 + 0.108232937008977*G0_0_2 + 0.0743408268364859*G0_1_0 + 0.0371867233846402*G0_1_1 + 0.0808321934624012*G0_1_2;
    A[834] = 0.104709984267275*G0_0_0 - 0.0838658472122006*G0_0_1 - 0.0777006799142206*G0_0_2 - 0.083865847212201*G0_1_0 - 0.334680510461758*G0_1_1 - 0.167731694424401*G0_1_2;
    A[835] = -0.271397840842282*G0_0_0 - 0.0733296089198857*G0_0_1 - 0.146659217839771*G0_0_2 - 0.0733296089198855*G0_1_0 + 0.121346149991982*G0_1_1 - 0.0109602974186305*G0_1_2;
    A[836] = 0.0260959462348351*G0_0_0 + 0.0319675341376732*G0_0_1 + 0.0319675341376726*G0_1_0 + 0.0391439193522533*G0_1_1;
    A[837] = 0.0260959462348343*G0_0_0 + 0.015657567740901*G0_0_1 + 0.0156575677409011*G0_1_0 + 0.185933616923199*G0_1_1;
    A[838] = -0.185933616923198*G0_0_0 - 0.0903572138381151*G0_0_1 - 0.0903572138381149*G0_1_0 - 0.156575677409009*G0_1_1;
    A[839] = 0.0260959462348356*G0_0_0 + 0.031967534137673*G0_0_1 + 0.0319675341376732*G0_1_0 + 0.0391439193522529*G0_1_1;
    A[840] = -0.00480034930989793*G0_0_0 - 0.00480034930989793*G0_0_1 - 0.00480034930989794*G0_0_2 - 0.00480034930989786*G0_1_0 - 0.00480034930989781*G0_1_1 - 0.00480034930989787*G0_1_2;
    A[841] = 0.00480034930989798*G0_0_0 - 0.0105310191030675*G0_1_0;
    A[842] = 0.0372571824394738*G0_0_1 - 0.0178705039816151*G0_1_1;
    A[843] = 0.00480034930989783*G0_0_2 + 0.0048003493098978*G0_1_2;
    A[844] = 0.069911039963123*G0_0_1 + 0.139822079926246*G0_0_2 + 0.0895482395048363*G0_1_1 + 0.0699110399631228*G0_1_2;
    A[845] = -0.038726384212495*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0574176057031957*G0_1_1 - 0.0699110399631228*G0_1_2;
    A[846] = 0.0189782768992839*G0_0_1 + 0.0788945694544646*G0_0_2 + 0.0283728175438248*G0_1_1 + 0.0599162925551813*G0_1_2;
    A[847] = -0.00537576492437605*G0_0_1 - 0.0328352243499809*G0_0_2 - 0.00537576492437617*G0_1_1 - 0.027459459425605*G0_1_2;
    A[848] = -0.00537576492437617*G0_0_0 - 0.00537576492437603*G0_0_2 - 0.0274594594256053*G0_1_0 - 0.032835224349981*G0_1_2;
    A[849] = -0.00228991928210658*G0_0_0 - 0.00228991928210672*G0_0_2 - 0.00887914570640225*G0_1_0 - 0.0111690649885092*G0_1_2;
    A[850] = -0.00228991928210666*G0_0_0 - 0.00228991928210672*G0_0_2 - 0.0116844599266473*G0_1_0 - 0.0139743792087541*G0_1_2;
    A[851] = -0.00537576492437607*G0_0_0 - 0.00537576492437591*G0_0_2 - 0.00537576492437618*G0_1_0 - 0.010751529848752*G0_1_2;
    A[852] = -0.0328352243499813*G0_0_0 - 0.00537576492437609*G0_0_1 + 0.0209159009072198*G0_1_0 - 0.0328352243499811*G0_1_1;
    A[853] = 0.0788945694544649*G0_0_0 + 0.0189782768992838*G0_0_1 - 0.0246345732456837*G0_1_0 + 0.100717304493345*G0_1_1;
    A[854] = -0.108637424175618*G0_0_0 - 0.038726384212495*G0_0_1 + 0.0263830016434179*G0_1_0 - 0.147063705006412*G0_1_1;
    A[855] = 0.139822079926246*G0_0_0 + 0.0699110399631229*G0_0_1 + 0.0699110399631229*G0_1_0 + 0.179096479009672*G0_1_1;
    A[856] = 0.00537576492437609*G0_0_0 + 0.00537576492437611*G0_0_1 + 0.00537576492437623*G0_1_0 + 0.00537576492437626*G0_1_1;
    A[857] = 0.00228991928210653*G0_0_0 + 0.00228991928210659*G0_0_1 + 0.0022899192821062*G0_1_0 + 0.00228991928210622*G0_1_1;
    A[858] = 0.00228991928210683*G0_0_0 + 0.00228991928210681*G0_0_1 + 0.00228991928210687*G0_1_0 + 0.00228991928210687*G0_1_1;
    A[859] = 0.00537576492437603*G0_0_0 + 0.00537576492437603*G0_0_1 + 0.00537576492437613*G0_1_0 + 0.00537576492437614*G0_1_1;
    A[860] = 0.0328352243499811*G0_0_0 + 0.0274594594256052*G0_0_1 + 0.0328352243499811*G0_0_2 + 0.0274594594256052*G0_1_0 + 0.0220836945012289*G0_1_1 + 0.0274594594256052*G0_1_2;
    A[861] = -0.0788945694544648*G0_0_0 - 0.0599162925551811*G0_0_1 - 0.0788945694544647*G0_0_2 - 0.059916292555181*G0_1_0 - 0.0315434750113565*G0_1_1 - 0.059916292555181*G0_1_2;
    A[862] = 0.108637424175618*G0_0_0 + 0.0699110399631228*G0_0_1 + 0.108637424175618*G0_0_2 + 0.0699110399631229*G0_1_0 + 0.012493434259927*G0_1_1 + 0.0699110399631227*G0_1_2;
    A[863] = -0.139822079926246*G0_0_0 - 0.0699110399631228*G0_0_1 - 0.139822079926246*G0_0_2 - 0.0699110399631229*G0_1_0 + 0.0196371995417134*G0_1_1 - 0.0699110399631228*G0_1_2;
    A[864] = 0.00537576492437593*G0_0_1 + 0.00537576492437595*G0_0_2 + 0.00537576492437594*G0_1_0 + 0.0107515298487521*G0_1_1 + 0.0107515298487522*G0_1_2;
    A[865] = 0.0022899192821069*G0_0_1 + 0.00228991928210686*G0_0_2 + 0.00228991928210677*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.0139743792087541*G0_1_2;
    A[866] = 0.0022899192821068*G0_0_1 + 0.00228991928210686*G0_0_2 + 0.00228991928210685*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.0111690649885096*G0_1_2;
    A[867] = 0.00537576492437588*G0_0_1 + 0.005375764924376*G0_0_2 + 0.0053757649243758*G0_1_0 + 0.0328352243499808*G0_1_1 + 0.032835224349981*G0_1_2;
    A[868] = 0.0118736555368504*G0_0_0 - 0.00939454064454051*G0_0_1 + 0.0118736555368497*G0_0_2 + 0.0264873854283584*G0_1_0 - 0.0641307878721066*G0_1_1 + 0.0477555816097477*G0_1_2;
    A[869] = -0.0429604514890968*G0_0_0 - 0.000685018588664029*G0_0_1 - 0.0429604514890962*G0_0_2 - 0.00704590548340546*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0493213383838371*G0_1_2;
    A[870] = 0.124738623002512*G0_0_0 + 0.0284445813959708*G0_0_1 + 0.124738623002512*G0_0_2 + 0.0962940416065415*G0_1_0 - 0.0644569872000423*G0_1_1 + 0.192588083213082*G0_1_2;
    A[871] = 0.00254435475789588*G0_0_0 - 0.00329461321214786*G0_0_1 + 0.00254435475789675*G0_0_2 + 0.00844856259352731*G0_1_0 - 0.0292600797158085*G0_1_1 + 0.0142875305635727*G0_1_2;
    A[872] = -0.0429604514890975*G0_0_0 - 0.000685018588665241*G0_0_1 - 0.0429604514890978*G0_0_2 - 0.0422754329004326*G0_1_0 + 0.0439716694056964*G0_1_1 - 0.0845508658008662*G0_1_2;
    A[873] = 0.0118736555368502*G0_0_0 - 0.00939454064454032*G0_0_1 + 0.0118736555368498*G0_0_2 + 0.0212681961813909*G0_1_0 - 0.0187890812890813*G0_1_1 + 0.0425363923627811*G0_1_2;
    A[874] = -0.0118736555368504*G0_0_0 - 0.0212681961813909*G0_0_1 - 0.0212681961813909*G0_1_0 - 0.0306627368259315*G0_1_1;
    A[875] = 0.0429604514890975*G0_0_0 + 0.0422754329004331*G0_0_1 + 0.042275432900433*G0_1_0 + 0.0642612676032812*G0_1_1;
    A[876] = -0.124738623002512*G0_0_0 - 0.0962940416065414*G0_0_1 - 0.0962940416065415*G0_1_0 - 0.128522535206562*G0_1_1;
    A[877] = -0.00254435475789594*G0_0_0 - 0.00583896797004387*G0_0_1 - 0.00583896797004379*G0_1_0 - 0.00913358118219116*G0_1_1;
    A[878] = 0.0429604514890975*G0_0_0 + 0.0422754329004324*G0_0_1 + 0.0422754329004327*G0_1_0 + 0.0642612676032806*G0_1_1;
    A[879] = -0.0118736555368502*G0_0_0 - 0.0212681961813907*G0_0_1 - 0.0212681961813908*G0_1_0 - 0.0306627368259316*G0_1_1;
    A[880] = 0.0093945406445404*G0_0_1 + 0.00939454064454073*G0_1_0 + 0.0187890812890812*G0_1_1;
    A[881] = 0.00329461321214795*G0_0_1 + 0.00329461321214755*G0_1_0 + 0.029260079715809*G0_1_1;
    A[882] = 0.00939454064454073*G0_0_1 + 0.00939454064454088*G0_1_0 + 0.0641307878721067*G0_1_1;
    A[883] = 0.00329461321214799*G0_0_1 + 0.00329461321214859*G0_1_0 + 0.00658922642429605*G0_1_1;
    A[884] = 0.00329461321214779*G0_0_1 + 0.0032946132121475*G0_1_0 + 0.0292600797158084*G0_1_1;
    A[885] = 0.00939454064454036*G0_0_1 + 0.00939454064454039*G0_1_0 + 0.0187890812890811*G0_1_1;
    A[886] = -0.0212681961813903*G0_0_1 - 0.0118736555368497*G0_0_2 - 0.0212681961813899*G0_1_0 - 0.0613254736518617*G0_1_1 - 0.0425363923627801*G0_1_2;
    A[887] = -0.00583896797004485*G0_0_1 - 0.00254435475789661*G0_0_2 - 0.00583896797004506*G0_1_0 - 0.0435476102793817*G0_1_1 - 0.0142875305635724*G0_1_2;
    A[888] = -0.0212681961813903*G0_0_1 - 0.0118736555368498*G0_0_2 - 0.0212681961813901*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0477555816097479*G0_1_2;
    A[889] = 0.0422754329004324*G0_0_1 + 0.0429604514890967*G0_0_2 + 0.0422754329004316*G0_1_0 + 0.128522535206561*G0_1_1 + 0.0845508658008644*G0_1_2;
    A[890] = 0.0422754329004327*G0_0_1 + 0.0429604514890965*G0_0_2 + 0.0422754329004329*G0_1_0 + 0.176636936077039*G0_1_1 + 0.0493213383838373*G0_1_2;
    A[891] = -0.0962940416065408*G0_0_1 - 0.124738623002511*G0_0_2 - 0.0962940416065408*G0_1_0 - 0.257045070413124*G0_1_1 - 0.192588083213082*G0_1_2;
    A[892] = -0.00260959462348326*G0_0_1 - 0.00260959462348293*G0_1_0 - 0.0505608958299929*G0_1_1;
    A[893] = -0.00260959462348408*G0_0_1 - 0.00260959462348462*G0_1_0 - 0.156575677409011*G0_1_1;
    A[894] = -0.0352295274170279*G0_0_1 - 0.0352295274170276*G0_1_0 + 0.096228801740954*G0_1_1;
    A[895] = -0.00260959462348269*G0_0_1 - 0.00260959462348341*G0_1_0 - 0.0505608958299922*G0_1_1;
    A[896] = -0.017870503981615*G0_0_0 - 0.0178705039816149*G0_0_1 - 0.0178705039816149*G0_0_2 - 0.0178705039816149*G0_1_0 - 0.0178705039816149*G0_1_1 - 0.0178705039816149*G0_1_2 - 0.0551276864210886*G0_2_0 - 0.0551276864210886*G0_2_1 - 0.0551276864210887*G0_2_2;
    A[897] = -0.00480034930989795*G0_0_0 - 0.00480034930989794*G0_1_0;
    A[898] = -0.00480034930989796*G0_0_1 - 0.00480034930989797*G0_1_1;
    A[899] = 0.0105310191030676*G0_0_2 + 0.0105310191030676*G0_1_2 + 0.0153313684129656*G0_2_2;
    A[900] = 0.0107515298487522*G0_0_1 + 0.00537576492437618*G0_0_2 + 0.0107515298487522*G0_1_1 + 0.0053757649243762*G0_1_2 + 0.00537576492437623*G0_2_1;
    A[901] = 0.0139743792087538*G0_0_1 + 0.0116844599266471*G0_0_2 + 0.0139743792087538*G0_1_1 + 0.0116844599266471*G0_1_2 + 0.011684459926647*G0_2_1 + 0.00939454064454029*G0_2_2;
    A[902] = 0.0111690649885095*G0_0_1 + 0.00887914570640276*G0_0_2 + 0.0111690649885096*G0_1_1 + 0.00887914570640279*G0_1_2 + 0.00887914570640284*G0_2_1 + 0.00658922642429614*G0_2_2;
    A[903] = 0.0328352243499809*G0_0_1 + 0.027459459425605*G0_0_2 + 0.032835224349981*G0_1_1 + 0.027459459425605*G0_1_2 + 0.0274594594256049*G0_2_1 + 0.0220836945012289*G0_2_2;
    A[904] = 0.010751529848752*G0_0_0 + 0.00537576492437586*G0_0_2 + 0.010751529848752*G0_1_0 + 0.00537576492437582*G0_1_2 + 0.00537576492437593*G0_2_0;
    A[905] = 0.013974379208754*G0_0_0 + 0.0116844599266474*G0_0_2 + 0.013974379208754*G0_1_0 + 0.0116844599266475*G0_1_2 + 0.0116844599266473*G0_2_0 + 0.00939454064454058*G0_2_2;
    A[906] = 0.0111690649885094*G0_0_0 + 0.00887914570640233*G0_0_2 + 0.0111690649885093*G0_1_0 + 0.0088791457064023*G0_1_2 + 0.00887914570640274*G0_2_0 + 0.00658922642429586*G0_2_2;
    A[907] = 0.0328352243499809*G0_0_0 + 0.027459459425605*G0_0_2 + 0.0328352243499809*G0_1_0 + 0.027459459425605*G0_1_2 + 0.0274594594256049*G0_2_0 + 0.0220836945012289*G0_2_2;
    A[908] = 0.00537576492437578*G0_0_0 + 0.00537576492437578*G0_0_1 + 0.0053757649243757*G0_1_0 + 0.00537576492437574*G0_1_1;
    A[909] = 0.00228991928210703*G0_0_0 + 0.00228991928210701*G0_0_1 + 0.00228991928210709*G0_1_0 + 0.00228991928210702*G0_1_1;
    A[910] = 0.00228991928210675*G0_0_0 + 0.00228991928210664*G0_0_1 + 0.00228991928210673*G0_1_0 + 0.00228991928210664*G0_1_1;
    A[911] = 0.0053757649243761*G0_0_0 + 0.00537576492437623*G0_0_1 + 0.00537576492437611*G0_1_0 + 0.00537576492437626*G0_1_1;
    A[912] = 0.179096479009671*G0_0_0 + 0.179096479009672*G0_0_1 + 0.109185439046549*G0_0_2 + 0.179096479009672*G0_1_0 + 0.179096479009672*G0_1_1 + 0.109185439046549*G0_1_2 + 0.109185439046549*G0_2_0 + 0.109185439046549*G0_2_1 + 0.179096479009672*G0_2_2;
    A[913] = -0.147063705006412*G0_0_0 - 0.147063705006412*G0_0_1 - 0.17344670664983*G0_0_2 - 0.147063705006412*G0_1_0 - 0.147063705006412*G0_1_1 - 0.17344670664983*G0_1_2 - 0.108337320793917*G0_2_0 - 0.108337320793917*G0_2_1 - 0.243357746612953*G0_2_2;
    A[914] = 0.100717304493345*G0_0_0 + 0.100717304493345*G0_0_1 + 0.125351877739029*G0_0_2 + 0.100717304493345*G0_1_0 + 0.100717304493345*G0_1_1 + 0.12535187773903*G0_1_2 + 0.0817390275940617*G0_2_0 + 0.0817390275940617*G0_2_1 + 0.185268170294211*G0_2_2;
    A[915] = -0.032835224349981*G0_0_0 - 0.032835224349981*G0_0_1 - 0.0537511252572013*G0_0_2 - 0.032835224349981*G0_1_0 - 0.032835224349981*G0_1_1 - 0.0537511252572014*G0_1_2 - 0.027459459425605*G0_2_0 - 0.027459459425605*G0_2_1 - 0.0812105846828065*G0_2_2;
    A[916] = 0.0895482395048356*G0_0_0 + 0.0196371995417132*G0_0_1 + 0.0895482395048356*G0_0_2 + 0.0895482395048356*G0_1_0 + 0.0196371995417132*G0_1_1 + 0.0895482395048357*G0_1_2 + 0.0196371995417133*G0_2_0 + 0.0895482395048358*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[917] = -0.0574176057031953*G0_0_0 + 0.0124934342599272*G0_0_1 - 0.0574176057031955*G0_0_2 - 0.0574176057031954*G0_1_0 + 0.0124934342599273*G0_1_1 - 0.0574176057031955*G0_1_2 - 0.0186912214907007*G0_2_0 - 0.0574176057031954*G0_2_1 - 0.0186912214907007*G0_2_2;
    A[918] = 0.0283728175438243*G0_0_0 - 0.0315434750113567*G0_0_1 + 0.0283728175438243*G0_0_2 + 0.0283728175438243*G0_1_0 - 0.0315434750113568*G0_1_1 + 0.0283728175438244*G0_1_2 + 0.00939454064454081*G0_2_0 + 0.0283728175438242*G0_2_1 + 0.00939454064454083*G0_2_2;
    A[919] = -0.00537576492437607*G0_0_0 + 0.0220836945012291*G0_0_1 - 0.00537576492437608*G0_0_2 - 0.00537576492437609*G0_1_0 + 0.0220836945012291*G0_1_1 - 0.0053757649243761*G0_1_2 - 0.00537576492437601*G0_2_1;
    A[920] = 0.0196371995417133*G0_0_0 + 0.0895482395048356*G0_0_1 + 0.0895482395048356*G0_0_2 + 0.0196371995417133*G0_1_0 + 0.0895482395048356*G0_1_1 + 0.0895482395048356*G0_1_2 + 0.0895482395048357*G0_2_0 + 0.0196371995417132*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[921] = 0.0124934342599272*G0_0_0 - 0.0574176057031952*G0_0_1 - 0.0574176057031951*G0_0_2 + 0.0124934342599273*G0_1_0 - 0.0574176057031951*G0_1_1 - 0.057417605703195*G0_1_2 - 0.0574176057031951*G0_2_0 - 0.0186912214907006*G0_2_1 - 0.0186912214907006*G0_2_2;
    A[922] = -0.0315434750113567*G0_0_0 + 0.0283728175438241*G0_0_1 + 0.0283728175438241*G0_0_2 - 0.0315434750113568*G0_1_0 + 0.0283728175438241*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.028372817543824*G0_2_0 + 0.00939454064454065*G0_2_1 + 0.00939454064454067*G0_2_2;
    A[923] = 0.0220836945012291*G0_0_0 - 0.00537576492437569*G0_0_1 - 0.00537576492437572*G0_0_2 + 0.0220836945012291*G0_1_0 - 0.00537576492437564*G0_1_1 - 0.00537576492437567*G0_1_2 - 0.00537576492437595*G0_2_0;
    A[924] = 0.0187890812890813*G0_0_0 + 0.0187890812890807*G0_0_1 + 0.00939454064454025*G0_0_2 + 0.0187890812890813*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064454017*G0_1_2 + 0.00939454064454085*G0_2_0 + 0.00939454064454033*G0_2_1;
    A[925] = 0.00658922642429568*G0_0_0 + 0.00658922642429654*G0_0_1 + 0.00329461321214871*G0_0_2 + 0.00658922642429564*G0_1_0 + 0.00658922642429654*G0_1_1 + 0.00329461321214873*G0_1_2 + 0.00329461321214729*G0_2_0 + 0.0032946132121478*G0_2_1;
    A[926] = 0.0187890812890812*G0_0_0 + 0.0187890812890806*G0_0_1 + 0.00939454064454053*G0_0_2 + 0.0187890812890813*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064454058*G0_1_2 + 0.00939454064454066*G0_2_0 + 0.00939454064454026*G0_2_1;
    A[927] = 0.0292600797158083*G0_0_0 + 0.0292600797158087*G0_0_1 + 0.0259654665036606*G0_0_2 + 0.0292600797158083*G0_1_0 + 0.0292600797158087*G0_1_1 + 0.0259654665036606*G0_1_2 + 0.0259654665036608*G0_2_0 + 0.0259654665036609*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[928] = 0.0292600797158081*G0_0_0 + 0.029260079715808*G0_0_1 + 0.0259654665036596*G0_0_2 + 0.0292600797158082*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.0259654665036597*G0_1_2 + 0.0259654665036606*G0_2_0 + 0.0259654665036607*G0_2_1 + 0.022670853291512*G0_2_2;
    A[929] = 0.0641307878721071*G0_0_0 + 0.0641307878721068*G0_0_1 + 0.0547362472275664*G0_0_2 + 0.0641307878721072*G0_1_0 + 0.0641307878721069*G0_1_1 + 0.0547362472275665*G0_1_2 + 0.0547362472275665*G0_2_0 + 0.0547362472275664*G0_2_1 + 0.0453417065830263*G0_2_2;
    A[930] = -0.0644569872000419*G0_0_0 - 0.257045070413123*G0_0_1 - 0.160751028806582*G0_0_2 - 0.0644569872000421*G0_1_0 - 0.257045070413124*G0_1_1 - 0.160751028806582*G0_1_2 - 0.0929015685960121*G0_2_0 - 0.160751028806583*G0_2_1 - 0.0644569872000418*G0_2_2;
    A[931] = 0.0439716694056969*G0_0_0 + 0.128522535206562*G0_0_1 + 0.0862471023061292*G0_0_2 + 0.0439716694056968*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0862471023061293*G0_1_2 + 0.0446566879943614*G0_2_0 + 0.0862471023061297*G0_2_1 + 0.0439716694056968*G0_2_2;
    A[932] = -0.0187890812890812*G0_0_0 - 0.0613254736518626*G0_0_1 - 0.0400572774704717*G0_0_2 - 0.0187890812890813*G0_1_0 - 0.0613254736518626*G0_1_1 - 0.0400572774704717*G0_1_2 - 0.00939454064454071*G0_2_0 - 0.040057277470472*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[933] = 0.127315597693201*G0_0_0 + 0.176636936077038*G0_0_1 + 0.134361503176606*G0_0_2 + 0.127315597693201*G0_1_0 + 0.176636936077038*G0_1_1 + 0.134361503176606*G0_1_2 + 0.128000616281866*G0_2_0 + 0.134361503176606*G0_2_1 + 0.0920860702761734*G0_2_2;
    A[934] = -0.0292600797158082*G0_0_0 - 0.0435476102793805*G0_0_1 - 0.0377086423093365*G0_0_2 - 0.0292600797158083*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.0377086423093365*G0_1_2 - 0.0259654665036607*G0_2_0 - 0.0377086423093361*G0_2_1 - 0.0318696743392924*G0_2_2;
    A[935] = -0.064130787872107*G0_0_0 - 0.111886369481855*G0_0_1 - 0.0906181733004641*G0_0_2 - 0.0641307878721071*G0_1_0 - 0.111886369481855*G0_1_1 - 0.0906181733004642*G0_1_2 - 0.0547362472275665*G0_2_0 - 0.0906181733004646*G0_2_1 - 0.0693499771190736*G0_2_2;
    A[936] = -0.257045070413123*G0_0_0 - 0.0644569872000416*G0_0_1 - 0.160751028806582*G0_0_2 - 0.257045070413123*G0_1_0 - 0.0644569872000416*G0_1_1 - 0.160751028806582*G0_1_2 - 0.160751028806583*G0_2_0 - 0.092901568596012*G0_2_1 - 0.0644569872000415*G0_2_2;
    A[937] = 0.128522535206562*G0_0_0 + 0.0439716694056962*G0_0_1 + 0.0862471023061288*G0_0_2 + 0.128522535206562*G0_1_0 + 0.0439716694056962*G0_1_1 + 0.0862471023061288*G0_1_2 + 0.0862471023061294*G0_2_0 + 0.044656687994361*G0_2_1 + 0.0439716694056966*G0_2_2;
    A[938] = -0.0613254736518619*G0_0_0 - 0.0187890812890809*G0_0_1 - 0.0400572774704717*G0_0_2 - 0.0613254736518618*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.0400572774704717*G0_1_2 - 0.0400572774704715*G0_2_0 - 0.00939454064454036*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[939] = 0.176636936077038*G0_0_0 + 0.1273155976932*G0_0_1 + 0.134361503176605*G0_0_2 + 0.176636936077038*G0_1_0 + 0.1273155976932*G0_1_1 + 0.134361503176605*G0_1_2 + 0.134361503176606*G0_2_0 + 0.128000616281865*G0_2_1 + 0.0920860702761731*G0_2_2;
    A[940] = -0.0435476102793802*G0_0_0 - 0.0292600797158086*G0_0_1 - 0.0377086423093359*G0_0_2 - 0.0435476102793802*G0_1_0 - 0.0292600797158086*G0_1_1 - 0.0377086423093359*G0_1_2 - 0.0377086423093363*G0_2_0 - 0.025965466503661*G0_2_1 - 0.031869674339292*G0_2_2;
    A[941] = -0.111886369481854*G0_0_0 - 0.0641307878721066*G0_0_1 - 0.0906181733004638*G0_0_2 - 0.111886369481854*G0_1_0 - 0.0641307878721067*G0_1_1 - 0.0906181733004639*G0_1_2 - 0.0906181733004644*G0_2_0 - 0.0547362472275663*G0_2_1 - 0.0693499771190735*G0_2_2;
    A[942] = -0.128522535206561*G0_0_0 - 0.128522535206562*G0_0_1 - 0.0322284936000209*G0_0_2 - 0.128522535206561*G0_1_0 - 0.128522535206562*G0_1_1 - 0.0322284936000209*G0_1_2 - 0.0322284936000213*G0_2_0 - 0.0322284936000212*G0_2_1 - 0.0606730749959916*G0_2_2;
    A[943] = 0.0642612676032808*G0_0_0 + 0.0642612676032804*G0_0_1 + 0.0219858347028478*G0_0_2 + 0.0642612676032807*G0_1_0 + 0.0642612676032803*G0_1_1 + 0.0219858347028479*G0_1_2 + 0.0219858347028487*G0_2_0 + 0.0219858347028485*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[944] = -0.0306627368259308*G0_0_0 - 0.0306627368259312*G0_0_1 - 0.00939454064454025*G0_0_2 - 0.0306627368259307*G0_1_0 - 0.0306627368259311*G0_1_1 - 0.00939454064454016*G0_1_2 - 0.00939454064454057*G0_2_0 - 0.0093945406445408*G0_2_1;
    A[945] = 0.0642612676032805*G0_0_0 + 0.064261267603281*G0_0_1 + 0.0219858347028483*G0_0_2 + 0.0642612676032805*G0_1_0 + 0.0642612676032811*G0_1_1 + 0.0219858347028482*G0_1_2 + 0.0219858347028486*G0_2_0 + 0.0219858347028488*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[946] = -0.00913358118219224*G0_0_0 - 0.00913358118219186*G0_0_1 - 0.00329461321214842*G0_0_2 - 0.00913358118219226*G0_1_0 - 0.0091335811821919*G0_1_1 - 0.00329461321214844*G0_1_2 - 0.00329461321214844*G0_2_0 - 0.00329461321214817*G0_2_1;
    A[947] = -0.0306627368259311*G0_0_0 - 0.0306627368259313*G0_0_1 - 0.00939454064454072*G0_0_2 - 0.0306627368259311*G0_1_0 - 0.0306627368259313*G0_1_1 - 0.00939454064454078*G0_1_2 - 0.00939454064454072*G0_2_0 - 0.00939454064454088*G0_2_1;
    A[948] = 0.0962288017409521*G0_0_0 + 0.0962288017409528*G0_0_1 + 0.13145832915798*G0_0_2 + 0.0962288017409524*G0_1_0 + 0.0962288017409527*G0_1_1 + 0.13145832915798*G0_1_2 + 0.131458329157981*G0_2_0 + 0.131458329157981*G0_2_1 + 0.166687856575009*G0_2_2;
    A[949] = -0.0505608958299914*G0_0_0 - 0.0505608958299923*G0_0_1 - 0.0479513012065087*G0_0_2 - 0.0505608958299916*G0_1_0 - 0.0505608958299924*G0_1_1 - 0.0479513012065088*G0_1_2 - 0.0479513012065098*G0_2_0 - 0.0479513012065089*G0_2_1 - 0.0453417065830257*G0_2_2;
    A[950] = -0.0505608958299925*G0_0_0 - 0.0505608958299917*G0_0_1 - 0.0479513012065078*G0_0_2 - 0.0505608958299924*G0_1_0 - 0.0505608958299918*G0_1_1 - 0.0479513012065077*G0_1_2 - 0.0479513012065086*G0_2_0 - 0.0479513012065091*G0_2_1 - 0.0453417065830252*G0_2_2;
    A[951] = -0.156575677409009*G0_0_0 - 0.156575677409008*G0_0_1 - 0.153966082785526*G0_0_2 - 0.156575677409009*G0_1_0 - 0.156575677409008*G0_1_1 - 0.153966082785526*G0_1_2 - 0.153966082785527*G0_2_0 - 0.153966082785526*G0_2_1 - 0.151356488162044*G0_2_2;
    A[952] = 0.0214691349673987*G0_0_0 + 0.0214691349673987*G0_0_1 + 0.0214691349673986*G0_0_2 + 0.0214691349673986*G0_1_0 + 0.0214691349673987*G0_1_1 + 0.0214691349673986*G0_1_2 + 0.0551276864210888*G0_2_0 + 0.0551276864210887*G0_2_1 + 0.0551276864210887*G0_2_2;
    A[953] = -0.00336507226698182*G0_0_0 - 0.00336507226698181*G0_1_0;
    A[954] = -0.00336507226698188*G0_0_1 - 0.00336507226698184*G0_1_1;
    A[955] = -0.0232632312710436*G0_0_2 - 0.0232632312710436*G0_1_2 - 0.0388177200243171*G0_2_2;
    A[956] = 0.0139743792087537*G0_0_1 + 0.00228991928210654*G0_0_2 + 0.0139743792087537*G0_1_1 + 0.00228991928210661*G0_1_2 + 0.00228991928210648*G0_2_1;
    A[957] = -0.0263047138047132*G0_0_1 - 0.0131523569023567*G0_0_2 - 0.0263047138047133*G0_1_1 - 0.0131523569023568*G0_1_2 - 0.0131523569023564*G0_2_1 + 6.52398655871952e-05*G0_2_2;
    A[958] = -0.00523223722008456*G0_0_1 - 0.00578025209101621*G0_0_2 - 0.00523223722008459*G0_1_1 - 0.00578025209101618*G0_1_2 - 0.00578025209101627*G0_2_1 - 0.00626302709636065*G0_2_2;
    A[959] = -0.0788945694544645*G0_0_1 - 0.0599162925551807*G0_0_2 - 0.0788945694544647*G0_1_1 - 0.0599162925551809*G0_1_2 - 0.059916292555181*G0_2_1 - 0.0315434750113566*G0_2_2;
    A[960] = 0.0139743792087542*G0_0_0 + 0.00228991928210716*G0_0_2 + 0.0139743792087543*G0_1_0 + 0.00228991928210723*G0_1_2 + 0.00228991928210696*G0_2_0;
    A[961] = -0.0263047138047137*G0_0_0 - 0.0131523569023571*G0_0_2 - 0.0263047138047138*G0_1_0 - 0.0131523569023572*G0_1_2 - 0.0131523569023569*G0_2_0 + 6.5239865586878e-05*G0_2_2;
    A[962] = -0.00523223722008415*G0_0_0 - 0.00578025209101551*G0_0_2 - 0.00523223722008403*G0_1_0 - 0.00578025209101538*G0_1_2 - 0.00578025209101601*G0_2_0 - 0.00626302709636038*G0_2_2;
    A[963] = -0.0788945694544645*G0_0_0 - 0.0599162925551808*G0_0_2 - 0.0788945694544647*G0_1_0 - 0.0599162925551809*G0_1_2 - 0.059916292555181*G0_2_0 - 0.0315434750113566*G0_2_2;
    A[964] = 0.00228991928210711*G0_0_0 + 0.00228991928210726*G0_0_1 + 0.0022899192821072*G0_1_0 + 0.00228991928210734*G0_1_1;
    A[965] = 0.00178757231708576*G0_0_0 + 0.00178757231708564*G0_0_1 + 0.00178757231708575*G0_1_0 + 0.00178757231708561*G0_1_1;
    A[966] = 0.00178757231708625*G0_0_0 + 0.00178757231708662*G0_0_1 + 0.00178757231708625*G0_1_0 + 0.00178757231708666*G0_1_1;
    A[967] = 0.00228991928210653*G0_0_0 + 0.0022899192821062*G0_0_1 + 0.00228991928210659*G0_1_0 + 0.00228991928210622*G0_1_1;
    A[968] = -0.147063705006412*G0_0_0 - 0.147063705006412*G0_0_1 - 0.108337320793917*G0_0_2 - 0.147063705006412*G0_1_0 - 0.147063705006412*G0_1_1 - 0.108337320793917*G0_1_2 - 0.17344670664983*G0_2_0 - 0.17344670664983*G0_2_1 - 0.243357746612953*G0_2_2;
    A[969] = 0.304226541205705*G0_0_0 + 0.304226541205705*G0_0_1 + 0.226095278178609*G0_0_2 + 0.304226541205705*G0_1_0 + 0.304226541205705*G0_1_1 + 0.226095278178609*G0_1_2 + 0.226095278178609*G0_2_0 + 0.226095278178609*G0_2_1 + 0.428965164208217*G0_2_2;
    A[970] = -0.204187731314465*G0_0_0 - 0.204187731314465*G0_0_1 - 0.219493003781196*G0_0_2 - 0.204187731314466*G0_1_0 - 0.204187731314466*G0_1_1 - 0.219493003781196*G0_1_2 - 0.167692550505049*G0_2_0 - 0.167692550505049*G0_2_1 - 0.387185554286247*G0_2_2;
    A[971] = 0.0788945694544647*G0_0_0 + 0.0788945694544647*G0_0_1 + 0.103529142700149*G0_0_2 + 0.0788945694544649*G0_1_0 + 0.0788945694544649*G0_1_1 + 0.103529142700149*G0_1_2 + 0.0599162925551811*G0_2_0 + 0.0599162925551811*G0_2_1 + 0.185268170294211*G0_2_2;
    A[972] = -0.0574176057031954*G0_0_0 - 0.0186912214907006*G0_0_1 - 0.0574176057031954*G0_0_2 - 0.0574176057031954*G0_1_0 - 0.0186912214907006*G0_1_1 - 0.0574176057031954*G0_1_2 + 0.0124934342599271*G0_2_0 - 0.0574176057031956*G0_2_1 + 0.0124934342599272*G0_2_2;
    A[973] = 0.00506261356955813*G0_0_0 + 0.000260959462348406*G0_0_1 + 0.00506261356955809*G0_0_2 + 0.00506261356955817*G0_1_0 + 0.000260959462348342*G0_1_1 + 0.00506261356955812*G0_1_2 + 0.00026095946234874*G0_2_0 + 0.00506261356955821*G0_2_1 + 0.000260959462348679*G0_2_2;
    A[974] = -0.000482775005344526*G0_0_0 - 0.00626302709636012*G0_0_1 - 0.000482775005344508*G0_0_2 - 0.00048277500534448*G0_1_0 - 0.00626302709636004*G0_1_1 - 0.000482775005344461*G0_1_2 + 6.52398655868641e-05*G0_2_0 - 0.000482775005344674*G0_2_1 + 6.52398655868567e-05*G0_2_2;
    A[975] = -0.00228991928210659*G0_0_0 + 0.00658922642429559*G0_0_1 - 0.00228991928210661*G0_0_2 - 0.00228991928210665*G0_1_0 + 0.00658922642429551*G0_1_1 - 0.00228991928210667*G0_1_2 - 0.00228991928210669*G0_2_1;
    A[976] = -0.0186912214907005*G0_0_0 - 0.0574176057031952*G0_0_1 - 0.0574176057031952*G0_0_2 - 0.0186912214907005*G0_1_0 - 0.0574176057031953*G0_1_1 - 0.0574176057031952*G0_1_2 - 0.0574176057031954*G0_2_0 + 0.0124934342599274*G0_2_1 + 0.0124934342599274*G0_2_2;
    A[977] = 0.000260959462348309*G0_0_0 + 0.00506261356955785*G0_0_1 + 0.00506261356955779*G0_0_2 + 0.000260959462348255*G0_1_0 + 0.00506261356955777*G0_1_1 + 0.00506261356955771*G0_1_2 + 0.00506261356955769*G0_2_0 + 0.000260959462348515*G0_2_1 + 0.000260959462348507*G0_2_2;
    A[978] = -0.00626302709636015*G0_0_0 - 0.000482775005344257*G0_0_1 - 0.000482775005344229*G0_0_2 - 0.00626302709636009*G0_1_0 - 0.000482775005344245*G0_1_1 - 0.000482775005344216*G0_1_2 - 0.000482775005344205*G0_2_0 + 6.52398655872626e-05*G0_2_1 + 6.52398655872247e-05*G0_2_2;
    A[979] = 0.00658922642429548*G0_0_0 - 0.00228991928210733*G0_0_1 - 0.00228991928210726*G0_0_2 + 0.00658922642429546*G0_1_0 - 0.00228991928210741*G0_1_1 - 0.00228991928210734*G0_1_2 - 0.00228991928210696*G0_2_0;
    A[980] = -0.014874689353856*G0_0_0 - 0.0148746893538556*G0_0_1 + 6.52398655873449e-05*G0_0_2 - 0.014874689353856*G0_1_0 - 0.0148746893538555*G0_1_1 + 6.52398655875021e-05*G0_1_2 + 6.52398655867337e-05*G0_2_0 + 6.5239865587493e-05*G0_2_1;
    A[981] = 0.00450155072550948*G0_0_0 + 0.00450155072550866*G0_0_1 + 0.00257697469068967*G0_0_2 + 0.00450155072550962*G0_1_0 + 0.00450155072550874*G0_1_1 + 0.00257697469068972*G0_1_2 + 0.00257697469069074*G0_2_0 + 0.00257697469069008*G0_2_1;
    A[982] = -0.0148746893538564*G0_0_0 - 0.0148746893538557*G0_0_1 + 6.52398655865116e-05*G0_0_2 - 0.0148746893538564*G0_1_0 - 0.0148746893538557*G0_1_1 + 6.52398655865878e-05*G0_1_2 + 6.52398655869522e-05*G0_2_0 + 6.52398655874912e-05*G0_2_1;
    A[983] = -0.0189848008858421*G0_0_0 - 0.0189848008858425*G0_0_1 - 0.00949240044292112*G0_0_2 - 0.018984800885842*G0_1_0 - 0.0189848008858425*G0_1_1 - 0.00949240044292119*G0_1_2 - 0.00949240044292101*G0_2_0 - 0.00949240044292133*G0_2_1 - 0.000652398655870788*G0_2_2;
    A[984] = -0.0189848008858418*G0_0_0 - 0.0189848008858416*G0_0_1 - 0.00949240044292003*G0_0_2 - 0.018984800885842*G0_1_0 - 0.0189848008858417*G0_1_1 - 0.00949240044292012*G0_1_2 - 0.00949240044292104*G0_2_0 - 0.00949240044292114*G0_2_1 - 0.000652398655869883*G0_2_2;
    A[985] = -0.127087258163646*G0_0_0 - 0.127087258163646*G0_0_1 - 0.0893133759887227*G0_0_2 - 0.127087258163647*G0_1_0 - 0.127087258163646*G0_1_1 - 0.0893133759887228*G0_1_2 - 0.0893133759887233*G0_2_0 - 0.089313375988723*G0_2_1 - 0.06654466289883*G0_2_2;
    A[986] = 0.132306447410613*G0_0_0 + 0.121346149991982*G0_0_1 + 0.194675758911868*G0_0_2 + 0.132306447410613*G0_1_0 + 0.121346149991983*G0_1_1 + 0.194675758911868*G0_1_2 + 0.0589768384907265*G0_2_0 + 0.194675758911869*G0_2_1 - 0.00339247301052928*G0_2_2;
    A[987] = -0.0436454700777624*G0_0_0 + 0.0371867233846385*G0_0_1 - 0.0371541034518468*G0_0_2 - 0.0436454700777624*G0_1_0 + 0.0371867233846384*G0_1_1 - 0.0371541034518469*G0_1_2 - 0.00975335990526966*G0_2_0 - 0.0371541034518468*G0_2_1 - 0.00678494602105738*G0_2_2;
    A[988] = 0.0148746893538565*G0_0_0 - 0.0254435475789627*G0_0_1 + 0.0136351319077013*G0_0_2 + 0.0148746893538565*G0_1_0 - 0.0254435475789627*G0_1_1 + 0.0136351319077014*G0_1_2 - 6.52398655869299e-05*G0_2_0 + 0.0136351319077019*G0_2_1 - 0.000130479731173965*G0_2_2;
    A[989] = -0.166948816037355*G0_0_0 - 0.334680510461757*G0_0_1 - 0.250814663249556*G0_0_2 - 0.166948816037356*G0_1_0 - 0.334680510461757*G0_1_1 - 0.250814663249556*G0_1_2 - 0.160783648739377*G0_2_0 - 0.250814663249556*G0_2_1 - 0.0622388317700804*G0_2_2;
    A[990] = 0.0189848008858419*G0_0_0 + 0.0567586830607657*G0_0_1 + 0.0378717419733049*G0_0_2 + 0.0189848008858421*G0_1_0 + 0.0567586830607656*G0_1_1 + 0.0378717419733049*G0_1_2 + 0.00949240044292116*G0_2_0 + 0.0378717419733033*G0_2_1 + 0.0127870136550694*G0_2_2;
    A[991] = 0.127087258163646*G0_0_0 + 0.20941996853455*G0_0_1 + 0.173472802596064*G0_0_2 + 0.127087258163647*G0_1_0 + 0.20941996853455*G0_1_1 + 0.173472802596065*G0_1_2 + 0.0893133759887231*G0_2_0 + 0.173472802596066*G0_2_1 + 0.0846813455320388*G0_2_2;
    A[992] = 0.121346149991982*G0_0_0 + 0.132306447410613*G0_0_1 + 0.194675758911868*G0_0_2 + 0.121346149991982*G0_1_0 + 0.132306447410613*G0_1_1 + 0.194675758911868*G0_1_2 + 0.194675758911868*G0_2_0 + 0.0589768384907264*G0_2_1 - 0.00339247301052947*G0_2_2;
    A[993] = 0.0371867233846391*G0_0_0 - 0.0436454700777609*G0_0_1 - 0.037154103451846*G0_0_2 + 0.0371867233846391*G0_1_0 - 0.0436454700777609*G0_1_1 - 0.0371541034518459*G0_1_2 - 0.0371541034518464*G0_2_0 - 0.0097533599052691*G0_2_1 - 0.00678494602105716*G0_2_2;
    A[994] = -0.0254435475789637*G0_0_0 + 0.0148746893538558*G0_0_1 + 0.0136351319077015*G0_0_2 - 0.0254435475789636*G0_1_0 + 0.0148746893538557*G0_1_1 + 0.0136351319077016*G0_1_2 + 0.0136351319077011*G0_2_0 - 6.52398655874377e-05*G0_2_1 - 0.000130479731174179*G0_2_2;
    A[995] = -0.334680510461756*G0_0_0 - 0.166948816037354*G0_0_1 - 0.250814663249556*G0_0_2 - 0.334680510461756*G0_1_0 - 0.166948816037355*G0_1_1 - 0.250814663249556*G0_1_2 - 0.250814663249556*G0_2_0 - 0.160783648739377*G0_2_1 - 0.0622388317700799*G0_2_2;
    A[996] = 0.0567586830607651*G0_0_0 + 0.0189848008858425*G0_0_1 + 0.0378717419733036*G0_0_2 + 0.056758683060765*G0_1_0 + 0.0189848008858425*G0_1_1 + 0.0378717419733034*G0_1_2 + 0.0378717419733038*G0_2_0 + 0.00949240044292158*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[997] = 0.209419968534549*G0_0_0 + 0.127087258163646*G0_0_1 + 0.173472802596064*G0_0_2 + 0.209419968534549*G0_1_0 + 0.127087258163646*G0_1_1 + 0.173472802596064*G0_1_2 + 0.173472802596065*G0_2_0 + 0.0893133759887229*G0_2_1 + 0.0846813455320386*G0_2_2;
    A[998] = 0.0476251018785737*G0_0_0 + 0.047625101878574*G0_0_1 - 0.014744209622681*G0_0_2 + 0.0476251018785738*G0_1_0 + 0.0476251018785741*G0_1_1 - 0.014744209622681*G0_1_2 - 0.0147442096226812*G0_2_0 - 0.0147442096226814*G0_2_1 + 0.0476251018785744*G0_2_2;
    A[999] = 0.00880738185425652*G0_0_0 + 0.00880738185425689*G0_0_1 + 0.00231601522834207*G0_0_2 + 0.00880738185425645*G0_1_0 + 0.00880738185425687*G0_1_1 + 0.00231601522834192*G0_1_2 + 0.00231601522834105*G0_2_0 + 0.00231601522834108*G0_2_1 - 0.000652398655870781*G0_2_2;
    A[1000] = -0.00130479731174194*G0_0_0 - 0.00130479731174131*G0_0_1 - 6.52398655873354e-05*G0_0_2 - 0.00130479731174194*G0_1_0 - 0.0013047973117413*G0_1_1 - 6.52398655874945e-05*G0_1_2 - 6.52398655872626e-05*G0_2_0 - 6.52398655869152e-05*G0_2_1;
    A[1001] = 0.00880738185425653*G0_0_0 + 0.0088073818542558*G0_0_1 + 0.00231601522834075*G0_0_2 + 0.00880738185425648*G0_1_0 + 0.00880738185425571*G0_1_1 + 0.00231601522834085*G0_1_2 + 0.00231601522834094*G0_2_0 + 0.00231601522834105*G0_2_1 - 0.000652398655871239*G0_2_2;
    A[1002] = -0.0156575677409004*G0_0_0 - 0.0156575677409011*G0_0_1 - 0.00257697469068966*G0_0_2 - 0.0156575677409003*G0_1_0 - 0.015657567740901*G0_1_1 - 0.00257697469068972*G0_1_2 - 0.00257697469068909*G0_2_0 - 0.00257697469068934*G0_2_1;
    A[1003] = -0.00130479731174168*G0_0_0 - 0.00130479731174091*G0_0_1 - 6.52398655865036e-05*G0_0_2 - 0.00130479731174172*G0_1_0 - 0.00130479731174093*G0_1_1 - 6.52398655865782e-05*G0_1_2 - 6.52398655868918e-05*G0_2_0 - 6.52398655869276e-05*G0_2_1;
    A[1004] = -0.156575677409008*G0_0_0 - 0.156575677409009*G0_0_1 - 0.0662184635708931*G0_0_2 - 0.156575677409008*G0_1_0 - 0.156575677409009*G0_1_1 - 0.0662184635708931*G0_1_2 - 0.0662184635708934*G0_2_0 - 0.0662184635708934*G0_2_1 - 0.161794866655978*G0_2_2;
    A[1005] = 0.039143919352251*G0_0_0 + 0.0391439193522518*G0_0_1 + 0.00717638521457919*G0_0_2 + 0.0391439193522509*G0_1_0 + 0.0391439193522517*G0_1_1 + 0.0071763852145794*G0_1_2 + 0.00717638521458005*G0_2_0 + 0.00717638521457909*G0_2_1 + 0.00130479731174158*G0_2_2;
    A[1006] = 0.0391439193522529*G0_0_0 + 0.0391439193522523*G0_0_1 + 0.00717638521457905*G0_0_2 + 0.0391439193522527*G0_1_0 + 0.0391439193522522*G0_1_1 + 0.00717638521457905*G0_1_2 + 0.00717638521457886*G0_2_0 + 0.00717638521457927*G0_2_1 + 0.0013047973117411*G0_2_2;
    A[1007] = 0.185933616923198*G0_0_0 + 0.185933616923196*G0_0_1 + 0.170276049182297*G0_0_2 + 0.185933616923198*G0_1_0 + 0.185933616923197*G0_1_1 + 0.170276049182297*G0_1_2 + 0.170276049182298*G0_2_0 + 0.170276049182298*G0_2_1 + 0.180714427676233*G0_2_2;
    A[1008] = -0.0155544887532734*G0_0_0 - 0.0155544887532734*G0_0_1 - 0.0155544887532734*G0_0_2 - 0.0155544887532734*G0_1_0 - 0.0155544887532734*G0_1_1 - 0.0155544887532734*G0_1_2 - 0.0388177200243169*G0_2_0 - 0.0388177200243169*G0_2_1 - 0.0388177200243169*G0_2_2;
    A[1009] = -0.00336507226698207*G0_0_0 - 0.00336507226698207*G0_1_0;
    A[1010] = -0.00336507226698201*G0_0_1 - 0.00336507226698204*G0_1_1;
    A[1011] = 0.0336585514536903*G0_0_2 + 0.0336585514536902*G0_1_2 + 0.0551276864210891*G0_2_2;
    A[1012] = 0.0111690649885095*G0_0_1 + 0.00228991928210686*G0_0_2 + 0.0111690649885095*G0_1_1 + 0.00228991928210684*G0_1_2 + 0.00228991928210679*G0_2_1;
    A[1013] = -0.00523223722008447*G0_0_1 + 0.000548014870931503*G0_0_2 - 0.00523223722008458*G0_1_1 + 0.000548014870931439*G0_1_2 + 0.000548014870931407*G0_2_1 + 6.52398655873042e-05*G0_2_2;
    A[1014] = -0.00960330821441936*G0_0_1 - 0.00480165410720948*G0_0_2 - 0.00960330821441926*G0_1_1 - 0.00480165410720941*G0_1_2 - 0.00480165410720974*G0_2_1 + 0.00026095946234813*G0_2_2;
    A[1015] = 0.108637424175618*G0_0_1 + 0.0699110399631226*G0_0_2 + 0.108637424175618*G0_1_1 + 0.0699110399631225*G0_1_2 + 0.0699110399631232*G0_2_1 + 0.0124934342599273*G0_2_2;
    A[1016] = 0.0111690649885092*G0_0_0 + 0.00228991928210643*G0_0_2 + 0.0111690649885092*G0_1_0 + 0.00228991928210645*G0_1_2 + 0.00228991928210658*G0_2_0;
    A[1017] = -0.0052322372200843*G0_0_0 + 0.000548014870931946*G0_0_2 - 0.00523223722008446*G0_1_0 + 0.000548014870931876*G0_1_2 + 0.000548014870931756*G0_2_0 + 6.5239865587525e-05*G0_2_2;
    A[1018] = -0.00960330821441947*G0_0_0 - 0.00480165410721022*G0_0_2 - 0.00960330821441934*G0_1_0 - 0.00480165410721019*G0_1_2 - 0.00480165410720993*G0_2_0 + 0.000260959462348024*G0_2_2;
    A[1019] = 0.108637424175618*G0_0_0 + 0.0699110399631229*G0_0_2 + 0.108637424175618*G0_1_0 + 0.0699110399631229*G0_1_2 + 0.0699110399631232*G0_2_0 + 0.0124934342599273*G0_2_2;
    A[1020] = 0.00228991928210621*G0_0_0 + 0.00228991928210626*G0_0_1 + 0.00228991928210623*G0_1_0 + 0.00228991928210629*G0_1_1;
    A[1021] = 0.00178757231708656*G0_0_0 + 0.00178757231708637*G0_0_1 + 0.00178757231708654*G0_1_0 + 0.00178757231708635*G0_1_1;
    A[1022] = 0.00178757231708605*G0_0_0 + 0.0017875723170861*G0_0_1 + 0.00178757231708607*G0_1_0 + 0.00178757231708611*G0_1_1;
    A[1023] = 0.00228991928210683*G0_0_0 + 0.00228991928210687*G0_0_1 + 0.00228991928210681*G0_1_0 + 0.00228991928210687*G0_1_1;
    A[1024] = 0.100717304493345*G0_0_0 + 0.100717304493345*G0_0_1 + 0.0817390275940617*G0_0_2 + 0.100717304493345*G0_1_0 + 0.100717304493345*G0_1_1 + 0.0817390275940617*G0_1_2 + 0.125351877739029*G0_2_0 + 0.12535187773903*G0_2_1 + 0.185268170294211*G0_2_2;
    A[1025] = -0.204187731314465*G0_0_0 - 0.204187731314466*G0_0_1 - 0.167692550505049*G0_0_2 - 0.204187731314465*G0_1_0 - 0.204187731314466*G0_1_1 - 0.167692550505049*G0_1_2 - 0.219493003781196*G0_2_0 - 0.219493003781196*G0_2_1 - 0.387185554286247*G0_2_2;
    A[1026] = 0.281001149056702*G0_0_0 + 0.281001149056702*G0_0_1 + 0.202869886029607*G0_0_2 + 0.281001149056702*G0_1_0 + 0.281001149056702*G0_1_1 + 0.202869886029607*G0_1_2 + 0.202869886029607*G0_2_0 + 0.202869886029607*G0_2_1 + 0.428965164208218*G0_2_2;
    A[1027] = -0.108637424175618*G0_0_0 - 0.108637424175618*G0_0_1 - 0.135020425819037*G0_0_2 - 0.108637424175618*G0_1_0 - 0.108637424175618*G0_1_1 - 0.135020425819037*G0_1_2 - 0.0699110399631234*G0_2_0 - 0.0699110399631233*G0_2_1 - 0.243357746612954*G0_2_2;
    A[1028] = 0.0283728175438241*G0_0_0 + 0.00939454064454071*G0_0_1 + 0.0283728175438241*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.00939454064454071*G0_1_1 + 0.0283728175438241*G0_1_2 - 0.0315434750113569*G0_2_0 + 0.0283728175438243*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1029] = -0.00048277500534458*G0_0_0 + 6.52398655868346e-05*G0_0_1 - 0.000482775005344598*G0_0_2 - 0.000482775005344542*G0_1_0 + 6.52398655868372e-05*G0_1_1 - 0.000482775005344562*G0_1_2 - 0.0062630270963605*G0_2_0 - 0.000482775005344628*G0_2_1 - 0.00626302709636043*G0_2_2;
    A[1030] = 0.0132175967679438*G0_0_0 + 6.52398655871285e-05*G0_0_1 + 0.0132175967679439*G0_0_2 + 0.0132175967679438*G0_1_0 + 6.52398655871006e-05*G0_1_1 + 0.0132175967679438*G0_1_2 + 6.52398655871072e-05*G0_2_0 + 0.0132175967679441*G0_2_1 + 6.52398655871239e-05*G0_2_2;
    A[1031] = -0.0022899192821068*G0_0_0 + 0.00939454064454062*G0_0_1 - 0.00228991928210681*G0_0_2 - 0.00228991928210677*G0_1_0 + 0.00939454064454067*G0_1_1 - 0.00228991928210678*G0_1_2 - 0.00228991928210684*G0_2_1;
    A[1032] = 0.00939454064454047*G0_0_0 + 0.0283728175438239*G0_0_1 + 0.0283728175438239*G0_0_2 + 0.00939454064454046*G0_1_0 + 0.0283728175438239*G0_1_1 + 0.0283728175438238*G0_1_2 + 0.028372817543824*G0_2_0 - 0.0315434750113571*G0_2_1 - 0.0315434750113571*G0_2_2;
    A[1033] = 6.52398655873005e-05*G0_0_0 - 0.0004827750053441*G0_0_1 - 0.000482775005344084*G0_0_2 + 6.52398655873292e-05*G0_1_0 - 0.000482775005344049*G0_1_1 - 0.000482775005344033*G0_1_2 - 0.000482775005344076*G0_2_0 - 0.00626302709636036*G0_2_1 - 0.00626302709636036*G0_2_2;
    A[1034] = 6.52398655868967e-05*G0_0_0 + 0.0132175967679437*G0_0_1 + 0.0132175967679437*G0_0_2 + 6.52398655868534e-05*G0_1_0 + 0.0132175967679437*G0_1_1 + 0.0132175967679437*G0_1_2 + 0.0132175967679435*G0_2_0 + 6.52398655867478e-05*G0_2_1 + 6.52398655867555e-05*G0_2_2;
    A[1035] = 0.00939454064454073*G0_0_0 - 0.00228991928210624*G0_0_1 - 0.00228991928210629*G0_0_2 + 0.00939454064454074*G0_1_0 - 0.00228991928210626*G0_1_1 - 0.00228991928210631*G0_1_2 - 0.00228991928210642*G0_2_0;
    A[1036] = -0.00117431758056721*G0_0_0 - 0.00117431758056773*G0_0_1 + 6.52398655864063e-05*G0_0_2 - 0.00117431758056727*G0_1_0 - 0.00117431758056776*G0_1_1 + 6.52398655864092e-05*G0_1_2 + 6.52398655871797e-05*G0_2_0 + 6.52398655868387e-05*G0_2_1;
    A[1037] = -0.0105036183595215*G0_0_0 - 0.0105036183595209*G0_0_1 + 0.00257697469069049*G0_0_2 - 0.0105036183595215*G0_1_0 - 0.0105036183595208*G0_1_1 + 0.0025769746906905*G0_1_2 + 0.00257697469068959*G0_2_0 + 0.00257697469068965*G0_2_1;
    A[1038] = -0.00117431758056735*G0_0_0 - 0.00117431758056776*G0_0_1 + 6.52398655870323e-05*G0_0_2 - 0.00117431758056749*G0_1_0 - 0.00117431758056796*G0_1_1 + 6.52398655869479e-05*G0_1_2 + 6.52398655870589e-05*G0_2_0 + 6.52398655869415e-05*G0_2_1;
    A[1039] = 0.00352295274170225*G0_0_0 + 0.00352295274170291*G0_0_1 - 0.00296841388421203*G0_0_2 + 0.00352295274170235*G0_1_0 + 0.00352295274170288*G0_1_1 - 0.00296841388421206*G0_1_2 - 0.00296841388421287*G0_2_0 - 0.00296841388421262*G0_2_1 - 0.000652398655870802*G0_2_2;
    A[1040] = 0.0035229527417018*G0_0_0 + 0.00352295274170143*G0_0_1 - 0.0029684138842137*G0_0_2 + 0.00352295274170183*G0_1_0 + 0.00352295274170157*G0_1_1 - 0.00296841388421376*G0_1_2 - 0.00296841388421276*G0_2_0 - 0.00296841388421264*G0_2_1 - 0.000652398655871279*G0_2_2;
    A[1041] = 0.124738623002512*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0623693115012556*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.0623693115012557*G0_1_2 + 0.0623693115012562*G0_2_0 + 0.062369311501256*G0_2_1 + 0.0476251018785741*G0_2_2;
    A[1042] = -0.0510175748891021*G0_0_0 - 0.0528442911255404*G0_0_1 - 0.0887914570640252*G0_0_2 - 0.0510175748891021*G0_1_0 - 0.0528442911255403*G0_1_1 - 0.0887914570640251*G0_1_2 - 0.00463203045668202*G0_2_0 - 0.0887914570640252*G0_2_1 + 0.0846813455320406*G0_2_2;
    A[1043] = -0.0155923278753126*G0_0_0 - 0.00619778723077223*G0_0_1 - 0.0250847283182339*G0_0_2 - 0.0155923278753127*G0_1_0 - 0.00619778723077228*G0_1_1 - 0.0250847283182339*G0_1_2 + 0.00329461321214777*G0_2_0 - 0.0250847283182352*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[1044] = 0.0011743175805673*G0_0_0 - 0.0528442911255419*G0_0_1 - 0.0137656116388756*G0_0_2 + 0.00117431758056745*G0_1_0 - 0.0528442911255419*G0_1_1 - 0.0137656116388755*G0_1_2 - 6.52398655870647e-05*G0_2_0 - 0.0137656116388758*G0_2_1 - 0.000130479731174391*G0_2_2;
    A[1045] = 0.182410664181495*G0_0_0 + 0.104709984267274*G0_0_1 + 0.188575831479474*G0_0_2 + 0.182410664181495*G0_1_0 + 0.104709984267274*G0_1_1 + 0.188575831479474*G0_1_2 + 0.0985448169692949*G0_2_0 + 0.188575831479474*G0_2_1 - 0.062238831770083*G0_2_2;
    A[1046] = -0.0035229527417019*G0_0_0 + 0.104709984267276*G0_0_1 + 0.0303691574307882*G0_0_2 - 0.00352295274170194*G0_1_0 + 0.104709984267276*G0_1_1 + 0.0303691574307881*G0_1_2 + 0.00296841388421266*G0_2_0 + 0.0303691574307903*G0_2_1 - 0.00678494602105703*G0_2_2;
    A[1047] = -0.124738623002511*G0_0_0 - 0.271397840842283*G0_0_1 - 0.198068231922396*G0_0_2 - 0.124738623002511*G0_1_0 - 0.271397840842283*G0_1_1 - 0.198068231922396*G0_1_2 - 0.0623693115012561*G0_2_0 - 0.198068231922397*G0_2_1 - 0.00339247301052766*G0_2_2;
    A[1048] = -0.0528442911255399*G0_0_0 - 0.0510175748891015*G0_0_1 - 0.0887914570640248*G0_0_2 - 0.0528442911255399*G0_1_0 - 0.0510175748891015*G0_1_1 - 0.0887914570640247*G0_1_2 - 0.0887914570640254*G0_2_0 - 0.00463203045668221*G0_2_1 + 0.0846813455320407*G0_2_2;
    A[1049] = -0.00619778723077315*G0_0_0 - 0.015592327875314*G0_0_1 - 0.0250847283182353*G0_0_2 - 0.0061977872307732*G0_1_0 - 0.015592327875314*G0_1_1 - 0.0250847283182354*G0_1_2 - 0.0250847283182351*G0_2_0 + 0.00329461321214734*G0_2_1 + 0.0127870136550691*G0_2_2;
    A[1050] = -0.0528442911255409*G0_0_0 + 0.00117431758056782*G0_0_1 - 0.0137656116388759*G0_0_2 - 0.0528442911255409*G0_1_0 + 0.00117431758056786*G0_1_1 - 0.0137656116388759*G0_1_2 - 0.0137656116388754*G0_2_0 - 6.5239865586786e-05*G0_2_1 - 0.000130479731174207*G0_2_2;
    A[1051] = 0.104709984267273*G0_0_0 + 0.182410664181494*G0_0_1 + 0.188575831479474*G0_0_2 + 0.104709984267273*G0_1_0 + 0.182410664181494*G0_1_1 + 0.188575831479474*G0_1_2 + 0.188575831479475*G0_2_0 + 0.098544816969295*G0_2_1 - 0.0622388317700831*G0_2_2;
    A[1052] = 0.104709984267276*G0_0_0 - 0.00352295274170282*G0_0_1 + 0.0303691574307899*G0_0_2 + 0.104709984267276*G0_1_0 - 0.00352295274170279*G0_1_1 + 0.0303691574307899*G0_1_2 + 0.0303691574307897*G0_2_0 + 0.0029684138842123*G0_2_1 - 0.00678494602105695*G0_2_2;
    A[1053] = -0.271397840842283*G0_0_0 - 0.124738623002511*G0_0_1 - 0.198068231922396*G0_0_2 - 0.271397840842283*G0_1_0 - 0.124738623002511*G0_1_1 - 0.198068231922396*G0_1_2 - 0.198068231922397*G0_2_0 - 0.0623693115012559*G0_2_1 - 0.0033924730105278*G0_2_2;
    A[1054] = -0.0150051690850297*G0_0_0 - 0.0150051690850298*G0_0_1 + 0.0227687130898934*G0_0_2 - 0.0150051690850297*G0_1_0 - 0.0150051690850298*G0_1_1 + 0.0227687130898935*G0_1_2 + 0.0227687130898935*G0_2_0 + 0.0227687130898938*G0_2_1 - 0.0665446628988291*G0_2_2;
    A[1055] = -0.000652398655871104*G0_0_0 - 0.000652398655871342*G0_0_1 + 0.00884000178705001*G0_0_2 - 0.000652398655871089*G0_1_0 - 0.000652398655871408*G0_1_1 + 0.00884000178705*G0_1_2 + 0.00884000178705046*G0_2_0 + 0.00884000178705058*G0_2_1 - 0.000652398655871335*G0_2_2;
    A[1056] = -0.0150051690850296*G0_0_0 - 0.0150051690850301*G0_0_1 - 6.52398655863957e-05*G0_0_2 - 0.0150051690850296*G0_1_0 - 0.01500516908503*G0_1_1 - 6.52398655863974e-05*G0_1_2 - 6.52398655868557e-05*G0_2_0 - 6.5239865586991e-05*G0_2_1;
    A[1057] = -0.000652398655870781*G0_0_0 - 0.000652398655870553*G0_0_1 + 0.00884000178705071*G0_0_2 - 0.000652398655870794*G0_1_0 - 0.00065239865587054*G0_1_1 + 0.00884000178705062*G0_1_2 + 0.00884000178705058*G0_2_0 + 0.00884000178705026*G0_2_1 - 0.000652398655871068*G0_2_2;
    A[1058] = -0.000652398655871138*G0_0_0 - 0.000652398655870744*G0_0_1 - 0.00257697469069044*G0_0_2 - 0.000652398655871149*G0_1_0 - 0.000652398655870737*G0_1_1 - 0.00257697469069045*G0_1_2 - 0.00257697469069024*G0_2_0 - 0.00257697469069036*G0_2_1;
    A[1059] = -0.01500516908503*G0_0_0 - 0.0150051690850303*G0_0_1 - 6.5239865587041e-05*G0_0_2 - 0.0150051690850299*G0_1_0 - 0.0150051690850303*G0_1_1 - 6.52398655869581e-05*G0_1_2 - 6.52398655871899e-05*G0_2_0 - 6.52398655867576e-05*G0_2_1;
    A[1060] = 0.0260959462348343*G0_0_0 + 0.0260959462348347*G0_0_1 + 0.0104383784939333*G0_0_2 + 0.0260959462348343*G0_1_0 + 0.0260959462348345*G0_1_1 + 0.0104383784939333*G0_1_2 + 0.0104383784939325*G0_2_0 + 0.0104383784939328*G0_2_1 + 0.180714427676232*G0_2_2;
    A[1061] = 0.0260959462348353*G0_0_0 + 0.0260959462348347*G0_0_1 - 0.00587158790283799*G0_0_2 + 0.0260959462348353*G0_1_0 + 0.0260959462348347*G0_1_1 - 0.00587158790283796*G0_1_2 - 0.00587158790283756*G0_2_0 - 0.00587158790283711*G0_2_1 + 0.00130479731174215*G0_2_2;
    A[1062] = 0.0260959462348341*G0_0_0 + 0.0260959462348348*G0_0_1 - 0.00587158790283671*G0_0_2 + 0.0260959462348342*G0_1_0 + 0.0260959462348351*G0_1_1 - 0.00587158790283657*G0_1_2 - 0.00587158790283733*G0_2_0 - 0.00587158790283735*G0_2_1 + 0.00130479731174219*G0_2_2;
    A[1063] = -0.185933616923197*G0_0_0 - 0.185933616923196*G0_0_1 - 0.0955764030850828*G0_0_2 - 0.185933616923197*G0_1_0 - 0.185933616923196*G0_1_1 - 0.095576403085083*G0_1_2 - 0.0955764030850823*G0_2_0 - 0.0955764030850823*G0_2_1 - 0.161794866655977*G0_2_2;
    A[1064] = 0.00480034930989781*G0_0_0 + 0.00480034930989784*G0_0_1 + 0.00480034930989784*G0_0_2 + 0.00480034930989781*G0_1_0 + 0.00480034930989784*G0_1_1 + 0.00480034930989784*G0_1_2 + 0.0153313684129655*G0_2_0 + 0.0153313684129655*G0_2_1 + 0.0153313684129655*G0_2_2;
    A[1065] = -0.00480034930989787*G0_0_0 - 0.00480034930989786*G0_1_0;
    A[1066] = -0.00480034930989786*G0_0_1 - 0.00480034930989786*G0_1_1;
    A[1067] = -0.0372571824394741*G0_0_2 - 0.0372571824394741*G0_1_2 - 0.0551276864210891*G0_2_2;
    A[1068] = 0.0328352243499812*G0_0_1 + 0.00537576492437603*G0_0_2 + 0.0328352243499812*G0_1_1 + 0.00537576492437603*G0_1_2 + 0.00537576492437618*G0_2_1;
    A[1069] = -0.0788945694544649*G0_0_1 - 0.0189782768992835*G0_0_2 - 0.0788945694544648*G0_1_1 - 0.0189782768992835*G0_1_2 - 0.0189782768992838*G0_2_1 + 0.00939454064454037*G0_2_2;
    A[1070] = 0.108637424175618*G0_0_1 + 0.0387263842124945*G0_0_2 + 0.108637424175618*G0_1_1 + 0.0387263842124945*G0_1_2 + 0.0387263842124953*G0_2_1 - 0.0186912214907003*G0_2_2;
    A[1071] = -0.139822079926246*G0_0_1 - 0.0699110399631225*G0_0_2 - 0.139822079926246*G0_1_1 - 0.0699110399631225*G0_1_2 - 0.0699110399631234*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[1072] = 0.0328352243499811*G0_0_0 + 0.00537576492437606*G0_0_2 + 0.032835224349981*G0_1_0 + 0.00537576492437602*G0_1_2 + 0.00537576492437613*G0_2_0;
    A[1073] = -0.0788945694544651*G0_0_0 - 0.018978276899284*G0_0_2 - 0.0788945694544649*G0_1_0 - 0.0189782768992839*G0_1_2 - 0.018978276899284*G0_2_0 + 0.00939454064454016*G0_2_2;
    A[1074] = 0.108637424175618*G0_0_0 + 0.0387263842124952*G0_0_2 + 0.108637424175618*G0_1_0 + 0.0387263842124952*G0_1_2 + 0.0387263842124954*G0_2_0 - 0.0186912214907002*G0_2_2;
    A[1075] = -0.139822079926246*G0_0_0 - 0.0699110399631229*G0_0_2 - 0.139822079926246*G0_1_0 - 0.0699110399631229*G0_1_2 - 0.0699110399631234*G0_2_0 + 0.019637199541713*G0_2_2;
    A[1076] = 0.00537576492437612*G0_0_0 + 0.00537576492437609*G0_0_1 + 0.00537576492437608*G0_1_0 + 0.00537576492437605*G0_1_1;
    A[1077] = 0.00228991928210663*G0_0_0 + 0.00228991928210679*G0_0_1 + 0.00228991928210664*G0_1_0 + 0.00228991928210681*G0_1_1;
    A[1078] = 0.00228991928210668*G0_0_0 + 0.00228991928210657*G0_0_1 + 0.00228991928210668*G0_1_0 + 0.00228991928210656*G0_1_1;
    A[1079] = 0.00537576492437603*G0_0_0 + 0.00537576492437613*G0_0_1 + 0.00537576492437603*G0_1_0 + 0.00537576492437614*G0_1_1;
    A[1080] = -0.032835224349981*G0_0_0 - 0.032835224349981*G0_0_1 - 0.027459459425605*G0_0_2 - 0.032835224349981*G0_1_0 - 0.032835224349981*G0_1_1 - 0.027459459425605*G0_1_2 - 0.0537511252572013*G0_2_0 - 0.0537511252572014*G0_2_1 - 0.0812105846828065*G0_2_2;
    A[1081] = 0.0788945694544647*G0_0_0 + 0.0788945694544649*G0_0_1 + 0.0599162925551811*G0_0_2 + 0.0788945694544647*G0_1_0 + 0.0788945694544649*G0_1_1 + 0.0599162925551811*G0_1_2 + 0.103529142700149*G0_2_0 + 0.103529142700149*G0_2_1 + 0.185268170294211*G0_2_2;
    A[1082] = -0.108637424175618*G0_0_0 - 0.108637424175618*G0_0_1 - 0.0699110399631234*G0_0_2 - 0.108637424175618*G0_1_0 - 0.108637424175618*G0_1_1 - 0.0699110399631233*G0_1_2 - 0.135020425819037*G0_2_0 - 0.135020425819037*G0_2_1 - 0.243357746612954*G0_2_2;
    A[1083] = 0.139822079926246*G0_0_0 + 0.139822079926246*G0_0_1 + 0.0699110399631235*G0_0_2 + 0.139822079926246*G0_1_0 + 0.139822079926246*G0_1_1 + 0.0699110399631235*G0_1_2 + 0.0699110399631235*G0_2_0 + 0.0699110399631235*G0_2_1 + 0.179096479009673*G0_2_2;
    A[1084] = -0.00537576492437603*G0_0_0 - 0.00537576492437598*G0_0_2 - 0.00537576492437603*G0_1_0 - 0.00537576492437598*G0_1_2 + 0.0220836945012291*G0_2_0 - 0.00537576492437599*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1085] = -0.00228991928210672*G0_0_0 - 0.00228991928210673*G0_0_2 - 0.00228991928210674*G0_1_0 - 0.00228991928210675*G0_1_2 + 0.00658922642429583*G0_2_0 - 0.00228991928210662*G0_2_1 + 0.00658922642429579*G0_2_2;
    A[1086] = -0.00228991928210662*G0_0_0 - 0.00228991928210664*G0_0_2 - 0.00228991928210661*G0_1_0 - 0.00228991928210663*G0_1_2 + 0.00939454064454062*G0_2_0 - 0.00228991928210698*G0_2_1 + 0.0093945406445406*G0_2_2;
    A[1087] = -0.00537576492437607*G0_0_0 - 0.00537576492437609*G0_0_2 - 0.00537576492437607*G0_1_0 - 0.00537576492437609*G0_1_2 - 0.00537576492437591*G0_2_1;
    A[1088] = -0.00537576492437603*G0_0_1 - 0.00537576492437597*G0_0_2 - 0.00537576492437603*G0_1_1 - 0.00537576492437597*G0_1_2 - 0.0053757649243759*G0_2_0 + 0.0220836945012292*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1089] = -0.00228991928210674*G0_0_1 - 0.00228991928210678*G0_0_2 - 0.00228991928210672*G0_1_1 - 0.00228991928210675*G0_1_2 - 0.00228991928210692*G0_2_0 + 0.00658922642429582*G0_2_1 + 0.00658922642429583*G0_2_2;
    A[1090] = -0.00228991928210681*G0_0_1 - 0.00228991928210681*G0_0_2 - 0.00228991928210684*G0_1_1 - 0.00228991928210684*G0_1_2 - 0.00228991928210664*G0_2_0 + 0.0093945406445407*G0_2_1 + 0.00939454064454067*G0_2_2;
    A[1091] = -0.0053757649243762*G0_0_1 - 0.00537576492437611*G0_0_2 - 0.00537576492437616*G0_1_1 - 0.00537576492437608*G0_1_2 - 0.00537576492437613*G0_2_0;
    A[1092] = -0.0118736555368503*G0_0_0 - 0.0118736555368502*G0_0_1 + 0.0093945406445406*G0_0_2 - 0.0118736555368502*G0_1_0 - 0.0118736555368502*G0_1_1 + 0.00939454064454056*G0_1_2 + 0.00939454064454043*G0_2_0 + 0.00939454064454049*G0_2_1;
    A[1093] = -0.00254435475789627*G0_0_0 - 0.0025443547578963*G0_0_1 + 0.00329461321214759*G0_0_2 - 0.00254435475789626*G0_1_0 - 0.00254435475789631*G0_1_1 + 0.00329461321214759*G0_1_2 + 0.00329461321214807*G0_2_0 + 0.00329461321214834*G0_2_1;
    A[1094] = -0.0118736555368495*G0_0_0 - 0.0118736555368496*G0_0_1 + 0.00939454064454088*G0_0_2 - 0.0118736555368495*G0_1_0 - 0.0118736555368496*G0_1_1 + 0.0093945406445409*G0_1_2 + 0.00939454064454069*G0_2_0 + 0.00939454064454043*G0_2_1;
    A[1095] = 0.0429604514890971*G0_0_0 + 0.0429604514890969*G0_0_1 + 0.000685018588664204*G0_0_2 + 0.0429604514890971*G0_1_0 + 0.042960451489097*G0_1_1 + 0.000685018588664271*G0_1_2 + 0.000685018588664856*G0_2_0 + 0.000685018588664528*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[1096] = 0.0429604514890969*G0_0_0 + 0.0429604514890969*G0_0_1 + 0.000685018588664708*G0_0_2 + 0.0429604514890968*G0_1_0 + 0.0429604514890969*G0_1_1 + 0.000685018588664706*G0_1_2 + 0.00068501858866473*G0_2_0 + 0.000685018588664769*G0_2_1 + 0.0226708532915133*G0_2_2;
    A[1097] = -0.124738623002512*G0_0_0 - 0.124738623002512*G0_0_1 - 0.0284445813959707*G0_0_2 - 0.124738623002512*G0_1_0 - 0.124738623002512*G0_1_1 - 0.0284445813959707*G0_1_2 - 0.0284445813959709*G0_2_0 - 0.0284445813959708*G0_2_1 - 0.0606730749959917*G0_2_2;
    A[1098] = 0.0118736555368498*G0_0_0 + 0.0212681961813901*G0_0_2 + 0.0118736555368497*G0_1_0 + 0.0212681961813901*G0_1_2 - 0.0146137298915084*G0_2_0 + 0.0212681961813899*G0_2_1 - 0.0693499771190746*G0_2_2;
    A[1099] = 0.00254435475789613*G0_0_0 + 0.0058389679700438*G0_0_2 + 0.00254435475789609*G0_1_0 + 0.00583896797004379*G0_1_2 - 0.00590420783563137*G0_2_0 + 0.0058389679700447*G0_2_1 - 0.0318696743392923*G0_2_2;
    A[1100] = 0.0118736555368495*G0_0_0 + 0.0212681961813904*G0_0_2 + 0.0118736555368495*G0_1_0 + 0.0212681961813903*G0_1_2 - 0.0093945406445407*G0_2_0 + 0.0212681961813906*G0_2_1 - 0.018789081289081*G0_2_2;
    A[1101] = -0.0429604514890963*G0_0_0 - 0.0422754329004316*G0_0_2 - 0.0429604514890963*G0_1_0 - 0.0422754329004315*G0_1_2 - 0.0359145460056912*G0_2_0 - 0.0422754329004318*G0_2_1 + 0.0920860702761748*G0_2_2;
    A[1102] = -0.0429604514890969*G0_0_0 - 0.042275432900432*G0_0_2 - 0.0429604514890969*G0_1_0 - 0.0422754329004319*G0_1_2 - 0.000685018588664688*G0_2_0 - 0.0422754329004335*G0_2_1 + 0.0439716694056968*G0_2_2;
    A[1103] = 0.124738623002512*G0_0_0 + 0.0962940416065403*G0_0_2 + 0.124738623002512*G0_1_0 + 0.0962940416065401*G0_1_2 + 0.0284445813959708*G0_2_0 + 0.0962940416065411*G0_2_1 - 0.0644569872000428*G0_2_2;
    A[1104] = 0.0118736555368498*G0_0_1 + 0.0212681961813903*G0_0_2 + 0.0118736555368499*G0_1_1 + 0.0212681961813903*G0_1_2 + 0.0212681961813903*G0_2_0 - 0.0146137298915082*G0_2_1 - 0.0693499771190744*G0_2_2;
    A[1105] = 0.0025443547578962*G0_0_1 + 0.00583896797004442*G0_0_2 + 0.0025443547578962*G0_1_1 + 0.00583896797004441*G0_1_2 + 0.00583896797004444*G0_2_0 - 0.00590420783563121*G0_2_1 - 0.0318696743392922*G0_2_2;
    A[1106] = 0.0118736555368503*G0_0_1 + 0.0212681961813909*G0_0_2 + 0.0118736555368503*G0_1_1 + 0.0212681961813909*G0_1_2 + 0.0212681961813905*G0_2_0 - 0.00939454064454056*G0_2_1 - 0.0187890812890809*G0_2_2;
    A[1107] = -0.0429604514890963*G0_0_1 - 0.0422754329004324*G0_0_2 - 0.0429604514890963*G0_1_1 - 0.0422754329004324*G0_1_2 - 0.0422754329004321*G0_2_0 - 0.0359145460056914*G0_2_1 + 0.0920860702761746*G0_2_2;
    A[1108] = -0.0429604514890968*G0_0_1 - 0.042275432900433*G0_0_2 - 0.0429604514890968*G0_1_1 - 0.042275432900433*G0_1_2 - 0.0422754329004329*G0_2_0 - 0.000685018588664303*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[1109] = 0.124738623002512*G0_0_1 + 0.0962940416065409*G0_0_2 + 0.124738623002512*G0_1_1 + 0.0962940416065409*G0_1_2 + 0.0962940416065412*G0_2_0 + 0.0284445813959708*G0_2_1 - 0.0644569872000424*G0_2_2;
    A[1110] = -0.00939454064454041*G0_0_2 - 0.00939454064454041*G0_1_2 - 0.00939454064454062*G0_2_0 - 0.00939454064454088*G0_2_1 + 0.0453417065830254*G0_2_2;
    A[1111] = -0.00329461321214796*G0_0_2 - 0.00329461321214792*G0_1_2 - 0.00329461321214784*G0_2_0 - 0.0032946132121479*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[1112] = -0.00939454064454061*G0_0_2 - 0.00939454064454058*G0_1_2 - 0.00939454064454072*G0_2_0 - 0.00939454064454062*G0_2_1;
    A[1113] = -0.00329461321214805*G0_0_2 - 0.00329461321214803*G0_1_2 - 0.00329461321214799*G0_2_0 - 0.00329461321214767*G0_2_1 + 0.022670853291513*G0_2_2;
    A[1114] = -0.00329461321214765*G0_0_2 - 0.00329461321214765*G0_1_2 - 0.00329461321214793*G0_2_0 - 0.00329461321214776*G0_2_1;
    A[1115] = -0.00939454064454082*G0_0_2 - 0.00939454064454083*G0_1_2 - 0.00939454064454039*G0_2_0 - 0.00939454064454087*G0_2_1;
    A[1116] = 0.00260959462348391*G0_0_2 + 0.00260959462348388*G0_1_2 + 0.00260959462348443*G0_2_0 + 0.00260959462348406*G0_2_1 - 0.151356488162043*G0_2_2;
    A[1117] = 0.00260959462348358*G0_0_2 + 0.00260959462348348*G0_1_2 + 0.00260959462348345*G0_2_0 + 0.00260959462348293*G0_2_1 - 0.0453417065830261*G0_2_2;
    A[1118] = 0.00260959462348312*G0_0_2 + 0.0026095946234831*G0_1_2 + 0.00260959462348333*G0_2_0 + 0.00260959462348361*G0_2_1 - 0.0453417065830261*G0_2_2;
    A[1119] = 0.0352295274170275*G0_0_2 + 0.0352295274170275*G0_1_2 + 0.0352295274170264*G0_2_0 + 0.0352295274170265*G0_2_1 + 0.166687856575009*G0_2_2;
    A[1120] = -0.0178705039816148*G0_0_0 - 0.0178705039816148*G0_0_1 - 0.0178705039816148*G0_0_2 - 0.0551276864210886*G0_1_0 - 0.0551276864210886*G0_1_1 - 0.0551276864210886*G0_1_2 - 0.0178705039816148*G0_2_0 - 0.0178705039816148*G0_2_1 - 0.0178705039816148*G0_2_2;
    A[1121] = -0.00480034930989791*G0_0_0 - 0.00480034930989793*G0_2_0;
    A[1122] = 0.0105310191030676*G0_0_1 + 0.0153313684129656*G0_1_1 + 0.0105310191030676*G0_2_1;
    A[1123] = -0.00480034930989794*G0_0_2 - 0.00480034930989796*G0_2_2;
    A[1124] = 0.0274594594256051*G0_0_1 + 0.0328352243499811*G0_0_2 + 0.022083694501229*G0_1_1 + 0.0274594594256052*G0_1_2 + 0.0274594594256052*G0_2_1 + 0.0328352243499812*G0_2_2;
    A[1125] = 0.00887914570640248*G0_0_1 + 0.0111690649885092*G0_0_2 + 0.00658922642429581*G0_1_1 + 0.00887914570640239*G0_1_2 + 0.00887914570640247*G0_2_1 + 0.0111690649885092*G0_2_2;
    A[1126] = 0.0116844599266473*G0_0_1 + 0.0139743792087541*G0_0_2 + 0.00939454064454052*G0_1_1 + 0.0116844599266474*G0_1_2 + 0.0116844599266473*G0_2_1 + 0.013974379208754*G0_2_2;
    A[1127] = 0.00537576492437599*G0_0_1 + 0.0107515298487521*G0_0_2 + 0.00537576492437595*G0_1_2 + 0.00537576492437594*G0_2_1 + 0.010751529848752*G0_2_2;
    A[1128] = 0.00537576492437595*G0_0_0 + 0.00537576492437587*G0_0_2 + 0.00537576492437602*G0_2_0 + 0.00537576492437588*G0_2_2;
    A[1129] = 0.00228991928210682*G0_0_0 + 0.00228991928210691*G0_0_2 + 0.0022899192821067*G0_2_0 + 0.00228991928210688*G0_2_2;
    A[1130] = 0.0022899192821066*G0_0_0 + 0.00228991928210643*G0_0_2 + 0.00228991928210672*G0_2_0 + 0.00228991928210646*G0_2_2;
    A[1131] = 0.00537576492437599*G0_0_0 + 0.00537576492437602*G0_0_2 + 0.00537576492437594*G0_2_0 + 0.00537576492437602*G0_2_2;
    A[1132] = 0.0107515298487517*G0_0_0 + 0.0053757649243758*G0_0_1 + 0.00537576492437575*G0_1_0 + 0.0107515298487518*G0_2_0 + 0.00537576492437581*G0_2_1;
    A[1133] = 0.0139743792087542*G0_0_0 + 0.0116844599266474*G0_0_1 + 0.0116844599266475*G0_1_0 + 0.00939454064454039*G0_1_1 + 0.0139743792087543*G0_2_0 + 0.0116844599266475*G0_2_1;
    A[1134] = 0.0111690649885091*G0_0_0 + 0.0088791457064023*G0_0_1 + 0.00887914570640228*G0_1_0 + 0.00658922642429598*G0_1_1 + 0.0111690649885091*G0_2_0 + 0.00887914570640233*G0_2_1;
    A[1135] = 0.0328352243499811*G0_0_0 + 0.0274594594256052*G0_0_1 + 0.0274594594256052*G0_1_0 + 0.0220836945012289*G0_1_1 + 0.0328352243499811*G0_2_0 + 0.0274594594256052*G0_2_1;
    A[1136] = 0.0895482395048356*G0_0_0 + 0.0895482395048356*G0_0_1 + 0.0196371995417133*G0_0_2 + 0.0196371995417132*G0_1_0 + 0.0196371995417132*G0_1_1 + 0.0895482395048358*G0_1_2 + 0.0895482395048356*G0_2_0 + 0.0895482395048357*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[1137] = -0.0574176057031954*G0_0_0 - 0.0574176057031955*G0_0_1 + 0.0124934342599271*G0_0_2 - 0.0186912214907006*G0_1_0 - 0.0186912214907006*G0_1_1 - 0.0574176057031956*G0_1_2 - 0.0574176057031954*G0_2_0 - 0.0574176057031954*G0_2_1 + 0.0124934342599272*G0_2_2;
    A[1138] = 0.0283728175438241*G0_0_0 + 0.0283728175438241*G0_0_1 - 0.0315434750113569*G0_0_2 + 0.00939454064454071*G0_1_0 + 0.00939454064454071*G0_1_1 + 0.0283728175438243*G0_1_2 + 0.0283728175438241*G0_2_0 + 0.0283728175438241*G0_2_1 - 0.0315434750113569*G0_2_2;
    A[1139] = -0.00537576492437603*G0_0_0 - 0.00537576492437603*G0_0_1 + 0.0220836945012291*G0_0_2 - 0.00537576492437599*G0_1_2 - 0.00537576492437598*G0_2_0 - 0.00537576492437598*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1140] = 0.179096479009671*G0_0_0 + 0.109185439046549*G0_0_1 + 0.179096479009672*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009672*G0_1_1 + 0.109185439046549*G0_1_2 + 0.179096479009672*G0_2_0 + 0.109185439046549*G0_2_1 + 0.179096479009672*G0_2_2;
    A[1141] = -0.147063705006411*G0_0_0 - 0.17344670664983*G0_0_1 - 0.147063705006412*G0_0_2 - 0.108337320793917*G0_1_0 - 0.243357746612952*G0_1_1 - 0.108337320793917*G0_1_2 - 0.147063705006411*G0_2_0 - 0.17344670664983*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1142] = 0.100717304493345*G0_0_0 + 0.125351877739029*G0_0_1 + 0.100717304493345*G0_0_2 + 0.0817390275940616*G0_1_0 + 0.18526817029421*G0_1_1 + 0.0817390275940616*G0_1_2 + 0.100717304493345*G0_2_0 + 0.125351877739029*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1143] = -0.032835224349981*G0_0_0 - 0.0537511252572011*G0_0_1 - 0.032835224349981*G0_0_2 - 0.0274594594256051*G0_1_0 - 0.0812105846828062*G0_1_1 - 0.0274594594256051*G0_1_2 - 0.032835224349981*G0_2_0 - 0.0537511252572011*G0_2_1 - 0.032835224349981*G0_2_2;
    A[1144] = 0.0196371995417131*G0_0_0 + 0.0895482395048355*G0_0_1 + 0.0895482395048355*G0_0_2 + 0.0895482395048355*G0_1_0 + 0.019637199541713*G0_1_1 + 0.0196371995417131*G0_1_2 + 0.0196371995417131*G0_2_0 + 0.0895482395048356*G0_2_1 + 0.0895482395048356*G0_2_2;
    A[1145] = 0.0124934342599274*G0_0_0 - 0.057417605703195*G0_0_1 - 0.057417605703195*G0_0_2 - 0.057417605703195*G0_1_0 - 0.0186912214907002*G0_1_1 - 0.0186912214907002*G0_1_2 + 0.0124934342599274*G0_2_0 - 0.057417605703195*G0_2_1 - 0.057417605703195*G0_2_2;
    A[1146] = -0.0315434750113568*G0_0_0 + 0.0283728175438241*G0_0_1 + 0.0283728175438241*G0_0_2 + 0.028372817543824*G0_1_0 + 0.00939454064454048*G0_1_1 + 0.00939454064454046*G0_1_2 - 0.0315434750113568*G0_2_0 + 0.0283728175438241*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1147] = 0.0220836945012292*G0_0_0 - 0.00537576492437574*G0_0_1 - 0.00537576492437572*G0_0_2 - 0.00537576492437591*G0_1_0 + 0.0220836945012292*G0_2_0 - 0.00537576492437575*G0_2_1 - 0.00537576492437574*G0_2_2;
    A[1148] = 0.0187890812890813*G0_0_0 + 0.00939454064454023*G0_0_1 + 0.0187890812890806*G0_0_2 + 0.00939454064454067*G0_1_0 + 0.00939454064453995*G0_1_2 + 0.0187890812890813*G0_2_0 + 0.00939454064454025*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1149] = 0.0292600797158085*G0_0_0 + 0.0259654665036615*G0_0_1 + 0.0292600797158091*G0_0_2 + 0.0259654665036602*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0259654665036608*G0_1_2 + 0.0292600797158086*G0_2_0 + 0.0259654665036616*G0_2_1 + 0.0292600797158092*G0_2_2;
    A[1150] = 0.0641307878721069*G0_0_0 + 0.0547362472275656*G0_0_1 + 0.0641307878721065*G0_0_2 + 0.0547362472275661*G0_1_0 + 0.0453417065830254*G0_1_1 + 0.0547362472275659*G0_1_2 + 0.064130787872107*G0_2_0 + 0.0547362472275657*G0_2_1 + 0.0641307878721065*G0_2_2;
    A[1151] = 0.00658922642429563*G0_0_0 + 0.00329461321214803*G0_0_1 + 0.00658922642429615*G0_0_2 + 0.00329461321214814*G0_1_0 + 0.00329461321214843*G0_1_2 + 0.0065892264242957*G0_2_0 + 0.00329461321214806*G0_2_1 + 0.00658922642429611*G0_2_2;
    A[1152] = 0.029260079715808*G0_0_0 + 0.02596546650366*G0_0_1 + 0.0292600797158077*G0_0_2 + 0.0259654665036606*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0259654665036602*G0_1_2 + 0.029260079715808*G0_2_0 + 0.0259654665036598*G0_2_1 + 0.0292600797158077*G0_2_2;
    A[1153] = 0.0187890812890814*G0_0_0 + 0.00939454064454068*G0_0_1 + 0.0187890812890811*G0_0_2 + 0.00939454064454056*G0_1_0 + 0.00939454064454074*G0_1_2 + 0.0187890812890812*G0_2_0 + 0.00939454064454068*G0_2_1 + 0.018789081289081*G0_2_2;
    A[1154] = -0.0644569872000418*G0_0_0 - 0.160751028806583*G0_0_1 - 0.257045070413123*G0_0_2 - 0.0929015685960121*G0_1_0 - 0.064456987200042*G0_1_1 - 0.160751028806583*G0_1_2 - 0.0644569872000419*G0_2_0 - 0.160751028806583*G0_2_1 - 0.257045070413123*G0_2_2;
    A[1155] = 0.127315597693201*G0_0_0 + 0.134361503176606*G0_0_1 + 0.176636936077038*G0_0_2 + 0.128000616281865*G0_1_0 + 0.0920860702761734*G0_1_1 + 0.134361503176606*G0_1_2 + 0.127315597693201*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1156] = -0.0641307878721069*G0_0_0 - 0.0906181733004645*G0_0_1 - 0.111886369481855*G0_0_2 - 0.0547362472275661*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.0906181733004641*G0_1_2 - 0.0641307878721069*G0_2_0 - 0.0906181733004646*G0_2_1 - 0.111886369481855*G0_2_2;
    A[1157] = 0.0439716694056968*G0_0_0 + 0.0862471023061293*G0_0_1 + 0.128522535206562*G0_0_2 + 0.0446566879943618*G0_1_0 + 0.043971669405697*G0_1_1 + 0.0862471023061298*G0_1_2 + 0.0439716694056967*G0_2_0 + 0.0862471023061293*G0_2_1 + 0.128522535206562*G0_2_2;
    A[1158] = -0.0292600797158081*G0_0_0 - 0.0377086423093365*G0_0_1 - 0.0435476102793805*G0_0_2 - 0.0259654665036606*G0_1_0 - 0.0318696743392923*G0_1_1 - 0.0377086423093363*G0_1_2 - 0.029260079715808*G0_2_0 - 0.0377086423093365*G0_2_1 - 0.0435476102793805*G0_2_2;
    A[1159] = -0.0187890812890814*G0_0_0 - 0.0400572774704716*G0_0_1 - 0.0613254736518626*G0_0_2 - 0.00939454064454062*G0_1_0 - 0.0187890812890814*G0_1_1 - 0.0400572774704722*G0_1_2 - 0.0187890812890813*G0_2_0 - 0.0400572774704716*G0_2_1 - 0.0613254736518625*G0_2_2;
    A[1160] = -0.128522535206561*G0_0_0 - 0.0322284936000207*G0_0_1 - 0.128522535206561*G0_0_2 - 0.0322284936000209*G0_1_0 - 0.0606730749959911*G0_1_1 - 0.0322284936000208*G0_1_2 - 0.128522535206561*G0_2_0 - 0.0322284936000206*G0_2_1 - 0.128522535206561*G0_2_2;
    A[1161] = 0.0642612676032806*G0_0_0 + 0.0219858347028479*G0_0_1 + 0.0642612676032805*G0_0_2 + 0.0219858347028481*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.0219858347028481*G0_1_2 + 0.0642612676032807*G0_2_0 + 0.021985834702848*G0_2_1 + 0.0642612676032804*G0_2_2;
    A[1162] = -0.0306627368259309*G0_0_0 - 0.00939454064454048*G0_0_1 - 0.0306627368259311*G0_0_2 - 0.00939454064454019*G0_1_0 - 0.0093945406445405*G0_1_2 - 0.0306627368259309*G0_2_0 - 0.00939454064454049*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[1163] = 0.0642612676032804*G0_0_0 + 0.0219858347028478*G0_0_1 + 0.0642612676032806*G0_0_2 + 0.0219858347028486*G0_1_0 + 0.0226708532915135*G0_1_1 + 0.0219858347028486*G0_1_2 + 0.0642612676032802*G0_2_0 + 0.0219858347028478*G0_2_1 + 0.0642612676032806*G0_2_2;
    A[1164] = -0.00913358118219179*G0_0_0 - 0.00329461321214784*G0_0_1 - 0.00913358118219155*G0_0_2 - 0.00329461321214816*G0_1_0 - 0.00329461321214785*G0_1_2 - 0.0091335811821916*G0_2_0 - 0.00329461321214788*G0_2_1 - 0.00913358118219148*G0_2_2;
    A[1165] = -0.0306627368259308*G0_0_0 - 0.00939454064454051*G0_0_1 - 0.0306627368259311*G0_0_2 - 0.00939454064454084*G0_1_0 - 0.00939454064454082*G0_1_2 - 0.0306627368259309*G0_2_0 - 0.0093945406445405*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[1166] = -0.257045070413123*G0_0_0 - 0.160751028806582*G0_0_1 - 0.0644569872000411*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000414*G0_1_1 - 0.0929015685960116*G0_1_2 - 0.257045070413123*G0_2_0 - 0.160751028806582*G0_2_1 - 0.0644569872000412*G0_2_2;
    A[1167] = 0.128522535206561*G0_0_0 + 0.0862471023061283*G0_0_1 + 0.0439716694056961*G0_0_2 + 0.086247102306129*G0_1_0 + 0.0439716694056962*G0_1_1 + 0.044656687994361*G0_1_2 + 0.128522535206561*G0_2_0 + 0.0862471023061284*G0_2_1 + 0.0439716694056962*G0_2_2;
    A[1168] = -0.0613254736518616*G0_0_0 - 0.0400572774704715*G0_0_1 - 0.0187890812890807*G0_0_2 - 0.0400572774704713*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.00939454064454015*G0_1_2 - 0.0613254736518617*G0_2_0 - 0.0400572774704717*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[1169] = 0.176636936077037*G0_0_0 + 0.134361503176605*G0_0_1 + 0.1273155976932*G0_0_2 + 0.134361503176605*G0_1_0 + 0.0920860702761731*G0_1_1 + 0.128000616281865*G0_1_2 + 0.176636936077037*G0_2_0 + 0.134361503176606*G0_2_1 + 0.1273155976932*G0_2_2;
    A[1170] = -0.0435476102793804*G0_0_0 - 0.0377086423093356*G0_0_1 - 0.0292600797158088*G0_0_2 - 0.0377086423093363*G0_1_0 - 0.031869674339292*G0_1_1 - 0.0259654665036608*G0_1_2 - 0.0435476102793805*G0_2_0 - 0.0377086423093357*G0_2_1 - 0.029260079715809*G0_2_2;
    A[1171] = -0.111886369481854*G0_0_0 - 0.0906181733004642*G0_0_1 - 0.0641307878721065*G0_0_2 - 0.0906181733004639*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.0547362472275658*G0_1_2 - 0.111886369481854*G0_2_0 - 0.0906181733004644*G0_2_1 - 0.0641307878721066*G0_2_2;
    A[1172] = 0.096228801740952*G0_0_0 + 0.13145832915798*G0_0_1 + 0.096228801740952*G0_0_2 + 0.131458329157981*G0_1_0 + 0.166687856575008*G0_1_1 + 0.131458329157981*G0_1_2 + 0.0962288017409522*G0_2_0 + 0.13145832915798*G0_2_1 + 0.096228801740952*G0_2_2;
    A[1173] = -0.0505608958299915*G0_0_0 - 0.0479513012065089*G0_0_1 - 0.0505608958299924*G0_0_2 - 0.0479513012065098*G0_1_0 - 0.0453417065830252*G0_1_1 - 0.0479513012065092*G0_1_2 - 0.0505608958299917*G0_2_0 - 0.047951301206509*G0_2_1 - 0.0505608958299925*G0_2_2;
    A[1174] = -0.156575677409009*G0_0_0 - 0.153966082785525*G0_0_1 - 0.156575677409008*G0_0_2 - 0.153966082785525*G0_1_0 - 0.151356488162043*G0_1_1 - 0.153966082785525*G0_1_2 - 0.156575677409009*G0_2_0 - 0.153966082785525*G0_2_1 - 0.156575677409008*G0_2_2;
    A[1175] = -0.0505608958299924*G0_0_0 - 0.0479513012065079*G0_0_1 - 0.050560895829992*G0_0_2 - 0.04795130120651*G0_1_0 - 0.0453417065830261*G0_1_1 - 0.0479513012065099*G0_1_2 - 0.0505608958299924*G0_2_0 - 0.0479513012065076*G0_2_1 - 0.0505608958299919*G0_2_2;
    A[1176] = 0.0214691349673985*G0_0_0 + 0.0214691349673985*G0_0_1 + 0.0214691349673985*G0_0_2 + 0.0551276864210885*G0_1_0 + 0.0551276864210885*G0_1_1 + 0.0551276864210885*G0_1_2 + 0.0214691349673985*G0_2_0 + 0.0214691349673985*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[1177] = -0.00336507226698202*G0_0_0 - 0.003365072266982*G0_2_0;
    A[1178] = -0.0232632312710436*G0_0_1 - 0.0388177200243171*G0_1_1 - 0.0232632312710436*G0_2_1;
    A[1179] = -0.00336507226698192*G0_0_2 - 0.00336507226698191*G0_2_2;
    A[1180] = -0.0599162925551811*G0_0_1 - 0.0788945694544647*G0_0_2 - 0.0315434750113565*G0_1_1 - 0.059916292555181*G0_1_2 - 0.059916292555181*G0_2_1 - 0.0788945694544646*G0_2_2;
    A[1181] = -0.00578025209101576*G0_0_1 - 0.0052322372200843*G0_0_2 - 0.00626302709636052*G0_1_1 - 0.00578025209101577*G0_1_2 - 0.00578025209101574*G0_2_1 - 0.00523223722008427*G0_2_2;
    A[1182] = -0.0131523569023568*G0_0_1 - 0.0263047138047133*G0_0_2 + 6.52398655874677e-05*G0_1_1 - 0.0131523569023567*G0_1_2 - 0.0131523569023568*G0_2_1 - 0.0263047138047133*G0_2_2;
    A[1183] = 0.00228991928210678*G0_0_1 + 0.0139743792087539*G0_0_2 + 0.00228991928210681*G0_1_2 + 0.00228991928210679*G0_2_1 + 0.0139743792087539*G0_2_2;
    A[1184] = 0.00228991928210687*G0_0_0 + 0.00228991928210685*G0_0_2 + 0.0022899192821067*G0_2_0 + 0.00228991928210676*G0_2_2;
    A[1185] = 0.00178757231708605*G0_0_0 + 0.00178757231708609*G0_0_2 + 0.00178757231708606*G0_2_0 + 0.00178757231708606*G0_2_2;
    A[1186] = 0.0017875723170863*G0_0_0 + 0.00178757231708634*G0_0_2 + 0.00178757231708624*G0_2_0 + 0.00178757231708629*G0_2_2;
    A[1187] = 0.00228991928210676*G0_0_0 + 0.00228991928210666*G0_0_2 + 0.00228991928210677*G0_2_0 + 0.00228991928210666*G0_2_2;
    A[1188] = 0.0139743792087543*G0_0_0 + 0.0022899192821069*G0_0_1 + 0.00228991928210698*G0_1_0 + 0.0139743792087541*G0_2_0 + 0.0022899192821068*G0_2_1;
    A[1189] = -0.0263047138047139*G0_0_0 - 0.013152356902357*G0_0_1 - 0.0131523569023571*G0_1_0 + 6.52398655871533e-05*G0_1_1 - 0.0263047138047137*G0_2_0 - 0.013152356902357*G0_2_1;
    A[1190] = -0.00523223722008379*G0_0_0 - 0.00578025209101549*G0_0_1 - 0.0057802520910155*G0_1_0 - 0.00626302709636047*G0_1_1 - 0.00523223722008413*G0_2_0 - 0.00578025209101569*G0_2_1;
    A[1191] = -0.0788945694544648*G0_0_0 - 0.059916292555181*G0_0_1 - 0.0599162925551811*G0_1_0 - 0.0315434750113565*G0_1_1 - 0.0788945694544647*G0_2_0 - 0.059916292555181*G0_2_1;
    A[1192] = -0.0574176057031953*G0_0_0 - 0.0574176057031954*G0_0_1 - 0.0186912214907007*G0_0_2 + 0.0124934342599272*G0_1_0 + 0.0124934342599273*G0_1_1 - 0.0574176057031954*G0_1_2 - 0.0574176057031955*G0_2_0 - 0.0574176057031955*G0_2_1 - 0.0186912214907007*G0_2_2;
    A[1193] = 0.00506261356955813*G0_0_0 + 0.00506261356955817*G0_0_1 + 0.00026095946234874*G0_0_2 + 0.000260959462348404*G0_1_0 + 0.000260959462348348*G0_1_1 + 0.00506261356955822*G0_1_2 + 0.00506261356955809*G0_2_0 + 0.00506261356955812*G0_2_1 + 0.000260959462348679*G0_2_2;
    A[1194] = -0.000482775005344579*G0_0_0 - 0.000482775005344542*G0_0_1 - 0.0062630270963605*G0_0_2 + 6.52398655868359e-05*G0_1_0 + 6.52398655868368e-05*G0_1_1 - 0.000482775005344614*G0_1_2 - 0.000482775005344598*G0_2_0 - 0.000482775005344563*G0_2_1 - 0.00626302709636043*G0_2_2;
    A[1195] = -0.00228991928210672*G0_0_0 - 0.00228991928210674*G0_0_1 + 0.00658922642429583*G0_0_2 - 0.00228991928210662*G0_1_2 - 0.00228991928210673*G0_2_0 - 0.00228991928210675*G0_2_1 + 0.00658922642429579*G0_2_2;
    A[1196] = -0.147063705006411*G0_0_0 - 0.108337320793917*G0_0_1 - 0.147063705006411*G0_0_2 - 0.17344670664983*G0_1_0 - 0.243357746612952*G0_1_1 - 0.17344670664983*G0_1_2 - 0.147063705006412*G0_2_0 - 0.108337320793917*G0_2_1 - 0.147063705006412*G0_2_2;
    A[1197] = 0.304226541205704*G0_0_0 + 0.226095278178609*G0_0_1 + 0.304226541205704*G0_0_2 + 0.226095278178609*G0_1_0 + 0.428965164208215*G0_1_1 + 0.226095278178609*G0_1_2 + 0.304226541205704*G0_2_0 + 0.226095278178609*G0_2_1 + 0.304226541205704*G0_2_2;
    A[1198] = -0.204187731314465*G0_0_0 - 0.219493003781196*G0_0_1 - 0.204187731314465*G0_0_2 - 0.167692550505049*G0_1_0 - 0.387185554286245*G0_1_1 - 0.167692550505049*G0_1_2 - 0.204187731314465*G0_2_0 - 0.219493003781196*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1199] = 0.0788945694544647*G0_0_0 + 0.103529142700149*G0_0_1 + 0.0788945694544647*G0_0_2 + 0.059916292555181*G0_1_0 + 0.18526817029421*G0_1_1 + 0.0599162925551809*G0_1_2 + 0.0788945694544646*G0_2_0 + 0.103529142700148*G0_2_1 + 0.0788945694544646*G0_2_2;
    A[1200] = -0.0186912214907004*G0_0_0 - 0.0574176057031952*G0_0_1 - 0.0574176057031953*G0_0_2 - 0.0574176057031951*G0_1_0 + 0.0124934342599274*G0_1_1 + 0.0124934342599273*G0_1_2 - 0.0186912214907004*G0_2_0 - 0.0574176057031954*G0_2_1 - 0.0574176057031954*G0_2_2;
    A[1201] = 0.000260959462348283*G0_0_0 + 0.00506261356955803*G0_0_1 + 0.005062613569558*G0_0_2 + 0.00506261356955758*G0_1_0 + 0.000260959462348217*G0_1_1 + 0.00026095946234824*G0_1_2 + 0.000260959462348299*G0_2_0 + 0.0050626135695582*G0_2_1 + 0.00506261356955818*G0_2_2;
    A[1202] = -0.00626302709636028*G0_0_0 - 0.00048277500534473*G0_0_1 - 0.000482775005344737*G0_0_2 - 0.000482775005344235*G0_1_0 + 6.52398655870773e-05*G0_1_1 + 6.52398655870955e-05*G0_1_2 - 0.00626302709636026*G0_2_0 - 0.000482775005344868*G0_2_1 - 0.000482775005344873*G0_2_2;
    A[1203] = 0.00658922642429578*G0_0_0 - 0.0022899192821068*G0_0_1 - 0.00228991928210688*G0_0_2 - 0.0022899192821069*G0_1_0 + 0.00658922642429576*G0_2_0 - 0.00228991928210671*G0_2_1 - 0.00228991928210678*G0_2_2;
    A[1204] = -0.0148746893538559*G0_0_0 + 6.52398655876157e-05*G0_0_1 - 0.0148746893538552*G0_0_2 + 6.5239865587175e-05*G0_1_0 + 6.52398655878151e-05*G0_1_2 - 0.0148746893538558*G0_2_0 + 6.52398655873686e-05*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1205] = -0.0189848008858425*G0_0_0 - 0.00949240044292221*G0_0_1 - 0.018984800885843*G0_0_2 - 0.00949240044292083*G0_1_0 - 0.00065239865587089*G0_1_1 - 0.00949240044292134*G0_1_2 - 0.018984800885843*G0_2_0 - 0.0094924004429222*G0_2_1 - 0.0189848008858432*G0_2_2;
    A[1206] = -0.127087258163646*G0_0_0 - 0.089313375988722*G0_0_1 - 0.127087258163645*G0_0_2 - 0.0893133759887225*G0_1_0 - 0.0665446628988287*G0_1_1 - 0.089313375988722*G0_1_2 - 0.127087258163646*G0_2_0 - 0.0893133759887223*G0_2_1 - 0.127087258163646*G0_2_2;
    A[1207] = 0.00450155072550967*G0_0_0 + 0.00257697469069014*G0_0_1 + 0.00450155072550873*G0_0_2 + 0.00257697469068974*G0_1_0 + 0.00257697469068931*G0_1_2 + 0.00450155072550946*G0_2_0 + 0.00257697469068994*G0_2_1 + 0.00450155072550856*G0_2_2;
    A[1208] = -0.0189848008858416*G0_0_0 - 0.00949240044292018*G0_0_1 - 0.0189848008858415*G0_0_2 - 0.00949240044292077*G0_1_0 - 0.000652398655870366*G0_1_1 - 0.00949240044292063*G0_1_2 - 0.0189848008858415*G0_2_0 - 0.00949240044292009*G0_2_1 - 0.0189848008858415*G0_2_2;
    A[1209] = -0.0148746893538561*G0_0_0 + 6.52398655869532e-05*G0_0_1 - 0.0148746893538554*G0_0_2 + 6.52398655872197e-05*G0_1_0 + 6.52398655871476e-05*G0_1_2 - 0.014874689353856*G0_2_0 + 6.52398655869435e-05*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1210] = 0.132306447410612*G0_0_0 + 0.194675758911868*G0_0_1 + 0.121346149991982*G0_0_2 + 0.0589768384907259*G0_1_0 - 0.00339247301052917*G0_1_1 + 0.194675758911868*G0_1_2 + 0.132306447410612*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1211] = -0.166948816037356*G0_0_0 - 0.250814663249556*G0_0_1 - 0.334680510461756*G0_0_2 - 0.160783648739376*G0_1_0 - 0.0622388317700806*G0_1_1 - 0.250814663249556*G0_1_2 - 0.166948816037356*G0_2_0 - 0.250814663249556*G0_2_1 - 0.334680510461756*G0_2_2;
    A[1212] = 0.127087258163646*G0_0_0 + 0.173472802596065*G0_0_1 + 0.20941996853455*G0_0_2 + 0.0893133759887223*G0_1_0 + 0.084681345532039*G0_1_1 + 0.173472802596065*G0_1_2 + 0.127087258163646*G0_2_0 + 0.173472802596065*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1213] = -0.0436454700777615*G0_0_0 - 0.0371541034518464*G0_0_1 + 0.0371867233846389*G0_0_2 - 0.00975335990526962*G0_1_0 - 0.00678494602105677*G0_1_1 - 0.0371541034518463*G0_1_2 - 0.0436454700777615*G0_2_0 - 0.0371541034518463*G0_2_1 + 0.037186723384639*G0_2_2;
    A[1214] = 0.0189848008858415*G0_0_0 + 0.037871741973304*G0_0_1 + 0.0567586830607652*G0_0_2 + 0.00949240044292077*G0_1_0 + 0.0127870136550689*G0_1_1 + 0.0378717419733036*G0_1_2 + 0.0189848008858414*G0_2_0 + 0.037871741973304*G0_2_1 + 0.0567586830607651*G0_2_2;
    A[1215] = 0.0148746893538561*G0_0_0 + 0.0136351319077016*G0_0_1 - 0.0254435475789631*G0_0_2 - 6.52398655871464e-05*G0_1_0 - 0.000130479731174187*G0_1_1 + 0.0136351319077018*G0_1_2 + 0.014874689353856*G0_2_0 + 0.0136351319077016*G0_2_1 - 0.025443547578963*G0_2_2;
    A[1216] = 0.0476251018785735*G0_0_0 - 0.0147442096226817*G0_0_1 + 0.0476251018785734*G0_0_2 - 0.0147442096226819*G0_1_0 + 0.0476251018785734*G0_1_1 - 0.0147442096226821*G0_1_2 + 0.0476251018785738*G0_2_0 - 0.0147442096226817*G0_2_1 + 0.0476251018785736*G0_2_2;
    A[1217] = 0.00880738185425633*G0_0_0 + 0.00231601522834134*G0_0_1 + 0.00880738185425653*G0_0_2 + 0.0023160152283417*G0_1_0 - 0.000652398655870779*G0_1_1 + 0.00231601522834169*G0_1_2 + 0.00880738185425614*G0_2_0 + 0.00231601522834126*G0_2_1 + 0.00880738185425647*G0_2_2;
    A[1218] = -0.00130479731174154*G0_0_0 - 6.52398655875285e-05*G0_0_1 - 0.00130479731174141*G0_0_2 - 6.5239865587201e-05*G0_1_0 - 6.52398655870389e-05*G0_1_2 - 0.00130479731174131*G0_2_0 - 6.52398655872809e-05*G0_2_1 - 0.00130479731174124*G0_2_2;
    A[1219] = 0.0088073818542565*G0_0_0 + 0.00231601522834136*G0_0_1 + 0.00880738185425665*G0_0_2 + 0.00231601522834165*G0_1_0 - 0.000652398655871202*G0_1_1 + 0.00231601522834179*G0_1_2 + 0.00880738185425659*G0_2_0 + 0.00231601522834138*G0_2_1 + 0.0088073818542567*G0_2_2;
    A[1220] = -0.0156575677409008*G0_0_0 - 0.00257697469069008*G0_0_1 - 0.0156575677409012*G0_0_2 - 0.00257697469069002*G0_1_0 - 0.0025769746906901*G0_1_2 - 0.0156575677409009*G0_2_0 - 0.00257697469068989*G0_2_1 - 0.0156575677409013*G0_2_2;
    A[1221] = -0.00130479731174166*G0_0_0 - 6.52398655871074e-05*G0_0_1 - 0.00130479731174131*G0_0_2 - 6.52398655868048e-05*G0_1_0 - 6.52398655868354e-05*G0_1_2 - 0.00130479731174158*G0_2_0 - 6.5239865587098e-05*G0_2_1 - 0.00130479731174127*G0_2_2;
    A[1222] = 0.121346149991981*G0_0_0 + 0.194675758911867*G0_0_1 + 0.132306447410612*G0_0_2 + 0.194675758911867*G0_1_0 - 0.00339247301052961*G0_1_1 + 0.0589768384907258*G0_1_2 + 0.121346149991982*G0_2_0 + 0.194675758911868*G0_2_1 + 0.132306447410612*G0_2_2;
    A[1223] = 0.0371867233846397*G0_0_0 - 0.0371541034518455*G0_0_1 - 0.0436454700777608*G0_0_2 - 0.0371541034518458*G0_1_0 - 0.00678494602105665*G0_1_1 - 0.00975335990526928*G0_1_2 + 0.0371867233846395*G0_2_0 - 0.0371541034518458*G0_2_1 - 0.0436454700777611*G0_2_2;
    A[1224] = -0.0254435475789639*G0_0_0 + 0.0136351319077014*G0_0_1 + 0.0148746893538553*G0_0_2 + 0.013635131907701*G0_1_0 - 0.000130479731174247*G0_1_1 - 6.52398655876302e-05*G0_1_2 - 0.0254435475789637*G0_2_0 + 0.0136351319077015*G0_2_1 + 0.0148746893538554*G0_2_2;
    A[1225] = -0.334680510461755*G0_0_0 - 0.250814663249556*G0_0_1 - 0.166948816037355*G0_0_2 - 0.250814663249555*G0_1_0 - 0.0622388317700801*G0_1_1 - 0.160783648739376*G0_1_2 - 0.334680510461755*G0_2_0 - 0.250814663249556*G0_2_1 - 0.166948816037355*G0_2_2;
    A[1226] = 0.0567586830607651*G0_0_0 + 0.0378717419733033*G0_0_1 + 0.0189848008858429*G0_0_2 + 0.0378717419733041*G0_1_0 + 0.0127870136550688*G0_1_1 + 0.00949240044292138*G0_1_2 + 0.0567586830607652*G0_2_0 + 0.0378717419733036*G0_2_1 + 0.018984800885843*G0_2_2;
    A[1227] = 0.209419968534549*G0_0_0 + 0.173472802596064*G0_0_1 + 0.127087258163645*G0_0_2 + 0.173472802596064*G0_1_0 + 0.084681345532039*G0_1_1 + 0.0893133759887217*G0_1_2 + 0.20941996853455*G0_2_0 + 0.173472802596065*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1228] = -0.156575677409008*G0_0_0 - 0.0662184635708931*G0_0_1 - 0.156575677409009*G0_0_2 - 0.0662184635708929*G0_1_0 - 0.161794866655976*G0_1_1 - 0.066218463570893*G0_1_2 - 0.156575677409008*G0_2_0 - 0.0662184635708932*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1229] = 0.0391439193522517*G0_0_0 + 0.00717638521458053*G0_0_1 + 0.0391439193522524*G0_0_2 + 0.00717638521457997*G0_1_0 + 0.00130479731174142*G0_1_1 + 0.00717638521457979*G0_1_2 + 0.0391439193522519*G0_2_0 + 0.0071763852145806*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[1230] = 0.185933616923198*G0_0_0 + 0.170276049182297*G0_0_1 + 0.185933616923197*G0_0_2 + 0.170276049182297*G0_1_0 + 0.180714427676232*G0_1_1 + 0.170276049182298*G0_1_2 + 0.185933616923199*G0_2_0 + 0.170276049182298*G0_2_1 + 0.185933616923198*G0_2_2;
    A[1231] = 0.0391439193522518*G0_0_0 + 0.00717638521457858*G0_0_1 + 0.0391439193522514*G0_0_2 + 0.00717638521457965*G0_1_0 + 0.00130479731174151*G0_1_1 + 0.00717638521457927*G0_1_2 + 0.039143919352252*G0_2_0 + 0.00717638521457846*G0_2_1 + 0.0391439193522512*G0_2_2;
    A[1232] = -0.0155544887532733*G0_0_0 - 0.0155544887532733*G0_0_1 - 0.0155544887532733*G0_0_2 - 0.0388177200243167*G0_1_0 - 0.0388177200243168*G0_1_1 - 0.0388177200243167*G0_1_2 - 0.0155544887532733*G0_2_0 - 0.0155544887532733*G0_2_1 - 0.0155544887532733*G0_2_2;
    A[1233] = -0.00336507226698187*G0_0_0 - 0.00336507226698187*G0_2_0;
    A[1234] = 0.03365855145369*G0_0_1 + 0.055127686421089*G0_1_1 + 0.03365855145369*G0_2_1;
    A[1235] = -0.00336507226698197*G0_0_2 - 0.00336507226698198*G0_2_2;
    A[1236] = 0.069911039963123*G0_0_1 + 0.108637424175618*G0_0_2 + 0.0124934342599269*G0_1_1 + 0.0699110399631227*G0_1_2 + 0.0699110399631229*G0_2_1 + 0.108637424175618*G0_2_2;
    A[1237] = -0.00480165410720976*G0_0_1 - 0.00960330821441937*G0_0_2 + 0.000260959462348599*G0_1_1 - 0.00480165410720959*G0_1_2 - 0.00480165410720972*G0_2_1 - 0.00960330821441927*G0_2_2;
    A[1238] = 0.000548014870931887*G0_0_1 - 0.00523223722008412*G0_0_2 + 6.52398655866622e-05*G0_1_1 + 0.000548014870931324*G0_1_2 + 0.000548014870931822*G0_2_1 - 0.00523223722008426*G0_2_2;
    A[1239] = 0.00228991928210654*G0_0_1 + 0.0111690649885092*G0_0_2 + 0.00228991928210671*G0_1_2 + 0.00228991928210656*G0_2_1 + 0.0111690649885093*G0_2_2;
    A[1240] = 0.00228991928210642*G0_0_0 + 0.00228991928210663*G0_0_2 + 0.00228991928210656*G0_2_0 + 0.00228991928210676*G0_2_2;
    A[1241] = 0.00178757231708649*G0_0_0 + 0.00178757231708617*G0_0_2 + 0.0017875723170865*G0_2_0 + 0.00178757231708616*G0_2_2;
    A[1242] = 0.00178757231708595*G0_0_0 + 0.00178757231708621*G0_0_2 + 0.001787572317086*G0_2_0 + 0.0017875723170863*G0_2_2;
    A[1243] = 0.00228991928210661*G0_0_0 + 0.00228991928210651*G0_0_2 + 0.00228991928210663*G0_2_0 + 0.00228991928210652*G0_2_2;
    A[1244] = 0.0111690649885089*G0_0_0 + 0.00228991928210652*G0_0_1 + 0.00228991928210651*G0_1_0 + 0.011169064988509*G0_2_0 + 0.00228991928210665*G0_2_1;
    A[1245] = -0.00523223722008393*G0_0_0 + 0.000548014870931894*G0_0_1 + 0.00054801487093181*G0_1_0 + 6.52398655872291e-05*G0_1_1 - 0.00523223722008401*G0_2_0 + 0.000548014870931833*G0_2_1;
    A[1246] = -0.00960330821441967*G0_0_0 - 0.00480165410720997*G0_0_1 - 0.00480165410721007*G0_1_0 + 0.000260959462348257*G0_1_1 - 0.00960330821441957*G0_2_0 - 0.00480165410720985*G0_2_1;
    A[1247] = 0.108637424175618*G0_0_0 + 0.0699110399631228*G0_0_1 + 0.0699110399631228*G0_1_0 + 0.012493434259927*G0_1_1 + 0.108637424175618*G0_2_0 + 0.0699110399631227*G0_2_1;
    A[1248] = 0.0283728175438243*G0_0_0 + 0.0283728175438243*G0_0_1 + 0.00939454064454081*G0_0_2 - 0.0315434750113567*G0_1_0 - 0.0315434750113568*G0_1_1 + 0.0283728175438242*G0_1_2 + 0.0283728175438243*G0_2_0 + 0.0283728175438244*G0_2_1 + 0.00939454064454083*G0_2_2;
    A[1249] = -0.000482775005344526*G0_0_0 - 0.00048277500534448*G0_0_1 + 6.5239865586864e-05*G0_0_2 - 0.00626302709636012*G0_1_0 - 0.00626302709636004*G0_1_1 - 0.00048277500534467*G0_1_2 - 0.000482775005344508*G0_2_0 - 0.00048277500534446*G0_2_1 + 6.52398655868569e-05*G0_2_2;
    A[1250] = 0.0132175967679438*G0_0_0 + 0.0132175967679438*G0_0_1 + 6.52398655871054e-05*G0_0_2 + 6.5239865587125e-05*G0_1_0 + 6.52398655871002e-05*G0_1_1 + 0.0132175967679441*G0_1_2 + 0.0132175967679439*G0_2_0 + 0.0132175967679438*G0_2_1 + 6.5239865587123e-05*G0_2_2;
    A[1251] = -0.00228991928210662*G0_0_0 - 0.00228991928210661*G0_0_1 + 0.00939454064454062*G0_0_2 - 0.00228991928210698*G0_1_2 - 0.00228991928210664*G0_2_0 - 0.00228991928210663*G0_2_1 + 0.0093945406445406*G0_2_2;
    A[1252] = 0.100717304493345*G0_0_0 + 0.0817390275940616*G0_0_1 + 0.100717304493345*G0_0_2 + 0.125351877739029*G0_1_0 + 0.18526817029421*G0_1_1 + 0.125351877739029*G0_1_2 + 0.100717304493345*G0_2_0 + 0.0817390275940616*G0_2_1 + 0.100717304493345*G0_2_2;
    A[1253] = -0.204187731314465*G0_0_0 - 0.167692550505049*G0_0_1 - 0.204187731314465*G0_0_2 - 0.219493003781196*G0_1_0 - 0.387185554286245*G0_1_1 - 0.219493003781196*G0_1_2 - 0.204187731314465*G0_2_0 - 0.167692550505049*G0_2_1 - 0.204187731314465*G0_2_2;
    A[1254] = 0.281001149056702*G0_0_0 + 0.202869886029606*G0_0_1 + 0.281001149056702*G0_0_2 + 0.202869886029606*G0_1_0 + 0.428965164208216*G0_1_1 + 0.202869886029606*G0_1_2 + 0.281001149056702*G0_2_0 + 0.202869886029606*G0_2_1 + 0.281001149056702*G0_2_2;
    A[1255] = -0.108637424175618*G0_0_0 - 0.135020425819036*G0_0_1 - 0.108637424175618*G0_0_2 - 0.0699110399631227*G0_1_0 - 0.243357746612953*G0_1_1 - 0.0699110399631226*G0_1_2 - 0.108637424175618*G0_2_0 - 0.135020425819036*G0_2_1 - 0.108637424175618*G0_2_2;
    A[1256] = 0.00939454064454055*G0_0_0 + 0.028372817543824*G0_0_1 + 0.0283728175438241*G0_0_2 + 0.0283728175438239*G0_1_0 - 0.0315434750113571*G0_1_1 - 0.031543475011357*G0_1_2 + 0.00939454064454058*G0_2_0 + 0.0283728175438241*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1257] = 6.52398655873156e-05*G0_0_0 - 0.00048277500534418*G0_0_1 - 0.000482775005344161*G0_0_2 - 0.000482775005344021*G0_1_0 - 0.0062630270963602*G0_1_1 - 0.00626302709636023*G0_1_2 + 6.52398655873111e-05*G0_2_0 - 0.000482775005344372*G0_2_1 - 0.000482775005344357*G0_2_2;
    A[1258] = 6.52398655869847e-05*G0_0_0 + 0.0132175967679439*G0_0_1 + 0.0132175967679439*G0_0_2 + 0.0132175967679435*G0_1_0 + 6.52398655869531e-05*G0_1_1 + 6.52398655869099e-05*G0_1_2 + 6.52398655869602e-05*G0_2_0 + 0.013217596767944*G0_2_1 + 0.0132175967679441*G0_2_2;
    A[1259] = 0.00939454064454047*G0_0_0 - 0.00228991928210671*G0_0_1 - 0.00228991928210665*G0_0_2 - 0.00228991928210649*G0_1_0 + 0.00939454064454048*G0_2_0 - 0.00228991928210683*G0_2_1 - 0.00228991928210678*G0_2_2;
    A[1260] = -0.00117431758056698*G0_0_0 + 6.52398655869297e-05*G0_0_1 - 0.00117431758056785*G0_0_2 + 6.52398655863161e-05*G0_1_0 + 6.52398655861871e-05*G0_1_2 - 0.00117431758056687*G0_2_0 + 6.52398655872279e-05*G0_2_1 - 0.0011743175805676*G0_2_2;
    A[1261] = 0.00352295274170199*G0_0_0 - 0.00296841388421202*G0_0_1 + 0.00352295274170313*G0_0_2 - 0.00296841388421277*G0_1_0 - 0.000652398655870915*G0_1_1 - 0.00296841388421254*G0_1_2 + 0.00352295274170215*G0_2_0 - 0.00296841388421208*G0_2_1 + 0.00352295274170315*G0_2_2;
    A[1262] = 0.124738623002511*G0_0_0 + 0.0623693115012547*G0_0_1 + 0.124738623002511*G0_0_2 + 0.0623693115012556*G0_1_0 + 0.0476251018785735*G0_1_1 + 0.062369311501255*G0_1_2 + 0.124738623002511*G0_2_0 + 0.0623693115012549*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1263] = -0.0105036183595215*G0_0_0 + 0.00257697469069001*G0_0_1 - 0.0105036183595206*G0_0_2 + 0.0025769746906901*G0_1_0 + 0.00257697469069021*G0_1_2 - 0.0105036183595214*G0_2_0 + 0.00257697469069014*G0_2_1 - 0.0105036183595205*G0_2_2;
    A[1264] = 0.00352295274170224*G0_0_0 - 0.00296841388421302*G0_0_1 + 0.00352295274170137*G0_0_2 - 0.00296841388421262*G0_1_0 - 0.000652398655870937*G0_1_1 - 0.00296841388421241*G0_1_2 + 0.00352295274170225*G0_2_0 - 0.00296841388421306*G0_2_1 + 0.00352295274170157*G0_2_2;
    A[1265] = -0.00117431758056727*G0_0_0 + 6.52398655868418e-05*G0_0_1 - 0.00117431758056753*G0_0_2 + 6.52398655867767e-05*G0_1_0 + 6.52398655867897e-05*G0_1_2 - 0.0011743175805673*G0_2_0 + 6.52398655869224e-05*G0_2_1 - 0.00117431758056759*G0_2_2;
    A[1266] = -0.0510175748891023*G0_0_0 - 0.088791457064026*G0_0_1 - 0.0528442911255409*G0_0_2 - 0.00463203045668203*G0_1_0 + 0.08468134553204*G0_1_1 - 0.0887914570640248*G0_1_2 - 0.0510175748891021*G0_2_0 - 0.088791457064026*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[1267] = 0.182410664181496*G0_0_0 + 0.188575831479476*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0985448169692952*G0_1_0 - 0.0622388317700819*G0_1_1 + 0.188575831479475*G0_1_2 + 0.182410664181496*G0_2_0 + 0.188575831479476*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1268] = -0.124738623002511*G0_0_0 - 0.198068231922397*G0_0_1 - 0.271397840842283*G0_0_2 - 0.0623693115012555*G0_1_0 - 0.00339247301052815*G0_1_1 - 0.198068231922397*G0_1_2 - 0.124738623002511*G0_2_0 - 0.198068231922397*G0_2_1 - 0.271397840842283*G0_2_2;
    A[1269] = -0.0155923278753131*G0_0_0 - 0.0250847283182345*G0_0_1 - 0.00619778723077262*G0_0_2 + 0.00329461321214784*G0_1_0 + 0.0127870136550686*G0_1_1 - 0.0250847283182355*G0_1_2 - 0.0155923278753131*G0_2_0 - 0.0250847283182346*G0_2_1 - 0.00619778723077274*G0_2_2;
    A[1270] = -0.00352295274170223*G0_0_0 + 0.0303691574307887*G0_0_1 + 0.104709984267275*G0_0_2 + 0.00296841388421262*G0_1_0 - 0.00678494602105707*G0_1_1 + 0.0303691574307897*G0_1_2 - 0.00352295274170223*G0_2_0 + 0.0303691574307888*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1271] = 0.00117431758056728*G0_0_0 - 0.0137656116388757*G0_0_1 - 0.0528442911255411*G0_0_2 - 6.52398655868324e-05*G0_1_0 - 0.000130479731173802*G0_1_1 - 0.0137656116388756*G0_1_2 + 0.00117431758056732*G0_2_0 - 0.0137656116388757*G0_2_1 - 0.0528442911255411*G0_2_2;
    A[1272] = -0.0150051690850297*G0_0_0 + 0.0227687130898937*G0_0_1 - 0.0150051690850297*G0_0_2 + 0.0227687130898936*G0_1_0 - 0.0665446628988286*G0_1_1 + 0.0227687130898942*G0_1_2 - 0.0150051690850299*G0_2_0 + 0.0227687130898936*G0_2_1 - 0.0150051690850297*G0_2_2;
    A[1273] = -0.000652398655871126*G0_0_0 + 0.00884000178705031*G0_0_1 - 0.000652398655871381*G0_0_2 + 0.00884000178705048*G0_1_0 - 0.000652398655871366*G0_1_1 + 0.00884000178705014*G0_1_2 - 0.000652398655870918*G0_2_0 + 0.00884000178705047*G0_2_1 - 0.000652398655871239*G0_2_2;
    A[1274] = -0.0150051690850298*G0_0_0 - 6.52398655867523e-05*G0_0_1 - 0.0150051690850301*G0_0_2 - 6.52398655871612e-05*G0_1_0 - 6.52398655868953e-05*G0_1_2 - 0.01500516908503*G0_2_0 - 6.52398655870485e-05*G0_2_1 - 0.0150051690850303*G0_2_2;
    A[1275] = -0.000652398655870972*G0_0_0 + 0.00884000178705036*G0_0_1 - 0.000652398655871004*G0_0_2 + 0.00884000178704964*G0_1_0 - 0.000652398655871112*G0_1_1 + 0.00884000178704955*G0_1_2 - 0.00065239865587098*G0_2_0 + 0.00884000178705045*G0_2_1 - 0.000652398655871089*G0_2_2;
    A[1276] = -0.00065239865587092*G0_0_0 - 0.00257697469069002*G0_0_1 - 0.000652398655870427*G0_0_2 - 0.00257697469068941*G0_1_0 - 0.00257697469068968*G0_1_2 - 0.00065239865587094*G0_2_0 - 0.00257697469069015*G0_2_1 - 0.000652398655870459*G0_2_2;
    A[1277] = -0.0150051690850299*G0_0_0 - 6.52398655867815e-05*G0_0_1 - 0.0150051690850301*G0_0_2 - 6.5239865586972e-05*G0_1_0 - 6.52398655869111e-05*G0_1_2 - 0.01500516908503*G0_2_0 - 6.52398655868616e-05*G0_2_1 - 0.0150051690850301*G0_2_2;
    A[1278] = -0.0528442911255401*G0_0_0 - 0.0887914570640254*G0_0_1 - 0.051017574889102*G0_0_2 - 0.088791457064025*G0_1_0 + 0.0846813455320403*G0_1_1 - 0.00463203045668245*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0887914570640254*G0_2_1 - 0.051017574889102*G0_2_2;
    A[1279] = -0.00619778723077358*G0_0_0 - 0.0250847283182355*G0_0_1 - 0.0155923278753138*G0_0_2 - 0.0250847283182352*G0_1_0 + 0.012787013655069*G0_1_1 + 0.00329461321214758*G0_1_2 - 0.00619778723077334*G0_2_0 - 0.0250847283182353*G0_2_1 - 0.0155923278753135*G0_2_2;
    A[1280] = -0.0528442911255402*G0_0_0 - 0.0137656116388753*G0_0_1 + 0.00117431758056802*G0_0_2 - 0.0137656116388752*G0_1_0 - 0.000130479731174156*G0_1_1 - 6.5239865586256e-05*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0137656116388754*G0_2_1 + 0.00117431758056778*G0_2_2;
    A[1281] = 0.104709984267274*G0_0_0 + 0.188575831479475*G0_0_1 + 0.182410664181495*G0_0_2 + 0.188575831479475*G0_1_0 - 0.062238831770082*G0_1_1 + 0.0985448169692952*G0_1_2 + 0.104709984267274*G0_2_0 + 0.188575831479475*G0_2_1 + 0.182410664181495*G0_2_2;
    A[1282] = 0.104709984267275*G0_0_0 + 0.0303691574307892*G0_0_1 - 0.0035229527417032*G0_0_2 + 0.030369157430789*G0_1_0 - 0.00678494602105694*G0_1_1 + 0.00296841388421241*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0303691574307891*G0_2_1 - 0.0035229527417032*G0_2_2;
    A[1283] = -0.271397840842283*G0_0_0 - 0.198068231922397*G0_0_1 - 0.12473862300251*G0_0_2 - 0.198068231922396*G0_1_0 - 0.0033924730105281*G0_1_1 - 0.0623693115012546*G0_1_2 - 0.271397840842282*G0_2_0 - 0.198068231922396*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1284] = 0.0260959462348356*G0_0_0 + 0.0104383784939343*G0_0_1 + 0.0260959462348354*G0_0_2 + 0.0104383784939327*G0_1_0 + 0.180714427676232*G0_1_1 + 0.0104383784939336*G0_1_2 + 0.0260959462348356*G0_2_0 + 0.0104383784939344*G0_2_1 + 0.0260959462348355*G0_2_2;
    A[1285] = 0.0260959462348336*G0_0_0 - 0.00587158790283858*G0_0_1 + 0.0260959462348338*G0_0_2 - 0.00587158790283717*G0_1_0 + 0.00130479731174249*G0_1_1 - 0.00587158790283778*G0_1_2 + 0.0260959462348333*G0_2_0 - 0.00587158790283869*G0_2_1 + 0.0260959462348335*G0_2_2;
    A[1286] = -0.185933616923198*G0_0_0 - 0.0955764030850827*G0_0_1 - 0.185933616923197*G0_0_2 - 0.0955764030850826*G0_1_0 - 0.161794866655977*G0_1_1 - 0.0955764030850832*G0_1_2 - 0.185933616923198*G0_2_0 - 0.095576403085083*G0_2_1 - 0.185933616923197*G0_2_2;
    A[1287] = 0.026095946234835*G0_0_0 - 0.00587158790283703*G0_0_1 + 0.0260959462348344*G0_0_2 - 0.00587158790283771*G0_1_0 + 0.00130479731174201*G0_1_1 - 0.00587158790283719*G0_1_2 + 0.0260959462348349*G0_2_0 - 0.00587158790283711*G0_2_1 + 0.0260959462348344*G0_2_2;
    A[1288] = 0.0048003493098979*G0_0_0 + 0.00480034930989791*G0_0_1 + 0.00480034930989792*G0_0_2 + 0.0153313684129655*G0_1_0 + 0.0153313684129655*G0_1_1 + 0.0153313684129655*G0_1_2 + 0.0048003493098979*G0_2_0 + 0.00480034930989791*G0_2_1 + 0.00480034930989792*G0_2_2;
    A[1289] = -0.00480034930989797*G0_0_0 - 0.00480034930989798*G0_2_0;
    A[1290] = -0.0372571824394738*G0_0_1 - 0.0551276864210889*G0_1_1 - 0.0372571824394738*G0_2_1;
    A[1291] = -0.00480034930989782*G0_0_2 - 0.00480034930989782*G0_2_2;
    A[1292] = -0.069911039963123*G0_0_1 - 0.139822079926246*G0_0_2 + 0.0196371995417135*G0_1_1 - 0.0699110399631227*G0_1_2 - 0.069911039963123*G0_2_1 - 0.139822079926246*G0_2_2;
    A[1293] = 0.038726384212495*G0_0_1 + 0.108637424175618*G0_0_2 - 0.0186912214907008*G0_1_1 + 0.0387263842124949*G0_1_2 + 0.038726384212495*G0_2_1 + 0.108637424175618*G0_2_2;
    A[1294] = -0.0189782768992839*G0_0_1 - 0.0788945694544647*G0_0_2 + 0.00939454064454078*G0_1_1 - 0.0189782768992837*G0_1_2 - 0.0189782768992839*G0_2_1 - 0.0788945694544647*G0_2_2;
    A[1295] = 0.00537576492437609*G0_0_1 + 0.0328352243499809*G0_0_2 + 0.00537576492437604*G0_1_2 + 0.00537576492437611*G0_2_1 + 0.0328352243499809*G0_2_2;
    A[1296] = 0.00537576492437618*G0_0_0 + 0.00537576492437605*G0_0_2 + 0.00537576492437615*G0_2_0 + 0.005375764924376*G0_2_2;
    A[1297] = 0.0022899192821066*G0_0_0 + 0.00228991928210673*G0_0_2 + 0.00228991928210661*G0_2_0 + 0.00228991928210678*G0_2_2;
    A[1298] = 0.00228991928210665*G0_0_0 + 0.00228991928210672*G0_0_2 + 0.00228991928210666*G0_2_0 + 0.00228991928210668*G0_2_2;
    A[1299] = 0.00537576492437611*G0_0_0 + 0.00537576492437594*G0_0_2 + 0.00537576492437613*G0_2_0 + 0.00537576492437598*G0_2_2;
    A[1300] = 0.0328352243499813*G0_0_0 + 0.00537576492437611*G0_0_1 + 0.00537576492437614*G0_1_0 + 0.0328352243499812*G0_2_0 + 0.00537576492437606*G0_2_1;
    A[1301] = -0.0788945694544649*G0_0_0 - 0.0189782768992838*G0_0_1 - 0.0189782768992838*G0_1_0 + 0.00939454064454043*G0_1_1 - 0.0788945694544649*G0_2_0 - 0.0189782768992839*G0_2_1;
    A[1302] = 0.108637424175618*G0_0_0 + 0.038726384212495*G0_0_1 + 0.0387263842124952*G0_1_0 - 0.0186912214907005*G0_1_1 + 0.108637424175618*G0_2_0 + 0.0387263842124951*G0_2_1;
    A[1303] = -0.139822079926246*G0_0_0 - 0.0699110399631229*G0_0_1 - 0.0699110399631228*G0_1_0 + 0.0196371995417134*G0_1_1 - 0.139822079926246*G0_2_0 - 0.0699110399631228*G0_2_1;
    A[1304] = -0.00537576492437607*G0_0_0 - 0.00537576492437609*G0_0_1 + 0.0220836945012291*G0_1_0 + 0.0220836945012291*G0_1_1 - 0.00537576492437601*G0_1_2 - 0.00537576492437608*G0_2_0 - 0.0053757649243761*G0_2_1;
    A[1305] = -0.00228991928210659*G0_0_0 - 0.00228991928210665*G0_0_1 + 0.00658922642429559*G0_1_0 + 0.00658922642429552*G0_1_1 - 0.00228991928210669*G0_1_2 - 0.00228991928210661*G0_2_0 - 0.00228991928210667*G0_2_1;
    A[1306] = -0.0022899192821068*G0_0_0 - 0.00228991928210677*G0_0_1 + 0.00939454064454063*G0_1_0 + 0.00939454064454067*G0_1_1 - 0.00228991928210685*G0_1_2 - 0.00228991928210681*G0_2_0 - 0.00228991928210678*G0_2_1;
    A[1307] = -0.00537576492437607*G0_0_0 - 0.00537576492437607*G0_0_1 - 0.00537576492437591*G0_1_2 - 0.00537576492437609*G0_2_0 - 0.00537576492437609*G0_2_1;
    A[1308] = -0.032835224349981*G0_0_0 - 0.0274594594256051*G0_0_1 - 0.032835224349981*G0_0_2 - 0.0537511252572011*G0_1_0 - 0.0812105846828063*G0_1_1 - 0.0537511252572011*G0_1_2 - 0.032835224349981*G0_2_0 - 0.0274594594256051*G0_2_1 - 0.032835224349981*G0_2_2;
    A[1309] = 0.0788945694544647*G0_0_0 + 0.059916292555181*G0_0_1 + 0.0788945694544646*G0_0_2 + 0.103529142700149*G0_1_0 + 0.18526817029421*G0_1_1 + 0.103529142700148*G0_1_2 + 0.0788945694544647*G0_2_0 + 0.0599162925551809*G0_2_1 + 0.0788945694544646*G0_2_2;
    A[1310] = -0.108637424175618*G0_0_0 - 0.0699110399631227*G0_0_1 - 0.108637424175618*G0_0_2 - 0.135020425819036*G0_1_0 - 0.243357746612953*G0_1_1 - 0.135020425819036*G0_1_2 - 0.108637424175618*G0_2_0 - 0.0699110399631226*G0_2_1 - 0.108637424175618*G0_2_2;
    A[1311] = 0.139822079926246*G0_0_0 + 0.0699110399631227*G0_0_1 + 0.139822079926246*G0_0_2 + 0.0699110399631227*G0_1_0 + 0.179096479009672*G0_1_1 + 0.0699110399631227*G0_1_2 + 0.139822079926246*G0_2_0 + 0.0699110399631227*G0_2_1 + 0.139822079926246*G0_2_2;
    A[1312] = -0.00537576492437591*G0_0_1 - 0.00537576492437593*G0_0_2 - 0.00537576492437589*G0_1_0 + 0.0220836945012292*G0_1_1 + 0.0220836945012292*G0_1_2 - 0.00537576492437592*G0_2_1 - 0.00537576492437594*G0_2_2;
    A[1313] = -0.00228991928210691*G0_0_1 - 0.00228991928210686*G0_0_2 - 0.00228991928210689*G0_1_0 + 0.00658922642429577*G0_1_1 + 0.0065892264242958*G0_1_2 - 0.00228991928210689*G0_2_1 - 0.00228991928210684*G0_2_2;
    A[1314] = -0.00228991928210679*G0_0_1 - 0.00228991928210684*G0_0_2 - 0.00228991928210662*G0_1_0 + 0.00939454064454072*G0_1_1 + 0.00939454064454074*G0_1_2 - 0.00228991928210685*G0_2_1 - 0.00228991928210691*G0_2_2;
    A[1315] = -0.0053757649243759*G0_0_1 - 0.00537576492437601*G0_0_2 - 0.00537576492437616*G0_1_0 - 0.00537576492437585*G0_2_1 - 0.00537576492437596*G0_2_2;
    A[1316] = -0.0118736555368504*G0_0_0 + 0.00939454064454059*G0_0_1 - 0.0118736555368496*G0_0_2 + 0.00939454064454101*G0_1_0 + 0.00939454064454098*G0_1_2 - 0.0118736555368505*G0_2_0 + 0.00939454064454048*G0_2_1 - 0.0118736555368499*G0_2_2;
    A[1317] = 0.0429604514890969*G0_0_0 + 0.000685018588664034*G0_0_1 + 0.0429604514890962*G0_0_2 + 0.000685018588664752*G0_1_0 + 0.0226708532915131*G0_1_1 + 0.000685018588664638*G0_1_2 + 0.0429604514890969*G0_2_0 + 0.000685018588664022*G0_2_1 + 0.0429604514890963*G0_2_2;
    A[1318] = -0.124738623002512*G0_0_0 - 0.0284445813959707*G0_0_1 - 0.124738623002511*G0_0_2 - 0.0284445813959706*G0_1_0 - 0.0606730749959915*G0_1_1 - 0.0284445813959701*G0_1_2 - 0.124738623002512*G0_2_0 - 0.0284445813959707*G0_2_1 - 0.124738623002511*G0_2_2;
    A[1319] = -0.00254435475789585*G0_0_0 + 0.00329461321214787*G0_0_1 - 0.00254435475789671*G0_0_2 + 0.003294613212148*G0_1_0 + 0.00329461321214794*G0_1_2 - 0.00254435475789584*G0_2_0 + 0.00329461321214789*G0_2_1 - 0.00254435475789665*G0_2_2;
    A[1320] = 0.0429604514890974*G0_0_0 + 0.000685018588665243*G0_0_1 + 0.0429604514890978*G0_0_2 + 0.000685018588664225*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.000685018588664163*G0_1_2 + 0.0429604514890973*G0_2_0 + 0.000685018588665096*G0_2_1 + 0.0429604514890976*G0_2_2;
    A[1321] = -0.0118736555368501*G0_0_0 + 0.00939454064454041*G0_0_1 - 0.0118736555368497*G0_0_2 + 0.00939454064454069*G0_1_0 + 0.0093945406445408*G0_1_2 - 0.0118736555368502*G0_2_0 + 0.00939454064454043*G0_2_1 - 0.0118736555368498*G0_2_2;
    A[1322] = 0.0118736555368502*G0_0_0 + 0.0212681961813908*G0_0_1 - 0.014613729891508*G0_1_0 - 0.0693499771190739*G0_1_1 + 0.02126819618139*G0_1_2 + 0.0118736555368501*G0_2_0 + 0.0212681961813908*G0_2_1;
    A[1323] = -0.0429604514890973*G0_0_0 - 0.0422754329004329*G0_0_1 - 0.0359145460056917*G0_1_0 + 0.0920860702761739*G0_1_1 - 0.0422754329004323*G0_1_2 - 0.0429604514890974*G0_2_0 - 0.0422754329004329*G0_2_1;
    A[1324] = 0.124738623002512*G0_0_0 + 0.0962940416065413*G0_0_1 + 0.0284445813959705*G0_1_0 - 0.0644569872000423*G0_1_1 + 0.0962940416065411*G0_1_2 + 0.124738623002512*G0_2_0 + 0.0962940416065413*G0_2_1;
    A[1325] = 0.00254435475789602*G0_0_0 + 0.00583896797004394*G0_0_1 - 0.00590420783563172*G0_1_0 - 0.0318696743392919*G0_1_1 + 0.00583896797004462*G0_1_2 + 0.00254435475789593*G0_2_0 + 0.00583896797004394*G0_2_1;
    A[1326] = -0.0429604514890975*G0_0_0 - 0.0422754329004324*G0_0_1 - 0.000685018588664221*G0_1_0 + 0.0439716694056972*G0_1_1 - 0.0422754329004328*G0_1_2 - 0.0429604514890973*G0_2_0 - 0.0422754329004324*G0_2_1;
    A[1327] = 0.0118736555368501*G0_0_0 + 0.0212681961813907*G0_0_1 - 0.00939454064454067*G0_1_0 - 0.0187890812890814*G0_1_1 + 0.0212681961813906*G0_1_2 + 0.0118736555368502*G0_2_0 + 0.0212681961813907*G0_2_1;
    A[1328] = -0.00939454064454047*G0_0_1 - 0.00939454064454065*G0_1_0 + 0.0453417065830255*G0_1_1 - 0.00939454064454096*G0_1_2 - 0.00939454064454056*G0_2_1;
    A[1329] = -0.00329461321214791*G0_0_1 - 0.00329461321214805*G0_1_0 + 0.0226708532915132*G0_1_1 - 0.00329461321214787*G0_1_2 - 0.00329461321214789*G0_2_1;
    A[1330] = -0.00939454064454081*G0_0_1 - 0.00939454064454048*G0_1_0 - 0.00939454064454071*G0_1_2 - 0.0093945406445407*G0_2_1;
    A[1331] = -0.00329461321214793*G0_0_1 - 0.00329461321214734*G0_1_0 + 0.022670853291513*G0_1_1 - 0.00329461321214736*G0_1_2 - 0.00329461321214795*G0_2_1;
    A[1332] = -0.00329461321214781*G0_0_1 - 0.00329461321214828*G0_1_0 - 0.00329461321214809*G0_1_2 - 0.00329461321214783*G0_2_1;
    A[1333] = -0.00939454064454045*G0_0_1 - 0.00939454064454079*G0_1_0 - 0.00939454064454079*G0_1_2 - 0.00939454064454046*G0_2_1;
    A[1334] = 0.0212681961813903*G0_0_1 + 0.0118736555368496*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.069349977119074*G0_1_1 - 0.0146137298915076*G0_1_2 + 0.0212681961813903*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[1335] = 0.00583896797004481*G0_0_1 + 0.00254435475789661*G0_0_2 + 0.00583896797004425*G0_1_0 - 0.0318696743392923*G0_1_1 - 0.00590420783563124*G0_1_2 + 0.00583896797004486*G0_2_1 + 0.00254435475789659*G0_2_2;
    A[1336] = 0.0212681961813903*G0_0_1 + 0.0118736555368497*G0_0_2 + 0.0212681961813904*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.00939454064454102*G0_1_2 + 0.0212681961813904*G0_2_1 + 0.01187365553685*G0_2_2;
    A[1337] = -0.0422754329004323*G0_0_1 - 0.0429604514890966*G0_0_2 - 0.0422754329004322*G0_1_0 + 0.092086070276174*G0_1_1 - 0.0359145460056916*G0_1_2 - 0.0422754329004322*G0_2_1 - 0.0429604514890965*G0_2_2;
    A[1338] = -0.0422754329004326*G0_0_1 - 0.0429604514890965*G0_0_2 - 0.0422754329004325*G0_1_0 + 0.043971669405697*G0_1_1 - 0.000685018588664529*G0_1_2 - 0.0422754329004328*G0_2_1 - 0.0429604514890965*G0_2_2;
    A[1339] = 0.0962940416065407*G0_0_1 + 0.124738623002511*G0_0_2 + 0.0962940416065406*G0_1_0 - 0.0644569872000425*G0_1_1 + 0.0284445813959698*G0_1_2 + 0.0962940416065407*G0_2_1 + 0.124738623002511*G0_2_2;
    A[1340] = 0.0026095946234834*G0_0_1 + 0.00260959462348398*G0_1_0 - 0.151356488162043*G0_1_1 + 0.002609594623483*G0_1_2 + 0.00260959462348343*G0_2_1;
    A[1341] = 0.00260959462348404*G0_0_1 + 0.00260959462348303*G0_1_0 - 0.0453417065830266*G0_1_1 + 0.00260959462348342*G0_1_2 + 0.00260959462348403*G0_2_1;
    A[1342] = 0.0352295274170277*G0_0_1 + 0.035229527417027*G0_1_0 + 0.166687856575009*G0_1_1 + 0.0352295274170276*G0_1_2 + 0.0352295274170278*G0_2_1;
    A[1343] = 0.00260959462348262*G0_0_1 + 0.0026095946234836*G0_1_0 - 0.0453417065830256*G0_1_1 + 0.00260959462348341*G0_1_2 + 0.00260959462348279*G0_2_1;
    A[1344] = -0.0551276864210885*G0_0_0 - 0.0551276864210885*G0_0_1 - 0.0551276864210885*G0_0_2 - 0.0178705039816148*G0_1_0 - 0.0178705039816148*G0_1_1 - 0.0178705039816148*G0_1_2 - 0.0178705039816148*G0_2_0 - 0.0178705039816148*G0_2_1 - 0.0178705039816148*G0_2_2;
    A[1345] = 0.0153313684129654*G0_0_0 + 0.0105310191030674*G0_1_0 + 0.0105310191030674*G0_2_0;
    A[1346] = -0.00480034930989796*G0_1_1 - 0.00480034930989796*G0_2_1;
    A[1347] = -0.00480034930989798*G0_1_2 - 0.00480034930989798*G0_2_2;
    A[1348] = 0.00537576492437613*G0_1_1 + 0.00537576492437603*G0_1_2 + 0.00537576492437616*G0_2_1 + 0.00537576492437605*G0_2_2;
    A[1349] = 0.00228991928210682*G0_1_1 + 0.00228991928210688*G0_1_2 + 0.00228991928210677*G0_2_1 + 0.00228991928210685*G0_2_2;
    A[1350] = 0.00228991928210664*G0_1_1 + 0.00228991928210656*G0_1_2 + 0.00228991928210667*G0_2_1 + 0.0022899192821066*G0_2_2;
    A[1351] = 0.00537576492437602*G0_1_1 + 0.0053757649243762*G0_1_2 + 0.00537576492437597*G0_2_1 + 0.00537576492437614*G0_2_2;
    A[1352] = 0.0220836945012288*G0_0_0 + 0.0274594594256048*G0_0_2 + 0.0274594594256049*G0_1_0 + 0.0328352243499806*G0_1_2 + 0.0274594594256049*G0_2_0 + 0.0328352243499806*G0_2_2;
    A[1353] = 0.00658922642429577*G0_0_0 + 0.00887914570640257*G0_0_2 + 0.00887914570640253*G0_1_0 + 0.0111690649885095*G0_1_2 + 0.00887914570640243*G0_2_0 + 0.0111690649885094*G0_2_2;
    A[1354] = 0.00939454064454075*G0_0_0 + 0.0116844599266473*G0_0_2 + 0.0116844599266473*G0_1_0 + 0.0139743792087537*G0_1_2 + 0.0116844599266473*G0_2_0 + 0.0139743792087537*G0_2_2;
    A[1355] = 0.005375764924376*G0_0_2 + 0.00537576492437597*G0_1_0 + 0.0107515298487522*G0_1_2 + 0.00537576492437591*G0_2_0 + 0.0107515298487521*G0_2_2;
    A[1356] = 0.0220836945012288*G0_0_0 + 0.0274594594256047*G0_0_1 + 0.0274594594256046*G0_1_0 + 0.0328352243499805*G0_1_1 + 0.0274594594256047*G0_2_0 + 0.0328352243499805*G0_2_1;
    A[1357] = 0.00658922642429591*G0_0_0 + 0.00887914570640251*G0_0_1 + 0.00887914570640265*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.00887914570640268*G0_2_0 + 0.0111690649885096*G0_2_1;
    A[1358] = 0.00939454064454036*G0_0_0 + 0.0116844599266473*G0_0_1 + 0.0116844599266472*G0_1_0 + 0.0139743792087538*G0_1_1 + 0.0116844599266472*G0_2_0 + 0.0139743792087537*G0_2_1;
    A[1359] = 0.00537576492437594*G0_0_1 + 0.00537576492437593*G0_1_0 + 0.0107515298487521*G0_1_1 + 0.00537576492437595*G0_2_0 + 0.0107515298487521*G0_2_1;
    A[1360] = 0.0196371995417133*G0_0_0 + 0.0196371995417133*G0_0_1 + 0.0895482395048357*G0_0_2 + 0.0895482395048356*G0_1_0 + 0.0895482395048356*G0_1_1 + 0.0196371995417132*G0_1_2 + 0.0895482395048355*G0_2_0 + 0.0895482395048356*G0_2_1 + 0.0196371995417132*G0_2_2;
    A[1361] = -0.0186912214907005*G0_0_0 - 0.0186912214907005*G0_0_1 - 0.0574176057031954*G0_0_2 - 0.0574176057031952*G0_1_0 - 0.0574176057031953*G0_1_1 + 0.0124934342599274*G0_1_2 - 0.0574176057031952*G0_2_0 - 0.0574176057031952*G0_2_1 + 0.0124934342599274*G0_2_2;
    A[1362] = 0.00939454064454047*G0_0_0 + 0.00939454064454046*G0_0_1 + 0.028372817543824*G0_0_2 + 0.0283728175438239*G0_1_0 + 0.0283728175438239*G0_1_1 - 0.0315434750113571*G0_1_2 + 0.0283728175438239*G0_2_0 + 0.0283728175438238*G0_2_1 - 0.0315434750113571*G0_2_2;
    A[1363] = -0.0053757649243759*G0_0_2 - 0.00537576492437603*G0_1_0 - 0.00537576492437603*G0_1_1 + 0.0220836945012292*G0_1_2 - 0.00537576492437597*G0_2_0 - 0.00537576492437597*G0_2_1 + 0.0220836945012292*G0_2_2;
    A[1364] = 0.0196371995417131*G0_0_0 + 0.0895482395048355*G0_0_1 + 0.0196371995417131*G0_0_2 + 0.0895482395048355*G0_1_0 + 0.019637199541713*G0_1_1 + 0.0895482395048356*G0_1_2 + 0.0895482395048356*G0_2_0 + 0.0196371995417131*G0_2_1 + 0.0895482395048356*G0_2_2;
    A[1365] = -0.0186912214907004*G0_0_0 - 0.0574176057031951*G0_0_1 - 0.0186912214907004*G0_0_2 - 0.0574176057031952*G0_1_0 + 0.0124934342599274*G0_1_1 - 0.0574176057031954*G0_1_2 - 0.0574176057031953*G0_2_0 + 0.0124934342599273*G0_2_1 - 0.0574176057031954*G0_2_2;
    A[1366] = 0.00939454064454055*G0_0_0 + 0.0283728175438239*G0_0_1 + 0.00939454064454058*G0_0_2 + 0.028372817543824*G0_1_0 - 0.0315434750113571*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.0283728175438241*G0_2_0 - 0.031543475011357*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1367] = -0.00537576492437589*G0_0_1 - 0.00537576492437591*G0_1_0 + 0.0220836945012292*G0_1_1 - 0.00537576492437592*G0_1_2 - 0.00537576492437593*G0_2_0 + 0.0220836945012292*G0_2_1 - 0.00537576492437594*G0_2_2;
    A[1368] = 0.179096479009671*G0_0_0 + 0.109185439046549*G0_0_1 + 0.109185439046549*G0_0_2 + 0.109185439046549*G0_1_0 + 0.179096479009671*G0_1_1 + 0.179096479009671*G0_1_2 + 0.109185439046549*G0_2_0 + 0.179096479009671*G0_2_1 + 0.179096479009671*G0_2_2;
    A[1369] = -0.243357746612952*G0_0_0 - 0.108337320793916*G0_0_1 - 0.108337320793916*G0_0_2 - 0.173446706649829*G0_1_0 - 0.147063705006411*G0_1_1 - 0.147063705006411*G0_1_2 - 0.173446706649829*G0_2_0 - 0.147063705006411*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1370] = 0.185268170294209*G0_0_0 + 0.0817390275940609*G0_0_1 + 0.0817390275940608*G0_0_2 + 0.125351877739028*G0_1_0 + 0.100717304493344*G0_1_1 + 0.100717304493344*G0_1_2 + 0.125351877739028*G0_2_0 + 0.100717304493344*G0_2_1 + 0.100717304493344*G0_2_2;
    A[1371] = -0.0812105846828056*G0_0_0 - 0.0274594594256046*G0_0_1 - 0.0274594594256047*G0_0_2 - 0.0537511252572006*G0_1_0 - 0.0328352243499803*G0_1_1 - 0.0328352243499803*G0_1_2 - 0.0537511252572006*G0_2_0 - 0.0328352243499803*G0_2_1 - 0.0328352243499804*G0_2_2;
    A[1372] = 0.0453417065830254*G0_0_0 + 0.0547362472275657*G0_0_1 + 0.0547362472275655*G0_0_2 + 0.0547362472275659*G0_1_0 + 0.0641307878721061*G0_1_1 + 0.0641307878721059*G0_1_2 + 0.054736247227566*G0_2_0 + 0.0641307878721061*G0_2_1 + 0.0641307878721061*G0_2_2;
    A[1373] = 0.0226708532915125*G0_0_0 + 0.0259654665036605*G0_0_1 + 0.0259654665036606*G0_0_2 + 0.0259654665036608*G0_1_0 + 0.0292600797158092*G0_1_1 + 0.0292600797158091*G0_1_2 + 0.0259654665036608*G0_2_0 + 0.0292600797158093*G0_2_1 + 0.0292600797158091*G0_2_2;
    A[1374] = 0.00939454064454041*G0_0_1 + 0.00939454064454033*G0_0_2 + 0.0093945406445408*G0_1_0 + 0.0187890812890809*G0_1_1 + 0.0187890812890808*G0_1_2 + 0.00939454064454076*G0_2_0 + 0.0187890812890809*G0_2_1 + 0.0187890812890808*G0_2_2;
    A[1375] = 0.0226708532915131*G0_0_0 + 0.025965466503661*G0_0_1 + 0.0259654665036611*G0_0_2 + 0.0259654665036606*G0_1_0 + 0.0292600797158088*G0_1_1 + 0.0292600797158088*G0_1_2 + 0.0259654665036606*G0_2_0 + 0.0292600797158088*G0_2_1 + 0.0292600797158087*G0_2_2;
    A[1376] = 0.00329461321214797*G0_0_1 + 0.00329461321214798*G0_0_2 + 0.00329461321214738*G0_1_0 + 0.00658922642429505*G0_1_1 + 0.00658922642429545*G0_1_2 + 0.00329461321214741*G0_2_0 + 0.00658922642429507*G0_2_1 + 0.00658922642429543*G0_2_2;
    A[1377] = 0.00939454064454073*G0_0_1 + 0.00939454064454063*G0_0_2 + 0.00939454064454076*G0_1_0 + 0.0187890812890813*G0_1_1 + 0.0187890812890809*G0_1_2 + 0.0093945406445407*G0_2_0 + 0.0187890812890813*G0_2_1 + 0.018789081289081*G0_2_2;
    A[1378] = -0.0606730749959916*G0_0_0 - 0.0322284936000214*G0_0_1 - 0.0322284936000211*G0_0_2 - 0.0322284936000215*G0_1_0 - 0.128522535206562*G0_1_1 - 0.128522535206562*G0_1_2 - 0.0322284936000214*G0_2_0 - 0.128522535206562*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1379] = 0.0226708532915132*G0_0_0 + 0.0219858347028489*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0219858347028487*G0_1_0 + 0.0642612676032815*G0_1_1 + 0.0642612676032811*G0_1_2 + 0.0219858347028488*G0_2_0 + 0.0642612676032815*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[1380] = -0.00939454064454084*G0_0_1 - 0.00939454064454074*G0_0_2 - 0.00939454064454082*G0_1_0 - 0.0306627368259315*G0_1_1 - 0.0306627368259312*G0_1_2 - 0.00939454064454078*G0_2_0 - 0.0306627368259316*G0_2_1 - 0.0306627368259312*G0_2_2;
    A[1381] = 0.0226708532915132*G0_0_0 + 0.0219858347028485*G0_0_1 + 0.0219858347028485*G0_0_2 + 0.0219858347028482*G0_1_0 + 0.0642612676032809*G0_1_1 + 0.0642612676032807*G0_1_2 + 0.0219858347028481*G0_2_0 + 0.0642612676032808*G0_2_1 + 0.0642612676032806*G0_2_2;
    A[1382] = -0.00329461321214818*G0_0_1 - 0.00329461321214808*G0_0_2 - 0.00329461321214744*G0_1_0 - 0.00913358118219256*G0_1_1 - 0.00913358118219221*G0_1_2 - 0.00329461321214746*G0_2_0 - 0.00913358118219246*G0_2_1 - 0.00913358118219224*G0_2_2;
    A[1383] = -0.00939454064454058*G0_0_1 - 0.00939454064454062*G0_0_2 - 0.00939454064454074*G0_1_0 - 0.0306627368259307*G0_1_1 - 0.0306627368259311*G0_1_2 - 0.00939454064454068*G0_2_0 - 0.0306627368259307*G0_2_1 - 0.030662736825931*G0_2_2;
    A[1384] = -0.0644569872000424*G0_0_0 - 0.0929015685960126*G0_0_1 - 0.160751028806583*G0_0_2 - 0.160751028806582*G0_1_0 - 0.064456987200042*G0_1_1 - 0.257045070413123*G0_1_2 - 0.160751028806582*G0_2_0 - 0.064456987200042*G0_2_1 - 0.257045070413123*G0_2_2;
    A[1385] = 0.0920860702761737*G0_0_0 + 0.128000616281865*G0_0_1 + 0.134361503176606*G0_0_2 + 0.134361503176606*G0_1_0 + 0.1273155976932*G0_1_1 + 0.176636936077038*G0_1_2 + 0.134361503176606*G0_2_0 + 0.1273155976932*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1386] = -0.0693499771190732*G0_0_0 - 0.0547362472275655*G0_0_1 - 0.0906181733004635*G0_0_2 - 0.0906181733004638*G0_1_0 - 0.0641307878721061*G0_1_1 - 0.111886369481854*G0_1_2 - 0.0906181733004639*G0_2_0 - 0.0641307878721061*G0_2_1 - 0.111886369481854*G0_2_2;
    A[1387] = 0.0439716694056971*G0_0_0 + 0.0446566879943618*G0_0_1 + 0.0862471023061295*G0_0_2 + 0.0862471023061288*G0_1_0 + 0.0439716694056961*G0_1_1 + 0.128522535206562*G0_1_2 + 0.0862471023061286*G0_2_0 + 0.043971669405696*G0_2_1 + 0.128522535206561*G0_2_2;
    A[1388] = -0.0318696743392924*G0_0_0 - 0.025965466503661*G0_0_1 - 0.0377086423093366*G0_0_2 - 0.0377086423093361*G0_1_0 - 0.0292600797158085*G0_1_1 - 0.0435476102793803*G0_1_2 - 0.037708642309336*G0_2_0 - 0.0292600797158085*G0_2_1 - 0.0435476102793802*G0_2_2;
    A[1389] = -0.0187890812890812*G0_0_0 - 0.00939454064454065*G0_0_1 - 0.0400572774704717*G0_0_2 - 0.0400572774704711*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.0613254736518619*G0_1_2 - 0.0400572774704712*G0_2_0 - 0.0187890812890812*G0_2_1 - 0.0613254736518618*G0_2_2;
    A[1390] = -0.0644569872000422*G0_0_0 - 0.160751028806583*G0_0_1 - 0.0929015685960126*G0_0_2 - 0.160751028806582*G0_1_0 - 0.257045070413123*G0_1_1 - 0.0644569872000419*G0_1_2 - 0.160751028806582*G0_2_0 - 0.257045070413123*G0_2_1 - 0.064456987200042*G0_2_2;
    A[1391] = 0.0920860702761737*G0_0_0 + 0.134361503176606*G0_0_1 + 0.128000616281865*G0_0_2 + 0.134361503176606*G0_1_0 + 0.176636936077038*G0_1_1 + 0.1273155976932*G0_1_2 + 0.134361503176606*G0_2_0 + 0.176636936077038*G0_2_1 + 0.1273155976932*G0_2_2;
    A[1392] = -0.0693499771190735*G0_0_0 - 0.0906181733004635*G0_0_1 - 0.0547362472275657*G0_0_2 - 0.0906181733004637*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0641307878721059*G0_1_2 - 0.0906181733004637*G0_2_0 - 0.111886369481854*G0_2_1 - 0.0641307878721061*G0_2_2;
    A[1393] = 0.0439716694056967*G0_0_0 + 0.0862471023061297*G0_0_1 + 0.0446566879943615*G0_0_2 + 0.0862471023061287*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0439716694056966*G0_1_2 + 0.0862471023061288*G0_2_0 + 0.128522535206562*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[1394] = -0.0318696743392922*G0_0_0 - 0.0377086423093368*G0_0_1 - 0.0259654665036607*G0_0_2 - 0.0377086423093362*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.0292600797158089*G0_1_2 - 0.0377086423093363*G0_2_0 - 0.0435476102793805*G0_2_1 - 0.0292600797158089*G0_2_2;
    A[1395] = -0.018789081289081*G0_0_0 - 0.0400572774704716*G0_0_1 - 0.00939454064454042*G0_0_2 - 0.0400572774704713*G0_1_0 - 0.061325473651862*G0_1_1 - 0.0187890812890812*G0_1_2 - 0.0400572774704714*G0_2_0 - 0.0613254736518621*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[1396] = 0.166687856575009*G0_0_0 + 0.131458329157982*G0_0_1 + 0.131458329157982*G0_0_2 + 0.13145832915798*G0_1_0 + 0.0962288017409532*G0_1_1 + 0.0962288017409527*G0_1_2 + 0.13145832915798*G0_2_0 + 0.0962288017409533*G0_2_1 + 0.0962288017409527*G0_2_2;
    A[1397] = -0.151356488162042*G0_0_0 - 0.153966082785526*G0_0_1 - 0.153966082785526*G0_0_2 - 0.153966082785524*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.153966082785524*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[1398] = -0.0453417065830255*G0_0_0 - 0.0479513012065092*G0_0_1 - 0.0479513012065091*G0_0_2 - 0.0479513012065093*G0_1_0 - 0.050560895829992*G0_1_1 - 0.0505608958299915*G0_1_2 - 0.0479513012065094*G0_2_0 - 0.0505608958299921*G0_2_1 - 0.0505608958299917*G0_2_2;
    A[1399] = -0.0453417065830267*G0_0_0 - 0.0479513012065099*G0_0_1 - 0.0479513012065105*G0_0_2 - 0.0479513012065091*G0_1_0 - 0.0505608958299913*G0_1_1 - 0.0505608958299926*G0_1_2 - 0.047951301206509*G0_2_0 - 0.0505608958299912*G0_2_1 - 0.0505608958299925*G0_2_2;
    A[1400] = 0.0551276864210883*G0_0_0 + 0.0551276864210883*G0_0_1 + 0.0551276864210884*G0_0_2 + 0.0214691349673985*G0_1_0 + 0.0214691349673984*G0_1_1 + 0.0214691349673985*G0_1_2 + 0.0214691349673985*G0_2_0 + 0.0214691349673984*G0_2_1 + 0.0214691349673985*G0_2_2;
    A[1401] = -0.0388177200243166*G0_0_0 - 0.0232632312710432*G0_1_0 - 0.0232632312710432*G0_2_0;
    A[1402] = -0.00336507226698186*G0_1_1 - 0.00336507226698187*G0_2_1;
    A[1403] = -0.00336507226698192*G0_1_2 - 0.00336507226698192*G0_2_2;
    A[1404] = 0.00228991928210681*G0_1_1 + 0.00228991928210675*G0_1_2 + 0.00228991928210676*G0_2_1 + 0.00228991928210671*G0_2_2;
    A[1405] = 0.00178757231708601*G0_1_1 + 0.00178757231708593*G0_1_2 + 0.00178757231708602*G0_2_1 + 0.00178757231708592*G0_2_2;
    A[1406] = 0.00178757231708636*G0_1_1 + 0.00178757231708648*G0_1_2 + 0.00178757231708635*G0_2_1 + 0.00178757231708649*G0_2_2;
    A[1407] = 0.00228991928210666*G0_1_1 + 0.00228991928210653*G0_1_2 + 0.0022899192821067*G0_2_1 + 0.00228991928210656*G0_2_2;
    A[1408] = -0.0315434750113563*G0_0_0 - 0.0599162925551802*G0_0_2 - 0.0599162925551806*G0_1_0 - 0.0788945694544637*G0_1_2 - 0.0599162925551806*G0_2_0 - 0.0788945694544637*G0_2_2;
    A[1409] = -0.00626302709636029*G0_0_0 - 0.00578025209101594*G0_0_2 - 0.00578025209101579*G0_1_0 - 0.00523223722008475*G0_1_2 - 0.00578025209101569*G0_2_0 - 0.00523223722008455*G0_2_2;
    A[1410] = 6.523986558685e-05*G0_0_0 - 0.0131523569023568*G0_0_2 - 0.0131523569023569*G0_1_0 - 0.0263047138047131*G0_1_2 - 0.0131523569023569*G0_2_0 - 0.0263047138047133*G0_2_2;
    A[1411] = 0.00228991928210668*G0_0_2 + 0.00228991928210678*G0_1_0 + 0.0139743792087539*G0_1_2 + 0.00228991928210682*G0_2_0 + 0.013974379208754*G0_2_2;
    A[1412] = -0.0315434750113562*G0_0_0 - 0.0599162925551802*G0_0_1 - 0.0599162925551804*G0_1_0 - 0.0788945694544636*G0_1_1 - 0.0599162925551804*G0_2_0 - 0.0788945694544636*G0_2_1;
    A[1413] = -0.0062630270963604*G0_0_0 - 0.00578025209101605*G0_0_1 - 0.00578025209101598*G0_1_0 - 0.00523223722008453*G0_1_1 - 0.00578025209101611*G0_2_0 - 0.00523223722008467*G0_2_1;
    A[1414] = 6.52398655872334e-05*G0_0_0 - 0.0131523569023565*G0_0_1 - 0.0131523569023567*G0_1_0 - 0.0263047138047136*G0_1_1 - 0.0131523569023566*G0_2_0 - 0.0263047138047134*G0_2_1;
    A[1415] = 0.00228991928210677*G0_0_1 + 0.0022899192821069*G0_1_0 + 0.0139743792087542*G0_1_1 + 0.00228991928210686*G0_2_0 + 0.0139743792087541*G0_2_1;
    A[1416] = 0.0124934342599272*G0_0_0 + 0.0124934342599273*G0_0_1 - 0.0574176057031951*G0_0_2 - 0.0574176057031952*G0_1_0 - 0.0574176057031951*G0_1_1 - 0.0186912214907006*G0_1_2 - 0.0574176057031951*G0_2_0 - 0.0574176057031951*G0_2_1 - 0.0186912214907006*G0_2_2;
    A[1417] = 0.000260959462348305*G0_0_0 + 0.000260959462348256*G0_0_1 + 0.00506261356955769*G0_0_2 + 0.00506261356955785*G0_1_0 + 0.00506261356955776*G0_1_1 + 0.000260959462348512*G0_1_2 + 0.0050626135695578*G0_2_0 + 0.00506261356955771*G0_2_1 + 0.000260959462348507*G0_2_2;
    A[1418] = 6.52398655872997e-05*G0_0_0 + 6.52398655873273e-05*G0_0_1 - 0.000482775005344084*G0_0_2 - 0.000482775005344101*G0_1_0 - 0.000482775005344049*G0_1_1 - 0.00626302709636036*G0_1_2 - 0.000482775005344085*G0_2_0 - 0.000482775005344033*G0_2_1 - 0.00626302709636036*G0_2_2;
    A[1419] = -0.00228991928210692*G0_0_2 - 0.00228991928210674*G0_1_0 - 0.00228991928210672*G0_1_1 + 0.00658922642429582*G0_1_2 - 0.00228991928210678*G0_2_0 - 0.00228991928210675*G0_2_1 + 0.00658922642429583*G0_2_2;
    A[1420] = 0.0124934342599274*G0_0_0 - 0.057417605703195*G0_0_1 + 0.0124934342599274*G0_0_2 - 0.057417605703195*G0_1_0 - 0.0186912214907002*G0_1_1 - 0.057417605703195*G0_1_2 - 0.057417605703195*G0_2_0 - 0.0186912214907002*G0_2_1 - 0.057417605703195*G0_2_2;
    A[1421] = 0.000260959462348281*G0_0_0 + 0.00506261356955759*G0_0_1 + 0.000260959462348299*G0_0_2 + 0.00506261356955803*G0_1_0 + 0.000260959462348216*G0_1_1 + 0.0050626135695582*G0_1_2 + 0.005062613569558*G0_2_0 + 0.000260959462348247*G0_2_1 + 0.00506261356955818*G0_2_2;
    A[1422] = 6.52398655873154e-05*G0_0_0 - 0.000482775005344028*G0_0_1 + 6.52398655873115e-05*G0_0_2 - 0.000482775005344181*G0_1_0 - 0.0062630270963602*G0_1_1 - 0.000482775005344371*G0_1_2 - 0.000482775005344162*G0_2_0 - 0.00626302709636024*G0_2_1 - 0.000482775005344357*G0_2_2;
    A[1423] = -0.0022899192821069*G0_0_1 - 0.00228991928210691*G0_1_0 + 0.00658922642429577*G0_1_1 - 0.00228991928210689*G0_1_2 - 0.00228991928210686*G0_2_0 + 0.0065892264242958*G0_2_1 - 0.00228991928210684*G0_2_2;
    A[1424] = -0.243357746612952*G0_0_0 - 0.173446706649829*G0_0_1 - 0.173446706649829*G0_0_2 - 0.108337320793916*G0_1_0 - 0.147063705006411*G0_1_1 - 0.147063705006411*G0_1_2 - 0.108337320793916*G0_2_0 - 0.147063705006411*G0_2_1 - 0.147063705006411*G0_2_2;
    A[1425] = 0.428965164208214*G0_0_0 + 0.226095278178608*G0_0_1 + 0.226095278178608*G0_0_2 + 0.226095278178608*G0_1_0 + 0.304226541205703*G0_1_1 + 0.304226541205703*G0_1_2 + 0.226095278178608*G0_2_0 + 0.304226541205703*G0_2_1 + 0.304226541205703*G0_2_2;
    A[1426] = -0.387185554286243*G0_0_0 - 0.167692550505047*G0_0_1 - 0.167692550505047*G0_0_2 - 0.219493003781195*G0_1_0 - 0.204187731314464*G0_1_1 - 0.204187731314464*G0_1_2 - 0.219493003781195*G0_2_0 - 0.204187731314464*G0_2_1 - 0.204187731314463*G0_2_2;
    A[1427] = 0.185268170294209*G0_0_0 + 0.05991629255518*G0_0_1 + 0.0599162925551801*G0_0_2 + 0.103529142700148*G0_1_0 + 0.0788945694544633*G0_1_1 + 0.0788945694544634*G0_1_2 + 0.103529142700148*G0_2_0 + 0.0788945694544633*G0_2_1 + 0.0788945694544634*G0_2_2;
    A[1428] = -0.0665446628988285*G0_0_0 - 0.0893133759887214*G0_0_1 - 0.0893133759887215*G0_0_2 - 0.089313375988722*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.089313375988722*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1429] = -0.000652398655870765*G0_0_0 - 0.00949240044292117*G0_0_1 - 0.00949240044292098*G0_0_2 - 0.00949240044292139*G0_1_0 - 0.018984800885843*G0_1_1 - 0.0189848008858429*G0_1_2 - 0.00949240044292131*G0_2_0 - 0.0189848008858432*G0_2_1 - 0.0189848008858429*G0_2_2;
    A[1430] = 6.52398655870668e-05*G0_0_1 + 6.5239865587295e-05*G0_0_2 + 6.52398655869224e-05*G0_1_0 - 0.0148746893538558*G0_1_1 - 0.0148746893538554*G0_1_2 + 6.52398655868007e-05*G0_2_0 - 0.0148746893538558*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1431] = -0.000652398655871445*G0_0_0 - 0.0094924004429216*G0_0_1 - 0.00949240044292168*G0_0_2 - 0.00949240044292136*G0_1_0 - 0.0189848008858425*G0_1_1 - 0.0189848008858427*G0_1_2 - 0.00949240044292135*G0_2_0 - 0.0189848008858424*G0_2_1 - 0.0189848008858424*G0_2_2;
    A[1432] = 0.00257697469068993*G0_0_1 + 0.00257697469068953*G0_0_2 + 0.00257697469068995*G0_1_0 + 0.0045015507255089*G0_1_1 + 0.00450155072550842*G0_1_2 + 0.00257697469068994*G0_2_0 + 0.00450155072550893*G0_2_1 + 0.00450155072550841*G0_2_2;
    A[1433] = 6.52398655868685e-05*G0_0_1 + 6.52398655871322e-05*G0_0_2 + 6.52398655867821e-05*G0_1_0 - 0.0148746893538563*G0_1_1 - 0.0148746893538559*G0_1_2 + 6.52398655868428e-05*G0_2_0 - 0.0148746893538563*G0_2_1 - 0.014874689353856*G0_2_2;
    A[1434] = 0.0476251018785742*G0_0_0 - 0.0147442096226814*G0_0_1 - 0.0147442096226816*G0_0_2 - 0.0147442096226814*G0_1_0 + 0.0476251018785736*G0_1_1 + 0.0476251018785731*G0_1_2 - 0.0147442096226814*G0_2_0 + 0.0476251018785736*G0_2_1 + 0.047625101878573*G0_2_2;
    A[1435] = -0.000652398655871102*G0_0_0 + 0.00231601522834127*G0_0_1 + 0.00231601522834139*G0_0_2 + 0.00231601522834133*G0_1_0 + 0.00880738185425615*G0_1_1 + 0.00880738185425631*G0_1_2 + 0.00231601522834135*G0_2_0 + 0.00880738185425624*G0_2_1 + 0.00880738185425641*G0_2_2;
    A[1436] = -6.52398655869587e-05*G0_0_1 - 6.52398655870085e-05*G0_0_2 - 6.52398655868898e-05*G0_1_0 - 0.00130479731174121*G0_1_1 - 0.00130479731174136*G0_1_2 - 6.52398655867698e-05*G0_2_0 - 0.00130479731174123*G0_2_1 - 0.00130479731174137*G0_2_2;
    A[1437] = -0.000652398655871075*G0_0_0 + 0.00231601522834143*G0_0_1 + 0.00231601522834166*G0_0_2 + 0.0023160152283417*G0_1_0 + 0.00880738185425699*G0_1_1 + 0.00880738185425777*G0_1_2 + 0.00231601522834177*G0_2_0 + 0.00880738185425707*G0_2_1 + 0.00880738185425779*G0_2_2;
    A[1438] = -0.00257697469068964*G0_0_1 - 0.00257697469068994*G0_0_2 - 0.00257697469068991*G0_1_0 - 0.0156575677409007*G0_1_1 - 0.015657567740901*G0_1_2 - 0.00257697469068991*G0_2_0 - 0.0156575677409008*G0_2_1 - 0.0156575677409011*G0_2_2;
    A[1439] = -6.52398655873569e-05*G0_0_1 - 6.52398655871581e-05*G0_0_2 - 6.52398655867669e-05*G0_1_0 - 0.00130479731174206*G0_1_1 - 0.00130479731174194*G0_1_2 - 6.52398655868288e-05*G0_2_0 - 0.00130479731174207*G0_2_1 - 0.00130479731174197*G0_2_2;
    A[1440] = -0.00339247301052829*G0_0_0 + 0.0589768384907272*G0_0_1 + 0.194675758911868*G0_0_2 + 0.194675758911867*G0_1_0 + 0.132306447410612*G0_1_1 + 0.121346149991981*G0_1_2 + 0.194675758911867*G0_2_0 + 0.132306447410612*G0_2_1 + 0.121346149991981*G0_2_2;
    A[1441] = -0.0622388317700809*G0_0_0 - 0.160783648739375*G0_0_1 - 0.250814663249556*G0_0_2 - 0.250814663249555*G0_1_0 - 0.166948816037355*G0_1_1 - 0.334680510461756*G0_1_2 - 0.250814663249555*G0_2_0 - 0.166948816037355*G0_2_1 - 0.334680510461756*G0_2_2;
    A[1442] = 0.0846813455320382*G0_0_0 + 0.0893133759887211*G0_0_1 + 0.173472802596063*G0_0_2 + 0.173472802596064*G0_1_0 + 0.127087258163645*G0_1_1 + 0.209419968534549*G0_1_2 + 0.173472802596064*G0_2_0 + 0.127087258163645*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1443] = -0.00678494602105761*G0_0_0 - 0.00975335990527013*G0_0_1 - 0.0371541034518463*G0_0_2 - 0.037154103451846*G0_1_0 - 0.0436454700777613*G0_1_1 + 0.0371867233846404*G0_1_2 - 0.0371541034518459*G0_2_0 - 0.0436454700777611*G0_2_1 + 0.0371867233846404*G0_2_2;
    A[1444] = 0.0127870136550695*G0_0_0 + 0.00949240044292151*G0_0_1 + 0.0378717419733045*G0_0_2 + 0.037871741973304*G0_1_0 + 0.0189848008858423*G0_1_1 + 0.0567586830607657*G0_1_2 + 0.0378717419733038*G0_2_0 + 0.0189848008858422*G0_2_1 + 0.0567586830607658*G0_2_2;
    A[1445] = -0.00013047973117415*G0_0_0 - 6.52398655868998e-05*G0_0_1 + 0.0136351319077012*G0_0_2 + 0.013635131907701*G0_1_0 + 0.0148746893538561*G0_1_1 - 0.0254435475789641*G0_1_2 + 0.013635131907701*G0_2_0 + 0.0148746893538561*G0_2_1 - 0.0254435475789642*G0_2_2;
    A[1446] = -0.00339247301052866*G0_0_0 + 0.194675758911868*G0_0_1 + 0.0589768384907272*G0_0_2 + 0.194675758911867*G0_1_0 + 0.121346149991982*G0_1_1 + 0.132306447410612*G0_1_2 + 0.194675758911867*G0_2_0 + 0.121346149991982*G0_2_1 + 0.132306447410612*G0_2_2;
    A[1447] = -0.0622388317700807*G0_0_0 - 0.250814663249556*G0_0_1 - 0.160783648739376*G0_0_2 - 0.250814663249555*G0_1_0 - 0.334680510461756*G0_1_1 - 0.166948816037355*G0_1_2 - 0.250814663249555*G0_2_0 - 0.334680510461756*G0_2_1 - 0.166948816037355*G0_2_2;
    A[1448] = 0.0846813455320383*G0_0_0 + 0.173472802596064*G0_0_1 + 0.0893133759887215*G0_0_2 + 0.173472802596064*G0_1_0 + 0.209419968534549*G0_1_1 + 0.127087258163645*G0_1_2 + 0.173472802596064*G0_2_0 + 0.209419968534549*G0_2_1 + 0.127087258163645*G0_2_2;
    A[1449] = -0.00678494602105735*G0_0_0 - 0.0371541034518466*G0_0_1 - 0.00975335990526996*G0_0_2 - 0.0371541034518463*G0_1_0 + 0.0371867233846392*G0_1_1 - 0.0436454700777611*G0_1_2 - 0.0371541034518463*G0_2_0 + 0.0371867233846393*G0_2_1 - 0.0436454700777612*G0_2_2;
    A[1450] = 0.0127870136550694*G0_0_0 + 0.0378717419733046*G0_0_1 + 0.00949240044292122*G0_0_2 + 0.0378717419733043*G0_1_0 + 0.0567586830607656*G0_1_1 + 0.0189848008858426*G0_1_2 + 0.0378717419733045*G0_2_0 + 0.0567586830607656*G0_2_1 + 0.0189848008858427*G0_2_2;
    A[1451] = -0.000130479731174392*G0_0_0 + 0.013635131907701*G0_0_1 - 6.52398655871834e-05*G0_0_2 + 0.013635131907701*G0_1_0 - 0.0254435475789639*G0_1_1 + 0.0148746893538557*G0_1_2 + 0.0136351319077009*G0_2_0 - 0.0254435475789639*G0_2_1 + 0.0148746893538558*G0_2_2;
    A[1452] = -0.161794866655977*G0_0_0 - 0.0662184635708947*G0_0_1 - 0.0662184635708949*G0_0_2 - 0.0662184635708922*G0_1_0 - 0.156575677409008*G0_1_1 - 0.156575677409008*G0_1_2 - 0.0662184635708921*G0_2_0 - 0.156575677409008*G0_2_1 - 0.156575677409008*G0_2_2;
    A[1453] = 0.180714427676231*G0_0_0 + 0.170276049182297*G0_0_1 + 0.170276049182297*G0_0_2 + 0.170276049182296*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182296*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923197*G0_2_2;
    A[1454] = 0.00130479731174175*G0_0_0 + 0.00717638521457995*G0_0_1 + 0.0071763852145798*G0_0_2 + 0.00717638521457988*G0_1_0 + 0.0391439193522511*G0_1_1 + 0.0391439193522522*G0_1_2 + 0.00717638521457978*G0_2_0 + 0.0391439193522512*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[1455] = 0.00130479731174289*G0_0_0 + 0.00717638521458036*G0_0_1 + 0.00717638521458123*G0_0_2 + 0.0071763852145799*G0_1_0 + 0.0391439193522527*G0_1_1 + 0.039143919352253*G0_1_2 + 0.00717638521457981*G0_2_0 + 0.0391439193522526*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[1456] = -0.0388177200243166*G0_0_0 - 0.0388177200243166*G0_0_1 - 0.0388177200243166*G0_0_2 - 0.0155544887532732*G0_1_0 - 0.0155544887532731*G0_1_1 - 0.0155544887532732*G0_1_2 - 0.0155544887532732*G0_2_0 - 0.0155544887532731*G0_2_1 - 0.0155544887532732*G0_2_2;
    A[1457] = 0.0551276864210883*G0_0_0 + 0.0336585514536896*G0_1_0 + 0.0336585514536896*G0_2_0;
    A[1458] = -0.00336507226698212*G0_1_1 - 0.00336507226698211*G0_2_1;
    A[1459] = -0.00336507226698201*G0_1_2 - 0.00336507226698199*G0_2_2;
    A[1460] = 0.00228991928210701*G0_1_1 + 0.00228991928210697*G0_1_2 + 0.00228991928210704*G0_2_1 + 0.00228991928210703*G0_2_2;
    A[1461] = 0.00178757231708623*G0_1_1 + 0.00178757231708625*G0_1_2 + 0.0017875723170862*G0_2_1 + 0.00178757231708617*G0_2_2;
    A[1462] = 0.00178757231708615*G0_1_1 + 0.00178757231708616*G0_1_2 + 0.00178757231708614*G0_2_1 + 0.0017875723170862*G0_2_2;
    A[1463] = 0.00228991928210687*G0_1_1 + 0.00228991928210698*G0_1_2 + 0.00228991928210687*G0_2_1 + 0.00228991928210694*G0_2_2;
    A[1464] = 0.0124934342599269*G0_0_0 + 0.0699110399631218*G0_0_2 + 0.0699110399631225*G0_1_0 + 0.108637424175617*G0_1_2 + 0.0699110399631225*G0_2_0 + 0.108637424175617*G0_2_2;
    A[1465] = 0.000260959462348101*G0_0_0 - 0.00480165410720969*G0_0_2 - 0.00480165410720994*G0_1_0 - 0.00960330821441889*G0_1_2 - 0.00480165410720998*G0_2_0 - 0.00960330821441907*G0_2_2;
    A[1466] = 6.52398655873055e-05*G0_0_0 + 0.00054801487093154*G0_0_2 + 0.00054801487093164*G0_1_0 - 0.00523223722008496*G0_1_2 + 0.000548014870931668*G0_2_0 - 0.00523223722008479*G0_2_2;
    A[1467] = 0.00228991928210676*G0_0_2 + 0.00228991928210681*G0_1_0 + 0.0111690649885097*G0_1_2 + 0.00228991928210681*G0_2_0 + 0.0111690649885096*G0_2_2;
    A[1468] = 0.0124934342599269*G0_0_0 + 0.0699110399631219*G0_0_1 + 0.0699110399631222*G0_1_0 + 0.108637424175617*G0_1_1 + 0.0699110399631223*G0_2_0 + 0.108637424175617*G0_2_1;
    A[1469] = 0.000260959462348422*G0_0_0 - 0.00480165410720935*G0_0_1 - 0.0048016541072094*G0_1_0 - 0.00960330821441884*G0_1_1 - 0.00480165410720942*G0_2_0 - 0.00960330821441886*G0_2_1;
    A[1470] = 6.52398655869434e-05*G0_0_0 + 0.000548014870931228*G0_0_1 + 0.000548014870931488*G0_1_0 - 0.00523223722008452*G0_1_1 + 0.000548014870931472*G0_2_0 - 0.00523223722008451*G0_2_1;
    A[1471] = 0.00228991928210685*G0_0_1 + 0.0022899192821068*G0_1_0 + 0.0111690649885095*G0_1_1 + 0.00228991928210686*G0_2_0 + 0.0111690649885096*G0_2_1;
    A[1472] = -0.0315434750113567*G0_0_0 - 0.0315434750113568*G0_0_1 + 0.028372817543824*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.0283728175438241*G0_1_1 + 0.00939454064454065*G0_1_2 + 0.0283728175438241*G0_2_0 + 0.0283728175438241*G0_2_1 + 0.00939454064454067*G0_2_2;
    A[1473] = -0.00626302709636015*G0_0_0 - 0.00626302709636009*G0_0_1 - 0.00048277500534421*G0_0_2 - 0.000482775005344257*G0_1_0 - 0.000482775005344246*G0_1_1 + 6.52398655872624e-05*G0_1_2 - 0.000482775005344229*G0_2_0 - 0.000482775005344217*G0_2_1 + 6.52398655872258e-05*G0_2_2;
    A[1474] = 6.52398655868967e-05*G0_0_0 + 6.52398655868529e-05*G0_0_1 + 0.0132175967679435*G0_0_2 + 0.0132175967679437*G0_1_0 + 0.0132175967679437*G0_1_1 + 6.52398655867472e-05*G0_1_2 + 0.0132175967679437*G0_2_0 + 0.0132175967679437*G0_2_1 + 6.52398655867565e-05*G0_2_2;
    A[1475] = -0.00228991928210663*G0_0_2 - 0.00228991928210681*G0_1_0 - 0.00228991928210684*G0_1_1 + 0.0093945406445407*G0_1_2 - 0.00228991928210681*G0_2_0 - 0.00228991928210684*G0_2_1 + 0.00939454064454067*G0_2_2;
    A[1476] = -0.0315434750113568*G0_0_0 + 0.0283728175438239*G0_0_1 - 0.0315434750113568*G0_0_2 + 0.0283728175438241*G0_1_0 + 0.00939454064454048*G0_1_1 + 0.0283728175438241*G0_1_2 + 0.0283728175438241*G0_2_0 + 0.00939454064454046*G0_2_1 + 0.0283728175438242*G0_2_2;
    A[1477] = -0.00626302709636028*G0_0_0 - 0.000482775005344231*G0_0_1 - 0.00626302709636027*G0_0_2 - 0.000482775005344732*G0_1_0 + 6.5239865587079e-05*G0_1_1 - 0.000482775005344868*G0_1_2 - 0.000482775005344737*G0_2_0 + 6.5239865587096e-05*G0_2_1 - 0.000482775005344873*G0_2_2;
    A[1478] = 6.52398655869847e-05*G0_0_0 + 0.0132175967679435*G0_0_1 + 6.5239865586961e-05*G0_0_2 + 0.0132175967679439*G0_1_0 + 6.52398655869514e-05*G0_1_1 + 0.013217596767944*G0_1_2 + 0.0132175967679439*G0_2_0 + 6.52398655869134e-05*G0_2_1 + 0.0132175967679441*G0_2_2;
    A[1479] = -0.00228991928210662*G0_0_1 - 0.00228991928210679*G0_1_0 + 0.00939454064454072*G0_1_1 - 0.00228991928210685*G0_1_2 - 0.00228991928210684*G0_2_0 + 0.00939454064454074*G0_2_1 - 0.00228991928210691*G0_2_2;
    A[1480] = 0.185268170294209*G0_0_0 + 0.125351877739028*G0_0_1 + 0.125351877739028*G0_0_2 + 0.0817390275940609*G0_1_0 + 0.100717304493344*G0_1_1 + 0.100717304493344*G0_1_2 + 0.0817390275940608*G0_2_0 + 0.100717304493344*G0_2_1 + 0.100717304493344*G0_2_2;
    A[1481] = -0.387185554286243*G0_0_0 - 0.219493003781195*G0_0_1 - 0.219493003781195*G0_0_2 - 0.167692550505047*G0_1_0 - 0.204187731314464*G0_1_1 - 0.204187731314463*G0_1_2 - 0.167692550505047*G0_2_0 - 0.204187731314464*G0_2_1 - 0.204187731314463*G0_2_2;
    A[1482] = 0.428965164208214*G0_0_0 + 0.202869886029605*G0_0_1 + 0.202869886029605*G0_0_2 + 0.202869886029605*G0_1_0 + 0.2810011490567*G0_1_1 + 0.2810011490567*G0_1_2 + 0.202869886029605*G0_2_0 + 0.2810011490567*G0_2_1 + 0.2810011490567*G0_2_2;
    A[1483] = -0.243357746612951*G0_0_0 - 0.0699110399631217*G0_0_1 - 0.0699110399631217*G0_0_2 - 0.135020425819035*G0_1_0 - 0.108637424175616*G0_1_1 - 0.108637424175616*G0_1_2 - 0.135020425819035*G0_2_0 - 0.108637424175617*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1484] = 0.047625101878573*G0_0_0 + 0.0623693115012543*G0_0_1 + 0.0623693115012547*G0_0_2 + 0.0623693115012555*G0_1_0 + 0.124738623002511*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0623693115012556*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1485] = -0.000652398655870757*G0_0_0 - 0.00296841388421235*G0_0_1 - 0.00296841388421255*G0_0_2 - 0.00296841388421233*G0_1_0 + 0.00352295274170334*G0_1_1 + 0.0035229527417036*G0_1_2 - 0.00296841388421223*G0_2_0 + 0.00352295274170346*G0_2_1 + 0.00352295274170362*G0_2_2;
    A[1486] = 6.5239865586887e-05*G0_0_1 + 6.52398655869124e-05*G0_0_2 + 6.52398655874085e-05*G0_1_0 - 0.0011743175805679*G0_1_1 - 0.00117431758056775*G0_1_2 + 6.52398655874146e-05*G0_2_0 - 0.00117431758056792*G0_2_1 - 0.00117431758056767*G0_2_2;
    A[1487] = -0.000652398655870515*G0_0_0 - 0.00296841388421243*G0_0_1 - 0.00296841388421247*G0_0_2 - 0.00296841388421242*G0_1_0 + 0.00352295274170263*G0_1_1 + 0.00352295274170275*G0_1_2 - 0.00296841388421242*G0_2_0 + 0.00352295274170258*G0_2_1 + 0.00352295274170265*G0_2_2;
    A[1488] = 0.0025769746906902*G0_0_1 + 0.00257697469069035*G0_0_2 + 0.00257697469068978*G0_1_0 - 0.010503618359521*G0_1_1 - 0.0105036183595211*G0_1_2 + 0.00257697469068971*G0_2_0 - 0.0105036183595211*G0_2_1 - 0.0105036183595211*G0_2_2;
    A[1489] = 6.52398655871893e-05*G0_0_1 + 6.52398655871911e-05*G0_0_2 + 6.5239865587715e-05*G0_1_0 - 0.00117431758056701*G0_1_1 - 0.00117431758056714*G0_1_2 + 6.52398655877279e-05*G0_2_0 - 0.00117431758056695*G0_2_1 - 0.00117431758056706*G0_2_2;
    A[1490] = -0.0665446628988292*G0_0_0 + 0.0227687130898931*G0_0_1 + 0.0227687130898935*G0_0_2 + 0.0227687130898926*G0_1_0 - 0.0150051690850305*G0_1_1 - 0.01500516908503*G0_1_2 + 0.0227687130898927*G0_2_0 - 0.0150051690850305*G0_2_1 - 0.01500516908503*G0_2_2;
    A[1491] = -0.000652398655870683*G0_0_0 + 0.0088400017870509*G0_0_1 + 0.00884000178705037*G0_0_2 + 0.00884000178705088*G0_1_0 - 0.000652398655869584*G0_1_1 - 0.000652398655870209*G0_1_2 + 0.00884000178705087*G0_2_0 - 0.000652398655869505*G0_2_1 - 0.000652398655870251*G0_2_2;
    A[1492] = -6.52398655875397e-05*G0_0_1 - 6.52398655871535e-05*G0_0_2 - 6.52398655874102e-05*G0_1_0 - 0.0150051690850309*G0_1_1 - 0.0150051690850304*G0_1_2 - 6.5239865587416e-05*G0_2_0 - 0.015005169085031*G0_2_1 - 0.0150051690850304*G0_2_2;
    A[1493] = -0.000652398655871022*G0_0_0 + 0.00884000178705007*G0_0_1 + 0.00884000178705022*G0_0_2 + 0.00884000178705047*G0_1_0 - 0.000652398655871045*G0_1_1 - 0.000652398655871152*G0_1_2 + 0.00884000178705047*G0_2_0 - 0.000652398655871105*G0_2_1 - 0.000652398655871195*G0_2_2;
    A[1494] = -0.00257697469068985*G0_0_1 - 0.00257697469068987*G0_0_2 - 0.00257697469068977*G0_1_0 - 0.000652398655870948*G0_1_1 - 0.000652398655871059*G0_1_2 - 0.0025769746906897*G0_2_0 - 0.000652398655870895*G0_2_1 - 0.000652398655871054*G0_2_2;
    A[1495] = -6.52398655868795e-05*G0_0_1 - 6.52398655870156e-05*G0_0_2 - 6.52398655877383e-05*G0_1_0 - 0.0150051690850301*G0_1_1 - 0.0150051690850302*G0_1_2 - 6.52398655877499e-05*G0_2_0 - 0.01500516908503*G0_2_1 - 0.0150051690850302*G0_2_2;
    A[1496] = 0.0846813455320391*G0_0_0 - 0.0046320304566832*G0_0_1 - 0.0887914570640249*G0_0_2 - 0.0887914570640249*G0_1_0 - 0.0510175748891019*G0_1_1 - 0.0528442911255401*G0_1_2 - 0.0887914570640249*G0_2_0 - 0.0510175748891017*G0_2_1 - 0.05284429112554*G0_2_2;
    A[1497] = -0.0622388317700817*G0_0_0 + 0.0985448169692941*G0_0_1 + 0.188575831479475*G0_0_2 + 0.188575831479474*G0_1_0 + 0.182410664181495*G0_1_1 + 0.104709984267274*G0_1_2 + 0.188575831479474*G0_2_0 + 0.182410664181495*G0_2_1 + 0.104709984267273*G0_2_2;
    A[1498] = -0.00339247301052775*G0_0_0 - 0.0623693115012539*G0_0_1 - 0.198068231922395*G0_0_2 - 0.198068231922396*G0_1_0 - 0.12473862300251*G0_1_1 - 0.271397840842282*G0_1_2 - 0.198068231922396*G0_2_0 - 0.12473862300251*G0_2_1 - 0.271397840842282*G0_2_2;
    A[1499] = 0.0127870136550693*G0_0_0 + 0.00329461321214821*G0_0_1 - 0.0250847283182352*G0_0_2 - 0.0250847283182355*G0_1_0 - 0.0155923278753141*G0_1_1 - 0.00619778723077343*G0_1_2 - 0.0250847283182355*G0_2_0 - 0.015592327875314*G0_2_1 - 0.00619778723077351*G0_2_2;
    A[1500] = -0.00678494602105715*G0_0_0 + 0.00296841388421247*G0_0_1 + 0.0303691574307888*G0_0_2 + 0.0303691574307897*G0_1_0 - 0.00352295274170232*G0_1_1 + 0.104709984267275*G0_1_2 + 0.0303691574307897*G0_2_0 - 0.00352295274170226*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1501] = -0.000130479731174189*G0_0_0 - 6.52398655871877e-05*G0_0_1 - 0.0137656116388752*G0_0_2 - 0.0137656116388754*G0_1_0 + 0.00117431758056712*G0_1_1 - 0.0528442911255409*G0_1_2 - 0.0137656116388754*G0_2_0 + 0.00117431758056707*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[1502] = 0.0846813455320393*G0_0_0 - 0.0887914570640252*G0_0_1 - 0.00463203045668333*G0_0_2 - 0.0887914570640253*G0_1_0 - 0.0528442911255408*G0_1_1 - 0.0510175748891021*G0_1_2 - 0.0887914570640253*G0_2_0 - 0.0528442911255408*G0_2_1 - 0.051017574889102*G0_2_2;
    A[1503] = -0.0622388317700816*G0_0_0 + 0.188575831479475*G0_0_1 + 0.0985448169692943*G0_0_2 + 0.188575831479475*G0_1_0 + 0.104709984267274*G0_1_1 + 0.182410664181495*G0_1_2 + 0.188575831479475*G0_2_0 + 0.104709984267273*G0_2_1 + 0.182410664181495*G0_2_2;
    A[1504] = -0.0033924730105279*G0_0_0 - 0.198068231922396*G0_0_1 - 0.0623693115012546*G0_0_2 - 0.198068231922396*G0_1_0 - 0.271397840842282*G0_1_1 - 0.12473862300251*G0_1_2 - 0.198068231922396*G0_2_0 - 0.271397840842282*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1505] = 0.0127870136550693*G0_0_0 - 0.0250847283182346*G0_0_1 + 0.00329461321214824*G0_0_2 - 0.0250847283182346*G0_1_0 - 0.00619778723077212*G0_1_1 - 0.0155923278753133*G0_1_2 - 0.0250847283182346*G0_2_0 - 0.00619778723077212*G0_2_1 - 0.0155923278753133*G0_2_2;
    A[1506] = -0.00678494602105725*G0_0_0 + 0.0303691574307888*G0_0_1 + 0.00296841388421237*G0_0_2 + 0.0303691574307887*G0_1_0 + 0.104709984267275*G0_1_1 - 0.0035229527417032*G0_1_2 + 0.0303691574307887*G0_2_0 + 0.104709984267275*G0_2_1 - 0.00352295274170322*G0_2_2;
    A[1507] = -0.000130479731174077*G0_0_0 - 0.0137656116388753*G0_0_1 - 6.52398655869521e-05*G0_0_2 - 0.0137656116388754*G0_1_0 - 0.0528442911255412*G0_1_1 + 0.0011743175805674*G0_1_2 - 0.0137656116388754*G0_2_0 - 0.0528442911255412*G0_2_1 + 0.00117431758056732*G0_2_2;
    A[1508] = 0.180714427676232*G0_0_0 + 0.0104383784939343*G0_0_1 + 0.0104383784939346*G0_0_2 + 0.0104383784939338*G0_1_0 + 0.0260959462348344*G0_1_1 + 0.0260959462348345*G0_1_2 + 0.0104383784939337*G0_2_0 + 0.0260959462348341*G0_2_1 + 0.0260959462348344*G0_2_2;
    A[1509] = -0.161794866655975*G0_0_0 - 0.0955764030850825*G0_0_1 - 0.0955764030850818*G0_0_2 - 0.0955764030850821*G0_1_0 - 0.185933616923199*G0_1_1 - 0.185933616923198*G0_1_2 - 0.0955764030850821*G0_2_0 - 0.185933616923199*G0_2_1 - 0.185933616923198*G0_2_2;
    A[1510] = 0.00130479731174151*G0_0_0 - 0.00587158790283756*G0_0_1 - 0.00587158790283827*G0_0_2 - 0.00587158790283843*G0_1_0 + 0.0260959462348356*G0_1_1 + 0.0260959462348349*G0_1_2 - 0.00587158790283851*G0_2_0 + 0.0260959462348357*G0_2_1 + 0.026095946234835*G0_2_2;
    A[1511] = 0.00130479731174114*G0_0_0 - 0.00587158790283844*G0_0_1 - 0.00587158790283876*G0_0_2 - 0.00587158790283812*G0_1_0 + 0.026095946234835*G0_1_1 + 0.026095946234834*G0_1_2 - 0.00587158790283812*G0_2_0 + 0.026095946234835*G0_2_1 + 0.0260959462348339*G0_2_2;
    A[1512] = 0.0153313684129654*G0_0_0 + 0.0153313684129654*G0_0_1 + 0.0153313684129654*G0_0_2 + 0.0048003493098978*G0_1_0 + 0.00480034930989779*G0_1_1 + 0.0048003493098978*G0_1_2 + 0.00480034930989779*G0_2_0 + 0.00480034930989778*G0_2_1 + 0.0048003493098978*G0_2_2;
    A[1513] = -0.0551276864210885*G0_0_0 - 0.0372571824394735*G0_1_0 - 0.0372571824394735*G0_2_0;
    A[1514] = -0.00480034930989781*G0_1_1 - 0.00480034930989781*G0_2_1;
    A[1515] = -0.00480034930989777*G0_1_2 - 0.00480034930989777*G0_2_2;
    A[1516] = 0.00537576492437556*G0_1_1 + 0.00537576492437579*G0_1_2 + 0.00537576492437568*G0_2_1 + 0.00537576492437591*G0_2_2;
    A[1517] = 0.00228991928210709*G0_1_1 + 0.00228991928210669*G0_1_2 + 0.00228991928210697*G0_2_1 + 0.00228991928210657*G0_2_2;
    A[1518] = 0.00228991928210653*G0_1_1 + 0.00228991928210691*G0_1_2 + 0.00228991928210663*G0_2_1 + 0.00228991928210703*G0_2_2;
    A[1519] = 0.00537576492437613*G0_1_1 + 0.00537576492437594*G0_1_2 + 0.00537576492437605*G0_2_1 + 0.00537576492437584*G0_2_2;
    A[1520] = 0.0196371995417134*G0_0_0 - 0.0699110399631221*G0_0_2 - 0.0699110399631225*G0_1_0 - 0.139822079926245*G0_1_2 - 0.0699110399631225*G0_2_0 - 0.139822079926245*G0_2_2;
    A[1521] = -0.0186912214907003*G0_0_0 + 0.038726384212495*G0_0_2 + 0.0387263842124953*G0_1_0 + 0.108637424175617*G0_1_2 + 0.0387263842124952*G0_2_0 + 0.108637424175617*G0_2_2;
    A[1522] = 0.00939454064454042*G0_0_0 - 0.0189782768992837*G0_0_2 - 0.0189782768992837*G0_1_0 - 0.0788945694544643*G0_1_2 - 0.0189782768992837*G0_2_0 - 0.0788945694544643*G0_2_2;
    A[1523] = 0.00537576492437602*G0_0_2 + 0.00537576492437618*G0_1_0 + 0.032835224349981*G0_1_2 + 0.0053757649243761*G0_2_0 + 0.0328352243499809*G0_2_2;
    A[1524] = 0.0196371995417135*G0_0_0 - 0.0699110399631221*G0_0_1 - 0.0699110399631223*G0_1_0 - 0.139822079926245*G0_1_1 - 0.0699110399631223*G0_2_0 - 0.139822079926245*G0_2_1;
    A[1525] = -0.0186912214907007*G0_0_0 + 0.0387263842124947*G0_0_1 + 0.0387263842124947*G0_1_0 + 0.108637424175617*G0_1_1 + 0.0387263842124948*G0_2_0 + 0.108637424175617*G0_2_1;
    A[1526] = 0.0093945406445408*G0_0_0 - 0.0189782768992834*G0_0_1 - 0.0189782768992834*G0_1_0 - 0.0788945694544645*G0_1_1 - 0.0189782768992834*G0_2_0 - 0.0788945694544645*G0_2_1;
    A[1527] = 0.0053757649243758*G0_0_1 + 0.00537576492437588*G0_1_0 + 0.0328352243499808*G0_1_1 + 0.005375764924376*G0_2_0 + 0.032835224349981*G0_2_1;
    A[1528] = 0.0220836945012291*G0_0_0 + 0.0220836945012291*G0_0_1 - 0.00537576492437595*G0_0_2 - 0.00537576492437569*G0_1_0 - 0.00537576492437564*G0_1_1 - 0.00537576492437572*G0_2_0 - 0.00537576492437567*G0_2_1;
    A[1529] = 0.00658922642429548*G0_0_0 + 0.00658922642429545*G0_0_1 - 0.00228991928210696*G0_0_2 - 0.00228991928210733*G0_1_0 - 0.00228991928210741*G0_1_1 - 0.00228991928210726*G0_2_0 - 0.00228991928210734*G0_2_1;
    A[1530] = 0.00939454064454073*G0_0_0 + 0.00939454064454074*G0_0_1 - 0.00228991928210642*G0_0_2 - 0.00228991928210624*G0_1_0 - 0.00228991928210626*G0_1_1 - 0.00228991928210629*G0_2_0 - 0.00228991928210631*G0_2_1;
    A[1531] = -0.00537576492437613*G0_0_2 - 0.0053757649243762*G0_1_0 - 0.00537576492437616*G0_1_1 - 0.00537576492437611*G0_2_0 - 0.00537576492437608*G0_2_1;
    A[1532] = 0.0220836945012292*G0_0_0 - 0.0053757649243759*G0_0_1 + 0.0220836945012292*G0_0_2 - 0.00537576492437574*G0_1_0 - 0.00537576492437575*G0_1_2 - 0.00537576492437572*G0_2_0 - 0.00537576492437574*G0_2_2;
    A[1533] = 0.00658922642429579*G0_0_0 - 0.0022899192821069*G0_0_1 + 0.00658922642429576*G0_0_2 - 0.0022899192821068*G0_1_0 - 0.00228991928210671*G0_1_2 - 0.00228991928210688*G0_2_0 - 0.00228991928210679*G0_2_2;
    A[1534] = 0.00939454064454047*G0_0_0 - 0.00228991928210648*G0_0_1 + 0.00939454064454047*G0_0_2 - 0.00228991928210671*G0_1_0 - 0.00228991928210683*G0_1_2 - 0.00228991928210665*G0_2_0 - 0.00228991928210678*G0_2_2;
    A[1535] = -0.00537576492437616*G0_0_1 - 0.0053757649243759*G0_1_0 - 0.00537576492437585*G0_1_2 - 0.00537576492437601*G0_2_0 - 0.00537576492437596*G0_2_2;
    A[1536] = -0.0812105846828056*G0_0_0 - 0.0537511252572006*G0_0_1 - 0.0537511252572006*G0_0_2 - 0.0274594594256046*G0_1_0 - 0.0328352243499803*G0_1_1 - 0.0328352243499803*G0_1_2 - 0.0274594594256047*G0_2_0 - 0.0328352243499803*G0_2_1 - 0.0328352243499804*G0_2_2;
    A[1537] = 0.185268170294209*G0_0_0 + 0.103529142700148*G0_0_1 + 0.103529142700148*G0_0_2 + 0.05991629255518*G0_1_0 + 0.0788945694544633*G0_1_1 + 0.0788945694544633*G0_1_2 + 0.0599162925551801*G0_2_0 + 0.0788945694544634*G0_2_1 + 0.0788945694544634*G0_2_2;
    A[1538] = -0.243357746612951*G0_0_0 - 0.135020425819035*G0_0_1 - 0.135020425819035*G0_0_2 - 0.0699110399631217*G0_1_0 - 0.108637424175616*G0_1_1 - 0.108637424175617*G0_1_2 - 0.0699110399631217*G0_2_0 - 0.108637424175616*G0_2_1 - 0.108637424175617*G0_2_2;
    A[1539] = 0.179096479009671*G0_0_0 + 0.069911039963122*G0_0_1 + 0.069911039963122*G0_0_2 + 0.069911039963122*G0_1_0 + 0.139822079926244*G0_1_1 + 0.139822079926244*G0_1_2 + 0.069911039963122*G0_2_0 + 0.139822079926244*G0_2_1 + 0.139822079926244*G0_2_2;
    A[1540] = -0.0606730749959909*G0_0_0 - 0.0284445813959695*G0_0_1 - 0.0284445813959698*G0_0_2 - 0.0284445813959696*G0_1_0 - 0.12473862300251*G0_1_1 - 0.12473862300251*G0_1_2 - 0.0284445813959696*G0_2_0 - 0.12473862300251*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1541] = 0.022670853291513*G0_0_0 + 0.00068501858866436*G0_0_1 + 0.000685018588664378*G0_0_2 + 0.00068501858866462*G0_1_0 + 0.0429604514890971*G0_1_1 + 0.0429604514890966*G0_1_2 + 0.000685018588664748*G0_2_0 + 0.0429604514890973*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[1542] = 0.00939454064454081*G0_0_1 + 0.00939454064454056*G0_0_2 + 0.00939454064454022*G0_1_0 - 0.0118736555368497*G0_1_1 - 0.0118736555368497*G0_1_2 + 0.00939454064454037*G0_2_0 - 0.0118736555368497*G0_2_1 - 0.0118736555368494*G0_2_2;
    A[1543] = 0.0226708532915129*G0_0_0 + 0.000685018588664569*G0_0_1 + 0.000685018588664788*G0_0_2 + 0.000685018588664677*G0_1_0 + 0.0429604514890973*G0_1_1 + 0.0429604514890973*G0_1_2 + 0.000685018588664622*G0_2_0 + 0.0429604514890971*G0_2_1 + 0.0429604514890973*G0_2_2;
    A[1544] = 0.00329461321214765*G0_0_1 + 0.00329461321214774*G0_0_2 + 0.00329461321214813*G0_1_0 - 0.00254435475789656*G0_1_1 - 0.00254435475789657*G0_1_2 + 0.00329461321214813*G0_2_0 - 0.00254435475789646*G0_2_1 - 0.00254435475789659*G0_2_2;
    A[1545] = 0.00939454064454052*G0_0_1 + 0.00939454064454036*G0_0_2 + 0.00939454064454025*G0_1_0 - 0.0118736555368499*G0_1_1 - 0.01187365553685*G0_1_2 + 0.00939454064454018*G0_2_0 - 0.01187365553685*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1546] = 0.0453417065830258*G0_0_0 - 0.00939454064454023*G0_0_1 - 0.00939454064454063*G0_0_2 - 0.00939454064454041*G0_1_0 - 0.00939454064454037*G0_2_0;
    A[1547] = 0.0226708532915127*G0_0_0 - 0.00329461321214875*G0_0_1 - 0.00329461321214777*G0_0_2 - 0.00329461321214877*G0_1_0 - 0.00329461321214874*G0_2_0;
    A[1548] = -0.0093945406445398*G0_0_1 - 0.00939454064454037*G0_0_2 - 0.00939454064454024*G0_1_0 - 0.00939454064454039*G0_2_0;
    A[1549] = 0.0226708532915132*G0_0_0 - 0.00329461321214734*G0_0_1 - 0.00329461321214784*G0_0_2 - 0.00329461321214758*G0_1_0 - 0.00329461321214763*G0_2_0;
    A[1550] = -0.00329461321214826*G0_0_1 - 0.00329461321214807*G0_0_2 - 0.00329461321214814*G0_1_0 - 0.00329461321214814*G0_2_0;
    A[1551] = -0.00939454064454065*G0_0_1 - 0.00939454064454059*G0_0_2 - 0.00939454064454021*G0_1_0 - 0.00939454064454015*G0_2_0;
    A[1552] = -0.0693499771190737*G0_0_0 - 0.0146137298915076*G0_0_1 + 0.02126819618139*G0_0_2 + 0.0212681961813899*G0_1_0 + 0.0118736555368496*G0_1_1 + 0.02126819618139*G0_2_0 + 0.0118736555368497*G0_2_1;
    A[1553] = 0.0920860702761736*G0_0_0 - 0.0359145460056909*G0_0_1 - 0.0422754329004321*G0_0_2 - 0.0422754329004321*G0_1_0 - 0.0429604514890963*G0_1_1 - 0.0422754329004321*G0_2_0 - 0.0429604514890961*G0_2_1;
    A[1554] = -0.0644569872000425*G0_0_0 + 0.0284445813959691*G0_0_1 + 0.0962940416065397*G0_0_2 + 0.0962940416065403*G0_1_0 + 0.124738623002509*G0_1_1 + 0.0962940416065403*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1555] = -0.031869674339292*G0_0_0 - 0.00590420783563148*G0_0_1 + 0.00583896797004439*G0_0_2 + 0.00583896797004499*G0_1_0 + 0.00254435475789693*G0_1_1 + 0.00583896797004495*G0_2_0 + 0.00254435475789691*G0_2_1;
    A[1556] = 0.0439716694056967*G0_0_0 - 0.000685018588664596*G0_0_1 - 0.0422754329004322*G0_0_2 - 0.042275432900433*G0_1_0 - 0.0429604514890975*G0_1_1 - 0.0422754329004329*G0_2_0 - 0.0429604514890974*G0_2_1;
    A[1557] = -0.0187890812890812*G0_0_0 - 0.00939454064454052*G0_0_1 + 0.0212681961813903*G0_0_2 + 0.02126819618139*G0_1_0 + 0.0118736555368499*G0_1_1 + 0.0212681961813901*G0_2_0 + 0.01187365553685*G0_2_1;
    A[1558] = -0.0693499771190739*G0_0_0 + 0.0212681961813903*G0_0_1 - 0.0146137298915074*G0_0_2 + 0.0212681961813901*G0_1_0 + 0.0118736555368499*G0_1_2 + 0.0212681961813901*G0_2_0 + 0.0118736555368499*G0_2_2;
    A[1559] = 0.0920860702761739*G0_0_0 - 0.0422754329004319*G0_0_1 - 0.0359145460056907*G0_0_2 - 0.0422754329004322*G0_1_0 - 0.0429604514890964*G0_1_2 - 0.0422754329004322*G0_2_0 - 0.0429604514890962*G0_2_2;
    A[1560] = -0.0644569872000425*G0_0_0 + 0.0962940416065401*G0_0_1 + 0.0284445813959696*G0_0_2 + 0.0962940416065401*G0_1_0 + 0.12473862300251*G0_1_2 + 0.0962940416065401*G0_2_0 + 0.124738623002509*G0_2_2;
    A[1561] = -0.0318696743392921*G0_0_0 + 0.00583896797004374*G0_0_1 - 0.00590420783563162*G0_0_2 + 0.00583896797004415*G0_1_0 + 0.00254435475789585*G0_1_2 + 0.00583896797004424*G0_2_0 + 0.00254435475789598*G0_2_2;
    A[1562] = 0.0439716694056969*G0_0_0 - 0.0422754329004325*G0_0_1 - 0.000685018588664327*G0_0_2 - 0.042275432900432*G0_1_0 - 0.0429604514890966*G0_1_2 - 0.0422754329004321*G0_2_0 - 0.0429604514890967*G0_2_2;
    A[1563] = -0.0187890812890811*G0_0_0 + 0.0212681961813907*G0_0_1 - 0.00939454064454056*G0_0_2 + 0.0212681961813901*G0_1_0 + 0.01187365553685*G0_1_2 + 0.0212681961813901*G0_2_0 + 0.0118736555368497*G0_2_2;
    A[1564] = -0.151356488162042*G0_0_0 + 0.00260959462348381*G0_0_1 + 0.00260959462348354*G0_0_2 + 0.00260959462348387*G0_1_0 + 0.00260959462348392*G0_2_0;
    A[1565] = 0.166687856575007*G0_0_0 + 0.0352295274170271*G0_0_1 + 0.0352295274170261*G0_0_2 + 0.0352295274170263*G0_1_0 + 0.0352295274170261*G0_2_0;
    A[1566] = -0.0453417065830257*G0_0_0 + 0.0026095946234827*G0_0_1 + 0.00260959462348379*G0_0_2 + 0.0026095946234841*G0_1_0 + 0.00260959462348393*G0_2_0;
    A[1567] = -0.0453417065830257*G0_0_0 + 0.0026095946234838*G0_0_1 + 0.00260959462348362*G0_0_2 + 0.00260959462348293*G0_1_0 + 0.00260959462348303*G0_2_0;
    A[1568] = -0.0137421252872639*G0_0_0 - 0.013742125287264*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872638*G0_1_0 - 0.0137421252872639*G0_1_1 - 0.0137421252872639*G0_1_2 - 0.0137421252872637*G0_2_0 - 0.0137421252872637*G0_2_1 - 0.0137421252872638*G0_2_2;
    A[1569] = 0.047993054720485*G0_0_0 + 0.0143945239431348*G0_1_0 + 0.0143945239431349*G0_2_0;
    A[1570] = -0.0051774357329913*G0_0_1 + 0.0137421252872641*G0_1_1 - 0.00517743573299134*G0_2_1;
    A[1571] = -0.00517743573299119*G0_0_2 - 0.00517743573299121*G0_1_2 + 0.0137421252872639*G0_2_2;
    A[1572] = 0.0425363923627813*G0_0_1 + 0.0118736555368505*G0_0_2 - 0.0187890812890811*G0_1_1 - 0.0093945406445403*G0_1_2 + 0.0212681961813906*G0_2_1 + 0.0118736555368497*G0_2_2;
    A[1573] = 0.0403182369328201*G0_0_1 + 0.0540186087061077*G0_0_2 + 0.0148746893538562*G0_1_1 + 0.0149399292194426*G0_1_2 + 0.00123955744615472*G0_2_1 + 0.00117431758056832*G0_2_2;
    A[1574] = 0.0540186087061084*G0_0_1 + 0.0403182369328212*G0_0_2 + 0.00117431758056718*G0_1_1 + 0.00123955744615521*G0_1_2 + 0.0149399292194432*G0_2_1 + 0.0148746893538554*G0_2_2;
    A[1575] = 0.0118736555368502*G0_0_1 + 0.0425363923627806*G0_0_2 + 0.0118736555368503*G0_1_1 + 0.0212681961813903*G0_1_2 - 0.00939454064454057*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[1576] = -0.0644569872000425*G0_0_0 + 0.0284445813959699*G0_0_2 + 0.192588083213082*G0_1_0 + 0.124738623002511*G0_1_2 + 0.0962940416065404*G0_2_0 + 0.12473862300251*G0_2_2;
    A[1577] = 0.132306447410614*G0_0_0 + 0.0733296089198871*G0_0_2 + 0.010960297418631*G0_1_0 + 0.146659217839773*G0_1_2 - 0.0623693115012548*G0_2_0 - 0.12473862300251*G0_2_2;
    A[1578] = -0.0510175748891031*G0_0_0 - 0.0463855444324201*G0_0_2 + 0.00182671623643765*G0_1_0 - 0.0823327103709049*G0_1_2 + 0.037773882174923*G0_2_0 + 0.127087258163645*G0_2_2;
    A[1579] = 0.01187365553685*G0_0_0 + 0.0264873854283575*G0_0_2 + 0.0118736555368502*G0_1_0 + 0.0477555816097481*G0_1_2 - 0.00939454064454068*G0_2_0 - 0.0641307878721067*G0_2_2;
    A[1580] = -0.0644569872000429*G0_0_0 + 0.0284445813959697*G0_0_1 + 0.0962940416065407*G0_1_0 + 0.124738623002511*G0_1_1 + 0.192588083213081*G0_2_0 + 0.12473862300251*G0_2_1;
    A[1581] = 0.132306447410613*G0_0_0 + 0.0733296089198859*G0_0_1 - 0.0623693115012556*G0_1_0 - 0.124738623002511*G0_1_1 + 0.0109602974186307*G0_2_0 + 0.146659217839772*G0_2_1;
    A[1582] = -0.0510175748891018*G0_0_0 - 0.046385544432419*G0_0_1 + 0.0377738821749238*G0_1_0 + 0.127087258163646*G0_1_1 + 0.00182671623643827*G0_2_0 - 0.0823327103709039*G0_2_1;
    A[1583] = 0.0118736555368504*G0_0_0 + 0.0264873854283584*G0_0_1 - 0.00939454064454051*G0_1_0 - 0.0641307878721066*G0_1_1 + 0.0118736555368497*G0_2_0 + 0.0477555816097477*G0_2_1;
    A[1584] = 0.0187890812890813*G0_0_0 + 0.0187890812890813*G0_0_1 + 0.00939454064454085*G0_0_2 + 0.0187890812890807*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064454033*G0_1_2 + 0.00939454064454025*G0_2_0 + 0.00939454064454017*G0_2_1;
    A[1585] = -0.014874689353856*G0_0_0 - 0.014874689353856*G0_0_1 + 6.52398655867352e-05*G0_0_2 - 0.0148746893538556*G0_1_0 - 0.0148746893538555*G0_1_1 + 6.52398655874912e-05*G0_1_2 + 6.52398655873438e-05*G0_2_0 + 6.52398655875016e-05*G0_2_1;
    A[1586] = -0.0011743175805672*G0_0_0 - 0.00117431758056727*G0_0_1 + 6.52398655871819e-05*G0_0_2 - 0.00117431758056773*G0_1_0 - 0.00117431758056776*G0_1_1 + 6.52398655868404e-05*G0_1_2 + 6.52398655864051e-05*G0_2_0 + 6.52398655864098e-05*G0_2_1;
    A[1587] = -0.0118736555368503*G0_0_0 - 0.0118736555368502*G0_0_1 + 0.00939454064454043*G0_0_2 - 0.0118736555368502*G0_1_0 - 0.0118736555368502*G0_1_1 + 0.00939454064454049*G0_1_2 + 0.0093945406445406*G0_2_0 + 0.00939454064454056*G0_2_1;
    A[1588] = 0.0187890812890813*G0_0_0 + 0.00939454064454066*G0_0_1 + 0.0187890812890813*G0_0_2 + 0.00939454064454023*G0_1_0 + 0.00939454064454025*G0_1_2 + 0.0187890812890806*G0_2_0 + 0.00939454064453994*G0_2_1 + 0.0187890812890807*G0_2_2;
    A[1589] = -0.0148746893538559*G0_0_0 + 6.52398655871659e-05*G0_0_1 - 0.0148746893538558*G0_0_2 + 6.52398655876149e-05*G0_1_0 + 6.52398655873686e-05*G0_1_2 - 0.0148746893538552*G0_2_0 + 6.5239865587815e-05*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1590] = -0.00117431758056697*G0_0_0 + 6.52398655863126e-05*G0_0_1 - 0.00117431758056687*G0_0_2 + 6.52398655869305e-05*G0_1_0 + 6.52398655872295e-05*G0_1_2 - 0.00117431758056784*G0_2_0 + 6.5239865586186e-05*G0_2_1 - 0.0011743175805676*G0_2_2;
    A[1591] = -0.0118736555368504*G0_0_0 + 0.009394540644541*G0_0_1 - 0.0118736555368505*G0_0_2 + 0.00939454064454059*G0_1_0 + 0.00939454064454048*G0_1_2 - 0.0118736555368496*G0_2_0 + 0.00939454064454098*G0_2_1 - 0.0118736555368499*G0_2_2;
    A[1592] = 0.0453417065830254*G0_0_0 + 0.0547362472275659*G0_0_1 + 0.054736247227566*G0_0_2 + 0.0547362472275657*G0_1_0 + 0.0641307878721061*G0_1_1 + 0.0641307878721061*G0_1_2 + 0.0547362472275655*G0_2_0 + 0.0641307878721059*G0_2_1 + 0.0641307878721061*G0_2_2;
    A[1593] = -0.0665446628988285*G0_0_0 - 0.089313375988722*G0_0_1 - 0.089313375988722*G0_0_2 - 0.0893133759887214*G0_1_0 - 0.127087258163645*G0_1_1 - 0.127087258163645*G0_1_2 - 0.0893133759887214*G0_2_0 - 0.127087258163645*G0_2_1 - 0.127087258163645*G0_2_2;
    A[1594] = 0.047625101878573*G0_0_0 + 0.0623693115012555*G0_0_1 + 0.0623693115012556*G0_0_2 + 0.0623693115012543*G0_1_0 + 0.124738623002511*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0623693115012547*G0_2_0 + 0.12473862300251*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1595] = -0.0606730749959909*G0_0_0 - 0.0284445813959696*G0_0_1 - 0.0284445813959696*G0_0_2 - 0.0284445813959695*G0_1_0 - 0.12473862300251*G0_1_1 - 0.12473862300251*G0_1_2 - 0.0284445813959698*G0_2_0 - 0.12473862300251*G0_2_1 - 0.12473862300251*G0_2_2;
    A[1596] = 0.647179466623907*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842283*G0_1_0 + 0.542795681684567*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684564*G0_2_2;
    A[1597] = -0.268135847562929*G0_0_0 + 0.180714427676232*G0_0_1 + 0.0225077536275458*G0_0_2 - 0.073721048113409*G0_1_0 - 0.209419968534551*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0459941052388967*G0_2_0 + 0.0812236326559224*G0_2_1 - 0.023486351611351*G0_2_2;
    A[1598] = 0.105688582251081*G0_0_0 - 0.0743734467692796*G0_0_1 + 0.000652398655872151*G0_0_2 + 0.0300103381700597*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255405*G0_1_2 + 0.052844291125541*G0_2_0 + 0.000652398655871283*G0_2_1 + 0.053496689781411*G0_2_2;
    A[1599] = -0.268135847562931*G0_0_0 + 0.0225077536275446*G0_0_1 + 0.180714427676231*G0_0_2 - 0.0459941052388981*G0_1_0 - 0.023486351611352*G0_1_1 + 0.0812236326559231*G0_1_2 - 0.0737210481134107*G0_2_0 - 0.104709984267276*G0_2_1 - 0.209419968534551*G0_2_2;
    A[1600] = 0.113517366121533*G0_0_0 - 0.0851380245911492*G0_0_1 - 0.0851380245911501*G0_0_2 + 0.0127217737894824*G0_1_0 - 0.0137003717732886*G0_1_1 - 0.0198981590040619*G0_1_2 + 0.0127217737894822*G0_2_0 - 0.0198981590040623*G0_2_1 - 0.0137003717732882*G0_2_2;
    A[1601] = 0.105688582251081*G0_0_0 + 0.000652398655869746*G0_0_1 - 0.0743734467692805*G0_0_2 + 0.0528442911255404*G0_1_0 + 0.0534966897814109*G0_1_1 + 0.000652398655869846*G0_1_2 + 0.0300103381700607*G0_2_0 + 0.0528442911255404*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1602] = -0.0600206763401205*G0_0_0 - 0.0300103381700607*G0_0_1 - 0.0300103381700606*G0_0_2 - 0.0300103381700607*G0_1_0 - 0.0150051690850298*G0_1_2 - 0.03001033817006*G0_2_0 - 0.0150051690850296*G0_2_1;
    A[1603] = -0.0254435475789636*G0_0_0 - 0.00130479731174066*G0_0_1 - 0.0127217737894833*G0_0_2 - 0.00130479731174029*G0_1_0 - 0.000652398655871206*G0_1_2 - 0.0127217737894811*G0_2_0 - 0.000652398655870765*G0_2_1;
    A[1604] = -0.105688582251082*G0_0_0 - 0.0300103381700611*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0300103381700598*G0_1_0 - 0.0150051690850299*G0_1_2 - 0.0528442911255411*G0_2_0 - 0.0150051690850303*G0_2_1;
    A[1605] = -0.0254435475789645*G0_0_0 - 0.0127217737894822*G0_0_1 - 0.00130479731174082*G0_0_2 - 0.012721773789483*G0_1_0 - 0.000652398655871281*G0_1_2 - 0.00130479731174225*G0_2_0 - 0.000652398655870722*G0_2_1;
    A[1606] = -0.113517366121533*G0_0_0 - 0.0127217737894821*G0_0_1 - 0.0127217737894824*G0_0_2 - 0.0127217737894823*G0_1_0 - 0.000652398655871594*G0_1_2 - 0.0127217737894822*G0_2_0 - 0.000652398655871388*G0_2_1;
    A[1607] = -0.105688582251082*G0_0_0 - 0.0528442911255414*G0_0_1 - 0.0300103381700608*G0_0_2 - 0.0528442911255406*G0_1_0 - 0.01500516908503*G0_1_2 - 0.0300103381700608*G0_2_0 - 0.01500516908503*G0_2_1;
    A[1608] = -0.0456679059109617*G0_0_0 - 0.0756782440810213*G0_0_1 - 0.0378391220405109*G0_0_2 - 0.075678244081021*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255403*G0_1_2 - 0.0378391220405102*G0_2_0 - 0.0528442911255404*G0_2_1;
    A[1609] = 0.0391439193522532*G0_0_0 + 0.13569892042114*G0_0_1 + 0.0678494602105699*G0_0_2 + 0.135698920421142*G0_1_0 + 0.209419968534549*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0678494602105707*G0_2_0 + 0.104709984267274*G0_2_1;
    A[1610] = 0.0521918924696699*G0_0_0 - 0.271397840842283*G0_0_1 - 0.135698920421143*G0_0_2 - 0.271397840842283*G0_1_0 - 0.542795681684566*G0_1_1 - 0.271397840842284*G0_1_2 - 0.135698920421141*G0_2_0 - 0.271397840842282*G0_2_1;
    A[1611] = 0.0391439193522526*G0_0_0 + 0.0264221455627704*G0_0_1 - 0.00554538857490094*G0_0_2 + 0.0264221455627696*G0_1_0 + 0.0137003717732871*G0_1_1 - 0.00619778723077302*G0_1_2 - 0.00554538857490252*G0_2_0 - 0.00619778723077436*G0_2_1;
    A[1612] = -0.15657567740901*G0_0_0 - 0.0225077536275445*G0_0_1 + 0.0678494602105727*G0_0_2 - 0.0225077536275447*G0_1_0 + 0.0234863516113526*G0_1_1 + 0.104709984267277*G0_1_2 + 0.0678494602105703*G0_2_0 + 0.104709984267276*G0_2_1;
    A[1613] = 0.0521918924696704*G0_0_0 - 0.000652398655869635*G0_0_1 - 0.0378391220405119*G0_0_2 - 0.000652398655870067*G0_1_0 - 0.0534966897814107*G0_1_1 - 0.0528442911255413*G0_1_2 - 0.0378391220405096*G0_2_0 - 0.0528442911255402*G0_2_1;
    A[1614] = -0.0456679059109614*G0_0_0 - 0.0378391220405106*G0_0_1 - 0.0756782440810212*G0_0_2 - 0.0378391220405107*G0_1_0 - 0.0528442911255412*G0_1_2 - 0.0756782440810205*G0_2_0 - 0.0528442911255399*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1615] = 0.0391439193522525*G0_0_0 + 0.067849460210569*G0_0_1 + 0.13569892042114*G0_0_2 + 0.0678494602105713*G0_1_0 + 0.104709984267275*G0_1_2 + 0.135698920421141*G0_2_0 + 0.104709984267273*G0_2_1 + 0.209419968534548*G0_2_2;
    A[1616] = 0.0521918924696704*G0_0_0 - 0.135698920421142*G0_0_1 - 0.271397840842282*G0_0_2 - 0.135698920421141*G0_1_0 - 0.271397840842282*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842282*G0_2_1 - 0.542795681684564*G0_2_2;
    A[1617] = 0.0391439193522519*G0_0_0 - 0.00554538857490066*G0_0_1 + 0.0264221455627714*G0_0_2 - 0.00554538857490249*G0_1_0 - 0.00619778723077205*G0_1_2 + 0.0264221455627689*G0_2_0 - 0.00619778723077306*G0_2_1 + 0.0137003717732881*G0_2_2;
    A[1618] = -0.156575677409009*G0_0_0 + 0.0678494602105719*G0_0_1 - 0.0225077536275461*G0_0_2 + 0.0678494602105703*G0_1_0 + 0.104709984267275*G0_1_2 - 0.0225077536275445*G0_2_0 + 0.104709984267276*G0_2_1 + 0.0234863516113508*G0_2_2;
    A[1619] = 0.0521918924696687*G0_0_0 - 0.0378391220405128*G0_0_1 - 0.000652398655872109*G0_0_2 - 0.0378391220405109*G0_1_0 - 0.0528442911255411*G0_1_2 - 0.000652398655870583*G0_2_0 - 0.0528442911255407*G0_2_1 - 0.0534966897814112*G0_2_2;
    A[1620] = 0.19571959676126*G0_0_0 + 0.0750258454251497*G0_0_1 + 0.0750258454251484*G0_0_2 + 0.0750258454251489*G0_1_0 + 0.026095946234834*G0_1_2 + 0.0750258454251481*G0_2_0 + 0.0260959462348348*G0_2_1;
    A[1621] = -0.782878387045047*G0_0_0 - 0.316413348097372*G0_0_1 - 0.316413348097371*G0_0_2 - 0.316413348097372*G0_1_0 - 0.185933616923198*G0_1_2 - 0.316413348097371*G0_2_0 - 0.185933616923197*G0_2_1;
    A[1622] = 0.293579395141893*G0_0_0 + 0.0750258454251512*G0_0_1 + 0.0587158790283787*G0_0_2 + 0.0750258454251495*G0_1_0 + 0.0260959462348348*G0_1_2 + 0.058715879028378*G0_2_0 + 0.0260959462348347*G0_2_1;
    A[1623] = 0.293579395141895*G0_0_0 + 0.0587158790283787*G0_0_1 + 0.0750258454251522*G0_0_2 + 0.0587158790283804*G0_1_0 + 0.0260959462348366*G0_1_2 + 0.0750258454251524*G0_2_0 + 0.0260959462348361*G0_2_1;
    A[1624] = -0.00946499969937446*G0_0_0 - 0.00946499969937442*G0_0_1 - 0.00946499969937441*G0_0_2 - 0.0094649996993745*G0_1_0 - 0.0094649996993744*G0_1_1 - 0.00946499969937446*G0_1_2 - 0.00946499969937465*G0_2_0 - 0.00946499969937465*G0_2_1 - 0.00946499969937463*G0_2_2;
    A[1625] = -0.0345719095719094*G0_0_0 - 0.00472467106581684*G0_1_0 - 0.00472467106581675*G0_2_0;
    A[1626] = -0.00472467106581687*G0_0_1 - 0.0345719095719095*G0_1_1 - 0.00472467106581683*G0_2_1;
    A[1627] = -0.00227817610630123*G0_0_2 - 0.0022781761063013*G0_1_2 + 0.00946499969937472*G0_2_2;
    A[1628] = -0.0845508658008646*G0_0_1 - 0.0429604514890968*G0_0_2 + 0.0439716694056975*G0_1_1 - 0.000685018588664028*G0_1_2 - 0.0422754329004319*G0_2_1 - 0.0429604514890962*G0_2_2;
    A[1629] = -0.0808321934624014*G0_0_1 - 0.108232937008977*G0_0_2 - 0.0436454700777618*G0_1_1 - 0.0338921101724917*G0_1_2 - 0.00649136662591556*G0_2_1 - 0.00352295274170329*G0_2_2;
    A[1630] = -0.00939454064454048*G0_0_1 - 0.0377738821749242*G0_0_2 - 0.0155923278753133*G0_1_1 - 0.0188869410874619*G0_1_2 + 0.00949240044292103*G0_2_1 + 0.0189848008858425*G0_2_2;
    A[1631] = 0.00254435475789621*G0_0_1 + 0.0142875305635723*G0_0_2 + 0.00254435475789623*G0_1_1 + 0.00844856259352821*G0_1_2 - 0.00329461321214765*G0_2_1 - 0.0292600797158087*G0_2_2;
    A[1632] = 0.0439716694056967*G0_0_0 - 0.00068501858866464*G0_0_2 - 0.0845508658008652*G0_1_0 - 0.0429604514890972*G0_1_2 - 0.0422754329004323*G0_2_0 - 0.0429604514890966*G0_2_2;
    A[1633] = -0.0436454700777614*G0_0_0 - 0.0338921101724918*G0_0_2 - 0.0808321934624011*G0_1_0 - 0.108232937008977*G0_1_2 - 0.00649136662591536*G0_2_0 - 0.0035229527417028*G0_2_2;
    A[1634] = -0.0155923278753141*G0_0_0 - 0.0188869410874618*G0_0_2 - 0.00939454064454051*G0_1_0 - 0.0377738821749239*G0_1_2 + 0.00949240044292153*G0_2_0 + 0.0189848008858424*G0_2_2;
    A[1635] = 0.00254435475789618*G0_0_0 + 0.00844856259352798*G0_0_2 + 0.00254435475789612*G0_1_0 + 0.0142875305635725*G0_1_2 - 0.00329461321214768*G0_2_0 - 0.0292600797158082*G0_2_2;
    A[1636] = 0.127315597693201*G0_0_0 - 0.000685018588664651*G0_0_1 - 0.00704590548340603*G0_1_0 - 0.0429604514890974*G0_1_1 - 0.0493213383838381*G0_2_0 - 0.0429604514890967*G0_2_1;
    A[1637] = -0.166948816037356*G0_0_0 - 0.0061651672979789*G0_0_1 + 0.0838658472122014*G0_1_0 + 0.182410664181496*G0_1_1 + 0.167731694424402*G0_2_0 + 0.0777006799142215*G0_2_1;
    A[1638] = 0.182410664181495*G0_0_0 + 0.0838658472121996*G0_0_1 - 0.0061651672979801*G0_1_0 - 0.166948816037357*G0_1_1 + 0.0777006799142205*G0_2_0 + 0.167731694424401*G0_2_1;
    A[1639] = -0.0429604514890968*G0_0_0 - 0.00704590548340546*G0_0_1 - 0.000685018588664028*G0_1_0 + 0.127315597693201*G0_1_1 - 0.0429604514890962*G0_2_0 - 0.0493213383838371*G0_2_1;
    A[1640] = 0.00658922642429568*G0_0_0 + 0.00658922642429565*G0_0_1 + 0.00329461321214729*G0_0_2 + 0.00658922642429655*G0_1_0 + 0.00658922642429654*G0_1_1 + 0.00329461321214781*G0_1_2 + 0.00329461321214871*G0_2_0 + 0.00329461321214873*G0_2_1;
    A[1641] = 0.00450155072550949*G0_0_0 + 0.00450155072550962*G0_0_1 + 0.00257697469069074*G0_0_2 + 0.00450155072550866*G0_1_0 + 0.00450155072550874*G0_1_1 + 0.00257697469069008*G0_1_2 + 0.00257697469068967*G0_2_0 + 0.00257697469068972*G0_2_1;
    A[1642] = -0.0105036183595215*G0_0_0 - 0.0105036183595215*G0_0_1 + 0.00257697469068959*G0_0_2 - 0.0105036183595209*G0_1_0 - 0.0105036183595208*G0_1_1 + 0.00257697469068964*G0_1_2 + 0.00257697469069049*G0_2_0 + 0.0025769746906905*G0_2_1;
    A[1643] = -0.00254435475789627*G0_0_0 - 0.00254435475789626*G0_0_1 + 0.00329461321214807*G0_0_2 - 0.0025443547578963*G0_1_0 - 0.00254435475789631*G0_1_1 + 0.00329461321214834*G0_1_2 + 0.00329461321214759*G0_2_0 + 0.00329461321214759*G0_2_1;
    A[1644] = 0.0292600797158085*G0_0_0 + 0.0259654665036602*G0_0_1 + 0.0292600797158086*G0_0_2 + 0.0259654665036615*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0259654665036616*G0_1_2 + 0.0292600797158091*G0_2_0 + 0.0259654665036608*G0_2_1 + 0.0292600797158092*G0_2_2;
    A[1645] = -0.0189848008858425*G0_0_0 - 0.00949240044292083*G0_0_1 - 0.018984800885843*G0_0_2 - 0.00949240044292221*G0_1_0 - 0.000652398655870894*G0_1_1 - 0.0094924004429222*G0_1_2 - 0.018984800885843*G0_2_0 - 0.00949240044292133*G0_2_1 - 0.0189848008858432*G0_2_2;
    A[1646] = 0.00352295274170198*G0_0_0 - 0.00296841388421278*G0_0_1 + 0.00352295274170215*G0_0_2 - 0.00296841388421203*G0_1_0 - 0.000652398655870912*G0_1_1 - 0.00296841388421208*G0_1_2 + 0.00352295274170312*G0_2_0 - 0.00296841388421253*G0_2_1 + 0.00352295274170314*G0_2_2;
    A[1647] = 0.0429604514890969*G0_0_0 + 0.000685018588664759*G0_0_1 + 0.0429604514890969*G0_0_2 + 0.000685018588664034*G0_1_0 + 0.0226708532915131*G0_1_1 + 0.000685018588664022*G0_1_2 + 0.0429604514890962*G0_2_0 + 0.000685018588664644*G0_2_1 + 0.0429604514890963*G0_2_2;
    A[1648] = 0.0226708532915125*G0_0_0 + 0.0259654665036608*G0_0_1 + 0.0259654665036608*G0_0_2 + 0.0259654665036605*G0_1_0 + 0.0292600797158092*G0_1_1 + 0.0292600797158093*G0_1_2 + 0.0259654665036606*G0_2_0 + 0.0292600797158091*G0_2_1 + 0.0292600797158091*G0_2_2;
    A[1649] = -0.000652398655870768*G0_0_0 - 0.00949240044292139*G0_0_1 - 0.0094924004429213*G0_0_2 - 0.00949240044292118*G0_1_0 - 0.018984800885843*G0_1_1 - 0.0189848008858432*G0_1_2 - 0.009492400442921*G0_2_0 - 0.0189848008858429*G0_2_1 - 0.0189848008858429*G0_2_2;
    A[1650] = -0.000652398655870754*G0_0_0 - 0.00296841388421233*G0_0_1 - 0.00296841388421223*G0_0_2 - 0.00296841388421235*G0_1_0 + 0.00352295274170335*G0_1_1 + 0.00352295274170347*G0_1_2 - 0.00296841388421254*G0_2_0 + 0.0035229527417036*G0_2_1 + 0.00352295274170362*G0_2_2;
    A[1651] = 0.0226708532915129*G0_0_0 + 0.000685018588664619*G0_0_1 + 0.000685018588664748*G0_0_2 + 0.000685018588664357*G0_1_0 + 0.0429604514890971*G0_1_1 + 0.0429604514890973*G0_1_2 + 0.000685018588664378*G0_2_0 + 0.0429604514890966*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[1652] = -0.268135847562929*G0_0_0 - 0.0737210481134091*G0_0_1 - 0.0459941052388967*G0_0_2 + 0.180714427676232*G0_1_0 - 0.209419968534551*G0_1_1 + 0.0812236326559224*G0_1_2 + 0.0225077536275458*G0_2_0 - 0.104709984267275*G0_2_1 - 0.023486351611351*G0_2_2;
    A[1653] = 0.726119703984282*G0_0_0 + 0.270093043530542*G0_0_1 + 0.270093043530542*G0_0_2 + 0.270093043530542*G0_1_0 + 0.726119703984283*G0_1_1 + 0.270093043530543*G0_1_2 + 0.270093043530542*G0_2_0 + 0.270093043530543*G0_2_1 + 0.540186087061083*G0_2_2;
    A[1654] = -0.209419968534549*G0_0_0 + 0.180714427676233*G0_0_1 + 0.0812236326559231*G0_0_2 - 0.0737210481134073*G0_1_0 - 0.268135847562928*G0_1_1 - 0.0459941052388957*G0_1_2 - 0.104709984267274*G0_2_0 + 0.022507753627545*G0_2_1 - 0.0234863516113499*G0_2_2;
    A[1655] = 0.0655660649150237*G0_0_0 - 0.0748627457611824*G0_0_1 + 0.0278900425384809*G0_0_2 + 0.0278900425384802*G0_1_0 - 0.149725491522365*G0_1_1 + 0.177126235068943*G0_1_2 - 0.0748627457611817*G0_2_0 - 0.0283793415303826*G0_2_1 - 0.149725491522364*G0_2_2;
    A[1656] = -0.149725491522366*G0_0_0 + 0.0278900425384801*G0_0_1 + 0.177126235068944*G0_0_2 - 0.0748627457611828*G0_1_0 + 0.0655660649150229*G0_1_1 + 0.0278900425384802*G0_1_2 - 0.0283793415303843*G0_2_0 - 0.0748627457611836*G0_2_1 - 0.149725491522366*G0_2_2;
    A[1657] = -0.013700371773288*G0_0_0 - 0.0198981590040614*G0_0_1 - 0.08513802459115*G0_0_2 - 0.0198981590040608*G0_1_0 - 0.0137003717732881*G0_1_1 - 0.0851380245911492*G0_1_2 + 0.0127217737894828*G0_2_0 + 0.0127217737894834*G0_2_1 + 0.113517366121532*G0_2_2;
    A[1658] = -0.0254435475789635*G0_0_0 - 0.0241387502672223*G0_0_1 - 0.0127217737894811*G0_0_2 - 0.0241387502672219*G0_1_0 - 0.0228339529554816*G0_1_1 - 0.0120693751336118*G0_1_2 - 0.0127217737894818*G0_2_0 - 0.0120693751336125*G0_2_1;
    A[1659] = 0.0567586830607655*G0_0_0 + 0.0283793415303831*G0_0_1 + 0.0283793415303842*G0_0_2 + 0.0283793415303819*G0_1_0 - 0.00489298991903111*G0_1_1 + 0.0141896707651918*G0_1_2 + 0.0283793415303817*G0_2_0 + 0.0141896707651927*G0_2_1;
    A[1660] = 0.209419968534549*G0_0_0 + 0.0737210481134081*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0737210481134072*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.0368605240567036*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0368605240567031*G0_2_1;
    A[1661] = -0.0166361657247074*G0_0_0 + 0.00146789697570903*G0_0_1 - 0.0181040627004182*G0_0_2 + 0.00146789697571028*G0_1_0 + 0.0195719596761268*G0_1_1 - 0.0102752788299656*G0_1_2 - 0.0181040627004164*G0_2_0 - 0.0102752788299665*G0_2_1;
    A[1662] = 0.149725491522366*G0_0_0 + 0.0748627457611825*G0_0_1 + 0.0283793415303834*G0_0_2 + 0.0748627457611828*G0_1_0 + 0.0195719596761262*G0_1_1 + 0.0141896707651924*G0_1_2 + 0.0283793415303842*G0_2_0 + 0.0141896707651922*G0_2_1;
    A[1663] = 0.0137003717732884*G0_0_0 + 0.0198981590040619*G0_0_1 - 0.0127217737894818*G0_0_2 + 0.0198981590040609*G0_1_0 + 0.026095946234834*G0_1_1 - 0.0120693751336124*G0_1_2 - 0.0127217737894827*G0_2_0 - 0.0120693751336116*G0_2_1;
    A[1664] = -0.0228339529554802*G0_0_0 - 0.0241387502672217*G0_0_1 - 0.0120693751336105*G0_0_2 - 0.0241387502672232*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.0127217737894827*G0_1_2 - 0.0120693751336122*G0_2_0 - 0.0127217737894818*G0_2_1;
    A[1665] = -0.00489298991903095*G0_0_0 + 0.0283793415303833*G0_0_1 + 0.0141896707651916*G0_0_2 + 0.0283793415303841*G0_1_0 + 0.0567586830607663*G0_1_1 + 0.0283793415303838*G0_1_2 + 0.0141896707651925*G0_2_0 + 0.0283793415303838*G0_2_1;
    A[1666] = -0.022833952955481*G0_0_0 + 0.0737210481134092*G0_0_1 + 0.0368605240567042*G0_0_2 + 0.0737210481134081*G0_1_0 + 0.209419968534551*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0368605240567035*G0_2_0 + 0.104709984267274*G0_2_1;
    A[1667] = 0.019571959676125*G0_0_0 + 0.00146789697570858*G0_0_1 - 0.0102752788299675*G0_0_2 + 0.00146789697570971*G0_1_0 - 0.0166361657247068*G0_1_1 - 0.0181040627004161*G0_1_2 - 0.0102752788299661*G0_2_0 - 0.018104062700416*G0_2_1;
    A[1668] = 0.0195719596761269*G0_0_0 + 0.074862745761182*G0_0_1 + 0.0141896707651916*G0_0_2 + 0.0748627457611828*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0283793415303834*G0_1_2 + 0.0141896707651919*G0_2_0 + 0.0283793415303823*G0_2_1;
    A[1669] = 0.0260959462348352*G0_0_0 + 0.0198981590040615*G0_0_1 - 0.0120693751336107*G0_0_2 + 0.0198981590040612*G0_1_0 + 0.013700371773288*G0_1_1 - 0.0127217737894828*G0_1_2 - 0.0120693751336122*G0_2_0 - 0.0127217737894833*G0_2_1;
    A[1670] = -0.0880738185425672*G0_0_0 - 0.0887262171984385*G0_0_1 - 0.100795592332049*G0_0_2 - 0.0887262171984391*G0_1_0 - 0.0880738185425681*G0_1_1 - 0.100795592332049*G0_1_2 - 0.10079559233205*G0_2_0 - 0.10079559233205*G0_2_1 - 0.113517366121532*G0_2_2;
    A[1671] = 0.0195719596761263*G0_0_0 + 0.0606730749959919*G0_0_1 + 0.0748627457611837*G0_0_2 + 0.0606730749959911*G0_1_0 + 0.0929668084615995*G0_1_1 + 0.121346149991983*G0_1_2 + 0.074862745761183*G0_2_0 + 0.121346149991982*G0_2_1 + 0.149725491522366*G0_2_2;
    A[1672] = -0.0880738185425685*G0_0_0 + 0.00913358118219142*G0_0_1 + 0.0459941052388961*G0_0_2 + 0.00913358118219213*G0_1_0 - 0.185933616923199*G0_1_1 - 0.0812236326559231*G0_1_2 + 0.0459941052388959*G0_2_0 - 0.0812236326559246*G0_2_1 + 0.0234863516113507*G0_2_2;
    A[1673] = 0.0929668084615992*G0_0_0 + 0.0606730749959901*G0_0_1 + 0.121346149991981*G0_0_2 + 0.0606730749959915*G0_1_0 + 0.0195719596761259*G0_1_1 + 0.0748627457611818*G0_1_2 + 0.121346149991982*G0_2_0 + 0.0748627457611831*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1674] = 0.0929668084615991*G0_0_0 - 0.135046521765271*G0_0_1 - 0.270093043530541*G0_0_2 - 0.135046521765271*G0_1_0 + 0.0929668084615996*G0_1_1 - 0.270093043530542*G0_1_2 - 0.270093043530541*G0_2_0 - 0.270093043530541*G0_2_1 - 0.540186087061083*G0_2_2;
    A[1675] = -0.185933616923197*G0_0_0 + 0.00913358118219283*G0_0_1 - 0.0812236326559239*G0_0_2 + 0.00913358118219195*G0_1_0 - 0.0880738185425683*G0_1_1 + 0.0459941052388955*G0_1_2 - 0.0812236326559239*G0_2_0 + 0.0459941052388951*G0_2_1 + 0.0234863516113497*G0_2_2;
    A[1676] = 0.146789697570945*G0_0_0 + 0.141896707651913*G0_0_1 + 0.092966808461598*G0_0_2 + 0.141896707651914*G0_1_0 + 0.146789697570944*G0_1_1 + 0.092966808461597*G0_1_2 + 0.0929668084615992*G0_2_0 + 0.0929668084615971*G0_2_1;
    A[1677] = 0.146789697570948*G0_0_0 - 0.298472385060926*G0_0_1 - 0.102752788299665*G0_0_2 - 0.298472385060923*G0_1_0 - 0.782878387045049*G0_1_1 - 0.298472385060926*G0_1_2 - 0.102752788299663*G0_2_0 - 0.298472385060926*G0_2_1;
    A[1678] = -0.782878387045047*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060925*G0_0_2 - 0.298472385060924*G0_1_0 + 0.146789697570947*G0_1_1 - 0.102752788299662*G0_1_2 - 0.298472385060923*G0_2_0 - 0.102752788299661*G0_2_1;
    A[1679] = -0.0489298991903156*G0_0_0 - 0.0293579395141881*G0_0_1 + 0.0929668084616016*G0_0_2 - 0.0293579395141905*G0_1_0 - 0.0489298991903157*G0_1_1 + 0.0929668084616005*G0_1_2 + 0.0929668084615982*G0_2_0 + 0.0929668084615999*G0_2_1;
    A[1680] = -0.0137421252872639*G0_0_0 - 0.013742125287264*G0_0_1 - 0.013742125287264*G0_0_2 - 0.0137421252872638*G0_1_0 - 0.013742125287264*G0_1_1 - 0.0137421252872639*G0_1_2 - 0.0137421252872639*G0_2_0 - 0.013742125287264*G0_2_1 - 0.013742125287264*G0_2_2;
    A[1681] = 0.0137421252872644*G0_0_0 - 0.00517743573299117*G0_1_0 - 0.00517743573299103*G0_2_0;
    A[1682] = 0.0143945239431351*G0_0_1 + 0.0479930547204852*G0_1_1 + 0.014394523943135*G0_2_1;
    A[1683] = -0.00517743573299122*G0_0_2 - 0.00517743573299119*G0_1_2 + 0.013742125287264*G0_2_2;
    A[1684] = 0.192588083213082*G0_0_1 + 0.124738623002512*G0_0_2 - 0.0644569872000421*G0_1_1 + 0.0284445813959708*G0_1_2 + 0.0962940416065415*G0_2_1 + 0.124738623002512*G0_2_2;
    A[1685] = 0.0109602974186295*G0_0_1 + 0.146659217839771*G0_0_2 + 0.132306447410613*G0_1_1 + 0.0733296089198853*G0_1_2 - 0.0623693115012558*G0_2_1 - 0.124738623002511*G0_2_2;
    A[1686] = 0.00182671623643951*G0_0_1 - 0.0823327103709031*G0_0_2 - 0.0510175748891017*G0_1_1 - 0.0463855444324181*G0_1_2 + 0.0377738821749245*G0_2_1 + 0.127087258163647*G0_2_2;
    A[1687] = 0.0118736555368493*G0_0_1 + 0.0477555816097475*G0_0_2 + 0.0118736555368495*G0_1_1 + 0.026487385428357*G0_1_2 - 0.00939454064454097*G0_2_1 - 0.0641307878721069*G0_2_2;
    A[1688] = -0.0187890812890813*G0_0_0 - 0.00939454064454018*G0_0_2 + 0.0425363923627807*G0_1_0 + 0.0118736555368499*G0_1_2 + 0.0212681961813898*G0_2_0 + 0.0118736555368497*G0_2_2;
    A[1689] = 0.014874689353856*G0_0_0 + 0.0149399292194421*G0_0_2 + 0.0403182369328203*G0_1_0 + 0.0540186087061079*G0_1_2 + 0.00123955744615523*G0_2_0 + 0.00117431758056765*G0_2_2;
    A[1690] = 0.00117431758056772*G0_0_0 + 0.00123955744615541*G0_0_2 + 0.0540186087061078*G0_1_0 + 0.0403182369328201*G0_1_2 + 0.0149399292194429*G0_2_0 + 0.0148746893538561*G0_2_2;
    A[1691] = 0.0118736555368494*G0_0_0 + 0.0212681961813897*G0_0_2 + 0.0118736555368496*G0_1_0 + 0.0425363923627803*G0_1_2 - 0.00939454064454101*G0_2_0 - 0.0187890812890815*G0_2_2;
    A[1692] = -0.064130787872107*G0_0_0 - 0.00939454064454032*G0_0_1 + 0.0264873854283574*G0_1_0 + 0.0118736555368499*G0_1_1 + 0.047755581609747*G0_2_0 + 0.0118736555368495*G0_2_1;
    A[1693] = 0.127087258163647*G0_0_0 + 0.0377738821749238*G0_0_1 - 0.0463855444324186*G0_1_0 - 0.0510175748891019*G0_1_1 - 0.0823327103709026*G0_2_0 + 0.00182671623643915*G0_2_1;
    A[1694] = -0.124738623002512*G0_0_0 - 0.0623693115012556*G0_0_1 + 0.0733296089198851*G0_1_0 + 0.132306447410613*G0_1_1 + 0.146659217839771*G0_2_0 + 0.0109602974186301*G0_2_1;
    A[1695] = 0.124738623002512*G0_0_0 + 0.0962940416065415*G0_0_1 + 0.0284445813959708*G0_1_0 - 0.0644569872000423*G0_1_1 + 0.124738623002512*G0_2_0 + 0.192588083213082*G0_2_1;
    A[1696] = 0.0187890812890812*G0_0_0 + 0.0187890812890813*G0_0_1 + 0.00939454064454066*G0_0_2 + 0.0187890812890806*G0_1_0 + 0.0187890812890807*G0_1_1 + 0.00939454064454026*G0_1_2 + 0.00939454064454053*G0_2_0 + 0.00939454064454058*G0_2_1;
    A[1697] = -0.0148746893538564*G0_0_0 - 0.0148746893538564*G0_0_1 + 6.5239865586953e-05*G0_0_2 - 0.0148746893538557*G0_1_0 - 0.0148746893538557*G0_1_1 + 6.52398655874923e-05*G0_1_2 + 6.52398655865112e-05*G0_2_0 + 6.52398655865877e-05*G0_2_1;
    A[1698] = -0.00117431758056735*G0_0_0 - 0.00117431758056749*G0_0_1 + 6.52398655870585e-05*G0_0_2 - 0.00117431758056776*G0_1_0 - 0.00117431758056796*G0_1_1 + 6.52398655869414e-05*G0_1_2 + 6.52398655870317e-05*G0_2_0 + 6.52398655869479e-05*G0_2_1;
    A[1699] = -0.0118736555368495*G0_0_0 - 0.0118736555368495*G0_0_1 + 0.00939454064454069*G0_0_2 - 0.0118736555368496*G0_1_0 - 0.0118736555368496*G0_1_1 + 0.00939454064454043*G0_1_2 + 0.00939454064454088*G0_2_0 + 0.0093945406445409*G0_2_1;
    A[1700] = 0.0641307878721069*G0_0_0 + 0.0547362472275661*G0_0_1 + 0.0641307878721069*G0_0_2 + 0.0547362472275656*G0_1_0 + 0.0453417065830254*G0_1_1 + 0.0547362472275657*G0_1_2 + 0.0641307878721065*G0_2_0 + 0.0547362472275659*G0_2_1 + 0.0641307878721065*G0_2_2;
    A[1701] = -0.127087258163646*G0_0_0 - 0.0893133759887225*G0_0_1 - 0.127087258163646*G0_0_2 - 0.089313375988722*G0_1_0 - 0.0665446628988287*G0_1_1 - 0.0893133759887223*G0_1_2 - 0.127087258163645*G0_2_0 - 0.089313375988722*G0_2_1 - 0.127087258163646*G0_2_2;
    A[1702] = 0.124738623002511*G0_0_0 + 0.0623693115012556*G0_0_1 + 0.124738623002511*G0_0_2 + 0.0623693115012547*G0_1_0 + 0.0476251018785735*G0_1_1 + 0.0623693115012549*G0_1_2 + 0.124738623002511*G0_2_0 + 0.062369311501255*G0_2_1 + 0.12473862300251*G0_2_2;
    A[1703] = -0.124738623002512*G0_0_0 - 0.0284445813959706*G0_0_1 - 0.124738623002512*G0_0_2 - 0.0284445813959707*G0_1_0 - 0.0606730749959915*G0_1_1 - 0.0284445813959707*G0_1_2 - 0.124738623002511*G0_2_0 - 0.0284445813959701*G0_2_1 - 0.124738623002511*G0_2_2;
    A[1704] = 0.0093945406445408*G0_0_1 + 0.00939454064454076*G0_0_2 + 0.00939454064454041*G0_1_0 + 0.0187890812890809*G0_1_1 + 0.0187890812890809*G0_1_2 + 0.00939454064454033*G0_2_0 + 0.0187890812890809*G0_2_1 + 0.0187890812890808*G0_2_2;
    A[1705] = 6.52398655869219e-05*G0_0_1 + 6.52398655868009e-05*G0_0_2 + 6.52398655870779e-05*G0_1_0 - 0.0148746893538558*G0_1_1 - 0.0148746893538558*G0_1_2 + 6.5239865587292e-05*G0_2_0 - 0.0148746893538554*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1706] = 6.52398655874084e-05*G0_0_1 + 6.52398655874137e-05*G0_0_2 + 6.52398655868975e-05*G0_1_0 - 0.0011743175805679*G0_1_1 - 0.00117431758056792*G0_1_2 + 6.5239865586911e-05*G0_2_0 - 0.00117431758056774*G0_2_1 - 0.00117431758056767*G0_2_2;
    A[1707] = 0.00939454064454022*G0_0_1 + 0.00939454064454037*G0_0_2 + 0.00939454064454081*G0_1_0 - 0.0118736555368497*G0_1_1 - 0.0118736555368497*G0_1_2 + 0.00939454064454056*G0_2_0 - 0.0118736555368497*G0_2_1 - 0.0118736555368494*G0_2_2;
    A[1708] = 0.105688582251081*G0_0_0 + 0.0300103381700597*G0_0_1 + 0.052844291125541*G0_0_2 - 0.0743734467692796*G0_1_0 + 0.105688582251081*G0_1_1 + 0.00065239865587128*G0_1_2 + 0.000652398655872153*G0_2_0 + 0.0528442911255405*G0_2_1 + 0.053496689781411*G0_2_2;
    A[1709] = -0.209419968534549*G0_0_0 - 0.0737210481134073*G0_0_1 - 0.104709984267274*G0_0_2 + 0.180714427676233*G0_1_0 - 0.268135847562928*G0_1_1 + 0.022507753627545*G0_1_2 + 0.0812236326559231*G0_2_0 - 0.0459941052388957*G0_2_1 - 0.0234863516113499*G0_2_2;
    A[1710] = 0.542795681684567*G0_0_0 + 0.271397840842284*G0_0_1 + 0.271397840842285*G0_0_2 + 0.271397840842284*G0_1_0 + 0.647179466623906*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842285*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684566*G0_2_2;
    A[1711] = -0.0137003717732897*G0_0_0 + 0.0127217737894812*G0_0_1 - 0.0198981590040626*G0_0_2 - 0.0851380245911505*G0_1_0 + 0.113517366121531*G0_1_1 - 0.0851380245911501*G0_1_2 - 0.0198981590040634*G0_2_0 + 0.0127217737894816*G0_2_1 - 0.0137003717732885*G0_2_2;
    A[1712] = -0.0234863516113522*G0_0_0 - 0.0459941052388969*G0_0_1 + 0.0812236326559224*G0_0_2 + 0.0225077536275456*G0_1_0 - 0.268135847562928*G0_1_1 + 0.180714427676233*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0737210481134093*G0_2_1 - 0.209419968534551*G0_2_2;
    A[1713] = 0.0534966897814123*G0_0_0 + 0.0528442911255405*G0_0_1 + 0.000652398655871374*G0_0_2 + 0.000652398655871528*G0_1_0 + 0.105688582251081*G0_1_1 - 0.0743734467692798*G0_1_2 + 0.0528442911255414*G0_2_0 + 0.0300103381700599*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1714] = -0.105688582251082*G0_0_0 - 0.0756782440810214*G0_0_1 - 0.0528442911255412*G0_0_2 - 0.0756782440810215*G0_1_0 - 0.0456679059109607*G0_1_1 - 0.0378391220405101*G0_1_2 - 0.0528442911255416*G0_2_0 - 0.0378391220405111*G0_2_1;
    A[1715] = 0.20941996853455*G0_0_0 + 0.135698920421141*G0_0_1 + 0.104709984267275*G0_0_2 + 0.135698920421142*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0678494602105711*G0_1_2 + 0.104709984267277*G0_2_0 + 0.0678494602105714*G0_2_1;
    A[1716] = -0.542795681684567*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842282*G0_0_2 - 0.271397840842284*G0_1_0 + 0.0521918924696699*G0_1_1 - 0.13569892042114*G0_1_2 - 0.271397840842284*G0_2_0 - 0.135698920421142*G0_2_1;
    A[1717] = 0.0137003717732895*G0_0_0 + 0.0264221455627718*G0_0_1 - 0.00619778723077217*G0_0_2 + 0.0264221455627711*G0_1_0 + 0.039143919352253*G0_1_1 - 0.00554538857490269*G0_1_2 - 0.00619778723077275*G0_2_0 - 0.00554538857490091*G0_2_1;
    A[1718] = 0.0234863516113524*G0_0_0 - 0.0225077536275449*G0_0_1 + 0.104709984267274*G0_0_2 - 0.0225077536275456*G0_1_0 - 0.15657567740901*G0_1_1 + 0.0678494602105691*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0678494602105701*G0_2_1;
    A[1719] = -0.0534966897814124*G0_0_0 - 0.000652398655872186*G0_0_1 - 0.0528442911255413*G0_0_2 - 0.000652398655871362*G0_1_0 + 0.0521918924696695*G0_1_1 - 0.0378391220405102*G0_1_2 - 0.0528442911255413*G0_2_0 - 0.0378391220405115*G0_2_1;
    A[1720] = -0.0300103381700609*G0_0_1 - 0.0150051690850309*G0_0_2 - 0.0300103381700599*G0_1_0 - 0.0600206763401212*G0_1_1 - 0.0300103381700602*G0_1_2 - 0.0150051690850297*G0_2_0 - 0.0300103381700607*G0_2_1;
    A[1721] = -0.00130479731174149*G0_0_1 - 0.000652398655870719*G0_0_2 - 0.00130479731174117*G0_1_0 - 0.0254435475789641*G0_1_1 - 0.0127217737894819*G0_1_2 - 0.00065239865587078*G0_2_0 - 0.012721773789482*G0_2_1;
    A[1722] = -0.0300103381700597*G0_0_1 - 0.0150051690850296*G0_0_2 - 0.03001033817006*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0528442911255399*G0_1_2 - 0.0150051690850295*G0_2_0 - 0.0528442911255402*G0_2_1;
    A[1723] = -0.012721773789481*G0_0_1 - 0.000652398655869348*G0_0_2 - 0.012721773789482*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.00130479731174136*G0_1_2 - 0.000652398655870307*G0_2_0 - 0.00130479731174192*G0_2_1;
    A[1724] = -0.0127217737894812*G0_0_1 - 0.000652398655871309*G0_0_2 - 0.0127217737894824*G0_1_0 - 0.113517366121531*G0_1_1 - 0.0127217737894822*G0_1_2 - 0.000652398655871264*G0_2_0 - 0.0127217737894816*G0_2_1;
    A[1725] = -0.0528442911255401*G0_0_1 - 0.0150051690850304*G0_0_2 - 0.0528442911255401*G0_1_0 - 0.10568858225108*G0_1_1 - 0.0300103381700598*G0_1_2 - 0.0150051690850299*G0_2_0 - 0.0300103381700596*G0_2_1;
    A[1726] = -0.0378391220405109*G0_0_1 - 0.0528442911255409*G0_0_2 - 0.0378391220405101*G0_1_0 - 0.0456679059109612*G0_1_1 - 0.0756782440810219*G0_1_2 - 0.0528442911255394*G0_2_0 - 0.0756782440810205*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1727] = -0.00554538857490302*G0_0_1 - 0.0061977872307736*G0_0_2 - 0.00554538857490207*G0_1_0 + 0.0391439193522526*G0_1_1 + 0.0264221455627702*G0_1_2 - 0.00619778723077418*G0_2_0 + 0.0264221455627691*G0_2_1 + 0.0137003717732875*G0_2_2;
    A[1728] = -0.037839122040511*G0_0_1 - 0.0528442911255411*G0_0_2 - 0.0378391220405108*G0_1_0 + 0.0521918924696696*G0_1_1 - 0.000652398655871277*G0_1_2 - 0.05284429112554*G0_2_0 - 0.000652398655870949*G0_2_1 - 0.0534966897814114*G0_2_2;
    A[1729] = 0.0678494602105705*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105694*G0_1_0 + 0.0391439193522516*G0_1_1 + 0.13569892042114*G0_1_2 + 0.104709984267273*G0_2_0 + 0.13569892042114*G0_2_1 + 0.209419968534549*G0_2_2;
    A[1730] = 0.0678494602105715*G0_0_1 + 0.104709984267274*G0_0_2 + 0.0678494602105706*G0_1_0 - 0.156575677409009*G0_1_1 - 0.022507753627545*G0_1_2 + 0.104709984267275*G0_2_0 - 0.0225077536275443*G0_2_1 + 0.0234863516113504*G0_2_2;
    A[1731] = -0.135698920421142*G0_0_1 - 0.271397840842284*G0_0_2 - 0.13569892042114*G0_1_0 + 0.0521918924696703*G0_1_1 - 0.271397840842283*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842282*G0_2_1 - 0.542795681684566*G0_2_2;
    A[1732] = 0.075025845425152*G0_0_1 + 0.0260959462348362*G0_0_2 + 0.0750258454251501*G0_1_0 + 0.195719596761264*G0_1_1 + 0.0750258454251528*G0_1_2 + 0.0260959462348353*G0_2_0 + 0.0750258454251522*G0_2_1;
    A[1733] = 0.0750258454251495*G0_0_1 + 0.0260959462348364*G0_0_2 + 0.0750258454251509*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0587158790283798*G0_1_2 + 0.026095946234834*G0_2_0 + 0.0587158790283776*G0_2_1;
    A[1734] = -0.316413348097375*G0_0_1 - 0.185933616923198*G0_0_2 - 0.316413348097375*G0_1_0 - 0.782878387045049*G0_1_1 - 0.316413348097374*G0_1_2 - 0.1859336169232*G0_2_0 - 0.316413348097374*G0_2_1;
    A[1735] = 0.0587158790283779*G0_0_1 + 0.0260959462348339*G0_0_2 + 0.0587158790283794*G0_1_0 + 0.293579395141892*G0_1_1 + 0.0750258454251496*G0_1_2 + 0.0260959462348363*G0_2_0 + 0.0750258454251514*G0_2_1;
    A[1736] = -0.00946499969937472*G0_0_0 - 0.00946499969937471*G0_0_1 - 0.00946499969937467*G0_0_2 - 0.00946499969937471*G0_1_0 - 0.00946499969937476*G0_1_1 - 0.00946499969937468*G0_1_2 - 0.00946499969937471*G0_2_0 - 0.0094649996993747*G0_2_1 - 0.00946499969937467*G0_2_2;
    A[1737] = -0.0345719095719095*G0_0_0 - 0.00472467106581687*G0_1_0 - 0.00472467106581704*G0_2_0;
    A[1738] = -0.00227817610630123*G0_0_1 + 0.00946499969937454*G0_1_1 - 0.00227817610630115*G0_2_1;
    A[1739] = -0.00472467106581705*G0_0_2 - 0.00472467106581688*G0_1_2 - 0.0345719095719093*G0_2_2;
    A[1740] = 0.0142875305635721*G0_0_1 + 0.00254435475789596*G0_0_2 - 0.0292600797158083*G0_1_1 - 0.00329461321214781*G0_1_2 + 0.00844856259352822*G0_2_1 + 0.00254435475789701*G0_2_2;
    A[1741] = -0.0377738821749246*G0_0_1 - 0.00939454064454045*G0_0_2 + 0.0189848008858417*G0_1_1 + 0.00949240044292087*G0_1_2 - 0.0188869410874625*G0_2_1 - 0.0155923278753151*G0_2_2;
    A[1742] = -0.108232937008977*G0_0_1 - 0.0808321934624018*G0_0_2 - 0.00352295274170204*G0_1_1 - 0.00649136662591468*G0_1_2 - 0.0338921101724914*G0_2_1 - 0.0436454700777603*G0_2_2;
    A[1743] = -0.042960451489097*G0_0_1 - 0.0845508658008643*G0_0_2 - 0.0429604514890968*G0_1_1 - 0.0422754329004322*G0_1_2 - 0.000685018588664222*G0_2_1 + 0.0439716694056971*G0_2_2;
    A[1744] = 0.127315597693201*G0_0_0 - 0.000685018588664407*G0_0_2 - 0.0493213383838382*G0_1_0 - 0.0429604514890971*G0_1_2 - 0.00704590548340538*G0_2_0 - 0.0429604514890971*G0_2_2;
    A[1745] = -0.166948816037357*G0_0_0 - 0.00616516729797994*G0_0_2 + 0.167731694424402*G0_1_0 + 0.0777006799142212*G0_1_2 + 0.0838658472122009*G0_2_0 + 0.182410664181496*G0_2_2;
    A[1746] = 0.182410664181496*G0_0_0 + 0.0838658472122*G0_0_2 + 0.0777006799142199*G0_1_0 + 0.167731694424401*G0_1_2 - 0.00616516729798082*G0_2_0 - 0.166948816037357*G0_2_2;
    A[1747] = -0.0429604514890971*G0_0_0 - 0.00704590548340509*G0_0_2 - 0.042960451489097*G0_1_0 - 0.0493213383838381*G0_1_2 - 0.000685018588664377*G0_2_0 + 0.127315597693201*G0_2_2;
    A[1748] = 0.043971669405697*G0_0_0 - 0.00068501858866448*G0_0_1 - 0.0422754329004326*G0_1_0 - 0.042960451489097*G0_1_1 - 0.0845508658008649*G0_2_0 - 0.0429604514890971*G0_2_1;
    A[1749] = -0.0436454700777618*G0_0_0 - 0.0338921101724923*G0_0_1 - 0.00649136662591511*G0_1_0 - 0.00352295274170281*G0_1_1 - 0.0808321934624013*G0_2_0 - 0.108232937008978*G0_2_1;
    A[1750] = -0.015592327875314*G0_0_0 - 0.0188869410874615*G0_0_1 + 0.00949240044292113*G0_1_0 + 0.0189848008858425*G0_1_1 - 0.00939454064454047*G0_2_0 - 0.0377738821749238*G0_2_1;
    A[1751] = 0.00254435475789588*G0_0_0 + 0.00844856259352731*G0_0_1 - 0.00329461321214786*G0_1_0 - 0.0292600797158085*G0_1_1 + 0.00254435475789675*G0_2_0 + 0.0142875305635727*G0_2_1;
    A[1752] = 0.0292600797158083*G0_0_0 + 0.0292600797158083*G0_0_1 + 0.0259654665036608*G0_0_2 + 0.0292600797158087*G0_1_0 + 0.0292600797158087*G0_1_1 + 0.0259654665036609*G0_1_2 + 0.0259654665036606*G0_2_0 + 0.0259654665036606*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[1753] = -0.0189848008858421*G0_0_0 - 0.018984800885842*G0_0_1 - 0.00949240044292102*G0_0_2 - 0.0189848008858425*G0_1_0 - 0.0189848008858425*G0_1_1 - 0.00949240044292133*G0_1_2 - 0.00949240044292113*G0_2_0 - 0.00949240044292119*G0_2_1 - 0.00065239865587079*G0_2_2;
    A[1754] = 0.00352295274170224*G0_0_0 + 0.00352295274170235*G0_0_1 - 0.00296841388421287*G0_0_2 + 0.00352295274170291*G0_1_0 + 0.00352295274170288*G0_1_1 - 0.00296841388421261*G0_1_2 - 0.00296841388421203*G0_2_0 - 0.00296841388421206*G0_2_1 - 0.000652398655870801*G0_2_2;
    A[1755] = 0.0429604514890971*G0_0_0 + 0.0429604514890971*G0_0_1 + 0.000685018588664856*G0_0_2 + 0.0429604514890969*G0_1_0 + 0.042960451489097*G0_1_1 + 0.000685018588664526*G0_1_2 + 0.000685018588664204*G0_2_0 + 0.000685018588664271*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[1756] = 0.00658922642429563*G0_0_0 + 0.00329461321214814*G0_0_1 + 0.0065892264242957*G0_0_2 + 0.00329461321214803*G0_1_0 + 0.00329461321214806*G0_1_2 + 0.00658922642429615*G0_2_0 + 0.00329461321214843*G0_2_1 + 0.0065892264242961*G0_2_2;
    A[1757] = 0.00450155072550967*G0_0_0 + 0.00257697469068974*G0_0_1 + 0.00450155072550945*G0_0_2 + 0.00257697469069014*G0_1_0 + 0.00257697469068994*G0_1_2 + 0.00450155072550873*G0_2_0 + 0.00257697469068931*G0_2_1 + 0.00450155072550856*G0_2_2;
    A[1758] = -0.0105036183595215*G0_0_0 + 0.0025769746906901*G0_0_1 - 0.0105036183595214*G0_0_2 + 0.00257697469069001*G0_1_0 + 0.00257697469069014*G0_1_2 - 0.0105036183595206*G0_2_0 + 0.00257697469069021*G0_2_1 - 0.0105036183595205*G0_2_2;
    A[1759] = -0.00254435475789585*G0_0_0 + 0.003294613212148*G0_0_1 - 0.00254435475789584*G0_0_2 + 0.00329461321214787*G0_1_0 + 0.00329461321214789*G0_1_2 - 0.00254435475789671*G0_2_0 + 0.00329461321214794*G0_2_1 - 0.00254435475789665*G0_2_2;
    A[1760] = 0.0226708532915131*G0_0_0 + 0.0259654665036606*G0_0_1 + 0.0259654665036606*G0_0_2 + 0.025965466503661*G0_1_0 + 0.0292600797158088*G0_1_1 + 0.0292600797158088*G0_1_2 + 0.0259654665036611*G0_2_0 + 0.0292600797158088*G0_2_1 + 0.0292600797158087*G0_2_2;
    A[1761] = -0.000652398655871443*G0_0_0 - 0.00949240044292136*G0_0_1 - 0.00949240044292135*G0_0_2 - 0.0094924004429216*G0_1_0 - 0.0189848008858425*G0_1_1 - 0.0189848008858424*G0_1_2 - 0.00949240044292168*G0_2_0 - 0.0189848008858426*G0_2_1 - 0.0189848008858424*G0_2_2;
    A[1762] = -0.000652398655870515*G0_0_0 - 0.00296841388421242*G0_0_1 - 0.00296841388421243*G0_0_2 - 0.00296841388421242*G0_1_0 + 0.00352295274170263*G0_1_1 + 0.00352295274170257*G0_1_2 - 0.00296841388421246*G0_2_0 + 0.00352295274170276*G0_2_1 + 0.00352295274170266*G0_2_2;
    A[1763] = 0.0226708532915129*G0_0_0 + 0.000685018588664678*G0_0_1 + 0.000685018588664622*G0_0_2 + 0.000685018588664566*G0_1_0 + 0.0429604514890973*G0_1_1 + 0.0429604514890971*G0_1_2 + 0.000685018588664782*G0_2_0 + 0.0429604514890973*G0_2_1 + 0.0429604514890973*G0_2_2;
    A[1764] = -0.268135847562931*G0_0_0 - 0.0459941052388981*G0_0_1 - 0.0737210481134107*G0_0_2 + 0.0225077536275446*G0_1_0 - 0.023486351611352*G0_1_1 - 0.104709984267276*G0_1_2 + 0.180714427676231*G0_2_0 + 0.081223632655923*G0_2_1 - 0.209419968534551*G0_2_2;
    A[1765] = 0.0655660649150237*G0_0_0 + 0.0278900425384802*G0_0_1 - 0.0748627457611817*G0_0_2 - 0.0748627457611824*G0_1_0 - 0.149725491522365*G0_1_1 - 0.0283793415303826*G0_1_2 + 0.0278900425384809*G0_2_0 + 0.177126235068943*G0_2_1 - 0.149725491522364*G0_2_2;
    A[1766] = -0.0137003717732897*G0_0_0 - 0.0851380245911505*G0_0_1 - 0.0198981590040634*G0_0_2 + 0.0127217737894812*G0_1_0 + 0.113517366121531*G0_1_1 + 0.0127217737894816*G0_1_2 - 0.0198981590040627*G0_2_0 - 0.0851380245911501*G0_2_1 - 0.0137003717732885*G0_2_2;
    A[1767] = 0.726119703984282*G0_0_0 + 0.27009304353054*G0_0_1 + 0.270093043530541*G0_0_2 + 0.27009304353054*G0_1_0 + 0.540186087061082*G0_1_1 + 0.27009304353054*G0_1_2 + 0.270093043530541*G0_2_0 + 0.27009304353054*G0_2_1 + 0.726119703984282*G0_2_2;
    A[1768] = -0.149725491522366*G0_0_0 + 0.177126235068943*G0_0_1 + 0.0278900425384797*G0_0_2 - 0.0283793415303823*G0_1_0 - 0.149725491522364*G0_1_1 - 0.0748627457611828*G0_1_2 - 0.074862745761182*G0_2_0 + 0.0278900425384818*G0_2_1 + 0.0655660649150232*G0_2_2;
    A[1769] = -0.209419968534549*G0_0_0 + 0.0812236326559237*G0_0_1 + 0.180714427676232*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0234863516113528*G0_1_1 + 0.022507753627545*G0_1_2 - 0.0737210481134085*G0_2_0 - 0.0459941052388979*G0_2_1 - 0.26813584756293*G0_2_2;
    A[1770] = -0.0254435475789634*G0_0_0 - 0.0127217737894812*G0_0_1 - 0.024138750267222*G0_0_2 - 0.0127217737894825*G0_1_0 - 0.0120693751336117*G0_1_2 - 0.0241387502672226*G0_2_0 - 0.0120693751336112*G0_2_1 - 0.0228339529554806*G0_2_2;
    A[1771] = -0.0166361657247066*G0_0_0 - 0.0181040627004171*G0_0_1 + 0.0014678969757098*G0_0_2 - 0.0181040627004158*G0_1_0 - 0.0102752788299662*G0_1_2 + 0.00146789697571017*G0_2_0 - 0.0102752788299655*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[1772] = 0.0137003717732897*G0_0_0 - 0.0127217737894823*G0_0_1 + 0.0198981590040616*G0_0_2 - 0.0127217737894812*G0_1_0 - 0.0120693751336109*G0_1_2 + 0.0198981590040626*G0_2_0 - 0.0120693751336123*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[1773] = 0.0567586830607654*G0_0_0 + 0.0283793415303824*G0_0_1 + 0.0283793415303823*G0_0_2 + 0.0283793415303822*G0_1_0 + 0.0141896707651919*G0_1_2 + 0.0283793415303826*G0_2_0 + 0.0141896707651908*G0_2_1 - 0.00489298991903125*G0_2_2;
    A[1774] = 0.149725491522366*G0_0_0 + 0.0283793415303839*G0_0_1 + 0.0748627457611823*G0_0_2 + 0.0283793415303821*G0_1_0 + 0.0141896707651904*G0_1_2 + 0.0748627457611819*G0_2_0 + 0.0141896707651924*G0_2_1 + 0.0195719596761259*G0_2_2;
    A[1775] = 0.209419968534549*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0737210481134085*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0368605240567044*G0_1_2 + 0.0737210481134085*G0_2_0 + 0.0368605240567034*G0_2_1 - 0.0228339529554811*G0_2_2;
    A[1776] = -0.0880738185425674*G0_0_0 - 0.10079559233205*G0_0_1 - 0.0887262171984382*G0_0_2 - 0.10079559233205*G0_1_0 - 0.113517366121532*G0_1_1 - 0.10079559233205*G0_1_2 - 0.0887262171984387*G0_2_0 - 0.10079559233205*G0_2_1 - 0.0880738185425675*G0_2_2;
    A[1777] = 0.0195719596761274*G0_0_0 + 0.0748627457611826*G0_0_1 + 0.0606730749959918*G0_0_2 + 0.0748627457611833*G0_1_0 + 0.149725491522365*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0606730749959923*G0_2_0 + 0.121346149991982*G0_2_1 + 0.0929668084615994*G0_2_2;
    A[1778] = -0.0880738185425689*G0_0_0 + 0.0459941052388978*G0_0_1 + 0.00913358118219229*G0_0_2 + 0.0459941052388964*G0_1_0 + 0.0234863516113513*G0_1_1 - 0.0812236326559236*G0_1_2 + 0.00913358118219136*G0_2_0 - 0.0812236326559235*G0_2_1 - 0.185933616923199*G0_2_2;
    A[1779] = 0.092966808461598*G0_0_0 + 0.121346149991981*G0_0_1 + 0.0606730749959901*G0_0_2 + 0.121346149991982*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0748627457611828*G0_1_2 + 0.0606730749959902*G0_2_0 + 0.0748627457611824*G0_2_1 + 0.0195719596761266*G0_2_2;
    A[1780] = 0.0929668084616007*G0_0_0 - 0.27009304353054*G0_0_1 - 0.135046521765271*G0_0_2 - 0.270093043530542*G0_1_0 - 0.540186087061082*G0_1_1 - 0.270093043530541*G0_1_2 - 0.13504652176527*G0_2_0 - 0.270093043530541*G0_2_1 + 0.0929668084616003*G0_2_2;
    A[1781] = -0.185933616923198*G0_0_0 - 0.0812236326559237*G0_0_1 + 0.00913358118219296*G0_0_2 - 0.0812236326559226*G0_1_0 + 0.0234863516113529*G0_1_1 + 0.0459941052388968*G0_1_2 + 0.00913358118219295*G0_2_0 + 0.0459941052388979*G0_2_1 - 0.0880738185425682*G0_2_2;
    A[1782] = -0.0228339529554803*G0_0_0 - 0.0120693751336106*G0_0_1 - 0.0241387502672215*G0_0_2 - 0.0120693751336114*G0_1_0 - 0.0127217737894819*G0_1_2 - 0.0241387502672227*G0_2_0 - 0.0127217737894822*G0_2_1 - 0.0254435475789639*G0_2_2;
    A[1783] = -0.004892989919031*G0_0_0 + 0.0141896707651922*G0_0_1 + 0.0283793415303833*G0_0_2 + 0.0141896707651921*G0_1_0 + 0.028379341530383*G0_1_2 + 0.0283793415303837*G0_2_0 + 0.0283793415303838*G0_2_1 + 0.056758683060767*G0_2_2;
    A[1784] = -0.0228339529554812*G0_0_0 + 0.0368605240567044*G0_0_1 + 0.0737210481134101*G0_0_2 + 0.0368605240567041*G0_1_0 + 0.104709984267275*G0_1_2 + 0.0737210481134078*G0_2_0 + 0.104709984267274*G0_2_1 + 0.20941996853455*G0_2_2;
    A[1785] = 0.0195719596761255*G0_0_0 - 0.0102752788299673*G0_0_1 + 0.00146789697570878*G0_0_2 - 0.0102752788299661*G0_1_0 - 0.0181040627004161*G0_1_2 + 0.00146789697571032*G0_2_0 - 0.0181040627004161*G0_2_1 - 0.0166361657247063*G0_2_2;
    A[1786] = 0.0195719596761265*G0_0_0 + 0.0141896707651907*G0_0_1 + 0.0748627457611823*G0_0_2 + 0.014189670765191*G0_1_0 + 0.0283793415303826*G0_1_2 + 0.0748627457611822*G0_2_0 + 0.0283793415303824*G0_2_1 + 0.149725491522365*G0_2_2;
    A[1787] = 0.0260959462348354*G0_0_0 - 0.0120693751336101*G0_0_1 + 0.019898159004063*G0_0_2 - 0.0120693751336112*G0_1_0 - 0.0127217737894817*G0_1_2 + 0.019898159004061*G0_2_0 - 0.0127217737894831*G0_2_1 + 0.0137003717732881*G0_2_2;
    A[1788] = 0.146789697570944*G0_0_0 + 0.092966808461599*G0_0_1 + 0.141896707651913*G0_0_2 + 0.0929668084615986*G0_1_0 + 0.0929668084615976*G0_1_2 + 0.141896707651913*G0_2_0 + 0.0929668084615987*G0_2_1 + 0.146789697570943*G0_2_2;
    A[1789] = 0.146789697570951*G0_0_0 - 0.102752788299663*G0_0_1 - 0.298472385060924*G0_0_2 - 0.102752788299661*G0_1_0 - 0.298472385060923*G0_1_2 - 0.298472385060923*G0_2_0 - 0.298472385060925*G0_2_1 - 0.782878387045049*G0_2_2;
    A[1790] = -0.0489298991903166*G0_0_0 + 0.0929668084616015*G0_0_1 - 0.0293579395141877*G0_0_2 + 0.0929668084615985*G0_1_0 + 0.0929668084615994*G0_1_2 - 0.0293579395141909*G0_2_0 + 0.0929668084616014*G0_2_1 - 0.0489298991903164*G0_2_2;
    A[1791] = -0.782878387045047*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060925*G0_0_2 - 0.298472385060923*G0_1_0 - 0.102752788299662*G0_1_2 - 0.298472385060924*G0_2_0 - 0.102752788299664*G0_2_1 + 0.146789697570949*G0_2_2;
    A[1792] = -0.00946499969937456*G0_0_0 - 0.00946499969937462*G0_0_1 - 0.00946499969937457*G0_0_2 - 0.00946499969937464*G0_1_0 - 0.00946499969937461*G0_1_1 - 0.00946499969937462*G0_1_2 - 0.00946499969937439*G0_2_0 - 0.00946499969937443*G0_2_1 - 0.00946499969937441*G0_2_2;
    A[1793] = 0.00946499969937473*G0_0_0 - 0.00227817610630104*G0_1_0 - 0.00227817610630109*G0_2_0;
    A[1794] = -0.00472467106581679*G0_0_1 - 0.0345719095719093*G0_1_1 - 0.00472467106581681*G0_2_1;
    A[1795] = -0.00472467106581691*G0_0_2 - 0.00472467106581699*G0_1_2 - 0.0345719095719095*G0_2_2;
    A[1796] = -0.0493213383838389*G0_0_1 - 0.0429604514890976*G0_0_2 + 0.1273155976932*G0_1_1 - 0.000685018588665314*G0_1_2 - 0.00704590548340657*G0_2_1 - 0.0429604514890979*G0_2_2;
    A[1797] = 0.167731694424402*G0_0_1 + 0.0777006799142216*G0_0_2 - 0.166948816037356*G0_1_1 - 0.00616516729797883*G0_1_2 + 0.0838658472122009*G0_2_1 + 0.182410664181497*G0_2_2;
    A[1798] = 0.0777006799142201*G0_0_1 + 0.1677316944244*G0_0_2 + 0.182410664181495*G0_1_1 + 0.0838658472121985*G0_1_2 - 0.00616516729798021*G0_2_1 - 0.166948816037358*G0_2_2;
    A[1799] = -0.0429604514890968*G0_0_1 - 0.0493213383838377*G0_0_2 - 0.0429604514890969*G0_1_1 - 0.00704590548340474*G0_1_2 - 0.000685018588664756*G0_2_1 + 0.127315597693201*G0_2_2;
    A[1800] = -0.0292600797158093*G0_0_0 - 0.00329461321214824*G0_0_2 + 0.0142875305635717*G0_1_0 + 0.00254435475789633*G0_1_2 + 0.00844856259352777*G0_2_0 + 0.00254435475789655*G0_2_2;
    A[1801] = 0.0189848008858437*G0_0_0 + 0.00949240044292201*G0_0_2 - 0.0377738821749229*G0_1_0 - 0.00939454064454032*G0_1_2 - 0.0188869410874612*G0_2_0 - 0.0155923278753141*G0_2_2;
    A[1802] = -0.00352295274170347*G0_0_0 - 0.00649136662591575*G0_0_2 - 0.108232937008978*G0_1_0 - 0.0808321934624011*G0_1_2 - 0.0338921101724925*G0_2_0 - 0.0436454700777615*G0_2_2;
    A[1803] = -0.0429604514890966*G0_0_0 - 0.0422754329004323*G0_0_2 - 0.0429604514890967*G0_1_0 - 0.0845508658008649*G0_1_2 - 0.000685018588664582*G0_2_0 + 0.0439716694056966*G0_2_2;
    A[1804] = -0.0292600797158088*G0_0_0 - 0.00329461321214811*G0_0_1 + 0.00844856259352803*G0_1_0 + 0.00254435475789637*G0_1_1 + 0.0142875305635727*G0_2_0 + 0.00254435475789672*G0_2_1;
    A[1805] = 0.0189848008858418*G0_0_0 + 0.00949240044292122*G0_0_1 - 0.0188869410874629*G0_1_0 - 0.0155923278753141*G0_1_1 - 0.0377738821749247*G0_2_0 - 0.00939454064454114*G0_2_1;
    A[1806] = -0.00352295274170222*G0_0_0 - 0.00649136662591564*G0_0_1 - 0.0338921101724915*G0_1_0 - 0.0436454700777619*G0_1_1 - 0.108232937008977*G0_2_0 - 0.080832193462401*G0_2_1;
    A[1807] = -0.0429604514890975*G0_0_0 - 0.0422754329004326*G0_0_1 - 0.00068501858866524*G0_1_0 + 0.0439716694056964*G0_1_1 - 0.0429604514890978*G0_2_0 - 0.0845508658008661*G0_2_1;
    A[1808] = 0.0292600797158081*G0_0_0 + 0.0292600797158082*G0_0_1 + 0.0259654665036606*G0_0_2 + 0.029260079715808*G0_1_0 + 0.0292600797158081*G0_1_1 + 0.0259654665036607*G0_1_2 + 0.0259654665036596*G0_2_0 + 0.0259654665036597*G0_2_1 + 0.022670853291512*G0_2_2;
    A[1809] = -0.0189848008858418*G0_0_0 - 0.018984800885842*G0_0_1 - 0.00949240044292104*G0_0_2 - 0.0189848008858416*G0_1_0 - 0.0189848008858417*G0_1_1 - 0.00949240044292114*G0_1_2 - 0.00949240044292003*G0_2_0 - 0.00949240044292012*G0_2_1 - 0.000652398655869881*G0_2_2;
    A[1810] = 0.00352295274170182*G0_0_0 + 0.00352295274170183*G0_0_1 - 0.00296841388421277*G0_0_2 + 0.00352295274170143*G0_1_0 + 0.00352295274170157*G0_1_1 - 0.00296841388421264*G0_1_2 - 0.0029684138842137*G0_2_0 - 0.00296841388421375*G0_2_1 - 0.000652398655871279*G0_2_2;
    A[1811] = 0.0429604514890969*G0_0_0 + 0.0429604514890969*G0_0_1 + 0.000685018588664726*G0_0_2 + 0.0429604514890969*G0_1_0 + 0.0429604514890969*G0_1_1 + 0.00068501858866477*G0_1_2 + 0.000685018588664707*G0_2_0 + 0.000685018588664706*G0_2_1 + 0.0226708532915133*G0_2_2;
    A[1812] = 0.029260079715808*G0_0_0 + 0.0259654665036606*G0_0_1 + 0.029260079715808*G0_0_2 + 0.02596546650366*G0_1_0 + 0.0226708532915126*G0_1_1 + 0.0259654665036598*G0_1_2 + 0.0292600797158077*G0_2_0 + 0.0259654665036602*G0_2_1 + 0.0292600797158077*G0_2_2;
    A[1813] = -0.0189848008858416*G0_0_0 - 0.00949240044292077*G0_0_1 - 0.0189848008858415*G0_0_2 - 0.00949240044292019*G0_1_0 - 0.000652398655870366*G0_1_1 - 0.00949240044292009*G0_1_2 - 0.0189848008858415*G0_2_0 - 0.00949240044292063*G0_2_1 - 0.0189848008858415*G0_2_2;
    A[1814] = 0.00352295274170224*G0_0_0 - 0.00296841388421262*G0_0_1 + 0.00352295274170225*G0_0_2 - 0.00296841388421302*G0_1_0 - 0.000652398655870934*G0_1_1 - 0.00296841388421306*G0_1_2 + 0.00352295274170136*G0_2_0 - 0.00296841388421241*G0_2_1 + 0.00352295274170157*G0_2_2;
    A[1815] = 0.0429604514890974*G0_0_0 + 0.000685018588664225*G0_0_1 + 0.0429604514890973*G0_0_2 + 0.000685018588665242*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.000685018588665096*G0_1_2 + 0.0429604514890978*G0_2_0 + 0.00068501858866416*G0_2_1 + 0.0429604514890976*G0_2_2;
    A[1816] = 0.00329461321214738*G0_0_1 + 0.00329461321214741*G0_0_2 + 0.00329461321214797*G0_1_0 + 0.00658922642429505*G0_1_1 + 0.00658922642429507*G0_1_2 + 0.00329461321214799*G0_2_0 + 0.00658922642429544*G0_2_1 + 0.00658922642429542*G0_2_2;
    A[1817] = 0.00257697469068995*G0_0_1 + 0.00257697469068994*G0_0_2 + 0.00257697469068993*G0_1_0 + 0.0045015507255089*G0_1_1 + 0.00450155072550893*G0_1_2 + 0.00257697469068953*G0_2_0 + 0.00450155072550842*G0_2_1 + 0.00450155072550842*G0_2_2;
    A[1818] = 0.00257697469068978*G0_0_1 + 0.00257697469068971*G0_0_2 + 0.0025769746906902*G0_1_0 - 0.010503618359521*G0_1_1 - 0.0105036183595211*G0_1_2 + 0.00257697469069035*G0_2_0 - 0.0105036183595211*G0_2_1 - 0.0105036183595211*G0_2_2;
    A[1819] = 0.00329461321214813*G0_0_1 + 0.00329461321214813*G0_0_2 + 0.00329461321214765*G0_1_0 - 0.00254435475789656*G0_1_1 - 0.00254435475789646*G0_1_2 + 0.00329461321214774*G0_2_0 - 0.00254435475789657*G0_2_1 - 0.00254435475789659*G0_2_2;
    A[1820] = 0.113517366121533*G0_0_0 + 0.0127217737894824*G0_0_1 + 0.0127217737894822*G0_0_2 - 0.0851380245911492*G0_1_0 - 0.0137003717732886*G0_1_1 - 0.0198981590040623*G0_1_2 - 0.0851380245911501*G0_2_0 - 0.0198981590040619*G0_2_1 - 0.0137003717732882*G0_2_2;
    A[1821] = -0.149725491522366*G0_0_0 - 0.0748627457611828*G0_0_1 - 0.0283793415303843*G0_0_2 + 0.0278900425384801*G0_1_0 + 0.0655660649150229*G0_1_1 - 0.0748627457611836*G0_1_2 + 0.177126235068944*G0_2_0 + 0.0278900425384802*G0_2_1 - 0.149725491522366*G0_2_2;
    A[1822] = -0.0234863516113522*G0_0_0 + 0.0225077536275456*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0459941052388969*G0_1_0 - 0.268135847562928*G0_1_1 - 0.0737210481134093*G0_1_2 + 0.0812236326559224*G0_2_0 + 0.180714427676233*G0_2_1 - 0.209419968534551*G0_2_2;
    A[1823] = -0.149725491522366*G0_0_0 - 0.0283793415303823*G0_0_1 - 0.074862745761182*G0_0_2 + 0.177126235068943*G0_1_0 - 0.149725491522364*G0_1_1 + 0.0278900425384818*G0_1_2 + 0.0278900425384796*G0_2_0 - 0.0748627457611828*G0_2_1 + 0.0655660649150232*G0_2_2;
    A[1824] = 0.540186087061082*G0_0_0 + 0.270093043530539*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530539*G0_1_0 + 0.726119703984279*G0_1_1 + 0.270093043530541*G0_1_2 + 0.270093043530541*G0_2_0 + 0.270093043530541*G0_2_1 + 0.726119703984285*G0_2_2;
    A[1825] = -0.0234863516113519*G0_0_0 - 0.104709984267275*G0_0_1 + 0.022507753627544*G0_0_2 + 0.0812236326559226*G0_1_0 - 0.20941996853455*G0_1_1 + 0.18071442767623*G0_1_2 - 0.0459941052388971*G0_2_0 - 0.0737210481134092*G0_2_1 - 0.268135847562931*G0_2_2;
    A[1826] = -0.113517366121531*G0_0_0 - 0.100795592332049*G0_0_1 - 0.100795592332049*G0_0_2 - 0.100795592332049*G0_1_0 - 0.0880738185425663*G0_1_1 - 0.088726217198437*G0_1_2 - 0.100795592332048*G0_2_0 - 0.0887262171984364*G0_2_1 - 0.088073818542565*G0_2_2;
    A[1827] = 0.149725491522364*G0_0_0 + 0.0748627457611809*G0_0_1 + 0.12134614999198*G0_0_2 + 0.0748627457611816*G0_1_0 + 0.0195719596761242*G0_1_1 + 0.0606730749959897*G0_1_2 + 0.12134614999198*G0_2_0 + 0.0606730749959879*G0_2_1 + 0.0929668084615975*G0_2_2;
    A[1828] = 0.0234863516113517*G0_0_0 + 0.0459941052388988*G0_0_1 - 0.0812236326559229*G0_0_2 + 0.0459941052388964*G0_1_0 - 0.0880738185425658*G0_1_1 + 0.00913358118219185*G0_1_2 - 0.0812236326559229*G0_2_0 + 0.00913358118219538*G0_2_1 - 0.185933616923197*G0_2_2;
    A[1829] = 0.149725491522366*G0_0_0 + 0.121346149991983*G0_0_1 + 0.0748627457611824*G0_0_2 + 0.121346149991982*G0_1_0 + 0.0929668084615991*G0_1_1 + 0.06067307499599*G0_1_2 + 0.0748627457611829*G0_2_0 + 0.060673074995991*G0_2_1 + 0.0195719596761239*G0_2_2;
    A[1830] = -0.540186087061082*G0_0_0 - 0.270093043530543*G0_0_1 - 0.27009304353054*G0_0_2 - 0.270093043530539*G0_1_0 + 0.0929668084615985*G0_1_1 - 0.135046521765268*G0_1_2 - 0.270093043530541*G0_2_0 - 0.135046521765272*G0_2_1 + 0.0929668084616009*G0_2_2;
    A[1831] = 0.0234863516113515*G0_0_0 - 0.0812236326559223*G0_0_1 + 0.0459941052388961*G0_0_2 - 0.081223632655923*G0_1_0 - 0.185933616923197*G0_1_1 + 0.00913358118219196*G0_1_2 + 0.045994105238897*G0_2_0 + 0.0091335811821937*G0_2_1 - 0.0880738185425678*G0_2_2;
    A[1832] = -0.0127217737894825*G0_0_1 - 0.0120693751336106*G0_0_2 - 0.0127217737894815*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.0241387502672208*G0_1_2 - 0.0120693751336106*G0_2_0 - 0.0241387502672225*G0_2_1 - 0.0228339529554796*G0_2_2;
    A[1833] = -0.0181040627004168*G0_0_1 - 0.010275278829966*G0_0_2 - 0.0181040627004168*G0_1_0 - 0.0166361657247073*G0_1_1 + 0.00146789697570992*G0_1_2 - 0.0102752788299659*G0_2_0 + 0.00146789697570914*G0_2_1 + 0.0195719596761268*G0_2_2;
    A[1834] = -0.0127217737894826*G0_0_1 - 0.0120693751336115*G0_0_2 - 0.0127217737894815*G0_1_0 + 0.0137003717732888*G0_1_1 + 0.0198981590040616*G0_1_2 - 0.0120693751336109*G0_2_0 + 0.0198981590040618*G0_2_1 + 0.0260959462348355*G0_2_2;
    A[1835] = 0.0283793415303832*G0_0_1 + 0.0141896707651908*G0_0_2 + 0.0283793415303828*G0_1_0 + 0.0567586830607659*G0_1_1 + 0.0283793415303812*G0_1_2 + 0.014189670765191*G0_2_0 + 0.0283793415303829*G0_2_1 - 0.00489298991903276*G0_2_2;
    A[1836] = 0.0283793415303826*G0_0_1 + 0.0141896707651915*G0_0_2 + 0.0283793415303819*G0_1_0 + 0.149725491522364*G0_1_1 + 0.074862745761182*G0_1_2 + 0.0141896707651905*G0_2_0 + 0.0748627457611827*G0_2_1 + 0.0195719596761258*G0_2_2;
    A[1837] = 0.104709984267275*G0_0_1 + 0.0368605240567049*G0_0_2 + 0.104709984267276*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0737210481134098*G0_1_2 + 0.036860524056706*G0_2_0 + 0.0737210481134094*G0_2_1 - 0.0228339529554795*G0_2_2;
    A[1838] = -0.0120693751336104*G0_0_1 - 0.0127217737894825*G0_0_2 - 0.0120693751336106*G0_1_0 - 0.022833952955479*G0_1_1 - 0.0241387502672218*G0_1_2 - 0.0127217737894819*G0_2_0 - 0.0241387502672214*G0_2_1 - 0.0254435475789644*G0_2_2;
    A[1839] = -0.0102752788299666*G0_0_1 - 0.0181040627004174*G0_0_2 - 0.0102752788299667*G0_1_0 + 0.0195719596761255*G0_1_1 + 0.00146789697570865*G0_1_2 - 0.0181040627004162*G0_2_0 + 0.00146789697571044*G0_2_1 - 0.016636165724707*G0_2_2;
    A[1840] = -0.0120693751336109*G0_0_1 - 0.0127217737894818*G0_0_2 - 0.0120693751336109*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.0198981590040621*G0_1_2 - 0.0127217737894825*G0_2_0 + 0.0198981590040619*G0_2_1 + 0.0137003717732883*G0_2_2;
    A[1841] = 0.0141896707651907*G0_0_1 + 0.0283793415303828*G0_0_2 + 0.0141896707651903*G0_1_0 - 0.00489298991903308*G0_1_1 + 0.0283793415303817*G0_1_2 + 0.0283793415303826*G0_2_0 + 0.0283793415303809*G0_2_1 + 0.0567586830607646*G0_2_2;
    A[1842] = 0.014189670765192*G0_0_1 + 0.0283793415303837*G0_0_2 + 0.0141896707651926*G0_1_0 + 0.019571959676127*G0_1_1 + 0.0748627457611839*G0_1_2 + 0.0283793415303834*G0_2_0 + 0.0748627457611822*G0_2_1 + 0.149725491522366*G0_2_2;
    A[1843] = 0.0368605240567051*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0368605240567046*G0_1_0 - 0.022833952955479*G0_1_1 + 0.0737210481134093*G0_1_2 + 0.104709984267276*G0_2_0 + 0.0737210481134109*G0_2_1 + 0.209419968534551*G0_2_2;
    A[1844] = 0.0929668084616015*G0_0_1 + 0.0929668084616026*G0_0_2 + 0.0929668084615996*G0_1_0 + 0.146789697570947*G0_1_1 + 0.141896707651916*G0_1_2 + 0.0929668084615978*G0_2_0 + 0.141896707651915*G0_2_1 + 0.146789697570947*G0_2_2;
    A[1845] = 0.0929668084615998*G0_0_1 + 0.0929668084615991*G0_0_2 + 0.0929668084615984*G0_1_0 - 0.0489298991903166*G0_1_1 - 0.0293579395141909*G0_1_2 + 0.0929668084616003*G0_2_0 - 0.029357939514189*G0_2_1 - 0.0489298991903166*G0_2_2;
    A[1846] = -0.102752788299665*G0_0_1 - 0.298472385060924*G0_0_2 - 0.102752788299662*G0_1_0 + 0.146789697570946*G0_1_1 - 0.298472385060923*G0_1_2 - 0.298472385060924*G0_2_0 - 0.298472385060926*G0_2_1 - 0.782878387045049*G0_2_2;
    A[1847] = -0.298472385060923*G0_0_1 - 0.102752788299664*G0_0_2 - 0.298472385060925*G0_1_0 - 0.782878387045045*G0_1_1 - 0.298472385060924*G0_1_2 - 0.102752788299662*G0_2_0 - 0.298472385060924*G0_2_1 + 0.146789697570948*G0_2_2;
    A[1848] = -0.013742125287264*G0_0_0 - 0.0137421252872641*G0_0_1 - 0.0137421252872641*G0_0_2 - 0.013742125287264*G0_1_0 - 0.0137421252872641*G0_1_1 - 0.0137421252872641*G0_1_2 - 0.0137421252872642*G0_2_0 - 0.0137421252872642*G0_2_1 - 0.0137421252872642*G0_2_2;
    A[1849] = 0.0137421252872642*G0_0_0 - 0.00517743573299119*G0_1_0 - 0.00517743573299108*G0_2_0;
    A[1850] = -0.00517743573299123*G0_0_1 + 0.0137421252872641*G0_1_1 - 0.00517743573299131*G0_2_1;
    A[1851] = 0.0143945239431353*G0_0_2 + 0.0143945239431353*G0_1_2 + 0.0479930547204851*G0_2_2;
    A[1852] = 0.047755581609748*G0_0_1 + 0.0118736555368501*G0_0_2 - 0.0641307878721069*G0_1_1 - 0.00939454064454036*G0_1_2 + 0.0264873854283575*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[1853] = -0.0823327103709038*G0_0_1 + 0.00182671623643836*G0_0_2 + 0.127087258163646*G0_1_1 + 0.0377738821749232*G0_1_2 - 0.0463855444324188*G0_2_1 - 0.0510175748891016*G0_2_2;
    A[1854] = 0.146659217839771*G0_0_1 + 0.0109602974186308*G0_0_2 - 0.124738623002511*G0_1_1 - 0.0623693115012548*G0_1_2 + 0.0733296089198854*G0_2_1 + 0.132306447410612*G0_2_2;
    A[1855] = 0.124738623002511*G0_0_1 + 0.192588083213081*G0_0_2 + 0.124738623002511*G0_1_1 + 0.0962940416065405*G0_1_2 + 0.0284445813959707*G0_2_1 - 0.0644569872000421*G0_2_2;
    A[1856] = -0.0641307878721067*G0_0_0 - 0.00939454064454012*G0_0_2 + 0.0477555816097482*G0_1_0 + 0.0118736555368504*G0_1_2 + 0.0264873854283572*G0_2_0 + 0.0118736555368502*G0_2_2;
    A[1857] = 0.127087258163646*G0_0_0 + 0.0377738821749228*G0_0_2 - 0.0823327103709049*G0_1_0 + 0.00182671623643765*G0_1_2 - 0.0463855444324194*G0_2_0 - 0.0510175748891029*G0_2_2;
    A[1858] = -0.12473862300251*G0_0_0 - 0.0623693115012548*G0_0_2 + 0.146659217839772*G0_1_0 + 0.0109602974186307*G0_1_2 + 0.0733296089198866*G0_2_0 + 0.132306447410614*G0_2_2;
    A[1859] = 0.124738623002512*G0_0_0 + 0.0962940416065412*G0_0_2 + 0.124738623002512*G0_1_0 + 0.192588083213082*G0_1_2 + 0.0284445813959708*G0_2_0 - 0.0644569872000419*G0_2_2;
    A[1860] = -0.0187890812890809*G0_0_0 - 0.00939454064454017*G0_0_1 + 0.0212681961813904*G0_1_0 + 0.0118736555368502*G0_1_1 + 0.0425363923627807*G0_2_0 + 0.0118736555368501*G0_2_1;
    A[1861] = 0.0148746893538563*G0_0_0 + 0.0149399292194432*G0_0_1 + 0.00123955744615551*G0_1_0 + 0.00117431758056744*G0_1_1 + 0.0403182369328204*G0_2_0 + 0.0540186087061083*G0_2_1;
    A[1862] = 0.00117431758056752*G0_0_0 + 0.00123955744615452*G0_0_1 + 0.0149399292194427*G0_1_0 + 0.0148746893538565*G0_1_1 + 0.0540186087061081*G0_2_0 + 0.0403182369328197*G0_2_1;
    A[1863] = 0.0118736555368502*G0_0_0 + 0.0212681961813909*G0_0_1 - 0.00939454064454032*G0_1_0 - 0.0187890812890813*G0_1_1 + 0.0118736555368498*G0_2_0 + 0.0425363923627811*G0_2_1;
    A[1864] = 0.0641307878721071*G0_0_0 + 0.0641307878721072*G0_0_1 + 0.0547362472275665*G0_0_2 + 0.0641307878721068*G0_1_0 + 0.0641307878721069*G0_1_1 + 0.0547362472275664*G0_1_2 + 0.0547362472275664*G0_2_0 + 0.0547362472275665*G0_2_1 + 0.0453417065830263*G0_2_2;
    A[1865] = -0.127087258163646*G0_0_0 - 0.127087258163647*G0_0_1 - 0.0893133759887233*G0_0_2 - 0.127087258163646*G0_1_0 - 0.127087258163646*G0_1_1 - 0.089313375988723*G0_1_2 - 0.0893133759887227*G0_2_0 - 0.0893133759887228*G0_2_1 - 0.06654466289883*G0_2_2;
    A[1866] = 0.124738623002511*G0_0_0 + 0.124738623002511*G0_0_1 + 0.0623693115012562*G0_0_2 + 0.124738623002511*G0_1_0 + 0.124738623002511*G0_1_1 + 0.062369311501256*G0_1_2 + 0.0623693115012556*G0_2_0 + 0.0623693115012557*G0_2_1 + 0.0476251018785741*G0_2_2;
    A[1867] = -0.124738623002512*G0_0_0 - 0.124738623002512*G0_0_1 - 0.0284445813959709*G0_0_2 - 0.124738623002512*G0_1_0 - 0.124738623002512*G0_1_1 - 0.0284445813959708*G0_1_2 - 0.0284445813959707*G0_2_0 - 0.0284445813959707*G0_2_1 - 0.0606730749959917*G0_2_2;
    A[1868] = 0.0187890812890814*G0_0_0 + 0.00939454064454056*G0_0_1 + 0.0187890812890812*G0_0_2 + 0.00939454064454068*G0_1_0 + 0.00939454064454068*G0_1_2 + 0.018789081289081*G0_2_0 + 0.00939454064454074*G0_2_1 + 0.018789081289081*G0_2_2;
    A[1869] = -0.0148746893538561*G0_0_0 + 6.52398655872188e-05*G0_0_1 - 0.014874689353856*G0_0_2 + 6.52398655869532e-05*G0_1_0 + 6.52398655869431e-05*G0_1_2 - 0.0148746893538554*G0_2_0 + 6.52398655871549e-05*G0_2_1 - 0.0148746893538554*G0_2_2;
    A[1870] = -0.00117431758056727*G0_0_0 + 6.52398655867784e-05*G0_0_1 - 0.0011743175805673*G0_0_2 + 6.52398655868427e-05*G0_1_0 + 6.52398655869225e-05*G0_1_2 - 0.00117431758056753*G0_2_0 + 6.52398655867878e-05*G0_2_1 - 0.00117431758056759*G0_2_2;
    A[1871] = -0.0118736555368501*G0_0_0 + 0.00939454064454069*G0_0_1 - 0.0118736555368502*G0_0_2 + 0.00939454064454041*G0_1_0 + 0.00939454064454043*G0_1_2 - 0.0118736555368497*G0_2_0 + 0.0093945406445408*G0_2_1 - 0.0118736555368498*G0_2_2;
    A[1872] = 0.00939454064454076*G0_0_1 + 0.0093945406445407*G0_0_2 + 0.00939454064454073*G0_1_0 + 0.0187890812890813*G0_1_1 + 0.0187890812890813*G0_1_2 + 0.00939454064454063*G0_2_0 + 0.0187890812890809*G0_2_1 + 0.018789081289081*G0_2_2;
    A[1873] = 6.52398655867821e-05*G0_0_1 + 6.5239865586843e-05*G0_0_2 + 6.52398655868683e-05*G0_1_0 - 0.0148746893538563*G0_1_1 - 0.0148746893538563*G0_1_2 + 6.52398655871331e-05*G0_2_0 - 0.0148746893538559*G0_2_1 - 0.014874689353856*G0_2_2;
    A[1874] = 6.5239865587715e-05*G0_0_1 + 6.52398655877278e-05*G0_0_2 + 6.52398655871892e-05*G0_1_0 - 0.00117431758056701*G0_1_1 - 0.00117431758056695*G0_1_2 + 6.52398655871873e-05*G0_2_0 - 0.00117431758056714*G0_2_1 - 0.00117431758056707*G0_2_2;
    A[1875] = 0.00939454064454025*G0_0_1 + 0.00939454064454019*G0_0_2 + 0.00939454064454052*G0_1_0 - 0.0118736555368499*G0_1_1 - 0.01187365553685*G0_1_2 + 0.00939454064454036*G0_2_0 - 0.01187365553685*G0_2_1 - 0.01187365553685*G0_2_2;
    A[1876] = 0.105688582251081*G0_0_0 + 0.0528442911255404*G0_0_1 + 0.0300103381700607*G0_0_2 + 0.000652398655869746*G0_1_0 + 0.0534966897814109*G0_1_1 + 0.0528442911255404*G0_1_2 - 0.0743734467692805*G0_2_0 + 0.000652398655869839*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1877] = -0.013700371773288*G0_0_0 - 0.0198981590040608*G0_0_1 + 0.0127217737894828*G0_0_2 - 0.0198981590040614*G0_1_0 - 0.0137003717732881*G0_1_1 + 0.0127217737894834*G0_1_2 - 0.08513802459115*G0_2_0 - 0.0851380245911492*G0_2_1 + 0.113517366121532*G0_2_2;
    A[1878] = 0.0534966897814123*G0_0_0 + 0.000652398655871531*G0_0_1 + 0.0528442911255414*G0_0_2 + 0.0528442911255405*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0300103381700599*G0_1_2 + 0.000652398655871379*G0_2_0 - 0.0743734467692798*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1879] = -0.209419968534549*G0_0_0 - 0.104709984267275*G0_0_1 - 0.0737210481134085*G0_0_2 + 0.0812236326559237*G0_1_0 - 0.0234863516113528*G0_1_1 - 0.045994105238898*G0_1_2 + 0.180714427676232*G0_2_0 + 0.022507753627545*G0_2_1 - 0.26813584756293*G0_2_2;
    A[1880] = -0.0234863516113519*G0_0_0 + 0.0812236326559226*G0_0_1 - 0.0459941052388971*G0_0_2 - 0.104709984267275*G0_1_0 - 0.20941996853455*G0_1_1 - 0.0737210481134092*G0_1_2 + 0.022507753627544*G0_2_0 + 0.18071442767623*G0_2_1 - 0.268135847562931*G0_2_2;
    A[1881] = 0.542795681684566*G0_0_0 + 0.271397840842285*G0_0_1 + 0.271397840842284*G0_0_2 + 0.271397840842285*G0_1_0 + 0.542795681684568*G0_1_1 + 0.271397840842285*G0_1_2 + 0.271397840842284*G0_2_0 + 0.271397840842285*G0_2_1 + 0.647179466623908*G0_2_2;
    A[1882] = -0.105688582251082*G0_0_0 - 0.0528442911255409*G0_0_1 - 0.075678244081021*G0_0_2 - 0.0528442911255403*G0_1_0 - 0.03783912204051*G0_1_2 - 0.0756782440810213*G0_2_0 - 0.0378391220405105*G0_2_1 - 0.0456679059109612*G0_2_2;
    A[1883] = 0.0137003717732899*G0_0_0 - 0.00619778723077218*G0_0_1 + 0.026422145562771*G0_0_2 - 0.00619778723077198*G0_1_0 - 0.00554538857490139*G0_1_2 + 0.026422145562772*G0_2_0 - 0.00554538857490165*G0_2_1 + 0.0391439193522528*G0_2_2;
    A[1884] = -0.0534966897814125*G0_0_0 - 0.0528442911255418*G0_0_1 - 0.000652398655871522*G0_0_2 - 0.0528442911255405*G0_1_0 - 0.0378391220405104*G0_1_2 - 0.000652398655871519*G0_2_0 - 0.0378391220405116*G0_2_1 + 0.052191892469669*G0_2_2;
    A[1885] = 0.209419968534549*G0_0_0 + 0.104709984267275*G0_0_1 + 0.135698920421141*G0_0_2 + 0.104709984267274*G0_1_0 + 0.067849460210569*G0_1_2 + 0.135698920421141*G0_2_0 + 0.0678494602105703*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[1886] = 0.0234863516113521*G0_0_0 + 0.104709984267275*G0_0_1 - 0.0225077536275452*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105695*G0_1_2 - 0.022507753627544*G0_2_0 + 0.0678494602105709*G0_2_1 - 0.15657567740901*G0_2_2;
    A[1887] = -0.542795681684566*G0_0_0 - 0.271397840842283*G0_0_1 - 0.271397840842283*G0_0_2 - 0.271397840842285*G0_1_0 - 0.13569892042114*G0_1_2 - 0.271397840842284*G0_2_0 - 0.135698920421141*G0_2_1 + 0.0521918924696701*G0_2_2;
    A[1888] = -0.0528442911255411*G0_0_1 - 0.0378391220405106*G0_0_2 - 0.052844291125541*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0756782440810211*G0_1_2 - 0.0378391220405109*G0_2_0 - 0.0756782440810214*G0_2_1 - 0.0456679059109608*G0_2_2;
    A[1889] = -0.00619778723077337*G0_0_1 - 0.00554538857490308*G0_0_2 - 0.0061977872307718*G0_1_0 + 0.0137003717732881*G0_1_1 + 0.02642214556277*G0_1_2 - 0.00554538857490162*G0_2_0 + 0.0264221455627702*G0_2_1 + 0.0391439193522521*G0_2_2;
    A[1890] = -0.0528442911255405*G0_0_1 - 0.0378391220405106*G0_0_2 - 0.0528442911255419*G0_1_0 - 0.0534966897814111*G0_1_1 - 0.000652398655871134*G0_1_2 - 0.037839122040511*G0_2_0 - 0.000652398655869923*G0_2_1 + 0.05219189246967*G0_2_2;
    A[1891] = 0.104709984267275*G0_0_1 + 0.0678494602105713*G0_0_2 + 0.104709984267274*G0_1_0 + 0.209419968534549*G0_1_1 + 0.135698920421141*G0_1_2 + 0.0678494602105704*G0_2_0 + 0.135698920421141*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[1892] = 0.104709984267275*G0_0_1 + 0.0678494602105713*G0_0_2 + 0.104709984267277*G0_1_0 + 0.023486351611353*G0_1_1 - 0.0225077536275439*G0_1_2 + 0.0678494602105716*G0_2_0 - 0.0225077536275446*G0_2_1 - 0.15657567740901*G0_2_2;
    A[1893] = -0.271397840842285*G0_0_1 - 0.135698920421142*G0_0_2 - 0.271397840842284*G0_1_0 - 0.542795681684568*G0_1_1 - 0.271397840842284*G0_1_2 - 0.135698920421142*G0_2_0 - 0.271397840842285*G0_2_1 + 0.05219189246967*G0_2_2;
    A[1894] = -0.0150051690850306*G0_0_1 - 0.030010338170061*G0_0_2 - 0.0150051690850306*G0_1_0 - 0.0300103381700609*G0_1_2 - 0.0300103381700601*G0_2_0 - 0.0300103381700598*G0_2_1 - 0.0600206763401202*G0_2_2;
    A[1895] = -0.000652398655871225*G0_0_1 - 0.00130479731174162*G0_0_2 - 0.000652398655869933*G0_1_0 - 0.0127217737894813*G0_1_2 - 0.00130479731174186*G0_2_0 - 0.0127217737894829*G0_2_1 - 0.0254435475789651*G0_2_2;
    A[1896] = -0.0150051690850309*G0_0_1 - 0.0300103381700608*G0_0_2 - 0.0150051690850305*G0_1_0 - 0.0528442911255405*G0_1_2 - 0.0300103381700599*G0_2_0 - 0.0528442911255411*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1897] = -0.000652398655869731*G0_0_1 - 0.0127217737894806*G0_0_2 - 0.000652398655870198*G0_1_0 - 0.00130479731174036*G0_1_2 - 0.0127217737894828*G0_2_0 - 0.00130479731174149*G0_2_1 - 0.0254435475789634*G0_2_2;
    A[1898] = -0.000652398655870487*G0_0_1 - 0.0127217737894826*G0_0_2 - 0.000652398655872361*G0_1_0 - 0.0127217737894832*G0_1_2 - 0.0127217737894824*G0_2_0 - 0.0127217737894814*G0_2_1 - 0.113517366121532*G0_2_2;
    A[1899] = -0.0150051690850311*G0_0_1 - 0.0528442911255414*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0300103381700602*G0_1_2 - 0.0528442911255407*G0_2_0 - 0.0300103381700605*G0_2_1 - 0.105688582251081*G0_2_2;
    A[1900] = 0.0260959462348369*G0_0_1 + 0.0750258454251511*G0_0_2 + 0.026095946234834*G0_1_0 + 0.0750258454251501*G0_1_2 + 0.0750258454251509*G0_2_0 + 0.0750258454251526*G0_2_1 + 0.195719596761263*G0_2_2;
    A[1901] = 0.0260959462348341*G0_0_1 + 0.0750258454251505*G0_0_2 + 0.0260959462348368*G0_1_0 + 0.0587158790283798*G0_1_2 + 0.0750258454251511*G0_2_0 + 0.0587158790283787*G0_2_1 + 0.293579395141895*G0_2_2;
    A[1902] = 0.026095946234833*G0_0_1 + 0.0587158790283774*G0_0_2 + 0.0260959462348335*G0_1_0 + 0.0750258454251504*G0_1_2 + 0.0587158790283779*G0_2_0 + 0.0750258454251494*G0_2_1 + 0.293579395141894*G0_2_2;
    A[1903] = -0.185933616923197*G0_0_1 - 0.316413348097374*G0_0_2 - 0.185933616923198*G0_1_0 - 0.316413348097375*G0_1_2 - 0.316413348097373*G0_2_0 - 0.316413348097372*G0_2_1 - 0.782878387045051*G0_2_2;
    A[1904] = 0.0479930547204848*G0_0_0 + 0.0479930547204848*G0_0_1 + 0.0479930547204848*G0_0_2 + 0.03359853077735*G0_1_0 + 0.0335985307773498*G0_1_1 + 0.0335985307773499*G0_1_2 + 0.0335985307773497*G0_2_0 + 0.0335985307773497*G0_2_1 + 0.0335985307773497*G0_2_2;
    A[1905] = -0.0137421252872642*G0_0_0;
    A[1906] = 0.00517743573299135*G0_0_1 + 0.0189195610202555*G0_1_1;
    A[1907] = 0.00517743573299149*G0_0_2 + 0.0189195610202556*G0_2_2;
    A[1908] = -0.0425363923627812*G0_0_1 - 0.0118736555368502*G0_0_2 - 0.0613254736518625*G0_1_1 - 0.0212681961813909*G0_1_2 - 0.0212681961813911*G0_2_1;
    A[1909] = -0.0403182369328193*G0_0_1 - 0.0540186087061077*G0_0_2 - 0.0254435475789634*G0_1_1 - 0.0390786794866647*G0_1_2 - 0.0390786794866646*G0_2_1 - 0.0528442911255399*G0_2_2;
    A[1910] = -0.0540186087061088*G0_0_1 - 0.0403182369328203*G0_0_2 - 0.0528442911255414*G0_1_1 - 0.0390786794866656*G0_1_2 - 0.0390786794866656*G0_2_1 - 0.0254435475789647*G0_2_2;
    A[1911] = -0.0118736555368496*G0_0_1 - 0.0425363923627812*G0_0_2 - 0.0212681961813906*G0_1_2 - 0.02126819618139*G0_2_1 - 0.0613254736518617*G0_2_2;
    A[1912] = 0.018789081289081*G0_0_0 + 0.00939454064454033*G0_0_2 + 0.00939454064454054*G0_2_0;
    A[1913] = -0.0148746893538565*G0_0_0 - 0.0149399292194436*G0_0_2 - 0.0149399292194429*G0_2_0 - 0.0150051690850306*G0_2_2;
    A[1914] = -0.00117431758056734*G0_0_0 - 0.00123955744615433*G0_0_2 - 0.00123955744615454*G0_2_0 - 0.00130479731174097*G0_2_2;
    A[1915] = -0.0118736555368496*G0_0_0 - 0.0212681961813906*G0_0_2 - 0.02126819618139*G0_2_0 - 0.0306627368259309*G0_2_2;
    A[1916] = 0.0187890812890812*G0_0_0 + 0.00939454064454038*G0_0_1 + 0.00939454064454095*G0_1_0;
    A[1917] = -0.0148746893538561*G0_0_0 - 0.0149399292194428*G0_0_1 - 0.0149399292194435*G0_1_0 - 0.0150051690850302*G0_1_1;
    A[1918] = -0.0011743175805675*G0_0_0 - 0.00123955744615475*G0_0_1 - 0.00123955744615465*G0_1_0 - 0.00130479731174184*G0_1_1;
    A[1919] = -0.0118736555368504*G0_0_0 - 0.0212681961813909*G0_0_1 - 0.0212681961813909*G0_1_0 - 0.0306627368259315*G0_1_1;
    A[1920] = -0.0644569872000419*G0_0_0 - 0.0644569872000421*G0_0_1 - 0.0929015685960121*G0_0_2 - 0.257045070413124*G0_1_0 - 0.257045070413124*G0_1_1 - 0.160751028806583*G0_1_2 - 0.160751028806582*G0_2_0 - 0.160751028806582*G0_2_1 - 0.0644569872000418*G0_2_2;
    A[1921] = 0.132306447410613*G0_0_0 + 0.132306447410613*G0_0_1 + 0.0589768384907265*G0_0_2 + 0.121346149991982*G0_1_0 + 0.121346149991983*G0_1_1 + 0.194675758911869*G0_1_2 + 0.194675758911868*G0_2_0 + 0.194675758911868*G0_2_1 - 0.00339247301052929*G0_2_2;
    A[1922] = -0.0510175748891021*G0_0_0 - 0.0510175748891021*G0_0_1 - 0.00463203045668202*G0_0_2 - 0.0528442911255404*G0_1_0 - 0.0528442911255403*G0_1_1 - 0.0887914570640252*G0_1_2 - 0.0887914570640252*G0_2_0 - 0.0887914570640251*G0_2_1 + 0.0846813455320406*G0_2_2;
    A[1923] = 0.0118736555368498*G0_0_0 + 0.0118736555368497*G0_0_1 - 0.0146137298915084*G0_0_2 + 0.0212681961813899*G0_1_2 + 0.0212681961813901*G0_2_0 + 0.0212681961813901*G0_2_1 - 0.0693499771190746*G0_2_2;
    A[1924] = -0.0644569872000418*G0_0_0 - 0.0929015685960121*G0_0_1 - 0.0644569872000419*G0_0_2 - 0.160751028806583*G0_1_0 - 0.064456987200042*G0_1_1 - 0.160751028806583*G0_1_2 - 0.257045070413123*G0_2_0 - 0.160751028806583*G0_2_1 - 0.257045070413123*G0_2_2;
    A[1925] = 0.132306447410612*G0_0_0 + 0.0589768384907259*G0_0_1 + 0.132306447410612*G0_0_2 + 0.194675758911868*G0_1_0 - 0.0033924730105292*G0_1_1 + 0.194675758911868*G0_1_2 + 0.121346149991982*G0_2_0 + 0.194675758911868*G0_2_1 + 0.121346149991982*G0_2_2;
    A[1926] = -0.0510175748891023*G0_0_0 - 0.00463203045668203*G0_0_1 - 0.0510175748891021*G0_0_2 - 0.088791457064026*G0_1_0 + 0.08468134553204*G0_1_1 - 0.088791457064026*G0_1_2 - 0.0528442911255409*G0_2_0 - 0.0887914570640248*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[1927] = 0.0118736555368502*G0_0_0 - 0.014613729891508*G0_0_1 + 0.0118736555368501*G0_0_2 + 0.0212681961813908*G0_1_0 - 0.069349977119074*G0_1_1 + 0.0212681961813908*G0_1_2 + 0.02126819618139*G0_2_1;
    A[1928] = -0.0606730749959916*G0_0_0 - 0.0322284936000215*G0_0_1 - 0.0322284936000214*G0_0_2 - 0.0322284936000214*G0_1_0 - 0.128522535206562*G0_1_1 - 0.128522535206562*G0_1_2 - 0.0322284936000211*G0_2_0 - 0.128522535206562*G0_2_1 - 0.128522535206562*G0_2_2;
    A[1929] = 0.0476251018785742*G0_0_0 - 0.0147442096226814*G0_0_1 - 0.0147442096226814*G0_0_2 - 0.0147442096226814*G0_1_0 + 0.0476251018785736*G0_1_1 + 0.0476251018785736*G0_1_2 - 0.0147442096226816*G0_2_0 + 0.0476251018785731*G0_2_1 + 0.047625101878573*G0_2_2;
    A[1930] = -0.0665446628988292*G0_0_0 + 0.0227687130898926*G0_0_1 + 0.0227687130898927*G0_0_2 + 0.0227687130898931*G0_1_0 - 0.0150051690850305*G0_1_1 - 0.0150051690850305*G0_1_2 + 0.0227687130898935*G0_2_0 - 0.01500516908503*G0_2_1 - 0.01500516908503*G0_2_2;
    A[1931] = 0.0453417065830258*G0_0_0 - 0.00939454064454041*G0_0_1 - 0.00939454064454037*G0_0_2 - 0.00939454064454024*G0_1_0 - 0.00939454064454063*G0_2_0;
    A[1932] = -0.0600206763401205*G0_0_0 - 0.0300103381700607*G0_0_1 - 0.03001033817006*G0_0_2 - 0.0300103381700607*G0_1_0 - 0.0150051690850296*G0_1_2 - 0.0300103381700606*G0_2_0 - 0.0150051690850298*G0_2_1;
    A[1933] = -0.0254435475789635*G0_0_0 - 0.0241387502672219*G0_0_1 - 0.0127217737894818*G0_0_2 - 0.0241387502672223*G0_1_0 - 0.0228339529554816*G0_1_1 - 0.0120693751336125*G0_1_2 - 0.0127217737894811*G0_2_0 - 0.0120693751336118*G0_2_1;
    A[1934] = -0.105688582251082*G0_0_0 - 0.0756782440810215*G0_0_1 - 0.0528442911255416*G0_0_2 - 0.0756782440810214*G0_1_0 - 0.0456679059109607*G0_1_1 - 0.0378391220405111*G0_1_2 - 0.0528442911255413*G0_2_0 - 0.0378391220405101*G0_2_1;
    A[1935] = -0.0254435475789634*G0_0_0 - 0.0127217737894825*G0_0_1 - 0.0241387502672226*G0_0_2 - 0.0127217737894812*G0_1_0 - 0.0120693751336112*G0_1_2 - 0.0241387502672221*G0_2_0 - 0.0120693751336117*G0_2_1 - 0.0228339529554806*G0_2_2;
    A[1936] = -0.113517366121531*G0_0_0 - 0.100795592332049*G0_0_1 - 0.100795592332048*G0_0_2 - 0.100795592332049*G0_1_0 - 0.0880738185425663*G0_1_1 - 0.0887262171984364*G0_1_2 - 0.100795592332049*G0_2_0 - 0.0887262171984371*G0_2_1 - 0.088073818542565*G0_2_2;
    A[1937] = -0.105688582251082*G0_0_0 - 0.0528442911255403*G0_0_1 - 0.0756782440810213*G0_0_2 - 0.0528442911255409*G0_1_0 - 0.0378391220405105*G0_1_2 - 0.075678244081021*G0_2_0 - 0.03783912204051*G0_2_1 - 0.0456679059109612*G0_2_2;
    A[1938] = 0.647179466623908*G0_0_0 + 0.375781625781623*G0_0_1 + 0.375781625781623*G0_0_2 + 0.375781625781623*G0_1_0 + 0.647179466623906*G0_1_1 + 0.375781625781623*G0_1_2 + 0.375781625781623*G0_2_0 + 0.375781625781623*G0_2_1 + 0.647179466623906*G0_2_2;
    A[1939] = -0.268135847562929*G0_0_0 - 0.448850275239161*G0_0_1 - 0.290643601190473*G0_0_2 - 0.194414799449519*G0_1_0 - 0.584549195660302*G0_1_1 - 0.321632537344339*G0_1_2 - 0.222141742324033*G0_2_0 - 0.321632537344342*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1940] = 0.105688582251082*G0_0_0 + 0.180062029020361*G0_0_1 + 0.10503618359521*G0_0_2 + 0.0756782440810216*G0_1_0 + 0.255740273101382*G0_1_1 + 0.127870136550691*G0_1_2 + 0.0528442911255414*G0_2_0 + 0.127870136550692*G0_2_1 + 0.105688582251081*G0_2_2;
    A[1941] = -0.268135847562929*G0_0_0 - 0.290643601190475*G0_0_1 - 0.448850275239163*G0_0_2 - 0.222141742324033*G0_1_0 - 0.268135847562929*G0_1_1 - 0.321632537344342*G0_1_2 - 0.194414799449519*G0_2_0 - 0.32163253734434*G0_2_1 - 0.584549195660303*G0_2_2;
    A[1942] = 0.113517366121531*G0_0_0 + 0.198655390712681*G0_0_1 + 0.198655390712681*G0_0_2 + 0.100795592332049*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919137*G0_1_2 + 0.100795592332049*G0_2_0 + 0.166035457919137*G0_2_1 + 0.172233245149911*G0_2_2;
    A[1943] = 0.105688582251082*G0_0_0 + 0.105036183595211*G0_0_1 + 0.180062029020363*G0_0_2 + 0.052844291125541*G0_1_0 + 0.105688582251082*G0_1_1 + 0.127870136550693*G0_1_2 + 0.0756782440810212*G0_2_0 + 0.127870136550691*G0_2_1 + 0.255740273101383*G0_2_2;
    A[1944] = 0.0521918924696704*G0_0_0 + 0.323589733311955*G0_0_1 + 0.187890812890813*G0_0_2 + 0.323589733311952*G0_1_0 + 0.0521918924696698*G0_1_1 + 0.187890812890811*G0_1_2 + 0.18789081289081*G0_2_0 + 0.187890812890811*G0_2_1 + 0.323589733311952*G0_2_2;
    A[1945] = 0.0391439193522505*G0_0_0 - 0.09655500106889*G0_0_1 - 0.0287055408583187*G0_0_2 - 0.096555001068889*G0_1_0 - 0.0228339529554807*G0_1_1 - 0.0596944770121847*G0_1_2 - 0.0287055408583182*G0_2_0 - 0.0596944770121841*G0_2_1 - 0.096555001068888*G0_2_2;
    A[1946] = -0.0456679059109603*G0_0_0 + 0.0300103381700608*G0_0_1 - 0.00782878387044928*G0_0_2 + 0.0300103381700599*G0_1_0 + 0.0150051690850309*G0_1_2 - 0.00782878387045052*G0_2_0 + 0.0150051690850303*G0_2_1 + 0.0300103381700607*G0_2_2;
    A[1947] = -0.15657567740901*G0_0_0 - 0.134067923781465*G0_0_1 - 0.224425137619582*G0_0_2 - 0.134067923781465*G0_1_0 - 0.0880738185425681*G0_1_1 - 0.0972073997247605*G0_1_2 - 0.224425137619579*G0_2_0 - 0.0972073997247583*G0_2_1 - 0.292274597830151*G0_2_2;
    A[1948] = 0.0391439193522529*G0_0_0 + 0.0127217737894827*G0_0_1 + 0.0446893079271543*G0_0_2 + 0.0127217737894825*G0_1_0 + 0.0120693751336106*G0_1_2 + 0.0446893079271537*G0_2_0 + 0.0120693751336113*G0_2_1 + 0.0502346965020557*G0_2_2;
    A[1949] = 0.0521918924696697*G0_0_0 + 0.0528442911255403*G0_0_1 + 0.0900310145101813*G0_0_2 + 0.0528442911255405*G0_1_0 + 0.0378391220405111*G0_1_2 + 0.0900310145101798*G0_2_0 + 0.0378391220405099*G0_2_1 + 0.127870136550691*G0_2_2;
    A[1950] = 0.0521918924696704*G0_0_0 + 0.187890812890812*G0_0_1 + 0.323589733311954*G0_0_2 + 0.187890812890811*G0_1_0 + 0.323589733311953*G0_1_1 + 0.187890812890811*G0_1_2 + 0.323589733311952*G0_2_0 + 0.187890812890811*G0_2_1 + 0.0521918924696698*G0_2_2;
    A[1951] = 0.0391439193522512*G0_0_0 - 0.0287055408583178*G0_0_1 - 0.0965550010688891*G0_0_2 - 0.0287055408583185*G0_1_0 - 0.0965550010688883*G0_1_1 - 0.0596944770121844*G0_1_2 - 0.0965550010688887*G0_2_0 - 0.0596944770121837*G0_2_1 - 0.0228339529554799*G0_2_2;
    A[1952] = -0.0456679059109602*G0_0_0 - 0.00782878387044984*G0_0_1 + 0.0300103381700602*G0_0_2 - 0.00782878387045056*G0_1_0 + 0.0300103381700604*G0_1_1 + 0.0150051690850295*G0_1_2 + 0.0300103381700596*G0_2_0 + 0.0150051690850306*G0_2_1;
    A[1953] = -0.156575677409008*G0_0_0 - 0.224425137619581*G0_0_1 - 0.134067923781465*G0_0_2 - 0.224425137619578*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0972073997247594*G0_1_2 - 0.134067923781463*G0_2_0 - 0.0972073997247611*G0_2_1 - 0.0880738185425688*G0_2_2;
    A[1954] = 0.0391439193522521*G0_0_0 + 0.0446893079271543*G0_0_1 + 0.0127217737894821*G0_0_2 + 0.0446893079271543*G0_1_0 + 0.0502346965020567*G0_1_1 + 0.0120693751336122*G0_1_2 + 0.0127217737894825*G0_2_0 + 0.0120693751336103*G0_2_1;
    A[1955] = 0.0521918924696701*G0_0_0 + 0.0900310145101812*G0_0_1 + 0.0528442911255413*G0_0_2 + 0.0900310145101808*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0378391220405111*G0_1_2 + 0.0528442911255407*G0_2_0 + 0.0378391220405123*G0_2_1;
    A[1956] = -0.782878387045049*G0_0_0 - 0.466465038947676*G0_0_1 - 0.466465038947675*G0_0_2 - 0.466465038947674*G0_1_0 - 0.1500516908503*G0_1_1 - 0.335985307773499*G0_1_2 - 0.466465038947673*G0_2_0 - 0.335985307773499*G0_2_1 - 0.150051690850299*G0_2_2;
    A[1957] = 0.195719596761262*G0_0_0 + 0.120693751336112*G0_0_1 + 0.120693751336112*G0_0_2 + 0.120693751336111*G0_1_0 + 0.0456679059109619*G0_1_1 + 0.0717638521457963*G0_1_2 + 0.12069375133611*G0_2_0 + 0.0717638521457951*G0_2_1 + 0.0456679059109609*G0_2_2;
    A[1958] = 0.293579395141893*G0_0_0 + 0.218553549716743*G0_0_1 + 0.234863516113514*G0_0_2 + 0.218553549716742*G0_1_0 + 0.143527704291591*G0_1_1 + 0.185933616923196*G0_1_2 + 0.234863516113514*G0_2_0 + 0.185933616923198*G0_2_1 + 0.176147637085134*G0_2_2;
    A[1959] = 0.293579395141894*G0_0_0 + 0.234863516113514*G0_0_1 + 0.218553549716742*G0_0_2 + 0.234863516113515*G0_1_0 + 0.176147637085134*G0_1_1 + 0.185933616923198*G0_1_2 + 0.218553549716742*G0_2_0 + 0.185933616923195*G0_2_1 + 0.14352770429159*G0_2_2;
    A[1960] = -0.0345719095719092*G0_0_0 - 0.0345719095719092*G0_0_1 - 0.0345719095719093*G0_0_2 - 0.0298472385060924*G0_1_0 - 0.0298472385060922*G0_1_1 - 0.0298472385060924*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.0298472385060921*G0_2_2;
    A[1961] = -0.00946499969937442*G0_0_0;
    A[1962] = 0.00472467106581701*G0_0_1 - 0.0298472385060925*G0_1_1;
    A[1963] = 0.00227817610630118*G0_0_2 + 0.0117431758056758*G0_2_2;
    A[1964] = 0.0845508658008653*G0_0_1 + 0.0429604514890974*G0_0_2 + 0.128522535206562*G0_1_1 + 0.042275432900433*G0_1_2 + 0.0422754329004328*G0_2_1;
    A[1965] = 0.0808321934624005*G0_0_1 + 0.108232937008977*G0_0_2 + 0.0371867233846391*G0_1_1 + 0.0743408268364848*G0_1_2 + 0.0743408268364853*G0_2_1 + 0.104709984267274*G0_2_2;
    A[1966] = 0.00939454064454097*G0_0_1 + 0.037773882174924*G0_0_2 - 0.00619778723077289*G0_1_1 + 0.0188869410874622*G0_1_2 + 0.0188869410874614*G0_2_1 + 0.0567586830607657*G0_2_2;
    A[1967] = -0.00254435475789622*G0_0_1 - 0.014287530563572*G0_0_2 - 0.00583896797004407*G0_1_2 - 0.00583896797004381*G0_2_1 - 0.0435476102793804*G0_2_2;
    A[1968] = 0.00658922642429617*G0_0_0 + 0.00329461321214866*G0_0_2 + 0.00329461321214836*G0_2_0;
    A[1969] = 0.00450155072550963*G0_0_0 + 0.00192457603481895*G0_0_2 + 0.00192457603481905*G0_2_0 - 0.000652398655870764*G0_2_2;
    A[1970] = -0.0105036183595212*G0_0_0 - 0.0130805930502105*G0_0_2 - 0.0130805930502112*G0_2_0 - 0.0156575677409011*G0_2_2;
    A[1971] = -0.002544354757896*G0_0_0 - 0.00583896797004419*G0_0_2 - 0.00583896797004374*G0_2_0 - 0.00913358118219163*G0_2_2;
    A[1972] = 0.0292600797158093*G0_0_0 + 0.00329461321214863*G0_0_1 + 0.00329461321214781*G0_1_0;
    A[1973] = -0.0189848008858433*G0_0_0 - 0.00949240044292165*G0_0_1 - 0.00949240044292085*G0_1_0 - 0.000652398655871232*G0_1_1;
    A[1974] = 0.0035229527417031*G0_0_0 + 0.00649136662591528*G0_0_1 + 0.00649136662591531*G0_1_0 + 0.00880738185425735*G0_1_1;
    A[1975] = 0.0429604514890975*G0_0_0 + 0.042275432900433*G0_0_1 + 0.0422754329004331*G0_1_0 + 0.0642612676032812*G0_1_1;
    A[1976] = 0.0439716694056969*G0_0_0 + 0.0439716694056968*G0_0_1 + 0.0446566879943614*G0_0_2 + 0.128522535206562*G0_1_0 + 0.128522535206562*G0_1_1 + 0.0862471023061297*G0_1_2 + 0.0862471023061292*G0_2_0 + 0.0862471023061293*G0_2_1 + 0.0439716694056968*G0_2_2;
    A[1977] = -0.0436454700777624*G0_0_0 - 0.0436454700777624*G0_0_1 - 0.00975335990526966*G0_0_2 + 0.0371867233846385*G0_1_0 + 0.0371867233846384*G0_1_1 - 0.0371541034518468*G0_1_2 - 0.0371541034518467*G0_2_0 - 0.0371541034518469*G0_2_1 - 0.00678494602105738*G0_2_2;
    A[1978] = -0.0155923278753126*G0_0_0 - 0.0155923278753127*G0_0_1 + 0.00329461321214777*G0_0_2 - 0.00619778723077224*G0_1_0 - 0.00619778723077228*G0_1_1 - 0.0250847283182352*G0_1_2 - 0.0250847283182339*G0_2_0 - 0.0250847283182339*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[1979] = 0.00254435475789613*G0_0_0 + 0.0025443547578961*G0_0_1 - 0.00590420783563137*G0_0_2 + 0.0058389679700447*G0_1_2 + 0.0058389679700438*G0_2_0 + 0.00583896797004379*G0_2_1 - 0.0318696743392923*G0_2_2;
    A[1980] = 0.127315597693201*G0_0_0 + 0.128000616281865*G0_0_1 + 0.127315597693201*G0_0_2 + 0.134361503176606*G0_1_0 + 0.0920860702761734*G0_1_1 + 0.134361503176606*G0_1_2 + 0.176636936077038*G0_2_0 + 0.134361503176606*G0_2_1 + 0.176636936077038*G0_2_2;
    A[1981] = -0.166948816037356*G0_0_0 - 0.160783648739376*G0_0_1 - 0.166948816037356*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700806*G0_1_1 - 0.250814663249556*G0_1_2 - 0.334680510461756*G0_2_0 - 0.250814663249556*G0_2_1 - 0.334680510461756*G0_2_2;
    A[1982] = 0.182410664181496*G0_0_0 + 0.0985448169692952*G0_0_1 + 0.182410664181496*G0_0_2 + 0.188575831479476*G0_1_0 - 0.0622388317700819*G0_1_1 + 0.188575831479476*G0_1_2 + 0.104709984267275*G0_2_0 + 0.188575831479475*G0_2_1 + 0.104709984267275*G0_2_2;
    A[1983] = -0.0429604514890973*G0_0_0 - 0.0359145460056917*G0_0_1 - 0.0429604514890974*G0_0_2 - 0.0422754329004329*G0_1_0 + 0.0920860702761739*G0_1_1 - 0.0422754329004329*G0_1_2 - 0.0422754329004323*G0_2_1;
    A[1984] = 0.0226708532915132*G0_0_0 + 0.0219858347028487*G0_0_1 + 0.0219858347028488*G0_0_2 + 0.0219858347028489*G0_1_0 + 0.0642612676032815*G0_1_1 + 0.0642612676032815*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0642612676032811*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[1985] = -0.0006523986558711*G0_0_0 + 0.00231601522834134*G0_0_1 + 0.00231601522834135*G0_0_2 + 0.00231601522834125*G0_1_0 + 0.00880738185425615*G0_1_1 + 0.00880738185425624*G0_1_2 + 0.0023160152283414*G0_2_0 + 0.00880738185425631*G0_2_1 + 0.00880738185425641*G0_2_2;
    A[1986] = -0.00065239865587068*G0_0_0 + 0.00884000178705088*G0_0_1 + 0.00884000178705088*G0_0_2 + 0.0088400017870509*G0_1_0 - 0.000652398655869585*G0_1_1 - 0.000652398655869506*G0_1_2 + 0.00884000178705037*G0_2_0 - 0.00065239865587021*G0_2_1 - 0.00065239865587025*G0_2_2;
    A[1987] = 0.0226708532915127*G0_0_0 - 0.00329461321214877*G0_0_1 - 0.00329461321214873*G0_0_2 - 0.00329461321214874*G0_1_0 - 0.00329461321214777*G0_2_0;
    A[1988] = -0.0254435475789636*G0_0_0 - 0.0013047973117403*G0_0_1 - 0.0127217737894811*G0_0_2 - 0.00130479731174067*G0_1_0 - 0.000652398655870766*G0_1_2 - 0.0127217737894833*G0_2_0 - 0.000652398655871205*G0_2_1;
    A[1989] = 0.0567586830607655*G0_0_0 + 0.0283793415303819*G0_0_1 + 0.0283793415303817*G0_0_2 + 0.0283793415303831*G0_1_0 - 0.00489298991903111*G0_1_1 + 0.0141896707651927*G0_1_2 + 0.0283793415303842*G0_2_0 + 0.0141896707651918*G0_2_1;
    A[1990] = 0.20941996853455*G0_0_0 + 0.135698920421142*G0_0_1 + 0.104709984267277*G0_0_2 + 0.135698920421141*G0_1_0 + 0.039143919352252*G0_1_1 + 0.0678494602105714*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0678494602105711*G0_2_1;
    A[1991] = -0.0166361657247066*G0_0_0 - 0.0181040627004158*G0_0_1 + 0.00146789697571017*G0_0_2 - 0.0181040627004171*G0_1_0 - 0.0102752788299655*G0_1_2 + 0.00146789697570981*G0_2_0 - 0.0102752788299662*G0_2_1 + 0.0195719596761261*G0_2_2;
    A[1992] = 0.149725491522364*G0_0_0 + 0.0748627457611816*G0_0_1 + 0.12134614999198*G0_0_2 + 0.0748627457611809*G0_1_0 + 0.0195719596761242*G0_1_1 + 0.0606730749959879*G0_1_2 + 0.12134614999198*G0_2_0 + 0.0606730749959897*G0_2_1 + 0.0929668084615975*G0_2_2;
    A[1993] = 0.0137003717732899*G0_0_0 - 0.00619778723077198*G0_0_1 + 0.026422145562772*G0_0_2 - 0.00619778723077218*G0_1_0 - 0.00554538857490165*G0_1_2 + 0.026422145562771*G0_2_0 - 0.00554538857490139*G0_2_1 + 0.0391439193522528*G0_2_2;
    A[1994] = -0.268135847562929*G0_0_0 - 0.194414799449519*G0_0_1 - 0.222141742324033*G0_0_2 - 0.448850275239161*G0_1_0 - 0.584549195660302*G0_1_1 - 0.321632537344342*G0_1_2 - 0.290643601190473*G0_2_0 - 0.321632537344339*G0_2_1 - 0.268135847562928*G0_2_2;
    A[1995] = 0.72611970398428*G0_0_0 + 0.456026660453738*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453738*G0_1_0 + 0.912053320907479*G0_1_1 + 0.456026660453738*G0_1_2 + 0.456026660453738*G0_2_0 + 0.456026660453738*G0_2_1 + 0.726119703984278*G0_2_2;
    A[1996] = -0.20941996853455*G0_0_0 - 0.390134396210781*G0_0_1 - 0.290643601190472*G0_0_2 - 0.135698920421141*G0_1_0 - 0.584549195660301*G0_1_1 - 0.262916658315961*G0_1_2 - 0.104709984267275*G0_2_0 - 0.262916658315961*G0_2_1 - 0.209419968534549*G0_2_2;
    A[1997] = 0.0655660649150264*G0_0_0 + 0.140428810676207*G0_0_1 + 0.037676022376546*G0_0_2 + 0.037676022376544*G0_1_0 - 0.0371867233846386*G0_1_1 + 0.186912214907009*G0_1_2 + 0.140428810676206*G0_2_0 + 0.186912214907005*G0_2_1 - 0.0371867233846378*G0_2_2;
    A[1998] = -0.149725491522364*G0_0_0 - 0.177615534060845*G0_0_1 - 0.326851726591307*G0_0_2 - 0.0748627457611809*G0_1_0 - 0.0371867233846391*G0_1_1 - 0.224098938291644*G0_1_2 - 0.12134614999198*G0_2_0 - 0.224098938291644*G0_2_1 - 0.448197876583288*G0_2_2;
    A[1999] = -0.0137003717732901*G0_0_0 + 0.00619778723077134*G0_0_1 + 0.071437652817858*G0_0_2 + 0.00619778723077201*G0_1_0 + 0.0123955744615447*G0_1_1 + 0.00619778723077054*G0_1_2 - 0.0264221455627712*G0_2_0 + 0.00619778723077203*G0_2_1 + 0.172233245149908*G0_2_2;
    A[2000] = -0.0228339529554804*G0_0_0 - 0.0965550010688903*G0_0_1 - 0.0596944770121856*G0_0_2 - 0.0965550010688897*G0_1_0 + 0.0391439193522512*G0_1_1 - 0.0287055408583198*G0_1_2 - 0.0596944770121849*G0_2_0 - 0.0287055408583188*G0_2_1 - 0.0965550010688895*G0_2_2;
    A[2001] = -0.00489298991903157*G0_0_0 - 0.0332723314494139*G0_0_1 - 0.0190826606842232*G0_0_2 - 0.033272331449414*G0_1_0 - 0.00489298991903011*G0_1_1 - 0.0190826606842228*G0_1_2 - 0.0190826606842223*G0_2_0 - 0.0190826606842226*G0_2_1 - 0.033272331449414*G0_2_2;
    A[2002] = -0.0228339529554811*G0_0_0 + 0.00130479731174022*G0_0_1 - 0.0107645778218694*G0_0_2 + 0.00130479731174167*G0_1_0 + 0.000652398655869667*G0_1_2 - 0.0107645778218704*G0_2_0 + 0.000652398655870835*G0_2_1 + 0.00130479731174149*G0_2_2;
    A[2003] = 0.0195719596761272*G0_0_0 - 0.0552907860850537*G0_0_1 + 0.00538228891093656*G0_0_2 - 0.0552907860850546*G0_1_0 + 0.0195719596761281*G0_1_1 - 0.0411011153198625*G0_1_2 + 0.00538228891093516*G0_2_0 - 0.0411011153198637*G0_2_1 - 0.00880738185425571*G0_2_2;
    A[2004] = 0.0195719596761259*G0_0_0 + 0.0181040627004156*G0_0_1 + 0.0298472385060921*G0_0_2 + 0.0181040627004155*G0_1_0 + 0.0102752788299665*G0_1_2 + 0.0298472385060927*G0_2_0 + 0.0102752788299662*G0_2_1 + 0.0401225173360587*G0_2_2;
    A[2005] = 0.0260959462348332*G0_0_0 + 0.00619778723077231*G0_0_1 + 0.0381653213684446*G0_0_2 + 0.00619778723077199*G0_1_0 + 0.00554538857490025*G0_1_2 + 0.0381653213684447*G0_2_0 + 0.00554538857490163*G0_2_1 + 0.0502346965020557*G0_2_2;
    A[2006] = -0.0880738185425681*G0_0_0 - 0.0972073997247598*G0_0_1 - 0.134067923781465*G0_0_2 - 0.0972073997247604*G0_1_0 - 0.292274597830151*G0_1_1 - 0.22442513761958*G0_1_2 - 0.134067923781464*G0_2_0 - 0.224425137619579*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2007] = 0.019571959676127*G0_0_0 - 0.0411011153198652*G0_0_1 - 0.0552907860850556*G0_0_2 - 0.0411011153198644*G0_1_0 - 0.00880738185425687*G0_1_1 + 0.00538228891093587*G0_1_2 - 0.0552907860850556*G0_2_0 + 0.00538228891093459*G0_2_1 + 0.019571959676126*G0_2_2;
    A[2008] = -0.088073818542569*G0_0_0 + 0.000652398655871005*G0_0_1 + 0.0127217737894813*G0_0_2 + 0.000652398655870485*G0_1_0 + 0.00130479731174124*G0_1_1 + 0.000652398655870905*G0_1_2 + 0.0127217737894808*G0_2_0 + 0.0006523986558706*G0_2_1;
    A[2009] = 0.0929668084615976*G0_0_0 + 0.228013330226869*G0_0_1 + 0.363059851992139*G0_0_2 + 0.228013330226868*G0_1_0 + 0.456026660453739*G0_1_1 + 0.228013330226869*G0_1_2 + 0.363059851992138*G0_2_0 + 0.228013330226869*G0_2_1 + 0.0929668084615987*G0_2_2;
    A[2010] = 0.092966808461601*G0_0_0 + 0.0322937334656082*G0_0_1 - 0.0283793415303821*G0_0_2 + 0.0322937334656085*G0_1_0 - 0.00880738185425689*G0_1_1 - 0.0141896707651927*G0_1_2 - 0.0283793415303812*G0_2_0 - 0.014189670765191*G0_2_1;
    A[2011] = -0.185933616923199*G0_0_0 - 0.19506719810539*G0_0_1 - 0.104709984267276*G0_0_2 - 0.195067198105391*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105712*G0_1_2 - 0.104709984267276*G0_2_0 - 0.0678494602105709*G0_2_1;
    A[2012] = 0.146789697570948*G0_0_0 + 0.445262082631872*G0_0_1 + 0.249542485870612*G0_0_2 + 0.445262082631873*G0_1_0 - 0.0391439193522517*G0_1_1 + 0.249542485870609*G0_1_2 + 0.249542485870607*G0_2_0 + 0.249542485870609*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2013] = 0.146789697570944*G0_0_0 + 0.00489298991903187*G0_0_1 + 0.0538228891093447*G0_0_2 + 0.00489298991902859*G0_1_0 + 0.00978597983806059*G0_1_1 + 0.00489298991902907*G0_1_2 + 0.0538228891093486*G0_2_0 + 0.00489298991903136*G0_2_1 - 0.0391439193522526*G0_2_2;
    A[2014] = -0.782878387045046*G0_0_0 - 0.484406001984124*G0_0_1 - 0.484406001984119*G0_0_2 - 0.484406001984121*G0_1_0 - 0.039143919352252*G0_1_1 - 0.288686405222858*G0_1_2 - 0.484406001984123*G0_2_0 - 0.288686405222862*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2015] = -0.0489298991903186*G0_0_0 - 0.0195719596761277*G0_0_1 - 0.141896707651919*G0_0_2 - 0.0195719596761269*G0_1_0 - 0.0391439193522516*G0_1_1 - 0.0195719596761266*G0_1_2 - 0.141896707651916*G0_2_0 - 0.0195719596761258*G0_2_1 - 0.234863516113514*G0_2_2;
    A[2016] = 0.0137421252872639*G0_0_0 + 0.013742125287264*G0_0_1 + 0.013742125287264*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.0189195610202552*G0_2_0 + 0.0189195610202552*G0_2_1 + 0.0189195610202552*G0_2_2;
    A[2017] = -0.0137421252872644*G0_0_0;
    A[2018] = -0.0143945239431351*G0_0_1 + 0.03359853077735*G0_1_1;
    A[2019] = 0.00517743573299122*G0_0_2 + 0.0189195610202552*G0_2_2;
    A[2020] = -0.192588083213082*G0_0_1 - 0.124738623002512*G0_0_2 - 0.257045070413124*G0_1_1 - 0.0962940416065413*G0_1_2 - 0.096294041606541*G0_2_1;
    A[2021] = -0.0109602974186296*G0_0_1 - 0.146659217839771*G0_0_2 + 0.121346149991983*G0_1_1 - 0.0733296089198852*G0_1_2 - 0.0733296089198853*G0_2_1 - 0.271397840842282*G0_2_2;
    A[2022] = -0.0018267162364396*G0_0_1 + 0.0823327103709031*G0_0_2 - 0.052844291125541*G0_1_1 + 0.0359471659384848*G0_1_2 + 0.0359471659384845*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2023] = -0.0118736555368494*G0_0_1 - 0.0477555816097476*G0_0_2 - 0.0212681961813906*G0_1_2 - 0.0212681961813902*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2024] = 0.0187890812890813*G0_0_0 + 0.00939454064454021*G0_0_2 + 0.00939454064454058*G0_2_0;
    A[2025] = -0.0148746893538561*G0_0_0 - 0.0149399292194423*G0_0_2 - 0.0149399292194431*G0_2_0 - 0.0150051690850296*G0_2_2;
    A[2026] = -0.00117431758056764*G0_0_0 - 0.00123955744615534*G0_0_2 - 0.0012395574461543*G0_2_0 - 0.00130479731174204*G0_2_2;
    A[2027] = -0.0118736555368494*G0_0_0 - 0.0212681961813898*G0_0_2 - 0.0212681961813904*G0_2_0 - 0.0306627368259304*G0_2_2;
    A[2028] = 0.064130787872107*G0_0_0 + 0.00939454064454034*G0_0_1 + 0.00939454064454065*G0_1_0;
    A[2029] = -0.127087258163647*G0_0_0 - 0.0377738821749238*G0_0_1 - 0.0377738821749238*G0_1_0 - 0.0150051690850298*G0_1_1;
    A[2030] = 0.124738623002512*G0_0_0 + 0.0623693115012556*G0_0_1 + 0.0623693115012554*G0_1_0 + 0.0476251018785733*G0_1_1;
    A[2031] = -0.124738623002512*G0_0_0 - 0.0962940416065415*G0_0_1 - 0.0962940416065414*G0_1_0 - 0.128522535206562*G0_1_1;
    A[2032] = -0.0187890812890812*G0_0_0 - 0.0187890812890813*G0_0_1 - 0.00939454064454071*G0_0_2 - 0.0613254736518626*G0_1_0 - 0.0613254736518626*G0_1_1 - 0.040057277470472*G0_1_2 - 0.0400572774704717*G0_2_0 - 0.0400572774704717*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[2033] = 0.0148746893538565*G0_0_0 + 0.0148746893538565*G0_0_1 - 6.52398655869277e-05*G0_0_2 - 0.0254435475789627*G0_1_0 - 0.0254435475789627*G0_1_1 + 0.0136351319077019*G0_1_2 + 0.0136351319077013*G0_2_0 + 0.0136351319077014*G0_2_1 - 0.000130479731173961*G0_2_2;
    A[2034] = 0.00117431758056731*G0_0_0 + 0.00117431758056745*G0_0_1 - 6.52398655870654e-05*G0_0_2 - 0.0528442911255419*G0_1_0 - 0.0528442911255419*G0_1_1 - 0.0137656116388758*G0_1_2 - 0.0137656116388756*G0_2_0 - 0.0137656116388755*G0_2_1 - 0.000130479731174389*G0_2_2;
    A[2035] = 0.0118736555368495*G0_0_0 + 0.0118736555368495*G0_0_1 - 0.0093945406445407*G0_0_2 + 0.0212681961813906*G0_1_2 + 0.0212681961813904*G0_2_0 + 0.0212681961813903*G0_2_1 - 0.018789081289081*G0_2_2;
    A[2036] = -0.0641307878721069*G0_0_0 - 0.0547362472275661*G0_0_1 - 0.0641307878721069*G0_0_2 - 0.0906181733004645*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.0906181733004646*G0_1_2 - 0.111886369481855*G0_2_0 - 0.0906181733004641*G0_2_1 - 0.111886369481855*G0_2_2;
    A[2037] = 0.127087258163646*G0_0_0 + 0.0893133759887223*G0_0_1 + 0.127087258163646*G0_0_2 + 0.173472802596065*G0_1_0 + 0.084681345532039*G0_1_1 + 0.173472802596065*G0_1_2 + 0.20941996853455*G0_2_0 + 0.173472802596065*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2038] = -0.124738623002511*G0_0_0 - 0.0623693115012555*G0_0_1 - 0.124738623002511*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052813*G0_1_1 - 0.198068231922397*G0_1_2 - 0.271397840842283*G0_2_0 - 0.198068231922397*G0_2_1 - 0.271397840842283*G0_2_2;
    A[2039] = 0.124738623002512*G0_0_0 + 0.0284445813959705*G0_0_1 + 0.124738623002512*G0_0_2 + 0.0962940416065413*G0_1_0 - 0.0644569872000423*G0_1_1 + 0.0962940416065413*G0_1_2 + 0.096294041606541*G0_2_1;
    A[2040] = -0.00939454064454082*G0_0_1 - 0.00939454064454078*G0_0_2 - 0.00939454064454084*G0_1_0 - 0.0306627368259315*G0_1_1 - 0.0306627368259316*G0_1_2 - 0.00939454064454074*G0_2_0 - 0.0306627368259312*G0_2_1 - 0.0306627368259312*G0_2_2;
    A[2041] = -6.52398655868898e-05*G0_0_1 - 6.52398655867697e-05*G0_0_2 - 6.52398655869568e-05*G0_1_0 - 0.00130479731174121*G0_1_1 - 0.00130479731174123*G0_1_2 - 6.52398655870089e-05*G0_2_0 - 0.00130479731174136*G0_2_1 - 0.00130479731174137*G0_2_2;
    A[2042] = -6.52398655874103e-05*G0_0_1 - 6.52398655874148e-05*G0_0_2 - 6.5239865587532e-05*G0_1_0 - 0.0150051690850309*G0_1_1 - 0.015005169085031*G0_1_2 - 6.52398655871509e-05*G0_2_0 - 0.0150051690850304*G0_2_1 - 0.0150051690850304*G0_2_2;
    A[2043] = -0.00939454064454024*G0_0_1 - 0.00939454064454039*G0_0_2 - 0.0093945406445398*G0_1_0 - 0.00939454064454037*G0_2_0;
    A[2044] = -0.105688582251082*G0_0_0 - 0.0300103381700598*G0_0_1 - 0.0528442911255411*G0_0_2 - 0.0300103381700611*G0_1_0 - 0.0150051690850303*G0_1_2 - 0.0528442911255404*G0_2_0 - 0.0150051690850299*G0_2_1;
    A[2045] = 0.209419968534549*G0_0_0 + 0.0737210481134072*G0_0_1 + 0.104709984267274*G0_0_2 + 0.0737210481134081*G0_1_0 - 0.0228339529554811*G0_1_1 + 0.0368605240567031*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0368605240567036*G0_2_1;
    A[2046] = -0.542795681684567*G0_0_0 - 0.271397840842284*G0_0_1 - 0.271397840842284*G0_0_2 - 0.271397840842283*G0_1_0 + 0.0521918924696699*G0_1_1 - 0.135698920421142*G0_1_2 - 0.271397840842282*G0_2_0 - 0.13569892042114*G0_2_1;
    A[2047] = 0.0137003717732897*G0_0_0 - 0.0127217737894812*G0_0_1 + 0.0198981590040626*G0_0_2 - 0.0127217737894823*G0_1_0 - 0.0120693751336123*G0_1_2 + 0.0198981590040616*G0_2_0 - 0.0120693751336109*G0_2_1 + 0.0260959462348349*G0_2_2;
    A[2048] = 0.0234863516113518*G0_0_0 + 0.0459941052388964*G0_0_1 - 0.0812236326559229*G0_0_2 + 0.0459941052388988*G0_1_0 - 0.0880738185425658*G0_1_1 + 0.00913358118219538*G0_1_2 - 0.081223632655923*G0_2_0 + 0.00913358118219185*G0_2_1 - 0.185933616923197*G0_2_2;
    A[2049] = -0.0534966897814126*G0_0_0 - 0.0528442911255405*G0_0_1 - 0.000652398655871524*G0_0_2 - 0.0528442911255418*G0_1_0 - 0.0378391220405116*G0_1_2 - 0.000652398655871522*G0_2_0 - 0.0378391220405104*G0_2_1 + 0.052191892469669*G0_2_2;
    A[2050] = 0.105688582251082*G0_0_0 + 0.0756782440810216*G0_0_1 + 0.0528442911255414*G0_0_2 + 0.180062029020361*G0_1_0 + 0.255740273101382*G0_1_1 + 0.127870136550692*G0_1_2 + 0.10503618359521*G0_2_0 + 0.127870136550691*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2051] = -0.20941996853455*G0_0_0 - 0.135698920421141*G0_0_1 - 0.104709984267275*G0_0_2 - 0.390134396210781*G0_1_0 - 0.584549195660301*G0_1_1 - 0.262916658315961*G0_1_2 - 0.290643601190472*G0_2_0 - 0.262916658315961*G0_2_1 - 0.209419968534549*G0_2_2;
    A[2052] = 0.542795681684567*G0_0_0 + 0.271397840842283*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842283*G0_1_0 + 0.647179466623906*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684564*G0_2_2;
    A[2053] = -0.0137003717732897*G0_0_0 - 0.026422145562772*G0_0_1 + 0.00619778723077187*G0_0_2 + 0.0714376528178598*G0_1_0 + 0.17223324514991*G0_1_1 + 0.00619778723077103*G0_1_2 + 0.00619778723077219*G0_2_0 + 0.00619778723077255*G0_2_1 + 0.012395574461546*G0_2_2;
    A[2054] = -0.023486351611352*G0_0_0 + 0.0225077536275451*G0_0_1 - 0.104709984267274*G0_0_2 - 0.0459941052388989*G0_1_0 - 0.26813584756293*G0_1_1 + 0.0534966897814112*G0_1_2 + 0.0812236326559229*G0_2_0 + 0.0534966897814114*G0_2_1 - 0.20941996853455*G0_2_2;
    A[2055] = 0.0534966897814126*G0_0_0 + 0.000652398655872309*G0_0_1 + 0.0528442911255415*G0_0_2 + 0.0528442911255418*G0_1_0 + 0.105688582251083*G0_1_1 - 0.022181554299608*G0_1_2 + 0.000652398655871613*G0_2_0 - 0.022181554299609*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2056] = 0.0300103381700612*G0_0_1 + 0.015005169085031*G0_0_2 + 0.0300103381700606*G0_1_0 - 0.0456679059109606*G0_1_1 - 0.00782878387044949*G0_1_2 + 0.0150051690850303*G0_2_0 - 0.00782878387045017*G0_2_1 + 0.0300103381700602*G0_2_2;
    A[2057] = 0.00130479731174127*G0_0_1 + 0.00065239865587059*G0_0_2 + 0.00130479731174171*G0_1_0 - 0.0228339529554814*G0_1_1 - 0.0107645778218696*G0_1_2 + 0.000652398655870629*G0_2_0 - 0.0107645778218697*G0_2_1 + 0.00130479731174155*G0_2_2;
    A[2058] = 0.0300103381700598*G0_0_1 + 0.0150051690850297*G0_0_2 + 0.03001033817006*G0_1_0 + 0.0150051690850304*G0_1_2 + 0.01500516908503*G0_2_0 + 0.01500516908503*G0_2_1 + 0.0300103381700598*G0_2_2;
    A[2059] = 0.0127217737894808*G0_0_1 + 0.000652398655869278*G0_0_2 + 0.0127217737894804*G0_1_0 - 0.0880738185425696*G0_1_1 + 0.000652398655869033*G0_1_2 + 0.00065239865587028*G0_2_0 + 0.000652398655869985*G0_2_1 + 0.00130479731174133*G0_2_2;
    A[2060] = 0.0127217737894812*G0_0_1 + 0.000652398655871334*G0_0_2 + 0.0127217737894827*G0_1_0 + 0.0120693751336115*G0_1_2 + 0.000652398655871328*G0_2_0 + 0.0120693751336108*G0_2_1 + 0.00130479731174179*G0_2_2;
    A[2061] = 0.0528442911255402*G0_0_1 + 0.0150051690850304*G0_0_2 + 0.0528442911255424*G0_1_0 + 0.0378391220405122*G0_1_2 + 0.01500516908503*G0_2_0 + 0.0378391220405102*G0_2_1 + 0.0300103381700602*G0_2_2;
    A[2062] = 0.037839122040511*G0_0_1 + 0.0528442911255412*G0_0_2 + 0.0378391220405108*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0900310145101799*G0_1_2 + 0.0528442911255411*G0_2_0 + 0.0900310145101808*G0_2_1 + 0.0521918924696706*G0_2_2;
    A[2063] = 0.00554538857490293*G0_0_1 + 0.00619778723077343*G0_0_2 + 0.0055453885749026*G0_1_0 + 0.0502346965020573*G0_1_1 + 0.0381653213684452*G0_1_2 + 0.00619778723077254*G0_2_0 + 0.0381653213684455*G0_2_1 + 0.0260959462348341*G0_2_2;
    A[2064] = 0.037839122040511*G0_0_1 + 0.0528442911255411*G0_0_2 + 0.0378391220405104*G0_1_0 + 0.0300103381700602*G0_1_1 + 0.0150051690850303*G0_1_2 + 0.0528442911255408*G0_2_0 + 0.0150051690850304*G0_2_1;
    A[2065] = -0.0678494602105705*G0_0_1 - 0.104709984267275*G0_0_2 - 0.06784946021057*G0_1_0 - 0.29227459783015*G0_1_1 - 0.19506719810539*G0_1_2 - 0.104709984267275*G0_2_0 - 0.195067198105391*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2066] = -0.0678494602105715*G0_0_1 - 0.104709984267274*G0_0_2 - 0.0678494602105708*G0_1_0 - 0.0965550010688891*G0_1_1 - 0.0368605240567033*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0368605240567041*G0_2_1;
    A[2067] = 0.135698920421142*G0_0_1 + 0.271397840842284*G0_0_2 + 0.135698920421142*G0_1_0 + 0.323589733311952*G0_1_1 + 0.135698920421142*G0_1_2 + 0.271397840842283*G0_2_0 + 0.135698920421142*G0_2_1;
    A[2068] = -0.0750258454251521*G0_0_1 - 0.0260959462348366*G0_0_2 - 0.0750258454251523*G0_1_0 + 0.143527704291592*G0_1_1 - 0.0424059126316063*G0_1_2 - 0.0260959462348362*G0_2_0 - 0.0424059126316078*G0_2_1 - 0.0521918924696719*G0_2_2;
    A[2069] = -0.0750258454251492*G0_0_1 - 0.0260959462348362*G0_0_2 - 0.0750258454251474*G0_1_0 + 0.0456679059109637*G0_1_1 - 0.0260959462348326*G0_1_2 - 0.0260959462348332*G0_2_0 - 0.0260959462348343*G0_2_1 - 0.0521918924696686*G0_2_2;
    A[2070] = 0.316413348097375*G0_0_1 + 0.185933616923199*G0_0_2 + 0.316413348097373*G0_1_0 - 0.150051690850301*G0_1_1 + 0.185933616923196*G0_1_2 + 0.185933616923198*G0_2_0 + 0.185933616923199*G0_2_1 + 0.371867233846396*G0_2_2;
    A[2071] = -0.0587158790283774*G0_0_1 - 0.0260959462348336*G0_0_2 - 0.0587158790283781*G0_1_0 + 0.176147637085135*G0_1_1 - 0.00978597983806235*G0_1_2 - 0.0260959462348342*G0_2_0 - 0.00978597983806219*G0_2_1 - 0.0521918924696678*G0_2_2;
    A[2072] = -0.0345719095719095*G0_0_0 - 0.0345719095719096*G0_0_1 - 0.0345719095719095*G0_0_2 - 0.0298472385060924*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0298472385060924*G0_1_2 - 0.0298472385060925*G0_2_0 - 0.0298472385060925*G0_2_1 - 0.0298472385060925*G0_2_2;
    A[2073] = -0.00946499969937479*G0_0_0;
    A[2074] = 0.00227817610630118*G0_0_1 + 0.0117431758056756*G0_1_1;
    A[2075] = 0.00472467106581666*G0_0_2 - 0.0298472385060929*G0_2_2;
    A[2076] = -0.0142875305635717*G0_0_1 - 0.00254435475789591*G0_0_2 - 0.04354761027938*G0_1_1 - 0.00583896797004392*G0_1_2 - 0.00583896797004339*G0_2_1;
    A[2077] = 0.0377738821749236*G0_0_1 + 0.00939454064454081*G0_0_2 + 0.0567586830607655*G0_1_1 + 0.0188869410874621*G0_1_2 + 0.0188869410874611*G0_2_1 - 0.00619778723077384*G0_2_2;
    A[2078] = 0.108232937008978*G0_0_1 + 0.0808321934624012*G0_0_2 + 0.104709984267276*G0_1_1 + 0.0743408268364859*G0_1_2 + 0.0743408268364871*G0_2_1 + 0.0371867233846409*G0_2_2;
    A[2079] = 0.0429604514890963*G0_0_1 + 0.0845508658008651*G0_0_2 + 0.0422754329004328*G0_1_2 + 0.0422754329004314*G0_2_1 + 0.128522535206561*G0_2_2;
    A[2080] = 0.0292600797158087*G0_0_0 + 0.00329461321214757*G0_0_2 + 0.00329461321214758*G0_2_0;
    A[2081] = -0.0189848008858428*G0_0_0 - 0.00949240044292101*G0_0_2 - 0.0094924004429213*G0_2_0 - 0.00065239865587008*G0_2_2;
    A[2082] = 0.00352295274170245*G0_0_0 + 0.00649136662591444*G0_0_2 + 0.00649136662591509*G0_2_0 + 0.00880738185425568*G0_2_2;
    A[2083] = 0.0429604514890963*G0_0_0 + 0.0422754329004323*G0_0_2 + 0.0422754329004314*G0_2_0 + 0.0642612676032806*G0_2_2;
    A[2084] = 0.00658922642429559*G0_0_0 + 0.00329461321214757*G0_0_1 + 0.00329461321214776*G0_1_0;
    A[2085] = 0.00450155072550948*G0_0_0 + 0.00192457603481969*G0_0_1 + 0.00192457603481942*G0_1_0 - 0.000652398655869752*G0_1_1;
    A[2086] = -0.0105036183595215*G0_0_0 - 0.0130805930502117*G0_0_1 - 0.0130805930502111*G0_1_0 - 0.015657567740902*G0_1_1;
    A[2087] = -0.00254435475789594*G0_0_0 - 0.00583896797004379*G0_0_1 - 0.00583896797004387*G0_1_0 - 0.00913358118219116*G0_1_1;
    A[2088] = 0.127315597693201*G0_0_0 + 0.127315597693201*G0_0_1 + 0.128000616281865*G0_0_2 + 0.176636936077038*G0_1_0 + 0.176636936077038*G0_1_1 + 0.134361503176606*G0_1_2 + 0.134361503176606*G0_2_0 + 0.134361503176606*G0_2_1 + 0.0920860702761734*G0_2_2;
    A[2089] = -0.166948816037355*G0_0_0 - 0.166948816037356*G0_0_1 - 0.160783648739377*G0_0_2 - 0.334680510461757*G0_1_0 - 0.334680510461757*G0_1_1 - 0.250814663249556*G0_1_2 - 0.250814663249556*G0_2_0 - 0.250814663249556*G0_2_1 - 0.0622388317700804*G0_2_2;
    A[2090] = 0.182410664181495*G0_0_0 + 0.182410664181495*G0_0_1 + 0.0985448169692949*G0_0_2 + 0.104709984267274*G0_1_0 + 0.104709984267274*G0_1_1 + 0.188575831479474*G0_1_2 + 0.188575831479474*G0_2_0 + 0.188575831479474*G0_2_1 - 0.062238831770083*G0_2_2;
    A[2091] = -0.0429604514890963*G0_0_0 - 0.0429604514890963*G0_0_1 - 0.0359145460056912*G0_0_2 - 0.0422754329004318*G0_1_2 - 0.0422754329004316*G0_2_0 - 0.0422754329004315*G0_2_1 + 0.0920860702761748*G0_2_2;
    A[2092] = 0.0439716694056968*G0_0_0 + 0.0446566879943618*G0_0_1 + 0.0439716694056967*G0_0_2 + 0.0862471023061293*G0_1_0 + 0.043971669405697*G0_1_1 + 0.0862471023061293*G0_1_2 + 0.128522535206562*G0_2_0 + 0.0862471023061298*G0_2_1 + 0.128522535206562*G0_2_2;
    A[2093] = -0.0436454700777615*G0_0_0 - 0.00975335990526962*G0_0_1 - 0.0436454700777615*G0_0_2 - 0.0371541034518464*G0_1_0 - 0.00678494602105676*G0_1_1 - 0.0371541034518463*G0_1_2 + 0.0371867233846389*G0_2_0 - 0.0371541034518463*G0_2_1 + 0.037186723384639*G0_2_2;
    A[2094] = -0.0155923278753131*G0_0_0 + 0.00329461321214784*G0_0_1 - 0.0155923278753131*G0_0_2 - 0.0250847283182345*G0_1_0 + 0.0127870136550686*G0_1_1 - 0.0250847283182346*G0_1_2 - 0.00619778723077262*G0_2_0 - 0.0250847283182355*G0_2_1 - 0.00619778723077274*G0_2_2;
    A[2095] = 0.00254435475789601*G0_0_0 - 0.00590420783563172*G0_0_1 + 0.00254435475789593*G0_0_2 + 0.00583896797004394*G0_1_0 - 0.0318696743392919*G0_1_1 + 0.00583896797004394*G0_1_2 + 0.00583896797004461*G0_2_1;
    A[2096] = 0.0226708532915132*G0_0_0 + 0.0219858347028482*G0_0_1 + 0.0219858347028481*G0_0_2 + 0.0219858347028485*G0_1_0 + 0.0642612676032809*G0_1_1 + 0.0642612676032808*G0_1_2 + 0.0219858347028485*G0_2_0 + 0.0642612676032807*G0_2_1 + 0.0642612676032806*G0_2_2;
    A[2097] = -0.000652398655871076*G0_0_0 + 0.0023160152283417*G0_0_1 + 0.00231601522834178*G0_0_2 + 0.00231601522834143*G0_1_0 + 0.00880738185425699*G0_1_1 + 0.00880738185425707*G0_1_2 + 0.00231601522834166*G0_2_0 + 0.00880738185425777*G0_2_1 + 0.00880738185425779*G0_2_2;
    A[2098] = -0.00065239865587102*G0_0_0 + 0.00884000178705047*G0_0_1 + 0.00884000178705047*G0_0_2 + 0.00884000178705007*G0_1_0 - 0.000652398655871045*G0_1_1 - 0.000652398655871108*G0_1_2 + 0.00884000178705023*G0_2_0 - 0.000652398655871152*G0_2_1 - 0.000652398655871191*G0_2_2;
    A[2099] = 0.0226708532915132*G0_0_0 - 0.00329461321214758*G0_0_1 - 0.00329461321214762*G0_0_2 - 0.00329461321214734*G0_1_0 - 0.00329461321214785*G0_2_0;
    A[2100] = -0.0254435475789645*G0_0_0 - 0.012721773789483*G0_0_1 - 0.00130479731174225*G0_0_2 - 0.0127217737894822*G0_1_0 - 0.000652398655870723*G0_1_2 - 0.00130479731174083*G0_2_0 - 0.000652398655871282*G0_2_1;
    A[2101] = -0.0166361657247074*G0_0_0 + 0.00146789697571027*G0_0_1 - 0.0181040627004164*G0_0_2 + 0.00146789697570903*G0_1_0 + 0.0195719596761268*G0_1_1 - 0.0102752788299665*G0_1_2 - 0.0181040627004182*G0_2_0 - 0.0102752788299657*G0_2_1;
    A[2102] = 0.0137003717732895*G0_0_0 + 0.0264221455627712*G0_0_1 - 0.00619778723077276*G0_0_2 + 0.0264221455627718*G0_1_0 + 0.039143919352253*G0_1_1 - 0.00554538857490091*G0_1_2 - 0.00619778723077217*G0_2_0 - 0.00554538857490269*G0_2_1;
    A[2103] = 0.0567586830607654*G0_0_0 + 0.0283793415303822*G0_0_1 + 0.0283793415303826*G0_0_2 + 0.0283793415303825*G0_1_0 + 0.0141896707651908*G0_1_2 + 0.0283793415303823*G0_2_0 + 0.0141896707651919*G0_2_1 - 0.00489298991903125*G0_2_2;
    A[2104] = 0.149725491522366*G0_0_0 + 0.121346149991982*G0_0_1 + 0.0748627457611829*G0_0_2 + 0.121346149991983*G0_1_0 + 0.0929668084615991*G0_1_1 + 0.060673074995991*G0_1_2 + 0.0748627457611824*G0_2_0 + 0.06067307499599*G0_2_1 + 0.0195719596761239*G0_2_2;
    A[2105] = 0.209419968534549*G0_0_0 + 0.104709984267274*G0_0_1 + 0.135698920421141*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105703*G0_1_2 + 0.135698920421141*G0_2_0 + 0.067849460210569*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[2106] = -0.268135847562929*G0_0_0 - 0.222141742324033*G0_0_1 - 0.194414799449519*G0_0_2 - 0.290643601190475*G0_1_0 - 0.268135847562929*G0_1_1 - 0.32163253734434*G0_1_2 - 0.448850275239163*G0_2_0 - 0.321632537344342*G0_2_1 - 0.584549195660303*G0_2_2;
    A[2107] = 0.0655660649150264*G0_0_0 + 0.037676022376544*G0_0_1 + 0.140428810676206*G0_0_2 + 0.140428810676207*G0_1_0 - 0.0371867233846386*G0_1_1 + 0.186912214907005*G0_1_2 + 0.037676022376546*G0_2_0 + 0.186912214907009*G0_2_1 - 0.0371867233846377*G0_2_2;
    A[2108] = -0.0137003717732897*G0_0_0 + 0.0714376528178598*G0_0_1 + 0.0061977872307722*G0_0_2 - 0.026422145562772*G0_1_0 + 0.17223324514991*G0_1_1 + 0.00619778723077256*G0_1_2 + 0.00619778723077187*G0_2_0 + 0.00619778723077104*G0_2_1 + 0.012395574461546*G0_2_2;
    A[2109] = 0.726119703984282*G0_0_0 + 0.456026660453741*G0_0_1 + 0.45602666045374*G0_0_2 + 0.456026660453741*G0_1_0 + 0.726119703984282*G0_1_1 + 0.456026660453741*G0_1_2 + 0.45602666045374*G0_2_0 + 0.456026660453741*G0_2_1 + 0.91205332090748*G0_2_2;
    A[2110] = -0.149725491522366*G0_0_0 - 0.32685172659131*G0_0_1 - 0.177615534060846*G0_0_2 - 0.121346149991983*G0_1_0 - 0.448197876583291*G0_1_1 - 0.224098938291645*G0_1_2 - 0.0748627457611825*G0_2_0 - 0.224098938291646*G0_2_1 - 0.0371867233846417*G0_2_2;
    A[2111] = -0.209419968534549*G0_0_0 - 0.290643601190473*G0_0_1 - 0.390134396210782*G0_0_2 - 0.104709984267275*G0_1_0 - 0.20941996853455*G0_1_1 - 0.262916658315963*G0_1_2 - 0.135698920421141*G0_2_0 - 0.262916658315961*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2112] = -0.0880738185425677*G0_0_0 - 0.134067923781464*G0_0_1 - 0.0972073997247593*G0_0_2 - 0.134067923781464*G0_1_0 - 0.15657567740901*G0_1_1 - 0.22442513761958*G0_1_2 - 0.097207399724759*G0_2_0 - 0.22442513761958*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2113] = 0.0195719596761274*G0_0_0 - 0.0552907860850566*G0_0_1 - 0.0411011153198652*G0_0_2 - 0.0552907860850558*G0_1_0 + 0.0195719596761258*G0_1_1 + 0.00538228891093444*G0_1_2 - 0.0411011153198651*G0_2_0 + 0.00538228891093436*G0_2_1 - 0.00880738185425804*G0_2_2;
    A[2114] = -0.0880738185425687*G0_0_0 + 0.0127217737894827*G0_0_1 + 0.000652398655870718*G0_0_2 + 0.0127217737894815*G0_1_0 + 0.000652398655870814*G0_1_2 + 0.000652398655871115*G0_2_0 + 0.000652398655871029*G0_2_1 + 0.00130479731174096*G0_2_2;
    A[2115] = 0.0929668084615971*G0_0_0 + 0.363059851992138*G0_0_1 + 0.228013330226868*G0_0_2 + 0.363059851992138*G0_1_0 + 0.0929668084615974*G0_1_1 + 0.228013330226869*G0_1_2 + 0.228013330226868*G0_2_0 + 0.228013330226867*G0_2_1 + 0.456026660453739*G0_2_2;
    A[2116] = 0.0929668084616009*G0_0_0 - 0.0283793415303822*G0_0_1 + 0.0322937334656089*G0_0_2 - 0.0283793415303818*G0_1_0 - 0.0141896707651909*G0_1_2 + 0.0322937334656094*G0_2_0 - 0.0141896707651916*G0_2_1 - 0.00880738185425511*G0_2_2;
    A[2117] = -0.185933616923197*G0_0_0 - 0.104709984267274*G0_0_1 - 0.195067198105389*G0_0_2 - 0.104709984267273*G0_1_0 - 0.0678494602105693*G0_1_2 - 0.195067198105389*G0_2_0 - 0.0678494602105692*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2118] = -0.0228339529554806*G0_0_0 - 0.0596944770121854*G0_0_1 - 0.0965550010688901*G0_0_2 - 0.059694477012185*G0_1_0 - 0.0965550010688902*G0_1_1 - 0.0287055408583198*G0_1_2 - 0.0965550010688886*G0_2_0 - 0.0287055408583195*G0_2_1 + 0.0391439193522507*G0_2_2;
    A[2119] = -0.00489298991903076*G0_0_0 - 0.019082660684223*G0_0_1 - 0.0332723314494143*G0_0_2 - 0.0190826606842223*G0_1_0 - 0.0332723314494143*G0_1_1 - 0.0190826606842229*G0_1_2 - 0.0332723314494153*G0_2_0 - 0.0190826606842239*G0_2_1 - 0.00489298991903143*G0_2_2;
    A[2120] = -0.0228339529554812*G0_0_0 - 0.0107645778218701*G0_0_1 + 0.00130479731174214*G0_0_2 - 0.0107645778218699*G0_1_0 + 0.00130479731174092*G0_1_1 + 0.000652398655870808*G0_1_2 + 0.00130479731174286*G0_2_0 + 0.000652398655870391*G0_2_1;
    A[2121] = 0.019571959676126*G0_0_0 + 0.00538228891093574*G0_0_1 - 0.0552907860850543*G0_0_2 + 0.00538228891093472*G0_1_0 - 0.00880738185425518*G0_1_1 - 0.0411011153198635*G0_1_2 - 0.0552907860850553*G0_2_0 - 0.0411011153198623*G0_2_1 + 0.0195719596761289*G0_2_2;
    A[2122] = 0.0195719596761261*G0_0_0 + 0.0298472385060926*G0_0_1 + 0.0181040627004164*G0_0_2 + 0.0298472385060921*G0_1_0 + 0.0401225173360588*G0_1_1 + 0.0102752788299664*G0_1_2 + 0.0181040627004152*G0_2_0 + 0.0102752788299665*G0_2_1;
    A[2123] = 0.0260959462348345*G0_0_0 + 0.0381653213684453*G0_0_1 + 0.00619778723077283*G0_0_2 + 0.0381653213684455*G0_1_0 + 0.0502346965020559*G0_1_1 + 0.00554538857490109*G0_1_2 + 0.00619778723077288*G0_2_0 + 0.00554538857490018*G0_2_1;
    A[2124] = 0.146789697570947*G0_0_0 + 0.249542485870614*G0_0_1 + 0.445262082631874*G0_0_2 + 0.249542485870609*G0_1_0 + 0.352295274170273*G0_1_1 + 0.24954248587061*G0_1_2 + 0.445262082631873*G0_2_0 + 0.249542485870611*G0_2_1 - 0.0391439193522517*G0_2_2;
    A[2125] = 0.146789697570947*G0_0_0 + 0.0538228891093465*G0_0_1 + 0.00489298991903125*G0_0_2 + 0.0538228891093485*G0_1_0 - 0.0391439193522516*G0_1_1 + 0.00489298991903148*G0_1_2 + 0.00489298991903121*G0_2_0 + 0.00489298991903155*G0_2_1 + 0.00978597983806229*G0_2_2;
    A[2126] = -0.0489298991903189*G0_0_0 - 0.14189670765192*G0_0_1 - 0.0195719596761289*G0_0_2 - 0.141896707651916*G0_1_0 - 0.234863516113517*G0_1_1 - 0.019571959676128*G0_1_2 - 0.0195719596761275*G0_2_0 - 0.019571959676128*G0_2_1 - 0.0391439193522525*G0_2_2;
    A[2127] = -0.782878387045049*G0_0_0 - 0.484406001984121*G0_0_1 - 0.484406001984124*G0_0_2 - 0.484406001984124*G0_1_0 - 0.185933616923197*G0_1_1 - 0.288686405222861*G0_1_2 - 0.484406001984123*G0_2_0 - 0.288686405222857*G0_2_1 - 0.0391439193522517*G0_2_2;
    A[2128] = 0.00946499969937457*G0_0_0 + 0.00946499969937464*G0_0_1 + 0.00946499969937459*G0_0_2 + 0.0117431758056757*G0_1_0 + 0.0117431758056757*G0_1_1 + 0.0117431758056757*G0_1_2 + 0.0117431758056758*G0_2_0 + 0.0117431758056757*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[2129] = -0.00946499969937472*G0_0_0;
    A[2130] = 0.00472467106581679*G0_0_1 - 0.0298472385060926*G0_1_1;
    A[2131] = 0.00472467106581691*G0_0_2 - 0.0298472385060925*G0_2_2;
    A[2132] = 0.049321338383839*G0_0_1 + 0.0429604514890976*G0_0_2 + 0.176636936077039*G0_1_1 + 0.0422754329004325*G0_1_2 + 0.0422754329004327*G0_2_1;
    A[2133] = -0.167731694424402*G0_0_1 - 0.0777006799142215*G0_0_2 - 0.334680510461758*G0_1_1 - 0.0838658472122009*G0_1_2 - 0.0838658472122012*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2134] = -0.07770067991422*G0_0_1 - 0.1677316944244*G0_0_2 + 0.104709984267275*G0_1_1 - 0.0838658472122009*G0_1_2 - 0.0838658472121998*G0_2_1 - 0.334680510461757*G0_2_2;
    A[2135] = 0.0429604514890968*G0_0_1 + 0.0493213383838377*G0_0_2 + 0.0422754329004323*G0_1_2 + 0.0422754329004318*G0_2_1 + 0.176636936077038*G0_2_2;
    A[2136] = 0.0292600797158093*G0_0_0 + 0.00329461321214825*G0_0_2 + 0.00329461321214778*G0_2_0;
    A[2137] = -0.0189848008858437*G0_0_0 - 0.009492400442922*G0_0_2 - 0.00949240044292172*G0_2_0 - 0.000652398655871729*G0_2_2;
    A[2138] = 0.00352295274170326*G0_0_0 + 0.00649136662591562*G0_0_2 + 0.00649136662591511*G0_2_0 + 0.00880738185425702*G0_2_2;
    A[2139] = 0.0429604514890966*G0_0_0 + 0.0422754329004323*G0_0_2 + 0.0422754329004318*G0_2_0 + 0.06426126760328*G0_2_2;
    A[2140] = 0.0292600797158088*G0_0_0 + 0.00329461321214812*G0_0_1 + 0.00329461321214778*G0_1_0;
    A[2141] = -0.0189848008858417*G0_0_0 - 0.00949240044292122*G0_0_1 - 0.00949240044292104*G0_1_0 - 0.000652398655871437*G0_1_1;
    A[2142] = 0.00352295274170217*G0_0_0 + 0.00649136662591558*G0_0_1 + 0.0064913666259149*G0_1_0 + 0.00880738185425726*G0_1_1;
    A[2143] = 0.0429604514890975*G0_0_0 + 0.0422754329004327*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.0642612676032806*G0_1_1;
    A[2144] = -0.0292600797158082*G0_0_0 - 0.0292600797158083*G0_0_1 - 0.0259654665036607*G0_0_2 - 0.0435476102793805*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.0377086423093361*G0_1_2 - 0.0377086423093365*G0_2_0 - 0.0377086423093365*G0_2_1 - 0.0318696743392924*G0_2_2;
    A[2145] = 0.0189848008858419*G0_0_0 + 0.0189848008858421*G0_0_1 + 0.00949240044292116*G0_0_2 + 0.0567586830607657*G0_1_0 + 0.0567586830607656*G0_1_1 + 0.0378717419733033*G0_1_2 + 0.0378717419733049*G0_2_0 + 0.0378717419733049*G0_2_1 + 0.0127870136550694*G0_2_2;
    A[2146] = -0.0035229527417019*G0_0_0 - 0.00352295274170194*G0_0_1 + 0.00296841388421266*G0_0_2 + 0.104709984267276*G0_1_0 + 0.104709984267276*G0_1_1 + 0.0303691574307903*G0_1_2 + 0.0303691574307882*G0_2_0 + 0.0303691574307881*G0_2_1 - 0.00678494602105704*G0_2_2;
    A[2147] = -0.0429604514890969*G0_0_0 - 0.0429604514890969*G0_0_1 - 0.000685018588664691*G0_0_2 - 0.0422754329004335*G0_1_2 - 0.042275432900432*G0_2_0 - 0.0422754329004319*G0_2_1 + 0.0439716694056968*G0_2_2;
    A[2148] = -0.0292600797158081*G0_0_0 - 0.0259654665036606*G0_0_1 - 0.029260079715808*G0_0_2 - 0.0377086423093365*G0_1_0 - 0.0318696743392923*G0_1_1 - 0.0377086423093365*G0_1_2 - 0.0435476102793806*G0_2_0 - 0.0377086423093363*G0_2_1 - 0.0435476102793805*G0_2_2;
    A[2149] = 0.0189848008858415*G0_0_0 + 0.00949240044292076*G0_0_1 + 0.0189848008858414*G0_0_2 + 0.037871741973304*G0_1_0 + 0.0127870136550689*G0_1_1 + 0.0378717419733039*G0_1_2 + 0.0567586830607652*G0_2_0 + 0.0378717419733036*G0_2_1 + 0.0567586830607651*G0_2_2;
    A[2150] = -0.00352295274170223*G0_0_0 + 0.00296841388421262*G0_0_1 - 0.00352295274170223*G0_0_2 + 0.0303691574307887*G0_1_0 - 0.00678494602105708*G0_1_1 + 0.0303691574307888*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0303691574307897*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2151] = -0.0429604514890975*G0_0_0 - 0.000685018588664224*G0_0_1 - 0.0429604514890973*G0_0_2 - 0.0422754329004324*G0_1_0 + 0.0439716694056972*G0_1_1 - 0.0422754329004324*G0_1_2 - 0.0422754329004328*G0_2_1;
    A[2152] = -0.00329461321214744*G0_0_1 - 0.00329461321214746*G0_0_2 - 0.00329461321214818*G0_1_0 - 0.00913358118219256*G0_1_1 - 0.00913358118219246*G0_1_2 - 0.00329461321214808*G0_2_0 - 0.0091335811821922*G0_2_1 - 0.00913358118219224*G0_2_2;
    A[2153] = -0.00257697469068991*G0_0_1 - 0.00257697469068991*G0_0_2 - 0.00257697469068965*G0_1_0 - 0.0156575677409007*G0_1_1 - 0.0156575677409008*G0_1_2 - 0.00257697469068994*G0_2_0 - 0.015657567740901*G0_2_1 - 0.0156575677409011*G0_2_2;
    A[2154] = -0.00257697469068977*G0_0_1 - 0.0025769746906897*G0_0_2 - 0.00257697469068984*G0_1_0 - 0.000652398655870948*G0_1_1 - 0.000652398655870893*G0_1_2 - 0.00257697469068987*G0_2_0 - 0.000652398655871059*G0_2_1 - 0.000652398655871055*G0_2_2;
    A[2155] = -0.00329461321214814*G0_0_1 - 0.00329461321214814*G0_0_2 - 0.00329461321214826*G0_1_0 - 0.00329461321214807*G0_2_0;
    A[2156] = -0.113517366121533*G0_0_0 - 0.0127217737894823*G0_0_1 - 0.0127217737894822*G0_0_2 - 0.0127217737894821*G0_1_0 - 0.000652398655871388*G0_1_2 - 0.0127217737894824*G0_2_0 - 0.000652398655871597*G0_2_1;
    A[2157] = 0.149725491522366*G0_0_0 + 0.0748627457611828*G0_0_1 + 0.0283793415303842*G0_0_2 + 0.0748627457611825*G0_1_0 + 0.0195719596761262*G0_1_1 + 0.0141896707651922*G0_1_2 + 0.0283793415303834*G0_2_0 + 0.0141896707651924*G0_2_1;
    A[2158] = 0.0234863516113524*G0_0_0 - 0.0225077536275456*G0_0_1 + 0.104709984267275*G0_0_2 - 0.0225077536275449*G0_1_0 - 0.15657567740901*G0_1_1 + 0.0678494602105701*G0_1_2 + 0.104709984267274*G0_2_0 + 0.0678494602105691*G0_2_1;
    A[2159] = 0.149725491522366*G0_0_0 + 0.0283793415303821*G0_0_1 + 0.0748627457611819*G0_0_2 + 0.0283793415303839*G0_1_0 + 0.0141896707651924*G0_1_2 + 0.0748627457611823*G0_2_0 + 0.0141896707651904*G0_2_1 + 0.0195719596761259*G0_2_2;
    A[2160] = -0.540186087061082*G0_0_0 - 0.270093043530539*G0_0_1 - 0.270093043530541*G0_0_2 - 0.270093043530543*G0_1_0 + 0.0929668084615985*G0_1_1 - 0.135046521765272*G0_1_2 - 0.27009304353054*G0_2_0 - 0.135046521765268*G0_2_1 + 0.0929668084616009*G0_2_2;
    A[2161] = 0.0234863516113521*G0_0_0 + 0.104709984267275*G0_0_1 - 0.022507753627544*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0678494602105709*G0_1_2 - 0.0225077536275452*G0_2_0 + 0.0678494602105695*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2162] = 0.113517366121531*G0_0_0 + 0.100795592332049*G0_0_1 + 0.100795592332049*G0_0_2 + 0.198655390712681*G0_1_0 + 0.172233245149911*G0_1_1 + 0.166035457919137*G0_1_2 + 0.198655390712681*G0_2_0 + 0.166035457919137*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2163] = -0.149725491522364*G0_0_0 - 0.0748627457611809*G0_0_1 - 0.12134614999198*G0_0_2 - 0.177615534060845*G0_1_0 - 0.0371867233846391*G0_1_1 - 0.224098938291644*G0_1_2 - 0.326851726591307*G0_2_0 - 0.224098938291644*G0_2_1 - 0.448197876583288*G0_2_2;
    A[2164] = -0.023486351611352*G0_0_0 - 0.0459941052388989*G0_0_1 + 0.0812236326559229*G0_0_2 + 0.0225077536275451*G0_1_0 - 0.26813584756293*G0_1_1 + 0.0534966897814114*G0_1_2 - 0.104709984267274*G0_2_0 + 0.0534966897814112*G0_2_1 - 0.20941996853455*G0_2_2;
    A[2165] = -0.149725491522366*G0_0_0 - 0.121346149991983*G0_0_1 - 0.0748627457611824*G0_0_2 - 0.32685172659131*G0_1_0 - 0.448197876583291*G0_1_1 - 0.224098938291646*G0_1_2 - 0.177615534060846*G0_2_0 - 0.224098938291645*G0_2_1 - 0.0371867233846417*G0_2_2;
    A[2166] = 0.540186087061082*G0_0_0 + 0.270093043530543*G0_0_1 + 0.27009304353054*G0_0_2 + 0.270093043530543*G0_1_0 + 0.726119703984284*G0_1_1 + 0.270093043530541*G0_1_2 + 0.27009304353054*G0_2_0 + 0.270093043530541*G0_2_1 + 0.72611970398428*G0_2_2;
    A[2167] = -0.0234863516113516*G0_0_0 + 0.0812236326559223*G0_0_1 - 0.0459941052388962*G0_0_2 - 0.104709984267275*G0_1_0 - 0.20941996853455*G0_1_1 + 0.0534966897814131*G0_1_2 + 0.0225077536275455*G0_2_0 + 0.0534966897814121*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2168] = 0.0127217737894827*G0_0_1 + 0.0120693751336108*G0_0_2 + 0.0127217737894823*G0_1_0 + 0.0391439193522536*G0_1_1 + 0.0446893079271557*G0_1_2 + 0.0120693751336113*G0_2_0 + 0.0446893079271557*G0_2_1 + 0.0502346965020579*G0_2_2;
    A[2169] = 0.0181040627004166*G0_0_1 + 0.0102752788299659*G0_0_2 + 0.0181040627004161*G0_1_0 + 0.0195719596761254*G0_1_1 + 0.0298472385060918*G0_1_2 + 0.0102752788299662*G0_2_0 + 0.0298472385060919*G0_2_1 + 0.0401225173360586*G0_2_2;
    A[2170] = 0.0127217737894826*G0_0_1 + 0.0120693751336115*G0_0_2 + 0.0127217737894823*G0_1_0 + 0.00065239865587141*G0_1_2 + 0.0120693751336115*G0_2_0 + 0.000652398655871754*G0_2_1 + 0.00130479731174234*G0_2_2;
    A[2171] = -0.0283793415303836*G0_0_1 - 0.0141896707651911*G0_0_2 - 0.028379341530383*G0_1_0 + 0.092966808461599*G0_1_1 + 0.0322937334656078*G0_1_2 - 0.014189670765192*G0_2_0 + 0.0322937334656069*G0_2_1 - 0.00880738185425782*G0_2_2;
    A[2172] = -0.0283793415303823*G0_0_1 - 0.0141896707651912*G0_0_2 - 0.0283793415303823*G0_1_0 - 0.0141896707651915*G0_1_2 - 0.0141896707651913*G0_2_0 - 0.0141896707651904*G0_2_1 - 0.0332723314494142*G0_2_2;
    A[2173] = -0.104709984267275*G0_0_1 - 0.0368605240567048*G0_0_2 - 0.104709984267276*G0_1_0 - 0.067849460210571*G0_1_2 - 0.036860524056703*G0_2_0 - 0.0678494602105695*G0_2_1 - 0.096555001068888*G0_2_2;
    A[2174] = 0.0120693751336104*G0_0_1 + 0.0127217737894825*G0_0_2 + 0.0120693751336119*G0_1_0 + 0.0502346965020582*G0_1_1 + 0.0446893079271562*G0_1_2 + 0.012721773789482*G0_2_0 + 0.0446893079271544*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[2175] = 0.0102752788299666*G0_0_1 + 0.0181040627004173*G0_0_2 + 0.0102752788299651*G0_1_0 + 0.040122517336058*G0_1_1 + 0.0298472385060918*G0_1_2 + 0.0181040627004166*G0_2_0 + 0.029847238506093*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2176] = 0.0120693751336109*G0_0_1 + 0.0127217737894818*G0_0_2 + 0.0120693751336118*G0_1_0 + 0.00130479731174248*G0_1_1 + 0.000652398655871001*G0_1_2 + 0.0127217737894826*G0_2_0 + 0.000652398655870854*G0_2_1;
    A[2177] = -0.0141896707651907*G0_0_1 - 0.0283793415303828*G0_0_2 - 0.0141896707651913*G0_1_0 - 0.00880738185425751*G0_1_1 + 0.032293733465607*G0_1_2 - 0.0283793415303818*G0_2_0 + 0.0322937334656084*G0_2_1 + 0.0929668084615991*G0_2_2;
    A[2178] = -0.0141896707651919*G0_0_1 - 0.0283793415303837*G0_0_2 - 0.0141896707651916*G0_1_0 - 0.0332723314494144*G0_1_1 - 0.0141896707651917*G0_1_2 - 0.0283793415303843*G0_2_0 - 0.0141896707651919*G0_2_1;
    A[2179] = -0.0368605240567052*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0368605240567037*G0_1_0 - 0.0965550010688886*G0_1_1 - 0.0678494602105703*G0_1_2 - 0.104709984267274*G0_2_0 - 0.0678494602105705*G0_2_1;
    A[2180] = -0.0929668084616018*G0_0_1 - 0.0929668084616029*G0_0_2 - 0.0929668084616009*G0_1_0 - 0.234863516113518*G0_1_1 - 0.215291556437392*G0_1_2 - 0.0929668084616007*G0_2_0 - 0.21529155643739*G0_2_1 - 0.234863516113516*G0_2_2;
    A[2181] = -0.0929668084615996*G0_0_1 - 0.092966808461599*G0_0_2 - 0.0929668084615984*G0_1_0 - 0.0391439193522533*G0_1_1 - 0.0440369092712839*G0_1_2 - 0.0929668084615982*G0_2_0 - 0.0440369092712843*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[2182] = 0.102752788299665*G0_0_1 + 0.298472385060924*G0_0_2 + 0.102752788299663*G0_1_0 + 0.352295274170276*G0_1_1 + 0.102752788299665*G0_1_2 + 0.298472385060925*G0_2_0 + 0.102752788299665*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2183] = 0.298472385060923*G0_0_1 + 0.102752788299664*G0_0_2 + 0.298472385060926*G0_1_0 - 0.185933616923197*G0_1_1 + 0.102752788299665*G0_1_2 + 0.102752788299664*G0_2_0 + 0.102752788299662*G0_2_1 + 0.352295274170273*G0_2_2;
    A[2184] = 0.013742125287264*G0_0_0 + 0.0137421252872641*G0_0_1 + 0.0137421252872641*G0_0_2 + 0.0189195610202555*G0_1_0 + 0.0189195610202555*G0_1_1 + 0.0189195610202555*G0_1_2 + 0.0189195610202554*G0_2_0 + 0.0189195610202554*G0_2_1 + 0.0189195610202554*G0_2_2;
    A[2185] = -0.0137421252872642*G0_0_0;
    A[2186] = 0.00517743573299124*G0_0_1 + 0.0189195610202554*G0_1_1;
    A[2187] = -0.0143945239431353*G0_0_2 + 0.0335985307773502*G0_2_2;
    A[2188] = -0.047755581609748*G0_0_1 - 0.0118736555368501*G0_0_2 - 0.111886369481855*G0_1_1 - 0.0212681961813907*G0_1_2 - 0.0212681961813911*G0_2_1;
    A[2189] = 0.0823327103709037*G0_0_1 - 0.00182671623643845*G0_0_2 + 0.209419968534551*G0_1_1 + 0.0359471659384849*G0_1_2 + 0.0359471659384855*G0_2_1 - 0.0528442911255403*G0_2_2;
    A[2190] = -0.146659217839771*G0_0_1 - 0.0109602974186309*G0_0_2 - 0.271397840842283*G0_1_1 - 0.0733296089198857*G0_1_2 - 0.073329608919887*G0_2_1 + 0.121346149991981*G0_2_2;
    A[2191] = -0.124738623002511*G0_0_1 - 0.192588083213081*G0_0_2 - 0.0962940416065408*G0_1_2 - 0.0962940416065399*G0_2_1 - 0.257045070413124*G0_2_2;
    A[2192] = 0.0641307878721067*G0_0_0 + 0.00939454064454008*G0_0_2 + 0.00939454064454045*G0_2_0;
    A[2193] = -0.127087258163646*G0_0_0 - 0.0377738821749228*G0_0_2 - 0.037773882174923*G0_2_0 - 0.0150051690850298*G0_2_2;
    A[2194] = 0.12473862300251*G0_0_0 + 0.0623693115012549*G0_0_2 + 0.0623693115012553*G0_2_0 + 0.047625101878574*G0_2_2;
    A[2195] = -0.124738623002512*G0_0_0 - 0.0962940416065411*G0_0_2 - 0.09629404160654*G0_2_0 - 0.128522535206561*G0_2_2;
    A[2196] = 0.0187890812890809*G0_0_0 + 0.00939454064454013*G0_0_1 + 0.00939454064454093*G0_1_0;
    A[2197] = -0.0148746893538564*G0_0_0 - 0.0149399292194432*G0_0_1 - 0.0149399292194433*G0_1_0 - 0.0150051690850305*G0_1_1;
    A[2198] = -0.00117431758056747*G0_0_0 - 0.00123955744615451*G0_0_1 - 0.00123955744615447*G0_1_0 - 0.0013047973117412*G0_1_1;
    A[2199] = -0.0118736555368502*G0_0_0 - 0.0212681961813908*G0_0_1 - 0.0212681961813907*G0_1_0 - 0.0306627368259316*G0_1_1;
    A[2200] = -0.064130787872107*G0_0_0 - 0.0641307878721071*G0_0_1 - 0.0547362472275665*G0_0_2 - 0.111886369481855*G0_1_0 - 0.111886369481855*G0_1_1 - 0.0906181733004646*G0_1_2 - 0.0906181733004641*G0_2_0 - 0.0906181733004642*G0_2_1 - 0.0693499771190736*G0_2_2;
    A[2201] = 0.127087258163646*G0_0_0 + 0.127087258163647*G0_0_1 + 0.0893133759887231*G0_0_2 + 0.20941996853455*G0_1_0 + 0.20941996853455*G0_1_1 + 0.173472802596066*G0_1_2 + 0.173472802596064*G0_2_0 + 0.173472802596065*G0_2_1 + 0.0846813455320388*G0_2_2;
    A[2202] = -0.124738623002511*G0_0_0 - 0.124738623002511*G0_0_1 - 0.0623693115012561*G0_0_2 - 0.271397840842283*G0_1_0 - 0.271397840842283*G0_1_1 - 0.198068231922397*G0_1_2 - 0.198068231922396*G0_2_0 - 0.198068231922396*G0_2_1 - 0.00339247301052766*G0_2_2;
    A[2203] = 0.124738623002512*G0_0_0 + 0.124738623002512*G0_0_1 + 0.0284445813959708*G0_0_2 + 0.0962940416065411*G0_1_2 + 0.0962940416065403*G0_2_0 + 0.0962940416065401*G0_2_1 - 0.0644569872000428*G0_2_2;
    A[2204] = -0.0187890812890814*G0_0_0 - 0.00939454064454062*G0_0_1 - 0.0187890812890813*G0_0_2 - 0.0400572774704716*G0_1_0 - 0.0187890812890814*G0_1_1 - 0.0400572774704716*G0_1_2 - 0.0613254736518626*G0_2_0 - 0.0400572774704722*G0_2_1 - 0.0613254736518625*G0_2_2;
    A[2205] = 0.0148746893538561*G0_0_0 - 6.52398655871445e-05*G0_0_1 + 0.014874689353856*G0_0_2 + 0.0136351319077016*G0_1_0 - 0.000130479731174187*G0_1_1 + 0.0136351319077016*G0_1_2 - 0.0254435475789631*G0_2_0 + 0.0136351319077018*G0_2_1 - 0.025443547578963*G0_2_2;
    A[2206] = 0.00117431758056729*G0_0_0 - 6.52398655868321e-05*G0_0_1 + 0.00117431758056731*G0_0_2 - 0.0137656116388757*G0_1_0 - 0.000130479731173802*G0_1_1 - 0.0137656116388757*G0_1_2 - 0.0528442911255411*G0_2_0 - 0.0137656116388756*G0_2_1 - 0.0528442911255412*G0_2_2;
    A[2207] = 0.0118736555368501*G0_0_0 - 0.00939454064454067*G0_0_1 + 0.0118736555368502*G0_0_2 + 0.0212681961813907*G0_1_0 - 0.0187890812890814*G0_1_1 + 0.0212681961813907*G0_1_2 + 0.0212681961813906*G0_2_1;
    A[2208] = -0.00939454064454075*G0_0_1 - 0.00939454064454068*G0_0_2 - 0.00939454064454058*G0_1_0 - 0.0306627368259307*G0_1_1 - 0.0306627368259307*G0_1_2 - 0.00939454064454062*G0_2_0 - 0.0306627368259311*G0_2_1 - 0.030662736825931*G0_2_2;
    A[2209] = -6.52398655867682e-05*G0_0_1 - 6.52398655868278e-05*G0_0_2 - 6.52398655873561e-05*G0_1_0 - 0.00130479731174206*G0_1_1 - 0.00130479731174207*G0_1_2 - 6.52398655871564e-05*G0_2_0 - 0.00130479731174194*G0_2_1 - 0.00130479731174197*G0_2_2;
    A[2210] = -6.52398655877392e-05*G0_0_1 - 6.52398655877489e-05*G0_0_2 - 6.52398655868786e-05*G0_1_0 - 0.0150051690850301*G0_1_1 - 0.01500516908503*G0_1_2 - 6.52398655870169e-05*G0_2_0 - 0.0150051690850302*G0_2_1 - 0.0150051690850302*G0_2_2;
    A[2211] = -0.00939454064454021*G0_0_1 - 0.00939454064454015*G0_0_2 - 0.00939454064454065*G0_1_0 - 0.0093945406445406*G0_2_0;
    A[2212] = -0.105688582251082*G0_0_0 - 0.0528442911255406*G0_0_1 - 0.0300103381700608*G0_0_2 - 0.0528442911255414*G0_1_0 - 0.01500516908503*G0_1_2 - 0.0300103381700609*G0_2_0 - 0.01500516908503*G0_2_1;
    A[2213] = 0.0137003717732884*G0_0_0 + 0.0198981590040609*G0_0_1 - 0.0127217737894827*G0_0_2 + 0.0198981590040619*G0_1_0 + 0.026095946234834*G0_1_1 - 0.0120693751336116*G0_1_2 - 0.0127217737894818*G0_2_0 - 0.0120693751336124*G0_2_1;
    A[2214] = -0.0534966897814124*G0_0_0 - 0.000652398655871364*G0_0_1 - 0.0528442911255413*G0_0_2 - 0.000652398655872186*G0_1_0 + 0.0521918924696695*G0_1_1 - 0.0378391220405115*G0_1_2 - 0.0528442911255413*G0_2_0 - 0.0378391220405102*G0_2_1;
    A[2215] = 0.209419968534549*G0_0_0 + 0.104709984267275*G0_0_1 + 0.0737210481134085*G0_0_2 + 0.104709984267275*G0_1_0 + 0.0368605240567034*G0_1_2 + 0.0737210481134085*G0_2_0 + 0.0368605240567044*G0_2_1 - 0.0228339529554811*G0_2_2;
    A[2216] = 0.0234863516113515*G0_0_0 - 0.081223632655923*G0_0_1 + 0.045994105238897*G0_0_2 - 0.0812236326559223*G0_1_0 - 0.185933616923197*G0_1_1 + 0.0091335811821937*G0_1_2 + 0.0459941052388961*G0_2_0 + 0.00913358118219197*G0_2_1 - 0.0880738185425678*G0_2_2;
    A[2217] = -0.542795681684566*G0_0_0 - 0.271397840842285*G0_0_1 - 0.271397840842284*G0_0_2 - 0.271397840842283*G0_1_0 - 0.135698920421141*G0_1_2 - 0.271397840842283*G0_2_0 - 0.13569892042114*G0_2_1 + 0.0521918924696701*G0_2_2;
    A[2218] = 0.105688582251082*G0_0_0 + 0.052844291125541*G0_0_1 + 0.0756782440810212*G0_0_2 + 0.105036183595211*G0_1_0 + 0.105688582251082*G0_1_1 + 0.127870136550691*G0_1_2 + 0.180062029020363*G0_2_0 + 0.127870136550693*G0_2_1 + 0.255740273101383*G0_2_2;
    A[2219] = -0.0137003717732901*G0_0_0 + 0.00619778723077202*G0_0_1 - 0.0264221455627712*G0_0_2 + 0.00619778723077134*G0_1_0 + 0.0123955744615447*G0_1_1 + 0.00619778723077204*G0_1_2 + 0.071437652817858*G0_2_0 + 0.00619778723077055*G0_2_1 + 0.172233245149908*G0_2_2;
    A[2220] = 0.0534966897814126*G0_0_0 + 0.0528442911255418*G0_0_1 + 0.00065239865587161*G0_0_2 + 0.000652398655872305*G0_1_0 + 0.105688582251083*G0_1_1 - 0.022181554299609*G0_1_2 + 0.0528442911255415*G0_2_0 - 0.022181554299608*G0_2_1 + 0.105688582251082*G0_2_2;
    A[2221] = -0.209419968534549*G0_0_0 - 0.104709984267275*G0_0_1 - 0.135698920421141*G0_0_2 - 0.290643601190473*G0_1_0 - 0.20941996853455*G0_1_1 - 0.262916658315961*G0_1_2 - 0.390134396210782*G0_2_0 - 0.262916658315963*G0_2_1 - 0.584549195660302*G0_2_2;
    A[2222] = -0.0234863516113516*G0_0_0 - 0.104709984267275*G0_0_1 + 0.0225077536275455*G0_0_2 + 0.0812236326559223*G0_1_0 - 0.20941996853455*G0_1_1 + 0.0534966897814121*G0_1_2 - 0.0459941052388962*G0_2_0 + 0.0534966897814131*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2223] = 0.542795681684566*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842283*G0_0_2 + 0.271397840842282*G0_1_0 + 0.542795681684566*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842283*G0_2_0 + 0.271397840842283*G0_2_1 + 0.647179466623906*G0_2_2;
    A[2224] = 0.0528442911255412*G0_0_1 + 0.0378391220405106*G0_0_2 + 0.0528442911255404*G0_1_0 + 0.0521918924696694*G0_1_1 + 0.0900310145101795*G0_1_2 + 0.0378391220405102*G0_2_0 + 0.0900310145101806*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2225] = 0.00619778723077334*G0_0_1 + 0.00554538857490311*G0_0_2 + 0.0061977872307732*G0_1_0 + 0.0260959462348351*G0_1_1 + 0.0381653213684465*G0_1_2 + 0.00554538857490219*G0_2_0 + 0.038165321368446*G0_2_1 + 0.0502346965020579*G0_2_2;
    A[2226] = 0.0528442911255407*G0_0_1 + 0.0378391220405107*G0_0_2 + 0.0528442911255409*G0_1_0 + 0.0150051690850302*G0_1_2 + 0.037839122040511*G0_2_0 + 0.0150051690850302*G0_2_1 + 0.0300103381700605*G0_2_2;
    A[2227] = -0.104709984267274*G0_0_1 - 0.0678494602105712*G0_0_2 - 0.104709984267274*G0_1_0 - 0.185933616923197*G0_1_1 - 0.19506719810539*G0_1_2 - 0.067849460210569*G0_2_0 - 0.195067198105389*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2228] = -0.104709984267275*G0_0_1 - 0.0678494602105714*G0_0_2 - 0.104709984267276*G0_1_0 - 0.0368605240567048*G0_1_2 - 0.0678494602105722*G0_2_0 - 0.0368605240567046*G0_2_1 - 0.0965550010688905*G0_2_2;
    A[2229] = 0.271397840842284*G0_0_1 + 0.135698920421142*G0_0_2 + 0.271397840842283*G0_1_0 + 0.135698920421141*G0_1_2 + 0.13569892042114*G0_2_0 + 0.135698920421141*G0_2_1 + 0.323589733311952*G0_2_2;
    A[2230] = 0.0150051690850307*G0_0_1 + 0.0300103381700612*G0_0_2 + 0.01500516908503*G0_1_0 + 0.0300103381700604*G0_1_1 - 0.00782878387044988*G0_1_2 + 0.0300103381700606*G0_2_0 - 0.00782878387044933*G0_2_1 - 0.0456679059109595*G0_2_2;
    A[2231] = 0.000652398655871192*G0_0_1 + 0.00130479731174152*G0_0_2 + 0.000652398655871125*G0_1_0 + 0.00130479731174238*G0_1_1 - 0.0107645778218692*G0_1_2 + 0.00130479731174149*G0_2_0 - 0.0107645778218694*G0_2_1 - 0.0228339529554815*G0_2_2;
    A[2232] = 0.0150051690850309*G0_0_1 + 0.030010338170061*G0_0_2 + 0.0150051690850297*G0_1_0 + 0.0300103381700603*G0_1_1 + 0.01500516908503*G0_1_2 + 0.0300103381700601*G0_2_0 + 0.0150051690850306*G0_2_1;
    A[2233] = 0.000652398655869601*G0_0_1 + 0.0127217737894805*G0_0_2 + 0.000652398655870591*G0_1_0 + 0.00130479731174077*G0_1_1 + 0.000652398655870027*G0_1_2 + 0.0127217737894806*G0_2_0 + 0.000652398655868976*G0_2_1 - 0.08807381854257*G0_2_2;
    A[2234] = 0.000652398655870497*G0_0_1 + 0.0127217737894824*G0_0_2 + 0.000652398655871196*G0_1_0 + 0.00130479731174133*G0_1_1 + 0.0120693751336115*G0_1_2 + 0.0127217737894833*G0_2_0 + 0.0120693751336112*G0_2_1;
    A[2235] = 0.0150051690850311*G0_0_1 + 0.0528442911255413*G0_0_2 + 0.0150051690850303*G0_1_0 + 0.0300103381700609*G0_1_1 + 0.0378391220405113*G0_1_2 + 0.0528442911255409*G0_2_0 + 0.037839122040512*G0_2_1;
    A[2236] = -0.0260959462348371*G0_0_1 - 0.0750258454251512*G0_0_2 - 0.0260959462348366*G0_1_0 - 0.0521918924696708*G0_1_1 - 0.0424059126316077*G0_1_2 - 0.0750258454251523*G0_2_0 - 0.042405912631609*G0_2_1 + 0.143527704291591*G0_2_2;
    A[2237] = -0.0260959462348342*G0_0_1 - 0.0750258454251504*G0_0_2 - 0.0260959462348339*G0_1_0 - 0.0521918924696681*G0_1_1 - 0.026095946234834*G0_1_2 - 0.0750258454251499*G0_2_0 - 0.0260959462348338*G0_2_1 + 0.0456679059109627*G0_2_2;
    A[2238] = -0.0260959462348331*G0_0_1 - 0.0587158790283772*G0_0_2 - 0.0260959462348332*G0_1_0 - 0.0521918924696685*G0_1_1 - 0.00978597983806342*G0_1_2 - 0.0587158790283763*G0_2_0 - 0.00978597983806137*G0_2_1 + 0.176147637085137*G0_2_2;
    A[2239] = 0.185933616923197*G0_0_1 + 0.316413348097374*G0_0_2 + 0.185933616923199*G0_1_0 + 0.371867233846394*G0_1_1 + 0.185933616923199*G0_1_2 + 0.316413348097374*G0_2_0 + 0.185933616923197*G0_2_1 - 0.150051690850301*G0_2_2;
    A[2240] = 0.0335985307773499*G0_0_0 + 0.0335985307773499*G0_0_1 + 0.0335985307773499*G0_0_2 + 0.0479930547204848*G0_1_0 + 0.0479930547204849*G0_1_1 + 0.0479930547204849*G0_1_2 + 0.0335985307773497*G0_2_0 + 0.0335985307773498*G0_2_1 + 0.0335985307773498*G0_2_2;
    A[2241] = 0.0189195610202553*G0_0_0 + 0.00517743573299118*G0_1_0;
    A[2242] = -0.0137421252872641*G0_1_1;
    A[2243] = 0.00517743573299142*G0_1_2 + 0.0189195610202556*G0_2_2;
    A[2244] = 0.0187890812890813*G0_1_1 + 0.00939454064454039*G0_1_2 + 0.00939454064454031*G0_2_1;
    A[2245] = -0.014874689353856*G0_1_1 - 0.014939929219443*G0_1_2 - 0.014939929219443*G0_2_1 - 0.0150051690850301*G0_2_2;
    A[2246] = -0.00117431758056757*G0_1_1 - 0.00123955744615492*G0_1_2 - 0.00123955744615459*G0_2_1 - 0.00130479731174191*G0_2_2;
    A[2247] = -0.0118736555368499*G0_1_1 - 0.0212681961813907*G0_1_2 - 0.0212681961813903*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[2248] = -0.061325473651862*G0_0_0 - 0.0212681961813904*G0_0_2 - 0.0425363923627808*G0_1_0 - 0.01187365553685*G0_1_2 - 0.0212681961813905*G0_2_0;
    A[2249] = -0.0254435475789638*G0_0_0 - 0.0390786794866655*G0_0_2 - 0.0403182369328201*G0_1_0 - 0.0540186087061086*G0_1_2 - 0.039078679486665*G0_2_0 - 0.0528442911255407*G0_2_2;
    A[2250] = -0.052844291125541*G0_0_0 - 0.039078679486665*G0_0_2 - 0.0540186087061081*G0_1_0 - 0.0403182369328196*G0_1_2 - 0.0390786794866652*G0_2_0 - 0.0254435475789636*G0_2_2;
    A[2251] = -0.0212681961813904*G0_0_2 - 0.0118736555368497*G0_1_0 - 0.042536392362781*G0_1_2 - 0.0212681961813901*G0_2_0 - 0.061325473651862*G0_2_2;
    A[2252] = -0.0306627368259306*G0_0_0 - 0.0212681961813902*G0_0_1 - 0.0212681961813901*G0_1_0 - 0.0118736555368498*G0_1_1;
    A[2253] = -0.00130479731174212*G0_0_0 - 0.00123955744615457*G0_0_1 - 0.00123955744615496*G0_1_0 - 0.00117431758056721*G0_1_1;
    A[2254] = -0.0150051690850301*G0_0_0 - 0.0149399292194434*G0_0_1 - 0.0149399292194429*G0_1_0 - 0.0148746893538565*G0_1_1;
    A[2255] = 0.00939454064454074*G0_0_1 + 0.0093945406445404*G0_1_0 + 0.0187890812890812*G0_1_1;
    A[2256] = -0.257045070413123*G0_0_0 - 0.257045070413123*G0_0_1 - 0.160751028806583*G0_0_2 - 0.0644569872000416*G0_1_0 - 0.0644569872000416*G0_1_1 - 0.092901568596012*G0_1_2 - 0.160751028806582*G0_2_0 - 0.160751028806582*G0_2_1 - 0.0644569872000416*G0_2_2;
    A[2257] = 0.121346149991982*G0_0_0 + 0.121346149991982*G0_0_1 + 0.194675758911868*G0_0_2 + 0.132306447410613*G0_1_0 + 0.132306447410613*G0_1_1 + 0.0589768384907264*G0_1_2 + 0.194675758911868*G0_2_0 + 0.194675758911868*G0_2_1 - 0.00339247301052947*G0_2_2;
    A[2258] = -0.0528442911255399*G0_0_0 - 0.0528442911255399*G0_0_1 - 0.0887914570640254*G0_0_2 - 0.0510175748891015*G0_1_0 - 0.0510175748891014*G0_1_1 - 0.00463203045668219*G0_1_2 - 0.0887914570640248*G0_2_0 - 0.0887914570640247*G0_2_1 + 0.0846813455320407*G0_2_2;
    A[2259] = 0.0212681961813903*G0_0_2 + 0.0118736555368498*G0_1_0 + 0.0118736555368499*G0_1_1 - 0.0146137298915082*G0_1_2 + 0.0212681961813903*G0_2_0 + 0.0212681961813903*G0_2_1 - 0.0693499771190744*G0_2_2;
    A[2260] = -0.128522535206561*G0_0_0 - 0.0322284936000209*G0_0_1 - 0.128522535206561*G0_0_2 - 0.0322284936000207*G0_1_0 - 0.0606730749959911*G0_1_1 - 0.0322284936000206*G0_1_2 - 0.128522535206561*G0_2_0 - 0.0322284936000208*G0_2_1 - 0.128522535206561*G0_2_2;
    A[2261] = 0.0476251018785735*G0_0_0 - 0.0147442096226819*G0_0_1 + 0.0476251018785738*G0_0_2 - 0.0147442096226817*G0_1_0 + 0.0476251018785734*G0_1_1 - 0.0147442096226817*G0_1_2 + 0.0476251018785734*G0_2_0 - 0.0147442096226821*G0_2_1 + 0.0476251018785736*G0_2_2;
    A[2262] = -0.0150051690850297*G0_0_0 + 0.0227687130898936*G0_0_1 - 0.0150051690850299*G0_0_2 + 0.0227687130898937*G0_1_0 - 0.0665446628988286*G0_1_1 + 0.0227687130898936*G0_1_2 - 0.0150051690850297*G0_2_0 + 0.0227687130898942*G0_2_1 - 0.0150051690850297*G0_2_2;
    A[2263] = -0.00939454064454065*G0_0_1 - 0.00939454064454047*G0_1_0 + 0.0453417065830255*G0_1_1 - 0.00939454064454056*G0_1_2 - 0.00939454064454096*G0_2_1;
    A[2264] = -0.0644569872000424*G0_0_0 - 0.160751028806582*G0_0_1 - 0.160751028806582*G0_0_2 - 0.0929015685960126*G0_1_0 - 0.064456987200042*G0_1_1 - 0.064456987200042*G0_1_2 - 0.160751028806583*G0_2_0 - 0.257045070413123*G0_2_1 - 0.257045070413123*G0_2_2;
    A[2265] = -0.00339247301052829*G0_0_0 + 0.194675758911867*G0_0_1 + 0.194675758911867*G0_0_2 + 0.0589768384907272*G0_1_0 + 0.132306447410612*G0_1_1 + 0.132306447410612*G0_1_2 + 0.194675758911868*G0_2_0 + 0.121346149991981*G0_2_1 + 0.121346149991981*G0_2_2;
    A[2266] = 0.0846813455320391*G0_0_0 - 0.0887914570640249*G0_0_1 - 0.0887914570640249*G0_0_2 - 0.00463203045668321*G0_1_0 - 0.0510175748891019*G0_1_1 - 0.0510175748891017*G0_1_2 - 0.0887914570640249*G0_2_0 - 0.0528442911255401*G0_2_1 - 0.05284429112554*G0_2_2;
    A[2267] = -0.0693499771190737*G0_0_0 + 0.0212681961813899*G0_0_1 + 0.02126819618139*G0_0_2 - 0.0146137298915076*G0_1_0 + 0.0118736555368496*G0_1_1 + 0.0118736555368497*G0_1_2 + 0.02126819618139*G0_2_0;
    A[2268] = -0.0456679059109617*G0_0_0 - 0.075678244081021*G0_0_1 - 0.0378391220405101*G0_0_2 - 0.0756782440810213*G0_1_0 - 0.105688582251082*G0_1_1 - 0.0528442911255404*G0_1_2 - 0.0378391220405108*G0_2_0 - 0.0528442911255403*G0_2_1;
    A[2269] = -0.0228339529554802*G0_0_0 - 0.0241387502672232*G0_0_1 - 0.0120693751336122*G0_0_2 - 0.0241387502672217*G0_1_0 - 0.0254435475789637*G0_1_1 - 0.0127217737894818*G0_1_2 - 0.0120693751336105*G0_2_0 - 0.0127217737894827*G0_2_1;
    A[2270] = -0.0300103381700599*G0_0_1 - 0.0150051690850297*G0_0_2 - 0.0300103381700609*G0_1_0 - 0.0600206763401212*G0_1_1 - 0.0300103381700607*G0_1_2 - 0.0150051690850309*G0_2_0 - 0.0300103381700602*G0_2_1;
    A[2271] = -0.0880738185425674*G0_0_0 - 0.10079559233205*G0_0_1 - 0.0887262171984387*G0_0_2 - 0.10079559233205*G0_1_0 - 0.113517366121532*G0_1_1 - 0.10079559233205*G0_1_2 - 0.0887262171984382*G0_2_0 - 0.10079559233205*G0_2_1 - 0.0880738185425675*G0_2_2;
    A[2272] = -0.0127217737894815*G0_0_1 - 0.0120693751336106*G0_0_2 - 0.0127217737894825*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.0241387502672225*G0_1_2 - 0.0120693751336106*G0_2_0 - 0.0241387502672209*G0_2_1 - 0.0228339529554796*G0_2_2;
    A[2273] = -0.052844291125541*G0_0_1 - 0.0378391220405109*G0_0_2 - 0.0528442911255411*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0756782440810214*G0_1_2 - 0.0378391220405106*G0_2_0 - 0.0756782440810211*G0_2_1 - 0.0456679059109608*G0_2_2;
    A[2274] = 0.0521918924696704*G0_0_0 + 0.323589733311952*G0_0_1 + 0.18789081289081*G0_0_2 + 0.323589733311955*G0_1_0 + 0.0521918924696698*G0_1_1 + 0.187890812890811*G0_1_2 + 0.187890812890813*G0_2_0 + 0.187890812890811*G0_2_1 + 0.323589733311952*G0_2_2;
    A[2275] = -0.0228339529554804*G0_0_0 - 0.0965550010688897*G0_0_1 - 0.0596944770121849*G0_0_2 - 0.0965550010688903*G0_1_0 + 0.0391439193522512*G0_1_1 - 0.0287055408583188*G0_1_2 - 0.0596944770121856*G0_2_0 - 0.0287055408583198*G0_2_1 - 0.0965550010688895*G0_2_2;
    A[2276] = 0.0300103381700606*G0_0_1 + 0.0150051690850303*G0_0_2 + 0.0300103381700612*G0_1_0 - 0.0456679059109606*G0_1_1 - 0.00782878387045017*G0_1_2 + 0.015005169085031*G0_2_0 - 0.00782878387044949*G0_2_1 + 0.0300103381700602*G0_2_2;
    A[2277] = -0.0880738185425677*G0_0_0 - 0.134067923781464*G0_0_1 - 0.097207399724759*G0_0_2 - 0.134067923781464*G0_1_0 - 0.15657567740901*G0_1_1 - 0.22442513761958*G0_1_2 - 0.0972073997247593*G0_2_0 - 0.22442513761958*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2278] = 0.0127217737894823*G0_0_1 + 0.0120693751336113*G0_0_2 + 0.0127217737894827*G0_1_0 + 0.0391439193522536*G0_1_1 + 0.0446893079271557*G0_1_2 + 0.0120693751336108*G0_2_0 + 0.0446893079271557*G0_2_1 + 0.0502346965020579*G0_2_2;
    A[2279] = 0.0528442911255404*G0_0_1 + 0.0378391220405102*G0_0_2 + 0.0528442911255412*G0_1_0 + 0.0521918924696694*G0_1_1 + 0.0900310145101806*G0_1_2 + 0.0378391220405106*G0_2_0 + 0.0900310145101795*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2280] = 0.647179466623904*G0_0_0 + 0.375781625781622*G0_0_1 + 0.375781625781621*G0_0_2 + 0.375781625781622*G0_1_0 + 0.647179466623906*G0_1_1 + 0.375781625781623*G0_1_2 + 0.375781625781621*G0_2_0 + 0.375781625781623*G0_2_1 + 0.647179466623904*G0_2_2;
    A[2281] = -0.5845491956603*G0_0_0 - 0.194414799449519*G0_0_1 - 0.321632537344339*G0_0_2 - 0.448850275239161*G0_1_0 - 0.268135847562928*G0_1_1 - 0.290643601190473*G0_1_2 - 0.32163253734434*G0_2_0 - 0.222141742324031*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2282] = 0.255740273101381*G0_0_0 + 0.0756782440810214*G0_0_1 + 0.127870136550691*G0_0_2 + 0.18006202902036*G0_1_0 + 0.105688582251082*G0_1_1 + 0.105036183595211*G0_1_2 + 0.127870136550691*G0_2_0 + 0.0528442911255408*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2283] = -0.268135847562928*G0_0_0 - 0.222141742324032*G0_0_1 - 0.321632537344339*G0_0_2 - 0.290643601190473*G0_1_0 - 0.268135847562928*G0_1_1 - 0.448850275239161*G0_1_2 - 0.321632537344338*G0_2_0 - 0.194414799449518*G0_2_1 - 0.584549195660301*G0_2_2;
    A[2284] = 0.17223324514991*G0_0_0 + 0.10079559233205*G0_0_1 + 0.166035457919136*G0_0_2 + 0.198655390712681*G0_1_0 + 0.113517366121532*G0_1_1 + 0.19865539071268*G0_1_2 + 0.166035457919136*G0_2_0 + 0.10079559233205*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2285] = 0.105688582251081*G0_0_0 + 0.0528442911255407*G0_0_1 + 0.127870136550691*G0_0_2 + 0.10503618359521*G0_1_0 + 0.105688582251081*G0_1_1 + 0.180062029020361*G0_1_2 + 0.12787013655069*G0_2_0 + 0.0756782440810209*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2286] = 0.323589733311952*G0_0_0 + 0.187890812890811*G0_0_1 + 0.187890812890811*G0_0_2 + 0.187890812890811*G0_1_0 + 0.0521918924696703*G0_1_1 + 0.323589733311954*G0_1_2 + 0.187890812890811*G0_2_0 + 0.323589733311953*G0_2_1 + 0.0521918924696707*G0_2_2;
    A[2287] = -0.29227459783015*G0_0_0 - 0.224425137619579*G0_0_1 - 0.0972073997247588*G0_0_2 - 0.22442513761958*G0_1_0 - 0.156575677409008*G0_1_1 - 0.134067923781464*G0_1_2 - 0.0972073997247602*G0_2_0 - 0.134067923781463*G0_2_1 - 0.0880738185425674*G0_2_2;
    A[2288] = 0.12787013655069*G0_0_0 + 0.0900310145101802*G0_0_1 + 0.0378391220405102*G0_0_2 + 0.09003101451018*G0_1_0 + 0.0521918924696699*G0_1_1 + 0.0528442911255407*G0_1_2 + 0.0378391220405104*G0_2_0 + 0.0528442911255409*G0_2_1;
    A[2289] = -0.0965550010688889*G0_0_0 - 0.0287055408583188*G0_0_1 - 0.0596944770121844*G0_0_2 - 0.0287055408583186*G0_1_0 + 0.0391439193522509*G0_1_1 - 0.0965550010688898*G0_1_2 - 0.0596944770121845*G0_2_0 - 0.0965550010688899*G0_2_1 - 0.0228339529554811*G0_2_2;
    A[2290] = 0.0502346965020576*G0_0_0 + 0.0446893079271543*G0_0_1 + 0.0120693751336117*G0_0_2 + 0.0446893079271555*G0_1_0 + 0.0391439193522522*G0_1_1 + 0.0127217737894819*G0_1_2 + 0.0120693751336117*G0_2_0 + 0.0127217737894817*G0_2_1;
    A[2291] = 0.0300103381700598*G0_0_0 - 0.00782878387045025*G0_0_1 + 0.0150051690850302*G0_0_2 - 0.00782878387045067*G0_1_0 - 0.0456679059109605*G0_1_1 + 0.0300103381700607*G0_1_2 + 0.0150051690850296*G0_2_0 + 0.0300103381700607*G0_2_1;
    A[2292] = -0.150051690850299*G0_0_0 - 0.466465038947672*G0_0_1 - 0.335985307773498*G0_0_2 - 0.466465038947674*G0_1_0 - 0.782878387045047*G0_1_1 - 0.466465038947674*G0_1_2 - 0.335985307773498*G0_2_0 - 0.466465038947674*G0_2_1 - 0.150051690850299*G0_2_2;
    A[2293] = 0.143527704291591*G0_0_0 + 0.218553549716742*G0_0_1 + 0.185933616923198*G0_0_2 + 0.218553549716741*G0_1_0 + 0.293579395141891*G0_1_1 + 0.234863516113514*G0_1_2 + 0.185933616923197*G0_2_0 + 0.234863516113513*G0_2_1 + 0.176147637085135*G0_2_2;
    A[2294] = 0.0456679059109603*G0_0_0 + 0.120693751336109*G0_0_1 + 0.0717638521457944*G0_0_2 + 0.120693751336112*G0_1_0 + 0.195719596761261*G0_1_1 + 0.120693751336112*G0_1_2 + 0.0717638521457959*G0_2_0 + 0.120693751336111*G0_2_1 + 0.0456679059109599*G0_2_2;
    A[2295] = 0.176147637085136*G0_0_0 + 0.234863516113514*G0_0_1 + 0.185933616923199*G0_0_2 + 0.234863516113515*G0_1_0 + 0.293579395141893*G0_1_1 + 0.218553549716742*G0_1_2 + 0.185933616923198*G0_2_0 + 0.218553549716739*G0_2_1 + 0.143527704291591*G0_2_2;
    A[2296] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0298472385060924*G0_0_2 - 0.034571909571909*G0_1_0 - 0.034571909571909*G0_1_1 - 0.034571909571909*G0_1_2 - 0.0298472385060922*G0_2_0 - 0.0298472385060922*G0_2_1 - 0.0298472385060922*G0_2_2;
    A[2297] = -0.0298472385060925*G0_0_0 + 0.00472467106581659*G0_1_0;
    A[2298] = -0.00946499969937485*G0_1_1;
    A[2299] = 0.00227817610630107*G0_1_2 + 0.0117431758056757*G0_2_2;
    A[2300] = 0.00658922642429577*G0_1_1 + 0.00329461321214803*G0_1_2 + 0.00329461321214757*G0_2_1;
    A[2301] = 0.00450155072550928*G0_1_1 + 0.00192457603481902*G0_1_2 + 0.00192457603481932*G0_2_1 - 0.000652398655870568*G0_2_2;
    A[2302] = -0.0105036183595214*G0_1_1 - 0.0130805930502108*G0_1_2 - 0.0130805930502113*G0_2_1 - 0.0156575677409012*G0_2_2;
    A[2303] = -0.00254435475789603*G0_1_1 - 0.0058389679700442*G0_1_2 - 0.00583896797004424*G0_2_1 - 0.00913358118219224*G0_2_2;
    A[2304] = 0.128522535206562*G0_0_0 + 0.0422754329004326*G0_0_2 + 0.0845508658008648*G0_1_0 + 0.0429604514890966*G0_1_2 + 0.0422754329004324*G0_2_0;
    A[2305] = 0.0371867233846392*G0_0_0 + 0.0743408268364859*G0_0_2 + 0.0808321934624008*G0_1_0 + 0.108232937008977*G0_1_2 + 0.0743408268364854*G0_2_0 + 0.104709984267274*G0_2_2;
    A[2306] = -0.00619778723077259*G0_0_0 + 0.0188869410874618*G0_0_2 + 0.00939454064454019*G0_1_0 + 0.0377738821749233*G0_1_2 + 0.0188869410874618*G0_2_0 + 0.0567586830607662*G0_2_2;
    A[2307] = -0.00583896797004377*G0_0_2 - 0.00254435475789622*G0_1_0 - 0.0142875305635719*G0_1_2 - 0.00583896797004444*G0_2_0 - 0.043547610279381*G0_2_2;
    A[2308] = 0.0642612676032807*G0_0_0 + 0.0422754329004324*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.0429604514890966*G0_1_1;
    A[2309] = 0.0088073818542568*G0_0_0 + 0.00649136662591478*G0_0_1 + 0.00649136662591517*G0_1_0 + 0.00352295274170282*G0_1_1;
    A[2310] = -0.000652398655870646*G0_0_0 - 0.00949240044292028*G0_0_1 - 0.00949240044292101*G0_1_0 - 0.0189848008858426*G0_1_1;
    A[2311] = 0.00329461321214755*G0_0_1 + 0.00329461321214795*G0_1_0 + 0.029260079715809*G0_1_1;
    A[2312] = 0.128522535206562*G0_0_0 + 0.128522535206562*G0_0_1 + 0.0862471023061294*G0_0_2 + 0.0439716694056962*G0_1_0 + 0.0439716694056962*G0_1_1 + 0.044656687994361*G0_1_2 + 0.0862471023061288*G0_2_0 + 0.0862471023061288*G0_2_1 + 0.0439716694056966*G0_2_2;
    A[2313] = 0.0371867233846391*G0_0_0 + 0.0371867233846391*G0_0_1 - 0.0371541034518464*G0_0_2 - 0.0436454700777609*G0_1_0 - 0.0436454700777609*G0_1_1 - 0.0097533599052691*G0_1_2 - 0.037154103451846*G0_2_0 - 0.0371541034518459*G0_2_1 - 0.00678494602105716*G0_2_2;
    A[2314] = -0.00619778723077315*G0_0_0 - 0.0061977872307732*G0_0_1 - 0.0250847283182351*G0_0_2 - 0.015592327875314*G0_1_0 - 0.015592327875314*G0_1_1 + 0.00329461321214734*G0_1_2 - 0.0250847283182353*G0_2_0 - 0.0250847283182354*G0_2_1 + 0.0127870136550691*G0_2_2;
    A[2315] = 0.00583896797004443*G0_0_2 + 0.0025443547578962*G0_1_0 + 0.00254435475789621*G0_1_1 - 0.00590420783563121*G0_1_2 + 0.00583896797004442*G0_2_0 + 0.00583896797004441*G0_2_1 - 0.0318696743392921*G0_2_2;
    A[2316] = 0.0642612676032806*G0_0_0 + 0.0219858347028481*G0_0_1 + 0.0642612676032807*G0_0_2 + 0.0219858347028479*G0_1_0 + 0.0226708532915127*G0_1_1 + 0.021985834702848*G0_1_2 + 0.0642612676032805*G0_2_0 + 0.0219858347028481*G0_2_1 + 0.0642612676032803*G0_2_2;
    A[2317] = 0.00880738185425632*G0_0_0 + 0.0023160152283417*G0_0_1 + 0.00880738185425614*G0_0_2 + 0.00231601522834134*G0_1_0 - 0.000652398655870777*G0_1_1 + 0.00231601522834126*G0_1_2 + 0.00880738185425653*G0_2_0 + 0.00231601522834169*G0_2_1 + 0.00880738185425647*G0_2_2;
    A[2318] = -0.000652398655871128*G0_0_0 + 0.00884000178705048*G0_0_1 - 0.000652398655870918*G0_0_2 + 0.00884000178705031*G0_1_0 - 0.000652398655871366*G0_1_1 + 0.00884000178705047*G0_1_2 - 0.000652398655871379*G0_2_0 + 0.00884000178705014*G0_2_1 - 0.000652398655871238*G0_2_2;
    A[2319] = -0.00329461321214806*G0_0_1 - 0.00329461321214791*G0_1_0 + 0.0226708532915132*G0_1_1 - 0.00329461321214789*G0_1_2 - 0.00329461321214786*G0_2_1;
    A[2320] = 0.0920860702761737*G0_0_0 + 0.134361503176606*G0_0_1 + 0.134361503176606*G0_0_2 + 0.128000616281865*G0_1_0 + 0.1273155976932*G0_1_1 + 0.1273155976932*G0_1_2 + 0.134361503176606*G0_2_0 + 0.176636936077038*G0_2_1 + 0.176636936077038*G0_2_2;
    A[2321] = -0.0622388317700808*G0_0_0 - 0.250814663249555*G0_0_1 - 0.250814663249555*G0_0_2 - 0.160783648739375*G0_1_0 - 0.166948816037355*G0_1_1 - 0.166948816037355*G0_1_2 - 0.250814663249556*G0_2_0 - 0.334680510461756*G0_2_1 - 0.334680510461756*G0_2_2;
    A[2322] = -0.0622388317700817*G0_0_0 + 0.188575831479474*G0_0_1 + 0.188575831479474*G0_0_2 + 0.0985448169692941*G0_1_0 + 0.182410664181495*G0_1_1 + 0.182410664181495*G0_1_2 + 0.188575831479475*G0_2_0 + 0.104709984267274*G0_2_1 + 0.104709984267273*G0_2_2;
    A[2323] = 0.0920860702761736*G0_0_0 - 0.0422754329004321*G0_0_1 - 0.0422754329004321*G0_0_2 - 0.0359145460056908*G0_1_0 - 0.0429604514890963*G0_1_1 - 0.0429604514890961*G0_1_2 - 0.0422754329004321*G0_2_0;
    A[2324] = 0.0391439193522532*G0_0_0 + 0.135698920421142*G0_0_1 + 0.0678494602105707*G0_0_2 + 0.13569892042114*G0_1_0 + 0.209419968534549*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0678494602105699*G0_2_0 + 0.104709984267274*G0_2_1;
    A[2325] = -0.00489298991903097*G0_0_0 + 0.0283793415303841*G0_0_1 + 0.0141896707651925*G0_0_2 + 0.0283793415303833*G0_1_0 + 0.0567586830607663*G0_1_1 + 0.0283793415303838*G0_1_2 + 0.0141896707651916*G0_2_0 + 0.0283793415303838*G0_2_1;
    A[2326] = -0.00130479731174118*G0_0_1 - 0.000652398655870782*G0_0_2 - 0.00130479731174149*G0_1_0 - 0.0254435475789641*G0_1_1 - 0.012721773789482*G0_1_2 - 0.000652398655870718*G0_2_0 - 0.0127217737894819*G0_2_1;
    A[2327] = 0.0195719596761274*G0_0_0 + 0.0748627457611833*G0_0_1 + 0.0606730749959923*G0_0_2 + 0.0748627457611826*G0_1_0 + 0.149725491522365*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0606730749959918*G0_2_0 + 0.121346149991982*G0_2_1 + 0.0929668084615994*G0_2_2;
    A[2328] = -0.0181040627004168*G0_0_1 - 0.0102752788299659*G0_0_2 - 0.0181040627004168*G0_1_0 - 0.0166361657247073*G0_1_1 + 0.00146789697570914*G0_1_2 - 0.010275278829966*G0_2_0 + 0.00146789697570993*G0_2_1 + 0.0195719596761268*G0_2_2;
    A[2329] = -0.0061977872307718*G0_0_1 - 0.00554538857490162*G0_0_2 - 0.00619778723077337*G0_1_0 + 0.0137003717732881*G0_1_1 + 0.0264221455627702*G0_1_2 - 0.00554538857490308*G0_2_0 + 0.02642214556277*G0_2_1 + 0.0391439193522521*G0_2_2;
    A[2330] = 0.0391439193522505*G0_0_0 - 0.096555001068889*G0_0_1 - 0.0287055408583182*G0_0_2 - 0.09655500106889*G0_1_0 - 0.0228339529554807*G0_1_1 - 0.0596944770121841*G0_1_2 - 0.0287055408583186*G0_2_0 - 0.0596944770121847*G0_2_1 - 0.096555001068888*G0_2_2;
    A[2331] = -0.00489298991903157*G0_0_0 - 0.033272331449414*G0_0_1 - 0.0190826606842223*G0_0_2 - 0.0332723314494139*G0_1_0 - 0.00489298991903011*G0_1_1 - 0.0190826606842226*G0_1_2 - 0.0190826606842232*G0_2_0 - 0.0190826606842228*G0_2_1 - 0.033272331449414*G0_2_2;
    A[2332] = 0.00130479731174173*G0_0_1 + 0.000652398655870631*G0_0_2 + 0.00130479731174128*G0_1_0 - 0.0228339529554814*G0_1_1 - 0.0107645778218697*G0_1_2 + 0.000652398655870588*G0_2_0 - 0.0107645778218696*G0_2_1 + 0.00130479731174155*G0_2_2;
    A[2333] = 0.0195719596761274*G0_0_0 - 0.0552907860850558*G0_0_1 - 0.0411011153198651*G0_0_2 - 0.0552907860850566*G0_1_0 + 0.0195719596761258*G0_1_1 + 0.00538228891093436*G0_1_2 - 0.0411011153198652*G0_2_0 + 0.00538228891093444*G0_2_1 - 0.00880738185425805*G0_2_2;
    A[2334] = 0.018104062700416*G0_0_1 + 0.0102752788299662*G0_0_2 + 0.0181040627004166*G0_1_0 + 0.0195719596761254*G0_1_1 + 0.0298472385060919*G0_1_2 + 0.0102752788299659*G0_2_0 + 0.0298472385060918*G0_2_1 + 0.0401225173360586*G0_2_2;
    A[2335] = 0.0061977872307732*G0_0_1 + 0.00554538857490219*G0_0_2 + 0.00619778723077334*G0_1_0 + 0.0260959462348351*G0_1_1 + 0.038165321368446*G0_1_2 + 0.00554538857490312*G0_2_0 + 0.0381653213684465*G0_2_1 + 0.0502346965020579*G0_2_2;
    A[2336] = -0.5845491956603*G0_0_0 - 0.44885027523916*G0_0_1 - 0.32163253734434*G0_0_2 - 0.194414799449519*G0_1_0 - 0.268135847562928*G0_1_1 - 0.222141742324031*G0_1_2 - 0.321632537344339*G0_2_0 - 0.290643601190473*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2337] = 0.912053320907477*G0_0_0 + 0.456026660453739*G0_0_1 + 0.456026660453738*G0_0_2 + 0.456026660453739*G0_1_0 + 0.726119703984277*G0_1_1 + 0.456026660453737*G0_1_2 + 0.456026660453738*G0_2_0 + 0.456026660453737*G0_2_1 + 0.726119703984278*G0_2_2;
    A[2338] = -0.584549195660299*G0_0_0 - 0.135698920421142*G0_0_1 - 0.262916658315961*G0_0_2 - 0.39013439621078*G0_1_0 - 0.209419968534548*G0_1_1 - 0.290643601190472*G0_1_2 - 0.26291665831596*G0_2_0 - 0.104709984267274*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2339] = -0.0371867233846387*G0_0_0 + 0.0376760223765444*G0_0_1 + 0.186912214907006*G0_0_2 + 0.140428810676204*G0_1_0 + 0.0655660649150215*G0_1_1 + 0.037676022376543*G0_1_2 + 0.186912214907004*G0_2_0 + 0.140428810676205*G0_2_1 - 0.0371867233846404*G0_2_2;
    A[2340] = -0.0371867233846399*G0_0_0 - 0.0748627457611832*G0_0_1 - 0.224098938291644*G0_0_2 - 0.177615534060843*G0_1_0 - 0.149725491522365*G0_1_1 - 0.326851726591305*G0_1_2 - 0.224098938291644*G0_2_0 - 0.121346149991982*G0_2_1 - 0.448197876583289*G0_2_2;
    A[2341] = 0.0123955744615456*G0_0_0 + 0.00619778723077206*G0_0_1 + 0.00619778723077196*G0_0_2 + 0.00619778723077381*G0_1_0 - 0.013700371773288*G0_1_1 + 0.0714376528178603*G0_1_2 + 0.00619778723077359*G0_2_0 - 0.0264221455627699*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2342] = -0.29227459783015*G0_0_0 - 0.0972073997247604*G0_0_1 - 0.22442513761958*G0_0_2 - 0.0972073997247594*G0_1_0 - 0.0880738185425683*G0_1_1 - 0.134067923781465*G0_1_2 - 0.224425137619579*G0_2_0 - 0.134067923781464*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2343] = 0.456026660453738*G0_0_0 + 0.228013330226868*G0_0_1 + 0.228013330226869*G0_0_2 + 0.22801333022687*G0_1_0 + 0.0929668084615977*G0_1_1 + 0.363059851992139*G0_1_2 + 0.228013330226869*G0_2_0 + 0.36305985199214*G0_2_1 + 0.0929668084615979*G0_2_2;
    A[2344] = -0.292274597830149*G0_0_0 - 0.19506719810539*G0_0_1 - 0.0678494602105708*G0_0_2 - 0.195067198105391*G0_1_0 - 0.185933616923198*G0_1_1 - 0.104709984267274*G0_1_2 - 0.06784946021057*G0_2_0 - 0.104709984267274*G0_2_1;
    A[2345] = -0.00880738185425609*G0_0_0 - 0.041101115319864*G0_0_1 + 0.00538228891093477*G0_0_2 - 0.0411011153198646*G0_1_0 + 0.0195719596761283*G0_1_1 - 0.0552907860850558*G0_1_2 + 0.00538228891093554*G0_2_0 - 0.0552907860850559*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2346] = -0.00880738185425723*G0_0_0 + 0.0322937334656082*G0_0_1 - 0.0141896707651918*G0_0_2 + 0.032293733465608*G0_1_0 + 0.0929668084615991*G0_1_1 - 0.0283793415303831*G0_1_2 - 0.0141896707651923*G0_2_0 - 0.0283793415303831*G0_2_1;
    A[2347] = 0.00130479731174173*G0_0_0 + 0.000652398655870731*G0_0_1 + 0.000652398655870295*G0_0_2 + 0.000652398655870554*G0_1_0 - 0.0880738185425688*G0_1_1 + 0.0127217737894813*G0_1_2 + 0.000652398655871161*G0_2_0 + 0.0127217737894822*G0_2_1;
    A[2348] = -0.039143919352251*G0_0_0 + 0.445262082631869*G0_0_1 + 0.249542485870609*G0_0_2 + 0.445262082631867*G0_1_0 + 0.146789697570944*G0_1_1 + 0.249542485870606*G0_1_2 + 0.249542485870606*G0_2_0 + 0.249542485870607*G0_2_1 + 0.352295274170269*G0_2_2;
    A[2349] = -0.0391439193522528*G0_0_0 - 0.484406001984123*G0_0_1 - 0.288686405222862*G0_0_2 - 0.484406001984118*G0_1_0 - 0.782878387045045*G0_1_1 - 0.484406001984121*G0_1_2 - 0.288686405222857*G0_2_0 - 0.48440600198412*G0_2_1 - 0.185933616923197*G0_2_2;
    A[2350] = 0.00978597983806276*G0_0_0 + 0.00489298991903238*G0_0_1 + 0.00489298991903179*G0_0_2 + 0.00489298991903101*G0_1_0 + 0.146789697570948*G0_1_1 + 0.0538228891093475*G0_1_2 + 0.00489298991903186*G0_2_0 + 0.0538228891093487*G0_2_1 - 0.0391439193522522*G0_2_2;
    A[2351] = -0.0391439193522548*G0_0_0 - 0.0195719596761282*G0_0_1 - 0.019571959676128*G0_0_2 - 0.0195719596761271*G0_1_0 - 0.0489298991903143*G0_1_1 - 0.141896707651913*G0_1_2 - 0.0195719596761272*G0_2_0 - 0.141896707651915*G0_2_1 - 0.234863516113513*G0_2_2;
    A[2352] = 0.0189195610202551*G0_0_0 + 0.018919561020255*G0_0_1 + 0.0189195610202551*G0_0_2 + 0.0137421252872638*G0_1_0 + 0.0137421252872638*G0_1_1 + 0.0137421252872638*G0_1_2 + 0.0189195610202551*G0_2_0 + 0.0189195610202551*G0_2_1 + 0.0189195610202551*G0_2_2;
    A[2353] = 0.0335985307773501*G0_0_0 - 0.0143945239431346*G0_1_0;
    A[2354] = -0.0137421252872641*G0_1_1;
    A[2355] = 0.00517743573299126*G0_1_2 + 0.0189195610202552*G0_2_2;
    A[2356] = 0.0187890812890814*G0_1_1 + 0.00939454064454052*G0_1_2 + 0.00939454064454071*G0_2_1;
    A[2357] = -0.0148746893538565*G0_1_1 - 0.0149399292194429*G0_1_2 - 0.0149399292194435*G0_2_1 - 0.0150051690850292*G0_2_2;
    A[2358] = -0.0011743175805671*G0_1_1 - 0.00123955744615509*G0_1_2 - 0.0012395574461541*G0_2_1 - 0.00130479731174274*G0_2_2;
    A[2359] = -0.0118736555368503*G0_1_1 - 0.0212681961813904*G0_1_2 - 0.0212681961813909*G0_2_1 - 0.0306627368259309*G0_2_2;
    A[2360] = -0.257045070413124*G0_0_0 - 0.0962940416065407*G0_0_2 - 0.192588083213081*G0_1_0 - 0.12473862300251*G0_1_2 - 0.0962940416065409*G0_2_0;
    A[2361] = 0.121346149991983*G0_0_0 - 0.0733296089198859*G0_0_2 - 0.0109602974186314*G0_1_0 - 0.146659217839773*G0_1_2 - 0.0733296089198859*G0_2_0 - 0.271397840842283*G0_2_2;
    A[2362] = -0.0528442911255411*G0_0_0 + 0.0359471659384854*G0_0_2 - 0.00182671623643766*G0_1_0 + 0.0823327103709051*G0_1_2 + 0.0359471659384853*G0_2_0 + 0.209419968534551*G0_2_2;
    A[2363] = -0.0212681961813912*G0_0_2 - 0.0118736555368502*G0_1_0 - 0.0477555816097483*G0_1_2 - 0.0212681961813909*G0_2_0 - 0.111886369481855*G0_2_2;
    A[2364] = -0.128522535206562*G0_0_0 - 0.0962940416065406*G0_0_1 - 0.0962940416065402*G0_1_0 - 0.12473862300251*G0_1_1;
    A[2365] = 0.0476251018785739*G0_0_0 + 0.0623693115012557*G0_0_1 + 0.0623693115012553*G0_1_0 + 0.124738623002511*G0_1_1;
    A[2366] = -0.0150051690850301*G0_0_0 - 0.037773882174924*G0_0_1 - 0.0377738821749238*G0_1_0 - 0.127087258163646*G0_1_1;
    A[2367] = 0.00939454064454087*G0_0_1 + 0.00939454064454073*G0_1_0 + 0.0641307878721067*G0_1_1;
    A[2368] = -0.0613254736518619*G0_0_0 - 0.0613254736518619*G0_0_1 - 0.0400572774704715*G0_0_2 - 0.0187890812890809*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.00939454064454036*G0_1_2 - 0.0400572774704717*G0_2_0 - 0.0400572774704717*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[2369] = -0.0254435475789637*G0_0_0 - 0.0254435475789637*G0_0_1 + 0.0136351319077011*G0_0_2 + 0.0148746893538558*G0_1_0 + 0.0148746893538557*G0_1_1 - 6.52398655874386e-05*G0_1_2 + 0.0136351319077015*G0_2_0 + 0.0136351319077016*G0_2_1 - 0.000130479731174179*G0_2_2;
    A[2370] = -0.0528442911255409*G0_0_0 - 0.0528442911255409*G0_0_1 - 0.0137656116388754*G0_0_2 + 0.00117431758056782*G0_1_0 + 0.00117431758056786*G0_1_1 - 6.52398655867865e-05*G0_1_2 - 0.0137656116388759*G0_2_0 - 0.0137656116388759*G0_2_1 - 0.000130479731174205*G0_2_2;
    A[2371] = 0.0212681961813905*G0_0_2 + 0.0118736555368503*G0_1_0 + 0.0118736555368503*G0_1_1 - 0.00939454064454056*G0_1_2 + 0.0212681961813909*G0_2_0 + 0.0212681961813909*G0_2_1 - 0.0187890812890809*G0_2_2;
    A[2372] = -0.0306627368259309*G0_0_0 - 0.0093945406445402*G0_0_1 - 0.0306627368259309*G0_0_2 - 0.00939454064454047*G0_1_0 - 0.00939454064454049*G0_1_2 - 0.0306627368259311*G0_2_0 - 0.0093945406445405*G0_2_1 - 0.0306627368259312*G0_2_2;
    A[2373] = -0.00130479731174154*G0_0_0 - 6.52398655872028e-05*G0_0_1 - 0.00130479731174131*G0_0_2 - 6.52398655875272e-05*G0_1_0 - 6.52398655872805e-05*G0_1_2 - 0.00130479731174141*G0_2_0 - 6.52398655870398e-05*G0_2_1 - 0.00130479731174124*G0_2_2;
    A[2374] = -0.0150051690850298*G0_0_0 - 6.52398655871638e-05*G0_0_1 - 0.01500516908503*G0_0_2 - 6.52398655867524e-05*G0_1_0 - 6.52398655870468e-05*G0_1_2 - 0.0150051690850301*G0_2_0 - 6.52398655868936e-05*G0_2_1 - 0.0150051690850303*G0_2_2;
    A[2375] = -0.00939454064454048*G0_0_1 - 0.00939454064454082*G0_1_0 - 0.0093945406445407*G0_1_2 - 0.00939454064454071*G0_2_1;
    A[2376] = -0.0693499771190732*G0_0_0 - 0.0906181733004638*G0_0_1 - 0.0906181733004639*G0_0_2 - 0.0547362472275655*G0_1_0 - 0.0641307878721061*G0_1_1 - 0.0641307878721061*G0_1_2 - 0.0906181733004635*G0_2_0 - 0.111886369481854*G0_2_1 - 0.111886369481854*G0_2_2;
    A[2377] = 0.0846813455320382*G0_0_0 + 0.173472802596064*G0_0_1 + 0.173472802596064*G0_0_2 + 0.0893133759887211*G0_1_0 + 0.127087258163645*G0_1_1 + 0.127087258163645*G0_1_2 + 0.173472802596063*G0_2_0 + 0.209419968534549*G0_2_1 + 0.209419968534549*G0_2_2;
    A[2378] = -0.00339247301052779*G0_0_0 - 0.198068231922396*G0_0_1 - 0.198068231922396*G0_0_2 - 0.0623693115012539*G0_1_0 - 0.12473862300251*G0_1_1 - 0.12473862300251*G0_1_2 - 0.198068231922395*G0_2_0 - 0.271397840842282*G0_2_1 - 0.271397840842282*G0_2_2;
    A[2379] = -0.0644569872000425*G0_0_0 + 0.0962940416065403*G0_0_1 + 0.0962940416065403*G0_0_2 + 0.0284445813959691*G0_1_0 + 0.124738623002509*G0_1_1 + 0.12473862300251*G0_1_2 + 0.0962940416065397*G0_2_0;
    A[2380] = 0.0521918924696699*G0_0_0 - 0.271397840842283*G0_0_1 - 0.135698920421141*G0_0_2 - 0.271397840842283*G0_1_0 - 0.542795681684566*G0_1_1 - 0.271397840842282*G0_1_2 - 0.135698920421143*G0_2_0 - 0.271397840842284*G0_2_1;
    A[2381] = -0.022833952955481*G0_0_0 + 0.0737210481134081*G0_0_1 + 0.0368605240567035*G0_0_2 + 0.0737210481134092*G0_1_0 + 0.209419968534551*G0_1_1 + 0.104709984267274*G0_1_2 + 0.0368605240567042*G0_2_0 + 0.104709984267274*G0_2_1;
    A[2382] = -0.03001033817006*G0_0_1 - 0.0150051690850295*G0_0_2 - 0.0300103381700597*G0_1_0 - 0.105688582251081*G0_1_1 - 0.0528442911255402*G0_1_2 - 0.0150051690850296*G0_2_0 - 0.0528442911255399*G0_2_1;
    A[2383] = -0.0880738185425689*G0_0_0 + 0.0459941052388964*G0_0_1 + 0.00913358118219138*G0_0_2 + 0.0459941052388978*G0_1_0 + 0.0234863516113513*G0_1_1 - 0.0812236326559235*G0_1_2 + 0.00913358118219228*G0_2_0 - 0.0812236326559236*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2384] = -0.0127217737894815*G0_0_1 - 0.0120693751336109*G0_0_2 - 0.0127217737894826*G0_1_0 + 0.0137003717732888*G0_1_1 + 0.0198981590040618*G0_1_2 - 0.0120693751336115*G0_2_0 + 0.0198981590040616*G0_2_1 + 0.0260959462348355*G0_2_2;
    A[2385] = -0.0528442911255419*G0_0_1 - 0.037839122040511*G0_0_2 - 0.0528442911255405*G0_1_0 - 0.0534966897814111*G0_1_1 - 0.000652398655869923*G0_1_2 - 0.0378391220405106*G0_2_0 - 0.000652398655871125*G0_2_1 + 0.05219189246967*G0_2_2;
    A[2386] = -0.0456679059109603*G0_0_0 + 0.0300103381700599*G0_0_1 - 0.00782878387045052*G0_0_2 + 0.0300103381700608*G0_1_0 + 0.0150051690850303*G0_1_2 - 0.00782878387044928*G0_2_0 + 0.0150051690850309*G0_2_1 + 0.0300103381700607*G0_2_2;
    A[2387] = -0.0228339529554811*G0_0_0 + 0.00130479731174167*G0_0_1 - 0.0107645778218704*G0_0_2 + 0.00130479731174022*G0_1_0 + 0.000652398655870836*G0_1_2 - 0.0107645778218694*G0_2_0 + 0.000652398655869667*G0_2_1 + 0.00130479731174148*G0_2_2;
    A[2388] = 0.03001033817006*G0_0_1 + 0.01500516908503*G0_0_2 + 0.0300103381700598*G0_1_0 + 0.01500516908503*G0_1_2 + 0.0150051690850297*G0_2_0 + 0.0150051690850304*G0_2_1 + 0.0300103381700598*G0_2_2;
    A[2389] = -0.0880738185425687*G0_0_0 + 0.0127217737894815*G0_0_1 + 0.000652398655871113*G0_0_2 + 0.0127217737894827*G0_1_0 + 0.000652398655871026*G0_1_2 + 0.000652398655870718*G0_2_0 + 0.000652398655870813*G0_2_1 + 0.00130479731174096*G0_2_2;
    A[2390] = 0.0127217737894823*G0_0_1 + 0.0120693751336115*G0_0_2 + 0.0127217737894826*G0_1_0 + 0.000652398655871755*G0_1_2 + 0.0120693751336115*G0_2_0 + 0.000652398655871411*G0_2_1 + 0.00130479731174234*G0_2_2;
    A[2391] = 0.0528442911255409*G0_0_1 + 0.037839122040511*G0_0_2 + 0.0528442911255407*G0_1_0 + 0.0150051690850302*G0_1_2 + 0.0378391220405107*G0_2_0 + 0.0150051690850302*G0_2_1 + 0.0300103381700605*G0_2_2;
    A[2392] = 0.255740273101381*G0_0_0 + 0.18006202902036*G0_0_1 + 0.127870136550691*G0_0_2 + 0.0756782440810214*G0_1_0 + 0.105688582251082*G0_1_1 + 0.0528442911255408*G0_1_2 + 0.127870136550691*G0_2_0 + 0.105036183595211*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2393] = -0.584549195660299*G0_0_0 - 0.39013439621078*G0_0_1 - 0.26291665831596*G0_0_2 - 0.135698920421142*G0_1_0 - 0.209419968534548*G0_1_1 - 0.104709984267274*G0_1_2 - 0.262916658315961*G0_2_0 - 0.290643601190472*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2394] = 0.647179466623904*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842282*G0_1_0 + 0.542795681684565*G0_1_1 + 0.271397840842283*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842283*G0_2_1 + 0.542795681684565*G0_2_2;
    A[2395] = 0.17223324514991*G0_0_0 + 0.0714376528178604*G0_0_1 + 0.00619778723077294*G0_0_2 - 0.02642214556277*G0_1_0 - 0.0137003717732873*G0_1_1 + 0.00619778723077286*G0_1_2 + 0.00619778723077339*G0_2_0 + 0.00619778723077324*G0_2_1 + 0.0123955744615455*G0_2_2;
    A[2396] = -0.26813584756293*G0_0_0 - 0.0459941052388969*G0_0_1 + 0.0534966897814103*G0_0_2 + 0.0225077536275453*G0_1_0 - 0.0234863516113519*G0_1_1 - 0.104709984267277*G0_1_2 + 0.0534966897814109*G0_2_0 + 0.0812236326559232*G0_2_1 - 0.209419968534552*G0_2_2;
    A[2397] = 0.105688582251082*G0_0_0 + 0.0528442911255418*G0_0_1 - 0.0221815542996085*G0_0_2 + 0.000652398655869981*G0_1_0 + 0.0534966897814109*G0_1_1 + 0.0528442911255413*G0_1_2 - 0.0221815542996093*G0_2_0 + 0.000652398655870994*G0_2_1 + 0.105688582251083*G0_2_2;
    A[2398] = 0.127870136550691*G0_0_0 + 0.0378391220405109*G0_0_1 + 0.0900310145101799*G0_0_2 + 0.0378391220405112*G0_1_0 + 0.0528442911255413*G0_1_2 + 0.0900310145101807*G0_2_0 + 0.0528442911255413*G0_2_1 + 0.0521918924696701*G0_2_2;
    A[2399] = -0.29227459783015*G0_0_0 - 0.0678494602105698*G0_0_1 - 0.195067198105391*G0_0_2 - 0.0678494602105714*G0_1_0 - 0.104709984267274*G0_1_2 - 0.195067198105391*G0_2_0 - 0.104709984267274*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2400] = 0.323589733311952*G0_0_0 + 0.135698920421141*G0_0_1 + 0.135698920421141*G0_0_2 + 0.135698920421141*G0_1_0 + 0.271397840842281*G0_1_2 + 0.135698920421142*G0_2_0 + 0.271397840842283*G0_2_1;
    A[2401] = 0.0502346965020566*G0_0_0 + 0.00554538857490173*G0_0_1 + 0.0381653213684458*G0_0_2 + 0.00554538857490206*G0_1_0 + 0.00619778723077192*G0_1_2 + 0.0381653213684452*G0_2_0 + 0.00619778723077167*G0_2_1 + 0.0260959462348341*G0_2_2;
    A[2402] = -0.0965550010688888*G0_0_0 - 0.0678494602105701*G0_0_1 - 0.036860524056704*G0_0_2 - 0.0678494602105697*G0_1_0 - 0.104709984267274*G0_1_2 - 0.0368605240567039*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2403] = 0.03001033817006*G0_0_0 + 0.0378391220405104*G0_0_1 + 0.0150051690850301*G0_0_2 + 0.0378391220405107*G0_1_0 + 0.0528442911255408*G0_1_2 + 0.0150051690850306*G0_2_0 + 0.0528442911255419*G0_2_1;
    A[2404] = 0.143527704291592*G0_0_0 - 0.0750258454251507*G0_0_1 - 0.0424059126316068*G0_0_2 - 0.0750258454251487*G0_1_0 - 0.0260959462348343*G0_1_2 - 0.0424059126316064*G0_2_0 - 0.026095946234834*G0_2_1 - 0.0521918924696682*G0_2_2;
    A[2405] = -0.150051690850301*G0_0_0 + 0.316413348097373*G0_0_1 + 0.1859336169232*G0_0_2 + 0.316413348097371*G0_1_0 + 0.185933616923198*G0_1_2 + 0.185933616923199*G0_2_0 + 0.185933616923199*G0_2_1 + 0.371867233846398*G0_2_2;
    A[2406] = 0.0456679059109619*G0_0_0 - 0.0750258454251499*G0_0_1 - 0.0260959462348355*G0_0_2 - 0.0750258454251495*G0_1_0 - 0.0260959462348345*G0_1_2 - 0.0260959462348349*G0_2_0 - 0.0260959462348368*G0_2_1 - 0.0521918924696703*G0_2_2;
    A[2407] = 0.176147637085137*G0_0_0 - 0.0587158790283782*G0_0_1 - 0.00978597983806152*G0_0_2 - 0.0587158790283798*G0_1_0 - 0.0260959462348363*G0_1_2 - 0.00978597983806265*G0_2_0 - 0.0260959462348348*G0_2_1 - 0.0521918924696707*G0_2_2;
    A[2408] = -0.0298472385060925*G0_0_0 - 0.0298472385060925*G0_0_1 - 0.0298472385060925*G0_0_2 - 0.0345719095719096*G0_1_0 - 0.0345719095719097*G0_1_1 - 0.0345719095719096*G0_1_2 - 0.0298472385060924*G0_2_0 - 0.0298472385060926*G0_2_1 - 0.0298472385060924*G0_2_2;
    A[2409] = 0.0117431758056756*G0_0_0 + 0.0022781761063011*G0_1_0;
    A[2410] = -0.0094649996993747*G0_1_1;
    A[2411] = 0.00472467106581677*G0_1_2 - 0.0298472385060928*G0_2_2;
    A[2412] = 0.0292600797158091*G0_1_1 + 0.00329461321214811*G0_1_2 + 0.00329461321214845*G0_2_1;
    A[2413] = -0.0189848008858426*G0_1_1 - 0.00949240044292084*G0_1_2 - 0.00949240044292137*G0_2_1 - 0.000652398655870772*G0_2_2;
    A[2414] = 0.00352295274170325*G0_1_1 + 0.00649136662591538*G0_1_2 + 0.00649136662591539*G0_2_1 + 0.00880738185425707*G0_2_2;
    A[2415] = 0.0429604514890964*G0_1_1 + 0.0422754329004322*G0_1_2 + 0.0422754329004323*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[2416] = -0.0435476102793812*G0_0_0 - 0.00583896797004486*G0_0_2 - 0.0142875305635725*G0_1_0 - 0.00254435475789677*G0_1_2 - 0.00583896797004462*G0_2_0;
    A[2417] = 0.0567586830607662*G0_0_0 + 0.018886941087462*G0_0_2 + 0.0377738821749234*G0_1_0 + 0.00939454064454075*G0_1_2 + 0.0188869410874619*G0_2_0 - 0.00619778723077247*G0_2_2;
    A[2418] = 0.104709984267274*G0_0_0 + 0.0743408268364854*G0_0_2 + 0.108232937008977*G0_1_0 + 0.0808321934624004*G0_1_2 + 0.0743408268364857*G0_2_0 + 0.0371867233846386*G0_2_2;
    A[2419] = 0.0422754329004319*G0_0_2 + 0.0429604514890962*G0_1_0 + 0.0845508658008648*G0_1_2 + 0.0422754329004322*G0_2_0 + 0.128522535206562*G0_2_2;
    A[2420] = -0.00913358118219274*G0_0_0 - 0.00583896797004493*G0_0_1 - 0.00583896797004514*G0_1_0 - 0.00254435475789688*G0_1_1;
    A[2421] = -0.0156575677409001*G0_0_0 - 0.0130805930502101*G0_0_1 - 0.01308059305021*G0_1_0 - 0.0105036183595207*G0_1_1;
    A[2422] = -0.000652398655870979*G0_0_0 + 0.00192457603481845*G0_0_1 + 0.0019245760348186*G0_1_0 + 0.00450155072550883*G0_1_1;
    A[2423] = 0.00329461321214859*G0_0_1 + 0.00329461321214799*G0_1_0 + 0.00658922642429604*G0_1_1;
    A[2424] = 0.176636936077038*G0_0_0 + 0.176636936077038*G0_0_1 + 0.134361503176606*G0_0_2 + 0.1273155976932*G0_1_0 + 0.1273155976932*G0_1_1 + 0.128000616281865*G0_1_2 + 0.134361503176605*G0_2_0 + 0.134361503176605*G0_2_1 + 0.0920860702761731*G0_2_2;
    A[2425] = -0.334680510461756*G0_0_0 - 0.334680510461756*G0_0_1 - 0.250814663249556*G0_0_2 - 0.166948816037354*G0_1_0 - 0.166948816037355*G0_1_1 - 0.160783648739377*G0_1_2 - 0.250814663249556*G0_2_0 - 0.250814663249556*G0_2_1 - 0.0622388317700799*G0_2_2;
    A[2426] = 0.104709984267273*G0_0_0 + 0.104709984267273*G0_0_1 + 0.188575831479475*G0_0_2 + 0.182410664181494*G0_1_0 + 0.182410664181494*G0_1_1 + 0.098544816969295*G0_1_2 + 0.188575831479474*G0_2_0 + 0.188575831479474*G0_2_1 - 0.0622388317700831*G0_2_2;
    A[2427] = -0.0422754329004321*G0_0_2 - 0.0429604514890963*G0_1_0 - 0.0429604514890963*G0_1_1 - 0.0359145460056914*G0_1_2 - 0.0422754329004324*G0_2_0 - 0.0422754329004324*G0_2_1 + 0.0920860702761747*G0_2_2;
    A[2428] = 0.0642612676032804*G0_0_0 + 0.0219858347028486*G0_0_1 + 0.0642612676032802*G0_0_2 + 0.0219858347028479*G0_1_0 + 0.0226708532915135*G0_1_1 + 0.0219858347028478*G0_1_2 + 0.0642612676032807*G0_2_0 + 0.0219858347028486*G0_2_1 + 0.0642612676032806*G0_2_2;
    A[2429] = 0.0088073818542565*G0_0_0 + 0.00231601522834165*G0_0_1 + 0.00880738185425659*G0_0_2 + 0.00231601522834136*G0_1_0 - 0.000652398655871205*G0_1_1 + 0.00231601522834138*G0_1_2 + 0.00880738185425665*G0_2_0 + 0.0023160152283418*G0_2_1 + 0.0088073818542567*G0_2_2;
    A[2430] = -0.000652398655870971*G0_0_0 + 0.00884000178704965*G0_0_1 - 0.000652398655870979*G0_0_2 + 0.00884000178705036*G0_1_0 - 0.000652398655871112*G0_1_1 + 0.00884000178705045*G0_1_2 - 0.000652398655871002*G0_2_0 + 0.00884000178704954*G0_2_1 - 0.000652398655871091*G0_2_2;
    A[2431] = -0.00329461321214734*G0_0_1 - 0.00329461321214793*G0_1_0 + 0.022670853291513*G0_1_1 - 0.00329461321214795*G0_1_2 - 0.00329461321214736*G0_2_1;
    A[2432] = 0.0439716694056971*G0_0_0 + 0.0862471023061288*G0_0_1 + 0.0862471023061286*G0_0_2 + 0.0446566879943618*G0_1_0 + 0.0439716694056961*G0_1_1 + 0.043971669405696*G0_1_2 + 0.0862471023061295*G0_2_0 + 0.128522535206562*G0_2_1 + 0.128522535206561*G0_2_2;
    A[2433] = -0.00678494602105761*G0_0_0 - 0.037154103451846*G0_0_1 - 0.0371541034518459*G0_0_2 - 0.00975335990527013*G0_1_0 - 0.0436454700777613*G0_1_1 - 0.0436454700777611*G0_1_2 - 0.0371541034518462*G0_2_0 + 0.0371867233846404*G0_2_1 + 0.0371867233846404*G0_2_2;
    A[2434] = 0.0127870136550693*G0_0_0 - 0.0250847283182355*G0_0_1 - 0.0250847283182355*G0_0_2 + 0.00329461321214821*G0_1_0 - 0.0155923278753141*G0_1_1 - 0.015592327875314*G0_1_2 - 0.0250847283182352*G0_2_0 - 0.00619778723077343*G0_2_1 - 0.0061977872307735*G0_2_2;
    A[2435] = -0.031869674339292*G0_0_0 + 0.00583896797004499*G0_0_1 + 0.00583896797004495*G0_0_2 - 0.00590420783563148*G0_1_0 + 0.00254435475789693*G0_1_1 + 0.00254435475789691*G0_1_2 + 0.00583896797004439*G0_2_0;
    A[2436] = 0.0391439193522526*G0_0_0 + 0.0264221455627696*G0_0_1 - 0.00554538857490253*G0_0_2 + 0.0264221455627704*G0_1_0 + 0.0137003717732871*G0_1_1 - 0.00619778723077436*G0_1_2 - 0.00554538857490093*G0_2_0 - 0.00619778723077302*G0_2_1;
    A[2437] = 0.019571959676125*G0_0_0 + 0.00146789697570971*G0_0_1 - 0.0102752788299661*G0_0_2 + 0.00146789697570858*G0_1_0 - 0.0166361657247068*G0_1_1 - 0.018104062700416*G0_1_2 - 0.0102752788299675*G0_2_0 - 0.0181040627004161*G0_2_1;
    A[2438] = -0.012721773789482*G0_0_1 - 0.000652398655870307*G0_0_2 - 0.012721773789481*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.00130479731174192*G0_1_2 - 0.000652398655869344*G0_2_0 - 0.00130479731174137*G0_2_1;
    A[2439] = 0.092966808461598*G0_0_0 + 0.121346149991982*G0_0_1 + 0.0606730749959902*G0_0_2 + 0.121346149991981*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0748627457611824*G0_1_2 + 0.0606730749959901*G0_2_0 + 0.0748627457611828*G0_2_1 + 0.0195719596761266*G0_2_2;
    A[2440] = 0.0283793415303828*G0_0_1 + 0.014189670765191*G0_0_2 + 0.0283793415303832*G0_1_0 + 0.056758683060766*G0_1_1 + 0.0283793415303829*G0_1_2 + 0.0141896707651908*G0_2_0 + 0.0283793415303812*G0_2_1 - 0.00489298991903276*G0_2_2;
    A[2441] = 0.104709984267274*G0_0_1 + 0.0678494602105704*G0_0_2 + 0.104709984267275*G0_1_0 + 0.209419968534549*G0_1_1 + 0.135698920421141*G0_1_2 + 0.0678494602105713*G0_2_0 + 0.135698920421141*G0_2_1 + 0.0391439193522521*G0_2_2;
    A[2442] = -0.15657567740901*G0_0_0 - 0.134067923781465*G0_0_1 - 0.224425137619579*G0_0_2 - 0.134067923781465*G0_1_0 - 0.0880738185425681*G0_1_1 - 0.0972073997247583*G0_1_2 - 0.224425137619582*G0_2_0 - 0.0972073997247605*G0_2_1 - 0.292274597830151*G0_2_2;
    A[2443] = 0.0195719596761272*G0_0_0 - 0.0552907860850546*G0_0_1 + 0.00538228891093516*G0_0_2 - 0.0552907860850537*G0_1_0 + 0.0195719596761281*G0_1_1 - 0.0411011153198637*G0_1_2 + 0.00538228891093656*G0_2_0 - 0.0411011153198625*G0_2_1 - 0.00880738185425571*G0_2_2;
    A[2444] = 0.0127217737894804*G0_0_1 + 0.000652398655870281*G0_0_2 + 0.0127217737894808*G0_1_0 - 0.0880738185425696*G0_1_1 + 0.000652398655869992*G0_1_2 + 0.000652398655869278*G0_2_0 + 0.000652398655869031*G0_2_1 + 0.00130479731174133*G0_2_2;
    A[2445] = 0.0929668084615971*G0_0_0 + 0.363059851992138*G0_0_1 + 0.228013330226868*G0_0_2 + 0.363059851992138*G0_1_0 + 0.0929668084615975*G0_1_1 + 0.228013330226867*G0_1_2 + 0.228013330226868*G0_2_0 + 0.228013330226869*G0_2_1 + 0.456026660453739*G0_2_2;
    A[2446] = -0.028379341530383*G0_0_1 - 0.014189670765192*G0_0_2 - 0.0283793415303836*G0_1_0 + 0.092966808461599*G0_1_1 + 0.0322937334656068*G0_1_2 - 0.0141896707651911*G0_2_0 + 0.0322937334656078*G0_2_1 - 0.00880738185425783*G0_2_2;
    A[2447] = -0.104709984267274*G0_0_1 - 0.067849460210569*G0_0_2 - 0.104709984267274*G0_1_0 - 0.185933616923197*G0_1_1 - 0.195067198105389*G0_1_2 - 0.0678494602105712*G0_2_0 - 0.19506719810539*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2448] = -0.268135847562928*G0_0_0 - 0.290643601190473*G0_0_1 - 0.321632537344338*G0_0_2 - 0.222141742324032*G0_1_0 - 0.268135847562928*G0_1_1 - 0.194414799449518*G0_1_2 - 0.321632537344339*G0_2_0 - 0.448850275239161*G0_2_1 - 0.584549195660301*G0_2_2;
    A[2449] = -0.0371867233846387*G0_0_0 + 0.140428810676204*G0_0_1 + 0.186912214907004*G0_0_2 + 0.0376760223765444*G0_1_0 + 0.0655660649150215*G0_1_1 + 0.140428810676205*G0_1_2 + 0.186912214907006*G0_2_0 + 0.037676022376543*G0_2_1 - 0.0371867233846404*G0_2_2;
    A[2450] = 0.17223324514991*G0_0_0 - 0.02642214556277*G0_0_1 + 0.00619778723077339*G0_0_2 + 0.0714376528178604*G0_1_0 - 0.0137003717732873*G0_1_1 + 0.00619778723077324*G0_1_2 + 0.00619778723077294*G0_2_0 + 0.00619778723077285*G0_2_1 + 0.0123955744615455*G0_2_2;
    A[2451] = 0.726119703984278*G0_0_0 + 0.456026660453737*G0_0_1 + 0.456026660453737*G0_0_2 + 0.456026660453737*G0_1_0 + 0.726119703984278*G0_1_1 + 0.456026660453737*G0_1_2 + 0.456026660453737*G0_2_0 + 0.456026660453737*G0_2_1 + 0.912053320907479*G0_2_2;
    A[2452] = -0.448197876583289*G0_0_0 - 0.121346149991982*G0_0_1 - 0.224098938291644*G0_0_2 - 0.326851726591306*G0_1_0 - 0.149725491522364*G0_1_1 - 0.177615534060844*G0_1_2 - 0.224098938291644*G0_2_0 - 0.0748627457611826*G0_2_1 - 0.0371867233846381*G0_2_2;
    A[2453] = -0.209419968534547*G0_0_0 - 0.104709984267274*G0_0_1 - 0.262916658315959*G0_0_2 - 0.290643601190471*G0_1_0 - 0.209419968534549*G0_1_1 - 0.390134396210779*G0_1_2 - 0.26291665831596*G0_2_0 - 0.135698920421141*G0_2_1 - 0.584549195660301*G0_2_2;
    A[2454] = -0.0965550010688887*G0_0_0 - 0.0596944770121852*G0_0_1 - 0.0287055408583197*G0_0_2 - 0.0596944770121843*G0_1_0 - 0.0228339529554804*G0_1_1 - 0.0965550010688904*G0_1_2 - 0.0287055408583181*G0_2_0 - 0.0965550010688896*G0_2_1 + 0.0391439193522505*G0_2_2;
    A[2455] = -0.00880738185425669*G0_0_0 + 0.00538228891093502*G0_0_1 - 0.0411011153198656*G0_0_2 + 0.00538228891093491*G0_1_0 + 0.0195719596761266*G0_1_1 - 0.0552907860850567*G0_1_2 - 0.0411011153198647*G0_2_0 - 0.0552907860850569*G0_2_1 + 0.0195719596761268*G0_2_2;
    A[2456] = 0.0502346965020577*G0_0_0 + 0.0381653213684454*G0_0_1 + 0.00554538857490228*G0_0_2 + 0.0381653213684466*G0_1_0 + 0.0260959462348346*G0_1_1 + 0.00619778723077391*G0_1_2 + 0.00554538857490303*G0_2_0 + 0.00619778723077269*G0_2_1;
    A[2457] = -0.0332723314494136*G0_0_0 - 0.0190826606842214*G0_0_1 - 0.0190826606842218*G0_0_2 - 0.0190826606842223*G0_1_0 - 0.00489298991903047*G0_1_1 - 0.0332723314494128*G0_1_2 - 0.0190826606842225*G0_2_0 - 0.0332723314494128*G0_2_1 - 0.00489298991902992*G0_2_2;
    A[2458] = 0.0401225173360571*G0_0_0 + 0.0298472385060922*G0_0_1 + 0.0102752788299661*G0_0_2 + 0.0298472385060909*G0_1_0 + 0.0195719596761257*G0_1_1 + 0.0181040627004161*G0_1_2 + 0.0102752788299647*G0_2_0 + 0.0181040627004167*G0_2_1;
    A[2459] = 0.00130479731174195*G0_0_0 - 0.0107645778218705*G0_0_1 + 0.000652398655870156*G0_0_2 - 0.010764577821869*G0_1_0 - 0.022833952955481*G0_1_1 + 0.00130479731174168*G0_1_2 + 0.000652398655871386*G0_2_0 + 0.00130479731174066*G0_2_1;
    A[2460] = 0.352295274170268*G0_0_0 + 0.249542485870608*G0_0_1 + 0.249542485870607*G0_0_2 + 0.249542485870608*G0_1_0 + 0.146789697570948*G0_1_1 + 0.445262082631871*G0_1_2 + 0.24954248587061*G0_2_0 + 0.445262082631872*G0_2_1 - 0.0391439193522529*G0_2_2;
    A[2461] = -0.234863516113512*G0_0_0 - 0.141896707651913*G0_0_1 - 0.0195719596761249*G0_0_2 - 0.141896707651913*G0_1_0 - 0.0489298991903153*G0_1_1 - 0.0195719596761259*G0_1_2 - 0.0195719596761267*G0_2_0 - 0.0195719596761271*G0_2_1 - 0.0391439193522539*G0_2_2;
    A[2462] = -0.0391439193522518*G0_0_0 + 0.0538228891093483*G0_0_1 + 0.00489298991903152*G0_0_2 + 0.0538228891093454*G0_1_0 + 0.146789697570946*G0_1_1 + 0.0048929899190307*G0_1_2 + 0.00489298991903115*G0_2_0 + 0.00489298991903134*G0_2_1 + 0.00978597983806357*G0_2_2;
    A[2463] = -0.185933616923196*G0_0_0 - 0.48440600198412*G0_0_1 - 0.288686405222859*G0_0_2 - 0.484406001984121*G0_1_0 - 0.782878387045045*G0_1_1 - 0.484406001984122*G0_1_2 - 0.288686405222859*G0_2_0 - 0.484406001984119*G0_2_1 - 0.0391439193522512*G0_2_2;
    A[2464] = 0.0117431758056758*G0_0_0 + 0.0117431758056758*G0_0_1 + 0.0117431758056758*G0_0_2 + 0.00946499969937476*G0_1_0 + 0.00946499969937481*G0_1_1 + 0.00946499969937473*G0_1_2 + 0.0117431758056757*G0_2_0 + 0.0117431758056757*G0_2_1 + 0.0117431758056757*G0_2_2;
    A[2465] = -0.0298472385060925*G0_0_0 + 0.00472467106581688*G0_1_0;
    A[2466] = -0.00946499969937462*G0_1_1;
    A[2467] = 0.00472467106581678*G0_1_2 - 0.0298472385060924*G0_2_2;
    A[2468] = 0.0292600797158085*G0_1_1 + 0.00329461321214774*G0_1_2 + 0.00329461321214787*G0_2_1;
    A[2469] = -0.0189848008858419*G0_1_1 - 0.00949240044292078*G0_1_2 - 0.00949240044292078*G0_2_1 - 0.000652398655871673*G0_2_2;
    A[2470] = 0.00352295274170237*G0_1_1 + 0.00649136662591474*G0_1_2 + 0.00649136662591478*G0_2_1 + 0.00880738185425749*G0_2_2;
    A[2471] = 0.0429604514890967*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.0422754329004328*G0_2_1 + 0.0642612676032811*G0_2_2;
    A[2472] = 0.17663693607704*G0_0_0 + 0.042275432900433*G0_0_2 + 0.0493213383838385*G0_1_0 + 0.0429604514890973*G0_1_2 + 0.0422754329004331*G0_2_0;
    A[2473] = -0.334680510461759*G0_0_0 - 0.083865847212201*G0_0_2 - 0.167731694424402*G0_1_0 - 0.0777006799142211*G0_1_2 - 0.0838658472122006*G0_2_0 + 0.104709984267276*G0_2_2;
    A[2474] = 0.104709984267276*G0_0_0 - 0.0838658472122007*G0_0_2 - 0.0777006799142196*G0_1_0 - 0.167731694424401*G0_1_2 - 0.0838658472122004*G0_2_0 - 0.334680510461758*G0_2_2;
    A[2475] = 0.0422754329004333*G0_0_2 + 0.0429604514890968*G0_1_0 + 0.0493213383838382*G0_1_2 + 0.042275432900433*G0_2_0 + 0.17663693607704*G0_2_2;
    A[2476] = 0.0642612676032811*G0_0_0 + 0.0422754329004328*G0_0_1 + 0.0422754329004328*G0_1_0 + 0.0429604514890972*G0_1_1;
    A[2477] = 0.00880738185425644*G0_0_0 + 0.00649136662591459*G0_0_1 + 0.00649136662591497*G0_1_0 + 0.00352295274170251*G0_1_1;
    A[2478] = -0.000652398655871017*G0_0_0 - 0.00949240044292079*G0_0_1 - 0.00949240044292108*G0_1_0 - 0.0189848008858423*G0_1_1;
    A[2479] = 0.00329461321214751*G0_0_1 + 0.00329461321214779*G0_1_0 + 0.0292600797158084*G0_1_1;
    A[2480] = -0.0435476102793802*G0_0_0 - 0.0435476102793802*G0_0_1 - 0.0377086423093363*G0_0_2 - 0.0292600797158086*G0_1_0 - 0.0292600797158086*G0_1_1 - 0.025965466503661*G0_1_2 - 0.0377086423093359*G0_2_0 - 0.0377086423093359*G0_2_1 - 0.031869674339292*G0_2_2;
    A[2481] = 0.0567586830607651*G0_0_0 + 0.056758683060765*G0_0_1 + 0.0378717419733038*G0_0_2 + 0.0189848008858425*G0_1_0 + 0.0189848008858425*G0_1_1 + 0.00949240044292159*G0_1_2 + 0.0378717419733036*G0_2_0 + 0.0378717419733034*G0_2_1 + 0.0127870136550692*G0_2_2;
    A[2482] = 0.104709984267276*G0_0_0 + 0.104709984267276*G0_0_1 + 0.0303691574307897*G0_0_2 - 0.00352295274170282*G0_1_0 - 0.00352295274170279*G0_1_1 + 0.00296841388421229*G0_1_2 + 0.0303691574307899*G0_2_0 + 0.0303691574307899*G0_2_1 - 0.00678494602105695*G0_2_2;
    A[2483] = -0.0422754329004329*G0_0_2 - 0.0429604514890968*G0_1_0 - 0.0429604514890968*G0_1_1 - 0.000685018588664302*G0_1_2 - 0.042275432900433*G0_2_0 - 0.042275432900433*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[2484] = -0.00913358118219179*G0_0_0 - 0.00329461321214816*G0_0_1 - 0.0091335811821916*G0_0_2 - 0.00329461321214784*G0_1_0 - 0.00329461321214788*G0_1_2 - 0.00913358118219155*G0_2_0 - 0.00329461321214785*G0_2_1 - 0.00913358118219148*G0_2_2;
    A[2485] = -0.0156575677409008*G0_0_0 - 0.00257697469069002*G0_0_1 - 0.0156575677409009*G0_0_2 - 0.00257697469069008*G0_1_0 - 0.00257697469068989*G0_1_2 - 0.0156575677409012*G0_2_0 - 0.0025769746906901*G0_2_1 - 0.0156575677409013*G0_2_2;
    A[2486] = -0.000652398655870921*G0_0_0 - 0.00257697469068941*G0_0_1 - 0.00065239865587094*G0_0_2 - 0.00257697469069002*G0_1_0 - 0.00257697469069015*G0_1_2 - 0.000652398655870428*G0_2_0 - 0.00257697469068967*G0_2_1 - 0.000652398655870459*G0_2_2;
    A[2487] = -0.00329461321214828*G0_0_1 - 0.00329461321214781*G0_1_0 - 0.00329461321214783*G0_1_2 - 0.00329461321214809*G0_2_1;
    A[2488] = -0.0318696743392924*G0_0_0 - 0.0377086423093361*G0_0_1 - 0.037708642309336*G0_0_2 - 0.025965466503661*G0_1_0 - 0.0292600797158086*G0_1_1 - 0.0292600797158085*G0_1_2 - 0.0377086423093366*G0_2_0 - 0.0435476102793803*G0_2_1 - 0.0435476102793802*G0_2_2;
    A[2489] = 0.0127870136550695*G0_0_0 + 0.037871741973304*G0_0_1 + 0.0378717419733038*G0_0_2 + 0.00949240044292151*G0_1_0 + 0.0189848008858423*G0_1_1 + 0.0189848008858422*G0_1_2 + 0.0378717419733045*G0_2_0 + 0.0567586830607657*G0_2_1 + 0.0567586830607658*G0_2_2;
    A[2490] = -0.00678494602105715*G0_0_0 + 0.0303691574307897*G0_0_1 + 0.0303691574307897*G0_0_2 + 0.00296841388421247*G0_1_0 - 0.00352295274170232*G0_1_1 - 0.00352295274170226*G0_1_2 + 0.0303691574307888*G0_2_0 + 0.104709984267275*G0_2_1 + 0.104709984267275*G0_2_2;
    A[2491] = 0.0439716694056967*G0_0_0 - 0.042275432900433*G0_0_1 - 0.0422754329004329*G0_0_2 - 0.000685018588664596*G0_1_0 - 0.0429604514890975*G0_1_1 - 0.0429604514890974*G0_1_2 - 0.0422754329004322*G0_2_0;
    A[2492] = -0.15657567740901*G0_0_0 - 0.0225077536275447*G0_0_1 + 0.0678494602105703*G0_0_2 - 0.0225077536275445*G0_1_0 + 0.0234863516113526*G0_1_1 + 0.104709984267276*G0_1_2 + 0.0678494602105727*G0_2_0 + 0.104709984267277*G0_2_1;
    A[2493] = 0.0195719596761269*G0_0_0 + 0.0748627457611828*G0_0_1 + 0.0141896707651919*G0_0_2 + 0.074862745761182*G0_1_0 + 0.149725491522365*G0_1_1 + 0.0283793415303823*G0_1_2 + 0.0141896707651916*G0_2_0 + 0.0283793415303833*G0_2_1;
    A[2494] = -0.0127217737894824*G0_0_1 - 0.000652398655871264*G0_0_2 - 0.0127217737894812*G0_1_0 - 0.113517366121531*G0_1_1 - 0.0127217737894816*G0_1_2 - 0.000652398655871305*G0_2_0 - 0.0127217737894822*G0_2_1;
    A[2495] = 0.0929668084616007*G0_0_0 - 0.270093043530542*G0_0_1 - 0.13504652176527*G0_0_2 - 0.27009304353054*G0_1_0 - 0.540186087061082*G0_1_1 - 0.270093043530541*G0_1_2 - 0.135046521765271*G0_2_0 - 0.270093043530541*G0_2_1 + 0.0929668084616003*G0_2_2;
    A[2496] = 0.0283793415303819*G0_0_1 + 0.0141896707651905*G0_0_2 + 0.0283793415303826*G0_1_0 + 0.149725491522364*G0_1_1 + 0.0748627457611827*G0_1_2 + 0.0141896707651915*G0_2_0 + 0.074862745761182*G0_2_1 + 0.0195719596761259*G0_2_2;
    A[2497] = 0.104709984267277*G0_0_1 + 0.0678494602105716*G0_0_2 + 0.104709984267275*G0_1_0 + 0.023486351611353*G0_1_1 - 0.0225077536275446*G0_1_2 + 0.0678494602105713*G0_2_0 - 0.0225077536275439*G0_2_1 - 0.15657567740901*G0_2_2;
    A[2498] = 0.0391439193522529*G0_0_0 + 0.0127217737894825*G0_0_1 + 0.0446893079271537*G0_0_2 + 0.0127217737894827*G0_1_0 + 0.0120693751336113*G0_1_2 + 0.0446893079271542*G0_2_0 + 0.0120693751336106*G0_2_1 + 0.0502346965020557*G0_2_2;
    A[2499] = 0.0195719596761259*G0_0_0 + 0.0181040627004155*G0_0_1 + 0.0298472385060927*G0_0_2 + 0.0181040627004156*G0_1_0 + 0.0102752788299662*G0_1_2 + 0.0298472385060921*G0_2_0 + 0.0102752788299665*G0_2_1 + 0.0401225173360587*G0_2_2;
    A[2500] = 0.0127217737894827*G0_0_1 + 0.000652398655871331*G0_0_2 + 0.0127217737894812*G0_1_0 + 0.0120693751336108*G0_1_2 + 0.000652398655871334*G0_2_0 + 0.0120693751336115*G0_2_1 + 0.00130479731174179*G0_2_2;
    A[2501] = 0.0929668084616009*G0_0_0 - 0.0283793415303818*G0_0_1 + 0.0322937334656094*G0_0_2 - 0.0283793415303822*G0_1_0 - 0.0141896707651916*G0_1_2 + 0.0322937334656089*G0_2_0 - 0.0141896707651909*G0_2_1 - 0.00880738185425511*G0_2_2;
    A[2502] = -0.0283793415303823*G0_0_1 - 0.0141896707651913*G0_0_2 - 0.0283793415303823*G0_1_0 - 0.0141896707651904*G0_1_2 - 0.0141896707651912*G0_2_0 - 0.0141896707651915*G0_2_1 - 0.0332723314494142*G0_2_2;
    A[2503] = -0.104709984267276*G0_0_1 - 0.0678494602105722*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0368605240567046*G0_1_2 - 0.0678494602105714*G0_2_0 - 0.0368605240567048*G0_2_1 - 0.0965550010688905*G0_2_2;
    A[2504] = 0.17223324514991*G0_0_0 + 0.198655390712681*G0_0_1 + 0.166035457919136*G0_0_2 + 0.10079559233205*G0_1_0 + 0.113517366121532*G0_1_1 + 0.10079559233205*G0_1_2 + 0.166035457919136*G0_2_0 + 0.19865539071268*G0_2_1 + 0.172233245149909*G0_2_2;
    A[2505] = -0.0371867233846399*G0_0_0 - 0.177615534060844*G0_0_1 - 0.224098938291644*G0_0_2 - 0.0748627457611832*G0_1_0 - 0.149725491522365*G0_1_1 - 0.121346149991982*G0_1_2 - 0.224098938291644*G0_2_0 - 0.326851726591305*G0_2_1 - 0.448197876583289*G0_2_2;
    A[2506] = -0.26813584756293*G0_0_0 + 0.0225077536275453*G0_0_1 + 0.0534966897814109*G0_0_2 - 0.0459941052388969*G0_1_0 - 0.0234863516113519*G0_1_1 + 0.0812236326559232*G0_1_2 + 0.0534966897814104*G0_2_0 - 0.104709984267277*G0_2_1 - 0.209419968534552*G0_2_2;
    A[2507] = -0.448197876583289*G0_0_0 - 0.326851726591306*G0_0_1 - 0.224098938291644*G0_0_2 - 0.121346149991982*G0_1_0 - 0.149725491522364*G0_1_1 - 0.0748627457611826*G0_1_2 - 0.224098938291644*G0_2_0 - 0.177615534060844*G0_2_1 - 0.0371867233846381*G0_2_2;
    A[2508] = 0.726119703984283*G0_0_0 + 0.270093043530542*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530542*G0_1_0 + 0.540186087061082*G0_1_1 + 0.270093043530541*G0_1_2 + 0.270093043530541*G0_2_0 + 0.270093043530541*G0_2_1 + 0.726119703984282*G0_2_2;
    A[2509] = -0.209419968534553*G0_0_0 - 0.104709984267277*G0_0_1 + 0.05349668978141*G0_0_2 + 0.0812236326559222*G0_1_0 - 0.0234863516113531*G0_1_1 - 0.0459941052388971*G0_1_2 + 0.0534966897814103*G0_2_0 + 0.0225077536275439*G0_2_1 - 0.268135847562931*G0_2_2;
    A[2510] = 0.0502346965020569*G0_0_0 + 0.0120693751336114*G0_0_1 + 0.0446893079271557*G0_0_2 + 0.0120693751336112*G0_1_0 + 0.0127217737894822*G0_1_2 + 0.0446893079271538*G0_2_0 + 0.0127217737894813*G0_2_1 + 0.039143919352252*G0_2_2;
    A[2511] = -0.00880738185425673*G0_0_0 - 0.0141896707651914*G0_0_1 + 0.0322937334656089*G0_0_2 - 0.0141896707651918*G0_1_0 - 0.0283793415303831*G0_1_2 + 0.032293733465609*G0_2_0 - 0.0283793415303828*G0_2_1 + 0.0929668084616009*G0_2_2;
    A[2512] = -0.0965550010688895*G0_0_0 - 0.0368605240567041*G0_0_1 - 0.06784946021057*G0_0_2 - 0.0368605240567045*G0_1_0 - 0.104709984267276*G0_1_2 - 0.0678494602105714*G0_2_0 - 0.104709984267275*G0_2_1;
    A[2513] = 0.0401225173360579*G0_0_0 + 0.0102752788299648*G0_0_1 + 0.0298472385060914*G0_0_2 + 0.0102752788299658*G0_1_0 + 0.0181040627004157*G0_1_2 + 0.0298472385060925*G0_2_0 + 0.018104062700417*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2514] = -0.0332723314494136*G0_0_0 - 0.0141896707651915*G0_0_1 - 0.0141896707651918*G0_0_2 - 0.0141896707651907*G0_1_0 - 0.0283793415303823*G0_1_2 - 0.0141896707651909*G0_2_0 - 0.0283793415303826*G0_2_1;
    A[2515] = 0.00130479731174189*G0_0_0 + 0.0120693751336121*G0_0_1 + 0.000652398655871301*G0_0_2 + 0.0120693751336111*G0_1_0 + 0.0127217737894817*G0_1_2 + 0.000652398655870152*G0_2_0 + 0.0127217737894813*G0_2_1;
    A[2516] = -0.234863516113512*G0_0_0 - 0.092966808461598*G0_0_1 - 0.215291556437386*G0_0_2 - 0.0929668084615993*G0_1_0 - 0.0929668084615982*G0_1_2 - 0.215291556437386*G0_2_0 - 0.0929668084615984*G0_2_1 - 0.234863516113514*G0_2_2;
    A[2517] = 0.35229527417027*G0_0_0 + 0.10275278829966*G0_0_1 + 0.102752788299661*G0_0_2 + 0.102752788299661*G0_1_0 + 0.298472385060923*G0_1_2 + 0.102752788299659*G0_2_0 + 0.298472385060922*G0_2_1 - 0.185933616923201*G0_2_2;
    A[2518] = -0.0391439193522528*G0_0_0 - 0.0929668084616008*G0_0_1 - 0.0440369092712825*G0_0_2 - 0.0929668084615977*G0_1_0 - 0.0929668084615989*G0_1_2 - 0.044036909271284*G0_2_0 - 0.0929668084615996*G0_2_1 - 0.0391439193522524*G0_2_2;
    A[2519] = -0.185933616923201*G0_0_0 + 0.298472385060924*G0_0_1 + 0.102752788299659*G0_0_2 + 0.298472385060922*G0_1_0 + 0.102752788299661*G0_1_2 + 0.102752788299662*G0_2_0 + 0.102752788299662*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2520] = 0.0189195610202554*G0_0_0 + 0.0189195610202555*G0_0_1 + 0.0189195610202554*G0_0_2 + 0.013742125287264*G0_1_0 + 0.0137421252872641*G0_1_1 + 0.013742125287264*G0_1_2 + 0.0189195610202553*G0_2_0 + 0.0189195610202554*G0_2_1 + 0.0189195610202553*G0_2_2;
    A[2521] = 0.0189195610202554*G0_0_0 + 0.0051774357329912*G0_1_0;
    A[2522] = -0.013742125287264*G0_1_1;
    A[2523] = -0.0143945239431352*G0_1_2 + 0.03359853077735*G0_2_2;
    A[2524] = 0.0641307878721067*G0_1_1 + 0.0093945406445404*G0_1_2 + 0.00939454064454029*G0_2_1;
    A[2525] = -0.127087258163646*G0_1_1 - 0.0377738821749231*G0_1_2 - 0.0377738821749235*G0_2_1 - 0.0150051690850295*G0_2_2;
    A[2526] = 0.124738623002511*G0_1_1 + 0.0623693115012547*G0_1_2 + 0.0623693115012554*G0_2_1 + 0.0476251018785728*G0_2_2;
    A[2527] = -0.124738623002511*G0_1_1 - 0.0962940416065405*G0_1_2 - 0.0962940416065407*G0_2_1 - 0.128522535206562*G0_2_2;
    A[2528] = -0.111886369481855*G0_0_0 - 0.0212681961813904*G0_0_2 - 0.0477555816097481*G0_1_0 - 0.0118736555368503*G0_1_2 - 0.0212681961813907*G0_2_0;
    A[2529] = 0.209419968534551*G0_0_0 + 0.0359471659384852*G0_0_2 + 0.0823327103709049*G0_1_0 - 0.00182671623643763*G0_1_2 + 0.0359471659384854*G0_2_0 - 0.052844291125541*G0_2_2;
    A[2530] = -0.271397840842283*G0_0_0 - 0.0733296089198853*G0_0_2 - 0.146659217839772*G0_1_0 - 0.0109602974186306*G0_1_2 - 0.0733296089198861*G0_2_0 + 0.121346149991983*G0_2_2;
    A[2531] = -0.0962940416065412*G0_0_2 - 0.124738623002511*G0_1_0 - 0.192588083213082*G0_1_2 - 0.0962940416065407*G0_2_0 - 0.257045070413125*G0_2_2;
    A[2532] = -0.0306627368259304*G0_0_0 - 0.0212681961813901*G0_0_1 - 0.0212681961813904*G0_1_0 - 0.0118736555368502*G0_1_1;
    A[2533] = -0.00130479731174244*G0_0_0 - 0.00123955744615474*G0_0_1 - 0.00123955744615543*G0_1_0 - 0.00117431758056743*G0_1_1;
    A[2534] = -0.0150051690850297*G0_0_0 - 0.0149399292194431*G0_0_1 - 0.0149399292194427*G0_1_0 - 0.0148746893538563*G0_1_1;
    A[2535] = 0.00939454064454039*G0_0_1 + 0.00939454064454036*G0_1_0 + 0.0187890812890811*G0_1_1;
    A[2536] = -0.111886369481854*G0_0_0 - 0.111886369481854*G0_0_1 - 0.0906181733004644*G0_0_2 - 0.0641307878721066*G0_1_0 - 0.0641307878721067*G0_1_1 - 0.0547362472275663*G0_1_2 - 0.0906181733004638*G0_2_0 - 0.0906181733004639*G0_2_1 - 0.0693499771190735*G0_2_2;
    A[2537] = 0.209419968534549*G0_0_0 + 0.209419968534549*G0_0_1 + 0.173472802596065*G0_0_2 + 0.127087258163646*G0_1_0 + 0.127087258163646*G0_1_1 + 0.0893133759887229*G0_1_2 + 0.173472802596064*G0_2_0 + 0.173472802596064*G0_2_1 + 0.0846813455320386*G0_2_2;
    A[2538] = -0.271397840842283*G0_0_0 - 0.271397840842283*G0_0_1 - 0.198068231922397*G0_0_2 - 0.124738623002511*G0_1_0 - 0.124738623002511*G0_1_1 - 0.0623693115012559*G0_1_2 - 0.198068231922396*G0_2_0 - 0.198068231922396*G0_2_1 - 0.00339247301052779*G0_2_2;
    A[2539] = 0.0962940416065412*G0_0_2 + 0.124738623002512*G0_1_0 + 0.124738623002512*G0_1_1 + 0.0284445813959708*G0_1_2 + 0.0962940416065409*G0_2_0 + 0.0962940416065409*G0_2_1 - 0.0644569872000424*G0_2_2;
    A[2540] = -0.0306627368259308*G0_0_0 - 0.00939454064454084*G0_0_1 - 0.0306627368259309*G0_0_2 - 0.00939454064454051*G0_1_0 - 0.0093945406445405*G0_1_2 - 0.0306627368259311*G0_2_0 - 0.00939454064454082*G0_2_1 - 0.0306627368259311*G0_2_2;
    A[2541] = -0.00130479731174165*G0_0_0 - 6.52398655868026e-05*G0_0_1 - 0.00130479731174158*G0_0_2 - 6.52398655871074e-05*G0_1_0 - 6.5239865587098e-05*G0_1_2 - 0.00130479731174131*G0_2_0 - 6.52398655868354e-05*G0_2_1 - 0.00130479731174127*G0_2_2;
    A[2542] = -0.0150051690850299*G0_0_0 - 6.52398655869755e-05*G0_0_1 - 0.01500516908503*G0_0_2 - 6.52398655867816e-05*G0_1_0 - 6.5239865586862e-05*G0_1_2 - 0.0150051690850301*G0_2_0 - 6.52398655869071e-05*G0_2_1 - 0.0150051690850301*G0_2_2;
    A[2543] = -0.00939454064454079*G0_0_1 - 0.00939454064454045*G0_1_0 - 0.00939454064454046*G0_1_2 - 0.00939454064454079*G0_2_1;
    A[2544] = -0.0187890812890812*G0_0_0 - 0.0400572774704711*G0_0_1 - 0.0400572774704712*G0_0_2 - 0.00939454064454065*G0_1_0 - 0.0187890812890812*G0_1_1 - 0.0187890812890812*G0_1_2 - 0.0400572774704717*G0_2_0 - 0.0613254736518619*G0_2_1 - 0.0613254736518618*G0_2_2;
    A[2545] = -0.00013047973117415*G0_0_0 + 0.013635131907701*G0_0_1 + 0.013635131907701*G0_0_2 - 6.52398655868998e-05*G0_1_0 + 0.0148746893538561*G0_1_1 + 0.0148746893538561*G0_1_2 + 0.0136351319077012*G0_2_0 - 0.0254435475789641*G0_2_1 - 0.0254435475789643*G0_2_2;
    A[2546] = -0.000130479731174189*G0_0_0 - 0.0137656116388754*G0_0_1 - 0.0137656116388754*G0_0_2 - 6.52398655871869e-05*G0_1_0 + 0.00117431758056712*G0_1_1 + 0.00117431758056707*G0_1_2 - 0.0137656116388752*G0_2_0 - 0.0528442911255409*G0_2_1 - 0.0528442911255409*G0_2_2;
    A[2547] = -0.0187890812890812*G0_0_0 + 0.02126819618139*G0_0_1 + 0.0212681961813901*G0_0_2 - 0.00939454064454052*G0_1_0 + 0.0118736555368499*G0_1_1 + 0.01187365553685*G0_1_2 + 0.0212681961813903*G0_2_0;
    A[2548] = 0.0521918924696704*G0_0_0 - 0.00065239865587006*G0_0_1 - 0.0378391220405096*G0_0_2 - 0.000652398655869633*G0_1_0 - 0.0534966897814107*G0_1_1 - 0.0528442911255402*G0_1_2 - 0.0378391220405119*G0_2_0 - 0.0528442911255413*G0_2_1;
    A[2549] = 0.0260959462348352*G0_0_0 + 0.0198981590040612*G0_0_1 - 0.0120693751336122*G0_0_2 + 0.0198981590040615*G0_1_0 + 0.013700371773288*G0_1_1 - 0.0127217737894833*G0_1_2 - 0.0120693751336107*G0_2_0 - 0.0127217737894828*G0_2_1;
    A[2550] = -0.0528442911255401*G0_0_1 - 0.0150051690850299*G0_0_2 - 0.0528442911255401*G0_1_0 - 0.10568858225108*G0_1_1 - 0.0300103381700596*G0_1_2 - 0.0150051690850304*G0_2_0 - 0.0300103381700598*G0_2_1;
    A[2551] = -0.185933616923198*G0_0_0 - 0.0812236326559226*G0_0_1 + 0.00913358118219295*G0_0_2 - 0.0812236326559237*G0_1_0 + 0.0234863516113529*G0_1_1 + 0.0459941052388979*G0_1_2 + 0.00913358118219296*G0_2_0 + 0.0459941052388968*G0_2_1 - 0.0880738185425681*G0_2_2;
    A[2552] = 0.104709984267276*G0_0_1 + 0.036860524056706*G0_0_2 + 0.104709984267275*G0_1_0 + 0.20941996853455*G0_1_1 + 0.0737210481134094*G0_1_2 + 0.0368605240567049*G0_2_0 + 0.0737210481134098*G0_2_1 - 0.0228339529554795*G0_2_2;
    A[2553] = -0.271397840842284*G0_0_1 - 0.135698920421142*G0_0_2 - 0.271397840842285*G0_1_0 - 0.542795681684568*G0_1_1 - 0.271397840842285*G0_1_2 - 0.135698920421142*G0_2_0 - 0.271397840842284*G0_2_1 + 0.05219189246967*G0_2_2;
    A[2554] = 0.0521918924696697*G0_0_0 + 0.0528442911255405*G0_0_1 + 0.0900310145101798*G0_0_2 + 0.0528442911255403*G0_1_0 + 0.0378391220405099*G0_1_2 + 0.0900310145101813*G0_2_0 + 0.0378391220405111*G0_2_1 + 0.127870136550691*G0_2_2;
    A[2555] = 0.0260959462348332*G0_0_0 + 0.00619778723077199*G0_0_1 + 0.0381653213684447*G0_0_2 + 0.00619778723077231*G0_1_0 + 0.00554538857490163*G0_1_2 + 0.0381653213684446*G0_2_0 + 0.00554538857490025*G0_2_1 + 0.0502346965020557*G0_2_2;
    A[2556] = 0.0528442911255424*G0_0_1 + 0.01500516908503*G0_0_2 + 0.0528442911255402*G0_1_0 + 0.0378391220405102*G0_1_2 + 0.0150051690850304*G0_2_0 + 0.0378391220405122*G0_2_1 + 0.0300103381700602*G0_2_2;
    A[2557] = -0.185933616923197*G0_0_0 - 0.104709984267274*G0_0_1 - 0.195067198105389*G0_0_2 - 0.104709984267274*G0_1_0 - 0.0678494602105692*G0_1_2 - 0.195067198105389*G0_2_0 - 0.0678494602105693*G0_2_1 - 0.29227459783015*G0_2_2;
    A[2558] = -0.104709984267276*G0_0_1 - 0.036860524056703*G0_0_2 - 0.104709984267275*G0_1_0 - 0.0678494602105695*G0_1_2 - 0.0368605240567048*G0_2_0 - 0.067849460210571*G0_2_1 - 0.096555001068888*G0_2_2;
    A[2559] = 0.271397840842283*G0_0_1 + 0.13569892042114*G0_0_2 + 0.271397840842284*G0_1_0 + 0.135698920421141*G0_1_2 + 0.135698920421142*G0_2_0 + 0.135698920421141*G0_2_1 + 0.323589733311952*G0_2_2;
    A[2560] = 0.105688582251081*G0_0_0 + 0.10503618359521*G0_0_1 + 0.12787013655069*G0_0_2 + 0.0528442911255407*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0756782440810209*G0_1_2 + 0.127870136550691*G0_2_0 + 0.180062029020361*G0_2_1 + 0.255740273101382*G0_2_2;
    A[2561] = 0.0123955744615456*G0_0_0 + 0.00619778723077381*G0_0_1 + 0.00619778723077359*G0_0_2 + 0.00619778723077206*G0_1_0 - 0.013700371773288*G0_1_1 - 0.0264221455627699*G0_1_2 + 0.00619778723077196*G0_2_0 + 0.0714376528178603*G0_2_1 + 0.172233245149911*G0_2_2;
    A[2562] = 0.105688582251082*G0_0_0 + 0.000652398655869982*G0_0_1 - 0.0221815542996093*G0_0_2 + 0.0528442911255418*G0_1_0 + 0.0534966897814109*G0_1_1 + 0.000652398655870997*G0_1_2 - 0.0221815542996085*G0_2_0 + 0.0528442911255413*G0_2_1 + 0.105688582251083*G0_2_2;
    A[2563] = -0.209419968534547*G0_0_0 - 0.290643601190471*G0_0_1 - 0.26291665831596*G0_0_2 - 0.104709984267274*G0_1_0 - 0.209419968534549*G0_1_1 - 0.135698920421141*G0_1_2 - 0.262916658315959*G0_2_0 - 0.390134396210779*G0_2_1 - 0.584549195660301*G0_2_2;
    A[2564] = -0.209419968534553*G0_0_0 + 0.0812236326559222*G0_0_1 + 0.0534966897814103*G0_0_2 - 0.104709984267277*G0_1_0 - 0.0234863516113531*G0_1_1 + 0.0225077536275439*G0_1_2 + 0.05349668978141*G0_2_0 - 0.0459941052388971*G0_2_1 - 0.268135847562931*G0_2_2;
    A[2565] = 0.542795681684566*G0_0_0 + 0.271397840842284*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842284*G0_1_0 + 0.542795681684567*G0_1_1 + 0.271397840842284*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842284*G0_2_1 + 0.647179466623906*G0_2_2;
    A[2566] = 0.0300103381700598*G0_0_0 + 0.0150051690850299*G0_0_1 - 0.00782878387045031*G0_0_2 + 0.0150051690850303*G0_1_0 + 0.0300103381700607*G0_1_2 - 0.00782878387045031*G0_2_0 + 0.0300103381700606*G0_2_1 - 0.0456679059109599*G0_2_2;
    A[2567] = 0.00130479731174173*G0_0_0 + 0.000652398655870635*G0_0_1 + 0.000652398655870866*G0_0_2 + 0.00065239865587024*G0_1_0 + 0.0127217737894816*G0_1_2 + 0.000652398655870257*G0_2_0 + 0.0127217737894819*G0_2_1 - 0.088073818542569*G0_2_2;
    A[2568] = 0.0300103381700597*G0_0_0 + 0.0150051690850303*G0_0_1 + 0.0378391220405096*G0_0_2 + 0.0150051690850303*G0_1_0 + 0.0528442911255404*G0_1_2 + 0.0378391220405107*G0_2_0 + 0.0528442911255413*G0_2_1;
    A[2569] = 0.00130479731174145*G0_0_0 + 0.000652398655870634*G0_0_1 - 0.0107645778218701*G0_0_2 + 0.000652398655870466*G0_1_0 + 0.00130479731174073*G0_1_2 - 0.0107645778218703*G0_2_0 + 0.00130479731174051*G0_2_1 - 0.0228339529554819*G0_2_2;
    A[2570] = 0.00130479731174273*G0_0_0 + 0.000652398655871387*G0_0_1 + 0.0120693751336122*G0_0_2 + 0.000652398655872166*G0_1_0 + 0.0127217737894831*G0_1_2 + 0.0120693751336124*G0_2_0 + 0.0127217737894823*G0_2_1;
    A[2571] = 0.0300103381700597*G0_0_0 + 0.0150051690850302*G0_0_1 + 0.0150051690850301*G0_0_2 + 0.0150051690850297*G0_1_0 + 0.0300103381700599*G0_1_2 + 0.0150051690850299*G0_2_0 + 0.0300103381700611*G0_2_1;
    A[2572] = -0.0521918924696689*G0_0_0 - 0.026095946234835*G0_0_1 - 0.0424059126316059*G0_0_2 - 0.0260959462348336*G0_1_0 - 0.0750258454251495*G0_1_2 - 0.0424059126316068*G0_2_0 - 0.0750258454251507*G0_2_1 + 0.143527704291594*G0_2_2;
    A[2573] = -0.0521918924696715*G0_0_0 - 0.0260959462348346*G0_0_1 - 0.00978597983806385*G0_0_2 - 0.0260959462348373*G0_1_0 - 0.0587158790283802*G0_1_2 - 0.00978597983806287*G0_2_0 - 0.0587158790283773*G0_2_1 + 0.176147637085137*G0_2_2;
    A[2574] = -0.0521918924696688*G0_0_0 - 0.0260959462348352*G0_0_1 - 0.0260959462348365*G0_0_2 - 0.0260959462348339*G0_1_0 - 0.0750258454251508*G0_1_2 - 0.0260959462348339*G0_2_0 - 0.075025845425151*G0_2_1 + 0.0456679059109605*G0_2_2;
    A[2575] = 0.371867233846396*G0_0_0 + 0.185933616923196*G0_0_1 + 0.185933616923199*G0_0_2 + 0.185933616923198*G0_1_0 + 0.316413348097375*G0_1_2 + 0.185933616923197*G0_2_0 + 0.31641334809737*G0_2_1 - 0.150051690850302*G0_2_2;
    A[2576] = 0.0335985307773498*G0_0_0 + 0.0335985307773497*G0_0_1 + 0.0335985307773498*G0_0_2 + 0.0335985307773498*G0_1_0 + 0.0335985307773497*G0_1_1 + 0.0335985307773498*G0_1_2 + 0.0479930547204848*G0_2_0 + 0.0479930547204848*G0_2_1 + 0.0479930547204849*G0_2_2;
    A[2577] = 0.0189195610202553*G0_0_0 + 0.00517743573299106*G0_2_0;
    A[2578] = 0.0189195610202555*G0_1_1 + 0.00517743573299134*G0_2_1;
    A[2579] = -0.0137421252872639*G0_2_2;
    A[2580] = -0.0306627368259311*G0_1_1 - 0.0212681961813903*G0_1_2 - 0.0212681961813901*G0_2_1 - 0.0118736555368497*G0_2_2;
    A[2581] = -0.00130479731174177*G0_1_1 - 0.00123955744615488*G0_1_2 - 0.001239557446155*G0_2_1 - 0.00117431758056797*G0_2_2;
    A[2582] = -0.0150051690850302*G0_1_1 - 0.0149399292194425*G0_1_2 - 0.0149399292194431*G0_2_1 - 0.0148746893538555*G0_2_2;
    A[2583] = 0.00939454064453999*G0_1_2 + 0.00939454064454037*G0_2_1 + 0.0187890812890804*G0_2_2;
    A[2584] = -0.0306627368259312*G0_0_0 - 0.0212681961813905*G0_0_2 - 0.0212681961813906*G0_2_0 - 0.0118736555368502*G0_2_2;
    A[2585] = -0.0013047973117418*G0_0_0 - 0.00123955744615513*G0_0_2 - 0.00123955744615507*G0_2_0 - 0.00117431758056814*G0_2_2;
    A[2586] = -0.01500516908503*G0_0_0 - 0.0149399292194424*G0_0_2 - 0.014939929219443*G0_2_0 - 0.0148746893538556*G0_2_2;
    A[2587] = 0.00939454064454038*G0_0_2 + 0.00939454064454052*G0_2_0 + 0.0187890812890807*G0_2_2;
    A[2588] = -0.0613254736518618*G0_0_0 - 0.0212681961813904*G0_0_1 - 0.0212681961813902*G0_1_0 - 0.0425363923627808*G0_2_0 - 0.0118736555368501*G0_2_1;
    A[2589] = -0.025443547578964*G0_0_0 - 0.0390786794866647*G0_0_1 - 0.0390786794866653*G0_1_0 - 0.0528442911255404*G0_1_1 - 0.0403182369328197*G0_2_0 - 0.0540186087061075*G0_2_1;
    A[2590] = -0.0528442911255405*G0_0_0 - 0.0390786794866656*G0_0_1 - 0.0390786794866649*G0_1_0 - 0.025443547578964*G0_1_1 - 0.054018608706108*G0_2_0 - 0.0403182369328206*G0_2_1;
    A[2591] = -0.0212681961813899*G0_0_1 - 0.0212681961813903*G0_1_0 - 0.0613254736518617*G0_1_1 - 0.0118736555368497*G0_2_0 - 0.0425363923627801*G0_2_1;
    A[2592] = -0.128522535206561*G0_0_0 - 0.128522535206561*G0_0_1 - 0.0322284936000213*G0_0_2 - 0.128522535206562*G0_1_0 - 0.128522535206562*G0_1_1 - 0.0322284936000212*G0_1_2 - 0.0322284936000209*G0_2_0 - 0.0322284936000209*G0_2_1 - 0.0606730749959916*G0_2_2;
    A[2593] = 0.0476251018785737*G0_0_0 + 0.0476251018785738*G0_0_1 - 0.0147442096226812*G0_0_2 + 0.047625101878574*G0_1_0 + 0.0476251018785742*G0_1_1 - 0.0147442096226814*G0_1_2 - 0.014744209622681*G0_2_0 - 0.014744209622681*G0_2_1 + 0.0476251018785744*G0_2_2;
    A[2594] = -0.0150051690850297*G0_0_0 - 0.0150051690850297*G0_0_1 + 0.0227687130898935*G0_0_2 - 0.0150051690850298*G0_1_0 - 0.0150051690850298*G0_1_1 + 0.0227687130898938*G0_1_2 + 0.0227687130898934*G0_2_0 + 0.0227687130898935*G0_2_1 - 0.066544662898829*G0_2_2;
    A[2595] = -0.00939454064454062*G0_0_2 - 0.00939454064454089*G0_1_2 - 0.00939454064454041*G0_2_0 - 0.00939454064454041*G0_2_1 + 0.0453417065830254*G0_2_2;
    A[2596] = -0.257045070413123*G0_0_0 - 0.160751028806582*G0_0_1 - 0.257045070413123*G0_0_2 - 0.160751028806582*G0_1_0 - 0.0644569872000414*G0_1_1 - 0.160751028806582*G0_1_2 - 0.0644569872000411*G0_2_0 - 0.0929015685960116*G0_2_1 - 0.0644569872000412*G0_2_2;
    A[2597] = 0.121346149991981*G0_0_0 + 0.194675758911867*G0_0_1 + 0.121346149991982*G0_0_2 + 0.194675758911867*G0_1_0 - 0.00339247301052959*G0_1_1 + 0.194675758911868*G0_1_2 + 0.132306447410612*G0_2_0 + 0.0589768384907258*G0_2_1 + 0.132306447410612*G0_2_2;
    A[2598] = -0.0528442911255401*G0_0_0 - 0.088791457064025*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0887914570640255*G0_1_0 + 0.0846813455320403*G0_1_1 - 0.0887914570640254*G0_1_2 - 0.051017574889102*G0_2_0 - 0.00463203045668246*G0_2_1 - 0.051017574889102*G0_2_2;
    A[2599] = 0.0212681961813903*G0_0_1 + 0.0212681961813903*G0_1_0 - 0.069349977119074*G0_1_1 + 0.0212681961813903*G0_1_2 + 0.0118736555368496*G0_2_0 - 0.0146137298915076*G0_2_1 + 0.0118736555368496*G0_2_2;
    A[2600] = -0.0644569872000422*G0_0_0 - 0.160751028806582*G0_0_1 - 0.160751028806582*G0_0_2 - 0.160751028806583*G0_1_0 - 0.257045070413123*G0_1_1 - 0.257045070413123*G0_1_2 - 0.0929015685960126*G0_2_0 - 0.0644569872000419*G0_2_1 - 0.064456987200042*G0_2_2;
    A[2601] = -0.00339247301052864*G0_0_0 + 0.194675758911867*G0_0_1 + 0.194675758911867*G0_0_2 + 0.194675758911868*G0_1_0 + 0.121346149991982*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0589768384907272*G0_2_0 + 0.132306447410612*G0_2_1 + 0.132306447410612*G0_2_2;
    A[2602] = 0.0846813455320394*G0_0_0 - 0.0887914570640253*G0_0_1 - 0.0887914570640253*G0_0_2 - 0.0887914570640252*G0_1_0 - 0.0528442911255408*G0_1_1 - 0.0528442911255408*G0_1_2 - 0.00463203045668331*G0_2_0 - 0.0510175748891021*G0_2_1 - 0.051017574889102*G0_2_2;
    A[2603] = -0.0693499771190739*G0_0_0 + 0.0212681961813901*G0_0_1 + 0.0212681961813901*G0_0_2 + 0.0212681961813903*G0_1_0 - 0.0146137298915074*G0_2_0 + 0.0118736555368499*G0_2_1 + 0.0118736555368499*G0_2_2;
    A[2604] = -0.0456679059109614*G0_0_0 - 0.0378391220405108*G0_0_1 - 0.0756782440810205*G0_0_2 - 0.0378391220405106*G0_1_0 - 0.0528442911255399*G0_1_2 - 0.0756782440810212*G0_2_0 - 0.0528442911255412*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2605] = -0.0880738185425672*G0_0_0 - 0.0887262171984391*G0_0_1 - 0.10079559233205*G0_0_2 - 0.0887262171984385*G0_1_0 - 0.088073818542568*G0_1_1 - 0.10079559233205*G0_1_2 - 0.100795592332049*G0_2_0 - 0.100795592332049*G0_2_1 - 0.113517366121532*G0_2_2;
    A[2606] = -0.0378391220405101*G0_0_1 - 0.0528442911255395*G0_0_2 - 0.0378391220405109*G0_1_0 - 0.0456679059109612*G0_1_1 - 0.0756782440810205*G0_1_2 - 0.0528442911255409*G0_2_0 - 0.0756782440810219*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2607] = -0.0228339529554803*G0_0_0 - 0.0120693751336114*G0_0_1 - 0.0241387502672227*G0_0_2 - 0.0120693751336106*G0_1_0 - 0.0127217737894822*G0_1_2 - 0.0241387502672215*G0_2_0 - 0.0127217737894819*G0_2_1 - 0.0254435475789639*G0_2_2;
    A[2608] = -0.0120693751336106*G0_0_1 - 0.0127217737894819*G0_0_2 - 0.0120693751336104*G0_1_0 - 0.022833952955479*G0_1_1 - 0.0241387502672214*G0_1_2 - 0.0127217737894825*G0_2_0 - 0.0241387502672218*G0_2_1 - 0.0254435475789644*G0_2_2;
    A[2609] = -0.0150051690850306*G0_0_1 - 0.0300103381700601*G0_0_2 - 0.0150051690850306*G0_1_0 - 0.0300103381700598*G0_1_2 - 0.030010338170061*G0_2_0 - 0.0300103381700609*G0_2_1 - 0.0600206763401202*G0_2_2;
    A[2610] = 0.0521918924696704*G0_0_0 + 0.187890812890811*G0_0_1 + 0.323589733311952*G0_0_2 + 0.187890812890812*G0_1_0 + 0.323589733311953*G0_1_1 + 0.187890812890811*G0_1_2 + 0.323589733311954*G0_2_0 + 0.187890812890811*G0_2_1 + 0.0521918924696697*G0_2_2;
    A[2611] = -0.0880738185425681*G0_0_0 - 0.0972073997247604*G0_0_1 - 0.134067923781464*G0_0_2 - 0.0972073997247598*G0_1_0 - 0.292274597830151*G0_1_1 - 0.224425137619579*G0_1_2 - 0.134067923781465*G0_2_0 - 0.22442513761958*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2612] = 0.0378391220405108*G0_0_1 + 0.0528442911255411*G0_0_2 + 0.037839122040511*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0900310145101808*G0_1_2 + 0.0528442911255412*G0_2_0 + 0.0900310145101799*G0_2_1 + 0.0521918924696706*G0_2_2;
    A[2613] = -0.0228339529554806*G0_0_0 - 0.059694477012185*G0_0_1 - 0.0965550010688887*G0_0_2 - 0.0596944770121854*G0_1_0 - 0.0965550010688902*G0_1_1 - 0.0287055408583196*G0_1_2 - 0.0965550010688901*G0_2_0 - 0.0287055408583198*G0_2_1 + 0.0391439193522507*G0_2_2;
    A[2614] = 0.0120693751336119*G0_0_1 + 0.012721773789482*G0_0_2 + 0.0120693751336104*G0_1_0 + 0.0502346965020581*G0_1_1 + 0.0446893079271545*G0_1_2 + 0.0127217737894825*G0_2_0 + 0.0446893079271562*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[2615] = 0.01500516908503*G0_0_1 + 0.0300103381700606*G0_0_2 + 0.0150051690850307*G0_1_0 + 0.0300103381700604*G0_1_1 - 0.00782878387044934*G0_1_2 + 0.0300103381700612*G0_2_0 - 0.00782878387044988*G0_2_1 - 0.0456679059109595*G0_2_2;
    A[2616] = 0.323589733311952*G0_0_0 + 0.187890812890811*G0_0_1 + 0.187890812890811*G0_0_2 + 0.187890812890811*G0_1_0 + 0.0521918924696703*G0_1_1 + 0.323589733311953*G0_1_2 + 0.187890812890811*G0_2_0 + 0.323589733311954*G0_2_1 + 0.0521918924696707*G0_2_2;
    A[2617] = -0.29227459783015*G0_0_0 - 0.0972073997247594*G0_0_1 - 0.224425137619579*G0_0_2 - 0.0972073997247604*G0_1_0 - 0.0880738185425683*G0_1_1 - 0.134067923781464*G0_1_2 - 0.22442513761958*G0_2_0 - 0.134067923781465*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2618] = 0.127870136550691*G0_0_0 + 0.0378391220405112*G0_0_1 + 0.0900310145101807*G0_0_2 + 0.0378391220405109*G0_1_0 + 0.0528442911255413*G0_1_2 + 0.0900310145101799*G0_2_0 + 0.0528442911255413*G0_2_1 + 0.0521918924696701*G0_2_2;
    A[2619] = -0.0965550010688887*G0_0_0 - 0.0596944770121843*G0_0_1 - 0.0287055408583181*G0_0_2 - 0.0596944770121852*G0_1_0 - 0.0228339529554804*G0_1_1 - 0.0965550010688896*G0_1_2 - 0.0287055408583197*G0_2_0 - 0.0965550010688904*G0_2_1 + 0.0391439193522505*G0_2_2;
    A[2620] = 0.0502346965020569*G0_0_0 + 0.0120693751336112*G0_0_1 + 0.0446893079271538*G0_0_2 + 0.0120693751336114*G0_1_0 + 0.0127217737894813*G0_1_2 + 0.0446893079271557*G0_2_0 + 0.0127217737894822*G0_2_1 + 0.039143919352252*G0_2_2;
    A[2621] = 0.0300103381700598*G0_0_0 + 0.0150051690850303*G0_0_1 - 0.00782878387045031*G0_0_2 + 0.0150051690850299*G0_1_0 + 0.0300103381700606*G0_1_2 - 0.0078287838704503*G0_2_0 + 0.0300103381700607*G0_2_1 - 0.0456679059109599*G0_2_2;
    A[2622] = 0.647179466623903*G0_0_0 + 0.375781625781621*G0_0_1 + 0.375781625781621*G0_0_2 + 0.375781625781621*G0_1_0 + 0.647179466623905*G0_1_1 + 0.375781625781622*G0_1_2 + 0.375781625781621*G0_2_0 + 0.375781625781622*G0_2_1 + 0.647179466623905*G0_2_2;
    A[2623] = -0.584549195660301*G0_0_0 - 0.321632537344338*G0_0_1 - 0.194414799449519*G0_0_2 - 0.32163253734434*G0_1_0 - 0.268135847562927*G0_1_1 - 0.222141742324032*G0_1_2 - 0.448850275239159*G0_2_0 - 0.290643601190472*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2624] = 0.255740273101381*G0_0_0 + 0.127870136550691*G0_0_1 + 0.0756782440810207*G0_0_2 + 0.127870136550691*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255401*G0_1_2 + 0.18006202902036*G0_2_0 + 0.10503618359521*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2625] = -0.268135847562927*G0_0_0 - 0.32163253734434*G0_0_1 - 0.222141742324031*G0_0_2 - 0.321632537344338*G0_1_0 - 0.584549195660301*G0_1_1 - 0.194414799449519*G0_1_2 - 0.290643601190472*G0_2_0 - 0.44885027523916*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2626] = 0.17223324514991*G0_0_0 + 0.166035457919136*G0_0_1 + 0.10079559233205*G0_0_2 + 0.166035457919136*G0_1_0 + 0.17223324514991*G0_1_1 + 0.10079559233205*G0_1_2 + 0.19865539071268*G0_2_0 + 0.19865539071268*G0_2_1 + 0.113517366121532*G0_2_2;
    A[2627] = 0.105688582251081*G0_0_0 + 0.127870136550691*G0_0_1 + 0.0528442911255398*G0_0_2 + 0.12787013655069*G0_1_0 + 0.255740273101381*G0_1_1 + 0.0756782440810207*G0_1_2 + 0.10503618359521*G0_2_0 + 0.18006202902036*G0_2_1 + 0.10568858225108*G0_2_2;
    A[2628] = -0.150051690850299*G0_0_0 - 0.335985307773498*G0_0_1 - 0.466465038947671*G0_0_2 - 0.335985307773499*G0_1_0 - 0.1500516908503*G0_1_1 - 0.466465038947673*G0_1_2 - 0.466465038947674*G0_2_0 - 0.466465038947674*G0_2_1 - 0.782878387045047*G0_2_2;
    A[2629] = 0.14352770429159*G0_0_0 + 0.185933616923198*G0_0_1 + 0.218553549716742*G0_0_2 + 0.185933616923197*G0_1_0 + 0.176147637085136*G0_1_1 + 0.234863516113514*G0_1_2 + 0.218553549716741*G0_2_0 + 0.234863516113514*G0_2_1 + 0.293579395141892*G0_2_2;
    A[2630] = 0.176147637085135*G0_0_0 + 0.185933616923197*G0_0_1 + 0.234863516113513*G0_0_2 + 0.185933616923198*G0_1_0 + 0.143527704291592*G0_1_1 + 0.21855354971674*G0_1_2 + 0.234863516113514*G0_2_0 + 0.218553549716742*G0_2_1 + 0.293579395141892*G0_2_2;
    A[2631] = 0.045667905910961*G0_0_0 + 0.0717638521457953*G0_0_1 + 0.120693751336111*G0_0_2 + 0.0717638521457965*G0_1_0 + 0.0456679059109594*G0_1_1 + 0.120693751336112*G0_1_2 + 0.120693751336112*G0_2_0 + 0.120693751336113*G0_2_1 + 0.195719596761263*G0_2_2;
    A[2632] = -0.0298472385060923*G0_0_0 - 0.0298472385060923*G0_0_1 - 0.0298472385060924*G0_0_2 - 0.0298472385060922*G0_1_0 - 0.0298472385060922*G0_1_1 - 0.0298472385060922*G0_1_2 - 0.034571909571909*G0_2_0 - 0.034571909571909*G0_2_1 - 0.034571909571909*G0_2_2;
    A[2633] = -0.0298472385060925*G0_0_0 + 0.00472467106581651*G0_2_0;
    A[2634] = 0.0117431758056758*G0_1_1 + 0.00227817610630099*G0_2_1;
    A[2635] = -0.00946499969937489*G0_2_2;
    A[2636] = -0.00913358118219296*G0_1_1 - 0.00583896797004467*G0_1_2 - 0.00583896797004449*G0_2_1 - 0.00254435475789638*G0_2_2;
    A[2637] = -0.0156575677409005*G0_1_1 - 0.0130805930502107*G0_1_2 - 0.0130805930502108*G0_2_1 - 0.010503618359521*G0_2_2;
    A[2638] = -0.000652398655871322*G0_1_1 + 0.00192457603481833*G0_1_2 + 0.00192457603481868*G0_2_1 + 0.00450155072550851*G0_2_2;
    A[2639] = 0.00329461321214827*G0_1_2 + 0.00329461321214794*G0_2_1 + 0.00658922642429643*G0_2_2;
    A[2640] = 0.0642612676032813*G0_0_0 + 0.0422754329004327*G0_0_2 + 0.0422754329004329*G0_2_0 + 0.0429604514890969*G0_2_2;
    A[2641] = 0.00880738185425678*G0_0_0 + 0.00649136662591556*G0_0_2 + 0.00649136662591475*G0_2_0 + 0.0035229527417029*G0_2_2;
    A[2642] = -0.000652398655870758*G0_0_0 - 0.00949240044292173*G0_0_2 - 0.00949240044292092*G0_2_0 - 0.0189848008858431*G0_2_2;
    A[2643] = 0.0032946132121484*G0_0_2 + 0.00329461321214786*G0_2_0 + 0.0292600797158094*G0_2_2;
    A[2644] = 0.128522535206562*G0_0_0 + 0.0422754329004325*G0_0_1 + 0.0422754329004326*G0_1_0 + 0.0845508658008652*G0_2_0 + 0.0429604514890966*G0_2_1;
    A[2645] = 0.037186723384639*G0_0_0 + 0.074340826836485*G0_0_1 + 0.0743408268364848*G0_1_0 + 0.104709984267274*G0_1_1 + 0.0808321934624001*G0_2_0 + 0.108232937008977*G0_2_1;
    A[2646] = -0.00619778723077289*G0_0_0 + 0.0188869410874626*G0_0_1 + 0.0188869410874619*G0_1_0 + 0.0567586830607665*G0_1_1 + 0.00939454064454071*G0_2_0 + 0.0377738821749237*G0_2_1;
    A[2647] = -0.00583896797004506*G0_0_1 - 0.00583896797004485*G0_1_0 - 0.0435476102793817*G0_1_1 - 0.00254435475789661*G0_2_0 - 0.0142875305635724*G0_2_1;
    A[2648] = 0.0642612676032808*G0_0_0 + 0.0642612676032808*G0_0_1 + 0.0219858347028487*G0_0_2 + 0.0642612676032804*G0_1_0 + 0.0642612676032803*G0_1_1 + 0.0219858347028485*G0_1_2 + 0.0219858347028478*G0_2_0 + 0.0219858347028479*G0_2_1 + 0.0226708532915129*G0_2_2;
    A[2649] = 0.00880738185425652*G0_0_0 + 0.00880738185425645*G0_0_1 + 0.00231601522834107*G0_0_2 + 0.00880738185425689*G0_1_0 + 0.00880738185425687*G0_1_1 + 0.00231601522834108*G0_1_2 + 0.00231601522834207*G0_2_0 + 0.00231601522834192*G0_2_1 - 0.00065239865587078*G0_2_2;
    A[2650] = -0.000652398655871103*G0_0_0 - 0.000652398655871091*G0_0_1 + 0.00884000178705046*G0_0_2 - 0.000652398655871341*G0_1_0 - 0.00065239865587141*G0_1_1 + 0.00884000178705058*G0_1_2 + 0.00884000178705*G0_2_0 + 0.00884000178705*G0_2_1 - 0.000652398655871336*G0_2_2;
    A[2651] = -0.00329461321214783*G0_0_2 - 0.00329461321214791*G0_1_2 - 0.00329461321214796*G0_2_0 - 0.00329461321214792*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[2652] = 0.128522535206561*G0_0_0 + 0.086247102306129*G0_0_1 + 0.128522535206561*G0_0_2 + 0.0862471023061283*G0_1_0 + 0.0439716694056962*G0_1_1 + 0.0862471023061284*G0_1_2 + 0.043971669405696*G0_2_0 + 0.044656687994361*G0_2_1 + 0.0439716694056962*G0_2_2;
    A[2653] = 0.0371867233846396*G0_0_0 - 0.0371541034518458*G0_0_1 + 0.0371867233846395*G0_0_2 - 0.0371541034518455*G0_1_0 - 0.00678494602105665*G0_1_1 - 0.0371541034518458*G0_1_2 - 0.0436454700777608*G0_2_0 - 0.00975335990526928*G0_2_1 - 0.0436454700777611*G0_2_2;
    A[2654] = -0.00619778723077359*G0_0_0 - 0.0250847283182352*G0_0_1 - 0.00619778723077334*G0_0_2 - 0.0250847283182355*G0_1_0 + 0.012787013655069*G0_1_1 - 0.0250847283182353*G0_1_2 - 0.0155923278753138*G0_2_0 + 0.00329461321214758*G0_2_1 - 0.0155923278753135*G0_2_2;
    A[2655] = 0.00583896797004425*G0_0_1 + 0.00583896797004481*G0_1_0 - 0.0318696743392923*G0_1_1 + 0.00583896797004486*G0_1_2 + 0.00254435475789661*G0_2_0 - 0.00590420783563124*G0_2_1 + 0.00254435475789659*G0_2_2;
    A[2656] = 0.0920860702761737*G0_0_0 + 0.134361503176606*G0_0_1 + 0.134361503176606*G0_0_2 + 0.134361503176606*G0_1_0 + 0.176636936077038*G0_1_1 + 0.176636936077038*G0_1_2 + 0.128000616281865*G0_2_0 + 0.1273155976932*G0_2_1 + 0.1273155976932*G0_2_2;
    A[2657] = -0.0622388317700807*G0_0_0 - 0.250814663249555*G0_0_1 - 0.250814663249555*G0_0_2 - 0.250814663249556*G0_1_0 - 0.334680510461756*G0_1_1 - 0.334680510461756*G0_1_2 - 0.160783648739376*G0_2_0 - 0.166948816037355*G0_2_1 - 0.166948816037355*G0_2_2;
    A[2658] = -0.0622388317700816*G0_0_0 + 0.188575831479475*G0_0_1 + 0.188575831479475*G0_0_2 + 0.188575831479475*G0_1_0 + 0.104709984267274*G0_1_1 + 0.104709984267273*G0_1_2 + 0.0985448169692943*G0_2_0 + 0.182410664181495*G0_2_1 + 0.182410664181495*G0_2_2;
    A[2659] = 0.0920860702761739*G0_0_0 - 0.0422754329004322*G0_0_1 - 0.0422754329004322*G0_0_2 - 0.0422754329004319*G0_1_0 - 0.0359145460056907*G0_2_0 - 0.0429604514890964*G0_2_1 - 0.0429604514890962*G0_2_2;
    A[2660] = 0.0391439193522525*G0_0_0 + 0.0678494602105713*G0_0_1 + 0.135698920421141*G0_0_2 + 0.067849460210569*G0_1_0 + 0.104709984267273*G0_1_2 + 0.13569892042114*G0_2_0 + 0.104709984267275*G0_2_1 + 0.209419968534548*G0_2_2;
    A[2661] = 0.0195719596761263*G0_0_0 + 0.0606730749959911*G0_0_1 + 0.074862745761183*G0_0_2 + 0.0606730749959919*G0_1_0 + 0.0929668084615995*G0_1_1 + 0.121346149991982*G0_1_2 + 0.0748627457611837*G0_2_0 + 0.121346149991983*G0_2_1 + 0.149725491522366*G0_2_2;
    A[2662] = -0.00554538857490207*G0_0_1 - 0.00619778723077418*G0_0_2 - 0.00554538857490302*G0_1_0 + 0.0391439193522526*G0_1_1 + 0.0264221455627691*G0_1_2 - 0.00619778723077359*G0_2_0 + 0.0264221455627702*G0_2_1 + 0.0137003717732875*G0_2_2;
    A[2663] = -0.004892989919031*G0_0_0 + 0.0141896707651921*G0_0_1 + 0.0283793415303838*G0_0_2 + 0.0141896707651922*G0_1_0 + 0.0283793415303838*G0_1_2 + 0.0283793415303833*G0_2_0 + 0.028379341530383*G0_2_1 + 0.056758683060767*G0_2_2;
    A[2664] = -0.0102752788299667*G0_0_1 - 0.0181040627004162*G0_0_2 - 0.0102752788299666*G0_1_0 + 0.0195719596761255*G0_1_1 + 0.00146789697571044*G0_1_2 - 0.0181040627004174*G0_2_0 + 0.00146789697570865*G0_2_1 - 0.016636165724707*G0_2_2;
    A[2665] = -0.000652398655869935*G0_0_1 - 0.00130479731174185*G0_0_2 - 0.000652398655871223*G0_1_0 - 0.0127217737894829*G0_1_2 - 0.00130479731174162*G0_2_0 - 0.0127217737894813*G0_2_1 - 0.0254435475789651*G0_2_2;
    A[2666] = 0.0391439193522512*G0_0_0 - 0.0287055408583185*G0_0_1 - 0.0965550010688887*G0_0_2 - 0.0287055408583178*G0_1_0 - 0.0965550010688883*G0_1_1 - 0.0596944770121837*G0_1_2 - 0.0965550010688891*G0_2_0 - 0.0596944770121844*G0_2_1 - 0.0228339529554799*G0_2_2;
    A[2667] = 0.019571959676127*G0_0_0 - 0.0411011153198644*G0_0_1 - 0.0552907860850556*G0_0_2 - 0.0411011153198652*G0_1_0 - 0.00880738185425686*G0_1_1 + 0.00538228891093459*G0_1_2 - 0.0552907860850556*G0_2_0 + 0.00538228891093588*G0_2_1 + 0.019571959676126*G0_2_2;
    A[2668] = 0.00554538857490261*G0_0_1 + 0.00619778723077253*G0_0_2 + 0.00554538857490293*G0_1_0 + 0.0502346965020573*G0_1_1 + 0.0381653213684455*G0_1_2 + 0.00619778723077343*G0_2_0 + 0.0381653213684451*G0_2_1 + 0.0260959462348341*G0_2_2;
    A[2669] = -0.00489298991903075*G0_0_0 - 0.0190826606842223*G0_0_1 - 0.0332723314494153*G0_0_2 - 0.019082660684223*G0_1_0 - 0.0332723314494143*G0_1_1 - 0.0190826606842239*G0_1_2 - 0.0332723314494143*G0_2_0 - 0.0190826606842229*G0_2_1 - 0.00489298991903143*G0_2_2;
    A[2670] = 0.0102752788299651*G0_0_1 + 0.0181040627004166*G0_0_2 + 0.0102752788299666*G0_1_0 + 0.040122517336058*G0_1_1 + 0.029847238506093*G0_1_2 + 0.0181040627004173*G0_2_0 + 0.0298472385060918*G0_2_1 + 0.0195719596761265*G0_2_2;
    A[2671] = 0.000652398655871125*G0_0_1 + 0.00130479731174149*G0_0_2 + 0.000652398655871196*G0_1_0 + 0.00130479731174238*G0_1_1 - 0.0107645778218694*G0_1_2 + 0.00130479731174152*G0_2_0 - 0.0107645778218692*G0_2_1 - 0.0228339529554815*G0_2_2;
    A[2672] = -0.29227459783015*G0_0_0 - 0.22442513761958*G0_0_1 - 0.0972073997247602*G0_0_2 - 0.224425137619579*G0_1_0 - 0.156575677409008*G0_1_1 - 0.134067923781463*G0_1_2 - 0.0972073997247588*G0_2_0 - 0.134067923781464*G0_2_1 - 0.0880738185425674*G0_2_2;
    A[2673] = 0.456026660453738*G0_0_0 + 0.22801333022687*G0_0_1 + 0.228013330226869*G0_0_2 + 0.228013330226868*G0_1_0 + 0.0929668084615977*G0_1_1 + 0.36305985199214*G0_1_2 + 0.228013330226869*G0_2_0 + 0.363059851992139*G0_2_1 + 0.0929668084615979*G0_2_2;
    A[2674] = -0.29227459783015*G0_0_0 - 0.0678494602105714*G0_0_1 - 0.195067198105391*G0_0_2 - 0.0678494602105699*G0_1_0 - 0.104709984267274*G0_1_2 - 0.195067198105391*G0_2_0 - 0.104709984267274*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2675] = -0.0088073818542567*G0_0_0 + 0.0053822889109349*G0_0_1 - 0.0411011153198647*G0_0_2 + 0.00538228891093502*G0_1_0 + 0.0195719596761266*G0_1_1 - 0.0552907860850569*G0_1_2 - 0.0411011153198656*G0_2_0 - 0.0552907860850567*G0_2_1 + 0.0195719596761268*G0_2_2;
    A[2676] = -0.00880738185425672*G0_0_0 - 0.0141896707651918*G0_0_1 + 0.032293733465609*G0_0_2 - 0.0141896707651914*G0_1_0 - 0.0283793415303828*G0_1_2 + 0.0322937334656089*G0_2_0 - 0.0283793415303831*G0_2_1 + 0.0929668084616009*G0_2_2;
    A[2677] = 0.00130479731174173*G0_0_0 + 0.00065239865587024*G0_0_1 + 0.000652398655870258*G0_0_2 + 0.000652398655870634*G0_1_0 + 0.0127217737894819*G0_1_2 + 0.000652398655870867*G0_2_0 + 0.0127217737894816*G0_2_1 - 0.088073818542569*G0_2_2;
    A[2678] = -0.584549195660301*G0_0_0 - 0.32163253734434*G0_0_1 - 0.448850275239159*G0_0_2 - 0.321632537344338*G0_1_0 - 0.268135847562927*G0_1_1 - 0.290643601190472*G0_1_2 - 0.194414799449519*G0_2_0 - 0.222141742324032*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2679] = 0.912053320907479*G0_0_0 + 0.456026660453738*G0_0_1 + 0.456026660453739*G0_0_2 + 0.456026660453738*G0_1_0 + 0.726119703984278*G0_1_1 + 0.456026660453737*G0_1_2 + 0.456026660453739*G0_2_0 + 0.456026660453737*G0_2_1 + 0.726119703984279*G0_2_2;
    A[2680] = -0.584549195660301*G0_0_0 - 0.262916658315961*G0_0_1 - 0.135698920421141*G0_0_2 - 0.262916658315961*G0_1_0 - 0.209419968534548*G0_1_1 - 0.104709984267274*G0_1_2 - 0.390134396210782*G0_2_0 - 0.290643601190472*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2681] = -0.0371867233846397*G0_0_0 + 0.186912214907006*G0_0_1 + 0.0376760223765431*G0_0_2 + 0.186912214907003*G0_1_0 - 0.0371867233846405*G0_1_1 + 0.140428810676203*G0_1_2 + 0.140428810676203*G0_2_0 + 0.0376760223765437*G0_2_1 + 0.0655660649150218*G0_2_2;
    A[2682] = -0.0371867233846395*G0_0_0 - 0.224098938291644*G0_0_1 - 0.0748627457611825*G0_0_2 - 0.224098938291642*G0_1_0 - 0.448197876583288*G0_1_1 - 0.121346149991982*G0_1_2 - 0.177615534060843*G0_2_0 - 0.326851726591306*G0_2_1 - 0.149725491522365*G0_2_2;
    A[2683] = 0.0123955744615466*G0_0_0 + 0.00619778723077325*G0_0_1 + 0.00619778723077375*G0_0_2 + 0.0061977872307735*G0_1_0 + 0.172233245149911*G0_1_1 - 0.0264221455627694*G0_1_2 + 0.00619778723077277*G0_2_0 + 0.0714376528178601*G0_2_1 - 0.0137003717732881*G0_2_2;
    A[2684] = -0.0391439193522522*G0_0_0 + 0.249542485870608*G0_0_1 + 0.44526208263187*G0_0_2 + 0.249542485870605*G0_1_0 + 0.352295274170268*G0_1_1 + 0.249542485870607*G0_1_2 + 0.445262082631866*G0_2_0 + 0.249542485870605*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2685] = -0.0391439193522519*G0_0_0 - 0.288686405222861*G0_0_1 - 0.484406001984123*G0_0_2 - 0.288686405222856*G0_1_0 - 0.185933616923196*G0_1_1 - 0.48440600198412*G0_1_2 - 0.484406001984117*G0_2_0 - 0.484406001984122*G0_2_1 - 0.782878387045045*G0_2_2;
    A[2686] = -0.0391439193522529*G0_0_0 - 0.0195719596761259*G0_0_1 - 0.019571959676126*G0_0_2 - 0.0195719596761264*G0_1_0 - 0.234863516113511*G0_1_1 - 0.141896707651913*G0_1_2 - 0.0195719596761276*G0_2_0 - 0.141896707651912*G0_2_1 - 0.0489298991903142*G0_2_2;
    A[2687] = 0.00978597983806193*G0_0_0 + 0.00489298991903126*G0_0_1 + 0.00489298991902984*G0_0_2 + 0.00489298991903021*G0_1_0 - 0.0391439193522524*G0_1_1 + 0.0538228891093469*G0_1_2 + 0.00489298991903004*G0_2_0 + 0.0538228891093481*G0_2_1 + 0.146789697570947*G0_2_2;
    A[2688] = 0.0189195610202551*G0_0_0 + 0.0189195610202551*G0_0_1 + 0.0189195610202552*G0_0_2 + 0.0189195610202551*G0_1_0 + 0.0189195610202551*G0_1_1 + 0.0189195610202551*G0_1_2 + 0.0137421252872638*G0_2_0 + 0.0137421252872638*G0_2_1 + 0.0137421252872639*G0_2_2;
    A[2689] = 0.0335985307773502*G0_0_0 - 0.0143945239431347*G0_2_0;
    A[2690] = 0.0189195610202555*G0_1_1 + 0.00517743573299138*G0_2_1;
    A[2691] = -0.0137421252872638*G0_2_2;
    A[2692] = -0.0306627368259309*G0_1_1 - 0.0212681961813903*G0_1_2 - 0.0212681961813907*G0_2_1 - 0.0118736555368498*G0_2_2;
    A[2693] = -0.00130479731174202*G0_1_1 - 0.0012395574461551*G0_1_2 - 0.00123955744615444*G0_2_1 - 0.00117431758056806*G0_2_2;
    A[2694] = -0.01500516908503*G0_1_1 - 0.0149399292194427*G0_1_2 - 0.0149399292194433*G0_2_1 - 0.0148746893538555*G0_2_2;
    A[2695] = 0.0093945406445402*G0_1_2 + 0.00939454064454058*G0_2_1 + 0.0187890812890808*G0_2_2;
    A[2696] = -0.128522535206562*G0_0_0 - 0.0962940416065405*G0_0_2 - 0.0962940416065402*G0_2_0 - 0.12473862300251*G0_2_2;
    A[2697] = 0.0476251018785737*G0_0_0 + 0.0623693115012551*G0_0_2 + 0.0623693115012547*G0_2_0 + 0.12473862300251*G0_2_2;
    A[2698] = -0.0150051690850299*G0_0_0 - 0.0377738821749229*G0_0_2 - 0.0377738821749231*G0_2_0 - 0.127087258163645*G0_2_2;
    A[2699] = 0.00939454064454022*G0_0_2 + 0.00939454064454069*G0_2_0 + 0.0641307878721065*G0_2_2;
    A[2700] = -0.257045070413124*G0_0_0 - 0.0962940416065403*G0_0_1 - 0.0962940416065404*G0_1_0 - 0.192588083213081*G0_2_0 - 0.12473862300251*G0_2_1;
    A[2701] = 0.121346149991983*G0_0_0 - 0.0733296089198856*G0_0_1 - 0.0733296089198859*G0_1_0 - 0.271397840842282*G0_1_1 - 0.0109602974186308*G0_2_0 - 0.146659217839772*G0_2_1;
    A[2702] = -0.0528442911255409*G0_0_0 + 0.0359471659384847*G0_0_1 + 0.0359471659384848*G0_1_0 + 0.20941996853455*G0_1_1 - 0.00182671623643834*G0_2_0 + 0.0823327103709041*G0_2_1;
    A[2703] = -0.0212681961813901*G0_0_1 - 0.0212681961813903*G0_1_0 - 0.111886369481854*G0_1_1 - 0.0118736555368498*G0_2_0 - 0.0477555816097479*G0_2_1;
    A[2704] = -0.0306627368259308*G0_0_0 - 0.0306627368259307*G0_0_1 - 0.00939454064454057*G0_0_2 - 0.0306627368259312*G0_1_0 - 0.0306627368259312*G0_1_1 - 0.0093945406445408*G0_1_2 - 0.00939454064454025*G0_2_0 - 0.00939454064454016*G0_2_1;
    A[2705] = -0.00130479731174194*G0_0_0 - 0.00130479731174194*G0_0_1 - 6.52398655872634e-05*G0_0_2 - 0.00130479731174131*G0_1_0 - 0.0013047973117413*G0_1_1 - 6.52398655869142e-05*G0_1_2 - 6.52398655873354e-05*G0_2_0 - 6.52398655874943e-05*G0_2_1;
    A[2706] = -0.0150051690850296*G0_0_0 - 0.0150051690850296*G0_0_1 - 6.52398655868526e-05*G0_0_2 - 0.0150051690850301*G0_1_0 - 0.01500516908503*G0_1_1 - 6.52398655869913e-05*G0_1_2 - 6.52398655863961e-05*G0_2_0 - 6.52398655863979e-05*G0_2_1;
    A[2707] = -0.00939454064454072*G0_0_2 - 0.00939454064454062*G0_1_2 - 0.00939454064454061*G0_2_0 - 0.00939454064454058*G0_2_1;
    A[2708] = -0.0613254736518616*G0_0_0 - 0.0400572774704713*G0_0_1 - 0.0613254736518617*G0_0_2 - 0.0400572774704715*G0_1_0 - 0.0187890812890809*G0_1_1 - 0.0400572774704717*G0_1_2 - 0.0187890812890807*G0_2_0 - 0.00939454064454015*G0_2_1 - 0.0187890812890808*G0_2_2;
    A[2709] = -0.0254435475789639*G0_0_0 + 0.013635131907701*G0_0_1 - 0.0254435475789637*G0_0_2 + 0.0136351319077014*G0_1_0 - 0.000130479731174247*G0_1_1 + 0.0136351319077015*G0_1_2 + 0.0148746893538553*G0_2_0 - 6.52398655876292e-05*G0_2_1 + 0.0148746893538554*G0_2_2;
    A[2710] = -0.0528442911255402*G0_0_0 - 0.0137656116388752*G0_0_1 - 0.0528442911255404*G0_0_2 - 0.0137656116388753*G0_1_0 - 0.00013047973117416*G0_1_1 - 0.0137656116388754*G0_1_2 + 0.00117431758056802*G0_2_0 - 6.52398655862523e-05*G0_2_1 + 0.00117431758056778*G0_2_2;
    A[2711] = 0.0212681961813904*G0_0_1 + 0.0212681961813903*G0_1_0 - 0.0187890812890812*G0_1_1 + 0.0212681961813904*G0_1_2 + 0.0118736555368497*G0_2_0 - 0.00939454064454102*G0_2_1 + 0.01187365553685*G0_2_2;
    A[2712] = -0.0693499771190735*G0_0_0 - 0.0906181733004637*G0_0_1 - 0.0906181733004637*G0_0_2 - 0.0906181733004635*G0_1_0 - 0.111886369481854*G0_1_1 - 0.111886369481854*G0_1_2 - 0.0547362472275657*G0_2_0 - 0.0641307878721059*G0_2_1 - 0.0641307878721061*G0_2_2;
    A[2713] = 0.0846813455320383*G0_0_0 + 0.173472802596064*G0_0_1 + 0.173472802596064*G0_0_2 + 0.173472802596064*G0_1_0 + 0.209419968534549*G0_1_1 + 0.209419968534549*G0_1_2 + 0.0893133759887216*G0_2_0 + 0.127087258163645*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2714] = -0.00339247301052789*G0_0_0 - 0.198068231922396*G0_0_1 - 0.198068231922396*G0_0_2 - 0.198068231922396*G0_1_0 - 0.271397840842282*G0_1_1 - 0.271397840842282*G0_1_2 - 0.0623693115012546*G0_2_0 - 0.12473862300251*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2715] = -0.0644569872000425*G0_0_0 + 0.0962940416065401*G0_0_1 + 0.0962940416065401*G0_0_2 + 0.0962940416065401*G0_1_0 + 0.0284445813959696*G0_2_0 + 0.12473862300251*G0_2_1 + 0.124738623002509*G0_2_2;
    A[2716] = 0.0521918924696704*G0_0_0 - 0.135698920421141*G0_0_1 - 0.271397840842282*G0_0_2 - 0.135698920421142*G0_1_0 - 0.271397840842282*G0_1_2 - 0.271397840842282*G0_2_0 - 0.271397840842282*G0_2_1 - 0.542795681684564*G0_2_2;
    A[2717] = -0.0880738185425685*G0_0_0 + 0.00913358118219215*G0_0_1 + 0.0459941052388959*G0_0_2 + 0.00913358118219142*G0_1_0 - 0.185933616923199*G0_1_1 - 0.0812236326559246*G0_1_2 + 0.0459941052388961*G0_2_0 - 0.0812236326559231*G0_2_1 + 0.0234863516113507*G0_2_2;
    A[2718] = -0.0378391220405108*G0_0_1 - 0.05284429112554*G0_0_2 - 0.037839122040511*G0_1_0 + 0.0521918924696696*G0_1_1 - 0.000652398655870949*G0_1_2 - 0.0528442911255411*G0_2_0 - 0.000652398655871281*G0_2_1 - 0.0534966897814114*G0_2_2;
    A[2719] = -0.0228339529554812*G0_0_0 + 0.0368605240567041*G0_0_1 + 0.0737210481134078*G0_0_2 + 0.0368605240567044*G0_1_0 + 0.104709984267274*G0_1_2 + 0.0737210481134101*G0_2_0 + 0.104709984267275*G0_2_1 + 0.20941996853455*G0_2_2;
    A[2720] = -0.0120693751336109*G0_0_1 - 0.0127217737894825*G0_0_2 - 0.0120693751336109*G0_1_0 + 0.0260959462348354*G0_1_1 + 0.0198981590040619*G0_1_2 - 0.0127217737894818*G0_2_0 + 0.0198981590040621*G0_2_1 + 0.0137003717732883*G0_2_2;
    A[2721] = -0.0150051690850305*G0_0_1 - 0.0300103381700599*G0_0_2 - 0.0150051690850309*G0_1_0 - 0.0528442911255411*G0_1_2 - 0.0300103381700608*G0_2_0 - 0.0528442911255405*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2722] = -0.0456679059109602*G0_0_0 - 0.00782878387045057*G0_0_1 + 0.0300103381700596*G0_0_2 - 0.00782878387044984*G0_1_0 + 0.0300103381700604*G0_1_1 + 0.0150051690850306*G0_1_2 + 0.0300103381700602*G0_2_0 + 0.0150051690850295*G0_2_1;
    A[2723] = -0.088073818542569*G0_0_0 + 0.000652398655870484*G0_0_1 + 0.0127217737894808*G0_0_2 + 0.000652398655871*G0_1_0 + 0.00130479731174123*G0_1_1 + 0.000652398655870598*G0_1_2 + 0.0127217737894813*G0_2_0 + 0.000652398655870912*G0_2_1;
    A[2724] = 0.0378391220405104*G0_0_1 + 0.0528442911255408*G0_0_2 + 0.037839122040511*G0_1_0 + 0.0300103381700602*G0_1_1 + 0.0150051690850304*G0_1_2 + 0.0528442911255411*G0_2_0 + 0.0150051690850303*G0_2_1;
    A[2725] = -0.0228339529554812*G0_0_0 - 0.0107645778218699*G0_0_1 + 0.00130479731174286*G0_0_2 - 0.0107645778218701*G0_1_0 + 0.00130479731174092*G0_1_1 + 0.000652398655870392*G0_1_2 + 0.00130479731174214*G0_2_0 + 0.000652398655870807*G0_2_1;
    A[2726] = 0.0120693751336119*G0_0_1 + 0.0127217737894826*G0_0_2 + 0.0120693751336109*G0_1_0 + 0.00130479731174249*G0_1_1 + 0.000652398655870856*G0_1_2 + 0.0127217737894818*G0_2_0 + 0.000652398655871001*G0_2_1;
    A[2727] = 0.0150051690850297*G0_0_1 + 0.0300103381700601*G0_0_2 + 0.0150051690850309*G0_1_0 + 0.0300103381700603*G0_1_1 + 0.0150051690850306*G0_1_2 + 0.030010338170061*G0_2_0 + 0.0150051690850301*G0_2_1;
    A[2728] = 0.12787013655069*G0_0_0 + 0.09003101451018*G0_0_1 + 0.0378391220405104*G0_0_2 + 0.0900310145101802*G0_1_0 + 0.0521918924696699*G0_1_1 + 0.0528442911255409*G0_1_2 + 0.0378391220405102*G0_2_0 + 0.0528442911255407*G0_2_1;
    A[2729] = -0.292274597830149*G0_0_0 - 0.195067198105391*G0_0_1 - 0.06784946021057*G0_0_2 - 0.19506719810539*G0_1_0 - 0.185933616923198*G0_1_1 - 0.104709984267274*G0_1_2 - 0.0678494602105707*G0_2_0 - 0.104709984267274*G0_2_1;
    A[2730] = 0.323589733311952*G0_0_0 + 0.135698920421141*G0_0_1 + 0.135698920421142*G0_0_2 + 0.135698920421141*G0_1_0 + 0.271397840842283*G0_1_2 + 0.135698920421141*G0_2_0 + 0.271397840842281*G0_2_1;
    A[2731] = 0.0502346965020577*G0_0_0 + 0.0381653213684466*G0_0_1 + 0.00554538857490303*G0_0_2 + 0.0381653213684454*G0_1_0 + 0.0260959462348346*G0_1_1 + 0.00619778723077269*G0_1_2 + 0.00554538857490228*G0_2_0 + 0.00619778723077391*G0_2_1;
    A[2732] = -0.0965550010688895*G0_0_0 - 0.0368605240567045*G0_0_1 - 0.0678494602105714*G0_0_2 - 0.0368605240567042*G0_1_0 - 0.104709984267275*G0_1_2 - 0.06784946021057*G0_2_0 - 0.104709984267276*G0_2_1;
    A[2733] = 0.0300103381700597*G0_0_0 + 0.0150051690850303*G0_0_1 + 0.0378391220405107*G0_0_2 + 0.0150051690850303*G0_1_0 + 0.0528442911255413*G0_1_2 + 0.0378391220405096*G0_2_0 + 0.0528442911255404*G0_2_1;
    A[2734] = 0.255740273101381*G0_0_0 + 0.127870136550691*G0_0_1 + 0.18006202902036*G0_0_2 + 0.127870136550691*G0_1_0 + 0.105688582251081*G0_1_1 + 0.10503618359521*G0_1_2 + 0.0756782440810207*G0_2_0 + 0.0528442911255401*G0_2_1 + 0.105688582251081*G0_2_2;
    A[2735] = -0.584549195660301*G0_0_0 - 0.262916658315961*G0_0_1 - 0.390134396210782*G0_0_2 - 0.262916658315961*G0_1_0 - 0.209419968534548*G0_1_1 - 0.290643601190472*G0_1_2 - 0.135698920421141*G0_2_0 - 0.104709984267274*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2736] = 0.647179466623904*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842282*G0_1_0 + 0.542795681684564*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684563*G0_2_2;
    A[2737] = 0.17223324514991*G0_0_0 + 0.00619778723077271*G0_0_1 + 0.0714376528178598*G0_0_2 + 0.00619778723077378*G0_1_0 + 0.0123955744615467*G0_1_1 + 0.00619778723077371*G0_1_2 - 0.0264221455627692*G0_2_0 + 0.00619778723077309*G0_2_1 - 0.0137003717732882*G0_2_2;
    A[2738] = -0.268135847562928*G0_0_0 + 0.0534966897814122*G0_0_1 - 0.0459941052388962*G0_0_2 + 0.0534966897814109*G0_1_0 - 0.209419968534551*G0_1_1 + 0.0812236326559242*G0_1_2 + 0.0225077536275449*G0_2_0 - 0.104709984267275*G0_2_1 - 0.0234863516113504*G0_2_2;
    A[2739] = 0.105688582251081*G0_0_0 - 0.0221815542996099*G0_0_1 + 0.0528442911255405*G0_0_2 - 0.0221815542996094*G0_1_0 + 0.105688582251081*G0_1_1 + 0.000652398655870986*G0_1_2 + 0.00065239865587049*G0_2_0 + 0.0528442911255407*G0_2_1 + 0.0534966897814116*G0_2_2;
    A[2740] = 0.14352770429159*G0_0_0 - 0.042405912631608*G0_0_1 - 0.0750258454251513*G0_0_2 - 0.0424059126316065*G0_1_0 - 0.0521918924696686*G0_1_1 - 0.0260959462348344*G0_1_2 - 0.0750258454251491*G0_2_0 - 0.0260959462348352*G0_2_1;
    A[2741] = -0.1500516908503*G0_0_0 + 0.185933616923199*G0_0_1 + 0.316413348097375*G0_0_2 + 0.185933616923198*G0_1_0 + 0.371867233846398*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097371*G0_2_0 + 0.185933616923198*G0_2_1;
    A[2742] = 0.176147637085137*G0_0_0 - 0.00978597983806163*G0_0_1 - 0.0587158790283784*G0_0_2 - 0.0097859798380626*G0_1_0 - 0.0521918924696707*G0_1_1 - 0.0260959462348359*G0_1_2 - 0.0587158790283777*G0_2_0 - 0.0260959462348346*G0_2_1;
    A[2743] = 0.0456679059109618*G0_0_0 - 0.0260959462348354*G0_0_1 - 0.0750258454251482*G0_0_2 - 0.0260959462348341*G0_1_0 - 0.0521918924696701*G0_1_1 - 0.0260959462348334*G0_1_2 - 0.0750258454251516*G0_2_0 - 0.0260959462348354*G0_2_1;
    A[2744] = -0.0298472385060922*G0_0_0 - 0.0298472385060922*G0_0_1 - 0.0298472385060922*G0_0_2 - 0.0298472385060924*G0_1_0 - 0.0298472385060924*G0_1_1 - 0.0298472385060925*G0_1_2 - 0.0345719095719093*G0_2_0 - 0.0345719095719093*G0_2_1 - 0.0345719095719093*G0_2_2;
    A[2745] = 0.0117431758056758*G0_0_0 + 0.00227817610630136*G0_2_0;
    A[2746] = -0.0298472385060926*G0_1_1 + 0.00472467106581693*G0_2_1;
    A[2747] = -0.0094649996993748*G0_2_2;
    A[2748] = 0.0642612676032811*G0_1_1 + 0.0422754329004323*G0_1_2 + 0.0422754329004324*G0_2_1 + 0.0429604514890967*G0_2_2;
    A[2749] = 0.0088073818542567*G0_1_1 + 0.00649136662591519*G0_1_2 + 0.0064913666259154*G0_2_1 + 0.00352295274170295*G0_2_2;
    A[2750] = -0.000652398655870439*G0_1_1 - 0.00949240044292134*G0_1_2 - 0.00949240044292082*G0_2_1 - 0.0189848008858424*G0_2_2;
    A[2751] = 0.00329461321214846*G0_1_2 + 0.00329461321214787*G0_2_1 + 0.0292600797158091*G0_2_2;
    A[2752] = -0.00913358118219186*G0_0_0 - 0.00583896797004397*G0_0_2 - 0.00583896797004421*G0_2_0 - 0.00254435475789565*G0_2_2;
    A[2753] = -0.0156575677409007*G0_0_0 - 0.0130805930502106*G0_0_2 - 0.0130805930502102*G0_2_0 - 0.0105036183595209*G0_2_2;
    A[2754] = -0.000652398655870712*G0_0_0 + 0.00192457603481886*G0_0_2 + 0.0019245760348191*G0_2_0 + 0.00450155072550932*G0_2_2;
    A[2755] = 0.00329461321214823*G0_0_2 + 0.00329461321214803*G0_2_0 + 0.0065892264242959*G0_2_2;
    A[2756] = -0.0435476102793806*G0_0_0 - 0.00583896797004411*G0_0_1 - 0.00583896797004442*G0_1_0 - 0.014287530563572*G0_2_0 - 0.00254435475789597*G0_2_1;
    A[2757] = 0.0567586830607654*G0_0_0 + 0.018886941087461*G0_0_1 + 0.018886941087462*G0_1_0 - 0.00619778723077392*G0_1_1 + 0.0377738821749232*G0_2_0 + 0.00939454064453988*G0_2_1;
    A[2758] = 0.104709984267274*G0_0_0 + 0.0743408268364859*G0_0_1 + 0.0743408268364855*G0_1_0 + 0.0371867233846402*G0_1_1 + 0.108232937008977*G0_2_0 + 0.0808321934624012*G0_2_1;
    A[2759] = 0.0422754329004316*G0_0_1 + 0.0422754329004324*G0_1_0 + 0.128522535206561*G0_1_1 + 0.0429604514890967*G0_2_0 + 0.0845508658008643*G0_2_1;
    A[2760] = 0.0642612676032805*G0_0_0 + 0.0642612676032805*G0_0_1 + 0.0219858347028486*G0_0_2 + 0.064261267603281*G0_1_0 + 0.0642612676032811*G0_1_1 + 0.0219858347028488*G0_1_2 + 0.0219858347028483*G0_2_0 + 0.0219858347028482*G0_2_1 + 0.0226708532915132*G0_2_2;
    A[2761] = 0.00880738185425653*G0_0_0 + 0.00880738185425648*G0_0_1 + 0.00231601522834094*G0_0_2 + 0.0088073818542558*G0_1_0 + 0.00880738185425571*G0_1_1 + 0.00231601522834104*G0_1_2 + 0.00231601522834075*G0_2_0 + 0.00231601522834085*G0_2_1 - 0.000652398655871238*G0_2_2;
    A[2762] = -0.000652398655870782*G0_0_0 - 0.000652398655870793*G0_0_1 + 0.00884000178705058*G0_0_2 - 0.000652398655870553*G0_1_0 - 0.00065239865587054*G0_1_1 + 0.00884000178705027*G0_1_2 + 0.00884000178705071*G0_2_0 + 0.00884000178705062*G0_2_1 - 0.00065239865587107*G0_2_2;
    A[2763] = -0.00329461321214799*G0_0_2 - 0.00329461321214767*G0_1_2 - 0.00329461321214805*G0_2_0 - 0.00329461321214803*G0_2_1 + 0.022670853291513*G0_2_2;
    A[2764] = 0.176636936077037*G0_0_0 + 0.134361503176605*G0_0_1 + 0.176636936077037*G0_0_2 + 0.134361503176605*G0_1_0 + 0.0920860702761731*G0_1_1 + 0.134361503176606*G0_1_2 + 0.1273155976932*G0_2_0 + 0.128000616281865*G0_2_1 + 0.1273155976932*G0_2_2;
    A[2765] = -0.334680510461755*G0_0_0 - 0.250814663249555*G0_0_1 - 0.334680510461755*G0_0_2 - 0.250814663249556*G0_1_0 - 0.0622388317700801*G0_1_1 - 0.250814663249556*G0_1_2 - 0.166948816037355*G0_2_0 - 0.160783648739376*G0_2_1 - 0.166948816037355*G0_2_2;
    A[2766] = 0.104709984267274*G0_0_0 + 0.188575831479475*G0_0_1 + 0.104709984267274*G0_0_2 + 0.188575831479475*G0_1_0 - 0.062238831770082*G0_1_1 + 0.188575831479475*G0_1_2 + 0.182410664181495*G0_2_0 + 0.0985448169692952*G0_2_1 + 0.182410664181495*G0_2_2;
    A[2767] = -0.0422754329004322*G0_0_1 - 0.0422754329004323*G0_1_0 + 0.0920860702761739*G0_1_1 - 0.0422754329004322*G0_1_2 - 0.0429604514890966*G0_2_0 - 0.0359145460056916*G0_2_1 - 0.0429604514890965*G0_2_2;
    A[2768] = 0.0439716694056967*G0_0_0 + 0.0862471023061287*G0_0_1 + 0.0862471023061288*G0_0_2 + 0.0862471023061297*G0_1_0 + 0.128522535206562*G0_1_1 + 0.128522535206562*G0_1_2 + 0.0446566879943615*G0_2_0 + 0.0439716694056966*G0_2_1 + 0.0439716694056967*G0_2_2;
    A[2769] = -0.00678494602105735*G0_0_0 - 0.0371541034518463*G0_0_1 - 0.0371541034518463*G0_0_2 - 0.0371541034518466*G0_1_0 + 0.0371867233846392*G0_1_1 + 0.0371867233846393*G0_1_2 - 0.00975335990526996*G0_2_0 - 0.0436454700777611*G0_2_1 - 0.0436454700777612*G0_2_2;
    A[2770] = 0.0127870136550693*G0_0_0 - 0.0250847283182346*G0_0_1 - 0.0250847283182346*G0_0_2 - 0.0250847283182346*G0_1_0 - 0.00619778723077212*G0_1_1 - 0.00619778723077213*G0_1_2 + 0.00329461321214825*G0_2_0 - 0.0155923278753133*G0_2_1 - 0.0155923278753133*G0_2_2;
    A[2771] = -0.0318696743392921*G0_0_0 + 0.00583896797004415*G0_0_1 + 0.00583896797004424*G0_0_2 + 0.00583896797004375*G0_1_0 - 0.00590420783563161*G0_2_0 + 0.00254435475789585*G0_2_1 + 0.00254435475789598*G0_2_2;
    A[2772] = 0.0391439193522519*G0_0_0 - 0.00554538857490248*G0_0_1 + 0.0264221455627689*G0_0_2 - 0.00554538857490066*G0_1_0 - 0.00619778723077307*G0_1_2 + 0.0264221455627714*G0_2_0 - 0.00619778723077206*G0_2_1 + 0.0137003717732881*G0_2_2;
    A[2773] = 0.0929668084615992*G0_0_0 + 0.0606730749959915*G0_0_1 + 0.121346149991982*G0_0_2 + 0.0606730749959901*G0_1_0 + 0.0195719596761259*G0_1_1 + 0.0748627457611831*G0_1_2 + 0.121346149991981*G0_2_0 + 0.0748627457611818*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2774] = 0.0678494602105694*G0_0_1 + 0.104709984267273*G0_0_2 + 0.0678494602105705*G0_1_0 + 0.0391439193522515*G0_1_1 + 0.13569892042114*G0_1_2 + 0.104709984267275*G0_2_0 + 0.13569892042114*G0_2_1 + 0.209419968534548*G0_2_2;
    A[2775] = 0.0195719596761255*G0_0_0 - 0.010275278829966*G0_0_1 + 0.00146789697571033*G0_0_2 - 0.0102752788299673*G0_1_0 - 0.0181040627004161*G0_1_2 + 0.00146789697570878*G0_2_0 - 0.0181040627004161*G0_2_1 - 0.0166361657247063*G0_2_2;
    A[2776] = 0.0141896707651903*G0_0_1 + 0.0283793415303826*G0_0_2 + 0.0141896707651907*G0_1_0 - 0.00489298991903308*G0_1_1 + 0.0283793415303809*G0_1_2 + 0.0283793415303828*G0_2_0 + 0.0283793415303817*G0_2_1 + 0.0567586830607646*G0_2_2;
    A[2777] = -0.000652398655870195*G0_0_1 - 0.0127217737894828*G0_0_2 - 0.000652398655869732*G0_1_0 - 0.00130479731174146*G0_1_2 - 0.0127217737894806*G0_2_0 - 0.00130479731174036*G0_2_1 - 0.0254435475789634*G0_2_2;
    A[2778] = -0.156575677409008*G0_0_0 - 0.224425137619578*G0_0_1 - 0.134067923781463*G0_0_2 - 0.224425137619581*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0972073997247611*G0_1_2 - 0.134067923781465*G0_2_0 - 0.0972073997247594*G0_2_1 - 0.0880738185425688*G0_2_2;
    A[2779] = 0.0929668084615976*G0_0_0 + 0.228013330226868*G0_0_1 + 0.363059851992138*G0_0_2 + 0.228013330226869*G0_1_0 + 0.456026660453739*G0_1_1 + 0.228013330226869*G0_1_2 + 0.363059851992139*G0_2_0 + 0.228013330226869*G0_2_1 + 0.0929668084615987*G0_2_2;
    A[2780] = -0.06784946021057*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105705*G0_1_0 - 0.29227459783015*G0_1_1 - 0.195067198105391*G0_1_2 - 0.104709984267275*G0_2_0 - 0.19506719810539*G0_2_1 - 0.185933616923199*G0_2_2;
    A[2781] = 0.019571959676126*G0_0_0 + 0.00538228891093471*G0_0_1 - 0.0552907860850553*G0_0_2 + 0.00538228891093574*G0_1_0 - 0.00880738185425518*G0_1_1 - 0.0411011153198623*G0_1_2 - 0.0552907860850543*G0_2_0 - 0.0411011153198635*G0_2_1 + 0.0195719596761289*G0_2_2;
    A[2782] = -0.0141896707651913*G0_0_1 - 0.0283793415303818*G0_0_2 - 0.0141896707651907*G0_1_0 - 0.00880738185425751*G0_1_1 + 0.0322937334656084*G0_1_2 - 0.0283793415303828*G0_2_0 + 0.032293733465607*G0_2_1 + 0.0929668084615991*G0_2_2;
    A[2783] = 0.000652398655870592*G0_0_1 + 0.0127217737894806*G0_0_2 + 0.000652398655869598*G0_1_0 + 0.00130479731174077*G0_1_1 + 0.000652398655868977*G0_1_2 + 0.0127217737894805*G0_2_0 + 0.000652398655870021*G0_2_1 - 0.08807381854257*G0_2_2;
    A[2784] = -0.0965550010688888*G0_0_0 - 0.0287055408583186*G0_0_1 - 0.0596944770121845*G0_0_2 - 0.0287055408583188*G0_1_0 + 0.0391439193522509*G0_1_1 - 0.0965550010688899*G0_1_2 - 0.0596944770121844*G0_2_0 - 0.0965550010688897*G0_2_1 - 0.0228339529554811*G0_2_2;
    A[2785] = -0.00880738185425608*G0_0_0 - 0.0411011153198646*G0_0_1 + 0.00538228891093555*G0_0_2 - 0.041101115319864*G0_1_0 + 0.0195719596761283*G0_1_1 - 0.0552907860850559*G0_1_2 + 0.00538228891093477*G0_2_0 - 0.0552907860850558*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2786] = 0.0502346965020566*G0_0_0 + 0.00554538857490206*G0_0_1 + 0.0381653213684452*G0_0_2 + 0.00554538857490173*G0_1_0 + 0.00619778723077167*G0_1_2 + 0.0381653213684458*G0_2_0 + 0.00619778723077192*G0_2_1 + 0.0260959462348341*G0_2_2;
    A[2787] = -0.0332723314494136*G0_0_0 - 0.0190826606842223*G0_0_1 - 0.0190826606842225*G0_0_2 - 0.0190826606842214*G0_1_0 - 0.00489298991903048*G0_1_1 - 0.0332723314494128*G0_1_2 - 0.0190826606842218*G0_2_0 - 0.0332723314494128*G0_2_1 - 0.00489298991902992*G0_2_2;
    A[2788] = 0.0401225173360579*G0_0_0 + 0.0102752788299658*G0_0_1 + 0.0298472385060925*G0_0_2 + 0.0102752788299648*G0_1_0 + 0.018104062700417*G0_1_2 + 0.0298472385060914*G0_2_0 + 0.0181040627004157*G0_2_1 + 0.0195719596761262*G0_2_2;
    A[2789] = 0.00130479731174145*G0_0_0 + 0.000652398655870465*G0_0_1 - 0.0107645778218703*G0_0_2 + 0.000652398655870636*G0_1_0 + 0.00130479731174053*G0_1_2 - 0.0107645778218701*G0_2_0 + 0.00130479731174073*G0_2_1 - 0.0228339529554819*G0_2_2;
    A[2790] = -0.268135847562927*G0_0_0 - 0.321632537344338*G0_0_1 - 0.290643601190472*G0_0_2 - 0.32163253734434*G0_1_0 - 0.584549195660301*G0_1_1 - 0.44885027523916*G0_1_2 - 0.222141742324031*G0_2_0 - 0.194414799449519*G0_2_1 - 0.268135847562927*G0_2_2;
    A[2791] = -0.0371867233846398*G0_0_0 + 0.186912214907003*G0_0_1 + 0.140428810676203*G0_0_2 + 0.186912214907006*G0_1_0 - 0.0371867233846405*G0_1_1 + 0.0376760223765437*G0_1_2 + 0.0376760223765431*G0_2_0 + 0.140428810676203*G0_2_1 + 0.0655660649150218*G0_2_2;
    A[2792] = 0.17223324514991*G0_0_0 + 0.00619778723077378*G0_0_1 - 0.0264221455627692*G0_0_2 + 0.00619778723077272*G0_1_0 + 0.0123955744615467*G0_1_1 + 0.00619778723077309*G0_1_2 + 0.0714376528178598*G0_2_0 + 0.0061977872307737*G0_2_1 - 0.0137003717732882*G0_2_2;
    A[2793] = 0.726119703984276*G0_0_0 + 0.456026660453737*G0_0_1 + 0.456026660453736*G0_0_2 + 0.456026660453737*G0_1_0 + 0.912053320907478*G0_1_1 + 0.456026660453737*G0_1_2 + 0.456026660453736*G0_2_0 + 0.456026660453737*G0_2_1 + 0.726119703984276*G0_2_2;
    A[2794] = -0.448197876583287*G0_0_0 - 0.224098938291643*G0_0_1 - 0.121346149991982*G0_0_2 - 0.224098938291644*G0_1_0 - 0.0371867233846398*G0_1_1 - 0.0748627457611832*G0_1_2 - 0.326851726591305*G0_2_0 - 0.177615534060843*G0_2_1 - 0.149725491522365*G0_2_2;
    A[2795] = -0.209419968534549*G0_0_0 - 0.26291665831596*G0_0_1 - 0.104709984267273*G0_0_2 - 0.262916658315961*G0_1_0 - 0.5845491956603*G0_1_1 - 0.13569892042114*G0_1_2 - 0.290643601190472*G0_2_0 - 0.39013439621078*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2796] = 0.352295274170267*G0_0_0 + 0.249542485870605*G0_0_1 + 0.249542485870606*G0_0_2 + 0.24954248587061*G0_1_0 - 0.0391439193522523*G0_1_1 + 0.44526208263187*G0_1_2 + 0.249542485870608*G0_2_0 + 0.445262082631868*G0_2_1 + 0.146789697570946*G0_2_2;
    A[2797] = -0.234863516113511*G0_0_0 - 0.0195719596761259*G0_0_1 - 0.141896707651914*G0_0_2 - 0.0195719596761285*G0_1_0 - 0.0391439193522545*G0_1_1 - 0.0195719596761281*G0_1_2 - 0.141896707651915*G0_2_0 - 0.0195719596761258*G0_2_1 - 0.0489298991903159*G0_2_2;
    A[2798] = -0.185933616923197*G0_0_0 - 0.288686405222857*G0_0_1 - 0.48440600198412*G0_0_2 - 0.288686405222859*G0_1_0 - 0.0391439193522506*G0_1_1 - 0.484406001984118*G0_1_2 - 0.484406001984119*G0_2_0 - 0.484406001984119*G0_2_1 - 0.782878387045041*G0_2_2;
    A[2799] = -0.039143919352252*G0_0_0 + 0.0048929899190322*G0_0_1 + 0.0538228891093488*G0_0_2 + 0.0048929899190316*G0_1_0 + 0.00978597983806405*G0_1_1 + 0.0048929899190308*G0_1_2 + 0.0538228891093459*G0_2_0 + 0.00489298991903133*G0_2_1 + 0.146789697570945*G0_2_2;
    A[2800] = 0.0117431758056758*G0_0_0 + 0.0117431758056758*G0_0_1 + 0.0117431758056758*G0_0_2 + 0.0117431758056758*G0_1_0 + 0.0117431758056758*G0_1_1 + 0.0117431758056758*G0_1_2 + 0.00946499969937469*G0_2_0 + 0.00946499969937469*G0_2_1 + 0.00946499969937466*G0_2_2;
    A[2801] = -0.0298472385060926*G0_0_0 + 0.0047246710658167*G0_2_0;
    A[2802] = -0.0298472385060926*G0_1_1 + 0.00472467106581683*G0_2_1;
    A[2803] = -0.00946499969937473*G0_2_2;
    A[2804] = 0.0642612676032814*G0_1_1 + 0.0422754329004325*G0_1_2 + 0.0422754329004321*G0_2_1 + 0.0429604514890965*G0_2_2;
    A[2805] = 0.00880738185425658*G0_1_1 + 0.00649136662591545*G0_1_2 + 0.00649136662591516*G0_2_1 + 0.00352295274170281*G0_2_2;
    A[2806] = -0.0006523986558704*G0_1_1 - 0.00949240044292089*G0_1_2 - 0.00949240044292097*G0_2_1 - 0.0189848008858423*G0_2_2;
    A[2807] = 0.00329461321214783*G0_1_2 + 0.0032946132121477*G0_2_1 + 0.0292600797158087*G0_2_2;
    A[2808] = 0.0642612676032805*G0_0_0 + 0.0422754329004319*G0_0_2 + 0.0422754329004324*G0_2_0 + 0.0429604514890966*G0_2_2;
    A[2809] = 0.00880738185425682*G0_0_0 + 0.00649136662591512*G0_0_2 + 0.00649136662591527*G0_2_0 + 0.00352295274170298*G0_2_2;
    A[2810] = -0.00065239865587174*G0_0_0 - 0.00949240044292143*G0_0_2 - 0.00949240044292158*G0_2_0 - 0.0189848008858428*G0_2_2;
    A[2811] = 0.00329461321214756*G0_0_2 + 0.00329461321214774*G0_2_0 + 0.0292600797158084*G0_2_2;
    A[2812] = 0.176636936077038*G0_0_0 + 0.042275432900432*G0_0_1 + 0.0422754329004321*G0_1_0 + 0.049321338383838*G0_2_0 + 0.0429604514890967*G0_2_1;
    A[2813] = -0.334680510461757*G0_0_0 - 0.0838658472121996*G0_0_1 - 0.0838658472121997*G0_1_0 + 0.104709984267276*G0_1_1 - 0.167731694424402*G0_2_0 - 0.0777006799142214*G0_2_1;
    A[2814] = 0.104709984267275*G0_0_0 - 0.083865847212201*G0_0_1 - 0.0838658472122006*G0_1_0 - 0.334680510461758*G0_1_1 - 0.0777006799142206*G0_2_0 - 0.167731694424401*G0_2_1;
    A[2815] = 0.0422754329004328*G0_0_1 + 0.0422754329004327*G0_1_0 + 0.176636936077039*G0_1_1 + 0.0429604514890965*G0_2_0 + 0.0493213383838373*G0_2_1;
    A[2816] = -0.00913358118219223*G0_0_0 - 0.00913358118219226*G0_0_1 - 0.00329461321214844*G0_0_2 - 0.00913358118219186*G0_1_0 - 0.0091335811821919*G0_1_1 - 0.00329461321214817*G0_1_2 - 0.00329461321214842*G0_2_0 - 0.00329461321214844*G0_2_1;
    A[2817] = -0.0156575677409004*G0_0_0 - 0.0156575677409003*G0_0_1 - 0.0025769746906891*G0_0_2 - 0.0156575677409011*G0_1_0 - 0.015657567740901*G0_1_1 - 0.00257697469068934*G0_1_2 - 0.00257697469068966*G0_2_0 - 0.00257697469068972*G0_2_1;
    A[2818] = -0.000652398655871137*G0_0_0 - 0.000652398655871147*G0_0_1 - 0.00257697469069024*G0_0_2 - 0.000652398655870744*G0_1_0 - 0.000652398655870737*G0_1_1 - 0.00257697469069036*G0_1_2 - 0.00257697469069044*G0_2_0 - 0.00257697469069045*G0_2_1;
    A[2819] = -0.00329461321214794*G0_0_2 - 0.00329461321214775*G0_1_2 - 0.00329461321214765*G0_2_0 - 0.00329461321214765*G0_2_1;
    A[2820] = -0.0435476102793804*G0_0_0 - 0.0377086423093363*G0_0_1 - 0.0435476102793805*G0_0_2 - 0.0377086423093356*G0_1_0 - 0.031869674339292*G0_1_1 - 0.0377086423093357*G0_1_2 - 0.0292600797158088*G0_2_0 - 0.0259654665036609*G0_2_1 - 0.029260079715809*G0_2_2;
    A[2821] = 0.0567586830607651*G0_0_0 + 0.0378717419733041*G0_0_1 + 0.0567586830607652*G0_0_2 + 0.0378717419733033*G0_1_0 + 0.0127870136550688*G0_1_1 + 0.0378717419733036*G0_1_2 + 0.0189848008858429*G0_2_0 + 0.0094924004429214*G0_2_1 + 0.018984800885843*G0_2_2;
    A[2822] = 0.104709984267275*G0_0_0 + 0.030369157430789*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0303691574307892*G0_1_0 - 0.00678494602105694*G0_1_1 + 0.0303691574307891*G0_1_2 - 0.00352295274170319*G0_2_0 + 0.00296841388421241*G0_2_1 - 0.0035229527417032*G0_2_2;
    A[2823] = -0.0422754329004325*G0_0_1 - 0.0422754329004326*G0_1_0 + 0.043971669405697*G0_1_1 - 0.0422754329004328*G0_1_2 - 0.0429604514890965*G0_2_0 - 0.000685018588664528*G0_2_1 - 0.0429604514890965*G0_2_2;
    A[2824] = -0.0318696743392922*G0_0_0 - 0.0377086423093362*G0_0_1 - 0.0377086423093363*G0_0_2 - 0.0377086423093368*G0_1_0 - 0.0435476102793805*G0_1_1 - 0.0435476102793805*G0_1_2 - 0.0259654665036608*G0_2_0 - 0.0292600797158089*G0_2_1 - 0.0292600797158089*G0_2_2;
    A[2825] = 0.0127870136550694*G0_0_0 + 0.0378717419733043*G0_0_1 + 0.0378717419733045*G0_0_2 + 0.0378717419733046*G0_1_0 + 0.0567586830607656*G0_1_1 + 0.0567586830607656*G0_1_2 + 0.00949240044292121*G0_2_0 + 0.0189848008858426*G0_2_1 + 0.0189848008858427*G0_2_2;
    A[2826] = -0.00678494602105725*G0_0_0 + 0.0303691574307887*G0_0_1 + 0.0303691574307887*G0_0_2 + 0.0303691574307888*G0_1_0 + 0.104709984267275*G0_1_1 + 0.104709984267275*G0_1_2 + 0.00296841388421238*G0_2_0 - 0.00352295274170319*G0_2_1 - 0.00352295274170323*G0_2_2;
    A[2827] = 0.0439716694056969*G0_0_0 - 0.042275432900432*G0_0_1 - 0.0422754329004321*G0_0_2 - 0.0422754329004325*G0_1_0 - 0.000685018588664327*G0_2_0 - 0.0429604514890966*G0_2_1 - 0.0429604514890967*G0_2_2;
    A[2828] = -0.156575677409009*G0_0_0 + 0.0678494602105703*G0_0_1 - 0.0225077536275445*G0_0_2 + 0.0678494602105719*G0_1_0 + 0.104709984267276*G0_1_2 - 0.0225077536275461*G0_2_0 + 0.104709984267275*G0_2_1 + 0.0234863516113508*G0_2_2;
    A[2829] = 0.0929668084615991*G0_0_0 - 0.135046521765271*G0_0_1 - 0.270093043530541*G0_0_2 - 0.135046521765271*G0_1_0 + 0.0929668084615996*G0_1_1 - 0.270093043530541*G0_1_2 - 0.270093043530541*G0_2_0 - 0.270093043530542*G0_2_1 - 0.540186087061083*G0_2_2;
    A[2830] = 0.0678494602105707*G0_0_1 + 0.104709984267275*G0_0_2 + 0.0678494602105715*G0_1_0 - 0.156575677409009*G0_1_1 - 0.0225077536275443*G0_1_2 + 0.104709984267274*G0_2_0 - 0.022507753627545*G0_2_1 + 0.0234863516113504*G0_2_2;
    A[2831] = 0.0195719596761265*G0_0_0 + 0.014189670765191*G0_0_1 + 0.0748627457611822*G0_0_2 + 0.0141896707651906*G0_1_0 + 0.0283793415303824*G0_1_2 + 0.0748627457611823*G0_2_0 + 0.0283793415303826*G0_2_1 + 0.149725491522365*G0_2_2;
    A[2832] = 0.0141896707651926*G0_0_1 + 0.0283793415303834*G0_0_2 + 0.014189670765192*G0_1_0 + 0.019571959676127*G0_1_1 + 0.0748627457611822*G0_1_2 + 0.0283793415303837*G0_2_0 + 0.0748627457611839*G0_2_1 + 0.149725491522366*G0_2_2;
    A[2833] = -0.000652398655872361*G0_0_1 - 0.0127217737894823*G0_0_2 - 0.000652398655870487*G0_1_0 - 0.0127217737894814*G0_1_2 - 0.0127217737894826*G0_2_0 - 0.0127217737894832*G0_2_1 - 0.113517366121532*G0_2_2;
    A[2834] = 0.0391439193522521*G0_0_0 + 0.0446893079271543*G0_0_1 + 0.0127217737894825*G0_0_2 + 0.0446893079271543*G0_1_0 + 0.0502346965020567*G0_1_1 + 0.0120693751336103*G0_1_2 + 0.0127217737894821*G0_2_0 + 0.0120693751336122*G0_2_1;
    A[2835] = 0.092966808461601*G0_0_0 + 0.0322937334656085*G0_0_1 - 0.0283793415303812*G0_0_2 + 0.0322937334656082*G0_1_0 - 0.00880738185425689*G0_1_1 - 0.014189670765191*G0_1_2 - 0.0283793415303821*G0_2_0 - 0.0141896707651927*G0_2_1;
    A[2836] = -0.0678494602105708*G0_0_1 - 0.104709984267275*G0_0_2 - 0.0678494602105715*G0_1_0 - 0.096555001068889*G0_1_1 - 0.0368605240567041*G0_1_2 - 0.104709984267274*G0_2_0 - 0.0368605240567033*G0_2_1;
    A[2837] = 0.0195719596761261*G0_0_0 + 0.0298472385060921*G0_0_1 + 0.0181040627004152*G0_0_2 + 0.0298472385060926*G0_1_0 + 0.0401225173360588*G0_1_1 + 0.0102752788299665*G0_1_2 + 0.0181040627004164*G0_2_0 + 0.0102752788299664*G0_2_1;
    A[2838] = -0.0141896707651916*G0_0_1 - 0.0283793415303843*G0_0_2 - 0.0141896707651919*G0_1_0 - 0.0332723314494145*G0_1_1 - 0.0141896707651919*G0_1_2 - 0.0283793415303837*G0_2_0 - 0.0141896707651917*G0_2_1;
    A[2839] = 0.000652398655871197*G0_0_1 + 0.0127217737894833*G0_0_2 + 0.000652398655870494*G0_1_0 + 0.00130479731174132*G0_1_1 + 0.0120693751336112*G0_1_2 + 0.0127217737894824*G0_2_0 + 0.0120693751336115*G0_2_1;
    A[2840] = 0.0502346965020576*G0_0_0 + 0.0446893079271555*G0_0_1 + 0.0120693751336117*G0_0_2 + 0.0446893079271543*G0_1_0 + 0.0391439193522522*G0_1_1 + 0.0127217737894817*G0_1_2 + 0.0120693751336117*G0_2_0 + 0.0127217737894819*G0_2_1;
    A[2841] = -0.00880738185425723*G0_0_0 + 0.032293733465608*G0_0_1 - 0.0141896707651923*G0_0_2 + 0.0322937334656082*G0_1_0 + 0.0929668084615991*G0_1_1 - 0.0283793415303831*G0_1_2 - 0.0141896707651918*G0_2_0 - 0.0283793415303831*G0_2_1;
    A[2842] = -0.0965550010688888*G0_0_0 - 0.0678494602105697*G0_0_1 - 0.0368605240567039*G0_0_2 - 0.0678494602105701*G0_1_0 - 0.104709984267275*G0_1_2 - 0.036860524056704*G0_2_0 - 0.104709984267274*G0_2_1;
    A[2843] = 0.0401225173360571*G0_0_0 + 0.0298472385060909*G0_0_1 + 0.0102752788299647*G0_0_2 + 0.0298472385060922*G0_1_0 + 0.0195719596761257*G0_1_1 + 0.0181040627004167*G0_1_2 + 0.0102752788299661*G0_2_0 + 0.0181040627004161*G0_2_1;
    A[2844] = -0.0332723314494136*G0_0_0 - 0.0141896707651907*G0_0_1 - 0.0141896707651909*G0_0_2 - 0.0141896707651915*G0_1_0 - 0.0283793415303826*G0_1_2 - 0.0141896707651918*G0_2_0 - 0.0283793415303823*G0_2_1;
    A[2845] = 0.00130479731174273*G0_0_0 + 0.000652398655872169*G0_0_1 + 0.0120693751336124*G0_0_2 + 0.000652398655871387*G0_1_0 + 0.0127217737894823*G0_1_2 + 0.0120693751336122*G0_2_0 + 0.0127217737894831*G0_2_1;
    A[2846] = 0.17223324514991*G0_0_0 + 0.166035457919136*G0_0_1 + 0.19865539071268*G0_0_2 + 0.166035457919136*G0_1_0 + 0.17223324514991*G0_1_1 + 0.19865539071268*G0_1_2 + 0.10079559233205*G0_2_0 + 0.10079559233205*G0_2_1 + 0.113517366121532*G0_2_2;
    A[2847] = -0.0371867233846395*G0_0_0 - 0.224098938291642*G0_0_1 - 0.177615534060843*G0_0_2 - 0.224098938291645*G0_1_0 - 0.448197876583288*G0_1_1 - 0.326851726591306*G0_1_2 - 0.0748627457611825*G0_2_0 - 0.121346149991982*G0_2_1 - 0.149725491522365*G0_2_2;
    A[2848] = -0.268135847562928*G0_0_0 + 0.0534966897814109*G0_0_1 + 0.0225077536275449*G0_0_2 + 0.0534966897814122*G0_1_0 - 0.209419968534551*G0_1_1 - 0.104709984267275*G0_1_2 - 0.0459941052388962*G0_2_0 + 0.0812236326559242*G0_2_1 - 0.0234863516113505*G0_2_2;
    A[2849] = -0.448197876583287*G0_0_0 - 0.224098938291644*G0_0_1 - 0.326851726591305*G0_0_2 - 0.224098938291643*G0_1_0 - 0.0371867233846398*G0_1_1 - 0.177615534060843*G0_1_2 - 0.121346149991982*G0_2_0 - 0.0748627457611832*G0_2_1 - 0.149725491522365*G0_2_2;
    A[2850] = 0.726119703984278*G0_0_0 + 0.270093043530539*G0_0_1 + 0.270093043530541*G0_0_2 + 0.270093043530539*G0_1_0 + 0.72611970398428*G0_1_1 + 0.27009304353054*G0_1_2 + 0.270093043530541*G0_2_0 + 0.27009304353054*G0_2_1 + 0.540186087061082*G0_2_2;
    A[2851] = -0.20941996853455*G0_0_0 + 0.0534966897814115*G0_0_1 - 0.104709984267275*G0_0_2 + 0.0534966897814116*G0_1_0 - 0.268135847562929*G0_1_1 + 0.0225077536275447*G0_1_2 + 0.0812236326559241*G0_2_0 - 0.0459941052388952*G0_2_1 - 0.0234863516113501*G0_2_2;
    A[2852] = -0.234863516113511*G0_0_0 - 0.215291556437384*G0_0_1 - 0.0929668084615972*G0_0_2 - 0.215291556437386*G0_1_0 - 0.234863516113512*G0_1_1 - 0.092966808461599*G0_1_2 - 0.092966808461599*G0_2_0 - 0.0929668084615972*G0_2_1;
    A[2853] = 0.352295274170268*G0_0_0 + 0.102752788299661*G0_0_1 + 0.10275278829966*G0_0_2 + 0.10275278829966*G0_1_0 - 0.1859336169232*G0_1_1 + 0.298472385060924*G0_1_2 + 0.102752788299663*G0_2_0 + 0.298472385060925*G0_2_1;
    A[2854] = -0.1859336169232*G0_0_0 + 0.102752788299659*G0_0_1 + 0.298472385060922*G0_0_2 + 0.102752788299662*G0_1_0 + 0.352295274170269*G0_1_1 + 0.102752788299661*G0_1_2 + 0.298472385060923*G0_2_0 + 0.102752788299661*G0_2_1;
    A[2855] = -0.0391439193522517*G0_0_0 - 0.0440369092712835*G0_0_1 - 0.0929668084616*G0_0_2 - 0.044036909271283*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0929668084615993*G0_1_2 - 0.0929668084615988*G0_2_0 - 0.0929668084616003*G0_2_1;
    A[2856] = 0.0189195610202553*G0_0_0 + 0.0189195610202552*G0_0_1 + 0.0189195610202553*G0_0_2 + 0.0189195610202553*G0_1_0 + 0.0189195610202552*G0_1_1 + 0.0189195610202553*G0_1_2 + 0.0137421252872639*G0_2_0 + 0.013742125287264*G0_2_1 + 0.013742125287264*G0_2_2;
    A[2857] = 0.0189195610202552*G0_0_0 + 0.00517743573299106*G0_2_0;
    A[2858] = 0.0335985307773502*G0_1_1 - 0.0143945239431349*G0_2_1;
    A[2859] = -0.013742125287264*G0_2_2;
    A[2860] = -0.128522535206562*G0_1_1 - 0.0962940416065408*G0_1_2 - 0.0962940416065411*G0_2_1 - 0.124738623002511*G0_2_2;
    A[2861] = 0.0476251018785738*G0_1_1 + 0.0623693115012554*G0_1_2 + 0.0623693115012558*G0_2_1 + 0.124738623002511*G0_2_2;
    A[2862] = -0.0150051690850306*G0_1_1 - 0.0377738821749237*G0_1_2 - 0.0377738821749243*G0_2_1 - 0.127087258163647*G0_2_2;
    A[2863] = 0.00939454064454033*G0_1_2 + 0.00939454064454091*G0_2_1 + 0.0641307878721069*G0_2_2;
    A[2864] = -0.0306627368259306*G0_0_0 - 0.0212681961813903*G0_0_2 - 0.0212681961813901*G0_2_0 - 0.01187365553685*G0_2_2;
    A[2865] = -0.00130479731174225*G0_0_0 - 0.001239557446155*G0_0_2 - 0.00123955744615512*G0_2_0 - 0.00117431758056773*G0_2_2;
    A[2866] = -0.0150051690850295*G0_0_0 - 0.0149399292194427*G0_0_2 - 0.014939929219443*G0_2_0 - 0.014874689353856*G0_2_2;
    A[2867] = 0.00939454064454076*G0_0_2 + 0.00939454064454095*G0_2_0 + 0.0187890812890813*G0_2_2;
    A[2868] = -0.111886369481854*G0_0_0 - 0.0212681961813901*G0_0_1 - 0.0212681961813899*G0_1_0 - 0.0477555816097474*G0_2_0 - 0.0118736555368497*G0_2_1;
    A[2869] = 0.209419968534549*G0_0_0 + 0.0359471659384845*G0_0_1 + 0.0359471659384842*G0_1_0 - 0.0528442911255407*G0_1_1 + 0.0823327103709033*G0_2_0 - 0.00182671623643884*G0_2_1;
    A[2870] = -0.271397840842282*G0_0_0 - 0.0733296089198855*G0_0_1 - 0.0733296089198857*G0_1_0 + 0.121346149991982*G0_1_1 - 0.146659217839771*G0_2_0 - 0.0109602974186305*G0_2_1;
    A[2871] = -0.0962940416065408*G0_0_1 - 0.0962940416065408*G0_1_0 - 0.257045070413124*G0_1_1 - 0.124738623002511*G0_2_0 - 0.192588083213082*G0_2_1;
    A[2872] = -0.0306627368259311*G0_0_0 - 0.0306627368259311*G0_0_1 - 0.00939454064454072*G0_0_2 - 0.0306627368259313*G0_1_0 - 0.0306627368259313*G0_1_1 - 0.00939454064454088*G0_1_2 - 0.00939454064454073*G0_2_0 - 0.00939454064454078*G0_2_1;
    A[2873] = -0.00130479731174168*G0_0_0 - 0.00130479731174172*G0_0_1 - 6.52398655868925e-05*G0_0_2 - 0.00130479731174091*G0_1_0 - 0.00130479731174093*G0_1_1 - 6.52398655869266e-05*G0_1_2 - 6.5239865586504e-05*G0_2_0 - 6.52398655865793e-05*G0_2_1;
    A[2874] = -0.01500516908503*G0_0_0 - 0.0150051690850299*G0_0_1 - 6.52398655871911e-05*G0_0_2 - 0.0150051690850303*G0_1_0 - 0.0150051690850303*G0_1_1 - 6.52398655867562e-05*G0_1_2 - 6.52398655870404e-05*G0_2_0 - 6.52398655869579e-05*G0_2_1;
    A[2875] = -0.00939454064454039*G0_0_2 - 0.00939454064454088*G0_1_2 - 0.00939454064454082*G0_2_0 - 0.00939454064454083*G0_2_1;
    A[2876] = -0.111886369481854*G0_0_0 - 0.0906181733004639*G0_0_1 - 0.111886369481854*G0_0_2 - 0.0906181733004642*G0_1_0 - 0.0693499771190736*G0_1_1 - 0.0906181733004644*G0_1_2 - 0.0641307878721065*G0_2_0 - 0.0547362472275658*G0_2_1 - 0.0641307878721066*G0_2_2;
    A[2877] = 0.209419968534549*G0_0_0 + 0.173472802596064*G0_0_1 + 0.20941996853455*G0_0_2 + 0.173472802596064*G0_1_0 + 0.084681345532039*G0_1_1 + 0.173472802596065*G0_1_2 + 0.127087258163645*G0_2_0 + 0.0893133759887217*G0_2_1 + 0.127087258163645*G0_2_2;
    A[2878] = -0.271397840842283*G0_0_0 - 0.198068231922396*G0_0_1 - 0.271397840842283*G0_0_2 - 0.198068231922397*G0_1_0 - 0.00339247301052812*G0_1_1 - 0.198068231922396*G0_1_2 - 0.12473862300251*G0_2_0 - 0.0623693115012546*G0_2_1 - 0.12473862300251*G0_2_2;
    A[2879] = 0.0962940416065406*G0_0_1 + 0.0962940416065407*G0_1_0 - 0.0644569872000425*G0_1_1 + 0.0962940416065407*G0_1_2 + 0.124738623002511*G0_2_0 + 0.0284445813959698*G0_2_1 + 0.124738623002511*G0_2_2;
    A[2880] = -0.018789081289081*G0_0_0 - 0.0400572774704713*G0_0_1 - 0.0400572774704714*G0_0_2 - 0.0400572774704716*G0_1_0 - 0.0613254736518621*G0_1_1 - 0.0613254736518621*G0_1_2 - 0.00939454064454042*G0_2_0 - 0.0187890812890812*G0_2_1 - 0.0187890812890812*G0_2_2;
    A[2881] = -0.000130479731174392*G0_0_0 + 0.013635131907701*G0_0_1 + 0.0136351319077009*G0_0_2 + 0.013635131907701*G0_1_0 - 0.0254435475789639*G0_1_1 - 0.0254435475789639*G0_1_2 - 6.52398655871795e-05*G0_2_0 + 0.0148746893538557*G0_2_1 + 0.0148746893538558*G0_2_2;
    A[2882] = -0.000130479731174073*G0_0_0 - 0.0137656116388754*G0_0_1 - 0.0137656116388754*G0_0_2 - 0.0137656116388753*G0_1_0 - 0.0528442911255412*G0_1_1 - 0.0528442911255413*G0_1_2 - 6.52398655869509e-05*G0_2_0 + 0.0011743175805674*G0_2_1 + 0.00117431758056732*G0_2_2;
    A[2883] = -0.0187890812890811*G0_0_0 + 0.0212681961813901*G0_0_1 + 0.0212681961813901*G0_0_2 + 0.0212681961813907*G0_1_0 - 0.00939454064454056*G0_2_0 + 0.01187365553685*G0_2_1 + 0.0118736555368497*G0_2_2;
    A[2884] = 0.0521918924696687*G0_0_0 - 0.0378391220405109*G0_0_1 - 0.000652398655870581*G0_0_2 - 0.0378391220405128*G0_1_0 - 0.0528442911255407*G0_1_2 - 0.000652398655872104*G0_2_0 - 0.0528442911255411*G0_2_1 - 0.0534966897814112*G0_2_2;
    A[2885] = -0.185933616923197*G0_0_0 + 0.00913358118219196*G0_0_1 - 0.0812236326559239*G0_0_2 + 0.00913358118219285*G0_1_0 - 0.0880738185425683*G0_1_1 + 0.0459941052388951*G0_1_2 - 0.0812236326559239*G0_2_0 + 0.0459941052388955*G0_2_1 + 0.0234863516113497*G0_2_2;
    A[2886] = -0.13569892042114*G0_0_1 - 0.271397840842282*G0_0_2 - 0.135698920421142*G0_1_0 + 0.0521918924696702*G0_1_1 - 0.271397840842282*G0_1_2 - 0.271397840842284*G0_2_0 - 0.271397840842283*G0_2_1 - 0.542795681684566*G0_2_2;
    A[2887] = 0.0260959462348354*G0_0_0 - 0.0120693751336112*G0_0_1 + 0.019898159004061*G0_0_2 - 0.0120693751336101*G0_1_0 - 0.0127217737894831*G0_1_2 + 0.019898159004063*G0_2_0 - 0.0127217737894817*G0_2_1 + 0.0137003717732881*G0_2_2;
    A[2888] = 0.0368605240567046*G0_0_1 + 0.104709984267276*G0_0_2 + 0.0368605240567051*G0_1_0 - 0.022833952955479*G0_1_1 + 0.073721048113411*G0_1_2 + 0.104709984267275*G0_2_0 + 0.0737210481134093*G0_2_1 + 0.209419968534551*G0_2_2;
    A[2889] = -0.0150051690850298*G0_0_1 - 0.0528442911255407*G0_0_2 - 0.0150051690850311*G0_1_0 - 0.0300103381700605*G0_1_2 - 0.0528442911255414*G0_2_0 - 0.0300103381700602*G0_2_1 - 0.105688582251081*G0_2_2;
    A[2890] = 0.0521918924696701*G0_0_0 + 0.0900310145101808*G0_0_1 + 0.0528442911255407*G0_0_2 + 0.0900310145101812*G0_1_0 + 0.127870136550691*G0_1_1 + 0.0378391220405123*G0_1_2 + 0.0528442911255413*G0_2_0 + 0.0378391220405111*G0_2_1;
    A[2891] = -0.185933616923199*G0_0_0 - 0.195067198105391*G0_0_1 - 0.104709984267276*G0_0_2 - 0.19506719810539*G0_1_0 - 0.292274597830151*G0_1_1 - 0.0678494602105709*G0_1_2 - 0.104709984267276*G0_2_0 - 0.0678494602105712*G0_2_1;
    A[2892] = 0.135698920421142*G0_0_1 + 0.271397840842283*G0_0_2 + 0.135698920421142*G0_1_0 + 0.323589733311952*G0_1_1 + 0.135698920421142*G0_1_2 + 0.271397840842284*G0_2_0 + 0.135698920421142*G0_2_1;
    A[2893] = 0.0260959462348345*G0_0_0 + 0.0381653213684455*G0_0_1 + 0.00619778723077288*G0_0_2 + 0.0381653213684453*G0_1_0 + 0.0502346965020559*G0_1_1 + 0.00554538857490019*G0_1_2 + 0.00619778723077283*G0_2_0 + 0.00554538857490109*G0_2_1;
    A[2894] = -0.0368605240567037*G0_0_1 - 0.104709984267274*G0_0_2 - 0.0368605240567052*G0_1_0 - 0.0965550010688886*G0_1_1 - 0.0678494602105705*G0_1_2 - 0.104709984267275*G0_2_0 - 0.0678494602105703*G0_2_1;
    A[2895] = 0.0150051690850303*G0_0_1 + 0.0528442911255409*G0_0_2 + 0.0150051690850311*G0_1_0 + 0.0300103381700609*G0_1_1 + 0.037839122040512*G0_1_2 + 0.0528442911255413*G0_2_0 + 0.0378391220405113*G0_2_1;
    A[2896] = 0.0300103381700598*G0_0_0 - 0.00782878387045067*G0_0_1 + 0.0150051690850296*G0_0_2 - 0.00782878387045025*G0_1_0 - 0.0456679059109605*G0_1_1 + 0.0300103381700607*G0_1_2 + 0.0150051690850302*G0_2_0 + 0.0300103381700607*G0_2_1;
    A[2897] = 0.00130479731174173*G0_0_0 + 0.000652398655870554*G0_0_1 + 0.000652398655871162*G0_0_2 + 0.000652398655870727*G0_1_0 - 0.0880738185425688*G0_1_1 + 0.0127217737894821*G0_1_2 + 0.000652398655870296*G0_2_0 + 0.0127217737894813*G0_2_1;
    A[2898] = 0.03001033817006*G0_0_0 + 0.0378391220405107*G0_0_1 + 0.0150051690850306*G0_0_2 + 0.0378391220405104*G0_1_0 + 0.0528442911255419*G0_1_2 + 0.0150051690850301*G0_2_0 + 0.0528442911255408*G0_2_1;
    A[2899] = 0.00130479731174195*G0_0_0 - 0.010764577821869*G0_0_1 + 0.000652398655871385*G0_0_2 - 0.0107645778218705*G0_1_0 - 0.022833952955481*G0_1_1 + 0.00130479731174065*G0_1_2 + 0.000652398655870155*G0_2_0 + 0.00130479731174168*G0_2_1;
    A[2900] = 0.00130479731174189*G0_0_0 + 0.0120693751336111*G0_0_1 + 0.00065239865587015*G0_0_2 + 0.0120693751336121*G0_1_0 + 0.0127217737894813*G0_1_2 + 0.000652398655871302*G0_2_0 + 0.0127217737894817*G0_2_1;
    A[2901] = 0.0300103381700597*G0_0_0 + 0.0150051690850297*G0_0_1 + 0.0150051690850299*G0_0_2 + 0.0150051690850302*G0_1_0 + 0.0300103381700611*G0_1_2 + 0.0150051690850301*G0_2_0 + 0.0300103381700599*G0_2_1;
    A[2902] = 0.105688582251081*G0_0_0 + 0.12787013655069*G0_0_1 + 0.10503618359521*G0_0_2 + 0.127870136550691*G0_1_0 + 0.255740273101381*G0_1_1 + 0.18006202902036*G0_1_2 + 0.0528442911255398*G0_2_0 + 0.0756782440810207*G0_2_1 + 0.10568858225108*G0_2_2;
    A[2903] = 0.0123955744615466*G0_0_0 + 0.00619778723077349*G0_0_1 + 0.00619778723077277*G0_0_2 + 0.00619778723077326*G0_1_0 + 0.172233245149911*G0_1_1 + 0.0714376528178601*G0_1_2 + 0.00619778723077375*G0_2_0 - 0.0264221455627694*G0_2_1 - 0.0137003717732881*G0_2_2;
    A[2904] = 0.105688582251081*G0_0_0 - 0.0221815542996094*G0_0_1 + 0.000652398655870482*G0_0_2 - 0.0221815542996099*G0_1_0 + 0.105688582251081*G0_1_1 + 0.0528442911255407*G0_1_2 + 0.0528442911255405*G0_2_0 + 0.000652398655870982*G0_2_1 + 0.0534966897814115*G0_2_2;
    A[2905] = -0.209419968534549*G0_0_0 - 0.262916658315961*G0_0_1 - 0.290643601190473*G0_0_2 - 0.26291665831596*G0_1_0 - 0.5845491956603*G0_1_1 - 0.39013439621078*G0_1_2 - 0.104709984267273*G0_2_0 - 0.13569892042114*G0_2_1 - 0.209419968534548*G0_2_2;
    A[2906] = -0.20941996853455*G0_0_0 + 0.0534966897814116*G0_0_1 + 0.081223632655924*G0_0_2 + 0.0534966897814115*G0_1_0 - 0.268135847562929*G0_1_1 - 0.0459941052388952*G0_1_2 - 0.104709984267275*G0_2_0 + 0.0225077536275447*G0_2_1 - 0.0234863516113501*G0_2_2;
    A[2907] = 0.542795681684565*G0_0_0 + 0.271397840842282*G0_0_1 + 0.271397840842282*G0_0_2 + 0.271397840842282*G0_1_0 + 0.647179466623905*G0_1_1 + 0.271397840842282*G0_1_2 + 0.271397840842282*G0_2_0 + 0.271397840842282*G0_2_1 + 0.542795681684566*G0_2_2;
    A[2908] = -0.0521918924696712*G0_0_0 - 0.0424059126316076*G0_0_1 - 0.0260959462348361*G0_0_2 - 0.042405912631608*G0_1_0 + 0.143527704291592*G0_1_1 - 0.0750258454251496*G0_1_2 - 0.0260959462348354*G0_2_0 - 0.075025845425152*G0_2_1;
    A[2909] = -0.0521918924696676*G0_0_0 - 0.00978597983806205*G0_0_1 - 0.026095946234833*G0_0_2 - 0.00978597983806017*G0_1_0 + 0.176147637085138*G0_1_1 - 0.0587158790283768*G0_1_2 - 0.0260959462348336*G0_2_0 - 0.0587158790283768*G0_2_1;
    A[2910] = 0.371867233846396*G0_0_0 + 0.185933616923198*G0_0_1 + 0.185933616923196*G0_0_2 + 0.185933616923198*G0_1_0 - 0.150051690850302*G0_1_1 + 0.316413348097369*G0_1_2 + 0.1859336169232*G0_2_0 + 0.316413348097374*G0_2_1;
    A[2911] = -0.0521918924696704*G0_0_0 - 0.0260959462348359*G0_0_1 - 0.026095946234834*G0_0_2 - 0.0260959462348356*G0_1_0 + 0.0456679059109594*G0_1_1 - 0.0750258454251504*G0_1_2 - 0.026095946234836*G0_2_0 - 0.0750258454251511*G0_2_1;
    A[2912] = -0.0782878387045048*G0_0_0 - 0.0782878387045048*G0_0_1 - 0.0782878387045048*G0_0_2 - 0.0782878387045048*G0_1_0 - 0.0782878387045051*G0_1_1 - 0.078287838704505*G0_1_2 - 0.078287838704505*G0_2_0 - 0.0782878387045051*G0_2_1 - 0.0782878387045051*G0_2_2;
    A[2913] = 0.0440369092712838*G0_0_0;
    A[2914] = 0.0440369092712843*G0_1_1;
    A[2915] = 0.044036909271284*G0_2_2;
    A[2916] = -0.0505608958299929*G0_1_1 - 0.00260959462348314*G0_1_2 - 0.00260959462348388*G0_2_1;
    A[2917] = 0.0391439193522529*G0_1_1 + 0.031967534137673*G0_1_2 + 0.0319675341376737*G0_2_1 + 0.026095946234836*G0_2_2;
    A[2918] = 0.0260959462348355*G0_1_1 + 0.0319675341376736*G0_1_2 + 0.0319675341376736*G0_2_1 + 0.0391439193522526*G0_2_2;
    A[2919] = -0.00260959462348307*G0_1_2 - 0.00260959462348373*G0_2_1 - 0.0505608958299923*G0_2_2;
    A[2920] = -0.0505608958299926*G0_0_0 - 0.00260959462348358*G0_0_2 - 0.00260959462348318*G0_2_0;
    A[2921] = 0.0391439193522522*G0_0_0 + 0.0319675341376729*G0_0_2 + 0.0319675341376723*G0_2_0 + 0.0260959462348346*G0_2_2;
    A[2922] = 0.0260959462348346*G0_0_0 + 0.0319675341376722*G0_0_2 + 0.0319675341376724*G0_2_0 + 0.0391439193522524*G0_2_2;
    A[2923] = -0.00260959462348387*G0_0_2 - 0.00260959462348397*G0_2_0 - 0.0505608958299932*G0_2_2;
    A[2924] = -0.0505608958299925*G0_0_0 - 0.00260959462348376*G0_0_1 - 0.0026095946234827*G0_1_0;
    A[2925] = 0.0391439193522519*G0_0_0 + 0.0319675341376723*G0_0_1 + 0.0319675341376713*G0_1_0 + 0.0260959462348334*G0_1_1;
    A[2926] = 0.0260959462348351*G0_0_0 + 0.0319675341376726*G0_0_1 + 0.0319675341376732*G0_1_0 + 0.0391439193522533*G0_1_1;
    A[2927] = -0.00260959462348292*G0_0_1 - 0.00260959462348326*G0_1_0 - 0.050560895829993*G0_1_1;
    A[2928] = 0.0962288017409521*G0_0_0 + 0.0962288017409524*G0_0_1 + 0.131458329157981*G0_0_2 + 0.0962288017409528*G0_1_0 + 0.0962288017409527*G0_1_1 + 0.131458329157981*G0_1_2 + 0.13145832915798*G0_2_0 + 0.13145832915798*G0_2_1 + 0.166687856575009*G0_2_2;
    A[2929] = -0.156575677409008*G0_0_0 - 0.156575677409008*G0_0_1 - 0.0662184635708934*G0_0_2 - 0.156575677409009*G0_1_0 - 0.156575677409009*G0_1_1 - 0.0662184635708934*G0_1_2 - 0.0662184635708931*G0_2_0 - 0.0662184635708931*G0_2_1 - 0.161794866655978*G0_2_2;
    A[2930] = 0.0260959462348343*G0_0_0 + 0.0260959462348343*G0_0_1 + 0.0104383784939325*G0_0_2 + 0.0260959462348347*G0_1_0 + 0.0260959462348345*G0_1_1 + 0.0104383784939328*G0_1_2 + 0.0104383784939333*G0_2_0 + 0.0104383784939333*G0_2_1 + 0.180714427676232*G0_2_2;
    A[2931] = 0.00260959462348443*G0_0_2 + 0.00260959462348404*G0_1_2 + 0.00260959462348391*G0_2_0 + 0.00260959462348388*G0_2_1 - 0.151356488162043*G0_2_2;
    A[2932] = 0.096228801740952*G0_0_0 + 0.131458329157981*G0_0_1 + 0.0962288017409521*G0_0_2 + 0.13145832915798*G0_1_0 + 0.166687856575008*G0_1_1 + 0.13145832915798*G0_1_2 + 0.096228801740952*G0_2_0 + 0.131458329157981*G0_2_1 + 0.096228801740952*G0_2_2;
    A[2933] = -0.156575677409008*G0_0_0 - 0.0662184635708929*G0_0_1 - 0.156575677409008*G0_0_2 - 0.0662184635708931*G0_1_0 - 0.161794866655976*G0_1_1 - 0.0662184635708932*G0_1_2 - 0.156575677409009*G0_2_0 - 0.066218463570893*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2934] = 0.0260959462348356*G0_0_0 + 0.0104383784939327*G0_0_1 + 0.0260959462348356*G0_0_2 + 0.0104383784939343*G0_1_0 + 0.180714427676232*G0_1_1 + 0.0104383784939344*G0_1_2 + 0.0260959462348354*G0_2_0 + 0.0104383784939337*G0_2_1 + 0.0260959462348355*G0_2_2;
    A[2935] = 0.00260959462348399*G0_0_1 + 0.0026095946234834*G0_1_0 - 0.151356488162043*G0_1_1 + 0.00260959462348343*G0_1_2 + 0.002609594623483*G0_2_1;
    A[2936] = 0.166687856575009*G0_0_0 + 0.13145832915798*G0_0_1 + 0.13145832915798*G0_0_2 + 0.131458329157982*G0_1_0 + 0.0962288017409532*G0_1_1 + 0.0962288017409533*G0_1_2 + 0.131458329157982*G0_2_0 + 0.0962288017409527*G0_2_1 + 0.0962288017409527*G0_2_2;
    A[2937] = -0.161794866655977*G0_0_0 - 0.0662184635708922*G0_0_1 - 0.0662184635708921*G0_0_2 - 0.0662184635708947*G0_1_0 - 0.156575677409008*G0_1_1 - 0.156575677409008*G0_1_2 - 0.0662184635708949*G0_2_0 - 0.156575677409008*G0_2_1 - 0.156575677409008*G0_2_2;
    A[2938] = 0.180714427676232*G0_0_0 + 0.0104383784939338*G0_0_1 + 0.0104383784939337*G0_0_2 + 0.0104383784939343*G0_1_0 + 0.0260959462348344*G0_1_1 + 0.0260959462348341*G0_1_2 + 0.0104383784939346*G0_2_0 + 0.0260959462348345*G0_2_1 + 0.0260959462348344*G0_2_2;
    A[2939] = -0.151356488162042*G0_0_0 + 0.00260959462348387*G0_0_1 + 0.00260959462348392*G0_0_2 + 0.00260959462348381*G0_1_0 + 0.00260959462348354*G0_2_0;
    A[2940] = 0.19571959676126*G0_0_0 + 0.0750258454251489*G0_0_1 + 0.0750258454251482*G0_0_2 + 0.0750258454251497*G0_1_0 + 0.0260959462348348*G0_1_2 + 0.0750258454251484*G0_2_0 + 0.026095946234834*G0_2_1;
    A[2941] = 0.146789697570945*G0_0_0 + 0.141896707651914*G0_0_1 + 0.0929668084615992*G0_0_2 + 0.141896707651913*G0_1_0 + 0.146789697570944*G0_1_1 + 0.0929668084615971*G0_1_2 + 0.092966808461598*G0_2_0 + 0.092966808461597*G0_2_1;
    A[2942] = 0.0750258454251501*G0_0_1 + 0.0260959462348353*G0_0_2 + 0.075025845425152*G0_1_0 + 0.195719596761264*G0_1_1 + 0.0750258454251522*G0_1_2 + 0.0260959462348362*G0_2_0 + 0.0750258454251528*G0_2_1;
    A[2943] = 0.146789697570944*G0_0_0 + 0.0929668084615986*G0_0_1 + 0.141896707651913*G0_0_2 + 0.092966808461599*G0_1_0 + 0.0929668084615987*G0_1_2 + 0.141896707651913*G0_2_0 + 0.0929668084615976*G0_2_1 + 0.146789697570943*G0_2_2;
    A[2944] = 0.0929668084615996*G0_0_1 + 0.0929668084615978*G0_0_2 + 0.0929668084616015*G0_1_0 + 0.146789697570947*G0_1_1 + 0.141896707651915*G0_1_2 + 0.0929668084616026*G0_2_0 + 0.141896707651916*G0_2_1 + 0.146789697570947*G0_2_2;
    A[2945] = 0.026095946234834*G0_0_1 + 0.0750258454251509*G0_0_2 + 0.0260959462348369*G0_1_0 + 0.0750258454251526*G0_1_2 + 0.0750258454251511*G0_2_0 + 0.0750258454251501*G0_2_1 + 0.195719596761263*G0_2_2;
    A[2946] = -0.782878387045049*G0_0_0 - 0.466465038947674*G0_0_1 - 0.466465038947673*G0_0_2 - 0.466465038947676*G0_1_0 - 0.1500516908503*G0_1_1 - 0.335985307773499*G0_1_2 - 0.466465038947675*G0_2_0 - 0.335985307773499*G0_2_1 - 0.150051690850299*G0_2_2;
    A[2947] = 0.146789697570948*G0_0_0 + 0.445262082631873*G0_0_1 + 0.249542485870607*G0_0_2 + 0.445262082631872*G0_1_0 - 0.0391439193522517*G0_1_1 + 0.249542485870609*G0_1_2 + 0.249542485870612*G0_2_0 + 0.249542485870609*G0_2_1 + 0.352295274170271*G0_2_2;
    A[2948] = -0.0750258454251523*G0_0_1 - 0.0260959462348362*G0_0_2 - 0.0750258454251521*G0_1_0 + 0.143527704291592*G0_1_1 - 0.0424059126316078*G0_1_2 - 0.0260959462348366*G0_2_0 - 0.0424059126316063*G0_2_1 - 0.0521918924696719*G0_2_2;
    A[2949] = 0.146789697570947*G0_0_0 + 0.249542485870609*G0_0_1 + 0.445262082631873*G0_0_2 + 0.249542485870614*G0_1_0 + 0.352295274170273*G0_1_1 + 0.249542485870611*G0_1_2 + 0.445262082631874*G0_2_0 + 0.24954248587061*G0_2_1 - 0.0391439193522518*G0_2_2;
    A[2950] = -0.0929668084616009*G0_0_1 - 0.0929668084616007*G0_0_2 - 0.0929668084616018*G0_1_0 - 0.234863516113518*G0_1_1 - 0.21529155643739*G0_1_2 - 0.0929668084616029*G0_2_0 - 0.215291556437392*G0_2_1 - 0.234863516113516*G0_2_2;
    A[2951] = -0.0260959462348366*G0_0_1 - 0.0750258454251523*G0_0_2 - 0.0260959462348371*G0_1_0 - 0.0521918924696708*G0_1_1 - 0.042405912631609*G0_1_2 - 0.0750258454251512*G0_2_0 - 0.0424059126316077*G0_2_1 + 0.143527704291591*G0_2_2;
    A[2952] = -0.150051690850299*G0_0_0 - 0.466465038947674*G0_0_1 - 0.335985307773499*G0_0_2 - 0.466465038947672*G0_1_0 - 0.782878387045047*G0_1_1 - 0.466465038947674*G0_1_2 - 0.335985307773498*G0_2_0 - 0.466465038947674*G0_2_1 - 0.150051690850299*G0_2_2;
    A[2953] = -0.039143919352251*G0_0_0 + 0.445262082631867*G0_0_1 + 0.249542485870606*G0_0_2 + 0.445262082631869*G0_1_0 + 0.146789697570944*G0_1_1 + 0.249542485870607*G0_1_2 + 0.249542485870609*G0_2_0 + 0.249542485870606*G0_2_1 + 0.352295274170269*G0_2_2;
    A[2954] = 0.143527704291591*G0_0_0 - 0.0750258454251488*G0_0_1 - 0.0424059126316064*G0_0_2 - 0.0750258454251507*G0_1_0 - 0.026095946234834*G0_1_2 - 0.0424059126316068*G0_2_0 - 0.0260959462348343*G0_2_1 - 0.0521918924696682*G0_2_2;
    A[2955] = 0.352295274170268*G0_0_0 + 0.249542485870608*G0_0_1 + 0.24954248587061*G0_0_2 + 0.249542485870608*G0_1_0 + 0.146789697570948*G0_1_1 + 0.445262082631872*G0_1_2 + 0.249542485870607*G0_2_0 + 0.445262082631871*G0_2_1 - 0.0391439193522529*G0_2_2;
    A[2956] = -0.234863516113512*G0_0_0 - 0.0929668084615993*G0_0_1 - 0.215291556437386*G0_0_2 - 0.0929668084615979*G0_1_0 - 0.0929668084615984*G0_1_2 - 0.215291556437386*G0_2_0 - 0.0929668084615982*G0_2_1 - 0.234863516113514*G0_2_2;
    A[2957] = -0.0521918924696689*G0_0_0 - 0.0260959462348336*G0_0_1 - 0.0424059126316068*G0_0_2 - 0.026095946234835*G0_1_0 - 0.0750258454251507*G0_1_2 - 0.0424059126316059*G0_2_0 - 0.0750258454251495*G0_2_1 + 0.143527704291594*G0_2_2;
    A[2958] = -0.150051690850299*G0_0_0 - 0.335985307773499*G0_0_1 - 0.466465038947674*G0_0_2 - 0.335985307773498*G0_1_0 - 0.1500516908503*G0_1_1 - 0.466465038947674*G0_1_2 - 0.466465038947671*G0_2_0 - 0.466465038947673*G0_2_1 - 0.782878387045047*G0_2_2;
    A[2959] = -0.0391439193522522*G0_0_0 + 0.249542485870605*G0_0_1 + 0.445262082631866*G0_0_2 + 0.249542485870608*G0_1_0 + 0.352295274170268*G0_1_1 + 0.249542485870605*G0_1_2 + 0.44526208263187*G0_2_0 + 0.249542485870607*G0_2_1 + 0.146789697570944*G0_2_2;
    A[2960] = 0.14352770429159*G0_0_0 - 0.0424059126316065*G0_0_1 - 0.0750258454251491*G0_0_2 - 0.042405912631608*G0_1_0 - 0.0521918924696686*G0_1_1 - 0.0260959462348352*G0_1_2 - 0.0750258454251512*G0_2_0 - 0.0260959462348344*G0_2_1;
    A[2961] = 0.352295274170267*G0_0_0 + 0.24954248587061*G0_0_1 + 0.249542485870608*G0_0_2 + 0.249542485870605*G0_1_0 - 0.0391439193522523*G0_1_1 + 0.445262082631868*G0_1_2 + 0.249542485870606*G0_2_0 + 0.44526208263187*G0_2_1 + 0.146789697570946*G0_2_2;
    A[2962] = -0.234863516113511*G0_0_0 - 0.215291556437386*G0_0_1 - 0.092966808461599*G0_0_2 - 0.215291556437384*G0_1_0 - 0.234863516113512*G0_1_1 - 0.0929668084615972*G0_1_2 - 0.0929668084615972*G0_2_0 - 0.092966808461599*G0_2_1;
    A[2963] = -0.0521918924696712*G0_0_0 - 0.042405912631608*G0_0_1 - 0.0260959462348354*G0_0_2 - 0.0424059126316076*G0_1_0 + 0.143527704291592*G0_1_1 - 0.075025845425152*G0_1_2 - 0.026095946234836*G0_2_0 - 0.0750258454251496*G0_2_1;
    A[2964] = 1.76147637085135*G0_0_0 + 0.978597983806306*G0_0_1 + 0.978597983806306*G0_0_2 + 0.978597983806306*G0_1_0 + 1.76147637085136*G0_1_1 + 0.978597983806311*G0_1_2 + 0.978597983806305*G0_2_0 + 0.978597983806311*G0_2_1 + 1.76147637085136*G0_2_2;
    A[2965] = -1.17431758056756*G0_0_0 - 0.587158790283781*G0_0_1 - 0.58715879028378*G0_0_2 - 0.58715879028378*G0_1_0 - 0.293579395141887*G0_1_1 - 0.342509294332205*G0_1_2 - 0.58715879028378*G0_2_0 - 0.342509294332203*G0_2_1 - 0.293579395141888*G0_2_2;
    A[2966] = -0.293579395141892*G0_0_0 - 0.587158790283781*G0_0_1 - 0.342509294332205*G0_0_2 - 0.587158790283785*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283783*G0_1_2 - 0.342509294332209*G0_2_0 - 0.587158790283789*G0_2_1 - 0.293579395141894*G0_2_2;
    A[2967] = -0.293579395141895*G0_0_0 - 0.342509294332208*G0_0_1 - 0.587158790283783*G0_0_2 - 0.342509294332214*G0_1_0 - 0.293579395141896*G0_1_1 - 0.587158790283791*G0_1_2 - 0.587158790283788*G0_2_0 - 0.587158790283787*G0_2_1 - 1.17431758056757*G0_2_2;
    A[2968] = 0.044036909271284*G0_0_0 + 0.044036909271284*G0_0_1 + 0.044036909271284*G0_0_2 + 0.0440369092712836*G0_1_0 + 0.0440369092712835*G0_1_1 + 0.0440369092712836*G0_1_2 + 0.0440369092712837*G0_2_0 + 0.0440369092712836*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[2969] = -0.0782878387045046*G0_0_0;
    A[2970] = 0.0440369092712845*G0_1_1;
    A[2971] = 0.0440369092712842*G0_2_2;
    A[2972] = -0.0505608958299939*G0_1_1 - 0.00260959462348417*G0_1_2 - 0.00260959462348376*G0_2_1;
    A[2973] = 0.0391439193522537*G0_1_1 + 0.0319675341376733*G0_1_2 + 0.0319675341376735*G0_2_1 + 0.0260959462348361*G0_2_2;
    A[2974] = 0.026095946234834*G0_1_1 + 0.0319675341376724*G0_1_2 + 0.0319675341376725*G0_2_1 + 0.0391439193522517*G0_2_2;
    A[2975] = -0.00260959462348385*G0_1_2 - 0.00260959462348355*G0_2_1 - 0.0505608958299935*G0_2_2;
    A[2976] = 0.0962288017409542*G0_0_0 - 0.0352295274170267*G0_0_2 - 0.0352295274170279*G0_2_0;
    A[2977] = -0.15657567740901*G0_0_0 - 0.0903572138381164*G0_0_2 - 0.0903572138381155*G0_2_0 - 0.185933616923199*G0_2_2;
    A[2978] = 0.026095946234836*G0_0_0 + 0.0156575677409022*G0_0_2 + 0.0156575677409018*G0_2_0 + 0.1859336169232*G0_2_2;
    A[2979] = -0.00260959462348298*G0_0_2 - 0.00260959462348332*G0_2_0 - 0.15657567740901*G0_2_2;
    A[2980] = 0.0962288017409541*G0_0_0 - 0.0352295274170264*G0_0_1 - 0.0352295274170271*G0_1_0;
    A[2981] = -0.156575677409009*G0_0_0 - 0.0903572138381154*G0_0_1 - 0.0903572138381158*G0_1_0 - 0.185933616923198*G0_1_1;
    A[2982] = 0.0260959462348343*G0_0_0 + 0.0156575677409011*G0_0_1 + 0.015657567740901*G0_1_0 + 0.185933616923199*G0_1_1;
    A[2983] = -0.00260959462348462*G0_0_1 - 0.00260959462348408*G0_1_0 - 0.156575677409011*G0_1_1;
    A[2984] = -0.0505608958299915*G0_0_0 - 0.0505608958299916*G0_0_1 - 0.0479513012065098*G0_0_2 - 0.0505608958299924*G0_1_0 - 0.0505608958299924*G0_1_1 - 0.0479513012065089*G0_1_2 - 0.0479513012065087*G0_2_0 - 0.0479513012065088*G0_2_1 - 0.0453417065830257*G0_2_2;
    A[2985] = 0.039143919352251*G0_0_0 + 0.0391439193522509*G0_0_1 + 0.00717638521458006*G0_0_2 + 0.0391439193522518*G0_1_0 + 0.0391439193522517*G0_1_1 + 0.00717638521457909*G0_1_2 + 0.00717638521457919*G0_2_0 + 0.0071763852145794*G0_2_1 + 0.00130479731174159*G0_2_2;
    A[2986] = 0.0260959462348353*G0_0_0 + 0.0260959462348353*G0_0_1 - 0.00587158790283756*G0_0_2 + 0.0260959462348347*G0_1_0 + 0.0260959462348347*G0_1_1 - 0.00587158790283711*G0_1_2 - 0.00587158790283799*G0_2_0 - 0.00587158790283796*G0_2_1 + 0.00130479731174215*G0_2_2;
    A[2987] = 0.00260959462348345*G0_0_2 + 0.00260959462348293*G0_1_2 + 0.00260959462348358*G0_2_0 + 0.00260959462348348*G0_2_1 - 0.0453417065830261*G0_2_2;
    A[2988] = -0.0505608958299915*G0_0_0 - 0.0479513012065098*G0_0_1 - 0.0505608958299917*G0_0_2 - 0.0479513012065089*G0_1_0 - 0.0453417065830252*G0_1_1 - 0.047951301206509*G0_1_2 - 0.0505608958299924*G0_2_0 - 0.0479513012065092*G0_2_1 - 0.0505608958299925*G0_2_2;
    A[2989] = 0.0391439193522517*G0_0_0 + 0.00717638521457997*G0_0_1 + 0.0391439193522519*G0_0_2 + 0.00717638521458053*G0_1_0 + 0.00130479731174142*G0_1_1 + 0.00717638521458061*G0_1_2 + 0.0391439193522524*G0_2_0 + 0.00717638521457979*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[2990] = 0.0260959462348336*G0_0_0 - 0.00587158790283718*G0_0_1 + 0.0260959462348333*G0_0_2 - 0.00587158790283859*G0_1_0 + 0.00130479731174249*G0_1_1 - 0.00587158790283869*G0_1_2 + 0.0260959462348338*G0_2_0 - 0.00587158790283778*G0_2_1 + 0.0260959462348335*G0_2_2;
    A[2991] = 0.00260959462348303*G0_0_1 + 0.00260959462348404*G0_1_0 - 0.0453417065830267*G0_1_1 + 0.00260959462348403*G0_1_2 + 0.00260959462348343*G0_2_1;
    A[2992] = -0.151356488162042*G0_0_0 - 0.153966082785524*G0_0_1 - 0.153966082785524*G0_0_2 - 0.153966082785526*G0_1_0 - 0.156575677409009*G0_1_1 - 0.156575677409009*G0_1_2 - 0.153966082785526*G0_2_0 - 0.156575677409009*G0_2_1 - 0.156575677409009*G0_2_2;
    A[2993] = 0.180714427676231*G0_0_0 + 0.170276049182296*G0_0_1 + 0.170276049182296*G0_0_2 + 0.170276049182297*G0_1_0 + 0.185933616923198*G0_1_1 + 0.185933616923198*G0_1_2 + 0.170276049182297*G0_2_0 + 0.185933616923198*G0_2_1 + 0.185933616923197*G0_2_2;
    A[2994] = -0.161794866655975*G0_0_0 - 0.0955764030850821*G0_0_1 - 0.0955764030850822*G0_0_2 - 0.0955764030850825*G0_1_0 - 0.185933616923199*G0_1_1 - 0.185933616923199*G0_1_2 - 0.0955764030850818*G0_2_0 - 0.185933616923198*G0_2_1 - 0.185933616923198*G0_2_2;
    A[2995] = 0.166687856575007*G0_0_0 + 0.0352295274170263*G0_0_1 + 0.0352295274170261*G0_0_2 + 0.0352295274170271*G0_1_0 + 0.0352295274170261*G0_2_0;
    A[2996] = -0.782878387045047*G0_0_0 - 0.316413348097372*G0_0_1 - 0.316413348097371*G0_0_2 - 0.316413348097372*G0_1_0 - 0.185933616923197*G0_1_2 - 0.316413348097371*G0_2_0 - 0.185933616923198*G0_2_1;
    A[2997] = 0.146789697570948*G0_0_0 - 0.298472385060923*G0_0_1 - 0.102752788299663*G0_0_2 - 0.298472385060926*G0_1_0 - 0.782878387045049*G0_1_1 - 0.298472385060926*G0_1_2 - 0.102752788299665*G0_2_0 - 0.298472385060926*G0_2_1;
    A[2998] = 0.0750258454251508*G0_0_1 + 0.026095946234834*G0_0_2 + 0.0750258454251495*G0_1_0 + 0.293579395141893*G0_1_1 + 0.0587158790283776*G0_1_2 + 0.0260959462348364*G0_2_0 + 0.0587158790283798*G0_2_1;
    A[2999] = 0.146789697570951*G0_0_0 - 0.102752788299661*G0_0_1 - 0.298472385060923*G0_0_2 - 0.102752788299663*G0_1_0 - 0.298472385060925*G0_1_2 - 0.298472385060924*G0_2_0 - 0.298472385060923*G0_2_1 - 0.782878387045049*G0_2_2;
    A[3000] = 0.0929668084615984*G0_0_1 + 0.0929668084616003*G0_0_2 + 0.0929668084615998*G0_1_0 - 0.0489298991903166*G0_1_1 - 0.029357939514189*G0_1_2 + 0.0929668084615991*G0_2_0 - 0.0293579395141909*G0_2_1 - 0.0489298991903166*G0_2_2;
    A[3001] = 0.0260959462348368*G0_0_1 + 0.0750258454251511*G0_0_2 + 0.0260959462348341*G0_1_0 + 0.0587158790283787*G0_1_2 + 0.0750258454251505*G0_2_0 + 0.0587158790283798*G0_2_1 + 0.293579395141895*G0_2_2;
    A[3002] = 0.195719596761262*G0_0_0 + 0.120693751336111*G0_0_1 + 0.12069375133611*G0_0_2 + 0.120693751336112*G0_1_0 + 0.0456679059109619*G0_1_1 + 0.0717638521457951*G0_1_2 + 0.120693751336112*G0_2_0 + 0.0717638521457963*G0_2_1 + 0.0456679059109609*G0_2_2;
    A[3003] = 0.146789697570944*G0_0_0 + 0.00489298991902858*G0_0_1 + 0.0538228891093486*G0_0_2 + 0.00489298991903187*G0_1_0 + 0.00978597983806057*G0_1_1 + 0.00489298991903137*G0_1_2 + 0.0538228891093447*G0_2_0 + 0.00489298991902907*G0_2_1 - 0.0391439193522526*G0_2_2;
    A[3004] = -0.0750258454251474*G0_0_1 - 0.0260959462348332*G0_0_2 - 0.0750258454251492*G0_1_0 + 0.0456679059109638*G0_1_1 - 0.0260959462348342*G0_1_2 - 0.0260959462348362*G0_2_0 - 0.0260959462348326*G0_2_1 - 0.0521918924696686*G0_2_2;
    A[3005] = 0.146789697570947*G0_0_0 + 0.0538228891093485*G0_0_1 + 0.00489298991903124*G0_0_2 + 0.0538228891093465*G0_1_0 - 0.0391439193522516*G0_1_1 + 0.00489298991903155*G0_1_2 + 0.00489298991903125*G0_2_0 + 0.00489298991903148*G0_2_1 + 0.00978597983806229*G0_2_2;
    A[3006] = -0.0929668084615984*G0_0_1 - 0.0929668084615982*G0_0_2 - 0.0929668084615996*G0_1_0 - 0.0391439193522533*G0_1_1 - 0.0440369092712843*G0_1_2 - 0.092966808461599*G0_2_0 - 0.0440369092712839*G0_2_1 - 0.0391439193522521*G0_2_2;
    A[3007] = -0.0260959462348339*G0_0_1 - 0.0750258454251499*G0_0_2 - 0.0260959462348342*G0_1_0 - 0.0521918924696681*G0_1_1 - 0.0260959462348338*G0_1_2 - 0.0750258454251504*G0_2_0 - 0.026095946234834*G0_2_1 + 0.0456679059109627*G0_2_2;
    A[3008] = 0.143527704291591*G0_0_0 + 0.218553549716741*G0_0_1 + 0.185933616923197*G0_0_2 + 0.218553549716742*G0_1_0 + 0.293579395141891*G0_1_1 + 0.234863516113513*G0_1_2 + 0.185933616923198*G0_2_0 + 0.234863516113514*G0_2_1 + 0.176147637085135*G0_2_2;
    A[3009] = -0.0391439193522528*G0_0_0 - 0.484406001984118*G0_0_1 - 0.288686405222857*G0_0_2 - 0.484406001984123*G0_1_0 - 0.782878387045045*G0_1_1 - 0.48440600198412*G0_1_2 - 0.288686405222862*G0_2_0 - 0.484406001984121*G0_2_1 - 0.185933616923197*G0_2_2;
    A[3010] = -0.150051690850301*G0_0_0 + 0.316413348097371*G0_0_1 + 0.185933616923199*G0_0_2 + 0.316413348097373*G0_1_0 + 0.185933616923199*G0_1_2 + 0.1859336169232*G0_2_0 + 0.185933616923198*G0_2_1 + 0.371867233846398*G0_2_2;
    A[3011] = -0.234863516113512*G0_0_0 - 0.141896707651913*G0_0_1 - 0.0195719596761267*G0_0_2 - 0.141896707651913*G0_1_0 - 0.0489298991903153*G0_1_1 - 0.0195719596761271*G0_1_2 - 0.0195719596761249*G0_2_0 - 0.0195719596761259*G0_2_1 - 0.0391439193522539*G0_2_2;
    A[3012] = 0.35229527417027*G0_0_0 + 0.102752788299661*G0_0_1 + 0.102752788299659*G0_0_2 + 0.10275278829966*G0_1_0 + 0.298472385060922*G0_1_2 + 0.102752788299661*G0_2_0 + 0.298472385060923*G0_2_1 - 0.185933616923201*G0_2_2;
    A[3013] = -0.0521918924696715*G0_0_0 - 0.0260959462348373*G0_0_1 - 0.00978597983806287*G0_0_2 - 0.0260959462348346*G0_1_0 - 0.0587158790283772*G0_1_2 - 0.00978597983806385*G0_2_0 - 0.0587158790283802*G0_2_1 + 0.176147637085137*G0_2_2;
    A[3014] = 0.14352770429159*G0_0_0 + 0.185933616923197*G0_0_1 + 0.218553549716742*G0_0_2 + 0.185933616923198*G0_1_0 + 0.176147637085136*G0_1_1 + 0.234863516113514*G0_1_2 + 0.218553549716742*G0_2_0 + 0.234863516113514*G0_2_1 + 0.293579395141892*G0_2_2;
    A[3015] = -0.0391439193522519*G0_0_0 - 0.288686405222856*G0_0_1 - 0.484406001984117*G0_0_2 - 0.288686405222861*G0_1_0 - 0.185933616923196*G0_1_1 - 0.484406001984122*G0_1_2 - 0.484406001984123*G0_2_0 - 0.48440600198412*G0_2_1 - 0.782878387045045*G0_2_2;
    A[3016] = -0.1500516908503*G0_0_0 + 0.185933616923198*G0_0_1 + 0.316413348097371*G0_0_2 + 0.185933616923199*G0_1_0 + 0.371867233846398*G0_1_1 + 0.185933616923198*G0_1_2 + 0.316413348097375*G0_2_0 + 0.185933616923198*G0_2_1;
    A[3017] = -0.234863516113511*G0_0_0 - 0.0195719596761285*G0_0_1 - 0.141896707651915*G0_0_2 - 0.0195719596761259*G0_1_0 - 0.0391439193522545*G0_1_1 - 0.0195719596761259*G0_1_2 - 0.141896707651914*G0_2_0 - 0.0195719596761281*G0_2_1 - 0.0489298991903159*G0_2_2;
    A[3018] = 0.352295274170268*G0_0_0 + 0.10275278829966*G0_0_1 + 0.102752788299663*G0_0_2 + 0.102752788299661*G0_1_0 - 0.1859336169232*G0_1_1 + 0.298472385060925*G0_1_2 + 0.10275278829966*G0_2_0 + 0.298472385060924*G0_2_1;
    A[3019] = -0.0521918924696676*G0_0_0 - 0.00978597983806018*G0_0_1 - 0.0260959462348336*G0_0_2 - 0.00978597983806206*G0_1_0 + 0.176147637085138*G0_1_1 - 0.0587158790283768*G0_1_2 - 0.026095946234833*G0_2_0 - 0.0587158790283768*G0_2_1;
    A[3020] = -1.17431758056756*G0_0_0 - 0.58715879028378*G0_0_1 - 0.58715879028378*G0_0_2 - 0.587158790283781*G0_1_0 - 0.293579395141887*G0_1_1 - 0.342509294332203*G0_1_2 - 0.58715879028378*G0_2_0 - 0.342509294332205*G0_2_1 - 0.293579395141888*G0_2_2;
    A[3021] = 1.76147637085135*G0_0_0 + 0.782878387045041*G0_0_1 + 0.78287838704504*G0_0_2 + 0.782878387045041*G0_1_0 + 1.56575677409009*G0_1_1 + 0.782878387045047*G0_1_2 + 0.78287838704504*G0_2_0 + 0.782878387045047*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3022] = -0.293579395141893*G0_0_0 + 0.293579395141888*G0_0_1 + 0.0489298991903135*G0_0_2 + 0.293579395141893*G0_1_0 - 0.293579395141898*G0_1_1 + 0.0489298991903142*G0_1_2 + 0.0489298991903201*G0_2_0 + 0.0489298991903114*G0_2_1 + 0.0978597983806313*G0_2_2;
    A[3023] = -0.293579395141894*G0_0_0 + 0.0489298991903151*G0_0_1 + 0.293579395141889*G0_0_2 + 0.0489298991903176*G0_1_0 + 0.0978597983806316*G0_1_1 + 0.0489298991903127*G0_1_2 + 0.293579395141894*G0_2_0 + 0.0489298991903164*G0_2_1 - 0.293579395141896*G0_2_2;
    A[3024] = 0.0440369092712837*G0_0_0 + 0.0440369092712836*G0_0_1 + 0.0440369092712837*G0_0_2 + 0.0440369092712838*G0_1_0 + 0.044036909271284*G0_1_1 + 0.0440369092712839*G0_1_2 + 0.0440369092712836*G0_2_0 + 0.0440369092712837*G0_2_1 + 0.0440369092712837*G0_2_2;
    A[3025] = 0.0440369092712838*G0_0_0;
    A[3026] = -0.0782878387045055*G0_1_1;
    A[3027] = 0.0440369092712842*G0_2_2;
    A[3028] = 0.0962288017409538*G0_1_1 - 0.035229527417028*G0_1_2 - 0.0352295274170264*G0_2_1;
    A[3029] = -0.15657567740901*G0_1_1 - 0.0903572138381159*G0_1_2 - 0.0903572138381161*G0_2_1 - 0.1859336169232*G0_2_2;
    A[3030] = 0.0260959462348341*G0_1_1 + 0.0156575677408997*G0_1_2 + 0.0156575677409005*G0_2_1 + 0.1859336169232*G0_2_2;
    A[3031] = -0.00260959462348334*G0_1_2 - 0.00260959462348288*G0_2_1 - 0.15657567740901*G0_2_2;
    A[3032] = -0.0505608958299927*G0_0_0 - 0.00260959462348396*G0_0_2 - 0.00260959462348314*G0_2_0;
    A[3033] = 0.0391439193522515*G0_0_0 + 0.0319675341376729*G0_0_2 + 0.0319675341376715*G0_2_0 + 0.0260959462348351*G0_2_2;
    A[3034] = 0.0260959462348356*G0_0_0 + 0.0319675341376724*G0_0_2 + 0.0319675341376737*G0_2_0 + 0.0391439193522523*G0_2_2;
    A[3035] = -0.00260959462348375*G0_0_2 - 0.00260959462348322*G0_2_0 - 0.0505608958299921*G0_2_2;
    A[3036] = -0.15657567740901*G0_0_0 - 0.00260959462348391*G0_0_1 - 0.00260959462348406*G0_1_0;
    A[3037] = 0.185933616923199*G0_0_0 + 0.0156575677409006*G0_0_1 + 0.0156575677409016*G0_1_0 + 0.0260959462348353*G0_1_1;
    A[3038] = -0.185933616923198*G0_0_0 - 0.0903572138381149*G0_0_1 - 0.0903572138381151*G0_1_0 - 0.156575677409009*G0_1_1;
    A[3039] = -0.0352295274170276*G0_0_1 - 0.0352295274170279*G0_1_0 + 0.096228801740954*G0_1_1;
    A[3040] = -0.0505608958299925*G0_0_0 - 0.0505608958299924*G0_0_1 - 0.0479513012065086*G0_0_2 - 0.0505608958299917*G0_1_0 - 0.0505608958299918*G0_1_1 - 0.0479513012065091*G0_1_2 - 0.0479513012065078*G0_2_0 - 0.0479513012065077*G0_2_1 - 0.0453417065830252*G0_2_2;
    A[3041] = 0.0391439193522529*G0_0_0 + 0.0391439193522527*G0_0_1 + 0.00717638521457887*G0_0_2 + 0.0391439193522522*G0_1_0 + 0.0391439193522522*G0_1_1 + 0.00717638521457927*G0_1_2 + 0.00717638521457905*G0_2_0 + 0.00717638521457905*G0_2_1 + 0.0013047973117411*G0_2_2;
    A[3042] = 0.0260959462348341*G0_0_0 + 0.0260959462348342*G0_0_1 - 0.00587158790283732*G0_0_2 + 0.0260959462348348*G0_1_0 + 0.0260959462348351*G0_1_1 - 0.00587158790283736*G0_1_2 - 0.00587158790283671*G0_2_0 - 0.00587158790283658*G0_2_1 + 0.00130479731174219*G0_2_2;
    A[3043] = 0.00260959462348333*G0_0_2 + 0.00260959462348362*G0_1_2 + 0.00260959462348312*G0_2_0 + 0.0026095946234831*G0_2_1 - 0.0453417065830261*G0_2_2;
    A[3044] = -0.156575677409009*G0_0_0 - 0.153966082785525*G0_0_1 - 0.156575677409009*G0_0_2 - 0.153966082785525*G0_1_0 - 0.151356488162043*G0_1_1 - 0.153966082785525*G0_1_2 - 0.156575677409008*G0_2_0 - 0.153966082785525*G0_2_1 - 0.156575677409008*G0_2_2;
    A[3045] = 0.185933616923198*G0_0_0 + 0.170276049182297*G0_0_1 + 0.185933616923199*G0_0_2 + 0.170276049182297*G0_1_0 + 0.180714427676232*G0_1_1 + 0.170276049182298*G0_1_2 + 0.185933616923197*G0_2_0 + 0.170276049182298*G0_2_1 + 0.185933616923198*G0_2_2;
    A[3046] = -0.185933616923198*G0_0_0 - 0.0955764030850826*G0_0_1 - 0.185933616923198*G0_0_2 - 0.0955764030850827*G0_1_0 - 0.161794866655977*G0_1_1 - 0.095576403085083*G0_1_2 - 0.185933616923197*G0_2_0 - 0.0955764030850832*G0_2_1 - 0.185933616923197*G0_2_2;
    A[3047] = 0.035229527417027*G0_0_1 + 0.0352295274170277*G0_1_0 + 0.166687856575009*G0_1_1 + 0.0352295274170278*G0_1_2 + 0.0352295274170276*G0_2_1;
    A[3048] = -0.0453417065830255*G0_0_0 - 0.0479513012065093*G0_0_1 - 0.0479513012065094*G0_0_2 - 0.0479513012065092*G0_1_0 - 0.050560895829992*G0_1_1 - 0.0505608958299921*G0_1_2 - 0.0479513012065091*G0_2_0 - 0.0505608958299915*G0_2_1 - 0.0505608958299917*G0_2_2;
    A[3049] = 0.00130479731174175*G0_0_0 + 0.00717638521457988*G0_0_1 + 0.00717638521457977*G0_0_2 + 0.00717638521457995*G0_1_0 + 0.0391439193522511*G0_1_1 + 0.0391439193522512*G0_1_2 + 0.0071763852145798*G0_2_0 + 0.0391439193522521*G0_2_1 + 0.0391439193522522*G0_2_2;
    A[3050] = 0.00130479731174151*G0_0_0 - 0.00587158790283843*G0_0_1 - 0.0058715879028385*G0_0_2 - 0.00587158790283755*G0_1_0 + 0.0260959462348356*G0_1_1 + 0.0260959462348357*G0_1_2 - 0.00587158790283827*G0_2_0 + 0.0260959462348349*G0_2_1 + 0.026095946234835*G0_2_2;
    A[3051] = -0.0453417065830257*G0_0_0 + 0.0026095946234841*G0_0_1 + 0.00260959462348393*G0_0_2 + 0.00260959462348271*G0_1_0 + 0.0026095946234838*G0_2_0;
    A[3052] = 0.293579395141893*G0_0_0 + 0.0750258454251495*G0_0_1 + 0.058715879028378*G0_0_2 + 0.0750258454251512*G0_1_0 + 0.0260959462348347*G0_1_2 + 0.0587158790283787*G0_2_0 + 0.0260959462348348*G0_2_1;
    A[3053] = -0.782878387045047*G0_0_0 - 0.298472385060924*G0_0_1 - 0.298472385060923*G0_0_2 - 0.298472385060924*G0_1_0 + 0.146789697570947*G0_1_1 - 0.102752788299661*G0_1_2 - 0.298472385060925*G0_2_0 - 0.102752788299662*G0_2_1;
    A[3054] = -0.316413348097375*G0_0_1 - 0.1859336169232*G0_0_2 - 0.316413348097375*G0_1_0 - 0.782878387045049*G0_1_1 - 0.316413348097374*G0_1_2 - 0.185933616923198*G0_2_0 - 0.316413348097374*G0_2_1;
    A[3055] = -0.0489298991903166*G0_0_0 + 0.0929668084615985*G0_0_1 - 0.0293579395141909*G0_0_2 + 0.0929668084616015*G0_1_0 + 0.0929668084616014*G0_1_2 - 0.0293579395141877*G0_2_0 + 0.0929668084615994*G0_2_1 - 0.0489298991903164*G0_2_2;
    A[3056] = -0.102752788299662*G0_0_1 - 0.298472385060924*G0_0_2 - 0.102752788299665*G0_1_0 + 0.146789697570946*G0_1_1 - 0.298472385060926*G0_1_2 - 0.298472385060924*G0_2_0 - 0.298472385060923*G0_2_1 - 0.782878387045048*G0_2_2;
    A[3057] = 0.0260959462348335*G0_0_1 + 0.0587158790283779*G0_0_2 + 0.026095946234833*G0_1_0 + 0.0750258454251493*G0_1_2 + 0.0587158790283774*G0_2_0 + 0.0750258454251504*G0_2_1 + 0.293579395141894*G0_2_2;
    A[3058] = 0.293579395141893*G0_0_0 + 0.218553549716742*G0_0_1 + 0.234863516113514*G0_0_2 + 0.218553549716743*G0_1_0 + 0.143527704291591*G0_1_1 + 0.185933616923198*G0_1_2 + 0.234863516113514*G0_2_0 + 0.185933616923196*G0_2_1 + 0.176147637085134*G0_2_2;
    A[3059] = -0.782878387045046*G0_0_0 - 0.484406001984121*G0_0_1 - 0.484406001984123*G0_0_2 - 0.484406001984124*G0_1_0 - 0.0391439193522519*G0_1_1 - 0.288686405222862*G0_1_2 - 0.484406001984119*G0_2_0 - 0.288686405222858*G0_2_1 - 0.185933616923198*G0_2_2;
    A[3060] = 0.316413348097373*G0_0_1 + 0.185933616923198*G0_0_2 + 0.316413348097375*G0_1_0 - 0.150051690850301*G0_1_1 + 0.185933616923199*G0_1_2 + 0.185933616923199*G0_2_0 + 0.185933616923196*G0_2_1 + 0.371867233846396*G0_2_2;
    A[3061] = -0.0489298991903189*G0_0_0 - 0.141896707651916*G0_0_1 - 0.0195719596761275*G0_0_2 - 0.14189670765192*G0_1_0 - 0.234863516113517*G0_1_1 - 0.019571959676128*G0_1_2 - 0.0195719596761289*G0_2_0 - 0.019571959676128*G0_2_1 - 0.0391439193522526*G0_2_2;
    A[3062] = 0.102752788299663*G0_0_1 + 0.298472385060925*G0_0_2 + 0.102752788299665*G0_1_0 + 0.352295274170276*G0_1_1 + 0.102752788299665*G0_1_2 + 0.298472385060924*G0_2_0 + 0.102752788299665*G0_2_1 - 0.185933616923199*G0_2_2;
    A[3063] = -0.0260959462348332*G0_0_1 - 0.0587158790283763*G0_0_2 - 0.0260959462348331*G0_1_0 - 0.0521918924696685*G0_1_1 - 0.00978597983806137*G0_1_2 - 0.0587158790283772*G0_2_0 - 0.00978597983806342*G0_2_1 + 0.176147637085137*G0_2_2;
    A[3064] = 0.0456679059109603*G0_0_0 + 0.120693751336112*G0_0_1 + 0.0717638521457959*G0_0_2 + 0.120693751336109*G0_1_0 + 0.195719596761261*G0_1_1 + 0.120693751336111*G0_1_2 + 0.0717638521457944*G0_2_0 + 0.120693751336112*G0_2_1 + 0.0456679059109599*G0_2_2;
    A[3065] = 0.00978597983806276*G0_0_0 + 0.00489298991903101*G0_0_1 + 0.00489298991903185*G0_0_2 + 0.00489298991903237*G0_1_0 + 0.146789697570948*G0_1_1 + 0.0538228891093487*G0_1_2 + 0.0048929899190318*G0_2_0 + 0.0538228891093475*G0_2_1 - 0.0391439193522522*G0_2_2;
    A[3066] = 0.0456679059109619*G0_0_0 - 0.0750258454251495*G0_0_1 - 0.0260959462348349*G0_0_2 - 0.0750258454251499*G0_1_0 - 0.0260959462348368*G0_1_2 - 0.0260959462348355*G0_2_0 - 0.0260959462348345*G0_2_1 - 0.0521918924696703*G0_2_2;
    A[3067] = -0.0391439193522518*G0_0_0 + 0.0538228891093454*G0_0_1 + 0.00489298991903116*G0_0_2 + 0.0538228891093483*G0_1_0 + 0.146789697570945*G0_1_1 + 0.00489298991903134*G0_1_2 + 0.00489298991903152*G0_2_0 + 0.0048929899190307*G0_2_1 + 0.00978597983806356*G0_2_2;
    A[3068] = -0.0391439193522528*G0_0_0 - 0.0929668084615977*G0_0_1 - 0.044036909271284*G0_0_2 - 0.0929668084616008*G0_1_0 - 0.0929668084615996*G0_1_2 - 0.0440369092712825*G0_2_0 - 0.0929668084615989*G0_2_1 - 0.0391439193522523*G0_2_2;
    A[3069] = -0.0521918924696688*G0_0_0 - 0.0260959462348339*G0_0_1 - 0.0260959462348339*G0_0_2 - 0.0260959462348352*G0_1_0 - 0.075025845425151*G0_1_2 - 0.0260959462348365*G0_2_0 - 0.0750258454251508*G0_2_1 + 0.0456679059109605*G0_2_2;
    A[3070] = 0.176147637085135*G0_0_0 + 0.185933616923198*G0_0_1 + 0.234863516113514*G0_0_2 + 0.185933616923197*G0_1_0 + 0.143527704291592*G0_1_1 + 0.218553549716742*G0_1_2 + 0.234863516113513*G0_2_0 + 0.21855354971674*G0_2_1 + 0.293579395141892*G0_2_2;
    A[3071] = -0.0391439193522529*G0_0_0 - 0.0195719596761264*G0_0_1 - 0.0195719596761276*G0_0_2 - 0.0195719596761259*G0_1_0 - 0.234863516113511*G0_1_1 - 0.141896707651912*G0_1_2 - 0.0195719596761261*G0_2_0 - 0.141896707651913*G0_2_1 - 0.0489298991903142*G0_2_2;
    A[3072] = 0.176147637085137*G0_0_0 - 0.00978597983806261*G0_0_1 - 0.0587158790283777*G0_0_2 - 0.00978597983806162*G0_1_0 - 0.0521918924696707*G0_1_1 - 0.0260959462348346*G0_1_2 - 0.0587158790283784*G0_2_0 - 0.0260959462348359*G0_2_1;
    A[3073] = -0.185933616923197*G0_0_0 - 0.288686405222859*G0_0_1 - 0.484406001984119*G0_0_2 - 0.288686405222857*G0_1_0 - 0.0391439193522506*G0_1_1 - 0.484406001984119*G0_1_2 - 0.48440600198412*G0_2_0 - 0.484406001984118*G0_2_1 - 0.782878387045041*G0_2_2;
    A[3074] = -0.1859336169232*G0_0_0 + 0.102752788299662*G0_0_1 + 0.298472385060923*G0_0_2 + 0.102752788299659*G0_1_0 + 0.352295274170269*G0_1_1 + 0.102752788299661*G0_1_2 + 0.298472385060922*G0_2_0 + 0.102752788299661*G0_2_1;
    A[3075] = 0.371867233846396*G0_0_0 + 0.185933616923198*G0_0_1 + 0.1859336169232*G0_0_2 + 0.185933616923198*G0_1_0 - 0.150051690850302*G0_1_1 + 0.316413348097374*G0_1_2 + 0.185933616923196*G0_2_0 + 0.316413348097369*G0_2_1;
    A[3076] = -0.293579395141892*G0_0_0 - 0.587158790283785*G0_0_1 - 0.342509294332209*G0_0_2 - 0.587158790283781*G0_1_0 - 1.17431758056757*G0_1_1 - 0.587158790283789*G0_1_2 - 0.342509294332205*G0_2_0 - 0.587158790283783*G0_2_1 - 0.293579395141894*G0_2_2;
    A[3077] = -0.293579395141893*G0_0_0 + 0.293579395141893*G0_0_1 + 0.0489298991903201*G0_0_2 + 0.293579395141888*G0_1_0 - 0.293579395141898*G0_1_1 + 0.0489298991903114*G0_1_2 + 0.0489298991903135*G0_2_0 + 0.0489298991903142*G0_2_1 + 0.0978597983806313*G0_2_2;
    A[3078] = 1.56575677409009*G0_0_0 + 0.782878387045048*G0_0_1 + 0.782878387045044*G0_0_2 + 0.782878387045048*G0_1_0 + 1.76147637085136*G0_1_1 + 0.782878387045045*G0_1_2 + 0.782878387045044*G0_2_0 + 0.782878387045045*G0_2_1 + 1.56575677409009*G0_2_2;
    A[3079] = 0.0978597983806336*G0_0_0 + 0.0489298991903157*G0_0_1 + 0.0489298991903169*G0_0_2 + 0.0489298991903187*G0_1_0 - 0.293579395141891*G0_1_1 + 0.293579395141897*G0_1_2 + 0.0489298991903162*G0_2_0 + 0.293579395141892*G0_2_1 - 0.293579395141892*G0_2_2;
    A[3080] = 0.0440369092712844*G0_0_0 + 0.0440369092712844*G0_0_1 + 0.0440369092712843*G0_0_2 + 0.0440369092712843*G0_1_0 + 0.0440369092712843*G0_1_1 + 0.0440369092712843*G0_1_2 + 0.0440369092712845*G0_2_0 + 0.0440369092712845*G0_2_1 + 0.0440369092712845*G0_2_2;
    A[3081] = 0.0440369092712841*G0_0_0;
    A[3082] = 0.044036909271284*G0_1_1;
    A[3083] = -0.0782878387045051*G0_2_2;
    A[3084] = -0.156575677409009*G0_1_1 - 0.00260959462348273*G0_1_2 - 0.00260959462348332*G0_2_1;
    A[3085] = 0.185933616923199*G0_1_1 + 0.0156575677408998*G0_1_2 + 0.0156575677408999*G0_2_1 + 0.0260959462348333*G0_2_2;
    A[3086] = -0.185933616923198*G0_1_1 - 0.0903572138381142*G0_1_2 - 0.0903572138381157*G0_2_1 - 0.156575677409009*G0_2_2;
    A[3087] = -0.0352295274170273*G0_1_2 - 0.0352295274170276*G0_2_1 + 0.0962288017409537*G0_2_2;
    A[3088] = -0.15657567740901*G0_0_0 - 0.00260959462348319*G0_0_2 - 0.00260959462348363*G0_2_0;
    A[3089] = 0.185933616923199*G0_0_0 + 0.0156575677409007*G0_0_2 + 0.0156575677409019*G0_2_0 + 0.0260959462348359*G0_2_2;
    A[3090] = -0.185933616923198*G0_0_0 - 0.0903572138381145*G0_0_2 - 0.0903572138381162*G0_2_0 - 0.15657567740901*G0_2_2;
    A[3091] = -0.0352295274170275*G0_0_2 - 0.0352295274170276*G0_2_0 + 0.0962288017409538*G0_2_2;
    A[3092] = -0.0505608958299923*G0_0_0 - 0.00260959462348312*G0_0_1 - 0.00260959462348312*G0_1_0;
    A[3093] = 0.0391439193522519*G0_0_0 + 0.0319675341376724*G0_0_1 + 0.0319675341376728*G0_1_0 + 0.0260959462348351*G0_1_1;
    A[3094] = 0.0260959462348356*G0_0_0 + 0.0319675341376732*G0_0_1 + 0.031967534137673*G0_1_0 + 0.0391439193522529*G0_1_1;
    A[3095] = -0.00260959462348341*G0_0_1 - 0.00260959462348268*G0_1_0 - 0.0505608958299922*G0_1_1;
    A[3096] = -0.156575677409009*G0_0_0 - 0.156575677409009*G0_0_1 - 0.153966082785527*G0_0_2 - 0.156575677409008*G0_1_0 - 0.156575677409008*G0_1_1 - 0.153966082785526*G0_1_2 - 0.153966082785526*G0_2_0 - 0.153966082785526*G0_2_1 - 0.151356488162044*G0_2_2;
    A[3097] = 0.185933616923198*G0_0_0 + 0.185933616923198*G0_0_1 + 0.170276049182298*G0_0_2 + 0.185933616923196*G0_1_0 + 0.185933616923197*G0_1_1 + 0.170276049182298*G0_1_2 + 0.170276049182297*G0_2_0 + 0.170276049182298*G0_2_1 + 0.180714427676233*G0_2_2;
    A[3098] = -0.185933616923197*G0_0_0 - 0.185933616923197*G0_0_1 - 0.0955764030850823*G0_0_2 - 0.185933616923195*G0_1_0 - 0.185933616923196*G0_1_1 - 0.0955764030850823*G0_1_2 - 0.0955764030850828*G0_2_0 - 0.095576403085083*G0_2_1 - 0.161794866655977*G0_2_2;
    A[3099] = 0.0352295274170264*G0_0_2 + 0.0352295274170265*G0_1_2 + 0.0352295274170275*G0_2_0 + 0.0352295274170275*G0_2_1 + 0.166687856575009*G0_2_2;
    A[3100] = -0.0505608958299924*G0_0_0 - 0.04795130120651*G0_0_1 - 0.0505608958299924*G0_0_2 - 0.0479513012065079*G0_1_0 - 0.0453417065830261*G0_1_1 - 0.0479513012065076*G0_1_2 - 0.050560895829992*G0_2_0 - 0.0479513012065099*G0_2_1 - 0.0505608958299919*G0_2_2;
    A[3101] = 0.0391439193522518*G0_0_0 + 0.00717638521457966*G0_0_1 + 0.039143919352252*G0_0_2 + 0.00717638521457857*G0_1_0 + 0.0013047973117415*G0_1_1 + 0.00717638521457846*G0_1_2 + 0.0391439193522514*G0_2_0 + 0.00717638521457927*G0_2_1 + 0.0391439193522512*G0_2_2;
    A[3102] = 0.026095946234835*G0_0_0 - 0.00587158790283771*G0_0_1 + 0.0260959462348349*G0_0_2 - 0.00587158790283704*G0_1_0 + 0.00130479731174201*G0_1_1 - 0.00587158790283711*G0_1_2 + 0.0260959462348344*G0_2_0 - 0.00587158790283719*G0_2_1 + 0.0260959462348344*G0_2_2;
    A[3103] = 0.0026095946234836*G0_0_1 + 0.00260959462348262*G0_1_0 - 0.0453417065830256*G0_1_1 + 0.00260959462348279*G0_1_2 + 0.00260959462348342*G0_2_1;
    A[3104] = -0.0453417065830267*G0_0_0 - 0.0479513012065091*G0_0_1 - 0.047951301206509*G0_0_2 - 0.0479513012065099*G0_1_0 - 0.0505608958299913*G0_1_1 - 0.0505608958299912*G0_1_2 - 0.0479513012065105*G0_2_0 - 0.0505608958299926*G0_2_1 - 0.0505608958299925*G0_2_2;
    A[3105] = 0.00130479731174289*G0_0_0 + 0.0071763852145799*G0_0_1 + 0.00717638521457981*G0_0_2 + 0.00717638521458036*G0_1_0 + 0.0391439193522528*G0_1_1 + 0.0391439193522526*G0_1_2 + 0.00717638521458124*G0_2_0 + 0.039143919352253*G0_2_1 + 0.0391439193522529*G0_2_2;
    A[3106] = 0.00130479731174114*G0_0_0 - 0.00587158790283811*G0_0_1 - 0.00587158790283811*G0_0_2 - 0.00587158790283845*G0_1_0 + 0.026095946234835*G0_1_1 + 0.026095946234835*G0_1_2 - 0.00587158790283876*G0_2_0 + 0.026095946234834*G0_2_1 + 0.0260959462348339*G0_2_2;
    A[3107] = -0.0453417065830257*G0_0_0 + 0.00260959462348293*G0_0_1 + 0.00260959462348303*G0_0_2 + 0.0026095946234838*G0_1_0 + 0.00260959462348362*G0_2_0;
    A[3108] = 0.293579395141895*G0_0_0 + 0.0587158790283803*G0_0_1 + 0.0750258454251524*G0_0_2 + 0.0587158790283787*G0_1_0 + 0.0260959462348361*G0_1_2 + 0.0750258454251522*G0_2_0 + 0.0260959462348366*G0_2_1;
    A[3109] = -0.0489298991903156*G0_0_0 - 0.0293579395141905*G0_0_1 + 0.0929668084615982*G0_0_2 - 0.0293579395141881*G0_1_0 - 0.0489298991903156*G0_1_1 + 0.0929668084615999*G0_1_2 + 0.0929668084616016*G0_2_0 + 0.0929668084616005*G0_2_1;
    A[3110] = 0.0587158790283794*G0_0_1 + 0.0260959462348363*G0_0_2 + 0.058715879028378*G0_1_0 + 0.293579395141892*G0_1_1 + 0.0750258454251514*G0_1_2 + 0.0260959462348339*G0_2_0 + 0.0750258454251497*G0_2_1;
    A[3111] = -0.782878387045047*G0_0_0 - 0.298472385060923*G0_0_1 - 0.298472385060924*G0_0_2 - 0.298472385060924*G0_1_0 - 0.102752788299664*G0_1_2 - 0.298472385060925*G0_2_0 - 0.102752788299662*G0_2_1 + 0.146789697570949*G0_2_2;
    A[3112] = -0.298472385060925*G0_0_1 - 0.102752788299662*G0_0_2 - 0.298472385060923*G0_1_0 - 0.782878387045045*G0_1_1 - 0.298472385060924*G0_1_2 - 0.102752788299664*G0_2_0 - 0.298472385060924*G0_2_1 + 0.146789697570948*G0_2_2;
    A[3113] = -0.185933616923198*G0_0_1 - 0.316413348097373*G0_0_2 - 0.185933616923197*G0_1_0 - 0.316413348097372*G0_1_2 - 0.316413348097374*G0_2_0 - 0.316413348097375*G0_2_1 - 0.782878387045051*G0_2_2;
    A[3114] = 0.293579395141894*G0_0_0 + 0.234863516113515*G0_0_1 + 0.218553549716742*G0_0_2 + 0.234863516113514*G0_1_0 + 0.176147637085134*G0_1_1 + 0.185933616923195*G0_1_2 + 0.218553549716742*G0_2_0 + 0.185933616923198*G0_2_1 + 0.14352770429159*G0_2_2;
    A[3115] = -0.0489298991903185*G0_0_0 - 0.0195719596761269*G0_0_1 - 0.141896707651916*G0_0_2 - 0.0195719596761277*G0_1_0 - 0.0391439193522516*G0_1_1 - 0.0195719596761258*G0_1_2 - 0.141896707651919*G0_2_0 - 0.0195719596761266*G0_2_1 - 0.234863516113514*G0_2_2;
    A[3116] = -0.0587158790283781*G0_0_1 - 0.0260959462348342*G0_0_2 - 0.0587158790283774*G0_1_0 + 0.176147637085135*G0_1_1 - 0.00978597983806219*G0_1_2 - 0.0260959462348336*G0_2_0 - 0.00978597983806235*G0_2_1 - 0.0521918924696678*G0_2_2;
    A[3117] = -0.782878387045049*G0_0_0 - 0.484406001984124*G0_0_1 - 0.484406001984123*G0_0_2 - 0.484406001984121*G0_1_0 - 0.185933616923197*G0_1_1 - 0.288686405222857*G0_1_2 - 0.484406001984124*G0_2_0 - 0.288686405222861*G0_2_1 - 0.0391439193522517*G0_2_2;
    A[3118] = 0.298472385060926*G0_0_1 + 0.102752788299664*G0_0_2 + 0.298472385060923*G0_1_0 - 0.185933616923197*G0_1_1 + 0.102752788299662*G0_1_2 + 0.102752788299664*G0_2_0 + 0.102752788299664*G0_2_1 + 0.352295274170273*G0_2_2;
    A[3119] = 0.185933616923199*G0_0_1 + 0.316413348097374*G0_0_2 + 0.185933616923197*G0_1_0 + 0.371867233846394*G0_1_1 + 0.185933616923197*G0_1_2 + 0.316413348097374*G0_2_0 + 0.185933616923199*G0_2_1 - 0.150051690850301*G0_2_2;
    A[3120] = 0.176147637085136*G0_0_0 + 0.234863516113515*G0_0_1 + 0.185933616923198*G0_0_2 + 0.234863516113514*G0_1_0 + 0.293579395141893*G0_1_1 + 0.218553549716739*G0_1_2 + 0.185933616923199*G0_2_0 + 0.218553549716742*G0_2_1 + 0.143527704291591*G0_2_2;
    A[3121] = -0.0391439193522548*G0_0_0 - 0.0195719596761271*G0_0_1 - 0.0195719596761272*G0_0_2 - 0.0195719596761282*G0_1_0 - 0.0489298991903143*G0_1_1 - 0.141896707651915*G0_1_2 - 0.019571959676128*G0_2_0 - 0.141896707651913*G0_2_1 - 0.234863516113513*G0_2_2;
    A[3122] = 0.176147637085137*G0_0_0 - 0.0587158790283798*G0_0_1 - 0.00978597983806266*G0_0_2 - 0.0587158790283782*G0_1_0 - 0.0260959462348348*G0_1_2 - 0.00978597983806154*G0_2_0 - 0.0260959462348362*G0_2_1 - 0.0521918924696707*G0_2_2;
    A[3123] = -0.185933616923196*G0_0_0 - 0.484406001984121*G0_0_1 - 0.288686405222859*G0_0_2 - 0.48440600198412*G0_1_0 - 0.782878387045045*G0_1_1 - 0.484406001984119*G0_1_2 - 0.288686405222859*G0_2_0 - 0.484406001984122*G0_2_1 - 0.0391439193522512*G0_2_2;
    A[3124] = -0.185933616923201*G0_0_0 + 0.298472385060922*G0_0_1 + 0.102752788299662*G0_0_2 + 0.298472385060924*G0_1_0 + 0.102752788299662*G0_1_2 + 0.102752788299659*G0_2_0 + 0.102752788299661*G0_2_1 + 0.352295274170271*G0_2_2;
    A[3125] = 0.371867233846396*G0_0_0 + 0.185933616923198*G0_0_1 + 0.185933616923197*G0_0_2 + 0.185933616923196*G0_1_0 + 0.31641334809737*G0_1_2 + 0.185933616923199*G0_2_0 + 0.316413348097374*G0_2_1 - 0.150051690850302*G0_2_2;
    A[3126] = 0.045667905910961*G0_0_0 + 0.0717638521457965*G0_0_1 + 0.120693751336112*G0_0_2 + 0.0717638521457953*G0_1_0 + 0.0456679059109593*G0_1_1 + 0.120693751336113*G0_1_2 + 0.120693751336111*G0_2_0 + 0.120693751336112*G0_2_1 + 0.195719596761263*G0_2_2;
    A[3127] = 0.00978597983806193*G0_0_0 + 0.00489298991903021*G0_0_1 + 0.00489298991903005*G0_0_2 + 0.00489298991903127*G0_1_0 - 0.0391439193522524*G0_1_1 + 0.053822889109348*G0_1_2 + 0.00489298991902984*G0_2_0 + 0.0538228891093468*G0_2_1 + 0.146789697570947*G0_2_2;
    A[3128] = 0.0456679059109618*G0_0_0 - 0.0260959462348341*G0_0_1 - 0.0750258454251516*G0_0_2 - 0.0260959462348354*G0_1_0 - 0.0521918924696701*G0_1_1 - 0.0260959462348354*G0_1_2 - 0.0750258454251482*G0_2_0 - 0.0260959462348334*G0_2_1;
    A[3129] = -0.039143919352252*G0_0_0 + 0.0048929899190316*G0_0_1 + 0.0538228891093459*G0_0_2 + 0.0048929899190322*G0_1_0 + 0.00978597983806405*G0_1_1 + 0.00489298991903133*G0_1_2 + 0.0538228891093488*G0_2_0 + 0.00489298991903083*G0_2_1 + 0.146789697570945*G0_2_2;
    A[3130] = -0.0391439193522517*G0_0_0 - 0.044036909271283*G0_0_1 - 0.0929668084615989*G0_0_2 - 0.0440369092712835*G0_1_0 - 0.0391439193522525*G0_1_1 - 0.0929668084616003*G0_1_2 - 0.0929668084616*G0_2_0 - 0.0929668084615993*G0_2_1;
    A[3131] = -0.0521918924696704*G0_0_0 - 0.0260959462348356*G0_0_1 - 0.026095946234836*G0_0_2 - 0.0260959462348359*G0_1_0 + 0.0456679059109594*G0_1_1 - 0.0750258454251511*G0_1_2 - 0.026095946234834*G0_2_0 - 0.0750258454251504*G0_2_1;
    A[3132] = -0.293579395141895*G0_0_0 - 0.342509294332214*G0_0_1 - 0.587158790283788*G0_0_2 - 0.342509294332208*G0_1_0 - 0.293579395141896*G0_1_1 - 0.587158790283787*G0_1_2 - 0.587158790283783*G0_2_0 - 0.587158790283791*G0_2_1 - 1.17431758056757*G0_2_2;
    A[3133] = -0.293579395141894*G0_0_0 + 0.0489298991903177*G0_0_1 + 0.293579395141894*G0_0_2 + 0.0489298991903152*G0_1_0 + 0.0978597983806315*G0_1_1 + 0.0489298991903165*G0_1_2 + 0.293579395141889*G0_2_0 + 0.0489298991903127*G0_2_1 - 0.293579395141896*G0_2_2;
    A[3134] = 0.0978597983806336*G0_0_0 + 0.0489298991903187*G0_0_1 + 0.0489298991903162*G0_0_2 + 0.0489298991903156*G0_1_0 - 0.293579395141891*G0_1_1 + 0.293579395141892*G0_1_2 + 0.0489298991903169*G0_2_0 + 0.293579395141897*G0_2_1 - 0.293579395141892*G0_2_2;
    A[3135] = 1.5657567740901*G0_0_0 + 0.782878387045047*G0_0_1 + 0.78287838704505*G0_0_2 + 0.782878387045047*G0_1_0 + 1.56575677409009*G0_1_1 + 0.782878387045048*G0_1_2 + 0.78287838704505*G0_2_0 + 0.782878387045048*G0_2_1 + 1.76147637085136*G0_2_2;
  }

  /// Tabulate the tensor for the contribution from a local cell
  /// using the specified reference cell quadrature points/weights
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c,
                               unsigned int num_quadrature_points,
                               const double * const * quadrature_points,
                               const double* quadrature_weights) const
  {
    throw std::runtime_error("Quadrature version of tabulate_tensor not available when using the FFC tensor representation.");
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_cell_integral_1_0: public ufc::cell_integral
{
public:

  /// Constructor
  poisson3d_5_cell_integral_1_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_cell_integral_1_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Number of operations (multiply-add pairs) for Jacobian data:      18
    // Number of operations (multiply-add pairs) for geometry tensor:    56
    // Number of operations (multiply-add pairs) for tensor contraction: 3108
    // Total number of operations (multiply-add pairs):                  3182
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_20 = J_01*J_12 - J_02*J_11;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensor
    const double G0_0 = det*w[0][0]*(1.0);
    const double G0_1 = det*w[0][1]*(1.0);
    const double G0_2 = det*w[0][2]*(1.0);
    const double G0_3 = det*w[0][3]*(1.0);
    const double G0_4 = det*w[0][4]*(1.0);
    const double G0_5 = det*w[0][5]*(1.0);
    const double G0_6 = det*w[0][6]*(1.0);
    const double G0_7 = det*w[0][7]*(1.0);
    const double G0_8 = det*w[0][8]*(1.0);
    const double G0_9 = det*w[0][9]*(1.0);
    const double G0_10 = det*w[0][10]*(1.0);
    const double G0_11 = det*w[0][11]*(1.0);
    const double G0_12 = det*w[0][12]*(1.0);
    const double G0_13 = det*w[0][13]*(1.0);
    const double G0_14 = det*w[0][14]*(1.0);
    const double G0_15 = det*w[0][15]*(1.0);
    const double G0_16 = det*w[0][16]*(1.0);
    const double G0_17 = det*w[0][17]*(1.0);
    const double G0_18 = det*w[0][18]*(1.0);
    const double G0_19 = det*w[0][19]*(1.0);
    const double G0_20 = det*w[0][20]*(1.0);
    const double G0_21 = det*w[0][21]*(1.0);
    const double G0_22 = det*w[0][22]*(1.0);
    const double G0_23 = det*w[0][23]*(1.0);
    const double G0_24 = det*w[0][24]*(1.0);
    const double G0_25 = det*w[0][25]*(1.0);
    const double G0_26 = det*w[0][26]*(1.0);
    const double G0_27 = det*w[0][27]*(1.0);
    const double G0_28 = det*w[0][28]*(1.0);
    const double G0_29 = det*w[0][29]*(1.0);
    const double G0_30 = det*w[0][30]*(1.0);
    const double G0_31 = det*w[0][31]*(1.0);
    const double G0_32 = det*w[0][32]*(1.0);
    const double G0_33 = det*w[0][33]*(1.0);
    const double G0_34 = det*w[0][34]*(1.0);
    const double G0_35 = det*w[0][35]*(1.0);
    const double G0_36 = det*w[0][36]*(1.0);
    const double G0_37 = det*w[0][37]*(1.0);
    const double G0_38 = det*w[0][38]*(1.0);
    const double G0_39 = det*w[0][39]*(1.0);
    const double G0_40 = det*w[0][40]*(1.0);
    const double G0_41 = det*w[0][41]*(1.0);
    const double G0_42 = det*w[0][42]*(1.0);
    const double G0_43 = det*w[0][43]*(1.0);
    const double G0_44 = det*w[0][44]*(1.0);
    const double G0_45 = det*w[0][45]*(1.0);
    const double G0_46 = det*w[0][46]*(1.0);
    const double G0_47 = det*w[0][47]*(1.0);
    const double G0_48 = det*w[0][48]*(1.0);
    const double G0_49 = det*w[0][49]*(1.0);
    const double G0_50 = det*w[0][50]*(1.0);
    const double G0_51 = det*w[0][51]*(1.0);
    const double G0_52 = det*w[0][52]*(1.0);
    const double G0_53 = det*w[0][53]*(1.0);
    const double G0_54 = det*w[0][54]*(1.0);
    const double G0_55 = det*w[0][55]*(1.0);
    
    // Compute element tensor
    A[0] = 8.34957866207904e-05*G0_0 + 9.2785140528199e-06*G0_1 + 9.27851405281973e-06*G0_2 + 9.2785140528198e-06*G0_3 + 4.0749823735939e-06*G0_4 + 1.52962392545732e-05*G0_5 + 1.52962392545729e-05*G0_6 + 4.07498237359409e-06*G0_7 + 4.07498237359445e-06*G0_8 + 1.52962392545733e-05*G0_9 + 1.52962392545736e-05*G0_10 + 4.07498237359392e-06*G0_11 + 4.07498237359457e-06*G0_12 + 1.52962392545724e-05*G0_13 + 1.52962392545734e-05*G0_14 + 4.07498237359359e-06*G0_15 - 6.04020786312488e-05*G0_16 + 6.04020786312486e-05*G0_17 - 4.43831824040184e-05*G0_18 + 1.81868671452018e-05*G0_19 - 6.04020786312472e-05*G0_20 + 6.04020786312475e-05*G0_21 - 4.43831824040179e-05*G0_22 + 1.81868671452017e-05*G0_23 - 6.04020786312478e-05*G0_24 + 6.04020786312488e-05*G0_25 - 4.43831824040187e-05*G0_26 + 1.81868671452023e-05*G0_27 + 0.00017343767343768*G0_28 - 1.77151327324937e-05*G0_29 + 0.000173437673437679*G0_30 - 1.77151327324933e-05*G0_31 - 1.77151327324942e-05*G0_32 + 0.00017343767343768*G0_33 + 0.000149650214754385*G0_34 - 0.000121496703527958*G0_35 + 9.59527869250099e-05*G0_36 - 0.000121496703527959*G0_37 + 3.98465025201167e-05*G0_38 + 9.5952786925011e-05*G0_39 + 0.000149650214754386*G0_40 - 0.000121496703527958*G0_41 + 9.5952786925011e-05*G0_42 - 0.00012149670352796*G0_43 + 3.98465025201167e-05*G0_44 + 9.59527869250114e-05*G0_45 + 0.000149650214754386*G0_46 - 0.000121496703527958*G0_47 + 9.59527869250114e-05*G0_48 - 0.000121496703527959*G0_49 + 3.98465025201162e-05*G0_50 + 9.59527869250111e-05*G0_51 - 0.000280029576904596*G0_52 + 0.000204250963799583*G0_53 + 0.000204250963799582*G0_54 + 0.000204250963799582*G0_55;
    A[1] = 9.2785140528199e-06*G0_0 + 8.34957866207913e-05*G0_1 + 9.2785140528203e-06*G0_2 + 9.27851405282034e-06*G0_3 + 4.07498237359331e-06*G0_4 + 1.52962392545738e-05*G0_5 + 1.52962392545736e-05*G0_6 + 4.07498237359368e-06*G0_7 - 6.04020786312486e-05*G0_8 + 6.0402078631249e-05*G0_9 - 4.43831824040178e-05*G0_10 + 1.81868671452013e-05*G0_11 - 6.04020786312486e-05*G0_12 + 6.04020786312484e-05*G0_13 - 4.43831824040176e-05*G0_14 + 1.81868671452009e-05*G0_15 + 4.07498237359255e-06*G0_16 + 1.52962392545745e-05*G0_17 + 1.52962392545733e-05*G0_18 + 4.07498237359381e-06*G0_19 + 4.0749823735939e-06*G0_20 + 1.52962392545734e-05*G0_21 + 1.52962392545739e-05*G0_22 + 4.07498237359351e-06*G0_23 + 1.81868671452014e-05*G0_24 - 4.43831824040174e-05*G0_25 + 6.04020786312487e-05*G0_26 - 6.04020786312485e-05*G0_27 + 0.000149650214754391*G0_28 - 0.00012149670352796*G0_29 + 9.59527869250157e-05*G0_30 - 0.00012149670352796*G0_31 + 3.98465025201152e-05*G0_32 + 9.59527869250161e-05*G0_33 + 0.000173437673437684*G0_34 - 1.77151327324949e-05*G0_35 + 0.000173437673437684*G0_36 - 1.77151327324945e-05*G0_37 - 1.77151327324955e-05*G0_38 + 0.000173437673437685*G0_39 + 9.59527869250152e-05*G0_40 - 0.000121496703527961*G0_41 + 0.000149650214754391*G0_42 + 3.9846502520115e-05*G0_43 - 0.00012149670352796*G0_44 + 9.59527869250161e-05*G0_45 + 9.59527869250157e-05*G0_46 - 0.000121496703527961*G0_47 + 0.000149650214754391*G0_48 + 3.98465025201159e-05*G0_49 - 0.00012149670352796*G0_50 + 9.59527869250157e-05*G0_51 + 0.000204250963799586*G0_52 - 0.000280029576904588*G0_53 + 0.000204250963799588*G0_54 + 0.000204250963799588*G0_55;
    A[2] = 9.27851405281973e-06*G0_0 + 9.2785140528203e-06*G0_1 + 8.3495786620791e-05*G0_2 + 9.27851405282e-06*G0_3 - 6.04020786312488e-05*G0_4 + 6.04020786312487e-05*G0_5 - 4.43831824040183e-05*G0_6 + 1.81868671452016e-05*G0_7 + 4.07498237359381e-06*G0_8 + 1.52962392545736e-05*G0_9 + 1.52962392545736e-05*G0_10 + 4.07498237359354e-06*G0_11 + 1.81868671452017e-05*G0_12 - 4.43831824040185e-05*G0_13 + 6.04020786312489e-05*G0_14 - 6.04020786312491e-05*G0_15 + 4.07498237359327e-06*G0_16 + 1.52962392545736e-05*G0_17 + 1.52962392545739e-05*G0_18 + 4.07498237359365e-06*G0_19 + 1.81868671452026e-05*G0_20 - 4.43831824040191e-05*G0_21 + 6.04020786312498e-05*G0_22 - 6.04020786312492e-05*G0_23 + 4.07498237359448e-06*G0_24 + 1.52962392545734e-05*G0_25 + 1.52962392545737e-05*G0_26 + 4.07498237359374e-06*G0_27 + 9.59527869250147e-05*G0_28 - 0.000121496703527961*G0_29 + 0.000149650214754389*G0_30 + 3.98465025201159e-05*G0_31 - 0.000121496703527959*G0_32 + 9.59527869250137e-05*G0_33 + 9.59527869250149e-05*G0_34 - 0.000121496703527961*G0_35 + 0.00014965021475439*G0_36 + 3.98465025201163e-05*G0_37 - 0.000121496703527959*G0_38 + 9.59527869250136e-05*G0_39 + 0.000173437673437683*G0_40 - 1.77151327324946e-05*G0_41 + 0.000173437673437683*G0_42 - 1.77151327324951e-05*G0_43 - 1.77151327324954e-05*G0_44 + 0.000173437673437682*G0_45 + 9.59527869250145e-05*G0_46 + 3.98465025201161e-05*G0_47 + 9.59527869250147e-05*G0_48 - 0.000121496703527961*G0_49 - 0.000121496703527961*G0_50 + 0.000149650214754389*G0_51 + 0.000204250963799587*G0_52 + 0.000204250963799586*G0_53 - 0.000280029576904594*G0_54 + 0.000204250963799585*G0_55;
    A[3] = 9.2785140528198e-06*G0_0 + 9.27851405282034e-06*G0_1 + 9.27851405282e-06*G0_2 + 8.34957866207912e-05*G0_3 + 1.81868671452016e-05*G0_4 - 4.43831824040183e-05*G0_5 + 6.04020786312487e-05*G0_6 - 6.04020786312486e-05*G0_7 + 1.81868671452012e-05*G0_8 - 4.43831824040176e-05*G0_9 + 6.04020786312484e-05*G0_10 - 6.04020786312485e-05*G0_11 + 4.07498237359358e-06*G0_12 + 1.52962392545736e-05*G0_13 + 1.52962392545733e-05*G0_14 + 4.07498237359378e-06*G0_15 + 1.81868671452007e-05*G0_16 - 4.43831824040175e-05*G0_17 + 6.04020786312487e-05*G0_18 - 6.04020786312487e-05*G0_19 + 4.07498237359433e-06*G0_20 + 1.52962392545733e-05*G0_21 + 1.52962392545738e-05*G0_22 + 4.07498237359383e-06*G0_23 + 4.07498237359435e-06*G0_24 + 1.52962392545734e-05*G0_25 + 1.52962392545738e-05*G0_26 + 4.07498237359363e-06*G0_27 + 9.59527869250142e-05*G0_28 + 3.98465025201157e-05*G0_29 + 9.59527869250139e-05*G0_30 - 0.00012149670352796*G0_31 - 0.000121496703527959*G0_32 + 0.00014965021475439*G0_33 + 9.59527869250149e-05*G0_34 + 3.98465025201158e-05*G0_35 + 9.59527869250141e-05*G0_36 - 0.00012149670352796*G0_37 - 0.000121496703527959*G0_38 + 0.00014965021475439*G0_39 + 9.59527869250139e-05*G0_40 + 3.98465025201157e-05*G0_41 + 9.59527869250138e-05*G0_42 - 0.000121496703527961*G0_43 - 0.00012149670352796*G0_44 + 0.000149650214754389*G0_45 + 0.000173437673437683*G0_46 - 1.7715132732495e-05*G0_47 + 0.000173437673437683*G0_48 - 1.77151327324941e-05*G0_49 - 1.77151327324947e-05*G0_50 + 0.000173437673437682*G0_51 + 0.000204250963799586*G0_52 + 0.000204250963799585*G0_53 + 0.000204250963799589*G0_54 - 0.000280029576904591*G0_55;
    A[4] = 4.0749823735939e-06*G0_0 + 4.07498237359332e-06*G0_1 - 6.04020786312488e-05*G0_2 + 1.81868671452016e-05*G0_3 + 0.000886258481744638*G0_4 - 0.000858657000150101*G0_5 + 0.000659424487549521*G0_6 - 0.000299350614020768*G0_7 + 5.50356814824436e-05*G0_8 - 0.0001784059401247*G0_9 + 9.04994032887594e-05*G0_10 - 0.000144029549411501*G0_11 - 0.000144029549411501*G0_12 + 0.0003573137253693*G0_13 - 0.000456929981669588*G0_14 + 0.000443129240872319*G0_15 + 5.5035681482444e-05*G0_16 - 0.000178405940124699*G0_17 + 9.04994032887588e-05*G0_18 - 0.000144029549411501*G0_19 - 0.000144029549411502*G0_20 + 0.000357313725369299*G0_21 - 0.000456929981669589*G0_22 + 0.00044312924087232*G0_23 + 2.75178407412211e-05*G0_24 - 2.24993895422146e-05*G0_25 - 2.2499389542215e-05*G0_26 + 2.75178407412217e-05*G0_27 - 4.01476095920563e-05*G0_28 - 5.52029631890768e-05*G0_29 + 0.000110405926378154*G0_30 - 9.28413471816291e-05*G0_31 + 0.000308634748738932*G0_32 - 0.000341254681532478*G0_33 - 4.01476095920562e-05*G0_34 - 5.52029631890763e-05*G0_35 + 0.000110405926378156*G0_36 - 9.28413471816297e-05*G0_37 + 0.00030863474873893*G0_38 - 0.000341254681532476*G0_39 - 0.000266814322080539*G0_40 + 0.000135498182373189*G0_41 - 0.00026681432208054*G0_42 - 8.78228959826152e-06*G0_43 - 8.78228959826022e-06*G0_44 - 0.000555375266023443*G0_45 - 2.00738047960283e-05*G0_46 + 2.88560943942895e-05*G0_47 - 2.00738047960283e-05*G0_48 - 2.76014815945366e-05*G0_49 - 2.7601481594538e-05*G0_50 + 5.52029631890767e-05*G0_51 - 0.000301107071940419*G0_52 - 0.000301107071940421*G0_53 + 0.000727675423856019*G0_54 - 0.000777859935846088*G0_55;
    A[5] = 1.52962392545732e-05*G0_0 + 1.52962392545738e-05*G0_1 + 6.04020786312487e-05*G0_2 - 4.43831824040183e-05*G0_3 - 0.000858657000150101*G0_4 + 0.00168419222238676*G0_5 - 0.00130178624102242*G0_6 + 0.000659424487549523*G0_7 - 0.000178405940124699*G0_8 + 0.000347778668091187*G0_9 - 0.000261963152588166*G0_10 + 0.000357313725369299*G0_11 + 9.049940328876e-05*G0_12 - 0.000261963152588167*G0_13 + 0.000328541271828326*G0_14 - 0.000456929981669589*G0_15 - 0.0001784059401247*G0_16 + 0.000347778668091186*G0_17 - 0.000261963152588165*G0_18 + 0.0003573137253693*G0_19 + 9.0499403288761e-05*G0_20 - 0.000261963152588167*G0_21 + 0.000328541271828327*G0_22 - 0.00045692998166959*G0_23 - 2.24993895422142e-05*G0_24 + 1.12078743444487e-05*G0_25 + 1.12078743444494e-05*G0_26 - 2.24993895422147e-05*G0_27 + 0.00012797050557468*G0_28 - 0.000120442828776169*G0_29 - 0.000110405926378155*G0_30 - 7.02583167860987e-05*G0_31 - 4.5166060791063e-05*G0_32 + 0.000491808217502689*G0_33 + 0.000127970505574679*G0_34 - 0.00012044282877617*G0_35 - 0.000110405926378156*G0_36 - 7.02583167860976e-05*G0_37 - 4.51660607910617e-05*G0_38 + 0.000491808217502688*G0_39 + 0.000170627340766238*G0_40 - 0.000122952054375671*G0_41 + 0.000170627340766239*G0_42 - 0.000148044310370707*G0_43 - 0.000148044310370709*G0_44 + 0.000873210508627223*G0_45 - 0.000109569517844987*G0_46 - 2.25830303955302e-05*G0_47 - 0.000109569517844987*G0_48 + 0.000253431785549852*G0_49 + 0.000253431785549854*G0_50 - 0.000623960765743206*G0_51 + 6.27306399875853e-05*G0_52 + 6.27306399875878e-05*G0_53 - 0.000903321215821266*G0_54 + 0.00110405926378155*G0_55;
    A[6] = 1.52962392545729e-05*G0_0 + 1.52962392545736e-05*G0_1 - 4.43831824040183e-05*G0_2 + 6.04020786312486e-05*G0_3 + 0.000659424487549521*G0_4 - 0.00130178624102242*G0_5 + 0.00168419222238675*G0_6 - 0.000858657000150101*G0_7 + 9.0499403288759e-05*G0_8 - 0.000261963152588165*G0_9 + 0.000328541271828325*G0_10 - 0.000456929981669588*G0_11 - 0.0001784059401247*G0_12 + 0.000347778668091186*G0_13 - 0.000261963152588167*G0_14 + 0.000357313725369299*G0_15 + 9.04994032887581e-05*G0_16 - 0.000261963152588164*G0_17 + 0.000328541271828325*G0_18 - 0.000456929981669588*G0_19 - 0.000178405940124699*G0_20 + 0.000347778668091186*G0_21 - 0.000261963152588165*G0_22 + 0.0003573137253693*G0_23 - 2.24993895422143e-05*G0_24 + 1.12078743444492e-05*G0_25 + 1.12078743444497e-05*G0_26 - 2.24993895422153e-05*G0_27 + 0.000127970505574679*G0_28 - 7.02583167860984e-05*G0_29 + 0.000491808217502688*G0_30 - 0.000120442828776168*G0_31 - 4.51660607910629e-05*G0_32 - 0.000110405926378153*G0_33 + 0.000127970505574681*G0_34 - 7.02583167860973e-05*G0_35 + 0.000491808217502689*G0_36 - 0.000120442828776168*G0_37 - 4.51660607910637e-05*G0_38 - 0.000110405926378151*G0_39 - 0.000109569517844986*G0_40 - 2.25830303955333e-05*G0_41 - 0.000109569517844987*G0_42 + 0.000253431785549853*G0_43 + 0.000253431785549856*G0_44 - 0.000623960765743205*G0_45 + 0.000170627340766238*G0_46 - 0.000122952054375673*G0_47 + 0.000170627340766239*G0_48 - 0.000148044310370704*G0_49 - 0.000148044310370707*G0_50 + 0.000873210508627221*G0_51 + 6.27306399875874e-05*G0_52 + 6.27306399875858e-05*G0_53 + 0.00110405926378155*G0_54 - 0.000903321215821261*G0_55;
    A[7] = 4.07498237359408e-06*G0_0 + 4.07498237359368e-06*G0_1 + 1.81868671452016e-05*G0_2 - 6.04020786312486e-05*G0_3 - 0.000299350614020768*G0_4 + 0.000659424487549523*G0_5 - 0.000858657000150101*G0_6 + 0.000886258481744638*G0_7 - 0.000144029549411501*G0_8 + 0.000357313725369299*G0_9 - 0.000456929981669589*G0_10 + 0.00044312924087232*G0_11 + 5.50356814824432e-05*G0_12 - 0.000178405940124699*G0_13 + 9.04994032887594e-05*G0_14 - 0.000144029549411501*G0_15 - 0.0001440295494115*G0_16 + 0.000357313725369298*G0_17 - 0.000456929981669589*G0_18 + 0.00044312924087232*G0_19 + 5.5035681482443e-05*G0_20 - 0.000178405940124699*G0_21 + 9.0499403288759e-05*G0_22 - 0.000144029549411502*G0_23 + 2.75178407412214e-05*G0_24 - 2.24993895422154e-05*G0_25 - 2.24993895422149e-05*G0_26 + 2.75178407412217e-05*G0_27 - 4.01476095920573e-05*G0_28 - 9.28413471816308e-05*G0_29 - 0.000341254681532477*G0_30 - 5.52029631890756e-05*G0_31 + 0.000308634748738932*G0_32 + 0.000110405926378153*G0_33 - 4.01476095920574e-05*G0_34 - 9.28413471816294e-05*G0_35 - 0.000341254681532477*G0_36 - 5.52029631890762e-05*G0_37 + 0.000308634748738932*G0_38 + 0.000110405926378152*G0_39 - 2.00738047960283e-05*G0_40 + 2.88560943942912e-05*G0_41 - 2.00738047960287e-05*G0_42 - 2.76014815945376e-05*G0_43 - 2.7601481594538e-05*G0_44 + 5.52029631890767e-05*G0_45 - 0.00026681432208054*G0_46 + 0.000135498182373191*G0_47 - 0.000266814322080541*G0_48 - 8.78228959826278e-06*G0_49 - 8.78228959826323e-06*G0_50 - 0.000555375266023443*G0_51 - 0.000301107071940419*G0_52 - 0.000301107071940424*G0_53 - 0.000777859935846087*G0_54 + 0.000727675423856019*G0_55;
    A[8] = 4.07498237359445e-06*G0_0 - 6.04020786312486e-05*G0_1 + 4.0749823735938e-06*G0_2 + 1.81868671452012e-05*G0_3 + 5.50356814824436e-05*G0_4 - 0.000178405940124699*G0_5 + 9.0499403288759e-05*G0_6 - 0.000144029549411501*G0_7 + 0.000886258481744638*G0_8 - 0.0008586570001501*G0_9 + 0.000659424487549521*G0_10 - 0.000299350614020768*G0_11 + 0.000443129240872319*G0_12 - 0.000456929981669588*G0_13 + 0.0003573137253693*G0_14 - 0.000144029549411501*G0_15 + 5.50356814824438e-05*G0_16 - 0.000178405940124699*G0_17 + 9.04994032887591e-05*G0_18 - 0.000144029549411501*G0_19 + 2.75178407412213e-05*G0_20 - 2.24993895422138e-05*G0_21 - 2.24993895422157e-05*G0_22 + 2.75178407412218e-05*G0_23 - 0.000144029549411502*G0_24 + 0.0003573137253693*G0_25 - 0.00045692998166959*G0_26 + 0.000443129240872321*G0_27 + 0.000110405926378155*G0_28 - 5.52029631890793e-05*G0_29 - 4.01476095920549e-05*G0_30 + 0.000308634748738932*G0_31 - 9.2841347181629e-05*G0_32 - 0.000341254681532476*G0_33 - 0.00026681432208054*G0_34 + 0.000135498182373188*G0_35 - 0.000266814322080539*G0_36 - 8.78228959826325e-06*G0_37 - 8.78228959826105e-06*G0_38 - 0.000555375266023444*G0_39 - 4.01476095920551e-05*G0_40 - 5.52029631890782e-05*G0_41 + 0.000110405926378158*G0_42 - 9.28413471816295e-05*G0_43 + 0.00030863474873893*G0_44 - 0.000341254681532478*G0_45 - 2.00738047960287e-05*G0_46 - 2.76014815945392e-05*G0_47 + 5.52029631890781e-05*G0_48 + 2.88560943942899e-05*G0_49 - 2.76014815945398e-05*G0_50 - 2.00738047960288e-05*G0_51 - 0.000301107071940421*G0_52 + 0.000727675423856015*G0_53 - 0.000301107071940421*G0_54 - 0.000777859935846086*G0_55;
    A[9] = 1.52962392545733e-05*G0_0 + 6.0402078631249e-05*G0_1 + 1.52962392545736e-05*G0_2 - 4.43831824040176e-05*G0_3 - 0.0001784059401247*G0_4 + 0.000347778668091187*G0_5 - 0.000261963152588165*G0_6 + 0.000357313725369299*G0_7 - 0.0008586570001501*G0_8 + 0.00168419222238676*G0_9 - 0.00130178624102242*G0_10 + 0.000659424487549522*G0_11 - 0.000456929981669588*G0_12 + 0.000328541271828326*G0_13 - 0.000261963152588166*G0_14 + 9.04994032887592e-05*G0_15 - 0.0001784059401247*G0_16 + 0.000347778668091186*G0_17 - 0.000261963152588165*G0_18 + 0.000357313725369299*G0_19 - 2.24993895422149e-05*G0_20 + 1.12078743444486e-05*G0_21 + 1.12078743444497e-05*G0_22 - 2.2499389542215e-05*G0_23 + 9.04994032887594e-05*G0_24 - 0.000261963152588166*G0_25 + 0.000328541271828327*G0_26 - 0.000456929981669591*G0_27 - 0.000110405926378152*G0_28 - 0.000120442828776167*G0_29 + 0.000127970505574681*G0_30 - 4.51660607910634e-05*G0_31 - 7.02583167860997e-05*G0_32 + 0.00049180821750269*G0_33 + 0.000170627340766241*G0_34 - 0.000122952054375672*G0_35 + 0.000170627340766241*G0_36 - 0.000148044310370706*G0_37 - 0.00014804431037071*G0_38 + 0.000873210508627222*G0_39 + 0.000127970505574682*G0_40 - 0.000120442828776171*G0_41 - 0.000110405926378152*G0_42 - 7.02583167860996e-05*G0_43 - 4.51660607910601e-05*G0_44 + 0.000491808217502691*G0_45 - 0.000109569517844983*G0_46 + 0.000253431785549854*G0_47 - 0.000623960765743204*G0_48 - 2.25830303955325e-05*G0_49 + 0.000253431785549854*G0_50 - 0.000109569517844983*G0_51 + 6.27306399875892e-05*G0_52 - 0.000903321215821255*G0_53 + 6.27306399875901e-05*G0_54 + 0.00110405926378155*G0_55;
    A[10] = 1.52962392545736e-05*G0_0 - 4.43831824040178e-05*G0_1 + 1.52962392545736e-05*G0_2 + 6.04020786312483e-05*G0_3 + 9.04994032887594e-05*G0_4 - 0.000261963152588166*G0_5 + 0.000328541271828325*G0_6 - 0.000456929981669589*G0_7 + 0.000659424487549521*G0_8 - 0.00130178624102242*G0_9 + 0.00168419222238676*G0_10 - 0.000858657000150102*G0_11 + 0.0003573137253693*G0_12 - 0.000261963152588166*G0_13 + 0.000347778668091187*G0_14 - 0.0001784059401247*G0_15 + 9.04994032887577e-05*G0_16 - 0.000261963152588164*G0_17 + 0.000328541271828325*G0_18 - 0.000456929981669589*G0_19 - 2.24993895422142e-05*G0_20 + 1.12078743444498e-05*G0_21 + 1.12078743444488e-05*G0_22 - 2.24993895422149e-05*G0_23 - 0.000178405940124699*G0_24 + 0.000347778668091187*G0_25 - 0.000261963152588167*G0_26 + 0.000357313725369301*G0_27 + 0.000491808217502691*G0_28 - 7.02583167861007e-05*G0_29 + 0.000127970505574682*G0_30 - 4.5166060791063e-05*G0_31 - 0.000120442828776169*G0_32 - 0.000110405926378151*G0_33 - 0.000109569517844985*G0_34 - 2.2583030395534e-05*G0_35 - 0.000109569517844986*G0_36 + 0.000253431785549852*G0_37 + 0.000253431785549856*G0_38 - 0.000623960765743205*G0_39 + 0.00012797050557468*G0_40 - 7.02583167860981e-05*G0_41 + 0.000491808217502689*G0_42 - 0.000120442828776171*G0_43 - 4.51660607910649e-05*G0_44 - 0.000110405926378153*G0_45 + 0.000170627340766239*G0_46 - 0.000148044310370709*G0_47 + 0.000873210508627224*G0_48 - 0.000122952054375673*G0_49 - 0.000148044310370707*G0_50 + 0.000170627340766239*G0_51 + 6.2730639987585e-05*G0_52 + 0.00110405926378155*G0_53 + 6.273063998759e-05*G0_54 - 0.000903321215821262*G0_55;
    A[11] = 4.07498237359392e-06*G0_0 + 1.81868671452013e-05*G0_1 + 4.07498237359354e-06*G0_2 - 6.04020786312485e-05*G0_3 - 0.000144029549411501*G0_4 + 0.000357313725369299*G0_5 - 0.000456929981669588*G0_6 + 0.00044312924087232*G0_7 - 0.000299350614020768*G0_8 + 0.000659424487549522*G0_9 - 0.000858657000150102*G0_10 + 0.000886258481744641*G0_11 - 0.000144029549411502*G0_12 + 9.04994032887601e-05*G0_13 - 0.000178405940124701*G0_14 + 5.50356814824441e-05*G0_15 - 0.0001440295494115*G0_16 + 0.000357313725369298*G0_17 - 0.000456929981669588*G0_18 + 0.00044312924087232*G0_19 + 2.75178407412211e-05*G0_20 - 2.24993895422147e-05*G0_21 - 2.24993895422151e-05*G0_22 + 2.75178407412221e-05*G0_23 + 5.50356814824433e-05*G0_24 - 0.0001784059401247*G0_25 + 9.04994032887604e-05*G0_26 - 0.000144029549411502*G0_27 - 0.00034125468153248*G0_28 - 9.28413471816286e-05*G0_29 - 4.01476095920568e-05*G0_30 + 0.000308634748738933*G0_31 - 5.52029631890768e-05*G0_32 + 0.000110405926378153*G0_33 - 2.00738047960279e-05*G0_34 + 2.88560943942919e-05*G0_35 - 2.00738047960282e-05*G0_36 - 2.7601481594539e-05*G0_37 - 2.76014815945393e-05*G0_38 + 5.52029631890756e-05*G0_39 - 4.01476095920556e-05*G0_40 - 9.28413471816294e-05*G0_41 - 0.000341254681532479*G0_42 - 5.52029631890765e-05*G0_43 + 0.000308634748738934*G0_44 + 0.000110405926378153*G0_45 - 0.000266814322080539*G0_46 - 8.78228959826122e-06*G0_47 - 0.000555375266023446*G0_48 + 0.000135498182373189*G0_49 - 8.78228959826246e-06*G0_50 - 0.00026681432208054*G0_51 - 0.00030110707194042*G0_52 - 0.00077785993584609*G0_53 - 0.000301107071940419*G0_54 + 0.000727675423856017*G0_55;
    A[12] = 4.07498237359457e-06*G0_0 - 6.04020786312486e-05*G0_1 + 1.81868671452017e-05*G0_2 + 4.07498237359358e-06*G0_3 - 0.000144029549411501*G0_4 + 9.049940328876e-05*G0_5 - 0.0001784059401247*G0_6 + 5.50356814824432e-05*G0_7 + 0.000443129240872319*G0_8 - 0.000456929981669589*G0_9 + 0.0003573137253693*G0_10 - 0.000144029549411502*G0_11 + 0.000886258481744636*G0_12 - 0.000858657000150099*G0_13 + 0.000659424487549522*G0_14 - 0.000299350614020768*G0_15 + 2.75178407412221e-05*G0_16 - 2.24993895422151e-05*G0_17 - 2.24993895422149e-05*G0_18 + 2.75178407412215e-05*G0_19 + 5.50356814824427e-05*G0_20 - 0.000178405940124698*G0_21 + 9.04994032887586e-05*G0_22 - 0.000144029549411501*G0_23 - 0.000144029549411502*G0_24 + 0.0003573137253693*G0_25 - 0.00045692998166959*G0_26 + 0.00044312924087232*G0_27 + 0.000110405926378156*G0_28 + 0.000308634748738929*G0_29 - 0.000341254681532474*G0_30 - 5.52029631890764e-05*G0_31 - 9.28413471816299e-05*G0_32 - 4.01476095920544e-05*G0_33 - 0.000266814322080539*G0_34 - 8.78228959826288e-06*G0_35 - 0.000555375266023442*G0_36 + 0.000135498182373187*G0_37 - 8.78228959826181e-06*G0_38 - 0.000266814322080539*G0_39 - 2.00738047960262e-05*G0_40 - 2.76014815945388e-05*G0_41 + 5.52029631890799e-05*G0_42 + 2.885609439429e-05*G0_43 - 2.76014815945398e-05*G0_44 - 2.00738047960272e-05*G0_45 - 4.01476095920553e-05*G0_46 - 5.52029631890784e-05*G0_47 + 0.000110405926378157*G0_48 - 9.28413471816303e-05*G0_49 + 0.000308634748738929*G0_50 - 0.000341254681532476*G0_51 - 0.000301107071940419*G0_52 + 0.000727675423856018*G0_53 - 0.000777859935846087*G0_54 - 0.000301107071940418*G0_55;
    A[13] = 1.52962392545724e-05*G0_0 + 6.04020786312484e-05*G0_1 - 4.43831824040185e-05*G0_2 + 1.52962392545736e-05*G0_3 + 0.0003573137253693*G0_4 - 0.000261963152588167*G0_5 + 0.000347778668091186*G0_6 - 0.000178405940124699*G0_7 - 0.000456929981669588*G0_8 + 0.000328541271828326*G0_9 - 0.000261963152588166*G0_10 + 9.04994032887601e-05*G0_11 - 0.000858657000150099*G0_12 + 0.00168419222238675*G0_13 - 0.00130178624102242*G0_14 + 0.000659424487549521*G0_15 - 2.2499389542215e-05*G0_16 + 1.12078743444496e-05*G0_17 + 1.12078743444486e-05*G0_18 - 2.24993895422146e-05*G0_19 - 0.000178405940124699*G0_20 + 0.000347778668091184*G0_21 - 0.000261963152588165*G0_22 + 0.000357313725369299*G0_23 + 9.04994032887607e-05*G0_24 - 0.000261963152588166*G0_25 + 0.000328541271828327*G0_26 - 0.000456929981669589*G0_27 - 0.000110405926378156*G0_28 - 4.51660607910593e-05*G0_29 + 0.000491808217502684*G0_30 - 0.000120442828776169*G0_31 - 7.02583167860986e-05*G0_32 + 0.000127970505574677*G0_33 + 0.000170627340766237*G0_34 - 0.000148044310370705*G0_35 + 0.000873210508627219*G0_36 - 0.000122952054375669*G0_37 - 0.000148044310370708*G0_38 + 0.000170627340766238*G0_39 - 0.000109569517844989*G0_40 + 0.000253431785549855*G0_41 - 0.000623960765743207*G0_42 - 2.25830303955306e-05*G0_43 + 0.000253431785549855*G0_44 - 0.000109569517844988*G0_45 + 0.000127970505574678*G0_46 - 0.000120442828776168*G0_47 - 0.000110405926378156*G0_48 - 7.02583167860969e-05*G0_49 - 4.51660607910599e-05*G0_50 + 0.000491808217502686*G0_51 + 6.27306399875872e-05*G0_52 - 0.000903321215821263*G0_53 + 0.00110405926378155*G0_54 + 6.27306399875829e-05*G0_55;
    A[14] = 1.52962392545734e-05*G0_0 - 4.43831824040176e-05*G0_1 + 6.04020786312489e-05*G0_2 + 1.52962392545733e-05*G0_3 - 0.000456929981669588*G0_4 + 0.000328541271828326*G0_5 - 0.000261963152588167*G0_6 + 9.04994032887594e-05*G0_7 + 0.0003573137253693*G0_8 - 0.000261963152588166*G0_9 + 0.000347778668091187*G0_10 - 0.000178405940124701*G0_11 + 0.000659424487549522*G0_12 - 0.00130178624102242*G0_13 + 0.00168419222238676*G0_14 - 0.0008586570001501*G0_15 - 2.24993895422156e-05*G0_16 + 1.12078743444489e-05*G0_17 + 1.12078743444498e-05*G0_18 - 2.24993895422152e-05*G0_19 + 9.04994032887612e-05*G0_20 - 0.000261963152588165*G0_21 + 0.000328541271828325*G0_22 - 0.000456929981669589*G0_23 - 0.000178405940124699*G0_24 + 0.000347778668091187*G0_25 - 0.000261963152588167*G0_26 + 0.000357313725369301*G0_27 + 0.000491808217502689*G0_28 - 4.51660607910659e-05*G0_29 - 0.000110405926378152*G0_30 - 7.02583167860986e-05*G0_31 - 0.000120442828776168*G0_32 + 0.000127970505574679*G0_33 - 0.000109569517844986*G0_34 + 0.000253431785549852*G0_35 - 0.000623960765743207*G0_36 - 2.25830303955324e-05*G0_37 + 0.000253431785549856*G0_38 - 0.000109569517844988*G0_39 + 0.000170627340766238*G0_40 - 0.000148044310370706*G0_41 + 0.000873210508627223*G0_42 - 0.000122952054375673*G0_43 - 0.00014804431037071*G0_44 + 0.000170627340766238*G0_45 + 0.000127970505574678*G0_46 - 7.02583167860976e-05*G0_47 + 0.000491808217502689*G0_48 - 0.000120442828776169*G0_49 - 4.5166060791066e-05*G0_50 - 0.000110405926378154*G0_51 + 6.27306399875887e-05*G0_52 + 0.00110405926378155*G0_53 - 0.000903321215821268*G0_54 + 6.27306399875863e-05*G0_55;
    A[15] = 4.07498237359359e-06*G0_0 + 1.81868671452009e-05*G0_1 - 6.04020786312491e-05*G0_2 + 4.07498237359378e-06*G0_3 + 0.000443129240872319*G0_4 - 0.000456929981669589*G0_5 + 0.000357313725369299*G0_6 - 0.000144029549411501*G0_7 - 0.000144029549411501*G0_8 + 9.04994032887592e-05*G0_9 - 0.0001784059401247*G0_10 + 5.50356814824441e-05*G0_11 - 0.000299350614020768*G0_12 + 0.000659424487549521*G0_13 - 0.0008586570001501*G0_14 + 0.000886258481744638*G0_15 + 2.75178407412229e-05*G0_16 - 2.24993895422156e-05*G0_17 - 2.24993895422152e-05*G0_18 + 2.75178407412219e-05*G0_19 - 0.000144029549411501*G0_20 + 0.000357313725369298*G0_21 - 0.000456929981669588*G0_22 + 0.000443129240872319*G0_23 + 5.50356814824433e-05*G0_24 - 0.000178405940124699*G0_25 + 9.04994032887599e-05*G0_26 - 0.000144029549411502*G0_27 - 0.000341254681532479*G0_28 + 0.000308634748738934*G0_29 + 0.000110405926378152*G0_30 - 9.28413471816292e-05*G0_31 - 5.52029631890784e-05*G0_32 - 4.01476095920571e-05*G0_33 - 2.00738047960295e-05*G0_34 - 2.7601481594536e-05*G0_35 + 5.52029631890753e-05*G0_36 + 2.8856094394291e-05*G0_37 - 2.76014815945404e-05*G0_38 - 2.00738047960282e-05*G0_39 - 0.000266814322080539*G0_40 - 8.78228959826289e-06*G0_41 - 0.000555375266023444*G0_42 + 0.00013549818237319*G0_43 - 8.78228959826019e-06*G0_44 - 0.00026681432208054*G0_45 - 4.01476095920561e-05*G0_46 - 9.28413471816306e-05*G0_47 - 0.000341254681532478*G0_48 - 5.52029631890756e-05*G0_49 + 0.000308634748738934*G0_50 + 0.000110405926378151*G0_51 - 0.000301107071940422*G0_52 - 0.000777859935846088*G0_53 + 0.000727675423856016*G0_54 - 0.000301107071940423*G0_55;
    A[16] = -6.04020786312488e-05*G0_0 + 4.07498237359256e-06*G0_1 + 4.07498237359327e-06*G0_2 + 1.81868671452007e-05*G0_3 + 5.5035681482444e-05*G0_4 - 0.0001784059401247*G0_5 + 9.04994032887582e-05*G0_6 - 0.0001440295494115*G0_7 + 5.50356814824438e-05*G0_8 - 0.0001784059401247*G0_9 + 9.04994032887577e-05*G0_10 - 0.0001440295494115*G0_11 + 2.75178407412221e-05*G0_12 - 2.2499389542215e-05*G0_13 - 2.24993895422156e-05*G0_14 + 2.75178407412229e-05*G0_15 + 0.00088625848174464*G0_16 - 0.0008586570001501*G0_17 + 0.000659424487549521*G0_18 - 0.000299350614020768*G0_19 + 0.000443129240872319*G0_20 - 0.00045692998166959*G0_21 + 0.000357313725369299*G0_22 - 0.000144029549411501*G0_23 + 0.00044312924087232*G0_24 - 0.00045692998166959*G0_25 + 0.000357313725369299*G0_26 - 0.000144029549411501*G0_27 - 0.000266814322080546*G0_28 + 0.000135498182373191*G0_29 - 0.000266814322080546*G0_30 - 8.78228959826185e-06*G0_31 - 8.78228959826027e-06*G0_32 - 0.000555375266023447*G0_33 + 0.000110405926378151*G0_34 - 5.52029631890765e-05*G0_35 - 4.01476095920605e-05*G0_36 + 0.000308634748738932*G0_37 - 9.28413471816288e-05*G0_38 - 0.00034125468153248*G0_39 + 0.00011040592637815*G0_40 - 5.52029631890763e-05*G0_41 - 4.01476095920619e-05*G0_42 + 0.000308634748738932*G0_43 - 9.28413471816291e-05*G0_44 - 0.00034125468153248*G0_45 + 5.52029631890721e-05*G0_46 - 2.76014815945376e-05*G0_47 - 2.00738047960352e-05*G0_48 - 2.76014815945368e-05*G0_49 + 2.88560943942926e-05*G0_50 - 2.00738047960351e-05*G0_51 + 0.000727675423856009*G0_52 - 0.000301107071940431*G0_53 - 0.00030110707194043*G0_54 - 0.000777859935846094*G0_55;
    A[17] = 6.04020786312486e-05*G0_0 + 1.52962392545745e-05*G0_1 + 1.52962392545736e-05*G0_2 - 4.43831824040175e-05*G0_3 - 0.000178405940124699*G0_4 + 0.000347778668091186*G0_5 - 0.000261963152588164*G0_6 + 0.000357313725369298*G0_7 - 0.000178405940124699*G0_8 + 0.000347778668091186*G0_9 - 0.000261963152588164*G0_10 + 0.000357313725369298*G0_11 - 2.24993895422151e-05*G0_12 + 1.12078743444496e-05*G0_13 + 1.12078743444489e-05*G0_14 - 2.24993895422156e-05*G0_15 - 0.0008586570001501*G0_16 + 0.00168419222238675*G0_17 - 0.00130178624102242*G0_18 + 0.000659424487549522*G0_19 - 0.000456929981669589*G0_20 + 0.000328541271828327*G0_21 - 0.000261963152588166*G0_22 + 9.04994032887597e-05*G0_23 - 0.000456929981669589*G0_24 + 0.000328541271828327*G0_25 - 0.000261963152588166*G0_26 + 9.04994032887597e-05*G0_27 + 0.000170627340766243*G0_28 - 0.000122952054375673*G0_29 + 0.000170627340766244*G0_30 - 0.000148044310370705*G0_31 - 0.000148044310370707*G0_32 + 0.000873210508627223*G0_33 - 0.00011040592637815*G0_34 - 0.000120442828776169*G0_35 + 0.000127970505574682*G0_36 - 4.51660607910641e-05*G0_37 - 7.02583167860977e-05*G0_38 + 0.000491808217502688*G0_39 - 0.00011040592637815*G0_40 - 0.000120442828776169*G0_41 + 0.000127970505574682*G0_42 - 4.51660607910638e-05*G0_43 - 7.02583167860963e-05*G0_44 + 0.000491808217502688*G0_45 - 0.0006239607657432*G0_46 + 0.000253431785549854*G0_47 - 0.000109569517844981*G0_48 + 0.000253431785549852*G0_49 - 2.25830303955339e-05*G0_50 - 0.00010956951784498*G0_51 - 0.000903321215821257*G0_52 + 6.27306399875955e-05*G0_53 + 6.27306399875993e-05*G0_54 + 0.00110405926378155*G0_55;
    A[18] = -4.43831824040184e-05*G0_0 + 1.52962392545733e-05*G0_1 + 1.52962392545739e-05*G0_2 + 6.04020786312486e-05*G0_3 + 9.04994032887588e-05*G0_4 - 0.000261963152588165*G0_5 + 0.000328541271828324*G0_6 - 0.000456929981669589*G0_7 + 9.04994032887591e-05*G0_8 - 0.000261963152588165*G0_9 + 0.000328541271828325*G0_10 - 0.000456929981669588*G0_11 - 2.24993895422149e-05*G0_12 + 1.12078743444486e-05*G0_13 + 1.12078743444498e-05*G0_14 - 2.24993895422152e-05*G0_15 + 0.000659424487549521*G0_16 - 0.00130178624102242*G0_17 + 0.00168419222238676*G0_18 - 0.000858657000150101*G0_19 + 0.000357313725369301*G0_20 - 0.000261963152588167*G0_21 + 0.000347778668091188*G0_22 - 0.0001784059401247*G0_23 + 0.000357313725369301*G0_24 - 0.000261963152588166*G0_25 + 0.000347778668091187*G0_26 - 0.0001784059401247*G0_27 - 0.000109569517844986*G0_28 - 2.25830303955327e-05*G0_29 - 0.000109569517844986*G0_30 + 0.000253431785549853*G0_31 + 0.000253431785549854*G0_32 - 0.000623960765743203*G0_33 + 0.00049180821750269*G0_34 - 7.02583167861004e-05*G0_35 + 0.00012797050557468*G0_36 - 4.51660607910645e-05*G0_37 - 0.000120442828776169*G0_38 - 0.00011040592637815*G0_39 + 0.000491808217502689*G0_40 - 7.02583167860983e-05*G0_41 + 0.00012797050557468*G0_42 - 4.51660607910671e-05*G0_43 - 0.00012044282877617*G0_44 - 0.00011040592637815*G0_45 + 0.000873210508627222*G0_46 - 0.000148044310370707*G0_47 + 0.000170627340766238*G0_48 - 0.000148044310370707*G0_49 - 0.000122952054375671*G0_50 + 0.000170627340766238*G0_51 + 0.00110405926378154*G0_52 + 6.27306399875891e-05*G0_53 + 6.27306399875855e-05*G0_54 - 0.000903321215821261*G0_55;
    A[19] = 1.81868671452018e-05*G0_0 + 4.07498237359381e-06*G0_1 + 4.07498237359365e-06*G0_2 - 6.04020786312486e-05*G0_3 - 0.000144029549411501*G0_4 + 0.0003573137253693*G0_5 - 0.000456929981669588*G0_6 + 0.00044312924087232*G0_7 - 0.000144029549411501*G0_8 + 0.000357313725369299*G0_9 - 0.000456929981669589*G0_10 + 0.00044312924087232*G0_11 + 2.75178407412215e-05*G0_12 - 2.24993895422146e-05*G0_13 - 2.24993895422152e-05*G0_14 + 2.75178407412219e-05*G0_15 - 0.000299350614020768*G0_16 + 0.000659424487549522*G0_17 - 0.000858657000150101*G0_18 + 0.00088625848174464*G0_19 - 0.000144029549411503*G0_20 + 9.04994032887604e-05*G0_21 - 0.0001784059401247*G0_22 + 5.50356814824438e-05*G0_23 - 0.000144029549411502*G0_24 + 9.04994032887598e-05*G0_25 - 0.0001784059401247*G0_26 + 5.50356814824436e-05*G0_27 - 2.00738047960277e-05*G0_28 + 2.88560943942906e-05*G0_29 - 2.00738047960278e-05*G0_30 - 2.76014815945383e-05*G0_31 - 2.76014815945383e-05*G0_32 + 5.52029631890772e-05*G0_33 - 0.000341254681532478*G0_34 - 9.28413471816288e-05*G0_35 - 4.01476095920564e-05*G0_36 + 0.000308634748738931*G0_37 - 5.52029631890764e-05*G0_38 + 0.000110405926378153*G0_39 - 0.000341254681532477*G0_40 - 9.28413471816299e-05*G0_41 - 4.01476095920557e-05*G0_42 + 0.000308634748738933*G0_43 - 5.52029631890761e-05*G0_44 + 0.000110405926378154*G0_45 - 0.000555375266023444*G0_46 - 8.78228959826221e-06*G0_47 - 0.000266814322080539*G0_48 - 8.78228959826275e-06*G0_49 + 0.000135498182373189*G0_50 - 0.000266814322080539*G0_51 - 0.000777859935846088*G0_52 - 0.000301107071940421*G0_53 - 0.000301107071940419*G0_54 + 0.000727675423856022*G0_55;
    A[20] = -6.04020786312472e-05*G0_0 + 4.0749823735939e-06*G0_1 + 1.81868671452026e-05*G0_2 + 4.07498237359432e-06*G0_3 - 0.000144029549411502*G0_4 + 9.0499403288761e-05*G0_5 - 0.000178405940124699*G0_6 + 5.5035681482443e-05*G0_7 + 2.75178407412213e-05*G0_8 - 2.24993895422149e-05*G0_9 - 2.24993895422142e-05*G0_10 + 2.75178407412211e-05*G0_11 + 5.50356814824427e-05*G0_12 - 0.000178405940124699*G0_13 + 9.04994032887612e-05*G0_14 - 0.000144029549411501*G0_15 + 0.000443129240872319*G0_16 - 0.000456929981669589*G0_17 + 0.0003573137253693*G0_18 - 0.000144029549411503*G0_19 + 0.000886258481744637*G0_20 - 0.000858657000150097*G0_21 + 0.000659424487549522*G0_22 - 0.000299350614020769*G0_23 + 0.000443129240872318*G0_24 - 0.000456929981669588*G0_25 + 0.0003573137253693*G0_26 - 0.000144029549411502*G0_27 - 0.000266814322080534*G0_28 - 8.78228959826556e-06*G0_29 - 0.000555375266023436*G0_30 + 0.000135498182373188*G0_31 - 8.78228959826404e-06*G0_32 - 0.000266814322080533*G0_33 + 0.000110405926378162*G0_34 + 0.000308634748738928*G0_35 - 0.000341254681532469*G0_36 - 5.52029631890795e-05*G0_37 - 9.28413471816314e-05*G0_38 - 4.01476095920501e-05*G0_39 + 5.52029631890831e-05*G0_40 - 2.76014815945398e-05*G0_41 - 2.00738047960217e-05*G0_42 - 2.7601481594541e-05*G0_43 + 2.88560943942886e-05*G0_44 - 2.00738047960213e-05*G0_45 + 0.000110405926378161*G0_46 - 5.52029631890786e-05*G0_47 - 4.0147609592051e-05*G0_48 + 0.000308634748738928*G0_49 - 9.28413471816312e-05*G0_50 - 0.000341254681532471*G0_51 + 0.000727675423856024*G0_52 - 0.000301107071940411*G0_53 - 0.000777859935846082*G0_54 - 0.000301107071940412*G0_55;
    A[21] = 6.04020786312475e-05*G0_0 + 1.52962392545734e-05*G0_1 - 4.43831824040191e-05*G0_2 + 1.52962392545733e-05*G0_3 + 0.000357313725369299*G0_4 - 0.000261963152588167*G0_5 + 0.000347778668091186*G0_6 - 0.000178405940124699*G0_7 - 2.24993895422138e-05*G0_8 + 1.12078743444486e-05*G0_9 + 1.12078743444497e-05*G0_10 - 2.24993895422147e-05*G0_11 - 0.000178405940124698*G0_12 + 0.000347778668091184*G0_13 - 0.000261963152588165*G0_14 + 0.000357313725369298*G0_15 - 0.00045692998166959*G0_16 + 0.000328541271828327*G0_17 - 0.000261963152588167*G0_18 + 9.04994032887604e-05*G0_19 - 0.000858657000150097*G0_20 + 0.00168419222238675*G0_21 - 0.00130178624102242*G0_22 + 0.000659424487549522*G0_23 - 0.000456929981669588*G0_24 + 0.000328541271828326*G0_25 - 0.000261963152588167*G0_26 + 9.04994032887608e-05*G0_27 + 0.000170627340766237*G0_28 - 0.000148044310370705*G0_29 + 0.000873210508627218*G0_30 - 0.000122952054375671*G0_31 - 0.000148044310370707*G0_32 + 0.000170627340766236*G0_33 - 0.000110405926378158*G0_34 - 4.516606079106e-05*G0_35 + 0.000491808217502684*G0_36 - 0.000120442828776167*G0_37 - 7.02583167860986e-05*G0_38 + 0.000127970505574677*G0_39 - 0.000623960765743207*G0_40 + 0.000253431785549854*G0_41 - 0.000109569517844986*G0_42 + 0.000253431785549855*G0_43 - 2.25830303955298e-05*G0_44 - 0.000109569517844989*G0_45 - 0.000110405926378158*G0_46 - 0.000120442828776168*G0_47 + 0.000127970505574679*G0_48 - 4.51660607910597e-05*G0_49 - 7.02583167860991e-05*G0_50 + 0.000491808217502685*G0_51 - 0.000903321215821266*G0_52 + 6.27306399875894e-05*G0_53 + 0.00110405926378154*G0_54 + 6.27306399875827e-05*G0_55;
    A[22] = -4.43831824040179e-05*G0_0 + 1.52962392545739e-05*G0_1 + 6.04020786312498e-05*G0_2 + 1.52962392545738e-05*G0_3 - 0.000456929981669589*G0_4 + 0.000328541271828327*G0_5 - 0.000261963152588165*G0_6 + 9.0499403288759e-05*G0_7 - 2.24993895422157e-05*G0_8 + 1.12078743444497e-05*G0_9 + 1.12078743444488e-05*G0_10 - 2.24993895422151e-05*G0_11 + 9.04994032887586e-05*G0_12 - 0.000261963152588165*G0_13 + 0.000328541271828325*G0_14 - 0.000456929981669588*G0_15 + 0.000357313725369299*G0_16 - 0.000261963152588166*G0_17 + 0.000347778668091188*G0_18 - 0.0001784059401247*G0_19 + 0.000659424487549522*G0_20 - 0.00130178624102242*G0_21 + 0.00168419222238675*G0_22 - 0.000858657000150101*G0_23 + 0.0003573137253693*G0_24 - 0.000261963152588166*G0_25 + 0.000347778668091187*G0_26 - 0.0001784059401247*G0_27 - 0.000109569517844984*G0_28 + 0.000253431785549852*G0_29 - 0.000623960765743202*G0_30 - 2.25830303955329e-05*G0_31 + 0.000253431785549854*G0_32 - 0.000109569517844983*G0_33 + 0.000491808217502693*G0_34 - 4.51660607910651e-05*G0_35 - 0.000110405926378149*G0_36 - 7.02583167860992e-05*G0_37 - 0.000120442828776169*G0_38 + 0.000127970505574683*G0_39 + 0.000873210508627224*G0_40 - 0.000148044310370707*G0_41 + 0.00017062734076624*G0_42 - 0.000148044310370709*G0_43 - 0.000122952054375674*G0_44 + 0.000170627340766242*G0_45 + 0.000491808217502692*G0_46 - 7.02583167860996e-05*G0_47 + 0.000127970505574681*G0_48 - 4.51660607910657e-05*G0_49 - 0.000120442828776169*G0_50 - 0.000110405926378149*G0_51 + 0.00110405926378155*G0_52 + 6.27306399875902e-05*G0_53 - 0.00090332121582126*G0_54 + 6.27306399875918e-05*G0_55;
    A[23] = 1.81868671452017e-05*G0_0 + 4.07498237359351e-06*G0_1 - 6.04020786312492e-05*G0_2 + 4.07498237359383e-06*G0_3 + 0.00044312924087232*G0_4 - 0.00045692998166959*G0_5 + 0.0003573137253693*G0_6 - 0.000144029549411502*G0_7 + 2.75178407412218e-05*G0_8 - 2.2499389542215e-05*G0_9 - 2.24993895422149e-05*G0_10 + 2.75178407412221e-05*G0_11 - 0.000144029549411501*G0_12 + 0.000357313725369299*G0_13 - 0.000456929981669589*G0_14 + 0.000443129240872319*G0_15 - 0.000144029549411501*G0_16 + 9.04994032887598e-05*G0_17 - 0.0001784059401247*G0_18 + 5.50356814824438e-05*G0_19 - 0.000299350614020769*G0_20 + 0.000659424487549522*G0_21 - 0.000858657000150101*G0_22 + 0.000886258481744639*G0_23 - 0.000144029549411502*G0_24 + 9.04994032887599e-05*G0_25 - 0.0001784059401247*G0_26 + 5.50356814824437e-05*G0_27 - 2.00738047960286e-05*G0_28 - 2.76014815945373e-05*G0_29 + 5.52029631890774e-05*G0_30 + 2.88560943942906e-05*G0_31 - 2.760148159454e-05*G0_32 - 2.0073804796028e-05*G0_33 - 0.000341254681532478*G0_34 + 0.000308634748738933*G0_35 + 0.000110405926378155*G0_36 - 9.28413471816292e-05*G0_37 - 5.5202963189079e-05*G0_38 - 4.01476095920557e-05*G0_39 - 0.000555375266023443*G0_40 - 8.78228959826299e-06*G0_41 - 0.000266814322080539*G0_42 - 8.78228959826103e-06*G0_43 + 0.00013549818237319*G0_44 - 0.00026681432208054*G0_45 - 0.000341254681532477*G0_46 - 9.28413471816305e-05*G0_47 - 4.01476095920556e-05*G0_48 + 0.000308634748738933*G0_49 - 5.52029631890771e-05*G0_50 + 0.000110405926378154*G0_51 - 0.000777859935846088*G0_52 - 0.000301107071940422*G0_53 + 0.000727675423856021*G0_54 - 0.000301107071940421*G0_55;
    A[24] = -6.04020786312478e-05*G0_0 + 1.81868671452014e-05*G0_1 + 4.07498237359448e-06*G0_2 + 4.07498237359436e-06*G0_3 + 2.75178407412211e-05*G0_4 - 2.24993895422142e-05*G0_5 - 2.24993895422143e-05*G0_6 + 2.75178407412214e-05*G0_7 - 0.000144029549411502*G0_8 + 9.04994032887594e-05*G0_9 - 0.000178405940124699*G0_10 + 5.50356814824433e-05*G0_11 - 0.000144029549411502*G0_12 + 9.04994032887607e-05*G0_13 - 0.000178405940124699*G0_14 + 5.50356814824433e-05*G0_15 + 0.00044312924087232*G0_16 - 0.000456929981669589*G0_17 + 0.000357313725369301*G0_18 - 0.000144029549411502*G0_19 + 0.000443129240872318*G0_20 - 0.000456929981669588*G0_21 + 0.0003573137253693*G0_22 - 0.000144029549411502*G0_23 + 0.000886258481744639*G0_24 - 0.0008586570001501*G0_25 + 0.000659424487549523*G0_26 - 0.00029935061402077*G0_27 - 0.00055537526602344*G0_28 - 8.78228959826337e-06*G0_29 - 0.000266814322080536*G0_30 - 8.78228959826413e-06*G0_31 + 0.000135498182373188*G0_32 - 0.000266814322080535*G0_33 + 5.52029631890819e-05*G0_34 - 2.76014815945394e-05*G0_35 - 2.00738047960227e-05*G0_36 - 2.76014815945396e-05*G0_37 + 2.88560943942885e-05*G0_38 - 2.00738047960227e-05*G0_39 + 0.000110405926378159*G0_40 + 0.000308634748738931*G0_41 - 0.000341254681532473*G0_42 - 5.52029631890787e-05*G0_43 - 9.28413471816315e-05*G0_44 - 4.01476095920503e-05*G0_45 + 0.000110405926378159*G0_46 + 0.000308634748738931*G0_47 - 0.000341254681532474*G0_48 - 5.52029631890787e-05*G0_49 - 9.28413471816304e-05*G0_50 - 4.01476095920518e-05*G0_51 + 0.000727675423856024*G0_52 - 0.000777859935846081*G0_53 - 0.000301107071940416*G0_54 - 0.000301107071940415*G0_55;
    A[25] = 6.04020786312488e-05*G0_0 - 4.43831824040174e-05*G0_1 + 1.52962392545734e-05*G0_2 + 1.52962392545734e-05*G0_3 - 2.24993895422146e-05*G0_4 + 1.12078743444487e-05*G0_5 + 1.12078743444492e-05*G0_6 - 2.24993895422154e-05*G0_7 + 0.0003573137253693*G0_8 - 0.000261963152588165*G0_9 + 0.000347778668091187*G0_10 - 0.0001784059401247*G0_11 + 0.0003573137253693*G0_12 - 0.000261963152588166*G0_13 + 0.000347778668091187*G0_14 - 0.0001784059401247*G0_15 - 0.00045692998166959*G0_16 + 0.000328541271828327*G0_17 - 0.000261963152588166*G0_18 + 9.04994032887598e-05*G0_19 - 0.000456929981669588*G0_20 + 0.000328541271828326*G0_21 - 0.000261963152588166*G0_22 + 9.04994032887599e-05*G0_23 - 0.0008586570001501*G0_24 + 0.00168419222238676*G0_25 - 0.00130178624102242*G0_26 + 0.000659424487549524*G0_27 + 0.000873210508627223*G0_28 - 0.000148044310370708*G0_29 + 0.00017062734076624*G0_30 - 0.000148044310370706*G0_31 - 0.000122952054375672*G0_32 + 0.000170627340766239*G0_33 - 0.000623960765743204*G0_34 + 0.000253431785549854*G0_35 - 0.000109569517844986*G0_36 + 0.000253431785549853*G0_37 - 2.2583030395532e-05*G0_38 - 0.000109569517844987*G0_39 - 0.000110405926378153*G0_40 - 4.51660607910628e-05*G0_41 + 0.000491808217502688*G0_42 - 0.000120442828776169*G0_43 - 7.02583167860984e-05*G0_44 + 0.000127970505574679*G0_45 - 0.000110405926378153*G0_46 - 4.51660607910644e-05*G0_47 + 0.00049180821750269*G0_48 - 0.000120442828776169*G0_49 - 7.02583167860998e-05*G0_50 + 0.000127970505574681*G0_51 - 0.00090332121582126*G0_52 + 0.00110405926378154*G0_53 + 6.27306399875901e-05*G0_54 + 6.2730639987589e-05*G0_55;
    A[26] = -4.43831824040187e-05*G0_0 + 6.04020786312487e-05*G0_1 + 1.52962392545737e-05*G0_2 + 1.52962392545738e-05*G0_3 - 2.2499389542215e-05*G0_4 + 1.12078743444494e-05*G0_5 + 1.12078743444497e-05*G0_6 - 2.24993895422149e-05*G0_7 - 0.00045692998166959*G0_8 + 0.000328541271828327*G0_9 - 0.000261963152588167*G0_10 + 9.04994032887605e-05*G0_11 - 0.00045692998166959*G0_12 + 0.000328541271828327*G0_13 - 0.000261963152588167*G0_14 + 9.04994032887599e-05*G0_15 + 0.000357313725369299*G0_16 - 0.000261963152588166*G0_17 + 0.000347778668091187*G0_18 - 0.0001784059401247*G0_19 + 0.0003573137253693*G0_20 - 0.000261963152588167*G0_21 + 0.000347778668091187*G0_22 - 0.0001784059401247*G0_23 + 0.000659424487549523*G0_24 - 0.00130178624102242*G0_25 + 0.00168419222238676*G0_26 - 0.000858657000150103*G0_27 - 0.000623960765743205*G0_28 + 0.000253431785549854*G0_29 - 0.000109569517844985*G0_30 + 0.000253431785549853*G0_31 - 2.2583030395532e-05*G0_32 - 0.000109569517844985*G0_33 + 0.000873210508627225*G0_34 - 0.000148044310370708*G0_35 + 0.000170627340766242*G0_36 - 0.000148044310370706*G0_37 - 0.000122952054375673*G0_38 + 0.000170627340766242*G0_39 + 0.00049180821750269*G0_40 - 4.5166060791064e-05*G0_41 - 0.000110405926378153*G0_42 - 7.02583167860989e-05*G0_43 - 0.000120442828776169*G0_44 + 0.000127970505574682*G0_45 + 0.00049180821750269*G0_46 - 4.51660607910631e-05*G0_47 - 0.000110405926378154*G0_48 - 7.02583167860976e-05*G0_49 - 0.000120442828776168*G0_50 + 0.00012797050557468*G0_51 + 0.00110405926378155*G0_52 - 0.00090332121582126*G0_53 + 6.27306399875906e-05*G0_54 + 6.27306399875909e-05*G0_55;
    A[27] = 1.81868671452023e-05*G0_0 - 6.04020786312485e-05*G0_1 + 4.07498237359374e-06*G0_2 + 4.07498237359363e-06*G0_3 + 2.75178407412217e-05*G0_4 - 2.24993895422147e-05*G0_5 - 2.24993895422153e-05*G0_6 + 2.75178407412217e-05*G0_7 + 0.000443129240872321*G0_8 - 0.000456929981669591*G0_9 + 0.000357313725369301*G0_10 - 0.000144029549411502*G0_11 + 0.00044312924087232*G0_12 - 0.000456929981669589*G0_13 + 0.000357313725369301*G0_14 - 0.000144029549411502*G0_15 - 0.000144029549411501*G0_16 + 9.04994032887597e-05*G0_17 - 0.0001784059401247*G0_18 + 5.50356814824436e-05*G0_19 - 0.000144029549411502*G0_20 + 9.04994032887608e-05*G0_21 - 0.0001784059401247*G0_22 + 5.50356814824437e-05*G0_23 - 0.00029935061402077*G0_24 + 0.000659424487549524*G0_25 - 0.000858657000150103*G0_26 + 0.000886258481744641*G0_27 + 5.52029631890783e-05*G0_28 - 2.76014815945397e-05*G0_29 - 2.00738047960272e-05*G0_30 - 2.76014815945378e-05*G0_31 + 2.88560943942907e-05*G0_32 - 2.00738047960271e-05*G0_33 - 0.000555375266023445*G0_34 - 8.78228959826198e-06*G0_35 - 0.000266814322080541*G0_36 - 8.78228959826389e-06*G0_37 + 0.00013549818237319*G0_38 - 0.000266814322080541*G0_39 - 0.000341254681532477*G0_40 + 0.000308634748738932*G0_41 + 0.000110405926378157*G0_42 - 9.28413471816302e-05*G0_43 - 5.52029631890783e-05*G0_44 - 4.01476095920561e-05*G0_45 - 0.000341254681532478*G0_46 + 0.000308634748738932*G0_47 + 0.000110405926378155*G0_48 - 9.28413471816301e-05*G0_49 - 5.52029631890787e-05*G0_50 - 4.01476095920562e-05*G0_51 - 0.000777859935846089*G0_52 + 0.000727675423856017*G0_53 - 0.000301107071940422*G0_54 - 0.000301107071940421*G0_55;
    A[28] = 0.00017343767343768*G0_0 + 0.000149650214754392*G0_1 + 9.59527869250147e-05*G0_2 + 9.59527869250142e-05*G0_3 - 4.01476095920563e-05*G0_4 + 0.00012797050557468*G0_5 + 0.000127970505574679*G0_6 - 4.01476095920573e-05*G0_7 + 0.000110405926378155*G0_8 - 0.000110405926378152*G0_9 + 0.000491808217502691*G0_10 - 0.00034125468153248*G0_11 + 0.000110405926378156*G0_12 - 0.000110405926378156*G0_13 + 0.000491808217502689*G0_14 - 0.000341254681532479*G0_15 - 0.000266814322080546*G0_16 + 0.000170627340766243*G0_17 - 0.000109569517844986*G0_18 - 2.00738047960277e-05*G0_19 - 0.000266814322080534*G0_20 + 0.000170627340766237*G0_21 - 0.000109569517844984*G0_22 - 2.00738047960286e-05*G0_23 - 0.00055537526602344*G0_24 + 0.000873210508627223*G0_25 - 0.000623960765743205*G0_26 + 5.52029631890783e-05*G0_27 + 0.00455006242043304*G0_28 - 0.00180664243164253*G0_29 + 0.00103714658112812*G0_30 - 0.00180664243164253*G0_31 + 0.00051439124789822*G0_32 + 0.00103714658112812*G0_33 + 0.000618942314544201*G0_34 - 0.000213284175957802*G0_35 + 0.000694219082529305*G0_36 - 0.000213284175957794*G0_37 + 0.000577121887885805*G0_38 + 0.000694219082529304*G0_39 + 0.000694219082529301*G0_40 - 0.00125461279975176*G0_41 + 0.00227503121021652*G0_42 + 6.27306399875871e-05*G0_43 - 0.000903321215821264*G0_44 + 0.000518573290564058*G0_45 + 0.000694219082529304*G0_46 - 0.00125461279975176*G0_47 + 0.00227503121021652*G0_48 + 6.27306399875912e-05*G0_49 - 0.000903321215821264*G0_50 + 0.000518573290564061*G0_51 - 0.000351291583930491*G0_52 + 0.00140516633572197*G0_53 + 0.00077785993584609*G0_54 + 0.000777859935846086*G0_55;
    A[29] = -1.77151327324937e-05*G0_0 - 0.00012149670352796*G0_1 - 0.000121496703527961*G0_2 + 3.98465025201157e-05*G0_3 - 5.52029631890768e-05*G0_4 - 0.000120442828776169*G0_5 - 7.02583167860985e-05*G0_6 - 9.28413471816308e-05*G0_7 - 5.52029631890793e-05*G0_8 - 0.000120442828776167*G0_9 - 7.02583167861007e-05*G0_10 - 9.28413471816286e-05*G0_11 + 0.000308634748738929*G0_12 - 4.51660607910594e-05*G0_13 - 4.51660607910659e-05*G0_14 + 0.000308634748738934*G0_15 + 0.000135498182373191*G0_16 - 0.000122952054375673*G0_17 - 2.25830303955327e-05*G0_18 + 2.88560943942906e-05*G0_19 - 8.78228959826557e-06*G0_20 - 0.000148044310370705*G0_21 + 0.000253431785549852*G0_22 - 2.76014815945373e-05*G0_23 - 8.7822895982634e-06*G0_24 - 0.000148044310370708*G0_25 + 0.000253431785549854*G0_26 - 2.76014815945397e-05*G0_27 - 0.00180664243164253*G0_28 + 0.00526937375895737*G0_29 - 0.00180664243164252*G0_30 - 0.000940959599813817*G0_31 - 0.000940959599813818*G0_32 + 0.00051439124789822*G0_33 - 0.0002132841759578*G0_34 + 0.000225830303955315*G0_35 - 0.00125461279975175*G0_36 + 0.000414022223918077*G0_37 - 0.000639852527873397*G0_38 + 6.27306399875879e-05*G0_39 - 0.000213284175957797*G0_40 + 0.000225830303955317*G0_41 - 0.00125461279975176*G0_42 + 0.00041402222391808*G0_43 - 0.000639852527873394*G0_44 + 6.2730639987587e-05*G0_45 + 0.000577121887885808*G0_46 - 0.000639852527873395*G0_47 - 0.000903321215821266*G0_48 - 0.000639852527873399*G0_49 + 0.00263468687947868*G0_50 - 0.000903321215821263*G0_51 - 0.0006398525278734*G0_52 + 0.000677490911865939*G0_53 + 0.000677490911865944*G0_54 - 0.00120442828776168*G0_55;
    A[30] = 0.000173437673437679*G0_0 + 9.59527869250157e-05*G0_1 + 0.000149650214754389*G0_2 + 9.59527869250139e-05*G0_3 + 0.000110405926378154*G0_4 - 0.000110405926378155*G0_5 + 0.000491808217502688*G0_6 - 0.000341254681532477*G0_7 - 4.01476095920548e-05*G0_8 + 0.000127970505574681*G0_9 + 0.000127970505574682*G0_10 - 4.01476095920568e-05*G0_11 - 0.000341254681532474*G0_12 + 0.000491808217502684*G0_13 - 0.000110405926378152*G0_14 + 0.000110405926378152*G0_15 - 0.000266814322080546*G0_16 + 0.000170627340766244*G0_17 - 0.000109569517844986*G0_18 - 2.00738047960278e-05*G0_19 - 0.000555375266023436*G0_20 + 0.000873210508627218*G0_21 - 0.000623960765743202*G0_22 + 5.52029631890774e-05*G0_23 - 0.000266814322080536*G0_24 + 0.00017062734076624*G0_25 - 0.000109569517844986*G0_26 - 2.00738047960272e-05*G0_27 + 0.00103714658112812*G0_28 - 0.00180664243164252*G0_29 + 0.00455006242043302*G0_30 + 0.000514391247898219*G0_31 - 0.00180664243164252*G0_32 + 0.00103714658112811*G0_33 + 0.000694219082529305*G0_34 - 0.00125461279975175*G0_35 + 0.00227503121021651*G0_36 + 6.27306399875907e-05*G0_37 - 0.00090332121582126*G0_38 + 0.000518573290564058*G0_39 + 0.000618942314544194*G0_40 - 0.000213284175957799*G0_41 + 0.000694219082529303*G0_42 - 0.000213284175957799*G0_43 + 0.000577121887885806*G0_44 + 0.000694219082529301*G0_45 + 0.000694219082529302*G0_46 + 6.2730639987586e-05*G0_47 + 0.000518573290564062*G0_48 - 0.00125461279975175*G0_49 - 0.000903321215821263*G0_50 + 0.00227503121021651*G0_51 - 0.000351291583930494*G0_52 + 0.00077785993584609*G0_53 + 0.00140516633572197*G0_54 + 0.000777859935846082*G0_55;
    A[31] = -1.77151327324933e-05*G0_0 - 0.00012149670352796*G0_1 + 3.98465025201159e-05*G0_2 - 0.00012149670352796*G0_3 - 9.28413471816291e-05*G0_4 - 7.02583167860987e-05*G0_5 - 0.000120442828776168*G0_6 - 5.52029631890756e-05*G0_7 + 0.000308634748738932*G0_8 - 4.51660607910633e-05*G0_9 - 4.51660607910629e-05*G0_10 + 0.000308634748738933*G0_11 - 5.52029631890764e-05*G0_12 - 0.000120442828776169*G0_13 - 7.02583167860986e-05*G0_14 - 9.28413471816292e-05*G0_15 - 8.78228959826181e-06*G0_16 - 0.000148044310370705*G0_17 + 0.000253431785549853*G0_18 - 2.76014815945383e-05*G0_19 + 0.000135498182373188*G0_20 - 0.000122952054375671*G0_21 - 2.25830303955329e-05*G0_22 + 2.88560943942906e-05*G0_23 - 8.78228959826414e-06*G0_24 - 0.000148044310370706*G0_25 + 0.000253431785549853*G0_26 - 2.76014815945378e-05*G0_27 - 0.00180664243164253*G0_28 - 0.000940959599813817*G0_29 + 0.000514391247898219*G0_30 + 0.00526937375895737*G0_31 - 0.000940959599813814*G0_32 - 0.00180664243164252*G0_33 - 0.000213284175957799*G0_34 + 0.000414022223918079*G0_35 + 6.27306399875846e-05*G0_36 + 0.000225830303955312*G0_37 - 0.000639852527873388*G0_38 - 0.00125461279975176*G0_39 + 0.000577121887885809*G0_40 - 0.000639852527873394*G0_41 - 0.000903321215821263*G0_42 - 0.000639852527873398*G0_43 + 0.00263468687947868*G0_44 - 0.000903321215821263*G0_45 - 0.000213284175957798*G0_46 + 0.000225830303955318*G0_47 - 0.00125461279975176*G0_48 + 0.000414022223918075*G0_49 - 0.000639852527873395*G0_50 + 6.27306399875864e-05*G0_51 - 0.000639852527873402*G0_52 + 0.000677490911865954*G0_53 - 0.00120442828776168*G0_54 + 0.000677490911865954*G0_55;
    A[32] = -1.77151327324942e-05*G0_0 + 3.98465025201152e-05*G0_1 - 0.000121496703527959*G0_2 - 0.000121496703527959*G0_3 + 0.000308634748738932*G0_4 - 4.51660607910629e-05*G0_5 - 4.5166060791063e-05*G0_6 + 0.000308634748738932*G0_7 - 9.2841347181629e-05*G0_8 - 7.02583167860997e-05*G0_9 - 0.000120442828776169*G0_10 - 5.52029631890768e-05*G0_11 - 9.28413471816299e-05*G0_12 - 7.02583167860985e-05*G0_13 - 0.000120442828776168*G0_14 - 5.52029631890784e-05*G0_15 - 8.78228959826029e-06*G0_16 - 0.000148044310370707*G0_17 + 0.000253431785549854*G0_18 - 2.76014815945383e-05*G0_19 - 8.78228959826406e-06*G0_20 - 0.000148044310370707*G0_21 + 0.000253431785549854*G0_22 - 2.760148159454e-05*G0_23 + 0.000135498182373188*G0_24 - 0.000122952054375672*G0_25 - 2.2583030395532e-05*G0_26 + 2.88560943942907e-05*G0_27 + 0.00051439124789822*G0_28 - 0.000940959599813818*G0_29 - 0.00180664243164252*G0_30 - 0.000940959599813814*G0_31 + 0.00526937375895736*G0_32 - 0.00180664243164253*G0_33 + 0.000577121887885808*G0_34 - 0.000639852527873397*G0_35 - 0.000903321215821258*G0_36 - 0.000639852527873393*G0_37 + 0.00263468687947868*G0_38 - 0.000903321215821263*G0_39 - 0.000213284175957795*G0_40 + 0.000414022223918076*G0_41 + 6.2730639987589e-05*G0_42 + 0.000225830303955317*G0_43 - 0.000639852527873391*G0_44 - 0.00125461279975175*G0_45 - 0.000213284175957798*G0_46 + 0.000414022223918079*G0_47 + 6.27306399875866e-05*G0_48 + 0.000225830303955317*G0_49 - 0.000639852527873395*G0_50 - 0.00125461279975175*G0_51 - 0.000639852527873392*G0_52 - 0.00120442828776168*G0_53 + 0.000677490911865944*G0_54 + 0.000677490911865946*G0_55;
    A[33] = 0.00017343767343768*G0_0 + 9.59527869250161e-05*G0_1 + 9.59527869250137e-05*G0_2 + 0.00014965021475439*G0_3 - 0.000341254681532478*G0_4 + 0.000491808217502689*G0_5 - 0.000110405926378153*G0_6 + 0.000110405926378153*G0_7 - 0.000341254681532476*G0_8 + 0.00049180821750269*G0_9 - 0.000110405926378151*G0_10 + 0.000110405926378153*G0_11 - 4.01476095920544e-05*G0_12 + 0.000127970505574677*G0_13 + 0.000127970505574679*G0_14 - 4.01476095920572e-05*G0_15 - 0.000555375266023447*G0_16 + 0.000873210508627223*G0_17 - 0.000623960765743203*G0_18 + 5.52029631890772e-05*G0_19 - 0.000266814322080533*G0_20 + 0.000170627340766236*G0_21 - 0.000109569517844983*G0_22 - 2.0073804796028e-05*G0_23 - 0.000266814322080535*G0_24 + 0.000170627340766239*G0_25 - 0.000109569517844985*G0_26 - 2.00738047960271e-05*G0_27 + 0.00103714658112812*G0_28 + 0.00051439124789822*G0_29 + 0.00103714658112811*G0_30 - 0.00180664243164252*G0_31 - 0.00180664243164253*G0_32 + 0.00455006242043303*G0_33 + 0.000694219082529304*G0_34 + 6.27306399875868e-05*G0_35 + 0.000518573290564056*G0_36 - 0.00125461279975175*G0_37 - 0.000903321215821266*G0_38 + 0.00227503121021651*G0_39 + 0.000694219082529299*G0_40 + 6.27306399875875e-05*G0_41 + 0.000518573290564057*G0_42 - 0.00125461279975175*G0_43 - 0.000903321215821265*G0_44 + 0.00227503121021651*G0_45 + 0.000618942314544198*G0_46 - 0.000213284175957798*G0_47 + 0.000694219082529307*G0_48 - 0.000213284175957798*G0_49 + 0.000577121887885806*G0_50 + 0.000694219082529304*G0_51 - 0.000351291583930491*G0_52 + 0.000777859935846089*G0_53 + 0.000777859935846079*G0_54 + 0.00140516633572196*G0_55;
    A[34] = 0.000149650214754385*G0_0 + 0.000173437673437684*G0_1 + 9.59527869250149e-05*G0_2 + 9.59527869250149e-05*G0_3 - 4.01476095920562e-05*G0_4 + 0.000127970505574679*G0_5 + 0.000127970505574681*G0_6 - 4.01476095920574e-05*G0_7 - 0.00026681432208054*G0_8 + 0.000170627340766241*G0_9 - 0.000109569517844985*G0_10 - 2.00738047960279e-05*G0_11 - 0.000266814322080539*G0_12 + 0.000170627340766237*G0_13 - 0.000109569517844986*G0_14 - 2.00738047960295e-05*G0_15 + 0.000110405926378151*G0_16 - 0.00011040592637815*G0_17 + 0.00049180821750269*G0_18 - 0.000341254681532478*G0_19 + 0.000110405926378162*G0_20 - 0.000110405926378158*G0_21 + 0.000491808217502693*G0_22 - 0.000341254681532478*G0_23 + 5.52029631890819e-05*G0_24 - 0.000623960765743204*G0_25 + 0.000873210508627225*G0_26 - 0.000555375266023445*G0_27 + 0.000618942314544201*G0_28 - 0.0002132841759578*G0_29 + 0.000694219082529305*G0_30 - 0.000213284175957799*G0_31 + 0.000577121887885808*G0_32 + 0.000694219082529304*G0_33 + 0.00455006242043303*G0_34 - 0.00180664243164253*G0_35 + 0.00103714658112812*G0_36 - 0.00180664243164252*G0_37 + 0.000514391247898217*G0_38 + 0.00103714658112812*G0_39 + 0.00227503121021651*G0_40 - 0.00125461279975176*G0_41 + 0.000694219082529303*G0_42 - 0.000903321215821265*G0_43 + 6.2730639987586e-05*G0_44 + 0.00051857329056406*G0_45 + 0.00227503121021652*G0_46 - 0.00125461279975176*G0_47 + 0.000694219082529307*G0_48 - 0.000903321215821259*G0_49 + 6.27306399875876e-05*G0_50 + 0.000518573290564061*G0_51 + 0.00140516633572197*G0_52 - 0.000351291583930488*G0_53 + 0.000777859935846092*G0_54 + 0.000777859935846088*G0_55;
    A[35] = -0.000121496703527958*G0_0 - 1.77151327324949e-05*G0_1 - 0.000121496703527961*G0_2 + 3.98465025201158e-05*G0_3 - 5.52029631890763e-05*G0_4 - 0.00012044282877617*G0_5 - 7.02583167860973e-05*G0_6 - 9.28413471816293e-05*G0_7 + 0.000135498182373188*G0_8 - 0.000122952054375672*G0_9 - 2.2583030395534e-05*G0_10 + 2.88560943942919e-05*G0_11 - 8.78228959826288e-06*G0_12 - 0.000148044310370705*G0_13 + 0.000253431785549852*G0_14 - 2.7601481594536e-05*G0_15 - 5.52029631890765e-05*G0_16 - 0.000120442828776169*G0_17 - 7.02583167861004e-05*G0_18 - 9.28413471816288e-05*G0_19 + 0.000308634748738928*G0_20 - 4.51660607910601e-05*G0_21 - 4.51660607910652e-05*G0_22 + 0.000308634748738933*G0_23 - 2.76014815945394e-05*G0_24 + 0.000253431785549854*G0_25 - 0.000148044310370708*G0_26 - 8.78228959826203e-06*G0_27 - 0.000213284175957802*G0_28 + 0.000225830303955315*G0_29 - 0.00125461279975175*G0_30 + 0.000414022223918079*G0_31 - 0.000639852527873397*G0_32 + 6.27306399875868e-05*G0_33 - 0.00180664243164253*G0_34 + 0.00526937375895736*G0_35 - 0.00180664243164252*G0_36 - 0.000940959599813812*G0_37 - 0.000940959599813817*G0_38 + 0.000514391247898216*G0_39 - 0.00125461279975176*G0_40 + 0.000225830303955317*G0_41 - 0.000213284175957799*G0_42 - 0.000639852527873391*G0_43 + 0.000414022223918078*G0_44 + 6.27306399875861e-05*G0_45 - 0.000903321215821265*G0_46 - 0.000639852527873391*G0_47 + 0.000577121887885806*G0_48 + 0.00263468687947868*G0_49 - 0.000639852527873395*G0_50 - 0.00090332121582126*G0_51 + 0.000677490911865941*G0_52 - 0.000639852527873404*G0_53 + 0.000677490911865947*G0_54 - 0.00120442828776168*G0_55;
    A[36] = 9.59527869250099e-05*G0_0 + 0.000173437673437684*G0_1 + 0.00014965021475439*G0_2 + 9.59527869250141e-05*G0_3 + 0.000110405926378156*G0_4 - 0.000110405926378156*G0_5 + 0.000491808217502689*G0_6 - 0.000341254681532477*G0_7 - 0.000266814322080539*G0_8 + 0.000170627340766241*G0_9 - 0.000109569517844986*G0_10 - 2.00738047960282e-05*G0_11 - 0.000555375266023442*G0_12 + 0.000873210508627219*G0_13 - 0.000623960765743207*G0_14 + 5.52029631890754e-05*G0_15 - 4.01476095920605e-05*G0_16 + 0.000127970505574682*G0_17 + 0.00012797050557468*G0_18 - 4.01476095920564e-05*G0_19 - 0.000341254681532469*G0_20 + 0.000491808217502683*G0_21 - 0.000110405926378149*G0_22 + 0.000110405926378155*G0_23 - 2.00738047960227e-05*G0_24 - 0.000109569517844986*G0_25 + 0.000170627340766242*G0_26 - 0.000266814322080541*G0_27 + 0.000694219082529305*G0_28 - 0.00125461279975175*G0_29 + 0.00227503121021651*G0_30 + 6.27306399875846e-05*G0_31 - 0.000903321215821258*G0_32 + 0.000518573290564056*G0_33 + 0.00103714658112812*G0_34 - 0.00180664243164252*G0_35 + 0.00455006242043302*G0_36 + 0.000514391247898221*G0_37 - 0.00180664243164252*G0_38 + 0.00103714658112812*G0_39 + 0.000694219082529299*G0_40 - 0.000213284175957797*G0_41 + 0.000618942314544195*G0_42 + 0.000577121887885804*G0_43 - 0.000213284175957801*G0_44 + 0.000694219082529302*G0_45 + 0.000518573290564059*G0_46 + 6.2730639987585e-05*G0_47 + 0.000694219082529308*G0_48 - 0.000903321215821254*G0_49 - 0.00125461279975175*G0_50 + 0.00227503121021651*G0_51 + 0.000777859935846091*G0_52 - 0.000351291583930491*G0_53 + 0.00140516633572198*G0_54 + 0.000777859935846079*G0_55;
    A[37] = -0.000121496703527959*G0_0 - 1.77151327324945e-05*G0_1 + 3.98465025201162e-05*G0_2 - 0.00012149670352796*G0_3 - 9.28413471816297e-05*G0_4 - 7.02583167860977e-05*G0_5 - 0.000120442828776169*G0_6 - 5.52029631890762e-05*G0_7 - 8.78228959826326e-06*G0_8 - 0.000148044310370706*G0_9 + 0.000253431785549852*G0_10 - 2.7601481594539e-05*G0_11 + 0.000135498182373187*G0_12 - 0.000122952054375669*G0_13 - 2.25830303955324e-05*G0_14 + 2.8856094394291e-05*G0_15 + 0.000308634748738932*G0_16 - 4.51660607910641e-05*G0_17 - 4.51660607910645e-05*G0_18 + 0.000308634748738931*G0_19 - 5.52029631890795e-05*G0_20 - 0.000120442828776167*G0_21 - 7.02583167860993e-05*G0_22 - 9.28413471816292e-05*G0_23 - 2.76014815945396e-05*G0_24 + 0.000253431785549853*G0_25 - 0.000148044310370706*G0_26 - 8.78228959826388e-06*G0_27 - 0.000213284175957794*G0_28 + 0.000414022223918077*G0_29 + 6.27306399875907e-05*G0_30 + 0.000225830303955312*G0_31 - 0.000639852527873393*G0_32 - 0.00125461279975175*G0_33 - 0.00180664243164252*G0_34 - 0.000940959599813812*G0_35 + 0.000514391247898221*G0_36 + 0.00526937375895735*G0_37 - 0.000940959599813813*G0_38 - 0.00180664243164252*G0_39 - 0.000903321215821258*G0_40 - 0.000639852527873391*G0_41 + 0.000577121887885808*G0_42 + 0.00263468687947868*G0_43 - 0.000639852527873395*G0_44 - 0.000903321215821257*G0_45 - 0.00125461279975175*G0_46 + 0.000225830303955314*G0_47 - 0.000213284175957794*G0_48 - 0.000639852527873395*G0_49 + 0.000414022223918075*G0_50 + 6.27306399875913e-05*G0_51 + 0.000677490911865953*G0_52 - 0.000639852527873394*G0_53 - 0.00120442828776168*G0_54 + 0.000677490911865954*G0_55;
    A[38] = 3.98465025201167e-05*G0_0 - 1.77151327324955e-05*G0_1 - 0.000121496703527959*G0_2 - 0.000121496703527959*G0_3 + 0.00030863474873893*G0_4 - 4.51660607910616e-05*G0_5 - 4.51660607910636e-05*G0_6 + 0.000308634748738932*G0_7 - 8.78228959826105e-06*G0_8 - 0.00014804431037071*G0_9 + 0.000253431785549856*G0_10 - 2.76014815945393e-05*G0_11 - 8.78228959826178e-06*G0_12 - 0.000148044310370708*G0_13 + 0.000253431785549856*G0_14 - 2.76014815945404e-05*G0_15 - 9.28413471816288e-05*G0_16 - 7.02583167860976e-05*G0_17 - 0.000120442828776169*G0_18 - 5.52029631890764e-05*G0_19 - 9.28413471816314e-05*G0_20 - 7.02583167860986e-05*G0_21 - 0.000120442828776169*G0_22 - 5.5202963189079e-05*G0_23 + 2.88560943942885e-05*G0_24 - 2.2583030395532e-05*G0_25 - 0.000122952054375673*G0_26 + 0.00013549818237319*G0_27 + 0.000577121887885805*G0_28 - 0.000639852527873397*G0_29 - 0.00090332121582126*G0_30 - 0.000639852527873388*G0_31 + 0.00263468687947868*G0_32 - 0.000903321215821266*G0_33 + 0.000514391247898217*G0_34 - 0.000940959599813817*G0_35 - 0.00180664243164252*G0_36 - 0.000940959599813813*G0_37 + 0.00526937375895736*G0_38 - 0.00180664243164253*G0_39 + 6.27306399875915e-05*G0_40 + 0.000414022223918077*G0_41 - 0.000213284175957798*G0_42 - 0.000639852527873393*G0_43 + 0.000225830303955321*G0_44 - 0.00125461279975175*G0_45 + 6.27306399875862e-05*G0_46 + 0.00041402222391808*G0_47 - 0.000213284175957802*G0_48 - 0.000639852527873397*G0_49 + 0.000225830303955317*G0_50 - 0.00125461279975175*G0_51 - 0.00120442828776168*G0_52 - 0.000639852527873391*G0_53 + 0.00067749091186594*G0_54 + 0.000677490911865948*G0_55;
    A[39] = 9.5952786925011e-05*G0_0 + 0.000173437673437685*G0_1 + 9.59527869250136e-05*G0_2 + 0.00014965021475439*G0_3 - 0.000341254681532476*G0_4 + 0.000491808217502687*G0_5 - 0.000110405926378151*G0_6 + 0.000110405926378152*G0_7 - 0.000555375266023444*G0_8 + 0.000873210508627222*G0_9 - 0.000623960765743205*G0_10 + 5.52029631890756e-05*G0_11 - 0.000266814322080539*G0_12 + 0.000170627340766238*G0_13 - 0.000109569517844988*G0_14 - 2.00738047960282e-05*G0_15 - 0.00034125468153248*G0_16 + 0.000491808217502688*G0_17 - 0.00011040592637815*G0_18 + 0.000110405926378153*G0_19 - 4.01476095920501e-05*G0_20 + 0.000127970505574677*G0_21 + 0.000127970505574683*G0_22 - 4.01476095920557e-05*G0_23 - 2.00738047960227e-05*G0_24 - 0.000109569517844987*G0_25 + 0.000170627340766242*G0_26 - 0.000266814322080541*G0_27 + 0.000694219082529304*G0_28 + 6.27306399875879e-05*G0_29 + 0.000518573290564058*G0_30 - 0.00125461279975176*G0_31 - 0.000903321215821263*G0_32 + 0.00227503121021651*G0_33 + 0.00103714658112812*G0_34 + 0.000514391247898216*G0_35 + 0.00103714658112812*G0_36 - 0.00180664243164252*G0_37 - 0.00180664243164253*G0_38 + 0.00455006242043303*G0_39 + 0.000518573290564054*G0_40 + 6.27306399875864e-05*G0_41 + 0.000694219082529301*G0_42 - 0.000903321215821263*G0_43 - 0.00125461279975176*G0_44 + 0.00227503121021651*G0_45 + 0.000694219082529306*G0_46 - 0.000213284175957798*G0_47 + 0.000618942314544201*G0_48 + 0.00057712188788581*G0_49 - 0.000213284175957798*G0_50 + 0.000694219082529308*G0_51 + 0.000777859935846088*G0_52 - 0.000351291583930493*G0_53 + 0.000777859935846097*G0_54 + 0.00140516633572196*G0_55;
    A[40] = 0.000149650214754386*G0_0 + 9.59527869250152e-05*G0_1 + 0.000173437673437683*G0_2 + 9.59527869250139e-05*G0_3 - 0.000266814322080539*G0_4 + 0.000170627340766238*G0_5 - 0.000109569517844986*G0_6 - 2.00738047960283e-05*G0_7 - 4.01476095920551e-05*G0_8 + 0.000127970505574682*G0_9 + 0.00012797050557468*G0_10 - 4.01476095920556e-05*G0_11 - 2.00738047960262e-05*G0_12 - 0.000109569517844989*G0_13 + 0.000170627340766238*G0_14 - 0.000266814322080539*G0_15 + 0.00011040592637815*G0_16 - 0.00011040592637815*G0_17 + 0.000491808217502689*G0_18 - 0.000341254681532477*G0_19 + 5.52029631890831e-05*G0_20 - 0.000623960765743207*G0_21 + 0.000873210508627224*G0_22 - 0.000555375266023443*G0_23 + 0.000110405926378159*G0_24 - 0.000110405926378153*G0_25 + 0.00049180821750269*G0_26 - 0.000341254681532477*G0_27 + 0.000694219082529301*G0_28 - 0.000213284175957797*G0_29 + 0.000618942314544194*G0_30 + 0.000577121887885809*G0_31 - 0.000213284175957795*G0_32 + 0.000694219082529299*G0_33 + 0.00227503121021651*G0_34 - 0.00125461279975176*G0_35 + 0.000694219082529299*G0_36 - 0.000903321215821258*G0_37 + 6.27306399875915e-05*G0_38 + 0.000518573290564054*G0_39 + 0.00455006242043302*G0_40 - 0.00180664243164252*G0_41 + 0.00103714658112811*G0_42 - 0.00180664243164253*G0_43 + 0.000514391247898219*G0_44 + 0.00103714658112811*G0_45 + 0.00227503121021651*G0_46 - 0.000903321215821264*G0_47 + 0.000518573290564054*G0_48 - 0.00125461279975175*G0_49 + 6.27306399875923e-05*G0_50 + 0.0006942190825293*G0_51 + 0.00140516633572196*G0_52 + 0.000777859935846084*G0_53 - 0.000351291583930495*G0_54 + 0.000777859935846077*G0_55;
    A[41] = -0.000121496703527958*G0_0 - 0.000121496703527961*G0_1 - 1.77151327324946e-05*G0_2 + 3.98465025201157e-05*G0_3 + 0.000135498182373189*G0_4 - 0.000122952054375671*G0_5 - 2.25830303955333e-05*G0_6 + 2.88560943942912e-05*G0_7 - 5.52029631890782e-05*G0_8 - 0.000120442828776171*G0_9 - 7.02583167860981e-05*G0_10 - 9.28413471816295e-05*G0_11 - 2.76014815945388e-05*G0_12 + 0.000253431785549855*G0_13 - 0.000148044310370706*G0_14 - 8.78228959826289e-06*G0_15 - 5.52029631890763e-05*G0_16 - 0.000120442828776169*G0_17 - 7.02583167860983e-05*G0_18 - 9.28413471816299e-05*G0_19 - 2.76014815945398e-05*G0_20 + 0.000253431785549854*G0_21 - 0.000148044310370707*G0_22 - 8.78228959826299e-06*G0_23 + 0.000308634748738931*G0_24 - 4.51660607910629e-05*G0_25 - 4.51660607910638e-05*G0_26 + 0.000308634748738932*G0_27 - 0.00125461279975176*G0_28 + 0.000225830303955317*G0_29 - 0.000213284175957798*G0_30 - 0.000639852527873394*G0_31 + 0.000414022223918076*G0_32 + 6.27306399875874e-05*G0_33 - 0.00125461279975176*G0_34 + 0.000225830303955317*G0_35 - 0.000213284175957797*G0_36 - 0.000639852527873391*G0_37 + 0.000414022223918077*G0_38 + 6.27306399875864e-05*G0_39 - 0.00180664243164252*G0_40 + 0.00526937375895737*G0_41 - 0.00180664243164252*G0_42 - 0.000940959599813813*G0_43 - 0.000940959599813816*G0_44 + 0.00051439124789822*G0_45 - 0.000903321215821266*G0_46 + 0.00263468687947869*G0_47 - 0.000903321215821265*G0_48 - 0.000639852527873395*G0_49 - 0.000639852527873394*G0_50 + 0.000577121887885806*G0_51 + 0.000677490911865949*G0_52 + 0.000677490911865943*G0_53 - 0.000639852527873403*G0_54 - 0.00120442828776168*G0_55;
    A[42] = 9.5952786925011e-05*G0_0 + 0.000149650214754391*G0_1 + 0.000173437673437683*G0_2 + 9.59527869250138e-05*G0_3 - 0.00026681432208054*G0_4 + 0.000170627340766239*G0_5 - 0.000109569517844987*G0_6 - 2.00738047960287e-05*G0_7 + 0.000110405926378158*G0_8 - 0.000110405926378152*G0_9 + 0.00049180821750269*G0_10 - 0.000341254681532479*G0_11 + 5.52029631890799e-05*G0_12 - 0.000623960765743207*G0_13 + 0.000873210508627223*G0_14 - 0.000555375266023444*G0_15 - 4.01476095920619e-05*G0_16 + 0.000127970505574682*G0_17 + 0.00012797050557468*G0_18 - 4.01476095920557e-05*G0_19 - 2.00738047960217e-05*G0_20 - 0.000109569517844986*G0_21 + 0.00017062734076624*G0_22 - 0.000266814322080539*G0_23 - 0.000341254681532473*G0_24 + 0.000491808217502688*G0_25 - 0.000110405926378153*G0_26 + 0.000110405926378157*G0_27 + 0.00227503121021652*G0_28 - 0.00125461279975176*G0_29 + 0.000694219082529303*G0_30 - 0.000903321215821263*G0_31 + 6.2730639987589e-05*G0_32 + 0.000518573290564057*G0_33 + 0.000694219082529303*G0_34 - 0.000213284175957799*G0_35 + 0.000618942314544195*G0_36 + 0.000577121887885808*G0_37 - 0.000213284175957798*G0_38 + 0.000694219082529301*G0_39 + 0.00103714658112811*G0_40 - 0.00180664243164252*G0_41 + 0.00455006242043303*G0_42 + 0.000514391247898216*G0_43 - 0.00180664243164253*G0_44 + 0.00103714658112812*G0_45 + 0.000518573290564054*G0_46 - 0.000903321215821262*G0_47 + 0.00227503121021652*G0_48 + 6.27306399875916e-05*G0_49 - 0.00125461279975175*G0_50 + 0.000694219082529304*G0_51 + 0.000777859935846084*G0_52 + 0.00140516633572197*G0_53 - 0.00035129158393049*G0_54 + 0.000777859935846079*G0_55;
    A[43] = -0.00012149670352796*G0_0 + 3.9846502520115e-05*G0_1 - 1.7715132732495e-05*G0_2 - 0.000121496703527961*G0_3 - 8.78228959826151e-06*G0_4 - 0.000148044310370707*G0_5 + 0.000253431785549853*G0_6 - 2.76014815945376e-05*G0_7 - 9.28413471816295e-05*G0_8 - 7.02583167860996e-05*G0_9 - 0.000120442828776171*G0_10 - 5.52029631890765e-05*G0_11 + 2.885609439429e-05*G0_12 - 2.25830303955306e-05*G0_13 - 0.000122952054375673*G0_14 + 0.00013549818237319*G0_15 + 0.000308634748738932*G0_16 - 4.51660607910638e-05*G0_17 - 4.51660607910672e-05*G0_18 + 0.000308634748738933*G0_19 - 2.7601481594541e-05*G0_20 + 0.000253431785549855*G0_21 - 0.000148044310370709*G0_22 - 8.78228959826103e-06*G0_23 - 5.52029631890786e-05*G0_24 - 0.000120442828776169*G0_25 - 7.02583167860988e-05*G0_26 - 9.28413471816302e-05*G0_27 + 6.27306399875871e-05*G0_28 + 0.00041402222391808*G0_29 - 0.000213284175957799*G0_30 - 0.000639852527873398*G0_31 + 0.000225830303955318*G0_32 - 0.00125461279975175*G0_33 - 0.000903321215821265*G0_34 - 0.000639852527873391*G0_35 + 0.000577121887885804*G0_36 + 0.00263468687947868*G0_37 - 0.000639852527873393*G0_38 - 0.000903321215821263*G0_39 - 0.00180664243164253*G0_40 - 0.000940959599813813*G0_41 + 0.000514391247898216*G0_42 + 0.00526937375895737*G0_43 - 0.000940959599813816*G0_44 - 0.00180664243164252*G0_45 - 0.00125461279975176*G0_46 - 0.000639852527873395*G0_47 + 6.27306399875873e-05*G0_48 + 0.000225830303955318*G0_49 + 0.000414022223918077*G0_50 - 0.000213284175957799*G0_51 + 0.000677490911865942*G0_52 - 0.00120442828776169*G0_53 - 0.000639852527873393*G0_54 + 0.000677490911865945*G0_55;
    A[44] = 3.98465025201167e-05*G0_0 - 0.00012149670352796*G0_1 - 1.77151327324954e-05*G0_2 - 0.00012149670352796*G0_3 - 8.78228959826018e-06*G0_4 - 0.000148044310370709*G0_5 + 0.000253431785549856*G0_6 - 2.7601481594538e-05*G0_7 + 0.00030863474873893*G0_8 - 4.51660607910603e-05*G0_9 - 4.51660607910649e-05*G0_10 + 0.000308634748738934*G0_11 - 2.76014815945398e-05*G0_12 + 0.000253431785549855*G0_13 - 0.00014804431037071*G0_14 - 8.78228959826021e-06*G0_15 - 9.2841347181629e-05*G0_16 - 7.02583167860963e-05*G0_17 - 0.00012044282877617*G0_18 - 5.52029631890761e-05*G0_19 + 2.88560943942886e-05*G0_20 - 2.25830303955299e-05*G0_21 - 0.000122952054375674*G0_22 + 0.00013549818237319*G0_23 - 9.28413471816315e-05*G0_24 - 7.02583167860983e-05*G0_25 - 0.000120442828776169*G0_26 - 5.52029631890784e-05*G0_27 - 0.000903321215821264*G0_28 - 0.000639852527873394*G0_29 + 0.000577121887885806*G0_30 + 0.00263468687947868*G0_31 - 0.000639852527873391*G0_32 - 0.000903321215821265*G0_33 + 6.2730639987586e-05*G0_34 + 0.000414022223918078*G0_35 - 0.0002132841759578*G0_36 - 0.000639852527873395*G0_37 + 0.000225830303955321*G0_38 - 0.00125461279975176*G0_39 + 0.000514391247898219*G0_40 - 0.000940959599813816*G0_41 - 0.00180664243164253*G0_42 - 0.000940959599813816*G0_43 + 0.00526937375895737*G0_44 - 0.00180664243164253*G0_45 + 6.27306399875865e-05*G0_46 - 0.000639852527873395*G0_47 - 0.00125461279975176*G0_48 + 0.000414022223918076*G0_49 + 0.000225830303955317*G0_50 - 0.000213284175957801*G0_51 - 0.00120442828776169*G0_52 + 0.000677490911865939*G0_53 - 0.000639852527873393*G0_54 + 0.000677490911865947*G0_55;
    A[45] = 9.59527869250114e-05*G0_0 + 9.59527869250161e-05*G0_1 + 0.000173437673437682*G0_2 + 0.000149650214754389*G0_3 - 0.000555375266023443*G0_4 + 0.000873210508627223*G0_5 - 0.000623960765743205*G0_6 + 5.52029631890767e-05*G0_7 - 0.000341254681532478*G0_8 + 0.000491808217502691*G0_9 - 0.000110405926378153*G0_10 + 0.000110405926378153*G0_11 - 2.00738047960272e-05*G0_12 - 0.000109569517844988*G0_13 + 0.000170627340766238*G0_14 - 0.00026681432208054*G0_15 - 0.00034125468153248*G0_16 + 0.000491808217502688*G0_17 - 0.00011040592637815*G0_18 + 0.000110405926378154*G0_19 - 2.00738047960213e-05*G0_20 - 0.000109569517844989*G0_21 + 0.000170627340766242*G0_22 - 0.00026681432208054*G0_23 - 4.01476095920504e-05*G0_24 + 0.000127970505574679*G0_25 + 0.000127970505574682*G0_26 - 4.0147609592056e-05*G0_27 + 0.000518573290564058*G0_28 + 6.27306399875869e-05*G0_29 + 0.000694219082529301*G0_30 - 0.000903321215821263*G0_31 - 0.00125461279975175*G0_32 + 0.00227503121021651*G0_33 + 0.00051857329056406*G0_34 + 6.27306399875861e-05*G0_35 + 0.000694219082529302*G0_36 - 0.000903321215821257*G0_37 - 0.00125461279975175*G0_38 + 0.00227503121021651*G0_39 + 0.00103714658112811*G0_40 + 0.00051439124789822*G0_41 + 0.00103714658112812*G0_42 - 0.00180664243164252*G0_43 - 0.00180664243164253*G0_44 + 0.00455006242043303*G0_45 + 0.000694219082529303*G0_46 + 0.000577121887885808*G0_47 + 0.000694219082529306*G0_48 - 0.000213284175957796*G0_49 - 0.000213284175957798*G0_50 + 0.000618942314544199*G0_51 + 0.000777859935846092*G0_52 + 0.000777859935846083*G0_53 - 0.00035129158393049*G0_54 + 0.00140516633572196*G0_55;
    A[46] = 0.000149650214754386*G0_0 + 9.59527869250156e-05*G0_1 + 9.59527869250145e-05*G0_2 + 0.000173437673437683*G0_3 - 2.00738047960283e-05*G0_4 - 0.000109569517844987*G0_5 + 0.000170627340766238*G0_6 - 0.00026681432208054*G0_7 - 2.00738047960287e-05*G0_8 - 0.000109569517844983*G0_9 + 0.000170627340766239*G0_10 - 0.000266814322080539*G0_11 - 4.01476095920553e-05*G0_12 + 0.000127970505574678*G0_13 + 0.000127970505574678*G0_14 - 4.01476095920561e-05*G0_15 + 5.52029631890721e-05*G0_16 - 0.0006239607657432*G0_17 + 0.000873210508627222*G0_18 - 0.000555375266023444*G0_19 + 0.000110405926378161*G0_20 - 0.000110405926378158*G0_21 + 0.000491808217502692*G0_22 - 0.000341254681532477*G0_23 + 0.000110405926378159*G0_24 - 0.000110405926378153*G0_25 + 0.00049180821750269*G0_26 - 0.000341254681532478*G0_27 + 0.000694219082529304*G0_28 + 0.000577121887885808*G0_29 + 0.000694219082529302*G0_30 - 0.000213284175957798*G0_31 - 0.000213284175957798*G0_32 + 0.000618942314544198*G0_33 + 0.00227503121021652*G0_34 - 0.000903321215821265*G0_35 + 0.000518573290564059*G0_36 - 0.00125461279975175*G0_37 + 6.27306399875862e-05*G0_38 + 0.000694219082529306*G0_39 + 0.00227503121021651*G0_40 - 0.000903321215821266*G0_41 + 0.000518573290564054*G0_42 - 0.00125461279975176*G0_43 + 6.27306399875865e-05*G0_44 + 0.000694219082529303*G0_45 + 0.00455006242043302*G0_46 - 0.00180664243164253*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164252*G0_49 + 0.000514391247898223*G0_50 + 0.00103714658112811*G0_51 + 0.00140516633572196*G0_52 + 0.000777859935846084*G0_53 + 0.000777859935846088*G0_54 - 0.000351291583930494*G0_55;
    A[47] = -0.000121496703527958*G0_0 - 0.000121496703527961*G0_1 + 3.98465025201161e-05*G0_2 - 1.77151327324951e-05*G0_3 + 2.88560943942895e-05*G0_4 - 2.25830303955302e-05*G0_5 - 0.000122952054375673*G0_6 + 0.000135498182373191*G0_7 - 2.76014815945392e-05*G0_8 + 0.000253431785549854*G0_9 - 0.000148044310370709*G0_10 - 8.78228959826123e-06*G0_11 - 5.52029631890785e-05*G0_12 - 0.000120442828776168*G0_13 - 7.02583167860976e-05*G0_14 - 9.28413471816305e-05*G0_15 - 2.76014815945376e-05*G0_16 + 0.000253431785549854*G0_17 - 0.000148044310370707*G0_18 - 8.78228959826217e-06*G0_19 - 5.52029631890787e-05*G0_20 - 0.000120442828776168*G0_21 - 7.02583167860996e-05*G0_22 - 9.28413471816305e-05*G0_23 + 0.000308634748738931*G0_24 - 4.51660607910644e-05*G0_25 - 4.51660607910631e-05*G0_26 + 0.000308634748738932*G0_27 - 0.00125461279975176*G0_28 - 0.000639852527873395*G0_29 + 6.27306399875859e-05*G0_30 + 0.000225830303955318*G0_31 + 0.000414022223918079*G0_32 - 0.000213284175957798*G0_33 - 0.00125461279975176*G0_34 - 0.000639852527873391*G0_35 + 6.2730639987585e-05*G0_36 + 0.000225830303955314*G0_37 + 0.00041402222391808*G0_38 - 0.000213284175957798*G0_39 - 0.000903321215821264*G0_40 + 0.00263468687947869*G0_41 - 0.000903321215821262*G0_42 - 0.000639852527873395*G0_43 - 0.000639852527873395*G0_44 + 0.000577121887885808*G0_45 - 0.00180664243164253*G0_46 + 0.00526937375895737*G0_47 - 0.00180664243164253*G0_48 - 0.000940959599813814*G0_49 - 0.000940959599813815*G0_50 + 0.000514391247898218*G0_51 + 0.000677490911865949*G0_52 + 0.000677490911865945*G0_53 - 0.00120442828776168*G0_54 - 0.000639852527873393*G0_55;
    A[48] = 9.59527869250114e-05*G0_0 + 0.000149650214754391*G0_1 + 9.59527869250147e-05*G0_2 + 0.000173437673437683*G0_3 - 2.00738047960283e-05*G0_4 - 0.000109569517844987*G0_5 + 0.000170627340766239*G0_6 - 0.000266814322080541*G0_7 + 5.52029631890781e-05*G0_8 - 0.000623960765743204*G0_9 + 0.000873210508627224*G0_10 - 0.000555375266023446*G0_11 + 0.000110405926378157*G0_12 - 0.000110405926378156*G0_13 + 0.000491808217502689*G0_14 - 0.000341254681532478*G0_15 - 2.00738047960352e-05*G0_16 - 0.000109569517844981*G0_17 + 0.000170627340766238*G0_18 - 0.000266814322080539*G0_19 - 4.01476095920509e-05*G0_20 + 0.000127970505574679*G0_21 + 0.000127970505574681*G0_22 - 4.01476095920556e-05*G0_23 - 0.000341254681532474*G0_24 + 0.00049180821750269*G0_25 - 0.000110405926378154*G0_26 + 0.000110405926378155*G0_27 + 0.00227503121021652*G0_28 - 0.000903321215821266*G0_29 + 0.000518573290564062*G0_30 - 0.00125461279975176*G0_31 + 6.27306399875866e-05*G0_32 + 0.000694219082529307*G0_33 + 0.000694219082529307*G0_34 + 0.000577121887885806*G0_35 + 0.000694219082529308*G0_36 - 0.000213284175957794*G0_37 - 0.000213284175957802*G0_38 + 0.000618942314544201*G0_39 + 0.000518573290564054*G0_40 - 0.000903321215821265*G0_41 + 0.00227503121021652*G0_42 + 6.27306399875873e-05*G0_43 - 0.00125461279975176*G0_44 + 0.000694219082529306*G0_45 + 0.00103714658112812*G0_46 - 0.00180664243164253*G0_47 + 0.00455006242043303*G0_48 + 0.000514391247898224*G0_49 - 0.00180664243164252*G0_50 + 0.00103714658112812*G0_51 + 0.000777859935846085*G0_52 + 0.00140516633572197*G0_53 + 0.000777859935846096*G0_54 - 0.000351291583930493*G0_55;
    A[49] = -0.000121496703527959*G0_0 + 3.98465025201159e-05*G0_1 - 0.000121496703527961*G0_2 - 1.77151327324941e-05*G0_3 - 2.76014815945366e-05*G0_4 + 0.000253431785549852*G0_5 - 0.000148044310370704*G0_6 - 8.78228959826277e-06*G0_7 + 2.885609439429e-05*G0_8 - 2.25830303955325e-05*G0_9 - 0.000122952054375673*G0_10 + 0.000135498182373189*G0_11 - 9.28413471816303e-05*G0_12 - 7.02583167860969e-05*G0_13 - 0.000120442828776169*G0_14 - 5.52029631890756e-05*G0_15 - 2.76014815945368e-05*G0_16 + 0.000253431785549852*G0_17 - 0.000148044310370707*G0_18 - 8.78228959826274e-06*G0_19 + 0.000308634748738928*G0_20 - 4.51660607910598e-05*G0_21 - 4.51660607910657e-05*G0_22 + 0.000308634748738933*G0_23 - 5.52029631890787e-05*G0_24 - 0.000120442828776169*G0_25 - 7.02583167860976e-05*G0_26 - 9.28413471816301e-05*G0_27 + 6.27306399875912e-05*G0_28 - 0.000639852527873399*G0_29 - 0.00125461279975175*G0_30 + 0.000414022223918075*G0_31 + 0.000225830303955316*G0_32 - 0.000213284175957798*G0_33 - 0.000903321215821259*G0_34 + 0.00263468687947868*G0_35 - 0.000903321215821254*G0_36 - 0.000639852527873395*G0_37 - 0.000639852527873397*G0_38 + 0.000577121887885809*G0_39 - 0.00125461279975175*G0_40 - 0.000639852527873395*G0_41 + 6.27306399875917e-05*G0_42 + 0.000225830303955318*G0_43 + 0.000414022223918076*G0_44 - 0.000213284175957797*G0_45 - 0.00180664243164252*G0_46 - 0.000940959599813814*G0_47 + 0.000514391247898224*G0_48 + 0.00526937375895737*G0_49 - 0.00094095959981382*G0_50 - 0.00180664243164252*G0_51 + 0.00067749091186595*G0_52 - 0.00120442828776169*G0_53 + 0.000677490911865958*G0_54 - 0.000639852527873394*G0_55;
    A[50] = 3.98465025201162e-05*G0_0 - 0.00012149670352796*G0_1 - 0.000121496703527961*G0_2 - 1.77151327324947e-05*G0_3 - 2.7601481594538e-05*G0_4 + 0.000253431785549854*G0_5 - 0.000148044310370707*G0_6 - 8.78228959826329e-06*G0_7 - 2.76014815945398e-05*G0_8 + 0.000253431785549854*G0_9 - 0.000148044310370707*G0_10 - 8.78228959826247e-06*G0_11 + 0.000308634748738929*G0_12 - 4.51660607910598e-05*G0_13 - 4.5166060791066e-05*G0_14 + 0.000308634748738933*G0_15 + 2.88560943942926e-05*G0_16 - 2.25830303955339e-05*G0_17 - 0.000122952054375671*G0_18 + 0.000135498182373189*G0_19 - 9.28413471816313e-05*G0_20 - 7.02583167860991e-05*G0_21 - 0.000120442828776169*G0_22 - 5.52029631890771e-05*G0_23 - 9.28413471816304e-05*G0_24 - 7.02583167860998e-05*G0_25 - 0.000120442828776168*G0_26 - 5.52029631890788e-05*G0_27 - 0.000903321215821264*G0_28 + 0.00263468687947868*G0_29 - 0.000903321215821264*G0_30 - 0.000639852527873395*G0_31 - 0.000639852527873395*G0_32 + 0.000577121887885806*G0_33 + 6.27306399875875e-05*G0_34 - 0.000639852527873395*G0_35 - 0.00125461279975175*G0_36 + 0.000414022223918075*G0_37 + 0.000225830303955317*G0_38 - 0.000213284175957799*G0_39 + 6.27306399875922e-05*G0_40 - 0.000639852527873394*G0_41 - 0.00125461279975175*G0_42 + 0.000414022223918077*G0_43 + 0.000225830303955317*G0_44 - 0.000213284175957798*G0_45 + 0.000514391247898223*G0_46 - 0.000940959599813815*G0_47 - 0.00180664243164252*G0_48 - 0.00094095959981382*G0_49 + 0.00526937375895736*G0_50 - 0.00180664243164253*G0_51 - 0.00120442828776168*G0_52 + 0.000677490911865946*G0_53 + 0.00067749091186594*G0_54 - 0.000639852527873394*G0_55;
    A[51] = 9.59527869250111e-05*G0_0 + 9.59527869250157e-05*G0_1 + 0.000149650214754389*G0_2 + 0.000173437673437682*G0_3 + 5.52029631890766e-05*G0_4 - 0.000623960765743206*G0_5 + 0.000873210508627221*G0_6 - 0.000555375266023443*G0_7 - 2.00738047960288e-05*G0_8 - 0.000109569517844983*G0_9 + 0.000170627340766239*G0_10 - 0.00026681432208054*G0_11 - 0.000341254681532476*G0_12 + 0.000491808217502686*G0_13 - 0.000110405926378154*G0_14 + 0.000110405926378151*G0_15 - 2.00738047960351e-05*G0_16 - 0.00010956951784498*G0_17 + 0.000170627340766238*G0_18 - 0.000266814322080539*G0_19 - 0.000341254681532471*G0_20 + 0.000491808217502684*G0_21 - 0.000110405926378149*G0_22 + 0.000110405926378154*G0_23 - 4.01476095920519e-05*G0_24 + 0.000127970505574681*G0_25 + 0.00012797050557468*G0_26 - 4.01476095920562e-05*G0_27 + 0.000518573290564061*G0_28 - 0.000903321215821263*G0_29 + 0.00227503121021651*G0_30 + 6.27306399875864e-05*G0_31 - 0.00125461279975175*G0_32 + 0.000694219082529304*G0_33 + 0.000518573290564061*G0_34 - 0.00090332121582126*G0_35 + 0.00227503121021651*G0_36 + 6.27306399875913e-05*G0_37 - 0.00125461279975175*G0_38 + 0.000694219082529308*G0_39 + 0.0006942190825293*G0_40 + 0.000577121887885806*G0_41 + 0.000694219082529304*G0_42 - 0.000213284175957799*G0_43 - 0.000213284175957801*G0_44 + 0.000618942314544199*G0_45 + 0.00103714658112811*G0_46 + 0.000514391247898218*G0_47 + 0.00103714658112812*G0_48 - 0.00180664243164252*G0_49 - 0.00180664243164253*G0_50 + 0.00455006242043303*G0_51 + 0.000777859935846088*G0_52 + 0.00077785993584609*G0_53 + 0.00140516633572197*G0_54 - 0.000351291583930491*G0_55;
    A[52] = -0.000280029576904596*G0_0 + 0.000204250963799586*G0_1 + 0.000204250963799587*G0_2 + 0.000204250963799586*G0_3 - 0.000301107071940419*G0_4 + 6.27306399875853e-05*G0_5 + 6.27306399875873e-05*G0_6 - 0.000301107071940419*G0_7 - 0.000301107071940421*G0_8 + 6.27306399875892e-05*G0_9 + 6.2730639987585e-05*G0_10 - 0.00030110707194042*G0_11 - 0.000301107071940419*G0_12 + 6.27306399875872e-05*G0_13 + 6.27306399875887e-05*G0_14 - 0.000301107071940422*G0_15 + 0.000727675423856009*G0_16 - 0.000903321215821257*G0_17 + 0.00110405926378154*G0_18 - 0.000777859935846088*G0_19 + 0.000727675423856024*G0_20 - 0.000903321215821266*G0_21 + 0.00110405926378155*G0_22 - 0.000777859935846088*G0_23 + 0.000727675423856024*G0_24 - 0.00090332121582126*G0_25 + 0.00110405926378155*G0_26 - 0.000777859935846089*G0_27 - 0.000351291583930491*G0_28 - 0.0006398525278734*G0_29 - 0.000351291583930494*G0_30 - 0.000639852527873402*G0_31 - 0.000639852527873392*G0_32 - 0.000351291583930491*G0_33 + 0.00140516633572197*G0_34 + 0.000677490911865941*G0_35 + 0.000777859935846091*G0_36 + 0.000677490911865953*G0_37 - 0.00120442828776168*G0_38 + 0.000777859935846088*G0_39 + 0.00140516633572196*G0_40 + 0.000677490911865949*G0_41 + 0.000777859935846084*G0_42 + 0.000677490911865942*G0_43 - 0.00120442828776169*G0_44 + 0.000777859935846092*G0_45 + 0.00140516633572196*G0_46 + 0.000677490911865949*G0_47 + 0.000777859935846085*G0_48 + 0.00067749091186595*G0_49 - 0.00120442828776168*G0_50 + 0.000777859935846088*G0_51 + 0.0165608889567232*G0_52 - 0.00225830303955316*G0_53 - 0.00225830303955316*G0_54 - 0.00225830303955317*G0_55;
    A[53] = 0.000204250963799583*G0_0 - 0.000280029576904588*G0_1 + 0.000204250963799586*G0_2 + 0.000204250963799584*G0_3 - 0.000301107071940421*G0_4 + 6.27306399875878e-05*G0_5 + 6.27306399875858e-05*G0_6 - 0.000301107071940424*G0_7 + 0.000727675423856015*G0_8 - 0.000903321215821255*G0_9 + 0.00110405926378155*G0_10 - 0.00077785993584609*G0_11 + 0.000727675423856017*G0_12 - 0.000903321215821263*G0_13 + 0.00110405926378155*G0_14 - 0.000777859935846088*G0_15 - 0.000301107071940431*G0_16 + 6.27306399875955e-05*G0_17 + 6.27306399875891e-05*G0_18 - 0.000301107071940421*G0_19 - 0.000301107071940412*G0_20 + 6.27306399875895e-05*G0_21 + 6.27306399875902e-05*G0_22 - 0.000301107071940422*G0_23 - 0.000777859935846081*G0_24 + 0.00110405926378154*G0_25 - 0.00090332121582126*G0_26 + 0.000727675423856017*G0_27 + 0.00140516633572197*G0_28 + 0.000677490911865939*G0_29 + 0.00077785993584609*G0_30 + 0.000677490911865954*G0_31 - 0.00120442828776168*G0_32 + 0.000777859935846089*G0_33 - 0.000351291583930488*G0_34 - 0.000639852527873404*G0_35 - 0.000351291583930491*G0_36 - 0.000639852527873393*G0_37 - 0.000639852527873391*G0_38 - 0.000351291583930493*G0_39 + 0.000777859935846084*G0_40 + 0.000677490911865943*G0_41 + 0.00140516633572197*G0_42 - 0.00120442828776169*G0_43 + 0.00067749091186594*G0_44 + 0.000777859935846084*G0_45 + 0.000777859935846084*G0_46 + 0.000677490911865945*G0_47 + 0.00140516633572197*G0_48 - 0.00120442828776169*G0_49 + 0.000677490911865946*G0_50 + 0.00077785993584609*G0_51 - 0.00225830303955316*G0_52 + 0.0165608889567231*G0_53 - 0.00225830303955316*G0_54 - 0.00225830303955315*G0_55;
    A[54] = 0.000204250963799582*G0_0 + 0.000204250963799588*G0_1 - 0.000280029576904594*G0_2 + 0.000204250963799589*G0_3 + 0.000727675423856019*G0_4 - 0.000903321215821266*G0_5 + 0.00110405926378155*G0_6 - 0.000777859935846087*G0_7 - 0.000301107071940421*G0_8 + 6.27306399875901e-05*G0_9 + 6.273063998759e-05*G0_10 - 0.000301107071940419*G0_11 - 0.000777859935846087*G0_12 + 0.00110405926378155*G0_13 - 0.000903321215821268*G0_14 + 0.000727675423856016*G0_15 - 0.00030110707194043*G0_16 + 6.27306399875993e-05*G0_17 + 6.27306399875855e-05*G0_18 - 0.000301107071940419*G0_19 - 0.000777859935846082*G0_20 + 0.00110405926378154*G0_21 - 0.00090332121582126*G0_22 + 0.000727675423856021*G0_23 - 0.000301107071940416*G0_24 + 6.273063998759e-05*G0_25 + 6.27306399875907e-05*G0_26 - 0.000301107071940422*G0_27 + 0.00077785993584609*G0_28 + 0.000677490911865945*G0_29 + 0.00140516633572197*G0_30 - 0.00120442828776168*G0_31 + 0.000677490911865944*G0_32 + 0.000777859935846079*G0_33 + 0.000777859935846092*G0_34 + 0.000677490911865947*G0_35 + 0.00140516633572198*G0_36 - 0.00120442828776168*G0_37 + 0.000677490911865941*G0_38 + 0.000777859935846097*G0_39 - 0.000351291583930495*G0_40 - 0.000639852527873403*G0_41 - 0.00035129158393049*G0_42 - 0.000639852527873393*G0_43 - 0.000639852527873393*G0_44 - 0.00035129158393049*G0_45 + 0.000777859935846088*G0_46 - 0.00120442828776168*G0_47 + 0.000777859935846096*G0_48 + 0.000677490911865958*G0_49 + 0.000677490911865941*G0_50 + 0.00140516633572197*G0_51 - 0.00225830303955316*G0_52 - 0.00225830303955316*G0_53 + 0.0165608889567232*G0_54 - 0.00225830303955316*G0_55;
    A[55] = 0.000204250963799582*G0_0 + 0.000204250963799588*G0_1 + 0.000204250963799585*G0_2 - 0.000280029576904591*G0_3 - 0.000777859935846088*G0_4 + 0.00110405926378155*G0_5 - 0.000903321215821261*G0_6 + 0.000727675423856019*G0_7 - 0.000777859935846086*G0_8 + 0.00110405926378155*G0_9 - 0.000903321215821262*G0_10 + 0.000727675423856017*G0_11 - 0.000301107071940418*G0_12 + 6.27306399875829e-05*G0_13 + 6.27306399875863e-05*G0_14 - 0.000301107071940423*G0_15 - 0.000777859935846094*G0_16 + 0.00110405926378155*G0_17 - 0.000903321215821261*G0_18 + 0.000727675423856022*G0_19 - 0.000301107071940412*G0_20 + 6.27306399875828e-05*G0_21 + 6.27306399875918e-05*G0_22 - 0.000301107071940421*G0_23 - 0.000301107071940415*G0_24 + 6.2730639987589e-05*G0_25 + 6.27306399875909e-05*G0_26 - 0.000301107071940421*G0_27 + 0.000777859935846086*G0_28 - 0.00120442828776168*G0_29 + 0.000777859935846082*G0_30 + 0.000677490911865954*G0_31 + 0.000677490911865946*G0_32 + 0.00140516633572196*G0_33 + 0.000777859935846088*G0_34 - 0.00120442828776168*G0_35 + 0.000777859935846079*G0_36 + 0.000677490911865954*G0_37 + 0.000677490911865948*G0_38 + 0.00140516633572196*G0_39 + 0.000777859935846077*G0_40 - 0.00120442828776168*G0_41 + 0.000777859935846079*G0_42 + 0.000677490911865944*G0_43 + 0.000677490911865947*G0_44 + 0.00140516633572196*G0_45 - 0.000351291583930494*G0_46 - 0.000639852527873394*G0_47 - 0.000351291583930493*G0_48 - 0.000639852527873394*G0_49 - 0.000639852527873394*G0_50 - 0.000351291583930491*G0_51 - 0.00225830303955317*G0_52 - 0.00225830303955315*G0_53 - 0.00225830303955316*G0_54 + 0.0165608889567231*G0_55;
  }

  /// Tabulate the tensor for the contribution from a local cell
  /// using the specified reference cell quadrature points/weights
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c,
                               unsigned int num_quadrature_points,
                               const double * const * quadrature_points,
                               const double* quadrature_weights) const
  {
    throw std::runtime_error("Quadrature version of tabulate_tensor not available when using the FFC tensor representation.");
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson3d_5_form_0: public ufc::form
{
public:

  /// Constructor
  poisson3d_5_form_0() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_form_0()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(IndexSum(Product(SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 0), MultiIndex((Index(0),), {Index(0): 3})), SpatialDerivative(Argument(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 1), MultiIndex((Index(0),), {Index(0): 3}))), MultiIndex((Index(0),), {Index(0): 3})), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 0;
  }

  /// Return the number of cell domains
  virtual unsigned int num_cell_domains() const
  {
    return 1;
  }

  /// Return the number of exterior facet domains
  virtual unsigned int num_exterior_facet_domains() const
  {
    return 0;
  }

  /// Return the number of interior facet domains
  virtual unsigned int num_interior_facet_domains() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new dofmap for argument function i
  virtual ufc::dofmap* create_dofmap(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_cell_integral_0_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson3d_5_form_1: public ufc::form
{
public:

  /// Constructor
  poisson3d_5_form_1() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_form_1()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(Product(Argument(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 0), Coefficient(FiniteElement('Lagrange', Cell('tetrahedron', Space(3)), 5, None), 0)), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 1;
  }

  /// Return the number of cell domains
  virtual unsigned int num_cell_domains() const
  {
    return 1;
  }

  /// Return the number of exterior facet domains
  virtual unsigned int num_exterior_facet_domains() const
  {
    return 0;
  }

  /// Return the number of interior facet domains
  virtual unsigned int num_interior_facet_domains() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_finite_element_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new dofmap for argument function i
  virtual ufc::dofmap* create_dofmap(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    case 1:
      {
        return new poisson3d_5_dofmap_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    switch (i)
    {
    case 0:
      {
        return new poisson3d_5_cell_integral_1_0();
        break;
      }
    }
    
    return 0;
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

// DOLFIN wrappers

// Standard library includes
#include <string>

// DOLFIN includes
#include <dolfin/common/NoDeleter.h>
#include <dolfin/fem/FiniteElement.h>
#include <dolfin/fem/DofMap.h>
#include <dolfin/fem/Form.h>
#include <dolfin/function/FunctionSpace.h>
#include <dolfin/function/GenericFunction.h>
#include <dolfin/function/CoefficientAssigner.h>
#include <dolfin/adaptivity/ErrorControl.h>
#include <dolfin/adaptivity/GoalFunctional.h>

namespace Poisson3D_5
{

class CoefficientSpace_f: public dolfin::FunctionSpace
{
public:

  CoefficientSpace_f(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  CoefficientSpace_f(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~CoefficientSpace_f()
  {
  }

};

class Form_0_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  Form_0_FunctionSpace_0(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_0_FunctionSpace_0()
  {
  }

};

class Form_0_FunctionSpace_1: public dolfin::FunctionSpace
{
public:

  Form_0_FunctionSpace_1(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_0_FunctionSpace_1()
  {
  }

};

class Form_0: public dolfin::Form
{
public:

  // Constructor
  Form_0(const dolfin::FunctionSpace& V1, const dolfin::FunctionSpace& V0):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);
    _function_spaces[1] = reference_to_no_delete_pointer(V1);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Constructor
  Form_0(boost::shared_ptr<const dolfin::FunctionSpace> V1, boost::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = V0;
    _function_spaces[1] = V1;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Destructor
  ~Form_0()
  {}

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string& name) const
  {

    dolfin::error("No coefficients.");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {

    dolfin::error("No coefficients.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_0_FunctionSpace_0 TestSpace;
  typedef Form_0_FunctionSpace_1 TrialSpace;

  // Coefficients
};

class Form_1_FunctionSpace_0: public dolfin::FunctionSpace
{
public:

  Form_1_FunctionSpace_0(const dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(dolfin::Mesh& mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), mesh)))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
    dolfin::FunctionSpace(mesh,
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dofmap>(new poisson3d_5_dofmap_0()), *mesh)))
  {
      // Do nothing
  }

  ~Form_1_FunctionSpace_0()
  {
  }

};

typedef CoefficientSpace_f Form_1_FunctionSpace_1;

class Form_1: public dolfin::Form
{
public:

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0, const dolfin::GenericFunction& f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    this->f = f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace& V0, boost::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = reference_to_no_delete_pointer(V0);

    this->f = *f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0, const dolfin::GenericFunction& f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> V0, boost::shared_ptr<const dolfin::GenericFunction> f):
    dolfin::Form(1, 1), f(*this, 0)
  {
    _function_spaces[0] = V0;

    this->f = *f;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Destructor
  ~Form_1()
  {}

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string& name) const
  {
    if (name == "f")
      return 0;

    dolfin::error("Invalid coefficient.");
    return 0;
  }

  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {
    switch (i)
    {
    case 0:
      return "f";
    }

    dolfin::error("Invalid coefficient.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_1_FunctionSpace_0 TestSpace;
  typedef Form_1_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  dolfin::CoefficientAssigner f;
};

// Class typedefs
typedef Form_0 BilinearForm;
typedef Form_0 JacobianForm;
typedef Form_1 LinearForm;
typedef Form_1 ResidualForm;
typedef Form_0::TestSpace FunctionSpace;

}

#endif
